﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.IO;
using System.Diagnostics;

namespace KeybreezeMacroEditor
{
    public partial class frmEditor : Form
    {
        // keyboard API calls for macros:
        [DllImport("user32.dll")]
        static extern void keybd_event(byte bVk, byte bScan, uint dwFlags, UIntPtr dwExtraInfo);
        const int keybd_keyup = 0x2;
        const int keybd_keydown = 0x0;

        // mouse API calls for macros:
        [DllImport("user32.dll")]
        static extern bool SetCursorPos(int X, int Y);
        [DllImport("user32.dll")]
        static extern void mouse_event(uint dwFlags, int dx, int dy, int dwData, UIntPtr dwExtraInfo);
        [DllImport("user32.dll")]
        public static extern IntPtr FindWindow(String sClassName, String sAppName);
        [DllImport("user32.dll", EntryPoint = "SetWindowPos")]
        public static extern IntPtr SetWindowPos(IntPtr hWnd, int hWndInsertAfter, int x, int Y, int cx, int cy, int wFlags);
        [DllImport("user32.dll", CharSet = CharSet.Auto, ExactSpelling = true)]
        public static extern IntPtr GetForegroundWindow();
        [DllImportAttribute("User32.DLL")]
        private static extern bool ShowWindow(IntPtr hWnd, int nCmdShow);
        const int GWL_EXSTYLE = -20;
        const int WS_EX_LAYERED = 0x80000;
        const int LWA_ALPHA = 0x2;
        const int LWA_COLORKEY = 0x1;
        const int HWND_TOPMOST = -1;
        const int HWND_NOTOPMOST = -2;
        const int HWND_BOTTOM = 1;
        const int SWP_NOACTIVATE = 0x0010;
        const int SWP_NOMOVE = 0x0002;
        const int SWP_NOSIZE = 0x0001;

        [DllImport("user32.dll", SetLastError = true)]
        [return: MarshalAs(UnmanagedType.Bool)]
        static extern bool GetWindowRect(IntPtr hWnd, out RECT lpRect);
        [StructLayout(LayoutKind.Sequential)]
        private struct RECT
        {
            public int Left;
            public int Top;
            public int Right;
            public int Bottom;
        }

        const uint MOUSEEVENT_LEFTDOWN = 0x00000002;
        const uint MOUSEEVENT_LEFTUP = 0x00000004;
        const uint MOUSEEVENT_MIDDLEDOWN = 0x00000020;
        const uint MOUSEEVENT_MIDDLEUP = 0x00000040;
        const uint MOUSEEVENT_MOVE = 0x00000001;
        const uint MOUSEEVENT_ABSOLUTE = 0x00008000;
        const uint MOUSEEVENT_RIGHTDOWN = 0x00000008;
        const uint MOUSEEVENT_RIGHTUP = 0x00000010;
        const uint MOUSEEVENT_WHEEL = 0x00000800;
        const uint MOUSEEVENT_XDOWN = 0x00000080;
        const uint MOUSEEVENT_XUP = 0x00000100;

        frmMain frmMain_instance;
        frmRecording frmRecording_instance;
        public frmInputBox frmInputBox_instance = null;
        bool rewriting_history = false;
        public string WorkPath2;
        public string saved_macro_name = "";

        // this variable will be used to see if the user made any changes to the macro (basically whether or not the application should prompt the user to save it before closing).
        public string macro_saved_text = "";

        int wait_time_for_modifiers;
        int wait_time_between_keys;

        string macro_text;

        public frmEditor(frmMain parent, frmRecording frmRecording_parent)
        {
            InitializeComponent();
            frmMain_instance = parent;
            frmRecording_instance = frmRecording_parent;
            wait_time_for_modifiers = frmMain_instance.ReadIntFromINI("macros", "wait_time_for_modifiers", frmMain_instance.WorkPath + "\\options.ini", 50);
            wait_time_between_keys = frmMain_instance.ReadIntFromINI("macros", "wait_time_between_keys", frmMain_instance.WorkPath + "\\options.ini", 20);
        }

        private void frmEditor_Load(object sender, EventArgs e)
        {
            WorkPath2 = frmMain_instance.WorkPath;
            lstCommand.Items.Add("<delay>");
            lstCommand.Items.Add("<inputbox>");
            lstCommand.Items.Add("<keybreeze entry>");
            lstCommand.Items.Add("<keydown>");
            lstCommand.Items.Add("<keyup>");
            lstCommand.Items.Add("<keypress>");
            lstCommand.Items.Add("<messagebox>");
            lstCommand.Items.Add("<mouseclick>");
            lstCommand.Items.Add("<mousedown>");
            lstCommand.Items.Add("<mouseup>");
            lstCommand.Items.Add("<positionwindow>");
            lstCommand.Items.Add("<type>");
            lstCommand.Items.Add("<waitforwindow>");
        }

        private void btnRemoveKeyDelays_Click(object sender, EventArgs e)
        {
            string macro_text = txtMacro.Text;
            int cursor_position = 0;
            int find_end_point = 0;
            string current_command_text = "";
            string current_command;
            string single_character;
            string this_parameter;
            int keydown_start_sequence = 0;
            int keydown_potential_end_sequence = 0;
            int shift_start_position = 0;
            bool adding_keys_to_array = false;
            bool ShiftOn = false;
            bool end_the_sequence = false;
            List<string> keydown_events = new List<string>();
            bool matching_keydown_event = false;
            string text_to_replace;
            string replacement_text;

            if (macro_text.Length < 7)
                return;
            // replace instances of "Space" with an actual space. We can't do this for numbers or symbols because they are the same whether Shift is pressed or not. We need to maintain those the way they were recorded.
            macro_text = macro_text.Replace("key=\"Space\"", "key=\" \"");
            
            do
            {
                try
                { single_character = macro_text.Substring(cursor_position, 1); }
                catch
                { break; }
                if (single_character == "<")
                {
                    // we are starting a new command.
                    find_end_point = macro_text.IndexOf(">", cursor_position);
                    if (find_end_point == -1)
                        return;
                    else
                    {
                        current_command_text = macro_text.Substring(cursor_position + 1, find_end_point - cursor_position - 1);
                        current_command = GetMacroCommand(current_command_text);
                        this_parameter = GetParameter(current_command_text);
                        if (current_command == "keydown")
                        {
                            // we have a keydown event. we need to see if it's a single character.
                            if (this_parameter.Length == 1)
                            {
                                // it is a single character.
                                if (ShiftOn == true)
                                {
                                    keydown_events.Add(this_parameter);
                                }
                                else
                                {
                                    keydown_events.Add(this_parameter.ToLower());
                                }
                                if (adding_keys_to_array == false)
                                {
                                    if (ShiftOn == true)
                                        keydown_start_sequence = shift_start_position;
                                    else
                                        keydown_start_sequence = cursor_position;
                                    adding_keys_to_array = true;
                                }
                                keydown_potential_end_sequence = find_end_point;
                            }
                            else if (adding_keys_to_array == true)
                            {
                                // it's not a single character. Let's see if it's the Shift key.
                                if (this_parameter == "LShiftKey" || this_parameter == "RShiftKey")
                                {
                                    ShiftOn = true;
                                    shift_start_position = cursor_position;
                                }
                                else
                                {
                                    // it's a more complex character that we can't convert to <type text>. End the keydown sequence.
                                    end_the_sequence = true;
                                }
                            }
                            else
                            {
                                if (this_parameter == "LShiftKey" || this_parameter == "RShiftKey")
                                {
                                    ShiftOn = true;
                                    shift_start_position = cursor_position;
                                }
                            }
                        }
                        else if (current_command == "delay")
                        {
                            // we need to skip over delays. do nothing.
                        }
                        else if (current_command == "keyup")
                        {
                            // did the user release Shift?
                            if (this_parameter == "LShiftKey" || this_parameter == "RShiftKey")
                                ShiftOn = false;
                            else if (this_parameter.Length > 1)
                            {
                                // it's a more complex character that we can't convert to <type text>. End the keydown sequence so we can convert what we have so far.
                                end_the_sequence = true;
                            }
                        }
                        else if (adding_keys_to_array == true)
                        {
                            // unknown command. end the sequence.
                            end_the_sequence = true;
                        }

                        cursor_position = find_end_point + 1;
                    }
                }
                else
                {
                    cursor_position++;
                }
                if (end_the_sequence == true)
                {
                    if (keydown_events.Count() > 1)
                    {
                        // we had several key events that we need to convert to text. Do this now.
                        text_to_replace = macro_text.Substring(keydown_start_sequence, keydown_potential_end_sequence - keydown_start_sequence + 1);
                        replacement_text = "<type text=\"";
                        foreach (string s in keydown_events)
                        {
                            replacement_text = replacement_text + s.ToLower();
                        }
                        replacement_text = replacement_text + "\">";
                        macro_text = macro_text.Replace(text_to_replace, replacement_text);
                        cursor_position = macro_text.IndexOf(replacement_text) + replacement_text.Length;
                        end_the_sequence = false;
                        adding_keys_to_array = false;
                        keydown_start_sequence = 0;
                        keydown_events.Clear();
                    }
                    else
                    {
                        end_the_sequence = false;
                        adding_keys_to_array = false;
                        keydown_start_sequence = 0;
                        keydown_events.Clear();
                    }
                }
            } while (cursor_position < macro_text.Length);

            // the loop may have exited without adding the key sequence. check it again.
            if (adding_keys_to_array == true || end_the_sequence == true)
            {
                if (keydown_events.Count() > 1)
                {
                    text_to_replace = macro_text.Substring(keydown_start_sequence, keydown_potential_end_sequence - keydown_start_sequence + 1);
                    replacement_text = "<type text=\"";
                    foreach (string s in keydown_events)
                    {
                        replacement_text = replacement_text + s;
                    }
                    replacement_text = replacement_text + "\">";
                    macro_text = macro_text.Replace(text_to_replace, replacement_text);
                    cursor_position = macro_text.IndexOf(replacement_text) + replacement_text.Length;
                    end_the_sequence = false;
                    keydown_start_sequence = 0;
                    keydown_events.Clear();
                }
                else
                    keydown_events.Clear();
            }

            // now we need to fix any left over <keyup> events that don't have a corresponding <keydown> event.
            cursor_position = 0;
            do
            {
                try
                { single_character = macro_text.Substring(cursor_position, 1); }
                catch
                { break; }
                if (single_character == "<")
                {
                    // we are starting a new command.
                    find_end_point = macro_text.IndexOf(">", cursor_position);
                    if (find_end_point == -1)
                        return;
                    else
                    {
                        current_command_text = macro_text.Substring(cursor_position + 1, find_end_point - cursor_position - 1);
                        current_command = GetMacroCommand(current_command_text);
                        this_parameter = GetParameter(current_command_text);
                        if (current_command == "keydown" && this_parameter.Length == 1)
                        {
                            keydown_events.Add(this_parameter);
                        }
                        else if (current_command == "keyup" && this_parameter.Length == 1)
                        {
                            foreach (string s in keydown_events)
                            {
                                if (s == this_parameter)
                                {
                                    matching_keydown_event = true;
                                    break;
                                }
                            }
                            if (matching_keydown_event == false)
                            {
                                // the keyup event does not have a matching keydown event. delete it.
                                string part1 = macro_text.Substring(0, cursor_position).TrimEnd();
                                string part2 = macro_text.Substring(find_end_point + 1).TrimStart();
                                macro_text = part1 + part2;
                            }
                            if (matching_keydown_event == true)
                            {
                                // reset some stuff.
                                matching_keydown_event = false;
                                keydown_events.Clear();
                            }
                        }
                        cursor_position = find_end_point + 1;
                    }
                }
                else
                {
                    cursor_position++;
                }
            } while (cursor_position < macro_text.Length);

            // finally, let's add the macro_text variable back to the txtMacro.Text property.
            if (macro_text.Substring(0, 7) != "<global")
            {
                int delay_between_keys = frmMain_instance.ReadIntFromINI("macros", "wait_time_between_keys", frmMain_instance.WorkPath + "\\options.ini", 20);
                if (delay_between_keys < 10)
                    delay_between_keys = 10;
                txtMacro.Text = "<global keydelay=\"" + delay_between_keys + "\">" + Environment.NewLine + macro_text;
            }
            else
            {
                txtMacro.Text = macro_text;
            }

            txtMacro.Enabled = true;
        }

        private void Macro_Delay(int delay_time)
        {
            // if we sleep for several seconds, the application won't respond to any requests to stop the macro, so we need to cut the delay down to small increments to allow other actions.
            if (delay_time < 250)
            {
                // this is too small to matter.
                System.Threading.Thread.Sleep(delay_time);
                return;
            }
            else
            {
                int elapsed_time = 0;
                do
                {
                    if (frmMain_instance.macro_running == false)
                        break;
                    System.Threading.Thread.Sleep(250);
                    Application.DoEvents();
                    elapsed_time = elapsed_time + 250;
                } while (elapsed_time < delay_time);
            }
        }

        private void Type_Characters(string text)
        {
            // for upper case characters, we need to hold Shift and type them. Any lower case letters need to be converted to upper case to get their appropiate ASCII code.
            /*  old method of inserting one character at a time... this didn't work for special characters.
            string this_letter;
             * for (int i = 0; i < text.Length; i++)
            {
                this_letter = text[i].ToString();
                Press_Key(this_letter);
                if (frmMain_instance.macro_running == false)
                    break;
                Application.DoEvents();
            }
             */
            Simulate.TextInput(text);
        }

        private int TypeSpecialCharacter(string this_letter, string action = "")
        {
            bool ShiftIsDown = false;
            byte NewCode = 0;

            switch (this_letter)
            {
                case "!":
                    ShiftIsDown = true;
                    NewCode = 49;
                    break;
                case "@":
                    ShiftIsDown = true;
                    NewCode = 50;
                    break;
                case "#":
                    ShiftIsDown = true;
                    NewCode = 51;
                    break;
                case "$":
                    ShiftIsDown = true;
                    NewCode = 52;
                    break;
                case "%":
                    ShiftIsDown = true;
                    NewCode = 53;
                    break;
                case "^":
                    ShiftIsDown = true;
                    NewCode = 54;
                    break;
                case "&":
                    ShiftIsDown = true;
                    NewCode = 55;
                    break;
                case "*":
                    ShiftIsDown = true;
                    NewCode = 56;
                    break;
                case "(":
                    ShiftIsDown = true;
                    NewCode = 57;
                    break;
                case ")":
                    ShiftIsDown = true;
                    NewCode = 48;
                    break;
                case "_":
                    ShiftIsDown = true;
                    NewCode = 189;
                    break;
                case "+":
                    ShiftIsDown = true;
                    NewCode = 187;
                    break;
                case "~":
                    ShiftIsDown = true;
                    NewCode = 192;
                    break;
                case "{":
                    ShiftIsDown = true;
                    NewCode = 219;
                    break;
                case "}":
                    ShiftIsDown = true;
                    NewCode = 221;
                    break;
                case "|":
                    ShiftIsDown = true;
                    NewCode = 220;
                    break;
                case ":":
                    ShiftIsDown = true;
                    NewCode = 186;
                    break;
                case "?":
                    ShiftIsDown = true;
                    NewCode = 191;
                    break;
                case "-":
                    NewCode = 189;
                    break;
                case "=":
                    NewCode = 187;
                    break;
                case "[":
                    NewCode = 219;
                    break;
                case "]":
                    NewCode = 221;
                    break;
                case @"\":
                    NewCode = 220;
                    break;
                case ";":
                    NewCode = 186;
                    break;
                case "'":
                    NewCode = 222;
                    break;
                case ",":
                    NewCode = 188;
                    break;
                case ".":
                    NewCode = 190;
                    break;
                case "/":
                    NewCode = 191;
                    break;
                case "<":
                    ShiftIsDown = true;
                    NewCode = 188;
                    break;
                case ">":
                    ShiftIsDown = true;
                    NewCode = 190;
                    break;
                default:
                    NewCode = 0;
                    break;
            }

            if (NewCode != 0)
            {
                if (ShiftIsDown == true)
                {
                    if (action == "keydown")
                    {
                        keybd_event(16, 0, keybd_keydown, UIntPtr.Zero);
                        System.Threading.Thread.Sleep(wait_time_for_modifiers);
                        keybd_event(NewCode, 0, keybd_keydown, UIntPtr.Zero);
                    }
                    else if (action == "keyup")
                    {
                        keybd_event(NewCode, 0, keybd_keyup, UIntPtr.Zero);
                        System.Threading.Thread.Sleep(wait_time_between_keys);
                        keybd_event(16, 0, keybd_keyup, UIntPtr.Zero);
                    }
                    else
                    {
                        keybd_event(16, 0, keybd_keydown, UIntPtr.Zero);
                        System.Threading.Thread.Sleep(wait_time_for_modifiers);
                        keybd_event(NewCode, 0, keybd_keydown, UIntPtr.Zero);
                        keybd_event(NewCode, 0, keybd_keyup, UIntPtr.Zero);
                        System.Threading.Thread.Sleep(wait_time_between_keys);
                        keybd_event(16, 0, keybd_keyup, UIntPtr.Zero);
                    }
                }
                else
                {
                    if (action == "keydown")
                    {
                        keybd_event(NewCode, 0, keybd_keydown, UIntPtr.Zero);
                    }
                    else if (action == "keyup")
                    {
                        keybd_event(NewCode, 0, keybd_keyup, UIntPtr.Zero);
                    }
                    else
                    {
                        keybd_event(NewCode, 0, keybd_keydown, UIntPtr.Zero);
                        keybd_event(NewCode, 0, keybd_keyup, UIntPtr.Zero);
                        System.Threading.Thread.Sleep(wait_time_between_keys);
                    }
                }
            }
            else
            {
                // unknown character. let's try to use the SendInput() call.
                if (action == "keydown")
                {
                    Simulate.TextInput(this_letter, "keydown");
                }
                else if (action == "keyup")
                {
                    Simulate.TextInput(this_letter, "keyup");
                }
                else
                {
                    Simulate.TextInput(this_letter, "keypress");
                    System.Threading.Thread.Sleep(wait_time_between_keys);
                }
            }
            return 1;
        }

        private string GetMacroCommand(string current_command_text)
        {
            // the command will be before the space. the first parameter will be after the space (and before the equal sign).
            int find_space = current_command_text.IndexOf(" ");
            int find_equal_sign = current_command_text.IndexOf("=");
            if (find_space == -1)
                return "";
            string the_command = current_command_text.Substring(0, find_space).ToLower();
            return the_command;
        }

        private string GetParameter(string current_command_text)
        {
            int keystart = current_command_text.ToLower().IndexOf("=");
            if (keystart == -1)
                return "";
            string second_part = current_command_text.Substring(keystart + 1);
            if (second_part.Substring(0, 1) == "\"")
            {
                // the first character is a quote. remove it.
                second_part = second_part.Substring(1);
            }
            if (second_part.Substring(second_part.Length - 1, 1) == "\"")
            {
                // the last character is a quote. remove it.
                second_part = second_part.Substring(0, second_part.Length - 1);
            }
            return second_part;
        }

        private void RunMacroCommand(string current_command_text)
        {
            // the command will be before the space. the first parameter will be after the space (and before the equal sign).
            int find_space = current_command_text.IndexOf(" ");
            int find_equal_sign = current_command_text.IndexOf("=");
            if (find_space == -1)
                return;
            string the_command = current_command_text.Substring(0, find_space).ToLower();
            string the_parameters = current_command_text.Substring(find_space + 1);
            string parameter_name;
            string parameter_value;
            List<string> parameters = new List<string>();

            Application.DoEvents();
            switch (the_command)
            {
                case "comment":
                    // if it's a comment, do nothing.
                    break;
                case "global":
                    parameters = GetNextParameter(the_parameters);
                    foreach (string parameter in parameters)
                    {
                        parameter_name = parameter.Substring(0, parameter.IndexOf("="));
                        parameter_value = parameter.Substring(parameter.IndexOf("=") + 1);
                        if (parameter_name.ToLower() == "keydelay")
                        {
                            wait_time_between_keys = Convert.ToInt32(parameter_value);
                        }
                        else if (parameter_name.ToLower() == "modifierdelay")
                        {
                            wait_time_for_modifiers = Convert.ToInt32(parameter_value);
                        }
                    }
                    break;
                case "keydown":
                    // this should only have one parameter (the key that was pressed).
                    parameters = GetNextParameter(the_parameters);
                    foreach (string parameter in parameters)
                    {
                        parameter_name = parameter.Substring(0, parameter.IndexOf("="));
                        parameter_value = parameter.Substring(parameter.IndexOf("=") + 1);
                        if (parameter_name.ToLower() == "key")
                        {
                            Press_Key(parameter_value.ToLower(), "keydown");
                        }
                    }
                    break;
                case "keyup":
                    // this should only have one parameter (the key that was pressed).
                    parameters = GetNextParameter(the_parameters);
                    foreach (string parameter in parameters)
                    {
                        parameter_name = parameter.Substring(0, parameter.IndexOf("="));
                        parameter_value = parameter.Substring(parameter.IndexOf("=") + 1);
                        if (parameter_name.ToLower() == "key")
                        {
                            Press_Key(parameter_value.ToLower(), "keyup");
                        }
                    }
                    break;
                case "keypress":
                    // this should only have one parameter (the key that was pressed).
                    parameters = GetNextParameter(the_parameters);
                    foreach (string parameter in parameters)
                    {
                        parameter_name = parameter.Substring(0, parameter.IndexOf("="));
                        parameter_value = parameter.Substring(parameter.IndexOf("=") + 1);
                        if (parameter_name.ToLower() == "key")
                        {
                            Press_Key(parameter_value.ToLower());
                        }
                    }
                    break;
                case "mouseclick":
                case "mousedown":
                case "mouseup":
                    // this should have two parameters (the button and the location).
                    string button = "";
                    string location = "";
                    parameters = GetNextParameter(the_parameters);
                    foreach (string parameter in parameters)
                    {
                        parameter_name = parameter.Substring(0, parameter.IndexOf("="));
                        parameter_value = parameter.Substring(parameter.IndexOf("=") + 1);
                        if (parameter_name.ToLower() == "button")
                        {
                            button = parameter_value.ToLower();
                        }
                        else if (parameter_name.ToLower() == "location")
                        {
                            location = parameter_value;
                        }
                    }
                    if (button.Length == 0 || location.Length == 0)
                        return;
                    if (location.IndexOf(",") == -1)
                        return;
                    string[] location_part = location.Split(',');
                    int left_position = Convert.ToInt32(location_part[0]);
                    int top_position = Convert.ToInt32(location_part[1]);
                    if (button == "left")
                    {
                        // alternate method for positioning the mouse. should not be necessary.
                        //Cursor.Position = new System.Drawing.Point(left_position, top_position);
                        SetCursorPos(left_position, top_position);
                        if (the_command == "mouseclick")
                        {
                            mouse_event(MOUSEEVENT_LEFTDOWN, left_position, top_position, 0, UIntPtr.Zero);
                            mouse_event(MOUSEEVENT_LEFTUP, left_position, top_position, 0, UIntPtr.Zero);
                        }
                        else if (the_command == "mousedown")
                        {
                            mouse_event(MOUSEEVENT_LEFTDOWN, left_position, top_position, 0, UIntPtr.Zero);
                        }
                        else if (the_command == "mouseup")
                        {
                            mouse_event(MOUSEEVENT_LEFTUP, left_position, top_position, 0, UIntPtr.Zero);
                        }
                    }
                    else if (button == "right")
                    {
                        if (the_command == "mouseclick")
                        {
                            mouse_event(MOUSEEVENT_RIGHTDOWN, left_position, top_position, 0, UIntPtr.Zero);
                            mouse_event(MOUSEEVENT_RIGHTUP, left_position, top_position, 0, UIntPtr.Zero);
                        }
                        else if (the_command == "mousedown")
                        {
                            mouse_event(MOUSEEVENT_RIGHTDOWN, left_position, top_position, 0, UIntPtr.Zero);
                        }
                        else if (the_command == "mouseup")
                        {
                            mouse_event(MOUSEEVENT_RIGHTUP, left_position, top_position, 0, UIntPtr.Zero);
                        }
                    }
                    break;
                case "delay":
                    // this should only have one parameter (the time).
                    parameters = GetNextParameter(the_parameters);
                    foreach (string parameter in parameters)
                    {
                        parameter_name = parameter.Substring(0, parameter.IndexOf("="));
                        parameter_value = parameter.Substring(parameter.IndexOf("=") + 1);
                        if (parameter_name.ToLower() == "time")
                        {
                            int delay_time = Convert.ToInt32(parameter_value);
                            Macro_Delay(delay_time);
                        }
                    }
                    break;
                case "type":
                    parameters = GetNextParameter(the_parameters);
                    foreach (string parameter in parameters)
                    {
                        parameter_name = parameter.Substring(0, parameter.IndexOf("="));
                        parameter_value = parameter.Substring(parameter.IndexOf("=") + 1);
                        if (parameter_name.ToLower() == "text")
                        {
                            Type_Characters(parameter_value);
                        }
                    }
                    break;
                case "inputbox":
                    string question = "";
                    string variable_name = "";
                    parameters = GetNextParameter(the_parameters);
                    foreach (string parameter in parameters)
                    {
                        parameter_name = parameter.Substring(0, parameter.IndexOf("="));
                        parameter_value = parameter.Substring(parameter.IndexOf("=") + 1);
                        if (parameter_name.ToLower() == "question")
                        {
                            question = parameter_value;
                        }
                        else if (parameter_name.ToLower() == "variable")
                        {
                            variable_name = parameter_value;
                        }
                    }
                    if (question.Length > 0 && variable_name.Length > 0)
                    {
                        frmInputBoxDialog frmInputBoxDialog_instance;
                        frmInputBoxDialog_instance = new frmInputBoxDialog();
                        frmInputBoxDialog_instance.ShowDialog();
                        string user_response = frmInputBoxDialog_instance.the_text;
                        if (user_response == "")
                        {
                            // the user didn't type anything or they hit Cancel. Stop the macro.
                            frmMain_instance.macro_running = false;
                            break;
                        }
                        else
                        {
                            // replace all instances of the variable name with whatever the user typed.
                            macro_text = macro_text.Replace(variable_name, user_response);
                        }
                    }
                    break;
                case "keybreeze":
                    string keybreeze_entry_text = "";
                    parameters = GetNextParameter(the_parameters);
                    foreach (string parameter in parameters)
                    {
                        parameter_name = parameter.Substring(0, parameter.IndexOf("="));
                        parameter_value = parameter.Substring(parameter.IndexOf("=") + 1);
                        if (parameter_name.ToLower() == "entry")
                        {
                            keybreeze_entry_text = parameter_value;
                        }
                    }
                    // we need to pass the entry to keybreeze.
                    string exe_file = frmMain_instance.InstallDir + "\\keybreeze.exe";
                    ProcessStartInfo startInfo = new ProcessStartInfo(exe_file);
                    startInfo.Arguments = "launch_keybreeze_entry:" + keybreeze_entry_text;
                    Process.Start(startInfo);
                    break;
                case "messagebox":
                    string messagebox_text = "";
                    string messagebox_title = "";
                    parameters = GetNextParameter(the_parameters);
                    foreach (string parameter in parameters)
                    {
                        parameter_name = parameter.Substring(0, parameter.IndexOf("="));
                        parameter_value = parameter.Substring(parameter.IndexOf("=") + 1);
                        if (parameter_name.ToLower() == "text")
                        {
                            messagebox_text = parameter_value;
                        }
                        else if (parameter_name.ToLower() == "title")
                        {
                            messagebox_title = parameter_value;
                        }
                    }
                    MessageBox.Show(this, messagebox_text, messagebox_title);
                    break;
                case "positionwindow":
                    string position_title = "";
                    string position_class = "";
                    string position_left = "";
                    string position_top = "";
                    string position_width = "";
                    string position_height = "";
                    parameters = GetNextParameter(the_parameters);
                    foreach (string parameter in parameters)
                    {
                        parameter_name = parameter.Substring(0, parameter.IndexOf("="));
                        parameter_value = parameter.Substring(parameter.IndexOf("=") + 1);
                        if (parameter_name.ToLower() == "title")
                        {
                            position_title = parameter_value;
                        }
                        else if (parameter_name.ToLower() == "class")
                        {
                            position_class = parameter_value;
                        }
                        else if (parameter_name.ToLower() == "left")
                        {
                            if (parameter_value.Length > 0)
                                position_left = parameter_value;
                        }
                        else if (parameter_name.ToLower() == "top")
                        {
                            if (parameter_value.Length > 0)
                                position_top = parameter_value;
                        }
                        else if (parameter_name.ToLower() == "width")
                        {
                            if (parameter_value.Length > 0)
                                position_width = parameter_value;
                        }
                        else if (parameter_name.ToLower() == "height")
                        {
                            if (parameter_value.Length > 0)
                                position_height = parameter_value;
                        }
                    }
                    PositionWindow(position_title, position_class, position_left, position_top, position_width, position_height);
                    break;
                case "waitforwindow":
                    string the_title = "";
                    string the_class = "";
                    string the_action = "";
                    int the_timeout = 0;
                    int time_elapsed = 0;
                    parameters = GetNextParameter(the_parameters);
                    foreach (string parameter in parameters)
                    {
                        parameter_name = parameter.Substring(0, parameter.IndexOf("="));
                        parameter_value = parameter.Substring(parameter.IndexOf("=") + 1);
                        if (parameter_name.ToLower() == "title")
                        {
                            the_title = parameter_value;
                        }
                        else if (parameter_name.ToLower() == "class")
                        {
                            the_class = parameter_value;
                        }
                        else if (parameter_name.ToLower() == "action")
                        {
                            the_action = parameter_value;
                        }
                        else if (parameter_name.ToLower() == "timeout")
                        {
                            the_timeout = Convert.ToInt32(parameter_value);
                        }
                    }
                    if (the_timeout == 0)
                        the_timeout = 90000;
                    int return_val = 0;
                    if (the_action == "to_close")
                    {
                        do
                        {
                            return_val = WaitForWindow("to_close", the_class, the_title);
                            if (return_val == 0)
                                break;
                            System.Threading.Thread.Sleep(100);
                            time_elapsed = time_elapsed + 100;
                            Application.DoEvents();
                            if (frmMain_instance.macro_running == false)
                                break;
                        } while (time_elapsed < the_timeout);
                    }
                    else if (the_action == "to_open")
                    {
                        do
                        {
                            return_val = WaitForWindow("to_open", the_class, the_title);
                            if (return_val == 1)
                                break;
                            System.Threading.Thread.Sleep(100);
                            time_elapsed = time_elapsed + 100;
                            Application.DoEvents();
                            if (frmMain_instance.macro_running == false)
                                break;
                        } while (time_elapsed < the_timeout);
                    }
                    break;
            }
        }

        private void PositionWindow(string position_title, string position_class, string position_left, string position_top, string position_width, string position_height)
        {
            // this function will resize a window, or position a window at certain coordinates on the screen.
            IntPtr exact_matching_window_handle = IntPtr.Zero;
            IntPtr partial_matching_window_handle = IntPtr.Zero;
            // validate the fields first.
            // trim out any wildcard characters on the edges.
            if (position_title.Substring(0, 1) == "*")
                position_title = position_title.Substring(1);
            if (position_title.Substring(position_title.Length - 1) == "*")
                position_title = position_title.Substring(0, position_title.Length - 1);
            if (position_title.Length > 0)
            {
                if (position_title.ToLower() == "getforegroundwindow()")
                {
                    // hide our recording window so that it doesn't get grabbed as our foreground window.
                    frmRecording_instance.Visible = false;
                    Application.DoEvents();
                    System.Threading.Thread.Sleep(100);
                    exact_matching_window_handle = GetForegroundWindow();
                }
                else
                {
                    Process[] processlist = Process.GetProcesses();
                    string window_title;
                    int find_match;
                    bool partial_match = false;
                    foreach (Process process in processlist)
                    {
                        Application.DoEvents();
                        if (frmMain_instance.macro_running == false)
                            break;
                        if (!String.IsNullOrEmpty(process.MainWindowTitle))
                        {
                            window_title = process.MainWindowTitle;
                            find_match = window_title.IndexOf(position_title);
                            if (find_match == 0)
                            {
                                // we have an exact match. Find the window handle and exit the loop.
                                try
                                {
                                    if (position_class.Length == 0)
                                        exact_matching_window_handle = FindWindow(null, process.MainWindowTitle);
                                    else
                                        exact_matching_window_handle = FindWindow(position_class, process.MainWindowTitle);
                                }
                                catch { }
                                break;
                            }
                            else if (find_match > 0)
                            {
                                // we have a partial match. keep a record of this. if we don't find anything, set focus to it.
                                partial_match = true;
                                try
                                {
                                    if (position_class.Length == 0)
                                        partial_matching_window_handle = FindWindow(null, process.MainWindowTitle);
                                    else
                                        partial_matching_window_handle = FindWindow(position_class, process.MainWindowTitle);
                                }
                                catch { }
                            }
                        }
                    }
                    if (exact_matching_window_handle == IntPtr.Zero && partial_match == true)
                        exact_matching_window_handle = partial_matching_window_handle;
                }
            }
            else
            {
                // we only have a class name.
                exact_matching_window_handle = FindWindow(position_class, null);
            }
            if (exact_matching_window_handle == IntPtr.Zero)
            {
                // we never found the window to resize.
                return;
            }
            // before moving/resizing, restore the window if it's minimized or maximized.
            ShowWindow(exact_matching_window_handle, 9);
            // we may have to get the current window's coordinates.
            RECT varRect = new RECT();
            bool found_window_rect = false;
            if ((position_top != "" && position_left != "") && (position_width == "" && position_height == ""))
            {
                // move window only. Don't resize.
                int position_left_int = 0;
                int position_top_int = 0;
                if (position_left == "left")
                    position_left_int = 0;
                else if (position_left == "right")
                {
                    // move window to the right.
                    //int screen_width = Screen.FromHandle(exact_matching_window_handle).Bounds.Width;
                    int screen_width = Screen.FromHandle(exact_matching_window_handle).WorkingArea.Width;
                    if (found_window_rect == false)
                    {
                        GetWindowRect(exact_matching_window_handle, out varRect);
                        found_window_rect = true;
                    }
                    int lngHeight = varRect.Bottom - varRect.Top;
                    int lngWidth = varRect.Right - varRect.Left;
                    position_left_int = screen_width - lngWidth;
                }
                else
                {
                    // the position is a regular number.
                    position_left_int = Convert.ToInt32(position_left);
                }
                if (position_top == "top")
                    position_top_int = 0;
                else if (position_top == "bottom")
                {
                    // move window to the bottom.
                    //int screen_height = Screen.FromHandle(exact_matching_window_handle).Bounds.Height;
                    int screen_height = Screen.FromHandle(exact_matching_window_handle).WorkingArea.Height;
                    if (found_window_rect == false)
                    {
                        GetWindowRect(exact_matching_window_handle, out varRect);
                        found_window_rect = true;
                    }
                    int lngHeight = varRect.Bottom - varRect.Top;
                    int lngWidth = varRect.Right - varRect.Left;
                    position_top_int = screen_height - lngHeight;
                }
                else
                {
                    // the position is a regular number.
                    position_top_int = Convert.ToInt32(position_top);
                }
                SetWindowPos(exact_matching_window_handle, 0, position_left_int, position_top_int, 0, 0, SWP_NOSIZE);
            }
            else if ((position_top == "" && position_left == "") && (position_width != "" && position_height != ""))
            {
                // resize window only. Don't move.
                int position_width_int = Convert.ToInt32(position_width);
                int position_height_int = Convert.ToInt32(position_height);
                SetWindowPos(exact_matching_window_handle, 0, 0, 0, position_width_int, position_height_int, SWP_NOMOVE);
            }
            else if ((position_top != "" && position_left != "") && (position_width != "" && position_height != ""))
            {
                // move window AND resize it.
                int position_left_int = 0;
                int position_top_int = 0;

                // Resize window first.
                // for the width and height, did the user specify a percentage or a number?
                int position_width_int = 0;
                int position_height_int = 0;
                int find_percent = position_width.IndexOf("%");
                if (find_percent != -1)
                {
                    // the user specified a percentage for the width. Convert it to a decimal and multiply it by the screen size.
                    int width1 = Convert.ToInt32(position_width.Substring(0, find_percent));
                    double position_width_percent = width1 * .01;
                    //int screen_width = Screen.FromHandle(exact_matching_window_handle).Bounds.Width;
                    int screen_width = Screen.FromHandle(exact_matching_window_handle).WorkingArea.Width;
                    position_width_int = Convert.ToInt32(Math.Ceiling(screen_width * position_width_percent));
                }
                else
                    position_width_int = Convert.ToInt32(position_width);
                find_percent = position_height.IndexOf("%");
                if (find_percent != -1)
                {
                    // the user specified a percentage for the height. Convert it to a decimal and multiply it by the screen size.
                    int height1 = Convert.ToInt32(position_height.Substring(0, find_percent));
                    double position_height_percent = height1 * .01;
                    //int screen_height = Screen.FromHandle(exact_matching_window_handle).Bounds.Height;
                    int screen_height = Screen.FromHandle(exact_matching_window_handle).WorkingArea.Height;
                    position_height_int = Convert.ToInt32(Math.Ceiling(screen_height * position_height_percent));
                }
                else
                    position_height_int = Convert.ToInt32(position_height);
                
                // Move window.
                if (position_left == "left")
                    position_left_int = 0;
                else if (position_left == "right")
                {
                    // move window to the right.
                    //int screen_width = Screen.FromHandle(exact_matching_window_handle).Bounds.Width;
                    int screen_width = Screen.FromHandle(exact_matching_window_handle).WorkingArea.Width;
                    int lngHeight = position_height_int;
                    int lngWidth = position_width_int;
                    position_left_int = screen_width - lngWidth;
                }
                else
                {
                    // the position is a regular number.
                    position_left_int = Convert.ToInt32(position_left);
                }
                if (position_top == "top")
                    position_top_int = 0;
                else if (position_top == "bottom")
                {
                    // move window to the bottom.
                    //int screen_height = Screen.FromHandle(exact_matching_window_handle).Bounds.Height;
                    int screen_height = Screen.FromHandle(exact_matching_window_handle).WorkingArea.Height;
                    int lngHeight = position_height_int;
                    int lngWidth = position_width_int;
                    position_top_int = screen_height - lngHeight;
                }
                else
                {
                    // the position is a regular number.
                    position_top_int = Convert.ToInt32(position_top);
                }
                SetWindowPos(exact_matching_window_handle, 0, position_left_int, position_top_int, position_width_int, position_height_int, 0);
            }
            if (position_title.ToLower() == "getforegroundwindow()")
            {
                // show our recording window again.
                frmRecording_instance.Visible = true;
            }
        }

        private int WaitForWindow(string action, string class_name, string title_name)
        {
            // this function will wait until the window opens or closes.
            // returns 1 if the window is open, 0 if it's closed.
            IntPtr exact_matching_window_handle = IntPtr.Zero;
            IntPtr partial_matching_window_handle = IntPtr.Zero;
            // validate the fields first.
            if (class_name.Length == 0 && title_name.Length == 0)
                return -1;
            // trim out any wildcard characters on the edges.
            if (title_name.Substring(0, 1) == "*")
                title_name = title_name.Substring(1);
            if (title_name.Substring(title_name.Length - 1) == "*")
                title_name = title_name.Substring(0, title_name.Length - 1);
            if (title_name.Length > 0)
            {
                Process[] processlist = Process.GetProcesses();
                string window_title;
                int find_match;
                bool partial_match = false;

                foreach (Process process in processlist)
                {
                    Application.DoEvents();
                    if (frmMain_instance.macro_running == false)
                        break;
                    if (!String.IsNullOrEmpty(process.MainWindowTitle))
                    {
                        window_title = process.MainWindowTitle;
                        find_match = window_title.IndexOf(title_name);
                        if (find_match == 0)
                        {
                            // we have an exact match. Find the window handle and exit the loop.
                            try
                            {
                                if (class_name.Length == 0)
                                    exact_matching_window_handle = FindWindow(null, process.MainWindowTitle);
                                else
                                    exact_matching_window_handle = FindWindow(class_name, process.MainWindowTitle);
                            }
                            catch { }
                            break;
                        }
                        else if (find_match > 0)
                        {
                            // we have a partial match. keep a record of this. if we don't find anything, set focus to it.
                            partial_match = true;
                            try
                            {
                                if (class_name.Length == 0)
                                    partial_matching_window_handle = FindWindow(null, process.MainWindowTitle);
                                else
                                    partial_matching_window_handle = FindWindow(class_name, process.MainWindowTitle);
                            }
                            catch { }
                        }
                    }
                }
                if (exact_matching_window_handle == IntPtr.Zero && partial_match == true)
                    exact_matching_window_handle = partial_matching_window_handle;
            }
            else
            {
                // we only have a class name.
                exact_matching_window_handle = FindWindow(class_name, null);
            }
            if (exact_matching_window_handle == IntPtr.Zero)
                return 0;
            else
                return 1;
        }

        private void redoToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            RedoAction();
        }

        private void saveMacroInKeybreezeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SaveInKeybreeze();
        }

        private void SaveInKeybreeze()
        {
            if (saved_macro_name.Length == 0)
            {
                // user has not saved the macro yet. We want to save it and add it to Keybreeze.
                frmSave frmSave_instance;
                frmSave_instance = new frmSave(this);
                frmSave_instance.Text = "Add macro to Keybreeze Desktop";
                frmSave_instance.ShowDialog();
            }
            MessageBox.Show("To add the macro to Keybreeze Desktop, press the hotkey, type 'add', and press Enter. Then type the following:" + Environment.NewLine + Environment.NewLine + "Keyword: " + saved_macro_name + Environment.NewLine + "Target: macro" + Environment.NewLine + Environment.NewLine + "Click 'Add' to add it as a keyword in Keybreeze Desktop. Now you can play back the macro by pressing the hotkey, typing the macro name, and pressing Enter.", "Keybreeze Macro Editor");
        }

        private void selectAllToolStripMenuItem_Click(object sender, EventArgs e)
        {
            txtMacro.SelectAll();
        }

        private void txtMacro_TextChanged(object sender, EventArgs e)
        {
            if (rewriting_history == false)
            {
                int cursor_position = txtMacro.SelectionStart;
                if (frmMain_instance.history_position < frmMain_instance.macro_text_history.Count() - 1)
                {
                    // the user hit undo a few times and then made a change. we need to erase everything after that change within the history list.
                    int range_to_remove = frmMain_instance.macro_text_history.Count() - 1 - frmMain_instance.history_position;
                    frmMain_instance.macro_text_history.RemoveRange(frmMain_instance.history_position, range_to_remove);
                    frmMain_instance.macro_text_cursor_history.RemoveRange(frmMain_instance.history_position, range_to_remove);
                }
                frmMain_instance.macro_text_history.Add(txtMacro.Text);
                frmMain_instance.macro_text_cursor_history.Add(cursor_position);
                frmMain_instance.history_position = frmMain_instance.macro_text_history.Count() - 1;
                if (frmMain_instance.history_position > 0)
                {
                    undoToolStripMenuItem.Enabled = true;
                    undoToolStripMenuItem1.Enabled = true;
                }
            }
        }

        private List<string> GetNextParameter(string the_parameters)
        {
            // return array of parameters and their values, with just an equal sign as the separator. Remove surrounding quotes as well.
            List<string> parameters = new List<string>();
            int cursor_position = 0;
            int find_equal_sign;
            int find_first_quote;
            int find_second_quote;
            string the_original_parameters = the_parameters;
            string this_parameter_name;
            string this_parameter_value;

            try
            {
                do
                {
                    find_equal_sign = the_parameters.IndexOf("=");
                    find_first_quote = the_parameters.IndexOf('"', find_equal_sign + 1);
                    find_second_quote = the_parameters.IndexOf('"', find_first_quote + 1);
                    if (find_second_quote == -1)
                        break;
                    this_parameter_name = the_parameters.Substring(0, find_equal_sign).Trim();
                    this_parameter_value = the_parameters.Substring(find_first_quote + 1, find_second_quote - find_first_quote - 1);
                    // process escape characters:
                    this_parameter_value = this_parameter_value.Replace("breezeleftarrow", "\\<");
                    this_parameter_value = this_parameter_value.Replace("breezerightarrow", "\\>");
                    this_parameter_value = this_parameter_value.Replace("breezequotes", "\\" + '"');
                    parameters.Add(this_parameter_name + "=" + this_parameter_value);
                    // trim this parameter out of the_parameters string so we can add the next parameter to the list.
                    cursor_position = find_second_quote + 1;
                    the_parameters = the_parameters.Substring(cursor_position);
                } while (cursor_position < the_original_parameters.Length - 1);
            }
            catch { }
            return parameters;
        }

        private void Press_Key(string key_stroke, string action = "")
        {
            // launch the specified keystroke using the keybd_event API calls.
            byte key_code = 0;
            bool found_match = false;
            string upper_case_letter;
            byte this_letter_byte;

            // if the key_stroke is only one character, see if we can get the ASCII code.
            if (key_stroke.Length == 1)
            {
                upper_case_letter = key_stroke.ToUpper();
                UInt16 unicode = upper_case_letter[0];
                this_letter_byte = (byte)unicode;
                if (this_letter_byte > 64 && this_letter_byte < 91)
                {
                    // the character is a letter.
                    if (key_stroke == upper_case_letter)
                    {
                        // it's an upper case letter, so we need to hold Shift before we type it.
                        if (action == "keydown")
                        {
                            keybd_event(16, 0, keybd_keydown, UIntPtr.Zero);
                            System.Threading.Thread.Sleep(wait_time_for_modifiers);
                            keybd_event(this_letter_byte, 0, keybd_keydown, UIntPtr.Zero);
                        }
                        else if (action == "keyup")
                        {
                            keybd_event(this_letter_byte, 0, keybd_keyup, UIntPtr.Zero);
                            keybd_event(16, 0, keybd_keyup, UIntPtr.Zero);
                        }
                        else
                        {
                            keybd_event(16, 0, keybd_keydown, UIntPtr.Zero);
                            System.Threading.Thread.Sleep(wait_time_for_modifiers);
                            keybd_event(this_letter_byte, 0, keybd_keydown, UIntPtr.Zero);
                            keybd_event(this_letter_byte, 0, keybd_keyup, UIntPtr.Zero);
                            keybd_event(16, 0, keybd_keyup, UIntPtr.Zero);
                        }
                        found_match = true;
                    }
                    else
                    {
                        // lower case letter
                        if (action == "keydown")
                        {
                            keybd_event(this_letter_byte, 0, keybd_keydown, UIntPtr.Zero);
                        }
                        else if (action == "keyup")
                        {
                            keybd_event(this_letter_byte, 0, keybd_keyup, UIntPtr.Zero);
                        }
                        else
                        {
                            keybd_event(this_letter_byte, 0, keybd_keydown, UIntPtr.Zero);
                            keybd_event(this_letter_byte, 0, keybd_keyup, UIntPtr.Zero);
                        }
                        found_match = true;
                    }
                }
                else if (this_letter_byte > 47 && this_letter_byte < 58)
                {
                    // it's a number.
                    if (action == "keydown")
                    {
                        keybd_event(this_letter_byte, 0, keybd_keydown, UIntPtr.Zero);
                    }
                    else if (action == "keyup")
                    {
                        keybd_event(this_letter_byte, 0, keybd_keyup, UIntPtr.Zero);
                    }
                    else
                    {
                        keybd_event(this_letter_byte, 0, keybd_keydown, UIntPtr.Zero);
                        keybd_event(this_letter_byte, 0, keybd_keyup, UIntPtr.Zero);
                    }
                    found_match = true;
                }
                else if (this_letter_byte == 32)
                {
                    // it's a space.
                    if (action == "keydown")
                    {
                        keybd_event(this_letter_byte, 0, keybd_keydown, UIntPtr.Zero);
                    }
                    else if (action == "keyup")
                    {
                        keybd_event(this_letter_byte, 0, keybd_keyup, UIntPtr.Zero);
                    }
                    else
                    {
                        keybd_event(this_letter_byte, 0, keybd_keydown, UIntPtr.Zero);
                        keybd_event(this_letter_byte, 0, keybd_keyup, UIntPtr.Zero);
                    }
                    found_match = true;
                }
                else if (this_letter_byte == 13 || this_letter_byte == 10)
                {
                    // it's a new line. do nothing.
                    found_match = true;
                }
                else
                {
                    // it's a special character. The ASCII code isn't enough. We need to find how to use the keybd_event call to insert it.
                    int return_code = TypeSpecialCharacter(key_stroke, action);
                    if (return_code == 0)
                        found_match = false;
                    else
                        found_match = true;
                }
            }
            else
            {
                // the keystroke is more than one character. Check for possible values.
                switch (key_stroke)
                {
                    case "lwin":
                        key_code = 91;
                        found_match = true;
                        break;
                    case "rwin":
                        key_code = 92;
                        found_match = true;
                        break;
                    case "lshiftkey":
                        key_code = 16;
                        found_match = true;
                        break;
                    case "rshiftkey":
                        key_code = 16;
                        found_match = true;
                        break;
                    case "lcontrolkey":
                        key_code = 17;
                        found_match = true;
                        break;
                    case "rcontrolkey":
                        key_code = 17;
                        found_match = true;
                        break;
                    case "lmenu":
                        key_code = 18;
                        found_match = true;
                        break;
                    case "rmenu":
                        key_code = 18;
                        found_match = true;
                        break;
                    case "return":
                    case "enter":
                        key_code = 13;
                        found_match = true;
                        break;
                    case "d1":
                        key_code = 49;
                        found_match = true;
                        break;
                    case "d2":
                        key_code = 50;
                        found_match = true;
                        break;
                    case "d3":
                        key_code = 51;
                        found_match = true;
                        break;
                    case "d4":
                        key_code = 52;
                        found_match = true;
                        break;
                    case "d5":
                        key_code = 53;
                        found_match = true;
                        break;
                    case "d6":
                        key_code = 54;
                        found_match = true;
                        break;
                    case "d7":
                        key_code = 55;
                        found_match = true;
                        break;
                    case "d8":
                        key_code = 56;
                        found_match = true;
                        break;
                    case "d9":
                        key_code = 57;
                        found_match = true;
                        break;
                    case "d0":
                        key_code = 48;
                        found_match = true;
                        break;
                    case "oemminus":
                        key_code = 0xBD;
                        found_match = true;
                        break;
                    case "oemplus":
                        key_code = 0xBB;
                        found_match = true;
                        break;
                    case "oemtilde":
                        key_code = 192;
                        found_match = true;
                        break;
                    case "oemopenbrackets":
                        key_code = 0xDB;
                        found_match = true;
                        break;
                    case "oem6":
                        key_code = 0xDD;
                        found_match = true;
                        break;
                    case "oem5":
                        key_code = 0xDC;
                        found_match = true;
                        break;
                    case "oem1":
                        key_code = 0xBA;
                        found_match = true;
                        break;
                    case "oem7":
                        key_code = 0xDE;
                        found_match = true;
                        break;
                    case "oemcomma":
                        key_code = 0xBC;
                        found_match = true;
                        break;
                    case "oemperiod":
                        key_code = 0xBE;
                        found_match = true;
                        break;
                    case "oemquestion":
                        key_code = 191;
                        found_match = true;
                        break;
                    case "escape":
                        key_code = 0x1B;
                        found_match = true;
                        break;
                    case "f1":
                        key_code = 0x70;
                        found_match = true;
                        break;
                    case "f2":
                        key_code = 0x71;
                        found_match = true;
                        break;
                    case "f3":
                        key_code = 0x72;
                        found_match = true;
                        break;
                    case "f4":
                        key_code = 0x73;
                        found_match = true;
                        break;
                    case "f5":
                        key_code = 0x74;
                        found_match = true;
                        break;
                    case "f6":
                        key_code = 0x75;
                        found_match = true;
                        break;
                    case "f7":
                        key_code = 0x76;
                        found_match = true;
                        break;
                    case "f8":
                        key_code = 0x77;
                        found_match = true;
                        break;
                    case "f9":
                        key_code = 0x78;
                        found_match = true;
                        break;
                    case "f10":
                        key_code = 0x79;
                        found_match = true;
                        break;
                    case "f11":
                        key_code = 0x7A;
                        found_match = true;
                        break;
                    case "f12":
                        key_code = 0x7B;
                        found_match = true;
                        break;
                    case "f13":
                        key_code = 0x7C;
                        found_match = true;
                        break;
                    case "f14":
                        key_code = 0x7D;
                        found_match = true;
                        break;
                    case "f15":
                        key_code = 0x7E;
                        found_match = true;
                        break;
                    case "f16":
                        key_code = 0x7F;
                        found_match = true;
                        break;
                    case "f17":
                        key_code = 0x80;
                        found_match = true;
                        break;
                    case "f18":
                        key_code = 0x81;
                        found_match = true;
                        break;
                    case "f19":
                        key_code = 0x82;
                        found_match = true;
                        break;
                    case "f20":
                        key_code = 0x83;
                        found_match = true;
                        break;
                    case "f21":
                        key_code = 0x84;
                        found_match = true;
                        break;
                    case "f22":
                        key_code = 0x85;
                        found_match = true;
                        break;
                    case "f23":
                        key_code = 0x86;
                        found_match = true;
                        break;
                    case "f24":
                        key_code = 0x87;
                        found_match = true;
                        break;
                    case "tab":
                        key_code = 0x09;
                        found_match = true;
                        break;
                    case "capital":
                        key_code = 0x14;
                        found_match = true;
                        break;
                    case "space":
                        key_code = 32;
                        found_match = true;
                        break;
                    case "up":
                        key_code = 0x26;
                        found_match = true;
                        break;
                    case "down":
                        key_code = 0x28;
                        found_match = true;
                        break;
                    case "left":
                        key_code = 0x25;
                        found_match = true;
                        break;
                    case "right":
                        key_code = 0x27;
                        found_match = true;
                        break;
                    case "apps":
                        key_code = 0x5D;
                        found_match = true;
                        break;
                    case "back":
                        key_code = 0x08;
                        found_match = true;
                        break;
                    case "insert":
                        key_code = 0x2D;
                        found_match = true;
                        break;
                    case "home":
                        key_code = 0x24;
                        found_match = true;
                        break;
                    case "pageup":
                        key_code = 0x21;
                        found_match = true;
                        break;
                    case "pagedown":
                        key_code = 0x22;
                        found_match = true;
                        break;
                    case "delete":
                        key_code = 0x2E;
                        found_match = true;
                        break;
                    case "end":
                        key_code = 0x23;
                        found_match = true;
                        break;
                    case "next":
                        key_code = 0x22;
                        found_match = true;
                        break;
                    case "pause":
                        key_code = 0x13;
                        found_match = true;
                        break;
                    case "scroll":
                        key_code = 0x91;
                        found_match = true;
                        break;
                    case "printscreen":
                        key_code = 0x2C;
                        found_match = true;
                        break;
                    case "numlock":
                        key_code = 0x90;
                        found_match = true;
                        break;
                    case "divide":
                        key_code = 0x6F;
                        found_match = true;
                        break;
                    case "multiply":
                        key_code = 0x6A;
                        found_match = true;
                        break;
                    case "subtract":
                        key_code = 0x6D;
                        found_match = true;
                        break;
                    case "add":
                        key_code = 0x6B;
                        found_match = true;
                        break;
                    case "decimal":
                        key_code = 0x6E;
                        found_match = true;
                        break;
                    case "NumPad0":
                        key_code = 0x60;
                        found_match = true;
                        break;
                    case "NumPad1":
                        key_code = 0x61;
                        found_match = true;
                        break;
                    case "NumPad2":
                        key_code = 0x62;
                        found_match = true;
                        break;
                    case "NumPad3":
                        key_code = 0x63;
                        found_match = true;
                        break;
                    case "NumPad4":
                        key_code = 0x64;
                        found_match = true;
                        break;
                    case "NumPad5":
                        key_code = 0x65;
                        found_match = true;
                        break;
                    case "NumPad6":
                        key_code = 0x66;
                        found_match = true;
                        break;
                    case "NumPad7":
                        key_code = 0x67;
                        found_match = true;
                        break;
                    case "NumPad8":
                        key_code = 0x68;
                        found_match = true;
                        break;
                    case "NumPad9":
                        key_code = 0x69;
                        found_match = true;
                        break;
                    default:
                        found_match = false;
                        break;
                }
                if (found_match == true)
                {
                    if (action == "keydown")
                        keybd_event(key_code, 0, keybd_keydown, UIntPtr.Zero);
                    else if (action == "keyup")
                        keybd_event(key_code, 0, keybd_keyup, UIntPtr.Zero);
                    else
                    {
                        keybd_event(key_code, 0, keybd_keydown, UIntPtr.Zero);
                        keybd_event(key_code, 0, keybd_keyup, UIntPtr.Zero);
                    }
                }
            }
            if (found_match == false)
            {
                MessageBox.Show("Unknown key was specified in macro: " + key_stroke + ". Please edit this part of the macro and correct this.", "Keybreeze Macro Recorder");
            }
            System.Threading.Thread.Sleep(wait_time_between_keys);
        }

        private void toolStripComboBox1_Click(object sender, EventArgs e)
        {

        }

        private void cutToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            txtMacro.Cut();
        }

        private void copyToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            txtMacro.Copy();
        }

        private void pasteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            txtMacro.Paste();
        }

        private void cutToolStripMenuItem2_Click_1(object sender, EventArgs e)
        {
            txtMacro.Cut();
        }

        private void undoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            UndoAction();
        }

        public void UndoAction()
        {
            rewriting_history = true;
            frmMain_instance.history_position = frmMain_instance.history_position - 1;
            if (frmMain_instance.history_position == 0)
            {
                undoToolStripMenuItem.Enabled = false;
                undoToolStripMenuItem1.Enabled = false;
            }
            if (frmMain_instance.history_position < 0)
            {
                frmMain_instance.history_position = 0;
            }
            txtMacro.Text = frmMain_instance.macro_text_history[frmMain_instance.history_position];
            int cursor_position = frmMain_instance.macro_text_cursor_history[frmMain_instance.history_position];
            redoToolStripMenuItem1.Enabled = true;
            redoToolStripMenuItem.Enabled = true;
            if (cursor_position > 0 && cursor_position < txtMacro.Text.Length)
            {
                txtMacro.Focus();
                txtMacro.SelectionStart = cursor_position;
                txtMacro.ScrollToCaret();
                txtMacro.Focus();
            }
            rewriting_history = false;
        }

        public void RedoAction()
        {
            rewriting_history = true;
            frmMain_instance.history_position = frmMain_instance.history_position + 1;
            if (frmMain_instance.history_position == frmMain_instance.macro_text_history.Count() - 1)
            {
                redoToolStripMenuItem1.Enabled = false;
                redoToolStripMenuItem.Enabled = false;
            }
            if (frmMain_instance.history_position > frmMain_instance.macro_text_history.Count() - 1)
            {
                frmMain_instance.history_position = frmMain_instance.history_position - 1;
            }
            txtMacro.Text = frmMain_instance.macro_text_history[frmMain_instance.history_position];
            int cursor_position = frmMain_instance.macro_text_cursor_history[frmMain_instance.history_position];
            undoToolStripMenuItem.Enabled = true;
            undoToolStripMenuItem1.Enabled = true;
            if (cursor_position > 0 && cursor_position < txtMacro.Text.Length)
            {
                txtMacro.Focus();
                txtMacro.SelectionStart = cursor_position;
                txtMacro.ScrollToCaret();
                txtMacro.Focus();
            }
            rewriting_history = false;
        }

        private void undoToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            UndoAction();
        }

        private void redoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            RedoAction();
        }

        private void cutToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            txtMacro.Cut();
        }

        private void copyToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            txtMacro.Copy();
        }

        private void pasteToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            txtMacro.Paste();
        }

        private void selectAllToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            txtMacro.SelectAll();
        }

        private void btnPlayback_Click(object sender, EventArgs e)
        {
            PlaybackMacro();
        }

        private void playBackToolStripMenuItem_Click(object sender, EventArgs e)
        {
            PlaybackMacro();
        }

        private void PlaybackMacro()
        {
            this.Hide();
            frmMain_instance.recording_mode = 0;
            frmMain_instance.ShowPlaybackForm();
            macro_text = txtMacro.Text;
            RunMacro();
            frmMain_instance.StopPlayback();
        }

        private void RunMacro()
        {
            int cursor_position = 0;
            int find_end_point = 0;
            string current_command_text = "";
            string single_character;

            frmMain_instance.macro_running = true;

            // process escape characters first.
            macro_text = macro_text.Replace("\\<", "breezeleftarrow");
            macro_text = macro_text.Replace("\\>", "breezerightarrow");
            macro_text = macro_text.Replace("\\" + '"', "breezequotes");

            do
            {
                if (frmMain_instance.macro_running == false)
                    break;
                try
                { single_character = macro_text.Substring(cursor_position, 1); }
                catch
                { return; }
                if (single_character == "<")
                {
                    // we are starting a new command.
                    find_end_point = macro_text.IndexOf(">", cursor_position);
                    if (find_end_point == -1)
                        return;
                    else
                    {
                        current_command_text = macro_text.Substring(cursor_position + 1, find_end_point - cursor_position - 1);
                        RunMacroCommand(current_command_text);
                        cursor_position = find_end_point + 1;
                    }
                }
                else
                {
                    cursor_position++;
                }
            } while (cursor_position < macro_text.Length);
        }

        private void btnRecordmore_Click(object sender, EventArgs e)
        {
            RecordMore();
        }

        private void RecordMore()
        {
            this.Hide();
            frmMain_instance.recording_mode = 1;
            frmMain_instance.recording_more = true;
            frmMain_instance.StartRecording();
        }

        private void recordMoreToolStripMenuItem_Click(object sender, EventArgs e)
        {
            RecordMore();
        }

        private void rerecordToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Rerecord();
        }

        private void Rerecord()
        {
            this.Hide();
            frmMain_instance.recording_mode = 1;
            frmMain_instance.StartRecording();
        }

        private void btnRerecord_Click(object sender, EventArgs e)
        {
            Rerecord();
        }

        private void btnExit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void btnAddMacroToKeybreeze_Click(object sender, EventArgs e)
        {
            SaveInKeybreeze();
        }

        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // New macro
            // .........
            if (txtMacro.Text == macro_saved_text)
            {
                // the user already saved changes. Create a new macro.
                txtMacro.Text = "";
                saved_macro_name = "";
                macro_saved_text = "";
                this.Text = "Untitled - Keybreeze Macro Editor";
                frmMain_instance.macro_text_history.Clear();
                frmMain_instance.macro_text_cursor_history.Clear();
                frmMain_instance.history_position = 0;
            }
            else
            {
                DialogResult result;
                result = MessageBox.Show("Do you want to save your changes?", "Keybreeze Macro Recorder", MessageBoxButtons.YesNoCancel);
                if (result == DialogResult.Yes)
                {
                    SaveMacro();
                    txtMacro.Text = "";
                    saved_macro_name = "";
                    this.Text = "Untitled - Keybreeze Macro Editor";
                    frmMain_instance.macro_text_history.Clear();
                    frmMain_instance.macro_text_cursor_history.Clear();
                    frmMain_instance.history_position = 0;
                }
                else if (result == DialogResult.No)
                {
                    txtMacro.Text = "";
                    saved_macro_name = "";
                    macro_saved_text = "";
                    this.Text = "Untitled - Keybreeze Macro Editor";
                    frmMain_instance.macro_text_history.Clear();
                    frmMain_instance.macro_text_cursor_history.Clear();
                    frmMain_instance.history_position = 0;
                }
            }
        }

        private void lstCommand_SelectedIndexChanged(object sender, EventArgs e)
        {

        }

        private void lstCommand_DoubleClick(object sender, EventArgs e)
        {
            string pretext = txtMacro.Text.Substring(0, txtMacro.SelectionStart);
            string posttext = txtMacro.Text.Substring(txtMacro.SelectionStart);
            string temp1;
            int temp1_length;
            if (lstCommand.SelectedItem.ToString() == "<delay>")
            {
                temp1 = pretext + "<delay time=\"1000\">";
                temp1_length = temp1.Length;
                txtMacro.Text = temp1 + posttext;
                txtMacro.SelectionStart = temp1_length;
                txtMacro.SelectionLength = 0;
                txtMacro.Focus();
            }
            else if (lstCommand.SelectedItem.ToString() == "<comment>")
            {
                temp1 = pretext + "<comment >";
                temp1_length = temp1.Length;
                txtMacro.Text = temp1 + posttext;
                txtMacro.SelectionStart = temp1_length - 1;
                txtMacro.SelectionLength = 0;
                txtMacro.Focus();
            }
            else if (lstCommand.SelectedItem.ToString() == "<inputbox>")
                ShowInputBoxForm();
            else if (lstCommand.SelectedItem.ToString() == "<keybreeze entry>")
            {
                temp1 = pretext + "<keybreeze entry=\"type keyword or entry here\">";
                temp1_length = temp1.Length;
                txtMacro.Text = temp1 + posttext;
                txtMacro.SelectionStart = temp1_length;
                txtMacro.SelectionLength = 0;
                txtMacro.Focus();
            }
            else if (lstCommand.SelectedItem.ToString() == "<keydown>")
            {
                temp1 = pretext + "<keydown key=\"\">";
                temp1_length = temp1.Length;
                txtMacro.Text = temp1 + posttext;
                txtMacro.SelectionStart = temp1_length;
                txtMacro.SelectionLength = 0;
                txtMacro.Focus();
            }
            else if (lstCommand.SelectedItem.ToString() == "<keyup>")
            {
                temp1 = pretext + "<keyup key=\"\">";
                temp1_length = temp1.Length;
                txtMacro.Text = temp1 + posttext;
                txtMacro.SelectionStart = temp1_length;
                txtMacro.SelectionLength = 0;
                txtMacro.Focus();
            }
            else if (lstCommand.SelectedItem.ToString() == "<keypress>")
            {
                temp1 = pretext + "<keypress key=\"\">";
                temp1_length = temp1.Length;
                txtMacro.Text = temp1 + posttext;
                txtMacro.SelectionStart = temp1_length;
                txtMacro.SelectionLength = 0;
                txtMacro.Focus();
            }
            else if (lstCommand.SelectedItem.ToString() == "<messagebox>")
            {
                temp1 = pretext + "<messagebox text=\"\" title=\"\">";
                temp1_length = temp1.Length;
                txtMacro.Text = temp1 + posttext;
                txtMacro.SelectionStart = temp1_length;
                txtMacro.SelectionLength = 0;
                txtMacro.Focus();
            }
            else if (lstCommand.SelectedItem.ToString() == "<mouseclick>")
            {
                this.Hide();
                MessageBox.Show("Press Enter to dismiss this box. Then click the area that you would like to add to the macro.", "Keybreeze Macro Recorder");
                frmMain_instance.TrackingNextMouseEventType = "mouseclick";
                frmMain_instance.TrackNextMouseEvent();
            }
            else if (lstCommand.SelectedItem.ToString() == "<mousedown>")
            {
                this.Hide();
                MessageBox.Show("Press Enter to dismiss this box. Then click the area that you would like to add to the macro.", "Keybreeze Macro Recorder");
                frmMain_instance.TrackingNextMouseEventType = "mousedown";
                frmMain_instance.TrackNextMouseEvent();
            }
            else if (lstCommand.SelectedItem.ToString() == "<mouseup>")
            {
                this.Hide();
                MessageBox.Show("Press Enter to dismiss this box. Then click the area that you would like to add to the macro.", "Keybreeze Macro Recorder");
                frmMain_instance.TrackingNextMouseEventType = "mouseup";
                frmMain_instance.TrackNextMouseEvent();
            }
            else if (lstCommand.SelectedItem.ToString() == "<positionwindow>")
            {
                temp1 = pretext + "<positionwindow title=\"\" class=\"\" left=\"\" top=\"\" width=\"\" height=\"\">";
                temp1_length = temp1.Length;
                txtMacro.Text = temp1 + posttext;
                txtMacro.SelectionStart = temp1_length;
                txtMacro.SelectionLength = 0;
                txtMacro.Focus();
            }
            else if (lstCommand.SelectedItem.ToString() == "<type>")
            {
                temp1 = pretext + "<type text=\"\">";
                temp1_length = temp1.Length;
                txtMacro.Text = temp1 + posttext;
                txtMacro.SelectionStart = temp1_length;
                txtMacro.SelectionLength = 0;
                txtMacro.Focus();
            }
            else if (lstCommand.SelectedItem.ToString() == "<waitforwindow>")
            {
                temp1 = pretext + "<waitforwindow title=\"\" class=\"\" action=\"to_open\">";
                temp1_length = temp1.Length;
                txtMacro.Text = temp1 + posttext;
                txtMacro.SelectionStart = temp1_length;
                txtMacro.SelectionLength = 0;
                txtMacro.Focus();
            }
        }

        public void ShowInputBoxForm()
        {
            if (frmInputBox_instance != null)
            {
                frmInputBox_instance.BringToFront();
            }
            else
            {
                frmInputBox_instance = new frmInputBox(this);
                frmInputBox_instance.Show();
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void frmEditor_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (txtMacro.Text == macro_saved_text)
            {
                // the user already saved changes. Let's just exit.
                System.Environment.Exit(-1);
            }
            else
            {
                DialogResult result;
                result = MessageBox.Show("Do you want to save your changes?", "Keybreeze Macro Recorder", MessageBoxButtons.YesNoCancel);
                if (result == DialogResult.Yes)
                {
                    e.Cancel = true;
                    SaveMacro();
                    return;
                }
                else if (result == DialogResult.No)
                {
                    System.Environment.Exit(-1);
                }
                else
                {
                    // user pressed Cancel.
                    e.Cancel = true;
                    return;
                }
            }
        }

        private void ShowOpenDialogBox()
        {
            openFileDialog1.Filter = "Keybreeze macros | *.kmcr";
            openFileDialog1.InitialDirectory = frmMain_instance.WorkPath + "\\macros";
            openFileDialog1.ShowDialog();
            string macro_name_without_path = openFileDialog1.SafeFileName;
            OpenFile(openFileDialog1.FileName, macro_name_without_path);
        }

        public void OpenFile(string macro_file, string macro_name_without_path)
        {
            saved_macro_name = macro_name_without_path;
            string the_text = "";
            if (saved_macro_name.Length > 5)
            {
                if (saved_macro_name.Substring(saved_macro_name.Length - 5) == ".kmcr")
                {
                    // strip off the extension.
                    saved_macro_name = saved_macro_name.Substring(0, saved_macro_name.Length - 5);
                }
            }
            try
            {
                the_text = File.ReadAllText(macro_file);
            }
            catch
            {
                MessageBox.Show("The macro could not be loaded. Please make sure the file exists and try again.", "Keybreeze Macro Recorder");
                return;
            }
            txtMacro.Text = the_text;
            this.Text = saved_macro_name + ".kcmr" + " - Keybreeze Macro Editor";
            macro_saved_text = the_text;
            frmMain_instance.macro_text_history.Clear();
            frmMain_instance.macro_text_cursor_history.Clear();
            frmMain_instance.history_position = 0;
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // Open
            // ....
            if (txtMacro.Text == macro_saved_text)
            {
                // the user already saved changes. Open the dialog box.
                ShowOpenDialogBox();
            }
            else
            {
                DialogResult result;
                result = MessageBox.Show("Do you want to save your changes?", "Keybreeze Macro Recorder", MessageBoxButtons.YesNoCancel);
                if (result == DialogResult.Yes)
                {
                    SaveMacro();
                    ShowOpenDialogBox();
                }
                else if (result == DialogResult.No)
                {
                    ShowOpenDialogBox();
                }
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            SaveMacro();
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SaveMacro();
        }

        private void saveAsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SaveMacroAs();
        }

        public void SaveMacro()
        {
            if (saved_macro_name.Length > 0)
            {
                // macro has already been saved once. Let's update it.
                try
                {
                    System.IO.File.WriteAllText(this.WorkPath2 + "\\macros\\" + saved_macro_name + ".kmcr", this.txtMacro.Text, Encoding.Unicode);
                    macro_saved_text = this.txtMacro.Text;
                }
                catch
                {
                    MessageBox.Show("Cannot write to the " + this.WorkPath2 + " directory. Please check your Windows permissions.", "Keybreeze Macro Recorder");
                    return;
                }
            }
            else
            {
                // the user has not saved the macro yet. Prompt for a name.
                frmSave frmSave_instance;
                frmSave_instance = new frmSave(this);
                frmSave_instance.Text = "Save macro";
                frmSave_instance.ShowDialog();
            }
        }

        public void SaveMacroAs()
        {
            frmSave frmSave_instance;
            frmSave_instance = new frmSave(this);
            frmSave_instance.Text = "Save macro as...";
            frmSave_instance.ShowDialog();
        }

        private void frmEditor_Resize(object sender, EventArgs e)
        {
            /*
            try
            {
                panel1.Left = this.Width - 241;
                panel1.Height = this.Height - 120;
                txtMacro.Width = this.Width - 256;
                txtMacro.Height = this.Height - 120;
            }
            catch
            {
            }
             * */
        }

        private void label5_Click(object sender, EventArgs e)
        {
            string TheURL = "http://keybreeze.com/article/5";
            try
            {
                System.Diagnostics.Process.Start(TheURL);
            }
            catch
            {
            }
        }

        private void label2_Click(object sender, EventArgs e)
        {

        }

        private void label2_Click_1(object sender, EventArgs e)
        {
            MessageBox.Show("By default, your macro will be played back at the exact speed that you recorded it. To speed it up, click the button above to remove delays that were recorded while you were typing. This button will add a millisecond delay that will occur between each keypress. You can manually shorten other delays as well, but don't shorten them too much, or Windows might not register all of your actions. Click the play back button to test your changes.", "Keybreeze Macro Recorder");
        }

        private void getWindowCoordinatesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmInputBoxDialog frmInputBoxDialog_instance = new frmInputBoxDialog();
            frmInputBoxDialog_instance.lblLabel.Text = "This will give you the location and size of any window on your screen. You can use these numbers for the <positionwindow> command. Type the window title below. Alternatively, you can use a percentage for the width and height of the <positionwindow> command, or you can specify 'right' or 'bottom' for the top and left positions.";
            frmInputBoxDialog_instance.ShowDialog();
            string the_text = frmInputBoxDialog_instance.the_text;
            IntPtr exact_matching_window_handle = IntPtr.Zero;
            if (the_text.Length > 0)
            {
                Process[] processlist = Process.GetProcesses();
                string window_title;
                int find_match;
                foreach (Process process in processlist)
                {
                    Application.DoEvents();
                    if (!String.IsNullOrEmpty(process.MainWindowTitle))
                    {
                        window_title = process.MainWindowTitle;
                        find_match = window_title.IndexOf(the_text);
                        if (find_match == 0)
                        {
                            // we have an exact match. Find the window handle and exit the loop.
                            try
                            {
                                exact_matching_window_handle = FindWindow(null, process.MainWindowTitle);
                                RECT varRect = new RECT();
                                GetWindowRect(exact_matching_window_handle, out varRect);
                                MessageBox.Show("Left: " + varRect.Left + Environment.NewLine + "Top: " + varRect.Top + Environment.NewLine + "Width: " + (varRect.Right - varRect.Left) + Environment.NewLine + "Height: " + (varRect.Bottom - varRect.Top), "Window coordinates for " + the_text);
                            }
                            catch { }
                            break;
                        }
                    }
                }
            }
        }
    }

    public class Simulate
    {
        [DllImport("USER32.DLL", CharSet = CharSet.Unicode)]
        static extern UInt32 SendInput(UInt32 numberOfInputs, INPUT[] input, Int32 sizeOfInputStructure);

        [StructLayout(LayoutKind.Sequential, Size = 24)]
        struct KEYBDINPUT
        {
            public UInt16 Vk;
            public UInt16 Scan;
            public UInt32 Flags;
            public UInt32 Time;
            public UInt32 ExtraInfo;
        }

        [StructLayout(LayoutKind.Explicit)]
        private struct INPUT
        {
            [FieldOffset(0)]
            public int Type;
            [FieldOffset(4)]
            public KEYBDINPUT ki;
        }

        public static void TextInput(string text, string action="keypress")
        {
            char[] chars = text.ToCharArray();

            for (int i = 0; i < chars.Length; i++)
            {
                UInt16 unicode = chars[i];

                INPUT down = new INPUT();
                down.Type = 1; //INPUT_KEYBOARD
                down.ki.Vk = 0;
                down.ki.Scan = unicode;
                down.ki.Time = 0;
                down.ki.Flags = 0x0004; //KEYEVENTF_UNICODE
                down.ki.ExtraInfo = 0;

                INPUT up = new INPUT();
                up.Type = 1; //INPUT_KEYBOARD
                up.ki.Vk = 0;
                up.ki.Scan = unicode;
                up.ki.Time = 0;
                up.ki.Flags = 0x0004; //KEYEVENTF_UNICODE
                up.ki.ExtraInfo = 0;

                INPUT[] input = new INPUT[2];
                if (action == "keydown")
                {
                    input[0] = down;
                    SendInput(1, input, Marshal.SizeOf(typeof(INPUT)));
                }
                else if (action == "keyup")
                {
                    input[0] = up;
                    SendInput(1, input, Marshal.SizeOf(typeof(INPUT)));
                }
                else
                {
                    input[0] = down;
                    input[1] = up;
                    SendInput(1, input, Marshal.SizeOf(typeof(INPUT)));
                }
            }
        }
    }
}
