﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using MouseKeyboardLibrary;
using System.Runtime.InteropServices;
using System.IO;

namespace KeybreezeMacroEditor
{
    public partial class frmMain : Form
    {
        [DllImport("kernel32")]
        private static extern int GetPrivateProfileString(string section, string key, string def, StringBuilder retVal, int size, string filePath);

        [DllImport("KERNEL32.DLL", EntryPoint = "WritePrivateProfileStringW",
          SetLastError = true,
          CharSet = CharSet.Unicode, ExactSpelling = true,
          CallingConvention = CallingConvention.StdCall)]
        private static extern int WritePrivateProfileString(
          string lpAppName,
          string lpKeyName,
          string lpString,
          string lpFilename);

        public string InstallDir = Path.GetDirectoryName(Application.ExecutablePath);
        public string WorkPath;

        MouseHook mouseHook = new MouseHook();
        KeyboardHook keyboardHook = new KeyboardHook();

        public List<string> macro_commands = new List<string>();
        public List<string> macro_text_history = new List<string>();
        public List<int> macro_text_cursor_history = new List<int>();
        public int history_position = 0;
        public string macro_text = "";
        public string stop_key;
        public bool macro_running = false;
        int lastTimeRecorded = 0;
        // recording mode 0: playback
        // recording mode 1: recording
        public int recording_mode = 1;
        public bool recording_more = false;
        public bool TrackingNextMouseEventOnly = false;
        public string TrackingNextMouseEventType = "";
        public string command_line_parameters = "";

        frmRecording frmRecording_instance;
        frmEditor frmEditor_instance;
        public frmMain(string temp_parameters)
        {
            InitializeComponent();
            frmRecording_instance = new frmRecording(this);
            frmEditor_instance = new frmEditor(this, frmRecording_instance);
            command_line_parameters = temp_parameters;
            string extracted = ReadINI("Extract", "Extract_Not_Install", InstallDir + "\\default settings\\options.ini");
            if (extracted == "1")
                WorkPath = InstallDir;
            else
                WorkPath = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + "\\Keybreeze5";
        }

        private void frmMain_Load(object sender, EventArgs e)
        {
            // set up event handlers for mouse and keyboard.
            //mouseHook.MouseMove += new MouseEventHandler(mouseHook_MouseMove);
            mouseHook.MouseDown += new MouseEventHandler(mouseHook_MouseDown);
            mouseHook.MouseUp += new MouseEventHandler(mouseHook_MouseUp);
            //mouseHook.MouseWheel += new MouseEventHandler(mouseHook_MouseWheel);

            keyboardHook.KeyDown += new KeyEventHandler(keyboardHook_KeyDown);
            keyboardHook.KeyUp += new KeyEventHandler(keyboardHook_KeyUp);
            //keyboardHook.KeyPress += new KeyPressEventHandler(keyboardHook_KeyPress);
        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            StartRecording();
        }

        public void StartRecording()
        {
            macro_text = "";
            macro_commands.Clear();
            stop_key = txtStopKey.Text;
            mouseHook.Start();
            keyboardHook.Start();
            this.Hide();
            frmRecording_instance.Show();
            lastTimeRecorded = Environment.TickCount;
        }

        void mouseHook_MouseMove(object sender, MouseEventArgs e)
        {
            SetXYLabel(e.X, e.Y);                         
        }

        void SetXYLabel(int x, int y)
        {
            // this may not be necessary. Try using Cursor.Pos instead.
            //curXYLabel.Text = String.Format("Current Mouse Point: X={0}, y={1}", x, y);
        }

        void keyboardHook_KeyPress(object sender, KeyPressEventArgs e)
        {
            AddKeyboardEvent(
                "keypress",
                "",
                0,
                e.KeyChar.ToString(),
                "",
                "",
                ""
                );
        }

        void keyboardHook_KeyUp(object sender, KeyEventArgs e)
        {

            AddKeyboardEvent(
                "keyup",
                e.KeyCode.ToString(),
                e.KeyValue,
                "",
                e.Shift.ToString(),
                e.Alt.ToString(),
                e.Control.ToString()
                );

        }

        void keyboardHook_KeyDown(object sender, KeyEventArgs e)
        {
            AddKeyboardEvent(
                "keydown",
                e.KeyCode.ToString(),
                e.KeyValue, 
                "",
                e.Shift.ToString(),
                e.Alt.ToString(),
                e.Control.ToString()
                );
        }

        void mouseHook_MouseWheel(object sender, MouseEventArgs e)
        {

            AddMouseEvent(
                "mousewheel",
                "",
                "",
                "",
                e.Delta.ToString()
                );

        }

        void mouseHook_MouseUp(object sender, MouseEventArgs e)
        {
            AddMouseEvent(
                "mouseup",
                e.Button.ToString(),
                e.X.ToString(),
                e.Y.ToString(),
                ""
                );
        }

        void mouseHook_MouseDown(object sender, MouseEventArgs e)
        {
            AddMouseEvent(
                "mousedown",
                e.Button.ToString(),
                e.X.ToString(),
                e.Y.ToString(),
                ""
                );
        }

        void AddMouseEvent(string eventType, string button, string x, string y, string delta)
        {
            if (TrackingNextMouseEventOnly == false)
            {
                // regular macro recording. Add the action as normal with a delay.
                InsertDelay();
                switch (eventType)
                {
                    case "mousedown":
                        macro_commands.Add("<mousedown button=\"" + button + "\" location=\"" + x + "," + y + "\">");
                        break;
                    case "mouseup":
                        macro_commands.Add("<mouseup button=\"" + button + "\" location=\"" + x + "," + y + "\">");
                        break;
                }
            }
            else
            {
                // we are just recording a single mouse event.
                StopTrackingSingleMouseEvent("button=\"" + button + "\" location=\"" + x + "," + y + "\">");
            }
        }

        void AddKeyboardEvent(string eventType, string keyCode, int keyValue, string keyChar, string shift, string alt, string control)
        {
            if (keyCode == stop_key)
            {
                if (recording_mode == 1)
                    StopRecording();
                else
                    StopPlayback();
                return;
            }
            if (recording_mode == 0)
            {
                // we are not recording. we are playing back the macro and just waiting for the user to press the stop key.
                return;
            }
            InsertDelay();
            switch (eventType)
            {
                case "keydown":
                    macro_commands.Add("<keydown key=\"" + keyCode + "\">");
                    break;
                case "keyup":
                    macro_commands.Add("<keyup key=\"" + keyCode + "\">");
                    break;
            }
        }

        private void InsertDelay()
        {
            int elapsed_time = Environment.TickCount - lastTimeRecorded;
            if (elapsed_time > 0)
                macro_commands.Add("<delay time=\"" + elapsed_time + "\">");
            lastTimeRecorded = Environment.TickCount;
        }

        public void StopRecording()
        {
            mouseHook.Stop();
            keyboardHook.Stop();
            frmRecording_instance.Hide();
            //ShortenCommands();
            AddCommandsToMacroText();
            frmEditor_instance.Show();
            if (recording_more == true)
            {
                frmEditor_instance.txtMacro.Text = frmEditor_instance.txtMacro.Text + "<delay time=\"100\">" + Environment.NewLine + macro_text;
                recording_more = false;
            }
            else
            {
                frmEditor_instance.txtMacro.Text = macro_text;
            }
        }

        public void StopTrackingSingleMouseEvent(string the_text)
        {
            string pretext = frmEditor_instance.txtMacro.Text.Substring(0, frmEditor_instance.txtMacro.SelectionStart);
            string posttext = frmEditor_instance.txtMacro.Text.Substring(frmEditor_instance.txtMacro.SelectionStart);
            string temp1;
            int temp1_length;
            mouseHook.Stop();
            temp1 = pretext + "<" + TrackingNextMouseEventType + " " + the_text;
            temp1_length = temp1.Length;
            frmEditor_instance.Show();
            frmEditor_instance.txtMacro.Text = temp1 + posttext;
            frmEditor_instance.txtMacro.SelectionStart = temp1_length;
            frmEditor_instance.txtMacro.SelectionLength = 0;
            frmEditor_instance.txtMacro.Focus();
        }

        public void StopPlayback()
        {
            macro_running = false;
            keyboardHook.Stop();
            frmRecording_instance.Hide();
            frmEditor_instance.Show();
        }

        // we are not going to shorten commands now. maybe some time in the future. we'll tell the user how to manually shorten commands in the editor window.
        /*
        private void ShortenCommands()
        {
            foreach (string s in macro_commands)
            {
                if (s.IndexOf("<mouseup") != -1)
                {
                    // we have a mouse up event. see if we can consolidate the mouse down and mouse up events into one click event.
                    string 
                }
            }
        }
         */

        private void AddCommandsToMacroText()
        {
            if (macro_commands.Count() > 0)
            {
                if (macro_commands[0].IndexOf("<delay") != -1)
                {
                    // we don't want the first command to be a delay. Let's remove it.
                    macro_commands.RemoveAt(0);
                }
            }
            if (macro_commands.Count() > 0)
            {
                if (macro_commands[macro_commands.Count() - 1].IndexOf("<mousedown") != -1)
                {
                    // the last command is a mousedown event. There is no mouseup event after it, so the user most likely clicked the box to stop the macro. Let's remove it.
                    macro_commands.RemoveAt(macro_commands.Count() - 1);
                }
            }
            if (macro_commands.Count() > 0)
            {
                if (macro_commands[macro_commands.Count() - 1].IndexOf("<delay") != -1)
                {
                    // the last command is a delay. Remove it.
                    macro_commands.RemoveAt(macro_commands.Count() - 1);
                }
            }
            foreach (string s in macro_commands)
            {
                macro_text = macro_text + s + Environment.NewLine;
            }
        }

        private void txtStopKey_KeyDown(object sender, KeyEventArgs e)
        {
            // e.KeyCode is the letter/number/key.
            // e.KeyValue is the ascii code number.
            int NewCode = e.KeyValue;
            if (NewCode > 95 && NewCode < 112)
            {
                txtStopKey.Text = "";
                return;
            }
            switch (NewCode)
            {
                case 8:
                    txtStopKey.Text = "Backspace";
                    break;
                case 9:
                    txtStopKey.Text = "";
                    break;
                case 13:
                    txtStopKey.Text = "Enter";
                    break;
                case 32:
                    txtStopKey.Text = "Space";
                    break;
                case 112:
                    txtStopKey.Text = "F1";
                    break;
                case 113:
                    txtStopKey.Text = "F2";
                    break;
                case 114:
                    txtStopKey.Text = "F3";
                    break;
                case 115:
                    txtStopKey.Text = "F4";
                    break;
                case 116:
                    txtStopKey.Text = "F5";
                    break;
                case 117:
                    txtStopKey.Text = "F6";
                    break;
                case 118:
                    txtStopKey.Text = "F7";
                    break;
                case 119:
                    txtStopKey.Text = "F8";
                    break;
                case 120:
                    txtStopKey.Text = "F9";
                    break;
                case 121:
                    txtStopKey.Text = "F10";
                    break;
                case 122:
                    txtStopKey.Text = "F11";
                    break;
                case 123:
                    txtStopKey.Text = "F12";
                    break;
                case 124:
                    txtStopKey.Text = "F13";
                    break;
                case 125:
                    txtStopKey.Text = "F14";
                    break;
                case 126:
                    txtStopKey.Text = "F15";
                    break;
                case 127:
                    txtStopKey.Text = "F16";
                    break;
                case 128:
                    txtStopKey.Text = "F17";
                    break;
                case 129:
                    txtStopKey.Text = "F18";
                    break;
                case 130:
                    txtStopKey.Text = "F19";
                    break;
                case 131:
                    txtStopKey.Text = "F20";
                    break;
                case 144:
                    txtStopKey.Text = "Num Lock";
                    break;
                case 145:
                    txtStopKey.Text = "Scroll Lock";
                    break;
                case 45:
                    txtStopKey.Text = "Insert";
                    break;
                case 36:
                    txtStopKey.Text = "Home";
                    break;
                case 33:
                    txtStopKey.Text = "Page Up";
                    break;
                case 46:
                    txtStopKey.Text = "Delete";
                    break;
                case 35:
                    txtStopKey.Text = "End";
                    break;
                case 34:
                    txtStopKey.Text = "Page Down";
                    break;
                case 19:
                    txtStopKey.Text = "Pause";
                    break;
                case 186:
                    // semicolon
                    txtStopKey.Text = ";";
                    break;
                default:
                    txtStopKey.Text = e.KeyCode.ToString();
                    break;
            }
            txtStopKey.Tag = e.KeyValue.ToString();
        }

        public int WriteINI(string Section, string Key, string Value, string thePath)
        {
            int try_once;
            try
            {
                try_once = WritePrivateProfileString(Section, Key, Value, thePath);
                if (try_once == 0)
                    return 0;
                else
                    return 1;
            }
            catch
            {
                return 0;
            }
        }

        /// <summary>
        /// Read Data Value From the Ini File
        /// </summary>
        /// <param name="Section"></param>
        /// <param name="Key"></param>
        /// <param name="Path"></param>
        /// <returns></returns>
        public string ReadINI(string Section, string Key, string thePath)
        {
            StringBuilder temp = new StringBuilder(255);
            int i = GetPrivateProfileString(Section, Key, "", temp, 255, thePath);
            return temp.ToString();
        }

        public int ReadIntFromINI(string Section, string Key, string thePath, int set_default = -1)
        {
            // this will read an integer from an INI file. If the integer is blank, normally it would
            // error out. However, this function allows the option to set a default value to insert in 
            // case the value is currently blank in the INI.
            StringBuilder temp = new StringBuilder(255);
            int i = GetPrivateProfileString(Section, Key, "", temp, 255, thePath);
            string the_text = temp.ToString();
            try
            {
                return Convert.ToInt32(the_text);
            }
            catch
            {
                if (set_default > -1)
                    WriteINI(Section, Key, set_default.ToString(), thePath);
                else
                    set_default = 0;

                return set_default;
            }

        }

        public void ShowPlaybackForm()
        {
            keyboardHook.Start();
            frmRecording_instance.Show();
        }

        public void TrackNextMouseEvent()
        {
            TrackingNextMouseEventOnly = true;
            mouseHook.Start();
        }

        private void frmMain_Shown(object sender, EventArgs e)
        {
            if (command_line_parameters.Length > 0)
            {
                // we have command line parameters. just show the form.
                this.Hide();
                macro_text = "";
                macro_commands.Clear();
                stop_key = txtStopKey.Text;
                string file_name_only = Path.GetFileName(command_line_parameters);
                frmEditor_instance.Show();
                frmEditor_instance.OpenFile(command_line_parameters, file_name_only);
            }
        }

        private void label3_Click(object sender, EventArgs e)
        {
            string TheURL = "http://keybreeze.com/article/5";
            try
            {
                System.Diagnostics.Process.Start(TheURL);
            }
            catch
            {
            }
        }
    }
}
