﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace KeybreezeMacroEditor
{
    public partial class frmSave : Form
    {
        frmEditor frmEditor_instance;
        public frmSave(frmEditor parent)
        {
            InitializeComponent();
            frmEditor_instance = parent;
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void frmSave_Load(object sender, EventArgs e)
        {
            
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            bool overwrite_existing = false;

            if (txtMacro.Text.Length == 0)
            {
                MessageBox.Show("Please specify a name.");
                return;
            }
            if (txtMacro.Text.Length > 5)
            {
                if (txtMacro.Text.Substring(txtMacro.Text.Length - 5) == ".kmcr")
                {
                    // strip off the extension.
                    txtMacro.Text = txtMacro.Text.Substring(0, txtMacro.Text.Length - 5);
                }
            }
            if (File.Exists(frmEditor_instance.WorkPath2 + "\\macros\\" + txtMacro.Text + ".kmcr") == true)
            {
                DialogResult result;
                result = MessageBox.Show("The macro, " + txtMacro.Text + ", already exists. Do you want to overwrite it?", "Macro already exists.", MessageBoxButtons.YesNo);
                if (result == DialogResult.Yes)
                {
                    overwrite_existing = true;
                }
                else
                {
                    return;
                }
            }
            if (this.Text == "Save macro")
            {
                // just save the macro as a .kcmr file. Do not add it to Keybreeze.
                try
                {
                    System.IO.File.WriteAllText(frmEditor_instance.WorkPath2 + "\\macros\\" + txtMacro.Text + ".kmcr", frmEditor_instance.txtMacro.Text, Encoding.Unicode);
                }
                catch
                {
                    MessageBox.Show("Keybreeze cannot write to the " + frmEditor_instance.WorkPath2 + "\\macros directory. Please check your Windows permissions.");
                    return;
                }
                this.Hide();
                frmEditor_instance.macro_saved_text = frmEditor_instance.txtMacro.Text;
                frmEditor_instance.saved_macro_name = txtMacro.Text;
                frmEditor_instance.Text = txtMacro.Text + ".kcmr" + " - Keybreeze Macro Editor";
                this.Close();
            }
            else if (this.Text == "Save macro as...")
            {
                // user selected the save as... option.
                try
                {
                    System.IO.File.WriteAllText(frmEditor_instance.WorkPath2 + "\\macros\\" + txtMacro.Text + ".kmcr", frmEditor_instance.txtMacro.Text, Encoding.Unicode);
                }
                catch
                {
                    MessageBox.Show("Keybreeze cannot write to the " + frmEditor_instance.WorkPath2 + "\\macros directory. Please check your Windows permissions.");
                    return;
                }
                this.Hide();
                frmEditor_instance.macro_saved_text = frmEditor_instance.txtMacro.Text;
                frmEditor_instance.saved_macro_name = txtMacro.Text;
                frmEditor_instance.Text = txtMacro.Text + ".kcmr" + " - Keybreeze Macro Editor";
                this.Close();
            }
            else
            {
                try
                {
                    System.IO.File.WriteAllText(frmEditor_instance.WorkPath2 + "\\macros\\" + txtMacro.Text + ".kmcr", frmEditor_instance.txtMacro.Text, Encoding.Unicode);
                }
                catch
                {
                    MessageBox.Show("Keybreeze cannot write to the " + frmEditor_instance.WorkPath2 + " directory. Please check your Windows permissions.");
                    return;
                }
                this.Hide();
                frmEditor_instance.macro_saved_text = frmEditor_instance.txtMacro.Text;
                frmEditor_instance.saved_macro_name = txtMacro.Text;
                frmEditor_instance.Text = txtMacro.Text + ".kcmr" + " - Keybreeze Macro Editor";
                this.Close();
            }
        }
    }
}
