﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Keybreeze
{
    public partial class frmAddFolder : Form
    {
        frmFolders frmLabels_instance;
        public frmAddFolder(frmFolders parent)
        {
            InitializeComponent();
            frmLabels_instance = parent;
        }

        private void frmAddLabel_Load(object sender, EventArgs e)
        {
            if (Globals.Keyword_Folders != null)
            {
                foreach (Keyword_Folder this_label in Globals.Keyword_Folders)
                {
                    lstExistingFolders.Items.Add(this_label.path);
                }
                if (lstExistingFolders.Items.Count == 0)
                    txtNestLabelUnder.Enabled = false;
                txtNestLabelUnder.Focus();
            }
            else
            {
                txtNestLabelUnder.Enabled = false;
            }
        }

        private string Get_Parent_Of_Folder(string label_value)
        {
            foreach (Keyword_Folder this_folder in Globals.Keyword_Folders)
            {
                if (this_folder.path.ToLower() == label_value.ToLower())
                {
                    return this_folder.parent;
                }
            }
            return "";
        }

        private void txtNestLabelUnder_TextChanged(object sender, EventArgs e)
        {
            bool found_match = Select_Label_In_List2(txtNestLabelUnder.Text, lstExistingFolders);
        }

        private bool Select_Label_In_List2(string the_text, ListBox this_listbox, bool create_new = false)
        {
            this_listbox.Items.Clear();
            string original_text = the_text;
            bool exact_match = false;
            the_text = the_text.ToLower();
            if (Globals.Keyword_Folders != null)
            {
                if (the_text.Length == 0)
                {
                    // if our text field is blank, we want to show all the labels in the listbox.
                    foreach (Keyword_Folder this_label in Globals.Keyword_Folders)
                    {
                        this_listbox.Items.Add(this_label.path);
                    }
                }
                else
                {
                    foreach (Keyword_Folder this_label in Globals.Keyword_Folders)
                    {
                        if (this_label.path.ToLower() == the_text)
                            exact_match = true;
                        if (this_label.path.Length >= the_text.Length && this_label.path.ToLower().IndexOf(the_text) > -1)
                        {
                            // we found a match. Let's show this in the list.
                            this_listbox.Items.Add(this_label.path);
                        }
                    }
                }
            }
            if (create_new == true && original_text.Length > 0 && exact_match == false)
            {
                this_listbox.Items.Add(original_text + " (create new)");
            }
            if (this_listbox.Items.Count > 0)
                this_listbox.SelectedIndex = 0;
            else
            {
                this_listbox.SelectedIndex = -1;
            }
            return true;
        }

        private bool Select_Label_In_List(string the_text, ListBox this_listbox)
        {
            int list_count = this_listbox.Items.Count;
            // first let's do a starts with search.
            for (int i = 0; i < list_count; i++)
            {
                string this_item = this_listbox.Items[i].ToString().ToLower();
                if (this_item.IndexOf("(create new)") != -1)
                    continue;
                int find_position = this_item.IndexOf(the_text);
                if (find_position == 0)
                {
                    this_listbox.SelectedIndex = i;
                    return true;
                }
                else if (find_position != -1)
                {
                    // our label doesn't appear first, but it may be after a parent, such as parent2/parent1/OUR_LABEL.
                    string char_before = this_item.Substring(find_position - 1, 1);
                    if (char_before == "/")
                    {
                        this_listbox.SelectedIndex = i;
                        return true;
                    }
                }
            }

            // contains search.
            for (int i = 0; i < list_count; i++)
            {
                string this_item = this_listbox.Items[i].ToString().ToLower();
                if (this_item.IndexOf("(create new)") != -1)
                    continue;
                int find_position = this_item.IndexOf(the_text);
                if (find_position != -1)
                {
                    this_listbox.SelectedIndex = i;
                    return true;
                }
            }
            // if we made it this far, we didn't find a matching list item. Return false.
            return false;
        }
        
        private void btnOk_Click(object sender, EventArgs e)
        {
            if (txtName.Text.Length == 0)
            {
                MessageBox.Show("Please specify a name for the folder.", "Keybreeze Desktop");
                return;
            }
            if (txtName.Text.ToLower() == "null")
            {
                MessageBox.Show("Null is not a valid name. Please choose another name.", "Keybreeze Desktop");
                return;
            }
            if (txtName.Text.IndexOf(",") != -1)
            {
                MessageBox.Show("The folder name cannot contain a comma.", "Keybreeze Desktop");
                return;
            }
            if (txtName.Text.IndexOf("/") != -1)
            {
                MessageBox.Show("The folder name cannot contain a slash.", "Keybreeze Desktop");
                return;
            }
            if (txtName.Text.IndexOf(@"\") != -1)
            {
                MessageBox.Show("The folder name cannot contain a slash.", "Keybreeze Desktop");
                return;
            }
            if (txtName.Text.IndexOf("\"") != -1)
            {
                MessageBox.Show("The folder name cannot contain a quote.", "Keybreeze Desktop");
                return;
            }
            // add folder to folder class.
            if (lstExistingFolders.SelectedIndex == -1)
            {
                // no parent folder.
                // does the folder already exist?
                string label_lower = txtName.Text.ToLower();
                foreach (Keyword_Folder this_folder in Globals.Keyword_Folders)
                {
                    if (this_folder.path.ToLower() == label_lower)
                    {
                        MessageBox.Show("The folder name, " + txtName.Text + ", already exists. Please specify a different name.", "Keybreeze Desktop");
                        return;
                    }
                }
                Keyword_Folder new_label = new Keyword_Folder();
                new_label.path = txtName.Text;
                Globals.Keyword_Folders.Add(new_label);
                frmLabels_instance.txtFolderForThisEntry.Text = txtName.Text;
            }
            else
            {
                // parent folder.
                // make sure the folder label exists.
                if (lstExistingFolders.SelectedIndex == -1)
                {
                    MessageBox.Show("The parent folder name you typed does not exist.", "Keybreeze Desktop");
                    return;
                }
                string parent_label = lstExistingFolders.Items[lstExistingFolders.SelectedIndex].ToString();
                bool found_it = false;
                foreach (Keyword_Folder this_folder in Globals.Keyword_Folders)
                {
                    if (this_folder.path.ToLower() == parent_label.ToLower())
                    {
                        found_it = true;
                        break;
                    }
                }
                if (found_it == false)
                {
                    MessageBox.Show("The parent folder name, " + parent_label + ", does not exist.", "Keybreeze Desktop");
                    return;
                }
                string new_folder_path = parent_label + "/" + txtName.Text;
                // make sure the parent folder plus the new folder does not exist.
                foreach (Keyword_Folder this_folder in Globals.Keyword_Folders)
                {
                    if (this_folder.path.ToLower() == new_folder_path.ToLower())
                    {
                        MessageBox.Show("The folder path, " + new_folder_path + ", already exists. Please specify a different name.", "Keybreeze Desktop");
                        return;
                    }
                }
                Keyword_Folder new_folder = new Keyword_Folder();
                new_folder.parent = parent_label;
                new_folder.path = new_folder_path;
                Globals.Keyword_Folders.Add(new_folder);
                frmLabels_instance.txtFolderForThisEntry.Text = new_folder.path;
            }
            frmLabels_instance.txtLabel.Text = "";
            frmLabels_instance.lstExistingLabels.SelectedIndex = -1;
            Globals.Labels_Save_To_File();
            this.Close();
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void btnRemove_Click(object sender, EventArgs e)
        {
            
        }

        private void txtName_TextChanged(object sender, EventArgs e)
        {
            bool found_match = Select_Label_In_List(txtNestLabelUnder.Text, lstExistingFolders);
        }

        private void txtNestLabelUnder_KeyDown(object sender, KeyEventArgs e)
        {
            try
            {
                int current_position = lstExistingFolders.SelectedIndex;
                int total_count = lstExistingFolders.Items.Count;
                if (e.KeyCode == Keys.Down)
                {
                    if (current_position < (total_count - 1))
                        lstExistingFolders.SelectedIndex = current_position + 1;
                    e.Handled = true;
                }
                else if (e.KeyCode == Keys.Up)
                {
                    if (current_position > 0)
                        lstExistingFolders.SelectedIndex = current_position - 1;
                    e.Handled = true;
                }
            }
            catch { }
        }

        private void lstExistingFolders_SelectedValueChanged(object sender, EventArgs e)
        {
            
        }

        private void lstExistingFolders_SelectedIndexChanged(object sender, EventArgs e)
        {
            
        }

        private void lstExistingFolders_DoubleClick(object sender, EventArgs e)
        {
            txtNestLabelUnder.Text = lstExistingFolders.Items[lstExistingFolders.SelectedIndex].ToString();
            txtNestLabelUnder.Focus();
        }
    }
}
