﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Keybreeze
{
    public partial class frmFolders : Form
    {
        public frmAdd frmAdd_instance;
        // selected_folder_guid is used to track the folder in case the user renames it.
        public string selected_folder_guid = "";

        public frmFolders(frmAdd parent)
        {
            InitializeComponent();
            frmAdd_instance = parent;
        }

        private bool Select_Label_In_List(string the_text, ListBox this_listbox, bool create_new = false)
        {
            this_listbox.Items.Clear();
            string original_text = the_text;
            bool exact_match = false;
            the_text = the_text.ToLower();
            if (Globals.Keyword_Folders != null)
            {
                if (the_text.Length == 0)
                {
                    // if our text field is blank, we want to show all the labels in the listbox.
                    foreach (Keyword_Folder this_label in Globals.Keyword_Folders)
                    {
                        this_listbox.Items.Add(this_label.path);
                    }
                }
                else
                {
                    foreach (Keyword_Folder this_label in Globals.Keyword_Folders)
                    {
                        if (this_label.path.ToLower() == the_text)
                            exact_match = true;
                        if (this_label.path.Length >= the_text.Length && this_label.path.ToLower().IndexOf(the_text) > -1)
                        {
                            // we found a match. Let's show this in the list.
                            this_listbox.Items.Add(this_label.path);
                        }
                    }
                }
            }
            if (create_new == true && original_text.Length > 0)
            {
                this_listbox.Items.Add(original_text + " (create new)");
            }
            if (this_listbox.Items.Count > 0)
                this_listbox.SelectedIndex = 0;
            else
            {
                this_listbox.SelectedIndex = -1;
            }
            return true;
        }

        private void txtLabel_TextChanged(object sender, EventArgs e)
        {
            bool found_match = Select_Label_In_List(txtLabel.Text, lstExistingLabels, true);
        }

        private void frmLabels_Load(object sender, EventArgs e)
        {
            if (Globals.Keyword_Folders != null)
            {
                foreach (Keyword_Folder this_label in Globals.Keyword_Folders)
                {
                    lstExistingLabels.Items.Add(this_label.path);
                }
            }
            txtLabel.Focus();
        }

        private string Get_Parent_Of_Folder(string folder_value)
        {
            foreach (Keyword_Folder this_folder in Globals.Keyword_Folders)
            {
                if (this_folder.path.ToLower() == folder_value.ToLower())
                {
                    return this_folder.parent;
                }
            }
            return "";
        }

        private void btnOk_Click(object sender, EventArgs e)
        {
            Click_Ok_Button();
        }

        private void Click_Ok_Button()
        {
            // if anything is selected in the "Existing Labels" listbox, we want to add that label.
            if (lstExistingLabels.SelectedIndex >= 0)
            {
                string selected_item = lstExistingLabels.Items[lstExistingLabels.SelectedIndex].ToString();
                if (selected_item.IndexOf("(create new)") != -1)
                {
                    // we are creating a new label.
                    frmAddFolder frmAddLabel_instance = new frmAddFolder(this);
                    frmAddLabel_instance.txtName.Text = selected_item.Replace(" (create new)", "");

                    frmAddLabel_instance.ShowDialog();
                }
                else
                {
                    // we are adding an existing folder.
                    txtFolderForThisEntry.Text = selected_item;
                    frmAdd_instance.txtFolder.Text = txtFolderForThisEntry.Text;
                    foreach (Keyword_Folder this_folder in Globals.Keyword_Folders)
                    {
                        if (this_folder.path == txtFolderForThisEntry.Text)
                        {
                            frmAdd_instance.selected_folder_guid = this_folder.GUID;
                            break;
                        }
                    }
                    this.Close();
                }
            }
            else
            {
                // populate the labels text field on the "Add keyword" window and close this form.
                frmAdd_instance.txtFolder.Text = txtFolderForThisEntry.Text;
                foreach (Keyword_Folder this_folder in Globals.Keyword_Folders)
                {
                    if (this_folder.path == txtFolderForThisEntry.Text)
                    {
                        frmAdd_instance.selected_folder_guid = this_folder.GUID;
                        break;
                    }
                }
                this.Close();
            }
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void btnRemove_Click(object sender, EventArgs e)
        {
            try
            {
                txtFolderForThisEntry.Text = "";
            }
            catch
            {
            }
        }

        private void lstExistingLabels_DoubleClick(object sender, EventArgs e)
        {
            Click_Ok_Button();
        }

        private void txtLabel_KeyDown(object sender, KeyEventArgs e)
        {
            try
            {
                int current_position = lstExistingLabels.SelectedIndex;
                int total_count = lstExistingLabels.Items.Count;
                if (e.KeyCode == Keys.Down)
                {
                    if (current_position < (total_count - 1))
                        lstExistingLabels.SelectedIndex = current_position + 1;
                }
                else if (e.KeyCode == Keys.Up)
                {
                    if (current_position > 0)
                        lstExistingLabels.SelectedIndex = current_position - 1;
                }
                e.Handled = true;
            }
            catch { }
        }

        private void btnManageFolders_Click(object sender, EventArgs e)
        {
            // get the GUID of this folder if one is selected.
            if (txtFolderForThisEntry.Text.Length > 0)
            {
                foreach (Keyword_Folder this_folder in Globals.Keyword_Folders)
                {
                    if (this_folder.path == txtFolderForThisEntry.Text)
                    {
                        selected_folder_guid = this_folder.GUID;
                    }
                }
            }
            frmManageFolders frmManageFolders_instance = new frmManageFolders(this, frmAdd_instance);
            // load all folders into the datagridview.
            frmManageFolders_instance.Manage_Setup_Columns(frmManageFolders_instance.dataGridView1);
            frmManageFolders_instance.Add_Folders(frmManageFolders_instance.dataGridView1);
            frmManageFolders_instance.ShowDialog();
        }

        public void Refresh_Folders()
        {
            // this function will reload the folders after the user closes the manage folders window.
            lstExistingLabels.Items.Clear();
            if (Globals.Keyword_Folders != null)
            {
                foreach (Keyword_Folder this_label in Globals.Keyword_Folders)
                {
                    lstExistingLabels.Items.Add(this_label.path);
                }
            }
        }

        private void lstExistingLabels_SelectedIndexChanged(object sender, EventArgs e)
        {

        }
    }
}
