﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Keybreeze
{
    public partial class frmManageFolders : Form
    {
        public frmFolders frmFolders_instance;
        public frmAdd frmAdd_instance;

        public frmManageFolders(frmFolders parent, frmAdd parent2)
        {
            InitializeComponent();
            frmFolders_instance = parent;
            frmAdd_instance = parent2;
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        public void Manage_Setup_Columns(DataGridView this_datagridview)
        {
            this_datagridview.ColumnCount = 1;
            this_datagridview.Columns[0].Name = "Path";
            this_datagridview.Columns[0].Width = 625;
            this_datagridview.RowHeadersWidth = 30;
        }

        public void Add_Folders(DataGridView this_data_grid_view)
        {
            this_data_grid_view.Rows.Clear();
            foreach (Keyword_Folder this_folder in Globals.Keyword_Folders)
            {
                string[] row;
                row = new string[] { this_folder.path };
                this_data_grid_view.Rows.Add(row);
            }
            this_data_grid_view.Sort(this.dataGridView1.Columns["Path"], ListSortDirection.Ascending);
        }

        private void btnRename_Click(object sender, EventArgs e)
        {
            int current_position = dataGridView1.CurrentRow.Index;
            string selected_path = dataGridView1.Rows[current_position].Cells[0].Value.ToString();
            frmRenameFolder frmRenameFolder_instance = new frmRenameFolder(this);
            frmRenameFolder_instance.old_folder_path = selected_path;
            frmRenameFolder_instance.Text = "Rename folder - " + selected_path;
            frmRenameFolder_instance.label1.Text = "Enter a new name for the folder, " + selected_path + ".";
            // we need to see if this folder is a nested folder, like work/backup/phone or just a single folder, like 'work'.
            if (selected_path.IndexOf("/") == -1)
            {
                frmRenameFolder_instance.lblNewName.Text = "New name: ";
                frmRenameFolder_instance.new_folder_name_for_label = "";
            }
            else
            {
                frmRenameFolder_instance.lblNewName.Text = "New name: " + selected_path.Substring(0, selected_path.LastIndexOf("/") + 1);
                frmRenameFolder_instance.new_folder_name_for_label = selected_path.Substring(0, selected_path.LastIndexOf("/") + 1);
            }
            frmRenameFolder_instance.ShowDialog();
        }

        private void btnDelete_Click(object sender, EventArgs e)
        {
            int current_position = dataGridView1.CurrentRow.Index;
            string selected_path = dataGridView1.Rows[current_position].Cells[0].Value.ToString();
            frmRemoveFolder frmRemoveFolder_instance = new frmRemoveFolder(this);
            frmRemoveFolder_instance.Remove_Setup_Columns(frmRemoveFolder_instance.dataGridView1);
            frmRemoveFolder_instance.folder_path_to_delete = selected_path;
            frmRemoveFolder_instance.Text = "Delete folder: " + selected_path;
            // check for subfolders within this folder.
            Add_Subfolders_To_Datagridview(selected_path, frmRemoveFolder_instance.dataGridView1);
            frmRemoveFolder_instance.ShowDialog();
        }

        private void Add_Subfolders_To_Datagridview(string selected_folder, DataGridView this_data_grid_view)
        {
            // first let's look for any entries that use our selected_folder.
            Add_Entries_To_Datagridview(selected_folder, this_data_grid_view);
            // look for any subfolders of our selected_folder.
            foreach (Keyword_Folder this_folder in Globals.Keyword_Folders)
            {
                if (this_folder.parent == selected_folder)
                {
                    string[] row;
                    row = new string[] { this_folder.path, "Keybreeze folder", this_folder.path, "N/A" };
                    this_data_grid_view.Rows.Add(row);
                    // check for entries within this folder.
                    Add_Entries_To_Datagridview(this_folder.path, this_data_grid_view);
                    // call this function recursively to get subfolders that will also be deleted.
                    Add_Subfolders_To_Datagridview(this_folder.path, this_data_grid_view);
                }
            }
        }

        private void Add_Entries_To_Datagridview(string selected_path, DataGridView this_data_grid_view)
        {
            string selected_path_lower = selected_path.ToLower();
            foreach (Keyword this_keyword in Globals.Keyword_List)
            {
                if (this_keyword.path.ToLower() == selected_path_lower)
                {
                    string current_type;
                    if (this_keyword.target.Length > 9 && this_keyword.target.IndexOf("function:") == 0)
                        current_type = "text function";
                    else if (this_keyword.target == "macro")
                        current_type = "macro";
                    else
                        current_type = "keyword";
                    string[] row;
                    row = new string[] { this_keyword.name, current_type, selected_path, this_keyword.target };
                    this_data_grid_view.Rows.Add(row);
                }
            }
        }

        private void frmManageFolders_Load(object sender, EventArgs e)
        {

        }

        private void frmManageFolders_FormClosing(object sender, FormClosingEventArgs e)
        {
            
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Please add a folder on the previous screen. Type a folder name in the text field and select the (create new) option.", "Keybreeze Desktop");
        }
    }
}
