﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace Keybreeze
{
    public partial class frmRemoveFolder : Form
    {
        public frmManageFolders frmManageFolders_instance;
        public string folder_path_to_delete = "";

        public frmRemoveFolder(frmManageFolders parent)
        {
            InitializeComponent();
            frmManageFolders_instance = parent;
        }

        private void btnDoNotDelete_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void label2_Click(object sender, EventArgs e)
        {

        }

        public void Remove_Setup_Columns(DataGridView this_datagridview)
        {
            this_datagridview.ColumnCount = 4;
            this_datagridview.Columns[0].Name = "Name";
            this_datagridview.Columns[1].Name = "Type";
            this_datagridview.Columns[2].Name = "Path";
            this_datagridview.Columns[3].Name = "Target";
            this_datagridview.Columns[0].Width = 175;
            this_datagridview.Columns[1].Width = 100;
            this_datagridview.Columns[2].Width = 100;
            this_datagridview.Columns[3].Width = 500;
            this_datagridview.RowHeadersWidth = 30;
        }

        private void frmRemoveFolder_Load(object sender, EventArgs e)
        {

        }

        private void btnDelete_Click(object sender, EventArgs e)
        {
            // loop through items that are associated with this folder.
            int folder_count = 0;
            for (int i = 0; i < dataGridView1.RowCount; i++)
            {
                if (dataGridView1.Rows[i].Cells[0].Value == null)
                    continue;
                string this_name = dataGridView1.Rows[i].Cells[0].Value.ToString();
                string this_type = dataGridView1.Rows[i].Cells[1].Value.ToString();
                string this_path = dataGridView1.Rows[i].Cells[2].Value.ToString();
                string this_target = dataGridView1.Rows[i].Cells[3].Value.ToString();
                if (this_type == "Keybreeze folder")
                {
                    folder_count = Globals.Keyword_Folders.Count();
                    for (int j = 0; j < folder_count; j++)
                    {
                        if (Globals.Keyword_Folders[j].path == this_name)
                        {
                            // remove this subfolders since it's associated with the folder we are deleting.
                            Globals.Keyword_Folders.RemoveAt(j);
                            break;
                        }
                    }
                }
                else
                {
                    for (int j = 0; j < Globals.keyword_count; j++)
                    {
                        if (Globals.Keyword_List[j].name == this_name)
                        {
                            // disassociate this keyword from the folder.
                            // is it a text function or macro? If so, we need to rename some files as well.
                            if (this_type == "text function")
                            {
                                string old_text_function_name = "";
                                string new_text_function_name = "";
                                try
                                {
                                    old_text_function_name = Globals.WorkPath + "\\text functions\\" + Globals.replace_invalid_filename_characters(this_target.Substring(9) + ".txt", this_path);
                                    new_text_function_name = Globals.WorkPath + "\\text functions\\" + Globals.replace_invalid_filename_characters(this_target.Substring(9) + ".txt", "");
                                    File.Move(old_text_function_name, new_text_function_name);
                                }
                                catch (Exception e2)
                                {
                                    MessageBox.Show("A text function in this folder, " + old_text_function_name + ", could not be renamed to " + new_text_function_name + ". " + e2.Message, "Keybreeze Desktop");
                                }
                            }
                            else if (this_type == "macro")
                            {
                                string old_macro_name = "";
                                string new_macro_name = "";
                                try
                                {
                                    old_macro_name = Globals.WorkPath + "\\macros\\" + Globals.replace_invalid_filename_characters(this_name + ".kmcr", this_path);
                                    new_macro_name = Globals.WorkPath + "\\macros\\" + Globals.replace_invalid_filename_characters(this_name + ".kmcr", "");
                                    File.Move(old_macro_name, new_macro_name);
                                }
                                catch (Exception e2)
                                {
                                    MessageBox.Show("A macro in this folder, " + old_macro_name + ", could not be renamed to the new folder, " + new_macro_name + ". " + e2.Message, "Keybreeze Desktop");
                                }
                            }
                            // finally, disassociate this keyword from the folder.
                            Globals.Keyword_List[j].path = "";
                            break;
                        }
                    }
                }
            }
            // finally, remove our folder.
            folder_count = Globals.Keyword_Folders.Count();
            for (int j = 0; j < folder_count; j++)
            {
                if (Globals.Keyword_Folders[j].path == folder_path_to_delete)
                {
                    Globals.Keyword_Folders.RemoveAt(j);
                    break;
                }
            }
            Globals.Keywords_Save_To_File();
            Globals.Labels_Save_To_File();
            // we have to delete the folder on the previous screens (frmManageFolders and frmFolders)
            frmManageFolders_instance.Add_Folders(frmManageFolders_instance.dataGridView1);
            // update the frmFolders window and Add keyword window.
            frmManageFolders_instance.frmFolders_instance.lstExistingLabels.Items.Clear();
            foreach (Keyword_Folder this_folder in Globals.Keyword_Folders)
            {
                frmManageFolders_instance.frmFolders_instance.lstExistingLabels.Items.Add(this_folder.path);
            }
            if (frmManageFolders_instance.frmFolders_instance.txtFolderForThisEntry.Text == folder_path_to_delete)
            {
                // remove this folder from the entry since we just deleted it.
                frmManageFolders_instance.frmFolders_instance.txtFolderForThisEntry.Text = "";
                frmManageFolders_instance.frmFolders_instance.selected_folder_guid = "";
            }
            if (frmManageFolders_instance.frmFolders_instance.frmAdd_instance.txtFolder.Text == folder_path_to_delete)
            {
                frmManageFolders_instance.frmFolders_instance.frmAdd_instance.txtFolder.Text = "";
                frmManageFolders_instance.frmFolders_instance.frmAdd_instance.selected_folder_guid = "";
            }
            this.Close();
        }
    }
}
