﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace Keybreeze
{
    public partial class frmRenameFolder : Form
    {
        frmManageFolders frmManageFolder_instance;
        public string old_folder_path = "";
        public string new_folder_name_for_label = "";

        public frmRenameFolder(frmManageFolders parent)
        {
            InitializeComponent();
            frmManageFolder_instance = parent;
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void frmRenameFolder_Load(object sender, EventArgs e)
        {

        }

        private void btnOk_Click(object sender, EventArgs e)
        {
            string new_folder_name = txtNewFolderName.Text;
            if (new_folder_name.Length == 0)
            {
                MessageBox.Show("Please specify a name for the folder.", "Keybreeze Desktop");
                return;
            }
            if (new_folder_name.ToLower() == "null")
            {
                MessageBox.Show("Null is not a valid name. Please choose another name.", "Keybreeze Desktop");
                return;
            }
            if (new_folder_name.IndexOf(",") != -1)
            {
                MessageBox.Show("The folder name cannot contain a comma.", "Keybreeze Desktop");
                return;
            }
            if (new_folder_name.IndexOf("/") != -1)
            {
                MessageBox.Show("The folder name cannot contain a slash.", "Keybreeze Desktop");
                return;
            }
            if (new_folder_name.IndexOf(@"\") != -1)
            {
                MessageBox.Show("The folder name cannot contain a slash.", "Keybreeze Desktop");
                return;
            }
            if (new_folder_name.IndexOf("\"") != -1)
            {
                MessageBox.Show("The folder name cannot contain a quote.", "Keybreeze Desktop");
                return;
            }
            string new_folder_path = lblNewName.Text.Substring(lblNewName.Text.IndexOf(":") + 2);
            // if we are here, rename the folder. First check to make sure that the new path doesn't already exist.
            int folder_count = Globals.Keyword_Folders.Count;
            for (int i = 0; i < folder_count; i++)
            {
                if (Globals.Keyword_Folders[i].path.ToLower() == new_folder_path.ToLower())
                {
                    MessageBox.Show("The folder name you specified, " + new_folder_path + ", already exists. Please choose a different name.", "Keybreeze Desktop");
                    return;
                }
            }
            for (int i = 0; i < folder_count; i++)
            {
                if (Globals.Keyword_Folders[i].path.ToLower() == old_folder_path.ToLower())
                {
                    // we found our folder to rename.
                    Globals.Keyword_Folders[i].path = new_folder_path;
                    break;
                }
            }
            // now we need to rename all of our folders that have this one as a parent folder.
            Rename_Children_Folders(old_folder_path, new_folder_path);
            Globals.Labels_Save_To_File();
            Globals.Keywords_Save_To_File();
            // we have to rename the folder on the previous screens (frmManageFolders and frmFolders.
            // update the list for the Manage Folders screen first.
            frmManageFolder_instance.Add_Folders(frmManageFolder_instance.dataGridView1);
            // update the frmFolders window and Add keyword window.
            frmManageFolder_instance.frmFolders_instance.lstExistingLabels.Items.Clear();
            foreach (Keyword_Folder this_folder in Globals.Keyword_Folders)
            {
                frmManageFolder_instance.frmFolders_instance.lstExistingLabels.Items.Add(this_folder.path);
                if (this_folder.GUID == frmManageFolder_instance.frmFolders_instance.selected_folder_guid)
                {
                    frmManageFolder_instance.frmFolders_instance.txtFolderForThisEntry.Text = this_folder.path;
                }
                if (this_folder.GUID == frmManageFolder_instance.frmFolders_instance.frmAdd_instance.selected_folder_guid)
                {
                    frmManageFolder_instance.frmFolders_instance.frmAdd_instance.txtFolder.Text = this_folder.path;
                    frmManageFolder_instance.frmFolders_instance.frmAdd_instance.txtFolder.Tag = this_folder.path;
                }
            }
            this.Close();
        }

        private void Rename_Children_Folders(string old_path, string new_path)
        {
            /*
                This function will rename all children folders with the updated path.
             *  Example:
             *  
             *  home
             *  home/music
             *  home/photos
             *  home/backup
             *  home/backup/computer
             *  home/backup/computer/windows
             *  
             *  When renaming home, it will look at all folders with 'home' as the parent folder, rename those, then run recursively to get folders below that.
             */

            // we need to rename this folder to the new path, but first we need to check child folders for this folder as well.
            int folder_count = Globals.Keyword_Folders.Count;
            for (int i = 0; i < folder_count; i++)
            {
                if (Globals.Keyword_Folders[i].parent != null && Globals.Keyword_Folders[i].parent.ToLower() == old_path.ToLower())
                {
                    string this_child_folder = Globals.Keyword_Folders[i].path;
                    Rename_Children_Folders(this_child_folder, ReplaceFirst(this_child_folder, old_path, new_path));
                    // now we renamed the children folders, we need to rename the parent folder.
                    Globals.Keyword_Folders[i].parent = new_path;
                }
            }
            // now we are ready to rename this folder to the new path.
            for (int i = 0; i < folder_count; i++)
            {
                if (Globals.Keyword_Folders[i].path == old_path)
                {
                    Globals.Keyword_Folders[i].path = new_path;
                }
            }
            // now we need to search our keywords and update those paths as well.
            foreach (Keyword this_keyword in Globals.Keyword_List)
            {
                if (this_keyword.path == old_path)
                {
                    this_keyword.path = new_path;
                    // if this keyword is a text function or macro, we also need to update the filename.
                    if (this_keyword.target.ToLower().IndexOf("function:") == 0)
                    {
                        string old_text_function_name = "";
                        string new_text_function_name = "";
                        try
                        {
                            old_text_function_name = Globals.WorkPath + "\\text functions\\" + Globals.replace_invalid_filename_characters(this_keyword.target.Substring(9) + ".txt", old_path);
                            new_text_function_name = Globals.WorkPath + "\\text functions\\" + Globals.replace_invalid_filename_characters(this_keyword.target.Substring(9) + ".txt", new_path);
                            File.Move(old_text_function_name, new_text_function_name);
                            this_keyword.path = new_path;
                        }
                        catch (Exception e)
                        {
                            MessageBox.Show("A text function in this folder, " + old_text_function_name + ", could not be renamed to the new folder, " + new_text_function_name + ". " + e.Message, "Keybreeze Desktop");
                        }
                    }
                    else if (this_keyword.target.ToLower() == "macro")
                    {
                        string old_macro_name = "";
                        string new_macro_name = "";
                        try
                        {
                            old_macro_name = Globals.WorkPath + "\\macros\\" + Globals.replace_invalid_filename_characters(this_keyword.name + ".kmcr", old_path);
                            new_macro_name = Globals.WorkPath + "\\macros\\" + Globals.replace_invalid_filename_characters(this_keyword.name + ".kmcr", new_path);
                            File.Move(old_macro_name, new_macro_name);
                            this_keyword.path = new_path;
                        }
                        catch (Exception e)
                        {
                            MessageBox.Show("A macro in this folder, " + old_macro_name + ", could not be renamed to the new folder, " + new_macro_name + ". " + e.Message, "Keybreeze Desktop");
                        }
                    }
                    else
                    {
                        // it's a regular keyword. we just need to update the path.
                        this_keyword.path = new_path;
                    }
                }
            }
        }

        private string ReplaceFirst(string text, string search, string replace)
        {
            int pos = text.IndexOf(search);
            if (pos < 0)
            {
                return text;
            }
            return text.Substring(0, pos) + replace + text.Substring(pos + search.Length);
        }

        private void txtNewFolderName_TextChanged(object sender, EventArgs e)
        {
            lblNewName.Text = "New name: " + new_folder_name_for_label + txtNewFolderName.Text;
        }
    }
}
