/*
Keybreeze Desktop
Copyright  2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.IO;
using System.Runtime.InteropServices;
using System.Text;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Media;
using System.Diagnostics;
using System.Threading;
using Newtonsoft.Json;

namespace Keybreeze
{
    // this class holds global variables and functions that can be used anywhere in the program.
    public static class Globals
    {
        [DllImport("kernel32")]
        private static extern int GetPrivateProfileString(string section, string key, string def, StringBuilder retVal, int size, string filePath);

        [DllImport("KERNEL32.DLL", EntryPoint = "WritePrivateProfileStringW",
          SetLastError = true,
          CharSet = CharSet.Unicode, ExactSpelling = true,
          CallingConvention = CallingConvention.StdCall)]
        private static extern int WritePrivateProfileString(
          string lpAppName,
          string lpKeyName,
          string lpString,
          string lpFilename);

        [DllImport("user32.dll")]
        public static extern IntPtr FindWindow(String sClassName, String sAppName);

        // functions for moving windows.
        [DllImport("user32.dll", SetLastError = true)]
        [return: MarshalAs(UnmanagedType.Bool)]
        public static extern bool GetWindowRect(IntPtr hWnd, out RECT lpRect);
        [StructLayout(LayoutKind.Sequential)]
        public struct RECT
        {
            public int Left;
            public int Top;
            public int Right;
            public int Bottom;
        }

        // set global variables
        // SET BOTH NUMBERS BELOW! You must also set the version in AssemblyInfo.cs file, update the release date in the Help > About window, updatecheck.exe, and the version in all other executable files.
        public static string version_string = "5.7.0";
        public static int version_numeric = 570;
        public static string new_note_text = "Type text here. Double-click the note to toggle between lock mode and edit mode. Right-click in lock mode to snooze, keep on top of other windows, or set other options.";
        public static string command_line_parameters = "";
        public static string InstallDir = Path.GetDirectoryName(Application.ExecutablePath);
        public static string WorkPath { get; set; }
        public static string OptionsINI { get; set; }
        public static string NotesINI { get; set; }
        public static string KeywordsJSONFile { get; set; }
        public static string FoldersJSONFile { get; set; }
        public static string LetterCommandsJSONFile { get; set; }
        public static int keyword_count { get; set; }
        
        public static bool valid_skin = true;
        public static string SkinFolder;
        public static string SkinINI;
        // if skin_icon_type = 1, use big icons. if 2, small icons. if 0, no icons.
        public static int skin_icon_type = 1;
        // if skin is embedded, it means it's part of the main form... the one with the textbox. In this case, we do not load frmSkin. We just set the picture as the form's background.
        public static int skin_embedded = 0;
        public static int main_textbox_left = 24;
        public static int main_textbox_top = 26;
        public static int skin_top_offset = 0;
        public static int skin_left_offset = 0;
        // the following two settings are for the popup window.
        public static int top_offset = 0;
        public static int left_offset = 0;
        public static int center_top_offset = 0;
        public static int center_left_offset = 0;

        public static bool sound_done_looping = false;
        public static int logging = 0;
        public static int description_character_limit = 0;

        public static int show_keyboard_help = Globals.ReadIntFromINI("options", "show_keyboard_help", Globals.OptionsINI, 1);

        public static int when_pressing_shift;
        public static int when_pressing_tab;
        public static int when_pressing_ctrl;

        // create keyword list
        //public List<keyword> keywords = new List<keyword>();
        
        public static List<string> autoindex_files = new List<string>();
        public static List<string> autoindex_filepaths = new List<string>();
        public static List<string> autoindex_icons = new List<string>();
        public static List<string> autoindex_blacklist = new List<string>();
        public static List<string> autoindex_folders_on_demand = new List<string>();

        public static List<History> History_List = new List<History>();
        public static int history_position = -1;

        public static List<string> autocomplete_list = new List<string>();
        public static List<int> keyword_icon = new List<int>();
        // we are going to create an array that shows the first occurrence of an ascii character
        // in the keyword list.
        // e.g. keyword_ascii[2] = 6   would indicate that the sixth keyword contains the first
        // keyword that starts with a "c".
        public static int[] keyword_ascii = new int[25];

        // public list for Notes
        public static List<Note> notelist = new List<Note>();
        public static string note_theme = "";

        // public list for Keywords
        public static List<Keyword> Keyword_List = new List<Keyword>();
        public static List<string> keyword_name_lower = new List<string>();
        public static List<Keyword_Folder> Keyword_Folders = new List<Keyword_Folder>();
        public static List<Letter_Command> Letter_Commands = new List<Letter_Command>();
        public static List<string> local_drives = new List<string>();

        // the following section is for resolving full paths for .lnk files
        [DllImport("shfolder.dll", CharSet = CharSet.Auto)] internal static extern int SHGetFolderPath(IntPtr hwndOwner, int nFolder, IntPtr hToken, int dwFlags, StringBuilder lpszPath);
        [Flags()]
        enum SLGP_FLAGS
        {
            /// <summary>Retrieves the standard short (8.3 format) file name</summary>
            SLGP_SHORTPATH = 0x1,
            /// <summary>Retrieves the Universal Naming Convention (UNC) path name of the file</summary>
            SLGP_UNCPRIORITY = 0x2,
            /// <summary>Retrieves the raw path name. A raw path is something that might not exist and may include environment variables that need to be expanded</summary>
            SLGP_RAWPATH = 0x4
        }

        [DllImport("user32.dll")]
        static extern int SetWindowLong(IntPtr hWnd, int nIndex, int dwNewLong);

        [DllImport("user32.dll", SetLastError = true)]
        private static extern System.UInt32 GetWindowLong(IntPtr hWnd, int nIndex);

        [DllImport("user32.dll")]
        static extern bool SetLayeredWindowAttributes(IntPtr hwnd, uint crKey, byte bAlpha, uint dwFlags);

        [DllImport("user32.dll", EntryPoint = "SetWindowPos")]
        public static extern IntPtr SetWindowPos(IntPtr hWnd, int hWndInsertAfter, int x, int Y, int cx, int cy, int wFlags);

        const int GWL_EXSTYLE = -20;
        const int WS_EX_LAYERED = 0x80000;
        const int LWA_ALPHA = 0x2;
        const int LWA_COLORKEY = 0x1;
        const int HWND_TOPMOST = -1;
        const int HWND_NOTOPMOST = -2;
        const int HWND_BOTTOM = 1;
        const int SWP_NOACTIVATE = 0x0010;
        const int SWP_NOMOVE = 0x0002;
        const int SWP_NOSIZE = 0x0001;

        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Auto)]
        struct WIN32_FIND_DATAW
        {
            public uint dwFileAttributes;
            public long ftCreationTime;
            public long ftLastAccessTime;
            public long ftLastWriteTime;
            public uint nFileSizeHigh;
            public uint nFileSizeLow;
            public uint dwReserved0;
            public uint dwReserved1;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 260)]
            public string cFileName;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 14)]
            public string cAlternateFileName;
        }

        [Flags()]
        enum SLR_FLAGS
        {
            /// <summary>
            /// Do not display a dialog box if the link cannot be resolved. When SLR_NO_UI is set,
            /// the high-order word of fFlags can be set to a time-out value that specifies the
            /// maximum amount of time to be spent resolving the link. The function returns if the
            /// link cannot be resolved within the time-out duration. If the high-order word is set
            /// to zero, the time-out duration will be set to the default value of 3,000 milliseconds
            /// (3 seconds). To specify a value, set the high word of fFlags to the desired time-out
            /// duration, in milliseconds.
            /// </summary>
            SLR_NO_UI = 0x1,
            /// <summary>Obsolete and no longer used</summary>
            SLR_ANY_MATCH = 0x2,
            /// <summary>If the link object has changed, update its path and list of identifiers.
            /// If SLR_UPDATE is set, you do not need to call IPersistFile::IsDirty to determine
            /// whether or not the link object has changed.</summary>
            SLR_UPDATE = 0x4,
            /// <summary>Do not update the link information</summary>
            SLR_NOUPDATE = 0x8,
            /// <summary>Do not execute the search heuristics</summary>
            SLR_NOSEARCH = 0x10,
            /// <summary>Do not use distributed link tracking</summary>
            SLR_NOTRACK = 0x20,
            /// <summary>Disable distributed link tracking. By default, distributed link tracking tracks
            /// removable media across multiple devices based on the volume name. It also uses the
            /// Universal Naming Convention (UNC) path to track remote file systems whose drive letter
            /// has changed. Setting SLR_NOLINKINFO disables both types of tracking.</summary>
            SLR_NOLINKINFO = 0x40,
            /// <summary>Call the Microsoft Windows Installer</summary>
            SLR_INVOKE_MSI = 0x80
        }


        /// <summary>The IShellLink interface allows Shell links to be created, modified, and resolved</summary>
        [ComImport(), InterfaceType(ComInterfaceType.InterfaceIsIUnknown), Guid("000214F9-0000-0000-C000-000000000046")]
        interface IShellLinkW
        {
            /// <summary>Retrieves the path and file name of a Shell link object</summary>
            void GetPath([Out(), MarshalAs(UnmanagedType.LPWStr)] StringBuilder pszFile, int cchMaxPath, out WIN32_FIND_DATAW pfd, SLGP_FLAGS fFlags);
            /// <summary>Retrieves the list of item identifiers for a Shell link object</summary>
            void GetIDList(out IntPtr ppidl);
            /// <summary>Sets the pointer to an item identifier list (PIDL) for a Shell link object.</summary>
            void SetIDList(IntPtr pidl);
            /// <summary>Retrieves the description string for a Shell link object</summary>
            void GetDescription([Out(), MarshalAs(UnmanagedType.LPWStr)] StringBuilder pszName, int cchMaxName);
            /// <summary>Sets the description for a Shell link object. The description can be any application-defined string</summary>
            void SetDescription([MarshalAs(UnmanagedType.LPWStr)] string pszName);
            /// <summary>Retrieves the name of the working directory for a Shell link object</summary>
            void GetWorkingDirectory([Out(), MarshalAs(UnmanagedType.LPWStr)] StringBuilder pszDir, int cchMaxPath);
            /// <summary>Sets the name of the working directory for a Shell link object</summary>
            void SetWorkingDirectory([MarshalAs(UnmanagedType.LPWStr)] string pszDir);
            /// <summary>Retrieves the command-line arguments associated with a Shell link object</summary>
            void GetArguments([Out(), MarshalAs(UnmanagedType.LPWStr)] StringBuilder pszArgs, int cchMaxPath);
            /// <summary>Sets the command-line arguments for a Shell link object</summary>
            void SetArguments([MarshalAs(UnmanagedType.LPWStr)] string pszArgs);
            /// <summary>Retrieves the hot key for a Shell link object</summary>
            void GetHotkey(out short pwHotkey);
            /// <summary>Sets a hot key for a Shell link object</summary>
            void SetHotkey(short wHotkey);
            /// <summary>Retrieves the show command for a Shell link object</summary>
            void GetShowCmd(out int piShowCmd);
            /// <summary>Sets the show command for a Shell link object. The show command sets the initial show state of the window.</summary>
            void SetShowCmd(int iShowCmd);
            /// <summary>Retrieves the location (path and index) of the icon for a Shell link object</summary>
            void GetIconLocation([Out(), MarshalAs(UnmanagedType.LPWStr)] StringBuilder pszIconPath,
                int cchIconPath, out int piIcon);
            /// <summary>Sets the location (path and index) of the icon for a Shell link object</summary>
            void SetIconLocation([MarshalAs(UnmanagedType.LPWStr)] string pszIconPath, int iIcon);
            /// <summary>Sets the relative path to the Shell link object</summary>
            void SetRelativePath([MarshalAs(UnmanagedType.LPWStr)] string pszPathRel, int dwReserved);
            /// <summary>Attempts to find the target of a Shell link, even if it has been moved or renamed</summary>
            void Resolve(IntPtr hwnd, SLR_FLAGS fFlags);
            /// <summary>Sets the path and file name of a Shell link object</summary>
            void SetPath([MarshalAs(UnmanagedType.LPWStr)] string pszFile);

        }

        [ComImport, Guid("0000010c-0000-0000-c000-000000000046"),
        InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
        public interface IPersist
        {
            [PreserveSig]
            void GetClassID(out Guid pClassID);
        }


        [ComImport, Guid("0000010b-0000-0000-C000-000000000046"),
        InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
        public interface IPersistFile : IPersist
        {
            new void GetClassID(out Guid pClassID);
            [PreserveSig]
            int IsDirty();

            [PreserveSig]
            void Load([In, MarshalAs(UnmanagedType.LPWStr)]
            string pszFileName, uint dwMode);

            [PreserveSig]
            void Save([In, MarshalAs(UnmanagedType.LPWStr)] string pszFileName,
                [In, MarshalAs(UnmanagedType.Bool)] bool fRemember);

            [PreserveSig]
            void SaveCompleted([In, MarshalAs(UnmanagedType.LPWStr)] string pszFileName);

            [PreserveSig]
            void GetCurFile([In, MarshalAs(UnmanagedType.LPWStr)] string ppszFileName);
        }

        const uint STGM_READ = 0;
        const int MAX_PATH = 260;

        // CLSID_ShellLink from ShlGuid.h 
        [
            ComImport(),
            Guid("00021401-0000-0000-C000-000000000046")
        ]
        public class ShellLink
        {
        }

        public static string ResolveShortcut(string filename)
        {
            // this gets the full path from a shortcut (.lnk file).
            ShellLink link = new ShellLink();
            ((IPersistFile)link).Load(filename, STGM_READ);
            StringBuilder sb = new StringBuilder(MAX_PATH);
            WIN32_FIND_DATAW data = new WIN32_FIND_DATAW();
            ((IShellLinkW)link).GetPath(sb, sb.Capacity, out data, 0);
            string final_string = sb.ToString();
            if (final_string.Length == 0)
                final_string = filename;
            // If the the shortcut's target resolves to the Program Files or System32 directory, and the user is on a
            // 64-bit machine, the final string may actually point to C:\Program Files (x86) or C:\Windows\SYSWOW64.
            // This is due to File System Redirection in Windows -- http://msdn.microsoft.com/en-us/library/aa365743%28VS.85%29.aspx.
            // Unfortunately the solution there doesn't appear to work for 32-bit apps on 64-bit machines.
            // We will provide a workaround here:
            string new_path = Validate_Shortcut_Path(final_string, "SysWOW64", "System32");
            if (File.Exists(new_path) == true && File.Exists(final_string) == false)
            {
                // the file is actually stored in System32 instead of SysWOW64. Let's update it.
                final_string = new_path;
            }
            new_path = Validate_Shortcut_Path(final_string, "Program Files (x86)", "Program Files");
            if (File.Exists(new_path) == true && File.Exists(final_string) == false)
            {
                // the file is actually stored in Program Files instead of Program Files (x86). Let's update it.
                final_string = new_path;
            }
            // the lnk may incorrectly resolve to the C:\Windows\Installer directory. Check for this.
            if (final_string.ToLower().IndexOf("windows\\installer") > -1)
                final_string = filename;
            if (File.Exists(final_string))
                return final_string;
            else
                return filename;
        }

        public static string Validate_Shortcut_Path(string final_string, string find_what, string replace_with)
        {
            string final_string_lower = final_string.ToLower();
            string find_what_lower = find_what.ToLower();
            int find_value = final_string_lower.IndexOf(find_what_lower);
            if (find_value > -1)
            {
                // the shortcut resolved to the find_what directory, which can be SysWOW64 or Program Files (x86), 
                // but this may not be correct. Let's check by replacing it with another value.
                string new_string = final_string.Substring(0, find_value) + replace_with + final_string.Substring(find_value + find_what.Length);
                if (File.Exists(new_string) == true && File.Exists(final_string) == false)
                {
                    // the file is actually stored at a different location. Let's update it.
                    final_string = new_string;
                }
            }
            return final_string;
        }


        public static int WriteINI(string Section, string Key, string Value, string thePath)
        {
            int try_once;
            try
            {
                try_once = WritePrivateProfileString(Section, Key, Value, thePath);
                if (try_once == 0)
                    return 0;
                else
                    return 1;
            }
            catch
            {
                return 0;
            }
        }

        /// <summary>
        /// Read Data Value From the Ini File
        /// </summary>
        /// <param name="Section"></param>
        /// <param name="Key"></param>
        /// <param name="Path"></param>
        /// <returns></returns>
        public static string ReadINI(string Section, string Key, string thePath)
        {
            StringBuilder temp = new StringBuilder(255);
            int i = GetPrivateProfileString(Section, Key, "", temp, 255, thePath);
            return temp.ToString();
        }

        public static int ReadIntFromINI(string Section, string Key, string thePath, int set_default = -1)
        {
            // this will read an integer from an INI file. This function allows the option to 
            // set a default value to insert in case the value is currently blank in the INI.
            StringBuilder temp = new StringBuilder(255);
            int i = GetPrivateProfileString(Section, Key, "", temp, 255, thePath);
            string the_text = temp.ToString();
            try
            {
                return Convert.ToInt32(the_text);
            }
            catch
            {
                if (set_default != -1)
                    WriteINI(Section, Key, set_default.ToString(), thePath);
                else
                    set_default = 0;

                return set_default;
            }
            
        }

        public static int Keywords_Save_To_File()
        {
            // first let's update our count.
            Globals.keyword_count = Globals.Keyword_List.Count;
            // now let's update our lowercase keywords.
            keyword_name_lower.Clear();
            foreach (Keyword this_keyword in Keyword_List)
            {
                keyword_name_lower.Add(this_keyword.name.ToLower());
            }
            // this will save the keyword list to our JSON file.
            string json = JsonConvert.SerializeObject(Keyword_List);
            try
            {
                File.WriteAllText(Globals.KeywordsJSONFile, json);
                return 1;
            }
            catch (Exception e)
            {
                return 0;
            }
        }

        public static int LetterCommands_Save_To_File()
        {
            string json = JsonConvert.SerializeObject(Letter_Commands);
            try
            {
                File.WriteAllText(Globals.LetterCommandsJSONFile, json);
                return 1;
            }
            catch (Exception e)
            {
                return 0;
            }
        }

        public static int Labels_Save_To_File()
        {
            // this will save the label list to our JSON file.
            string json = JsonConvert.SerializeObject(Keyword_Folders);
            try
            {
                File.WriteAllText(Globals.FoldersJSONFile, json);
                return 1;
            }
            catch (Exception e)
            {
                return 0;
            }
        }

        /*
         * commenting this out 2/28/2016.
        public static int Labels_Update_Keyword(string keyword, string path_string)
        {
            // this will update the labels for the specified keyword.
            try
            {
                string keyword_lower = keyword.ToLower();
                for (int i = 0; i < Globals.keyword_count; i++)
                {
                    if (Keyword_List[i].name.ToLower() == keyword_lower)
                    {
                        // we found our match.
                        Keyword_List[i].path = path_string;
                        Keywords_Save_To_File();
                        return 1;
                    }
                }
            }
            catch { }
            return 0;
        }
*/

        /*
        public static string Label_Get_Label_String_For_Keyword(string keyword)
        {
            string keyword_lower = keyword.ToLower();
            foreach (Keyword this_keyword in Globals.Keyword_List)
            {
                if (this_keyword.name.ToLower() == keyword_lower)
                {
                    string label_string = "";
                    int count = 0;
                    foreach (string this_label in this_keyword.label)
                    {
                        if (count > 0)
                        {
                            label_string += ", " + this_label;
                        }
                        else
                        {
                            label_string += this_label;
                        }
                        count++;
                    }
                    return label_string;
                }
            }
            return "";
        }
        */

        
        public static string Folder_Get_Last_Folder(string folder_string)
        {
            try
            {
                // takes a parameter like Work/HR/Documents/Document1 and returns Document1.
                int find_last_slash = folder_string.LastIndexOf("/");
                if (find_last_slash > -1)
                {
                    return folder_string.Substring(find_last_slash + 1);
                }
            }
            catch { }
            // if we are still here, we didn't find a slash. Just return the string.
            return folder_string;
        }
        

        public static void CreateKeywordsList()
        {
            if (File.Exists(KeywordsJSONFile) == false && File.Exists(Globals.WorkPath + "\\keywords.ini") == false)
            {
                // neither the legacy keywords.ini nor the keywords.json exist. Create a new keywords.json file.
                MessageBox.Show("The file that contains keywords for Keybreeze Desktop is missing: keywords.json. The legacy file, keywords.ini, is missing as well. Please reinstall the program to fix this problem.", "Keybreeze Desktop");
                Application.Exit();
            }
            else if (File.Exists(KeywordsJSONFile) == false && File.Exists(Globals.WorkPath + "\\keywords.ini") == true)
            {
                // the old keywords.ini file exists. Let's convert it to JSON.
                // read keywords from the keywords.ini file and make a list.
                string keyword_name = "";
                string keyword_target;
                string keyword_lower;
                string tempstring;
                string tempstring2;
                int tempint;
                int FindAEquals;
                int FindBEquals;
                bool NextLineIsTarget = false;
                bool KeywordWasBlank = false;
                StreamReader streamReader;
                
                List<string> keyword_names_lowercase = new List<string>();
                List<string> keyword_names = new List<string>();
                List<string> keyword_targets = new List<string>();

                string KeywordsINI = Globals.WorkPath + "\\keywords.ini";

                if (logging == 1)
                    DebugWrite("CreateKeywordsList -- starting function");
                try
                {
                    streamReader = new StreamReader(KeywordsINI, true);
                }
                catch
                {
                    // the file is missing. create it.
                    string temp_text = "[keywords]" + Environment.NewLine;
                    try
                    {
                        System.IO.File.WriteAllText(KeywordsINI, temp_text, Encoding.Unicode);
                        streamReader = new StreamReader(KeywordsINI, true);
                    }
                    catch
                    {
                        MessageBox.Show("Keybreeze Desktop cannot write to the " + Globals.WorkPath + " directory. Please check your Windows permissions. The program will now close.", "Keybreeze Desktop");
                        Application.Exit();
                        return;
                    }
                }
                string INIText = streamReader.ReadToEnd();
                streamReader.Close();
                if (INIText.Length == 0)
                {
                    string temp_text = "[keywords]" + Environment.NewLine;
                    INIText = temp_text;
                    try
                    {
                        System.IO.File.WriteAllText(KeywordsINI, temp_text, Encoding.Unicode);
                        streamReader = new StreamReader(KeywordsINI, true);
                    }
                    catch
                    {
                        MessageBox.Show("Keybreeze Desktop cannot write to the " + Globals.WorkPath + " directory. Please check your Windows permissions. The program will now close.", "Keybreeze Desktop");
                        Application.Exit();
                        return;
                    }
                }
                using (StreamReader reader = new StreamReader(KeywordsINI, true))
                {
                    string line = string.Empty;
                    do
                    {
                        line = reader.ReadLine();
                        if (line != null)
                        {
                            FindAEquals = line.IndexOf("A=");
                            if (FindAEquals > -1 && NextLineIsTarget == false)
                            {
                                keyword_name = line.Substring(FindAEquals + 2);
                                if (keyword_name.Length > 0)
                                {
                                    keyword_lower = keyword_name.ToLower();
                                    keyword_names_lowercase.Add(keyword_lower);
                                    keyword_names.Add(keyword_name);
                                    NextLineIsTarget = true;
                                    KeywordWasBlank = false;
                                }
                                else
                                    KeywordWasBlank = true;
                            }
                            else if (NextLineIsTarget == true)
                            {
                                FindBEquals = line.IndexOf("B=");
                                keyword_target = line.Substring(FindBEquals + 2);
                                if (keyword_target.Length > 0 && KeywordWasBlank == false)
                                {
                                    // before we add the target, we need to remove any quotation marks if it's a path to a file/folder.
                                    tempstring = keyword_target.Substring(0, 1);
                                    if (tempstring == "\"")
                                    {
                                        // first character is a quotation mark. see if it's a path to a file/folder.
                                        if (keyword_target.Length > 3)
                                        {
                                            tempstring = keyword_target.Substring(2, 2);
                                            if (tempstring == @":\" || tempstring == @":/")
                                            {
                                                // it's a file or folder. remove the first quote.
                                                keyword_target = keyword_target.Substring(1);
                                                // we need to remove the matching quote if it's there.
                                                tempint = keyword_target.IndexOf("\"");
                                                if (tempint > -1)
                                                {
                                                    tempstring = keyword_target.Substring(0, tempint);
                                                    tempstring2 = keyword_target.Substring(tempint + 1);
                                                    keyword_target = tempstring + tempstring2;
                                                }
                                            }
                                        }
                                    }
                                    keyword_targets.Add(keyword_target);
                                    keyword_count += 1;
                                    NextLineIsTarget = false;
                                    Keyword this_keyword = new Keyword();
                                    this_keyword.name = keyword_name;
                                    keyword_name_lower.Add(keyword_name.ToLower());
                                    this_keyword.target = keyword_target;
                                    Keyword_List.Add(this_keyword);
                                }
                            }
                        }
                    } while (line != null);
                }

                // now let's try putting everything into JSON and saving the file.
                int save_response = Keywords_Save_To_File();
                if (save_response == 0)
                {
                    MessageBox.Show("Keybreeze Desktop cannot write to the " + Globals.WorkPath + " directory. The program will now close.", "Keybreeze Desktop");
                    Application.Exit();
                }
                try
                {
                    // move our keywords.ini file to the backup folder.
                    if (Directory.Exists(Globals.WorkPath + "\\backup") == false)
                    {
                        Directory.CreateDirectory(Globals.WorkPath + "\\backup");
                    }
                    Random rnd = new Random();
                    int rand_num = rnd.Next(1, 1000);
                    File.Move(KeywordsINI, Globals.WorkPath + "\\backup\\keywords_before_json" + rand_num.ToString() + ".ini");
                }
                catch (Exception e)
                {
                    MessageBox.Show("Keybreeze Desktop cannot write to the " + Globals.WorkPath + " directory. The program will now close. " + e.Message, "Keybreeze Desktop");
                    Application.Exit();
                }
            }
            else
            {
                // read from our JSON file.
                try
                {
                    string json_from_file = File.ReadAllText(KeywordsJSONFile);
                    //Keyword_List = JsonConvert.DeserializeObject(json_from_file);
                    //Keyword_List = JsonConvert.DeserializeObject(List<Keyword>(json_from_file));
                    //Keyword_List = JsonConvert.DeserializeObject(Of List<Keyword>)(json_from_file);
                    //JsonSerializer serializer = new JsonSerializer();
                    //Keyword this_keyword = (Keyword)serializer.Deserialize(json_from_file, typeof(Keyword));
                    Keyword_List = JsonConvert.DeserializeObject<List<Keyword>>(json_from_file);
                    // add lowercase keywords to our lowercase variable.
                    keyword_name_lower.Clear();
                    foreach (Keyword this_keyword in Keyword_List)
                    {
                        keyword_name_lower.Add(this_keyword.name.ToLower());
                    }
                    Globals.keyword_count = Keyword_List.Count;
                }
                catch (Exception e)
                {
                    MessageBox.Show("An error occurred while reading the " + KeywordsJSONFile + " file. It may have gotten corrupted. Try posting it at keybreeze.com/forum to correct the problem. " + e.Message, "Keybreeze Desktop");
                    Application.Exit();
                }
                try
                {
                    // read our labels for our keywords.
                    // if folders.json does not exist but labels.json does exist, rename labels.json to folders.json.
                    if (File.Exists(FoldersJSONFile) == false && File.Exists(Globals.WorkPath + "\\labels.json") == true)
                    {
                        try
                        {
                            File.Move(Globals.WorkPath + "\\labels.json", FoldersJSONFile);
                        }
                        catch (Exception e3)
                        {
                            MessageBox.Show("An error occurred trying to rename " + Globals.WorkPath + "\\labels.json to " + FoldersJSONFile + ". " + e3.Message, "Keybreeze Desktop");
                            Application.Exit();
                        }
                    }
                    if (File.Exists(FoldersJSONFile))
                    {
                        string json_from_label_file = File.ReadAllText(FoldersJSONFile);
                        Keyword_Folders = JsonConvert.DeserializeObject<List<Keyword_Folder>>(json_from_label_file);
                        if (Keyword_Folders == null)
                            Keyword_Folders = new List<Keyword_Folder>();
                    }
                }
                catch (Exception e)
                {
                    MessageBox.Show("An error occurred while reading the " + FoldersJSONFile + " file. It may have gotten corrupted. Try posting it at keybreeze.com/forum to correct the problem. " + e.Message, "Keybreeze Desktop");
                    Application.Exit();
                }
            }
        }

        public static void CreateLetterCommandsList()
        {
            // first we need to see if the old lettercommands.ini file is in use. Convert that to JSON.
            string LetterCommandsINI = Globals.WorkPath + "\\lettercommands.ini";
            if (File.Exists(LetterCommandsINI) == true)
            {
                // load into our letter commands class. Then convert to JSON and save it to lettercommands.json.
                try
                {
                    for (int i = 1; i <= 250; i++)
                    {
                        string letter_name;
                        letter_name = Globals.ReadINI("letters", i + "A", LetterCommandsINI);
                        if (letter_name.Length > 0)
                        {
                            // a letter command exists.
                            string letter_target = Globals.ReadINI("letters", letter_name + "2", LetterCommandsINI) + "%keybreeze_entry%" + Globals.ReadINI("letters", letter_name + "3", LetterCommandsINI);
                            string letter_replacements = Globals.ReadINI("letters", letter_name + "-replacements", LetterCommandsINI);
                            Letter_Command this_letter_command = new Letter_Command();
                            this_letter_command.name = letter_name;
                            this_letter_command.url = letter_target;
                            if (letter_replacements.Length > 0)
                            {
                                string[] replacements = letter_replacements.Split(',');
                                foreach (string replacement in replacements)
                                {
                                    int find_arrows = replacement.IndexOf(">>");
                                    if (find_arrows == -1)
                                        continue;
                                    string replace_this = replacement.Substring(0, find_arrows).Trim();
                                    // remove the dot since letter commands were previously separate by ,.
                                    if (replace_this.Substring(0, 1) == ".")
                                        replace_this = replace_this.Substring(1);
                                    string replace_with = replacement.Substring(find_arrows + 2).Trim();
                                    this_letter_command.replace_this.Add(replace_this);
                                    this_letter_command.replace_with.Add(replace_with);
                                }
                            }
                            Letter_Commands.Add(this_letter_command);
                        }
                        else
                        {
                            // letter command does not exist, so we must have them all. let's exit the loop.
                            break;
                        }
                    }
                    // save to JSON file.
                    int save_response = LetterCommands_Save_To_File();
                    if (save_response == 0)
                    {
                        MessageBox.Show("Keybreeze Desktop cannot write to the " + Globals.LetterCommandsJSONFile + " file. The program will now close.", "Keybreeze Desktop");
                        Application.Exit();
                    }

                    // move our INI file to backup.
                    File.Move(LetterCommandsINI, Globals.WorkPath + "\\backup\\lettercommands.ini");
                }
                catch (Exception e)
                {
                    MessageBox.Show("An error occurred while loading the letter commands from the " + LetterCommandsINI + " file. " + e.Message);
                    Application.Exit();
                }
            }
            else if (File.Exists(LetterCommandsJSONFile) == true)
            {
                // load the letter commands from our JSON file.
                // read from our JSON file.
                try
                {
                    string json_from_file = File.ReadAllText(LetterCommandsJSONFile);
                    Letter_Commands = JsonConvert.DeserializeObject<List<Letter_Command>>(json_from_file);
                }
                catch (Exception e)
                {
                    MessageBox.Show("An error occurred while reading the " + LetterCommandsJSONFile + " file. It may have gotten corrupted. Try posting it at keybreeze.com/forum to correct the problem. " + e.Message, "Keybreeze Desktop");
                    Application.Exit();
                }
            }
        }

        public static void DebugWrite(string output)
        {
            string log_folder = Globals.WorkPath + "\\log\\";
            string datestring = DateTime.Now.Year.ToString() + DateTime.Now.Month.ToString() + DateTime.Now.Day.ToString();
            string log_file = log_folder + "keybreeze_log_" + datestring + ".log";
            DateTime my_time = DateTime.Now;
            string format = "HH:mm:ss.fff";
            string timestamp = my_time.ToString(format);
            string line_to_log = timestamp + "\t\t" + output;
            if (Directory.Exists(log_folder) == false)
            {
                Directory.CreateDirectory(log_folder);
            }
            try
            {
                if (!File.Exists(log_file))
                {
                    // Create a file to write to. 
                    using (StreamWriter sw = File.CreateText(log_file))
                    {
                        sw.WriteLine(line_to_log);
                    }
                }
                else
                {
                    using (StreamWriter sw = File.AppendText(log_file))
                    {
                        sw.WriteLine(line_to_log);
                    }
                }
            }
            catch
            { }
        }

        public static string GetTargetType(string target)
        {
            // this function will examine a target and return the file type or target type if it's not a file.
            // return value examples:   exe,txt,jpg
            //                          text (for text functions)
            //                          mcr (for macros)
            //                          dir (for directories)
            //                          kb (for preset keywords)
            //                          web (for websites)
            //                          err (if an error occurs)
            //                          remind (for a speed reminder)

            string fewchars;
            int FindEXE;

            fewchars = Left(target, 7);
            if (fewchars == "preset:")
                return "kb";
            if (fewchars == "remind:")
                return "remind";

            fewchars = Left(target, 9);
            if (fewchars == "function:")
                return "text";

            fewchars = Left(target, 2);
            if (fewchars == @"\\")
                return "unc";

            if (target == "macro")
                return "mcr";

            // Look for :\ or :/ to see if it's a file or directory, like C:\, "C:\, but not ftp:// or http://.
            int find_colon_slash = target.IndexOf(@":\");
            if (find_colon_slash == -1)
                find_colon_slash = target.IndexOf(@":/");
            if (find_colon_slash != -1 && find_colon_slash < 3)
            {
                // it's a file or directory
                // lets see if it's an exe or lnk file. If so, remove any command line switches.
                FindEXE = target.IndexOf(".exe");
                if (FindEXE == -1) FindEXE = target.LastIndexOf(".lnk");
                if (FindEXE == -1) FindEXE = target.LastIndexOf(".bat");
                if (FindEXE > -1)
                {
                    // it's an .exe or .bat or .lnk file. let's check for command line parameters.
                    if (FindEXE < (target.Length - 4))
                    {
                        // there are characters present after .exe, most likely switches/parameters.
                        return "exe";
                    }
                }

                // get the file attributes for file or directory
                // if it's a network directory that no longer exists, this can take 30 seconds or longer before it times out. To get around this, we'll start our own thread.
                int target_type = -1;
                Thread t = new Thread(new ThreadStart(delegate()
                {
                    if (File.Exists(target) || Directory.Exists(target))
                    {
                        FileAttributes attr = File.GetAttributes(target);
                        //detect whether its a directory or file
                        if ((attr & FileAttributes.Directory) == FileAttributes.Directory)
                        {
                            // it's a folder/directory
                            target_type = 5;
                        }
                        else
                        {
                            // it's a file
                            target_type = 4;
                        }
                    }
                    else
                    {
                        // file or directory does not exist.
                        target_type = -1;
                    }
                }));
                t.Start();
                bool completed = t.Join(3000);
                if (!completed)
                {
                    t.Abort();
                    // the thread timed out at a second. Let's assume it's a network path that is not accessible or doesn't exist.
                    return "err";
                }
                else
                {
                    if (target_type == 5)
                        return "dir";
                    else if (target_type == 4)
                    {
                        string file_extention = Path.GetExtension(target);
                        // remove the period.
                        if (file_extention[0] == '.')
                            file_extention = file_extention.Substring(1);
                        return file_extention;
                    }
                    else if (target_type == -1)
                    {
                        return "err";
                    }
                }
            }

            // if we are still here, it is probably a web address:
            return "web";
        }

        public static string ReturnShortenedPath(string PathToShorten, string folder_path)
        {
            // this will take a long path and shorten it so it fits in the autocomplete window.
            // it will also replace a text function name with a preview of the text.
            // it will also place the folder path before the description.
            string original_path_to_shorten = PathToShorten;
            string original_folder_path = folder_path;
            if (folder_path != null && folder_path.Length > 0)
                folder_path = folder_path + " - ";
            PathToShorten = folder_path + PathToShorten; 
            if (PathToShorten.Length < 9)
                return PathToShorten;
            int is_text_function = original_path_to_shorten.IndexOf("function:");
            if (is_text_function == 0)
            {
                // it's a text function. let's get some of the text.
                string text_function_file = original_path_to_shorten.Substring(9);
                text_function_file = Globals.replace_invalid_filename_characters(text_function_file, original_folder_path);
                string text_file = Globals.WorkPath + "\\text functions\\" + text_function_file + ".txt";
                //string the_text = "";
                string the_text = folder_path;
                try
                {
                    the_text += "text:" + File.ReadAllText(text_file);
                }
                catch
                {
                    the_text += "text:" + text_function_file;
                }
                int the_length = the_text.Length;
                if (the_length > description_character_limit)
                {
                    the_text = the_text.Substring(0, description_character_limit) + "...";
                }
                return the_text;
            }
            int path_length = PathToShorten.Length;
            if (path_length > description_character_limit)
            {
                PathToShorten = "..." + PathToShorten.Substring(path_length - description_character_limit + 3);
            }
            return PathToShorten;
        }

        public static string GetTargetType_For_AutoComplete(string target)
        {
            // This function is similar to GetTargetType, but it won't start a new thread to check the existence of a file or folder, since that slows everything down drastically.

            // this function will examine a target and return the file type or target type if it's not a file.
            // return value examples:   exe,txt,jpg
            //                          text (for text functions)
            //                          mcr (for macros)
            //                          dir (for directories)
            //                          kb (for preset keywords)
            //                          web (for websites)
            //                          err (if an error occurs)

            int FindEXE;
            target = target.ToLower();
            string fewchars;

            if (target == "keybreeze folder")
                return "dir";

            fewchars = Left(target, 7);
            if (fewchars == "preset:")
                return "kb";
            if (fewchars == "remind:")
                return "kb";

            fewchars = Left(target, 9);
            if (fewchars == "function:")
                return "text";

            fewchars = Left(target, 2);
            if (fewchars == @"\\")
            {
                int last_dot = target.LastIndexOf(".");
                int last_slash = target.LastIndexOf("\\");
                if (last_slash > last_dot)
                    return "unc_folder";
                else
                    return "unc_file";
            }

            if (target == "macro")
                return "mcr";

            // Look for :\ or :/ to see if it's a file or directory, like C:\, "C:\, but not ftp:// or http://.
            int find_colon_slash = target.IndexOf(@":\");
            if (find_colon_slash == -1)
                find_colon_slash = target.IndexOf(@":/");
            if (find_colon_slash != -1 && find_colon_slash < 3)
            {
                // it's a file or directory
                // lets see if it's an exe or lnk file. If so, remove any command line switches.
                FindEXE = target.IndexOf(".exe");
                if (FindEXE == -1) FindEXE = target.LastIndexOf(".lnk");
                if (FindEXE == -1) FindEXE = target.LastIndexOf(".bat");
                if (FindEXE > -1)
                {
                    return "exe";
                }
                else
                {
                    int last_dot = target.LastIndexOf(".");
                    int last_slash = target.LastIndexOf("\\");
                    if (last_slash > last_dot)
                    {
                        return "dir";
                    }
                    else
                    {
                        bool is_drive_local = false;
                        string drive_letter = target.Substring(0, 1);
                        foreach (string local_drive in local_drives)
                        {
                            if (local_drive == drive_letter)
                            {
                                is_drive_local = true;
                            }
                        }
                        if (is_drive_local == false)
                        {
                            // the file is not on a local drive. We don't want to check its existence or get its icon because it might take a long time.
                            return "err";
                        }
                    }
                    if (File.Exists(target) == true)
                    {
                        string file_extension = Path.GetExtension(target);
                        // remove the period.
                        if (file_extension[0] == '.')
                            file_extension = file_extension.Substring(1);
                        return file_extension;
                    }
                }
                // if we are here, it's probably a directory.
                return "dir";
            }

            // if we are still here, it is probably a web address:
            return "web";
        }

        public static string Left(string text, int length)
        {
            if (length < 0)
                return "";
            else if (length == 0 || text.Length == 0)
                return "";
            else if (text.Length <= length)
                return text;
            else
                return text.Substring(0, length);
        }

        public static int Update_AutoComplete_List(string new_entry = "")
        {
            string full_text;
            string AutoCompleteFile = Globals.WorkPath + "\\autocomplete.txt";

            try
            {
                full_text = File.ReadAllText(AutoCompleteFile);
            }
            catch
            {
                // could not open autocomplete.txt file.
                return -1;
            }
            if (new_entry.Length > 0)
            {
                // the user is adding a new entry. Let's see if it already exists in the list.
                new_entry = new_entry.ToLower();
                if (full_text.IndexOf("|" + new_entry + "|") != -1)
                {
                    // the entry is already in the autocomplete list, just exit.
                    return 1;
                }
                else
                {
                    // the entry is NOT in the autocomplete list. Let's add it.
                    // we need to make sure the list does not grow bigger than the threshold
                    string autocomplete_char_limit = ReadINI("options", "autocomplete_char_limit", OptionsINI);
                    int number_limit;
                    try
                    {
                        number_limit = Convert.ToInt32(autocomplete_char_limit);
                        if (number_limit < 100) number_limit = 1000;
                    }
                    catch
                    {
                        number_limit = 10000;
                    }
                    if (full_text.Length > number_limit)
                    {
                        // we are at the limit. let's remove something before adding something new.
                        string entry_to_remove = autocomplete_list[0] + "|";
                        autocomplete_list.RemoveAt(0);
                        full_text = full_text.Replace(entry_to_remove, "");
                        full_text = full_text.Replace("|" + entry_to_remove, "");
                    }
                    try
                    {
                        // now let's add something new.
                        full_text = full_text + new_entry + "|";
                        System.IO.File.WriteAllText(AutoCompleteFile, full_text, Encoding.Unicode);
                        autocomplete_list.Add(new_entry);
                        return 1;
                    }
                    catch
                    {
                        // we could not save anything to the autocomplete file.
                        return -2;
                    }
                    
                }
            }

            // we are not adding a new entry. We are just loading the autocomplete list from the text file.
            string[] entry = full_text.Split(new string[] { "|" }, StringSplitOptions.None);
            autocomplete_list.Clear();
            bool already_here = false;
            foreach (string e in entry)
            {
                if (e.Length > 0)
                    {
                    // make sure we are not adding a duplicate
                    already_here = false;
                    foreach (string current_autocomplete_value in autocomplete_list)
                    {
                        if (current_autocomplete_value == e)
                        {
                            already_here = true;
                        }
                    }
                    if (already_here == false) autocomplete_list.Add(e);
                }
            }
            return 1;
        }

        public static void Count_History_Items()
        {
            // this counts the number of items in history so we can keep track of where we are at and not overwrite existing history items.
            // this function should only run when the application loads.

            string HistoryJSONFile = Globals.WorkPath + "\\history.json";
            if (File.Exists(HistoryJSONFile) == false)
                return;

            string json_from_file = File.ReadAllText(HistoryJSONFile);
            History_List = JsonConvert.DeserializeObject<List<History>>(json_from_file);
            if (History_List == null)
            {
                History_List = new List<History>();
            }

            /*
            string HistoryINI = Globals.WorkPath + "\\history.ini";
            int find_entry = 0;
            int total_count = 0;

            StreamReader streamReader = new StreamReader(HistoryINI, true);
            string INIText = streamReader.ReadToEnd();
            streamReader.Close();
            using (StringReader reader = new StringReader(INIText))
            {
                string line = string.Empty;
                do
                {
                    line = reader.ReadLine();
                    if (line != null)
                    {
                        find_entry = line.IndexOf("e=");
                        if (find_entry > -1)
                        {
                            // we have a history record
                            total_count += 1;
                        }
                    }
                } while (line != null);
            }
            history_count = total_count;
            history_position = history_count;
            */
        }

        public static void History_Save_To_File()
        {
            // this will save the history list to our JSON file.
            string json = JsonConvert.SerializeObject(History_List);
            try
            {
                File.WriteAllText(Globals.WorkPath + "\\history.json", json);
                return;
            }
            catch (Exception e)
            {
                return;
            }
        }

        public static int autoindex_update()
        {
            // the automatic index will build a list of files in commonly used directories,
            // e.g. Desktop, My Documents, Start Menu, as well as user-defined directories.
            // if a user starts typing an unrecognized keyword, Keybreeze Desktop will search through
            // these files and folders and display any that match.

            string autoindex_settings = Globals.WorkPath + "\\autoindex\\indexoptions.ini";
            string current_folder = "";
            string folder_is_checked = "";
            string search_subfolders = "";
            string file_type_list = "";
            string file_types = "";
            string add_top_level_folder_names = "";
            bool legit_file = false;
            string line;
            string blacklist_file = Globals.WorkPath + "\\autoindex\\blacklist.txt";
            string filetype_mod;

            autoindex_filepaths.Clear();
            autoindex_files.Clear();
            autoindex_folders_on_demand.Clear();

            // load the blacklist into an array so we can cross-check them with the entries we are adding.
            autoindex_blacklist.Clear();
            try
            {
                System.IO.StreamReader file1 = new System.IO.StreamReader(blacklist_file, true);
                while ((line = file1.ReadLine()) != null)
                {
                    autoindex_blacklist.Add(line);
                }
                file1.Close();
            }
            catch
            {
                System.IO.File.WriteAllText(blacklist_file, "", Encoding.Unicode);
            }

            int autoindex_timeout = Globals.ReadIntFromINI("options", "autoindex_timeout", Globals.OptionsINI, 5000);
            if (autoindex_timeout < 100 || autoindex_timeout > 60000)
                autoindex_timeout = 5000;

            // loop through each folder and get a list of files.
            for (int i = 1; i <= 300; i++)
            {
                current_folder = ReadINI("Locations", "ITEM" + i + "-PATH", autoindex_settings);
                if (current_folder.Length > 0)
                {
                    // are we supposed to index this folder on demand?
                    int indexondemand = ReadIntFromINI("Locations", "ITEM" + i + "-indexondemand", autoindex_settings, 0);
                    if (indexondemand == 1)
                    {
                        autoindex_folders_on_demand.Add(current_folder);
                    }
                    // replace special names like "Desktop", "My Documents" with the real path.
                    current_folder = Replace_Special_Folder_Name(current_folder);
                    folder_is_checked = ReadINI("Locations", "ITEM" + i + "-CHECKED", autoindex_settings);
                    if (folder_is_checked == "1" && Directory.Exists(current_folder))
                    {
                        // we need to scan files in this folder.
                        search_subfolders = ReadINI("Locations", "ITEM" + i + "-SUBFOLDERS", autoindex_settings);
                        SearchOption MySearchOption = new SearchOption();
                        if (search_subfolders == "1")
                        {
                            MySearchOption = SearchOption.AllDirectories;
                        }
                        else
                        {
                            MySearchOption = SearchOption.TopDirectoryOnly;
                        }
                        file_type_list = ReadINI("Locations", "ITEM" + i + "-filetypes", autoindex_settings);
                        file_types = ReadINI("Locations", "ITEM" + i + "-FILETYPES", autoindex_settings);
                        if (file_type_list == "")
                        {
                            // search all file types
                            // start a new thread so we can kill it if it goes past the timeout limit.
                            Thread t = new Thread(new ThreadStart(delegate()
                            {
                                try
                                {
                                    foreach (string filePath in Directory.EnumerateFiles(current_folder, "*.*", MySearchOption))
                                    {
                                        legit_file = autoindex_is_file_legit(filePath);
                                        if (legit_file == true)
                                        {
                                            autoindex_filepaths.Add(filePath);
                                            autoindex_files.Add(Path.GetFileNameWithoutExtension(filePath));
                                        }
                                    }
                                }
                                catch
                                {

                                }
                            }));
                            try
                            {
                                t.Start();
                            }
                            catch { }
                            bool completed = t.Join(autoindex_timeout);
                            if (!completed)
                            {
                                t.Abort();
                                // commenting this line out as the thread seems to time out after resuming from hibernation.
                                //MessageBox.Show("Keybreeze Desktop was unable to index everything in the " + current_folder + " directory. The thread timed out. There may be too many files in this directory, or non-existent network paths may exist. You can exclude this directory from the autoindex by pressing the hotkey and typing 'options' as a keyword.", "Keybreeze Desktop");
                            }
                        }
                        else if (file_type_list != "")
                        {
                            // search for only certain file types.
                            // we have to search one file type at a time
                            string[] filetypes = file_type_list.Split(',');
                            Thread t = new Thread(new ThreadStart(delegate()
                            {
                                try
                                {
                                    foreach (string filetype in filetypes)
                                    {
                                        filetype_mod = filetype;
                                        if (filetype_mod.IndexOf(".") > -1)
                                            filetype_mod = filetype.Replace(".", "");
                                        foreach (string filePath in Directory.EnumerateFiles(current_folder, "*." + filetype_mod, MySearchOption))
                                        {
                                            legit_file = autoindex_is_file_legit(filePath);
                                            if (legit_file == true)
                                            {
                                                autoindex_filepaths.Add(filePath);
                                                autoindex_files.Add(Path.GetFileNameWithoutExtension(filePath));
                                            }
                                        }
                                    }
                                }
                                catch
                                {

                                }
                            }));
                            try
                            {
                                t.Start();
                            }
                            catch { }
                            bool completed = t.Join(autoindex_timeout);
                            if (!completed)
                            {
                                t.Abort();
                                // commenting this line out as the thread seems to time out after resuming from hibernation.
                                //MessageBox.Show("Keybreeze Desktop was unable to index everything in the " + current_folder + " directory. The thread timed out. There may be too many files in this directory, or non-existent network paths may exist. You can exclude this directory from the autoindex by pressing the hotkey and typing 'options' as a keyword.", "Keybreeze Desktop");
                            }
                        }

                        // are we supposed to add entries for subfolders in the top-level directory?
                        add_top_level_folder_names = ReadINI("Locations", "ITEM" + i + "-TOPLEVELFOLDERS", autoindex_settings);
                        if (add_top_level_folder_names == "1")
                        {
                            foreach (string folderPath in Directory.EnumerateDirectories(current_folder))
                            {
                                autoindex_filepaths.Add(folderPath);
                                //autoindex_files.Add(Path.GetDirectoryName(folderPath));
                                string last_folder = new DirectoryInfo(folderPath).Name;
                                autoindex_files.Add(last_folder);
                            }
                        }
                    }
                }
                else
                {
                    // the INI setting is blank, let's exit.
                    break;
                }
            }
            return 1;
        }

        public static List<string> autoindex_on_demand(string search_text, List<string> matches_already_found)
        {
            // this function will search for matching files as the user is typing a keyword (after the autoindex has finished running).
            // It is useful when looking for items that have just been downloaded or just been added to the Desktop recently.

            string autoindex_settings = Globals.WorkPath + "\\autoindex\\indexoptions.ini";
            string current_folder = "";
            string folder_is_checked = "";
            string search_subfolders = "";
            string file_type_list = "";
            string file_types = "";
            string add_top_level_folder_names = "";
            bool legit_file = false;
            string blacklist_file = Globals.WorkPath + "\\autoindex\\blacklist.txt";
            string filetype_mod;

            int search_text_length = search_text.Length;
            List<string> new_matches = new List<string>();

            int autoindex_timeout = Globals.ReadIntFromINI("options", "autoindex_timeout", Globals.OptionsINI, 5000);
            if (autoindex_timeout < 100 || autoindex_timeout > 60000)
                autoindex_timeout = 5000;

            // loop through each folder and get a list of files.
            for (int i = 1; i <= 300; i++)
            {
                current_folder = ReadINI("Locations", "ITEM" + i + "-PATH", autoindex_settings);
                if (current_folder.Length > 0)
                {
                    // are we supposed to index this folder on demand?
                    int indexondemand = ReadIntFromINI("Locations", "ITEM" + i + "-indexondemand", autoindex_settings, 0);
                    // replace special names like "Desktop", "My Documents" with the real path.
                    current_folder = Replace_Special_Folder_Name(current_folder);
                    folder_is_checked = ReadINI("Locations", "ITEM" + i + "-CHECKED", autoindex_settings);
                    if (folder_is_checked == "1" && Directory.Exists(current_folder) && indexondemand == 1)
                    {
                        // we need to scan files in this folder.
                        search_subfolders = ReadINI("Locations", "ITEM" + i + "-SUBFOLDERS", autoindex_settings);
                        SearchOption MySearchOption = new SearchOption();
                        if (search_subfolders == "1")
                        {
                            MySearchOption = SearchOption.AllDirectories;
                        }
                        else
                        {
                            MySearchOption = SearchOption.TopDirectoryOnly;
                        }
                        file_type_list = ReadINI("Locations", "ITEM" + i + "-filetypes", autoindex_settings);
                        file_types = ReadINI("Locations", "ITEM" + i + "-FILETYPES", autoindex_settings);
                        if (file_type_list == "")
                        {
                            // search all file types
                            // start a new thread so we can kill it if it goes past the timeout limit.
                            Thread t = new Thread(new ThreadStart(delegate()
                            {
                                try
                                {
                                    foreach (string filePath in Directory.EnumerateFiles(current_folder, "*.*", MySearchOption))
                                    {
                                        string file_name_only = Path.GetFileNameWithoutExtension(filePath);
                                        string file_name_lower = file_name_only.ToLower();
                                        // make sure the filename is longer than the search string.
                                        if (search_text_length <= file_name_only.Length)
                                        {
                                            int FindIt = file_name_lower.IndexOf(search_text);
                                            if (FindIt == 0)
                                            {
                                                // we found a starts with match.
                                                legit_file = autoindex_is_file_legit(filePath);
                                                if (legit_file == true)
                                                {
                                                    // was this file already found?
                                                    bool already_found = false;
                                                    foreach (string match_already_found in matches_already_found)
                                                    {
                                                        if (match_already_found == filePath)
                                                        {
                                                            already_found = true;
                                                            break;
                                                        }
                                                    }
                                                    if (already_found == false)
                                                        new_matches.Add(filePath);
                                                }
                                            }
                                            else if (FindIt > 0)
                                            {
                                                // we found a partial match.
                                                // only add a partial match to the list if it's 3 characters or greater.
                                                if (search_text_length > 2)
                                                {
                                                    legit_file = autoindex_is_file_legit(filePath);
                                                    if (legit_file == true)
                                                    {
                                                        // was this file already found?
                                                        bool already_found = false;
                                                        foreach (string match_already_found in matches_already_found)
                                                        {
                                                            if (match_already_found == filePath)
                                                            {
                                                                already_found = true;
                                                                break;
                                                            }
                                                        }
                                                        if (already_found == false)
                                                            new_matches.Add(filePath);
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                                catch
                                {

                                }
                            }));
                            try
                            {
                                t.Start();
                            }
                            catch { }
                            bool completed = t.Join(autoindex_timeout);
                            if (!completed)
                            {
                                t.Abort();
                                // commenting this line out as the thread seems to time out after resuming from hibernation.
                                //MessageBox.Show("Keybreeze Desktop was unable to index everything in the " + current_folder + " directory. The thread timed out. There may be too many files in this directory, or non-existent network paths may exist. You can exclude this directory from the autoindex by pressing the hotkey and typing 'options' as a keyword.", "Keybreeze Desktop");
                            }
                        }
                        else
                        {
                            // search for only certain file types.
                            // we have to search one file type at a time
                            string[] filetypes = file_type_list.Split(',');
                            Thread t = new Thread(new ThreadStart(delegate()
                            {
                                try
                                {
                                    foreach (string filetype in filetypes)
                                    {
                                        filetype_mod = filetype;
                                        if (filetype_mod.IndexOf(".") > -1)
                                            filetype_mod = filetype.Replace(".", "");
                                        foreach (string filePath in Directory.EnumerateFiles(current_folder, "*." + filetype_mod, MySearchOption))
                                        {
                                            string file_name_only = Path.GetFileNameWithoutExtension(filePath);
                                            string file_name_lower = file_name_only.ToLower();
                                            if (search_text_length <= file_name_only.Length && file_name_lower.IndexOf(search_text) == 0)
                                            {
                                                // we found a starts with match.
                                                legit_file = autoindex_is_file_legit(filePath);
                                                if (legit_file == true)
                                                {
                                                    // was this file already found?
                                                    bool already_found = false;
                                                    foreach (string match_already_found in matches_already_found)
                                                    {
                                                        if (match_already_found == filePath)
                                                        {
                                                            already_found = true;
                                                            break;
                                                        }
                                                    }
                                                    if (already_found == false)
                                                        new_matches.Add(filePath);
                                                }
                                            }
                                        }
                                    }
                                }
                                catch
                                {

                                }
                            }));
                            try
                            {
                                t.Start();
                            }
                            catch { }
                            bool completed = t.Join(autoindex_timeout);
                            if (!completed)
                            {
                                t.Abort();
                                // commenting this line out as the thread seems to time out after resuming from hibernation.
                                //MessageBox.Show("Keybreeze Desktop was unable to index everything in the " + current_folder + " directory. The thread timed out. There may be too many files in this directory, or non-existent network paths may exist. You can exclude this directory from the autoindex by pressing the hotkey and typing 'options' as a keyword.", "Keybreeze Desktop");
                            }
                        }

                        // are we supposed to add entries for subfolders in the top-level directory?
                        add_top_level_folder_names = ReadINI("Locations", "ITEM" + i + "-TOPLEVELFOLDERS", autoindex_settings);
                        if (add_top_level_folder_names == "1")
                        {
                            foreach (string folderPath in Directory.EnumerateDirectories(current_folder))
                            {
                                //string folder_name_only = Path.GetDirectoryName(folderPath);
                                string folder_name_only = new DirectoryInfo(folderPath).Name;
                                string folder_name_lower = folder_name_only.ToLower();
                                if (search_text_length <= folder_name_only.Length && folder_name_lower.IndexOf(search_text) == 0)
                                {
                                    // we found a starts with match.
                                    legit_file = autoindex_is_file_legit(folderPath);
                                    if (legit_file == true)
                                    {
                                        // was this folder already found?
                                        bool already_found = false;
                                        foreach (string match_already_found in matches_already_found)
                                        {
                                            if (match_already_found == folderPath)
                                            {
                                                already_found = true;
                                                break;
                                            }
                                        }
                                        if (already_found == false)
                                            new_matches.Add(folderPath);
                                    }
                                }
                            }
                        }
                    }
                }
                else
                {
                    // the INI setting is blank, let's exit.
                    break;
                }
            }
            return new_matches;
        }

        private static bool autoindex_is_file_legit(string filePath)
        {
            // this function will check if a file is "legit" before adding it to the automatic index list.
            // i.e. we don't want to add protected Windows files, dll files, etc. to the list
            // we also don't want to add anything that's in the blacklist.

            try
            {
                string filename_only = Path.GetFileName(filePath).ToLower();
                if (filename_only.Substring(0, 2) == "~$")
                    return false;
                else if (filename_only == "desktop.ini")
                    return false;
                else if (filename_only.IndexOf(".dll") > -1)
                    return false;
                else
                {
                    // check the blacklist
                    foreach (string this_target in autoindex_blacklist)
                    {
                        if (this_target == filePath)
                        {
                            return false;
                        }
                    }
                }
            }
            catch
            {

            }
            return true;
        }

        public static string Replace_Special_Folder_Name(string current_folder)
        {
            /*
            
             * these are deprecated constants but they may come in handy later.

            const int CSIDL_DESKTOPDIRECTORY = 0x0010;                     // <user name>\Desktop
            const int CSIDL_COMMON_DESKTOPDIRECTORY = 0x0019;              // All Users\Desktop
            //const int CSIDL_STARTMENU = 0x000b;                          // <user name>\Start Menu
            const int CSIDL_PROGRAMS = 0x0002;                             // Start Menu\Programs
            const int CSIDL_COMMON_PROGRAMS = 0x0017;                      // All Users\Start Menu\Programs
            const int CSIDL_FAVORITES = 0x0006;                            // <user name>\Favorites
            const int CSIDL_PERSONAL = 0x0005;                             // My Documents
            const int CSIDL_MYPICTURES = 0x0027;                           // C:\Program Files\My Pictures
            const int CSIDL_MYMUSIC = 0x000d;                              // "My Music" folder
            const int CSIDL_MYVIDEO = 0x000e;                              // "My Videos" folder
            const int CSIDL_CONTROLS = 0x0003;                             // My Computer\Control Panel
            const int CSIDL_PRINTERS = 0x0004;                             // My Computer\Printers
            const int CSIDL_DRIVES = 0x0011;                               // My Computer
            const int CSIDL_NETWORK = 0x0012;                              // Network Neighborhood (My Network Places)
            const int CSIDL_BITBUCKET = 0x000a;                            // <desktop>\Recycle Bin

            */

            string real_path = "";
            // did the user include % in the path? If so, it's probably an environment variable.
            if (current_folder.IndexOf("%") != -1)
            {
                current_folder = Environment.ExpandEnvironmentVariables(current_folder);
            }
            switch (current_folder.ToLower())
            {
                case "desktop":
                    real_path = Environment.GetFolderPath(Environment.SpecialFolder.Desktop);
                    break;
                case "start menu":
                    real_path = Environment.GetFolderPath(Environment.SpecialFolder.Programs);
                    break;
                case "start menu (all users)":
                    if (Directory.Exists(@"C:\ProgramData\Microsoft\Windows\Start Menu\Programs"))
                    {
                        real_path = @"C:\ProgramData\Microsoft\Windows\Start Menu\Programs";
                    }
                    else if (Directory.Exists(@"C:\Documents and Settings\All Users\Start Menu"))
                    {
                        real_path = @"C:\Documents and Settings\All Users\Start Menu";
                    }
                    break;
                case "my documents":
                    real_path = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
                    break;
                case "downloads":
                    real_path = Environment.GetFolderPath(Environment.SpecialFolder.UserProfile) + "\\Downloads";
                    break;
                default:
                    real_path = current_folder;
                    break;
            }
            // for some reason, real_path is still blank on Windows 10 despite setting it above.
            if (real_path == "")
            {
                current_folder = real_path;
                ///Left off here. This code never executes.
            }

            if (Directory.Exists(real_path))
                return real_path;
            else
                return current_folder;
        }

        public static bool DoesFileExist(string filepath)
        {
            if (File.Exists(filepath) == true)
                return true;
            else
                return false;
        }

        public static void BackupCommandList()
        {
            // this will back up the keywords.ini file to the %appdata%\keybreeze5\backup directory.

            string backup_folder = Globals.WorkPath + "\\backup";
            string[] filePaths;
            string datestring = DateTime.Now.Year.ToString() + DateTime.Now.Month.ToString() + DateTime.Now.Day.ToString() + "_";
            DateTime dt;

            if (Directory.Exists(backup_folder) == false)
            {
                Directory.CreateDirectory(backup_folder);
            }

            // clean up old files first. delete anything older than 30 days.
            filePaths = Directory.GetFiles(backup_folder, "*.json");
            foreach (string filePath in filePaths)
            {
                if (filePath.IndexOf("keywords") != -1)
                {
                    dt = File.GetCreationTime(filePath);
                    if (DateTime.Now.Subtract(dt).TotalDays > 30)
                    {
                        // over than 30 days old. Let's delete it.
                        try
                        {
                            File.Delete(filePath);
                        }
                        catch { }
                    }
                }
            }
            
            // create a backup keywords.ini file at this location if we don't already have 50 items for this particular day.
            for (int i = 1; i <= 50; i++)
            {
                if (File.Exists(Globals.WorkPath + "\\backup\\keywords" + datestring + i + ".json") == false)
                {
                    File.Copy(Globals.WorkPath + "\\keywords.json", Globals.WorkPath + "\\backup\\keywords" + datestring + i + ".json");
                    break;
                }
            }
        }

        public static void notes_load_existing(frmMain frmMain_instance)
        {
            // this will load any existing notes and alarms.

            string note_full_text = "";
            string[] filePaths;
            string INI_path = "";
            int existing_uniqueID;
            string notes_folder = WorkPath + "\\notes";
            filePaths = Directory.GetFiles(notes_folder, "*.txt");

            // load the theme first.
            if (note_theme == "")
                note_theme = ReadINI("notes", "color_theme", NotesINI);

            // loop through all .txt files in the %appdata%\keybreeze5\notes directory. These should contain the text for each note.
            foreach (string filePath in filePaths)
            {
                // load the text file first.
                try
                {
                    note_full_text = File.ReadAllText(filePath);
                }
                catch
                {
                    MessageBox.Show("The text for one of the notes could not be read from " + filePath + ". Please make sure this file exists and that permissions are set for read and write access.", "Keybreeze Desktop");
                    continue;
                }
                // get the INI file and unique ID.
                INI_path = filePath.Replace(".txt", ".ini");
                existing_uniqueID = Convert.ToInt32(Path.GetFileNameWithoutExtension(INI_path));

                // there are two types of hidden notes -- reminders, which we never show until their alarm time.
                // and hidden notes, which the user can manually hide. We will show the hidden notes if the option is set.
                int display_at_startup = ReadIntFromINI("notes", "display_at_startup", Globals.NotesINI, 1);
                int hidden_note = ReadIntFromINI("properties", "hidden", INI_path, 0);
                string hidden_reminder = ReadINI("properties", "hiddenreminder", INI_path);
                if (hidden_reminder != "1")
                {
                    if (hidden_note == 0 || display_at_startup == 1)
                        notes_load_existing_note(existing_uniqueID, frmMain_instance);
                }
                else
                {
                    // it's a hidden reminder, so we probably don't want to show the note.
                    // However, if the reminder no longer exists, we need to show it.
                    string reminders_file = Globals.WorkPath + "\\notes\\settings\\reminders.ini";
                    string reminders_ini_text = "";
                    try
                    {
                        reminders_ini_text = File.ReadAllText(reminders_file);
                        int FindID = reminders_ini_text.IndexOf(existing_uniqueID.ToString());
                        if (FindID == -1)
                        {
                            // there is no reminder for this hidden note. Let's show it.
                            WriteINI("properties", "hiddenreminder", "0", INI_path);
                            notes_load_existing_note(existing_uniqueID, frmMain_instance);
                        }
                    }
                    catch
                    {
                    }
                }
            }
        }

        public static void notes_load_existing_note(int existing_uniqueID, frmMain frmMain_instance, bool reminder = false)
        {
            notelist.Add(new Note(existing_uniqueID, true, frmMain_instance, reminder));
        }

        public static void notes_show_note(int existing_uniqueID, frmMain frmMain_instance, bool reminder = false)
        {
            // first we need to check if the note is already loaded. if show, just show it.
            // loop through all the notes until we find the ID.
            foreach (Note n in notelist)
            {
                if (n.frmNote_instance.UniqueID == existing_uniqueID)
                {
                    // note has already been loaded. It's just hidden. Show it.
                    if (reminder == true)
                    {
                        // center the screen
                        n.frmNote_instance.CenterScreen();
                        n.frmNote_instance.Show();
                        n.frmNote_instance.KeepNoteOnTop(1);
                        n.frmNote_instance.PlaySound(WorkPath + "\\notes\\" + existing_uniqueID + ".ini");
                    }
                    else
                    {
                        n.frmNote_instance.Show();
                    }
                    return;
                }
            }
            // if we are still here, the note has not been loaded. Let's load it.
            notes_load_existing_note(existing_uniqueID, frmMain_instance, reminder);
        }

        public static int notes_create_note(frmMain frmMain_instance)
        {
            if (note_theme == "")
            {
                note_theme = ReadINI("notes", "color_theme", NotesINI);
                if (note_theme == "")
                {
                    note_theme = "light colors";
                    WriteINI("notes", "color_theme", "light colors", NotesINI);
                }
            }

            try
            {
                notelist.Add(new Note(0, false, frmMain_instance));
                return 1;
            }
            catch
            {
                return 0;
            }
        }

        public static void notes_open_reminder_window(string existing_note_text = "", string note_backcolor = "", int tempUniqueID = 0)
        {
            // this will open up the reminder window.
            frmReminder frmReminder_instance = new frmReminder();
            if (existing_note_text.Length > 0)
            {
                frmReminder_instance.txtNote.Text = existing_note_text;
            }
            if (tempUniqueID > 0)
            {
                frmReminder_instance.txtNote.Tag = tempUniqueID;
                frmReminder_instance.txtNote.BackColor = System.Drawing.ColorTranslator.FromHtml(note_backcolor);
                frmReminder_instance.txtNote.ReadOnly = true;
            }
            frmReminder_instance.Show();
        }

        public static int notes_can_user_set_reminder()
        {
            string reminders_ini = WorkPath + "\\notes\\settings\\reminders.ini";
            string current_type;
            int i;
            for (i = 1; i <= 100; i++)
            {
                current_type = ReadINI("reminders", i + "-type", reminders_ini);
                if (current_type.Length == 0)
                {
                    break;
                }
            }
            return 1;
        }

        public static int notes_set_reminder(int type, int year, int month, int dayofmonth, int the_hour, int the_minute, int day, string NoteID, int set_day_of_month = 0, int set_month = 0)
        {
            // this will save a note reminder to the reminders.ini file. frmMain should have a timer to monitor these alarms.
            
            string reminders_ini = WorkPath + "\\notes\\settings\\reminders.ini";
            string current_type;
            int return_val;
            int i;
            
            for (i = 1; i <= 100; i++)
            {
                current_type = ReadINI("reminders", i + "-type", reminders_ini);
                if (current_type.Length == 0)
                {
                    break;
                }
            }

            if (i >= 100)
                return -1;

            // add the reminder at position, i
            if (type == 1)
            {
                // one time reminder
                return_val = WriteINI("reminders", i + "-type", type.ToString(), reminders_ini);
                if (return_val == 0)
                    return 0;
                return_val = WriteINI("reminders", i + "-year", year.ToString(), reminders_ini);
                if (return_val == 0)
                    return 0;
                return_val = WriteINI("reminders", i + "-month", month.ToString(), reminders_ini);
                if (return_val == 0)
                    return 0;
                return_val = WriteINI("reminders", i + "-dayofmonth", dayofmonth.ToString(), reminders_ini);
                if (return_val == 0)
                    return 0;
                return_val = WriteINI("reminders", i + "-timehour", the_hour.ToString(), reminders_ini);
                if (return_val == 0)
                    return 0;
                return_val = WriteINI("reminders", i + "-timeminute", the_minute.ToString(), reminders_ini);
                if (return_val == 0)
                    return 0;
                return_val = WriteINI("reminders", i + "-day", day.ToString(), reminders_ini);
                if (return_val == 0)
                    return 0;
                return_val = WriteINI("reminders", i + "-noteid", NoteID, reminders_ini);
                if (return_val == 0)
                    return 0;
            }
            else if (type == 2)
            {
                // daily reminder
                return_val = WriteINI("reminders", i + "-type", type.ToString(), reminders_ini);
                if (return_val == 0)
                    return 0;
                return_val = WriteINI("reminders", i + "-timehour", the_hour.ToString(), reminders_ini);
                if (return_val == 0)
                    return 0;
                return_val = WriteINI("reminders", i + "-timeminute", the_minute.ToString(), reminders_ini);
                if (return_val == 0)
                    return 0;
                return_val = WriteINI("reminders", i + "-lastran_day", "0", reminders_ini);
                if (return_val == 0)
                    return 0;
                return_val = WriteINI("reminders", i + "-lastran_hour", "0", reminders_ini);
                if (return_val == 0)
                    return 0;
                return_val = WriteINI("reminders", i + "-lastran_minute", "0", reminders_ini);
                if (return_val == 0)
                    return 0;
                return_val = WriteINI("reminders", i + "-noteid", NoteID, reminders_ini);
                if (return_val == 0)
                    return 0;
            }
            else if (type == 3)
            {
                // weekly reminder
                return_val = WriteINI("reminders", i + "-type", type.ToString(), reminders_ini);
                if (return_val == 0)
                    return 0;
                return_val = WriteINI("reminders", i + "-timehour", the_hour.ToString(), reminders_ini);
                if (return_val == 0)
                    return 0;
                return_val = WriteINI("reminders", i + "-timeminute", the_minute.ToString(), reminders_ini);
                if (return_val == 0)
                    return 0;
                return_val = WriteINI("reminders", i + "-day", day.ToString(), reminders_ini);
                if (return_val == 0)
                    return 0;
                if (set_day_of_month > 0)
                    return_val = WriteINI("reminders", i + "-lastran_dayofmonth", set_day_of_month.ToString(), reminders_ini);
                else
                    return_val = WriteINI("reminders", i + "-lastran_dayofmonth", "0", reminders_ini);
                if (return_val == 0)
                    return 0;
                return_val = WriteINI("reminders", i + "-noteid", NoteID, reminders_ini);
                if (return_val == 0)
                    return 0;
            }
            else if (type == 4)
            {
                // monthly reminder
                return_val = WriteINI("reminders", i + "-type", type.ToString(), reminders_ini);
                if (return_val == 0)
                    return 0;
                return_val = WriteINI("reminders", i + "-timehour", the_hour.ToString(), reminders_ini);
                if (return_val == 0)
                    return 0;
                return_val = WriteINI("reminders", i + "-timeminute", the_minute.ToString(), reminders_ini);
                if (return_val == 0)
                    return 0;
                return_val = WriteINI("reminders", i + "-dayofmonth", dayofmonth.ToString(), reminders_ini);
                if (return_val == 0)
                    return 0;
                if (set_month > 0)
                    return_val = WriteINI("reminders", i + "-lastran_month", set_month.ToString(), reminders_ini);
                else
                    return_val = WriteINI("reminders", i + "-lastran_month", month.ToString(), reminders_ini);
                if (return_val == 0)
                    return 0;
                return_val = WriteINI("reminders", i + "-noteid", NoteID, reminders_ini);
                if (return_val == 0)
                    return 0;
            }
            return 1;
        }

        public static int notes_delete_note(int UniqueID)
        {
            // loop through all the notes until we find the ID.
            int counter = 0;
            int check_reminder;
            string reminders_file = WorkPath + "\\notes\\settings\\reminders.ini";
            int reminder_type = 0;
            bool reminder_exists = false;

            foreach (Note n in notelist)
            {
                if (n.frmNote_instance.UniqueID == UniqueID)
                {
                    notelist.RemoveAt(counter);
                    break;
                }
                counter++;
            }
            // are there any reminders for this note? If so, delete them.
            for (int i = 1; i <= 100; i++)
            {
                check_reminder = ReadIntFromINI("reminders", i + "-noteid", reminders_file, 0);
                if (check_reminder == 0)
                    break;
                if (check_reminder == UniqueID)
                {
                    // we found the note reminder in the reminders.ini file.
                    // we don't want to delete a daily, weekly, or monthly reminder. These reminders should only be deleted from the 'View Reminders' window.
                    reminder_type = ReadIntFromINI("reminders", i + "-type", reminders_file, 0);
                    if (reminder_type == 1)
                    {
                        // this note has a one-time reminder. delete it.
                        Globals.WriteINI("reminders", i + "-type", null, reminders_file);
                        Globals.WriteINI("reminders", i + "-year", null, reminders_file);
                        Globals.WriteINI("reminders", i + "-month", null, reminders_file);
                        Globals.WriteINI("reminders", i + "-dayofmonth", null, reminders_file);
                        Globals.WriteINI("reminders", i + "-timehour", null, reminders_file);
                        Globals.WriteINI("reminders", i + "-timeminute", null, reminders_file);
                        Globals.WriteINI("reminders", i + "-day", null, reminders_file);
                        Globals.WriteINI("reminders", i + "-lastran_month", null, reminders_file);
                        Globals.WriteINI("reminders", i + "-lastran_day", null, reminders_file);
                        Globals.WriteINI("reminders", i + "-lastran_dayofmonth", null, reminders_file);
                        Globals.WriteINI("reminders", i + "-lastran_hour", null, reminders_file);
                        Globals.WriteINI("reminders", i + "-lastran_minute", null, reminders_file);
                        Globals.WriteINI("reminders", i + "-noteid", null, reminders_file);
                        reminder_exists = true;
                        break;
                    }
                    break;
                }
            }
            if (reminder_exists == true)
            {
                Reorganize_reminders();
            }
            if (reminder_type == 0 || reminder_type == 1)
            {
                // there is either no reminder or there is a one-time reminder. Either way, we want to delete the note.
                try
                {
                    System.IO.File.Delete(Globals.WorkPath + "\\notes\\" + UniqueID.ToString() + ".txt");
                    System.IO.File.Delete(Globals.WorkPath + "\\notes\\" + UniqueID.ToString() + ".ini");
                    return 1;
                }
                catch
                {
                    return 0;
                }
            }
            else
            {
                // it's a daily, weekly, or monthly reminder. Intead of deleting the note, we need to make it hidden.
                WriteINI("properties", "hiddenreminder", "1", Globals.WorkPath + "\\notes\\" + UniqueID.ToString() + ".ini");
                return 1;
            }
        }

        public static void notes_show_hidden_notes()
        {
            foreach (Note n in notelist)
            {
                n.frmNote_instance.Visible = true;
                WriteINI("properties", "hidden", "0", NotesINI + "\\" + n.frmNote_instance.UniqueID + ".ini");
            }
        }

        public static void notes_bring_notes_to_front(bool front = true)
        {
            // this will either bring notes to the front of the screen (usually when the user presses the hotkey)
            // or it will send notes to the back of the screen (usually when the main window becomes inactive).
            foreach (Note n in notelist)
            {
                if (n.frmNote_instance.Visible == true)
                {
                    if (front == true)
                        n.frmNote_instance.Activate();
                    else
                    {
                        // if any notes are supposed to stay on top, we don't want to send them to the bottom.
                        if (n.frmNote_instance.keepToolStripMenuItem.Checked == false)
                            SetWindowPos(n.frmNote_instance.Handle, HWND_BOTTOM, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE | SWP_NOACTIVATE);
                    }
                }
            }
        }

        public static void notes_reset_positions()
        {
            // this will place all notes in the top-left corner of the screen.
            // useful for when notes get off the screen.
            int toppoint = 0;
            int leftpoint = 0;
            foreach (Note n in notelist)
            {
                n.frmNote_instance.Visible = true;
                n.frmNote_instance.Top = toppoint;
                n.frmNote_instance.Left = leftpoint;
                toppoint = toppoint + 30;
                leftpoint = leftpoint + 30;
                WriteINI("properties", "hidden", "0", NotesINI + "\\" + n.frmNote_instance.UniqueID + ".ini");
                WriteINI("properties", "top", toppoint.ToString(), NotesINI + "\\" + n.frmNote_instance.UniqueID + ".ini");
                WriteINI("properties", "left", leftpoint.ToString(), NotesINI + "\\" + n.frmNote_instance.UniqueID + ".ini");
            }
        }

        public static void Reorganize_reminders()
        {
            // this will reorganize reminders in the reminders.ini file. Basically, if a user deletes
            // reminder1, the other reminders will keep their position of reminder2, reminder3, etc.
            // this will change reminder2 to reminder1, reminder3 to reminder2, etc.

            string check_reminder;
            string reminders_file = WorkPath + "\\notes\\settings\\reminders.ini";
            int the_counter = 1;

            for (int i = 1; i <= 100; i++)
            {
                check_reminder = ReadINI("reminders", i + "-type", WorkPath + "\\notes\\settings\\reminders.ini");
                if (check_reminder.Length > 0)
                {
                    if (i > the_counter)
                    {
                        // we skipped one. let's move everything past this point down one.
                        Globals.WriteINI("reminders", the_counter + "-type", Globals.ReadINI("reminders", i + "-type", reminders_file), reminders_file);
                        Globals.WriteINI("reminders", the_counter + "-year", Globals.ReadINI("reminders", i + "-year", reminders_file), reminders_file);
                        Globals.WriteINI("reminders", the_counter + "-month", Globals.ReadINI("reminders", i + "-month", reminders_file), reminders_file);
                        Globals.WriteINI("reminders", the_counter + "-dayofmonth", Globals.ReadINI("reminders", i + "-dayofmonth", reminders_file), reminders_file);
                        Globals.WriteINI("reminders", the_counter + "-timehour", Globals.ReadINI("reminders", i + "-timehour", reminders_file), reminders_file);
                        Globals.WriteINI("reminders", the_counter + "-timeminute", Globals.ReadINI("reminders", i + "-timeminute", reminders_file), reminders_file);
                        Globals.WriteINI("reminders", the_counter + "-day", Globals.ReadINI("reminders", i + "-day", reminders_file), reminders_file);
                        Globals.WriteINI("reminders", the_counter + "-noteid", Globals.ReadINI("reminders", i + "-noteid", reminders_file), reminders_file);
                        Globals.WriteINI("reminders", the_counter + "-lastran_month", Globals.ReadINI("reminders", i + "-lastran_month", reminders_file), reminders_file);
                        Globals.WriteINI("reminders", the_counter + "-lastran_day", Globals.ReadINI("reminders", i + "-lastran_day", reminders_file), reminders_file);
                        Globals.WriteINI("reminders", the_counter + "-lastran_dayofmonth", Globals.ReadINI("reminders", i + "-lastran_dayofmonth", reminders_file), reminders_file);
                        Globals.WriteINI("reminders", the_counter + "-lastran_hour", Globals.ReadINI("reminders", i + "-lastran_hour", reminders_file), reminders_file);
                        Globals.WriteINI("reminders", the_counter + "-lastran_minute", Globals.ReadINI("reminders", i + "-lastran_minute", reminders_file), reminders_file);
                        // remove the original entry now that we moved it down.
                        Globals.WriteINI("reminders", i + "-type", null, reminders_file);
                        Globals.WriteINI("reminders", i + "-year", null, reminders_file);
                        Globals.WriteINI("reminders", i + "-month", null, reminders_file);
                        Globals.WriteINI("reminders", i + "-dayofmonth", null, reminders_file);
                        Globals.WriteINI("reminders", i + "-timehour", null, reminders_file);
                        Globals.WriteINI("reminders", i + "-timeminute", null, reminders_file);
                        Globals.WriteINI("reminders", i + "-day", null, reminders_file);
                        Globals.WriteINI("reminders", i + "-lastran_month", null, reminders_file);
                        Globals.WriteINI("reminders", i + "-lastran_day", null, reminders_file);
                        Globals.WriteINI("reminders", i + "-lastran_dayofmonth", null, reminders_file);
                        Globals.WriteINI("reminders", i + "-lastran_hour", null, reminders_file);
                        Globals.WriteINI("reminders", i + "-lastran_minute", null, reminders_file);
                        Globals.WriteINI("reminders", i + "-noteid", null, reminders_file);
                    }
                    the_counter++;
                }
            }
        }

        public static void notes_hide_all_notes()
        {
            foreach (Note n in notelist)
            {
                n.frmNote_instance.Visible = false;
                WriteINI("properties", "hidden", "1", NotesINI + "\\" + n.frmNote_instance.UniqueID + ".ini");
            }
        }

        public static void notes_keep_all_notes_on_top(int on_top)
        {
            foreach (Note n in notelist)
            {
                if (on_top == 1)
                {
                    n.frmNote_instance.TopMost = true;
                    WriteINI("properties", "on_top", "1", NotesINI + "\\" + n.frmNote_instance.UniqueID + ".ini");
                }
                else
                {
                    n.frmNote_instance.TopMost = false;
                    WriteINI("properties", "on_top", "0", NotesINI + "\\" + n.frmNote_instance.UniqueID + ".ini");
                }

            }
        }

        public static void sound_play_sound(string file_path, bool loop = false)
        {
            if (File.Exists(file_path) == false)
                return;

            if (loop == true)
            {
                sound_done_looping = false;
                for (int i = 1; i <= 50; i++)
                {
                    SoundPlayer simpleSound = new SoundPlayer(file_path);
                    simpleSound.Play();
                    Application.DoEvents();
                    if (sound_done_looping == true)
                        break;
                    System.Threading.Thread.Sleep(2000);
                }
            }
            else
            {
                SoundPlayer simpleSound = new SoundPlayer(file_path);
                simpleSound.Play();
            }
        }

        public static int RunURL(string TheURL)
        {
            // add http:// if it's not there and https:// or ftp:// or anything else 
            // is not being used.
            if (TheURL.IndexOf("://") == -1)
            {
                TheURL = "http://" + TheURL;
            }
            try
            {
                System.Diagnostics.Process.Start(TheURL);
            }
            catch (System.ComponentModel.Win32Exception noBrowser)
            {
                if (noBrowser.ErrorCode == -2147467259)
                    return -1;
            }
            catch (System.Exception)
            {
                return -1;
            }
            return 1;
        }

        public static void MakeWindowTranslucent(IntPtr Handle, int percent, bool undo = false)
        {
            int newDwLong = ((int)GetWindowLong(Handle, GWL_EXSTYLE)) ^ WS_EX_LAYERED;
            SetWindowLong(Handle, GWL_EXSTYLE, newDwLong);
            if (undo == false)
            {
                double double_number = 255 * (percent * 0.01);
                int int_number = Convert.ToInt32(double_number);
                byte byte_number = (byte)int_number;
                SetLayeredWindowAttributes(Handle, 0, byte_number, LWA_ALPHA);
            }
            else
            {
                SetLayeredWindowAttributes(Handle, 0, 255, LWA_ALPHA);
            }
        }

        public static void SetWindowOnTop(IntPtr Handle, bool undo = false, string title_bar_chars = "")
        {
            if (undo == false)
                SetWindowPos(Handle, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
            else
            {
                // we have to identify the window with part of the title bar.
                Process[] processlist = Process.GetProcesses();
                string input = title_bar_chars.ToLower();
                string window_title;
                int find_match;
                IntPtr exact_matching_window_handle = IntPtr.Zero;
                IntPtr partial_matching_window_handle = IntPtr.Zero;
                bool partial_match = false;

                foreach (Process process in processlist)
                {
                    if (!String.IsNullOrEmpty(process.MainWindowTitle))
                    {
                        window_title = process.MainWindowTitle.ToLower();
                        find_match = window_title.IndexOf(input);
                        if (find_match == 0)
                        {
                            // we have an exact match. set focus to the window and exit the loop.
                            try
                            {
                                exact_matching_window_handle = FindWindow(null, process.MainWindowTitle);
                                SetWindowPos(exact_matching_window_handle, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
                            }
                            catch { }
                            break;
                        }
                        else if (find_match > 0)
                        {
                            // we have a partial match. keep a record of this. if we don't find anything, set focus to it.
                            partial_match = true;
                            partial_matching_window_handle = FindWindow(null, process.MainWindowTitle);
                        }
                    }
                }
                if (partial_match == true)
                {
                    try
                    {
                        SetWindowPos(partial_matching_window_handle, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
                    }
                    catch { }
                }
                SetWindowPos(Handle, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
            }
        }

        public static string replace_invalid_filename_characters(string the_filename, string the_path, bool reverse = false)
        {
            /*
             * this function will replace characters and words from filenames. useful for creating text functions since these are dependent on the keyword name.
             * these characters are not allowed in Windows:
             * / \ ? * : " < > |
             * reserved words (not going to include these yet):
             * CON, PRN, AUX, NUL
               COM0, COM1, COM2, COM3, COM4, COM5, COM6, COM7, COM8, COM9
               LPT0, LPT1, LPT2, LPT3, LPT4, LPT5, LPT6, LPT7, LPT8, and LPT9
             */
            string new_filename = the_path + the_filename;
            if (reverse == false)
            {
                new_filename = new_filename.Replace("/", "{forward_slash}");
                new_filename = new_filename.Replace(@"\", "{back_slash}");
                new_filename = new_filename.Replace("?", "{question}");
                new_filename = new_filename.Replace("*", "{asterisk}");
                new_filename = new_filename.Replace(":", "{colon}");
                new_filename = new_filename.Replace("\"", "{quote}");
                new_filename = new_filename.Replace("<", "{left_arrow}");
                new_filename = new_filename.Replace(">", "{right_arrow}");
                new_filename = new_filename.Replace("|", "{pipe}");
            }
            else
            {
                new_filename = new_filename.Replace("{forward_slash}", "/");
                new_filename = new_filename.Replace("{back_slash}", @"\");
                new_filename = new_filename.Replace("{question}", "?");
                new_filename = new_filename.Replace("{asterisk}", "*");
                new_filename = new_filename.Replace("{colon}", ":");
                new_filename = new_filename.Replace("{quote}", "\"");
                new_filename = new_filename.Replace("{left_arrow}", "<");
                new_filename = new_filename.Replace("{right_arrow}", ">");
                new_filename = new_filename.Replace("{pipe}", "|");
            }
            return new_filename;
        }
    }


    // Note section

    public class Note
    {
        public frmNote frmNote_instance;
        public Note(int uniqueID, bool existing_note, frmMain frmMain_instance, bool reminder = false)
        {
            if (uniqueID == 0)
            {
                Random random = new Random();
                uniqueID = random.Next(1, 999999999);
            }
            frmNote_instance = new frmNote(uniqueID, existing_note, reminder, frmMain_instance);
            frmNote_instance.Show();
        }
    }

    public class Keyword
    {
        public string name;
        public string target;
        //public List<string> label = new List<string>();
        public string path = "";
    }

    public class History
    {
        public string e;
    }

    public class Keyword_Folder
    {
        public string GUID = Guid.NewGuid().ToString();
        public string path;
        public string parent;
    }

    public class Letter_Command
    {
        public string name;
        // if the url contains %keybreeze_entry%, it will be replaced with the entry the user typed after the name.
        public string url;
        public string path = "";
        public List<string> replace_this = new List<string>();
        public List<string> replace_with = new List<string>();
    }

    public class Keybreeze_Clipboard
    {
        public string clipboard_text;
        public DateTime timestamp = DateTime.Now;
    }
}
