﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Text;

namespace Keybreeze
{
    static class Program
    {
        [DllImport("user32.dll")]
        public static extern IntPtr FindWindow(String sClassName, String sAppName);
        [DllImport("user32.dll", SetLastError = true, CharSet = CharSet.Auto)]
        static extern uint RegisterWindowMessage(string lpString);
        [DllImport("user32.dll", CharSet = CharSet.Auto)]
        public static extern IntPtr PostMessage(IntPtr hWnd, uint Msg, IntPtr wParam, IntPtr lParam);

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main(string[] args)
        {
            bool onlyInstance = false;
            string command_line_parameters = "";
            uint msg;

            if (args.Length > 0)
            {
                for (int i = 0; i < args.Length; i++)
                {

                    command_line_parameters = command_line_parameters + " " + args[i];
                }
            }
            Mutex mutex = new Mutex(true, "Keybreeze4000", out onlyInstance);
            if (!onlyInstance)
            {
                if (command_line_parameters.Length == 0)
                {
                    // keybreeze is already running, and there are no command line parameters. Just show the main form.
                    msg = RegisterWindowMessage("bring_keybreeze_to_front");
                    IntPtr Keybreezehwnd = FindWindow(null, "Keybreeze Interface");
                    PostMessage(Keybreezehwnd, msg, IntPtr.Zero, IntPtr.Zero);
                    Application.Exit();
                    return;
                }
                else
                {
                    // keybreeze is already running and the user specified a command line argument.
                    // command line argument is probably the path to a file, but it might also be an instruction from a macro. Send it to the existing instance.
                    command_line_parameters = command_line_parameters.Trim();
                    string file_path = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + "\\Keybreeze5\\command_line_argument.txt";
                    try
                    {
                        System.IO.File.WriteAllText(file_path, command_line_parameters, Encoding.Unicode);
                    }
                    catch
                    {
                        MessageBox.Show("Keybreeze Desktop could not open the file, " + file_path + ", for writing. Please make sure you have read and write permission to this directory in Windows.", "Keybreeze Desktop");
                        Application.Exit();
                        return;
                    }
                    if (command_line_parameters.IndexOf("launch_keybreeze_entry:") >= 0)
                    {
                        msg = RegisterWindowMessage("launch_keybreeze_entry");
                        IntPtr Keybreezehwnd = FindWindow(null, "Keybreeze Interface");
                        PostMessage(Keybreezehwnd, msg, IntPtr.Zero, IntPtr.Zero);
                        Application.Exit();
                        return;
                    }
                    else
                    {
                        msg = RegisterWindowMessage("add_keyword_to_keybreeze");
                        IntPtr Keybreezehwnd = FindWindow(null, "Keybreeze Interface");
                        PostMessage(Keybreezehwnd, msg, IntPtr.Zero, IntPtr.Zero);
                        Application.Exit();
                        return;
                    }
                }
            }
            else
            {
                // keybreeze is not running.
                if (command_line_parameters.Length == 0)
                {
                    Application.EnableVisualStyles();
                    Application.SetCompatibleTextRenderingDefault(false);
                    Application.Run(new frmMain());
                    GC.KeepAlive(mutex);
                    return;
                }
                else
                {
                    Application.EnableVisualStyles();
                    Application.SetCompatibleTextRenderingDefault(false);
                    Globals.command_line_parameters = command_line_parameters;
                    Application.Run(new frmMain());
                    GC.KeepAlive(mutex);
                    return;
                }
            }
        }
    }
}
