﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Newtonsoft.Json;

namespace Keybreeze
{
    public partial class frmExport : Form
    {
        //private Entry_To_Export entries_to_export = new Entry_To_Export();

        public frmExport()
        {
            InitializeComponent();
        }

        private void frmShare_Load(object sender, EventArgs e)
        {
            DataGridView_Setup_Columns(dataGridView1);
            DataGridView_Setup_Columns(dataGridView2);
            LoadEntries("");
        }

        private void DataGridView_Setup_Columns(DataGridView this_datagridview)
        {
            this_datagridview.ColumnCount = 4;
            this_datagridview.Columns[0].Name = "Name";
            this_datagridview.Columns[1].Name = "Type";
            this_datagridview.Columns[2].Name = "Path";
            this_datagridview.Columns[3].Name = "Target";
            this_datagridview.Columns[0].Width = 175;
            this_datagridview.Columns[1].Width = 100;
            this_datagridview.Columns[2].Width = 100;
            this_datagridview.Columns[3].Width = 500;
            this_datagridview.RowHeadersWidth = 30;
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            Add_Selected_Entry();
        }

        private void Add_Selected_Entry()
        {
            int current_position = dataGridView1.CurrentRow.Index;
            string selected_keyword = dataGridView1.Rows[current_position].Cells[0].Value.ToString();
            string selected_type = dataGridView1.Rows[current_position].Cells[1].Value.ToString();
            string selected_path = dataGridView1.Rows[current_position].Cells[2].Value.ToString();
            string selected_target = dataGridView1.Rows[current_position].Cells[3].Value.ToString();

            // if we are adding a folder, we want to add all items in that folder.
            if (selected_type == "Keybreeze folder")
            {
                LoadEntries("");
                Application.DoEvents();
                string original_type = selected_path;
                for (int h = 0; h < dataGridView1.RowCount; h++)
                {
                    selected_keyword = dataGridView1.Rows[h].Cells[0].Value.ToString();
                    selected_type = dataGridView1.Rows[h].Cells[1].Value.ToString();
                    selected_path = dataGridView1.Rows[h].Cells[2].Value.ToString();
                    selected_target = dataGridView1.Rows[h].Cells[3].Value.ToString();
                    // make sure this keyword is in the path as the folder that we want to select.
                    // don't add the Keybreeze folder, because we only want to add items inside of it.
                    if (selected_type == "Keybreeze folder")
                        continue;
                    if (selected_path == original_type)
                    {
                        // make sure we haven't already added this item.
                        for (int i = 0; i < dataGridView2.RowCount; i++)
                        {
                            string this_keyword = dataGridView2.Rows[i].Cells[0].Value.ToString();
                            string this_type = dataGridView2.Rows[i].Cells[1].Value.ToString();
                            string this_path = dataGridView2.Rows[i].Cells[2].Value.ToString();
                            string this_target = dataGridView2.Rows[i].Cells[3].Value.ToString();
                            if (this_keyword == selected_keyword && this_type == selected_type && this_path == selected_path && this_target == selected_target)
                            {
                                MessageBox.Show("The item, " + selected_keyword + ", has already been added.", "Keybreeze Desktop");
                                return;
                            }
                        }
                        string[] row;
                        row = new string[] { selected_keyword, selected_type, selected_path, selected_target };
                        dataGridView2.Rows.Add(row);
                    }
                }
            }
            else
            {
                // make sure we haven't already added this item.
                for (int i = 0; i < dataGridView2.RowCount; i++)
                {
                    string this_keyword = dataGridView2.Rows[i].Cells[0].Value.ToString();
                    string this_type = dataGridView2.Rows[i].Cells[1].Value.ToString();
                    string this_path = dataGridView2.Rows[i].Cells[2].Value.ToString();
                    string this_target = dataGridView2.Rows[i].Cells[3].Value.ToString();
                    if (this_keyword == selected_keyword && this_type == selected_type && this_path == selected_path && this_target == selected_target)
                    {
                        MessageBox.Show("The item, " + selected_keyword + ", has already been added.", "Keybreeze Desktop");
                        return;
                    }
                }
                string[] row;
                row = new string[] { selected_keyword, selected_type, selected_path, selected_target };
                dataGridView2.Rows.Add(row);
            }
            txtEntry.Focus();
        }

        private void txtEntry_TextChanged(object sender, EventArgs e)
        {
            tmrAutoCompleteDelay.Enabled = false;
            tmrAutoCompleteDelay.Interval = 250;
            tmrAutoCompleteDelay.Enabled = true;
        }

        private void LoadEntries(string matching_text)
        {
            dataGridView1.Rows.Clear();
            string matching_text_lower = matching_text.ToLower();
            int matching_text_length = matching_text.Length;

            // load folders first.
            foreach (Keyword_Folder this_folder in Globals.Keyword_Folders)
            {
                if (matching_text == "")
                {
                    // load all folders.
                    string[] row;
                    row = new string[] { this_folder.path, "Keybreeze folder", this_folder.path, "folder" };
                    dataGridView1.Rows.Add(row);
                }
                else
                {
                    // see if the folder name starts with the matching_text
                    if (this_folder.path.Length >= matching_text_length && this_folder.path.ToLower().IndexOf(matching_text_lower) == 0)
                    {
                        string[] row;
                        row = new string[] { this_folder.path, "Keybreeze folder", this_folder.path, "folder" };
                        dataGridView1.Rows.Add(row);
                    }
                    // now do a contains search if matching_text is more than one character.
                    else if (matching_text_length > 1 && this_folder.path.Length >= matching_text_length && this_folder.path.ToLower().IndexOf(matching_text_lower) != -1)
                    {
                        string[] row;
                        row = new string[] { this_folder.path, "Keybreeze folder", this_folder.path, "folder" };
                        dataGridView1.Rows.Add(row);
                    }
                }
            }

            // load our keywords.

            string current_keyword;
            string current_keyword_lower;
            string current_path;
            string current_target;
            string current_type;

            for (int i = 0; i < Globals.keyword_count; i++)
            {
                current_path = Globals.Keyword_List[i].path;
                current_keyword = Globals.Keyword_List[i].name;
                current_keyword_lower = current_keyword.ToLower();
                current_target = Globals.Keyword_List[i].target;
                // is the keyword a text function or macro?
                if (current_target.Length > 9 && current_target.IndexOf("function:") == 0)
                    current_type = "text function";
                else if (current_target == "macro")
                    current_type = "macro";
                else
                    current_type = "keyword";
                if (matching_text == "")
                {
                    // load all keywords.
                    string[] row;
                    row = new string[] { current_keyword, current_type, current_path, current_target };
                    dataGridView1.Rows.Add(row);
                }
                else
                {
                    // see if the keyword name starts with the matching_text
                    if (current_keyword.Length >= matching_text_length && current_keyword_lower.IndexOf(matching_text_lower) == 0)
                    {
                        string[] row;
                        row = new string[] { current_keyword, current_type, current_path, current_target };
                        dataGridView1.Rows.Add(row);
                    }
                    // now do a contains search.
                    else if (matching_text_length > 1 && current_keyword.Length >= matching_text_length && current_keyword_lower.IndexOf(matching_text_lower) != -1)
                    {
                        string[] row;
                        row = new string[] { current_keyword, current_type, current_path, current_target };
                        dataGridView1.Rows.Add(row);
                    }
                }
            }

            // load our letter commands
            // if the second character is a space, let's remove it so we can try to match the name of a letter command.
            if (matching_text_length > 1)
            {
                if (matching_text.Substring(1, 1) == " ")
                {
                    // remove the space.
                    matching_text = matching_text.Substring(0, 1);
                    matching_text_lower = matching_text.ToLower();
                    matching_text_length = matching_text.Length;
                }
            }
            foreach (Letter_Command this_letter_command in Globals.Letter_Commands)
            {

                if (matching_text == "" || matching_text.IndexOf("letter") != -1)
                {
                    // load all letter commands
                    string[] row;
                    row = new string[] { this_letter_command.name, "letter command", this_letter_command.path, this_letter_command.url };
                    dataGridView1.Rows.Add(row);
                }
                else
                {
                    if (this_letter_command.name.Length >= matching_text_length && this_letter_command.name.ToLower().IndexOf(matching_text_lower) == 0)
                    {
                        string[] row;
                        row = new string[] { this_letter_command.name, "letter command", this_letter_command.path, this_letter_command.url };
                        dataGridView1.Rows.Add(row);
                    }
                }
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            LoadEntries(txtEntry.Text);
            tmrAutoCompleteDelay.Enabled = false;
        }

        private void dataGridView1_CellContentClick(object sender, DataGridViewCellEventArgs e)
        {

        }

        private void txtEntry_KeyDown(object sender, KeyEventArgs e)
        {
            try
            {
                int current_position = dataGridView1.CurrentRow.Index;
                if (e.KeyCode == Keys.Down)
                {
                    if (current_position < dataGridView1.Rows.Count - 1)
                    {
                        dataGridView1.Rows[current_position].Selected = false;
                        dataGridView1.Rows[current_position + 1].Selected = true;
                        dataGridView1.Rows[current_position + 1].Cells[0].Selected = true;
                    }
                    //dataGridView1.Rows[i].Cells[1].Selected = true;
                    e.Handled = true;
                }
                else if (e.KeyCode == Keys.Up)
                {
                    if (current_position > 0)
                    {
                        dataGridView1.Rows[current_position].Selected = false;
                        dataGridView1.Rows[current_position - 1].Selected = true;
                        dataGridView1.Rows[current_position - 1].Cells[0].Selected = true;
                    }
                    e.Handled = true;
                }
            }
            catch { }
        }

        private void btnRemove_Click(object sender, EventArgs e)
        {
            try
            {
                int current_position = dataGridView2.CurrentRow.Index;
                if (current_position > -1)
                {
                    dataGridView2.Rows.RemoveAt(current_position);
                }
            }
            catch { }
        }

        private void dataGridView1_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            Add_Selected_Entry();
        }

        private void btnExport_Click(object sender, EventArgs e)
        {
            // export our entries to a JSON file.
            List<Entry_To_Export> Entries_To_Export = new List<Entry_To_Export>();
            for (int i = 0; i < dataGridView2.RowCount; i++)
            {
                Entry_To_Export entry_to_export = new Entry_To_Export();
                string this_keyword = dataGridView2.Rows[i].Cells[0].Value.ToString();
                string this_type = dataGridView2.Rows[i].Cells[1].Value.ToString();
                string this_path = dataGridView2.Rows[i].Cells[2].Value.ToString();
                string this_target = dataGridView2.Rows[i].Cells[3].Value.ToString();
                entry_to_export.entry = this_keyword;
                entry_to_export.target = this_target;
                entry_to_export.the_path = this_path;
                entry_to_export.type = this_type;
                if (this_type == "letter command")
                {
                    // we need to also export the character replacement options.
                    foreach (Letter_Command this_letter_command in Globals.Letter_Commands)
                    {
                        if (this_letter_command.name == this_keyword)
                        {
                            for (int j = 0; j < this_letter_command.replace_this.Count; j++)
                            {
                                entry_to_export.option_name.Add(this_letter_command.replace_this[j]);
                                entry_to_export.option_value.Add(this_letter_command.replace_with[j]);
                            }
                            break;
                        }
                    }
                }
                else if (this_type == "text function")
                {
                    // we need to export the text function text.
                    string TheTarget = this_target.Substring(9);
                    string filename_valid_characters = Globals.replace_invalid_filename_characters(TheTarget, this_path);
                    string text_file = Globals.WorkPath + "\\text functions\\" + filename_valid_characters + ".txt";
                    string the_text = "";
                    try
                    {
                        the_text = File.ReadAllText(text_file);
                    }
                    catch
                    {
                        MessageBox.Show("The file that contains the text for this text function, " + text_file + ", could not be found, or you may not have permission to access it in Windows. Please check your Windows permissions for this directory, or edit the text function and re-type the text.", "Keybreeze Desktop");
                        return;
                    }
                    entry_to_export.option_name.Add("text");
                    entry_to_export.option_value.Add(the_text);
                }
                else if (this_type == "macro")
                {
                    // we need to export the macro text.
                    string macro_file_name = this_keyword + ".kmcr";
                    string macro_file_path = Globals.WorkPath + "\\macros\\" + macro_file_name;
                    string macro_text = "";
                    try
                    {
                        macro_text = File.ReadAllText(macro_file_path);
                    }
                    catch (Exception e2)
                    {
                        MessageBox.Show("The macro, " + macro_file_name + ", could not be accessed at this path: " + macro_file_path + ". " + e2.Message, "Keybreeze Desktop");
                        return;
                    }
                    entry_to_export.option_name.Add("text");
                    entry_to_export.option_value.Add(macro_text);
                }
                Entries_To_Export.Add(entry_to_export);
            }

            // convert our Entries_To_Export to JSON.
            string json = JsonConvert.SerializeObject(Entries_To_Export);

            // we are ready to export our JSON!
            SaveFileDialog savefile = new SaveFileDialog();
            try
            {
                savefile.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.Desktop);
            }
            catch { }
            // set a default file name
            string datestring = DateTime.Now.Year.ToString() + DateTime.Now.Month.ToString() + DateTime.Now.Day.ToString();
            savefile.FileName = "keybreeze_" + datestring + ".kpack";
            savefile.Filter = "Keybreeze Pack File (*.kpack)|*.kpack";
            try
            {
                if (savefile.ShowDialog() == DialogResult.OK)
                {
                    File.WriteAllText(savefile.FileName, json);
                }
            }
            catch (Exception e3)
            {
                MessageBox.Show("Could not save Keybreeze Export file, " + savefile.FileName + ". " + e3.Message, "Keybreeze Desktop");
                return;
            }
            //this.Hide();
            //Application.DoEvents();
            //MessageBox.Show(savefile.FileName + " was saved successfully!", "Keybreeze Desktop");
            this.Close();
        }

        private void btnOptions_Click(object sender, EventArgs e)
        {

        }
    }

    public class Entry_To_Export
    {
        public string entry = "";
        public string target = "";
        public string the_path = "";
        // int type: 0 = folder, 1 = keyword, 2 = text function, 3 = macro, 4 = letter command.
        public string type = "";
        public List<string> option_name = new List<string>();
        public List<string> option_value = new List<string>();
    }
}
