﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Newtonsoft.Json;

namespace Keybreeze
{
    public partial class frmImport : Form
    {
        public List<Entry_To_Export> Entries_To_Import = new List<Entry_To_Export>();
        public string if_exists_do_this = "";
        public string if_exists_replace_with = "";

        public frmImport()
        {
            InitializeComponent();
        }

        private void btnBrowse_Click(object sender, EventArgs e)
        {
            openFD.Filter = "Keybreeze Pack File (*.kpack)|*.kpack";
            openFD.ShowDialog();
            txtFileToImport.Text = openFD.FileName;
            LoadFile(txtFileToImport.Text);
        }

        private void LoadFile(string file_to_import)
        {
            // make sure file exists.
            if (File.Exists(file_to_import) == false)
            {
                MessageBox.Show("The file, " + file_to_import + ", does not exist.", "Keybreeze Desktop");
                return;
            }
            string file_contents = "";
            try
            {
                file_contents = File.ReadAllText(file_to_import);
            }
            catch (Exception e)
            {
                MessageBox.Show("Cannot load " + file_to_import + ". " + e.Message, "Keybreeze Desktop");
                return;
            }
            // make sure it's valid JSON.
            try
            {
                Entries_To_Import = JsonConvert.DeserializeObject<List<Entry_To_Export>>(file_contents);
            }
            catch (Exception e)
            {
                MessageBox.Show("The file you selected, " + file_to_import + ", is an invalid or corrupted .kpack file. Try posting it at keybreeze.com/forum to see if it can be corrected.", "Keybreeze Desktop");
                return;
            }
            
            // now we need to parse through our Entries_To_Import and load them in the datagridview.
            dataGridView2.Rows.Clear();
            foreach (Entry_To_Export this_entry in Entries_To_Import)
            {
                // build a variable that contains our options for this entry. Options could include letter command settings, macro text, text function text, etc.
                string entry_options = "";
                int option_count = this_entry.option_name.Count;
                for (int i = 0; i < option_count; i++)
                {
                    entry_options += this_entry.option_name[i] + ":" + this_entry.option_value[i];
                    if (i < (option_count - 1))
                        entry_options += ",";
                }
                string[] row;
                row = new string[] { this_entry.entry, this_entry.type, this_entry.the_path, this_entry.target, entry_options};
                dataGridView2.Rows.Add(row);
            }
        }

        private void txtFileToImport_Leave(object sender, EventArgs e)
        {
            if (txtFileToImport.Text.Length > 0)
                LoadFile(txtFileToImport.Text);
        }

        private void frmImport_Load(object sender, EventArgs e)
        {
            Setup_Columns(dataGridView2);
        }

        private void Setup_Columns(DataGridView this_datagridview)
        {
            this_datagridview.ColumnCount = 5;
            this_datagridview.Columns[0].Name = "Name";
            this_datagridview.Columns[1].Name = "Type";
            this_datagridview.Columns[2].Name = "Path";
            this_datagridview.Columns[3].Name = "Target";
            this_datagridview.Columns[4].Name = "Options";
            this_datagridview.Columns[0].Width = 175;
            this_datagridview.Columns[1].Width = 100;
            this_datagridview.Columns[2].Width = 100;
            this_datagridview.Columns[3].Width = 500;
            this_datagridview.Columns[4].Width = 500;
            this_datagridview.RowHeadersWidth = 30;
        }

        private void btnRemove_Click(object sender, EventArgs e)
        {
            try
            {
                int current_position = dataGridView2.CurrentRow.Index;
                if (current_position > -1)
                {
                    dataGridView2.Rows.RemoveAt(current_position);
                }
            }
            catch { }
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void DeleteEntry(string entry_name, string type, string path_name)
        {
            // this will delete an existing entry so that a new one can replace it.
            string entry_name_lower = entry_name.ToLower();
            string path_name_lower = path_name.ToLower();
            if (type != "letter command")
            {
                for (int i = 0; i < Globals.keyword_count; i++)
                {
                    Keyword this_keyword = Globals.Keyword_List[i];
                    if (this_keyword.name.ToLower() == entry_name_lower && this_keyword.path.ToLower() == path_name_lower)
                    {
                        // we found our match.
                        Globals.Keyword_List.RemoveAt(i);
                        Globals.Keywords_Save_To_File();
                        break;
                    }
                }
            }
            else
            {
                // delete the letter command.
                for (int i = 0; i < Globals.Letter_Commands.Count; i++)
                {
                    Letter_Command this_letter_command = Globals.Letter_Commands[i];
                    if (this_letter_command.name.ToLower() == entry_name_lower)
                    {
                        // we found our match.
                        Globals.Letter_Commands.RemoveAt(i);
                        Globals.LetterCommands_Save_To_File();
                    }
                }
            }
        }

        private void btnImport_Click(object sender, EventArgs e)
        {
            btnImport.Enabled = false;
            Application.DoEvents();
            Globals.BackupCommandList();
            for (int i = 0; i < dataGridView2.RowCount; i++)
            {
                bool skip_this_one = false;
                if_exists_do_this = "";
                if_exists_replace_with = "";
                string this_keyword = dataGridView2.Rows[i].Cells[0].Value.ToString();
                string this_type = dataGridView2.Rows[i].Cells[1].Value.ToString();
                string this_keyword_lower = this_keyword.ToLower();
                string this_path = dataGridView2.Rows[i].Cells[2].Value.ToString();
                string this_path_lower = this_path.ToLower();
                // find this entry in our list of entries to export.
                foreach (Entry_To_Export this_entry in Entries_To_Import)
                {
                    if (this_entry.entry == this_keyword && this_entry.the_path == this_path)
                    {
                        // we found the entry. Now let's try to import it.
                        if (this_entry.type == "letter command")
                        {
                            // make sure the letter command doesn't already exist.
                            for (int h = 0; h < Globals.Letter_Commands.Count; h++)
                            {
                                Letter_Command this_letter_command = Globals.Letter_Commands[h];
                                if (this_letter_command.name == this_keyword_lower)
                                {
                                    // it already exists.
                                    string entry_options = "";
                                    int option_count = this_entry.option_name.Count;
                                    for (int j = 0; j < option_count; j++)
                                    {
                                        entry_options += this_entry.option_name[j] + ":" + this_entry.option_value[j];
                                        if (j < (option_count - 1))
                                            entry_options += ",";
                                    }
                                    frmImportConfirm frmImportConfirm_instance = new frmImportConfirm(this);
                                    string[] row;
                                    row = new string[] { this_entry.entry, this_entry.type, this_entry.the_path, this_entry.target, entry_options };
                                    frmImportConfirm_instance.btnRenamePath.Enabled = false;
                                    frmImportConfirm_instance.txtRenamePath.Enabled = false;
                                    frmImportConfirm_instance.dataGridView1.Rows.Add(row);
                                    frmImportConfirm_instance.ShowDialog();
                                    // execution will stop above until the user chooses something. Let's see what the user chose.
                                    if (if_exists_do_this == "rename_new_entry")
                                    {
                                        this_keyword = if_exists_replace_with;
                                        this_keyword_lower = this_keyword.ToLower();
                                    }
                                    else if (if_exists_do_this == "skip")
                                    {
                                        skip_this_one = true;
                                    }
                                    else if (if_exists_do_this == "replace_existing")
                                    {
                                        DeleteEntry(this_keyword, this_type, this_path);
                                    }
                                    else if (if_exists_do_this == "stop")
                                    {
                                        this.Close();
                                        return;
                                    }
                                    break;
                                }
                            }
                        }
                        else
                        {
                            // it must be a keyword, text function, or macro.
                            // make sure it doesn't already exist.
                            for (int h = 0; h < Globals.keyword_count; h++)
                            {
                                Keyword existing_keyword = Globals.Keyword_List[h];
                                if (existing_keyword.name.ToLower() == this_keyword_lower && existing_keyword.path.ToLower() == this_path_lower)
                                {
                                    // it already exists. Ask the user if he/she wants to rename it, skip it, replace it, cancel, etc.
                                    string entry_options = "";
                                    int option_count = this_entry.option_name.Count;
                                    for (int j = 0; j < option_count; j++)
                                    {
                                        entry_options += this_entry.option_name[j] + ":" + this_entry.option_value[j];
                                        if (j < (option_count - 1))
                                            entry_options += ",";
                                    }
                                    frmImportConfirm frmImportConfirm_instance = new frmImportConfirm(this);
                                    frmImportConfirm_instance.Setup_Columns(frmImportConfirm_instance.dataGridView1);
                                    frmImportConfirm_instance.Setup_Columns(frmImportConfirm_instance.dataGridView2);
                                    // new entry
                                    string[] row;
                                    row = new string[] { this_entry.entry, this_entry.type, this_entry.the_path, this_entry.target, entry_options};
                                    frmImportConfirm_instance.dataGridView1.Rows.Add(row);
                                    // existing entry
                                    string[] row2;
                                    row2 = new string[] { existing_keyword.name, "", existing_keyword.path, existing_keyword.target, "" };
                                    frmImportConfirm_instance.dataGridView2.Rows.Add(row2);
                                    frmImportConfirm_instance.ShowDialog();
                                    // execution will stop above until the user chooses something. Let's see what the user chose.
                                    if (if_exists_do_this == "rename_new_entry")
                                    {
                                        this_keyword = if_exists_replace_with;
                                        this_keyword_lower = this_keyword.ToLower();
                                    }
                                    else if (if_exists_do_this == "rename_new_path")
                                    {
                                        this_path = if_exists_replace_with;
                                        this_path_lower = this_path.ToLower();
                                    }
                                    else if (if_exists_do_this == "skip")
                                    {
                                        skip_this_one = true;
                                    }
                                    else if (if_exists_do_this == "replace_existing")
                                    {
                                        DeleteEntry(this_keyword, this_type, this_path);
                                    }
                                    else if (if_exists_do_this == "stop")
                                    {
                                        // save what we have so far first.
                                        int save_response1 = Globals.Keywords_Save_To_File();
                                        this.Close();
                                        return;
                                    }
                                    // since we already found it, we don't need to continue. break.
                                    break;
                                }
                            }
                        }
                        // Now we are ready to write the new entry to our file.
                        if (skip_this_one == true)
                            break;
                        // include this_keyword and this_path because they may have changed.
                        Write_New_Entry(this_entry, this_keyword, this_path);
                        break;
                    }
                }
            }
            // update our JSON file.
            int save_response = Globals.Keywords_Save_To_File();
            if (save_response == -1)
            {
                MessageBox.Show("Keybreeze was unable to open the file, " + Globals.KeywordsJSONFile + ", for writing. Please make sure that you have read and write permission in Windows for this directory.", "Keybreeze Desktop");
                btnImport.Enabled = true;
                return;
            }
            else
            {
                this.Close();
            }
        }

        private void Write_New_Path(string the_path)
        {
            // we need to write our path to the folders.json file if it doesn't already exist.
            string parent_of_this_folder = "";
            string path_so_far = "";
            string path_so_far_2 = "";
            string[] paths = the_path.Split('/');
            foreach (string this_folder in paths)
            {
                path_so_far += this_folder + "/";
                bool already_exists = false;
                foreach (Keyword_Folder existing_folder in Globals.Keyword_Folders)
                {
                    if (existing_folder.path.ToLower() == this_folder.ToLower())
                    {
                        already_exists = true;
                    }
                }
                if (already_exists == false)
                {
                    // add the folder.
                    Keyword_Folder new_folder = new Keyword_Folder();
                    new_folder.path = this_folder;
                    new_folder.parent = parent_of_this_folder;
                    if (path_so_far.Substring(path_so_far.Length - 1, 1) == "/")
                    {
                        // remove the trailing slash.
                        path_so_far_2 = path_so_far.Substring(0, path_so_far.Length - 1);
                    }
                    else
                        path_so_far_2 = path_so_far;
                    new_folder.path = path_so_far_2;
                    Globals.Keyword_Folders.Add(new_folder);
                }
                parent_of_this_folder = this_folder;
            }
            Globals.Labels_Save_To_File();
        }

        private void Write_New_Entry(Entry_To_Export this_entry, string this_keyword, string this_path)
        {
            // if this entry has a path, we need to write that to the folders.json file too.
            Write_New_Path(this_path);
            if (this_entry.type == "letter command")
            {
                Letter_Command new_letter_command = new Letter_Command();
                new_letter_command.name = this_keyword;
                new_letter_command.url = this_entry.target;
                if (this_entry.option_name.Count > 0)
                {
                    for (int i = 0; i < this_entry.option_name.Count; i++)
                    {
                        new_letter_command.replace_this.Add(this_entry.option_name[i]);
                        new_letter_command.replace_with.Add(this_entry.option_value[i]);
                    }
                }
                Globals.Letter_Commands.Add(new_letter_command);
                Globals.LetterCommands_Save_To_File();
            }
            else
            {
                // we need to see where the keyword will be placed in the list. It needs to be alphabetical.
                // use the CompareTo() function to see where it might be.
                int alpha_order;
                bool added_new = false;
                int the_count = 0;

                foreach (Keyword current_keyword in Globals.Keyword_List)
                {
                    alpha_order = current_keyword.name.CompareTo(this_entry.entry);
                    if (alpha_order > 0)
                    {
                        // the new keyword is larger than the current keyword. Let's insert it into our class.
                        Keyword new_keyword_item = new Keyword();
                        new_keyword_item.name = this_keyword;
                        new_keyword_item.path = this_path;
                        new_keyword_item.target = this_entry.target;
                        Globals.Keyword_List.Insert(the_count, new_keyword_item);
                        added_new = true;
                        break;
                    }
                    the_count += 1;
                }
                if (added_new == false)
                {
                    // we never added the new keyword to the list, so it must be the last one alphabetically. Add it now.
                    Keyword new_keyword_item = new Keyword();
                    new_keyword_item.name = this_keyword;
                    new_keyword_item.path = this_path;
                    new_keyword_item.target = this_entry.target;
                    Globals.Keyword_List.Add(new_keyword_item);
                    added_new = true;
                }

                // is it a macro or text function?
                // remove any bad characters in the filename.
                string new_filename = Globals.replace_invalid_filename_characters(this_keyword, this_path);
                if (this_entry.type == "macro")
                {
                    try
                    {
                        string macro_text = this_entry.option_value[0];
                        File.WriteAllText(Globals.WorkPath + "\\macros\\" + new_filename + ".kmcr", macro_text);
                    }
                    catch (Exception e)
                    {
                        MessageBox.Show("Could not save macro, " + new_filename + ".kmcr. " + e.Message, "Keybreeze Desktop");
                    }
                }
                else if (this_entry.type == "text function")
                {
                    try
                    {
                        string text_function_text = this_entry.option_value[0];
                        File.WriteAllText(Globals.WorkPath + "\\text functions\\" + new_filename + ".txt", text_function_text);
                    }
                    catch (Exception e)
                    {
                        MessageBox.Show("Could not save macro, " + new_filename + ".txt. " + e.Message, "Keybreeze Desktop");
                    }
                }
            }
        }

        private void txtFileToImport_TextChanged(object sender, EventArgs e)
        {

        }
    }
}
