﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Keybreeze
{
    public partial class frmAdd : Form
    {
        private List<string> letter_name = new List<string>();
        private List<string> letter_url1 = new List<string>();
        private List<string> letter_url2 = new List<string>();
        private List<string> letter_url3 = new List<string>();
        public List<string> replace_this = new List<string>();
        public List<string> replace_with = new List<string>();

        // if changing_entry = 0, we are adding a new one.
        // 1 = we are changing a keyword.
        // 2 = we are changing a text function.
        // 3 = we are adding a text function from the clipboard window.
        public int changing_entry = 0;
        public bool command_list_showing = false;
        bool finished_loading_form = false;
        bool finished_loading_letter_commands = false;
        frmMain frmMain_instance;

        // selected_folder_guid is used to track the folder in case the user renames it.
        public string selected_folder_guid = "";

        public frmAdd(frmMain parent)
        {
            InitializeComponent();
            frmMain_instance = parent;
        }

        private void lblViewExamples_Click(object sender, EventArgs e)
        {
            MessageBox.Show("The target can be a path to a file or folder on your computer. It can also be a path to a network location or web page.", "Keybreeze Desktop");
        }

        private void openFileDialog1_FileOk(object sender, CancelEventArgs e)
        {

        }

        private void button1_Click(object sender, EventArgs e)
        {
            openFD.ShowDialog();
            txtTarget.Text = openFD.FileName;
        }

        private void cmdCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void frmAdd_Load(object sender, EventArgs e)
        {
            Random rnd = new Random();
            int rand_num = rnd.Next(1, 4);
            if (rand_num == 3)
            {
                label11.Text = "Support Keybreeze Desktop";
            }
            if (this.Text != "Edit keyword")
            {
                comboBox1.Items.Add("Normal window");
                comboBox1.Items.Add("Minimized");
                comboBox1.Items.Add("Maximized");
                comboBox1.SelectedIndex = 0;
                ActiveControl = txtKeyword;
                LoadExistingLetterCommands();
            }
            if (changing_entry == 2)
            {
                // we are changing a text function.
                tabControl1.SelectTab(1);
            }
            else if (changing_entry == 3)
            {
                // we are adding a text function from the clipboard window.
                tabControl1.SelectTab(1);
                txtKeyword2.Focus();
            }
            finished_loading_form = true;
        }

        private void tabControl1_SelectedIndexChanged(object sender, EventArgs e)
        {
            // if the user started typing something in the first keyword field,
            // transfer that over to the second one.
            if (txtKeyword.Text.Length > 0 && txtKeyword2.Text == "Keyword/Function name")
            {
                txtKeyword2.Text = txtKeyword.Text;
                txtKeyword2.SelectionStart = txtKeyword2.Text.Length;
            }

            // select the first textbox in the tab
            if (tabControl1.SelectedIndex == 0)
            {
                txtKeyword.Focus();
            }
            else if (tabControl1.SelectedIndex == 1)
            {
                // we want to allow the user to press Enter to enter a new line, so let's disable the form's accept button
                this.AcceptButton = null;
                txtKeyword2.Focus();
            }
            else
            {
                this.AcceptButton = cmdAdd;
            }
        }

        private void txtKeyword2_Enter(object sender, EventArgs e)
        {
            if (txtKeyword2.Text == "Keyword/Function name")
            {
                txtKeyword2.SelectionStart = 0;
                txtKeyword2.SelectionLength = txtKeyword2.Text.Length;
            }
        }

        private void txtTextFunction_Enter(object sender, EventArgs e)
        {
            this.AcceptButton = null;
            if (txtTextFunction.Text == "[Type custom text here]")
            {
                txtTextFunction.SelectionStart = 0;
                txtTextFunction.SelectionLength = txtTextFunction.Text.Length;
            }
        }

        private void txtTextFunction_TextChanged(object sender, EventArgs e)
        {

        }

        private void label4_Click(object sender, EventArgs e)
        {
            // text function examples
            frmMain_instance.RunURL("http://www.keybreeze.com/article/2");
        }

        private void label5_Click(object sender, EventArgs e)
        {
            // text function date/time help
            frmMain_instance.RunURL("http://www.keybreeze.com/article/3");
        }

        private void cmdAdd_Click(object sender, EventArgs e)
        {
            int return_val = 0;
            if (changing_entry == 1)
            {
                // we are changing a keyword
                txtKeyword.Text = txtKeyword.Text.Trim();
                cmdAdd.Enabled = false;
                return_val = ChangeKeyword();
                if (return_val != -1)
                {
                    this.Close();
                }
                cmdAdd.Enabled = true;
                return;
            }
            else if (changing_entry == 2)
            {
                // we are changing a text function
                txtKeyword2.Text = txtKeyword2.Text.Trim();
                // we don't want to trim the text function any more since we fixed extra lines appearing at the bottom of the file.
                //txtTextFunction.Text = txtTextFunction.Text.Trim();
                cmdAdd.Enabled = false;
                return_val = ChangeTextFunction();
                if (return_val == 1)
                {
                    this.Close();
                    return;
                }
                else
                {
                    cmdAdd.Enabled = true;
                    return;
                }
            }
            if (tabControl1.SelectedIndex == 0)
            {
                cmdAdd.Enabled = false;
                AddKeyword();
                cmdAdd.Enabled = true;
            }
            else if (tabControl1.SelectedIndex == 1)
            {
                cmdAdd.Enabled = false;
                return_val = AddTextFunction();
                cmdAdd.Enabled = true;
            }
            else if (tabControl1.SelectedIndex == 2)
            {
                cmdAdd.Enabled = false;
                AddLetterCommand();
                cmdAdd.Enabled = true;
            }
            else if (tabControl1.SelectedIndex == 3)
            {
                MessageBox.Show("Please launch the macro editor to add macros.", "Keybreeze Desktop");
            }
        }

        private void AddKeyword()
        {
            int return_val;
            string the_keyword = txtKeyword.Text;
            string the_path = txtFolder.Text;
            string the_target = txtTarget.Text;
            int FindEXE;
            DialogResult result;

            // remove quotation marks
            // commenting this out because it messes up some command-line parameters. Andrew. 4/5/2016.
            // the_target = the_target.Replace("\"", "");
            // check for command line switches
            FindEXE = the_target.LastIndexOf(".exe");
            if (FindEXE == -1) FindEXE = the_target.LastIndexOf(".lnk");
            if (FindEXE == -1) FindEXE = the_target.LastIndexOf(".bat");
            if (FindEXE > 0)
            {
                if (comboBox1.SelectedIndex == 1)
                {
                    // minimized is selected, so lets add a command line switch that is specific to keybreeze.
                    the_target = the_target + " iwstate1";
                }
                else if (comboBox1.SelectedIndex == 2)
                {
                    // maximized is selected, so lets add a command line switch that is specific to keybreeze.
                    the_target = the_target + " xwstate1";
                }
            }
            else
            {
                if (comboBox1.SelectedIndex > 0)
                {
                    MessageBox.Show("Only executable files can be launched minimized or maximized. Please select 'Normal' for the dropdown box.", "Keybreeze Desktop");
                    return;
                }
            }
            Cursor.Current = Cursors.WaitCursor;
            this.Enabled = false;
            frmMain_instance.Enabled = false;
            return_val = frmMain_instance.AddKeywordToCommandList(the_keyword, the_path, the_target, false);
            this.Enabled = true;
            frmMain_instance.Enabled = true;
            Cursor.Current = Cursors.Default;
            if (return_val == -1)
            {
                // we could not open the INI file for writing.
                MessageBox.Show("Keybreeze Desktop was unable to open the file, " + Globals.KeywordsJSONFile + ", for writing. Please make sure that you have read and write permission in Windows for this directory.", "Keybreeze Desktop");
                return;
            }
            else if (return_val == 2)
            {
                // keyword already exists.
                result = MessageBox.Show("The keyword, " + the_keyword + ", is already in the command list. Do you want to replace it?", "Keybreeze Desktop", MessageBoxButtons.YesNo);
                if (result == DialogResult.Yes)
                {
                    // replace the existing keyword
                    Cursor.Current = Cursors.WaitCursor;
                    this.Enabled = false;
                    frmMain_instance.Enabled = false;
                    return_val = frmMain_instance.AddKeywordToCommandList(the_keyword, the_path, the_target, true);
                    this.Enabled = true;
                    frmMain_instance.Enabled = true;
                    Cursor.Current = Cursors.Default;
                    if (checkBox1.Checked == false)
                    {
                        if (command_list_showing == true)
                            frmMain_instance.UpdateCommandList(the_keyword);
                        this.Close();
                    }
                    else
                    {
                        ClearFields();
                    }
                }
                return;
            }
            else if (return_val == 0)
            {
                // an error occurred
            }
            else
            {
                // success!
                if (checkBox1.Checked == false)
                {
                    if (command_list_showing == true)
                        frmMain_instance.UpdateCommandList(the_keyword);
                    this.Close();
                }
                else
                {
                    ClearFields();
                }
            }
        }

        private int AddTextFunction()
        {
            int return_val;
            string text_function_name = txtKeyword2.Text;
            string the_path = txtFolder.Text;
            string text_function_target = "function:" + text_function_name;
            DialogResult result;
            string new_filename;

            if (txtKeyword2.Text.Length == 0 || txtTextFunction.Text.Length == 0)
            {
                MessageBox.Show("Please fill out all of the fields.", "Keybreeze Desktop");
                return 0;
            }

            if (frmMain_instance.Does_Keyword_Exist(text_function_name) == 1)
            {
                // keyword already exists.
                result = MessageBox.Show("The keyword, " + text_function_name + ", is already in the command list. Do you want to replace it?", "Keybreeze Desktop", MessageBoxButtons.YesNo);
                if (result == DialogResult.Yes)
                {
                    // replace the existing keyword
                    // update the text function text file first.
                    new_filename = Globals.replace_invalid_filename_characters(text_function_name, the_path);
                    return_val = WriteTextFile(Globals.WorkPath + "\\text functions\\" + new_filename + ".txt");
                    if (return_val == 1)
                    {
                        Cursor.Current = Cursors.WaitCursor;
                        this.Enabled = false;
                        frmMain_instance.Enabled = false;
                        return_val = frmMain_instance.AddKeywordToCommandList(text_function_name, the_path, text_function_target, true);
                        this.Enabled = true;
                        frmMain_instance.Enabled = true;
                        Cursor.Current = Cursors.Default;
                        if (checkBox1.Checked == false)
                        {
                            this.Close();
                        }
                        else
                        {
                            ClearFields();
                        }
                        return 1;
                    }
                    else
                        return -1;
                }
                return 0;
            }
            else
            {
                // keyword does not already exist. Let's add it.
                new_filename = Globals.replace_invalid_filename_characters(text_function_name, the_path);
                return_val = WriteTextFile(Globals.WorkPath + "\\text functions\\" + new_filename + ".txt");
                if (return_val == 1)
                {
                    Cursor.Current = Cursors.WaitCursor;
                    this.Enabled = false;
                    frmMain_instance.Enabled = false;
                    return_val = frmMain_instance.AddKeywordToCommandList(text_function_name, the_path, text_function_target, true);
                    this.Enabled = true;
                    frmMain_instance.Enabled = true;
                    Cursor.Current = Cursors.Default;
                    if (checkBox1.Checked == false)
                    {
                        this.Close();
                    }
                    else
                    {
                        ClearFields();
                    }
                    return 1;
                }
                else
                    return -1;
            }
        }

        private void ClearFields()
        {
            if (tabControl1.SelectedIndex == 0)
            {
                // clear keyword fields
                txtKeyword.Text = "";
                txtTarget.Text = "";
                txtKeyword.Focus();
            }
            else if (tabControl1.SelectedIndex == 1)
            {
                // clear text function fields
                txtKeyword2.Text = "";
                txtTextFunction.Text = "";
                txtKeyword2.Focus();
            }
            else if (tabControl1.SelectedIndex == 2)
            {
                // clear letter command fields
                txtExistingURL.Text = "";
                txtLetter.Text = "";
                txtNewURL.Text = "";
                txtLetter.Focus();
            }
            else if (tabControl1.SelectedIndex == 3)
            {
                // there should be nothing to clear for macros
            }
        }

        private void txtKeyword2_TextChanged(object sender, EventArgs e)
        {

        }

        private int WriteTextFile(string thepath)
        {
            // make sure the directory exists first.
            if (Directory.Exists(Globals.WorkPath + "\\text functions") == false)
            {
                try
                {
                    Directory.CreateDirectory(Globals.WorkPath + "\\text functions");
                    System.Threading.Thread.Sleep(400);
                }
                catch
                {
                    MessageBox.Show("Keybreeze Desktop was unable to write to the " + Globals.WorkPath + "\\text functions " + " directory. Please make sure your Windows permissions allow you to create files and folders under this directory.", "Keybreeze Desktop");
                    return -1;
                }
            }
            // find out if it's one line or multiple lines
            try
            {
                if (txtTextFunction.Lines.Count() > 1)
                {
                    StreamWriter sw = new StreamWriter(thepath, false, Encoding.Unicode);
                    String[] contents = txtTextFunction.Lines.ToArray();
                    for (int i = 0; i < contents.Length; i++)
                    {
                        // WriteLine always appends a new line to the output, so we want to use sw.Write for the last line to avoid this.
                        if (i < contents.Length - 1)
                            sw.WriteLine(contents[i]);
                        else
                            sw.Write(contents[i]);
                    }
                    sw.Close();
                    return 1;
                }
                else
                {
                    string TheText = txtTextFunction.Text.Replace(@"\n", @"\r\n");
                    System.IO.File.WriteAllText(thepath, TheText, Encoding.Unicode);
                    return 1;
                }
            }
            catch
            {
                MessageBox.Show("Keybreeze Desktop was unable to open the file, " + thepath + ", for writing. Please make sure that you have read and write permission in Windows for this directory.", "Keybreeze Desktop");
                return -1;
            }
        }

        public void LoadExistingLetterCommands()
        {
            string letter_name;
            listBox1.Items.Clear();

            int i = 0;
            foreach (Letter_Command this_letter_command in Globals.Letter_Commands)
            {
                listBox1.Items.Add(this_letter_command.name);
                if (i == 0)
                {
                    listBox1.SelectedIndex = 0;
                    txtExistingURL.Text = this_letter_command.url;
                    txtExistingURL.Tag = txtExistingURL.Text;
                    listBox1.Tag = this_letter_command.name;
                }
                i++;
            }
            finished_loading_letter_commands = true;
        }

        private void txtExistingLetter1_TextChanged(object sender, EventArgs e)
        {
            
        }

        private void txtExistingLetter2_TextChanged(object sender, EventArgs e)
        {
            
        }

        private void listBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (finished_loading_letter_commands == true)
            {
                if (listBox1.SelectedIndex == -1)
                    listBox1.SelectedIndex = 0;
                
                string selected_letter = listBox1.Items[listBox1.SelectedIndex].ToString();
                foreach (Letter_Command this_letter_command in Globals.Letter_Commands)
                {
                    if (this_letter_command.name == selected_letter)
                    {
                        txtExistingURL.Text = this_letter_command.url;
                        txtExistingURL.Tag = this_letter_command.url;
                    }
                }
            }
        }

        private void cmdChangeLetterCommand_Click(object sender, EventArgs e)
        {
            Change_Letter_Command();
        }

        private void Change_Letter_Command()
        {
            string current_letter = listBox1.Items[listBox1.SelectedIndex].ToString();;
            string char_replace;

            frmAdd_Extra frmAdd_Extra_instance;
            frmAdd_Extra_instance = new frmAdd_Extra(this);
            frmAdd_Extra_instance.txtLetter.Text = current_letter;
            frmAdd_Extra_instance.txtLetter.Tag = current_letter;
            frmAdd_Extra_instance.txtURL.Text = txtExistingURL.Text;
            frmAdd_Extra_instance.txtURL.Tag = txtExistingURL.Text;

            // lets see if this letter command has any character replacement options
            foreach (Letter_Command this_letter_command in Globals.Letter_Commands)
            {
                if (this_letter_command.name == current_letter)
                {
                    if (this_letter_command.replace_this.Count > 0)
                    {
                        // this letter command does have replacement options.
                        int replacement_count = 0;
                        foreach (string this_replacement in this_letter_command.replace_this)
                        {
                            string this_replacement_with = this_letter_command.replace_with[replacement_count];
                            frmAdd_Extra_instance.listBox1.Items.Add(this_replacement + " >> " + this_replacement_with);
                            replacement_count++;
                        }
                    }
                }
            }
            frmAdd_Extra_instance.Show();
        }

        private void label9_Click(object sender, EventArgs e)
        {

        }

        private void label15_Click(object sender, EventArgs e)
        {

        }

        private void cmdRemoveLetterCommand_Click(object sender, EventArgs e)
        {
            DialogResult result;
            string selected_letter_command = listBox1.Items[listBox1.SelectedIndex].ToString();
            int temp1 = listBox1.SelectedIndex + 1;

            letter_name.Clear();
            letter_url1.Clear();
            letter_url2.Clear();
            letter_url3.Clear();

            result = MessageBox.Show("Are you sure you want to delete the letter command, " + selected_letter_command + "?", "Keybreeze Desktop", MessageBoxButtons.YesNo);
            if (result == DialogResult.Yes)
            {
                int count = 0;
                foreach (Letter_Command this_letter_command in Globals.Letter_Commands)
                {
                    if (this_letter_command.name == selected_letter_command)
                    {
                        Globals.Letter_Commands.RemoveAt(count);
                        break;
                    }
                    count++;
                }
                int delete_response = Globals.LetterCommands_Save_To_File();
                if (delete_response == 0)
                {
                    MessageBox.Show("Keybreeze Desktop is unable to modify the file, " + Globals.LetterCommandsJSONFile + ".", "Keybreeze Desktop");
                    return;
                }
                listBox1.Items.Remove(listBox1.SelectedItem);
                Globals.LetterCommands_Save_To_File();
            }
        }

        private void tabPage3_Click(object sender, EventArgs e)
        {

        }

        private void AddLetterCommand()
        {
            // validate the fields first. we need at least two fields filled out.
            if (txtLetter.Text.Length == 0)
            {
                MessageBox.Show("Please specify a name in the 'Letter or phrase' textbox.", "Keybreeze Desktop");
                return;
            }
            if (txtNewURL.Text.Length == 0)
            {
                MessageBox.Show("Please specify a URL for the new letter command.", "Keybreeze Desktop");
                return;
            }
            if (txtLetter.Text.IndexOf(" ") > -1)
            {
                MessageBox.Show("Please do not include any spaces in the letter command name.", "Keybreeze Desktop");
                return;
            }
            if (txtNewURL.Text.IndexOf("%keybreeze_entry%") == -1)
            {
                MessageBox.Show("Please include %keybreeze_entry% in the URL. This will later be replaced by the actual entry that you type.", "Keybreeze Desktop");
                return;
            }

            string new_letter_command = txtLetter.Text.ToLower();

            Letter_Command this_letter_command = new Letter_Command();
            this_letter_command.name = new_letter_command;
            this_letter_command.url = txtNewURL.Text;
            int replacement_count = 0;
            foreach (string this_replacement in replace_this)
            {
                this_letter_command.replace_this.Add(this_replacement);
                this_letter_command.replace_with.Add(replace_with[replacement_count]);
                replacement_count++;
            }
            Globals.Letter_Commands.Add(this_letter_command);
            Globals.LetterCommands_Save_To_File();
            if (checkBox1.Checked == true)
            {
                // keep window open. clear fields. reset the letter command listbox.
                txtLetter.Text = "";
                txtNewURL.Text = "";
                replace_this.Clear();
                replace_with.Clear();
                LoadExistingLetterCommands();
                listBox1.SelectedIndex = listBox1.Items.Count - 1;
                txtLetter.Focus();
            }
            else
            {
                // close window
                this.Close();
            }
        }

        private void label14_Click(object sender, EventArgs e)
        {
            frmAdd_Extra frmAdd_Extra_instance;
            frmAdd_Extra_instance = new frmAdd_Extra(this);
            
            // we have to redesign the form a bit since we are not changing an existing letter command.
            frmAdd_Extra_instance.txtLetter.Text = txtLetter.Text;
            frmAdd_Extra_instance.txtLetter.Enabled = false;
            frmAdd_Extra_instance.txtURL.Text = txtNewURL.Text;
            frmAdd_Extra_instance.txtURL.Enabled = false;
            frmAdd_Extra_instance.Text = "Character replacement option";
            frmAdd_Extra_instance.adding_new = true;
            frmAdd_Extra_instance.Show();
        }

        private void tabControl1_Selecting(object sender, TabControlCancelEventArgs e)
        {
            if (finished_loading_form == true)
            {
                if (changing_entry == 1 || changing_entry == 2)
                    e.Cancel = true;
            }
        }

        private int ChangeTextFunction()
        {
            string the_keyword = txtKeyword2.Text;
            string the_path = txtFolder.Text;
            string full_text = txtTextFunction.Text;
            string file_path = Globals.WorkPath + "\\text functions\\" + the_keyword + ".txt";
            string the_target = "function:" + the_keyword;
            int return_val;
            string new_filename;

            if (txtFolder.Tag == null)
                txtFolder.Tag = "";

            if (txtKeyword2.Text.Length == 0 || txtTextFunction.Text.Length == 0)
            {
                MessageBox.Show("Please fill out all of the fields.", "Keybreeze Desktop");
                return 0;
            }
            if (the_keyword == txtKeyword2.Tag.ToString() && full_text == txtTextFunction.Tag.ToString() && the_path == txtFolder.Tag.ToString())
            {
                // same keyword and text function and path. Don't change anything.
                return 1;
            }
            else if (the_keyword == txtKeyword2.Tag.ToString() && full_text != txtTextFunction.Tag.ToString() && the_path == txtFolder.Tag.ToString())
            {
                // same keyword and path but different text function. Just update the text file.
                // replace any invalid filename characters like \ / " with valid ones.
                new_filename = Globals.replace_invalid_filename_characters(the_keyword, the_path);
                return_val = WriteTextFile(Globals.WorkPath + "\\text functions\\" + new_filename + ".txt");
                if (return_val == -1)
                    return -1;
                else
                    return 1;
            }
            else
            {
                // different keyword or different text function. Update both.
                if (the_keyword != txtKeyword2.Tag.ToString() || the_path != txtFolder.Tag.ToString())
                {
                    // see if the new keyword name already exists first.
                    string the_keyword_lower = the_keyword.ToLower();
                    string the_path_lower = the_path.ToLower();
                    for (int i = 0; i < Globals.keyword_count; i++)
                    {
                        if (Globals.keyword_name_lower[i] == the_keyword_lower && Globals.Keyword_List[i].path.ToLower() == the_path_lower)
                        {
                            // new keyword name already exists.
                            DialogResult result;
                            result = MessageBox.Show("The keyword, " + the_keyword + ", is already in the command list. Do you want to replace it?", "Keybreeze Desktop", MessageBoxButtons.YesNo);
                            if (result == DialogResult.Yes)
                            {
                                // delete old text function file and create a new one.
                                try
                                {
                                    System.IO.File.Delete(Globals.WorkPath + "\\text functions\\" + txtKeyword2.Tag.ToString() + ".txt");
                                }
                                catch
                                {
                                }
                                // replace any invalid filename characters like \ / " with valid ones.
                                new_filename = Globals.replace_invalid_filename_characters(the_keyword, the_path);
                                return_val = WriteTextFile(Globals.WorkPath + "\\text functions\\" + new_filename + ".txt");
                                if (return_val == 1)
                                {
                                    // replace the existing keyword
                                    return_val = frmMain_instance.ChangeKeywordInCommandList(txtKeyword2.Tag.ToString(), the_keyword, txtFolder.Tag.ToString(), the_path, the_target);
                                    if (command_list_showing == true)
                                        frmMain_instance.UpdateCommandList(the_keyword);
                                    return 1;
                                }
                                else
                                    return -1;
                            }
                            else
                            {
                                return -1;
                            }
                        }
                    }
                    // if we made it to this point, the new keyword does not already exist.
                    // let's change it.
                    // create new text function file and delete old one.
                    // replace any invalid filename characters like \ / " with valid ones.
                    new_filename = Globals.replace_invalid_filename_characters(the_keyword, the_path);
                    return_val = WriteTextFile(Globals.WorkPath + "\\text functions\\" + new_filename + ".txt");
                    if (return_val == 1)
                    {
                        try
                        {
                            new_filename = Globals.replace_invalid_filename_characters(txtKeyword2.Tag.ToString(), txtFolder.Tag.ToString());
                            System.IO.File.Delete(Globals.WorkPath + "\\text functions\\" + new_filename + ".txt");
                        }
                        catch
                        {
                        }
                        return_val = frmMain_instance.ChangeKeywordInCommandList(txtKeyword2.Tag.ToString(), the_keyword, txtFolder.Tag.ToString(), the_path, the_target);
                        if (command_list_showing == true)
                            frmMain_instance.UpdateCommandList(the_keyword);
                        return 1;
                    }
                    else
                        return -1;
                }
            }
            return 1;
        }

        private int ChangeKeyword()
        {
            string the_keyword = txtKeyword.Text;
            string the_path = txtFolder.Text;
            string the_target = txtTarget.Text;
            string currentIndex = comboBox1.SelectedIndex.ToString();
            int return_val;

            if (txtFolder.Tag == null)
                txtFolder.Tag = "";

            // remove quotation marks
            // commenting this out. Andrew. 4/5/2016 because it messes up some command line parameters.
            //the_target = the_target.Replace("\"", "");

            if (the_keyword == txtKeyword.Tag.ToString() && the_target == txtTarget.Tag.ToString() && currentIndex == comboBox1.Tag.ToString() && the_path == txtFolder.Tag.ToString())
            {
                // same keyword and target and run option and path. Don't change anything.
                return 1;
            }
            else if (the_keyword == txtKeyword.Tag.ToString() && (the_target != txtTarget.Tag.ToString() || currentIndex != comboBox1.Tag.ToString()))
            {
                // same keyword but different target or different run option.
                the_target = UpdateRunAsOption(the_target);
                return_val = frmMain_instance.ChangeTargetForKeyword(the_keyword, the_target);
                if (command_list_showing == true)
                    frmMain_instance.UpdateCommandList(the_keyword);
                return 1;
            }
            else
            {
                // the user updated the keyword, and they may have updated the target. let's just delete the current keyword and add a new one.
                if (the_keyword != txtKeyword.Tag.ToString() || the_path != txtFolder.Tag.ToString())
                {
                    // see if the new keyword name already exists first.
                    //foreach (string current_keyword in Globals.keyword_names)
                    //foreach(Keyword this_keyword in Globals.Keyword_List)
                    for (int i = 0; i < Globals.keyword_count; i++)
                    {
                        if (Globals.keyword_name_lower[i] == the_keyword.ToLower() && Globals.Keyword_List[i].path == the_path)
                        {
                            // new keyword name and path already exist.
                            DialogResult result;
                            if (the_path.Length > 0)
                                result = MessageBox.Show("The keyword, " + the_keyword + ", already exists at this path: " + the_path + ". Do you want to replace it?", "Keybreeze Desktop", MessageBoxButtons.YesNo);
                            else
                                result = MessageBox.Show("The keyword, " + the_keyword + ", already exists. Do you want to replace it?", "Keybreeze Desktop", MessageBoxButtons.YesNo);
                            if (result == DialogResult.Yes)
                            {
                                // replace the existing keyword
                                the_target = UpdateRunAsOption(the_target);
                                return_val = frmMain_instance.ChangeKeywordInCommandList(txtKeyword.Tag.ToString(), the_keyword, txtFolder.Tag.ToString(), the_path, the_target);
                                if (command_list_showing == true)
                                    frmMain_instance.UpdateCommandList(the_keyword);
                                return 1;
                            }
                            else
                            {
                                return -1;
                            }
                        }
                    }
                    // if we made it to this point, the new keyword does not already exist.
                    // let's change it.
                    the_target = UpdateRunAsOption(the_target);
                    return_val = frmMain_instance.ChangeKeywordInCommandList(txtKeyword.Tag.ToString(), the_keyword, txtFolder.Tag.ToString(), the_path, the_target);
                    if (command_list_showing == true)
                        frmMain_instance.UpdateCommandList(the_keyword);
                    return 1;
                }
            }
            return 1;
        }

        private string UpdateRunAsOption(string the_target)
        {
            // first lets see if the user changed the Run option.
            string currentIndex = comboBox1.SelectedIndex.ToString();
            string currentTag = comboBox1.Tag.ToString();
            if (currentIndex == currentTag)
            {
                // the user did not change the run as option. we still need to put the existing
                // value back in place though if it was originally ran minimized or maximized.
                if (currentIndex == "1")
                {
                    // minimized
                    the_target = the_target + " iwstate1";
                    return the_target;
                }
                else if (currentIndex == "2")
                {
                    // maximized
                    the_target = the_target + " xwstate1";
                    return the_target;
                }
                else
                    return the_target;
            }
            else
            {
                // the user did change it.
                if (currentIndex == "0")
                {
                    return the_target.Trim();
                }
                if (currentIndex == "1")
                {
                    // minimized
                    the_target = the_target + " iwstate1";
                    return the_target;
                }
                else if (currentIndex == "2")
                {
                    // maximized
                    the_target = the_target + " xwstate1";
                    return the_target;
                }
                else
                {
                    // we should not get to this point unless an option is added, like "Run hidden"
                    return the_target;
                }
            }
        }

        private void lblMultipleTargets_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Use macros to launch multiple targets. The <keybreeze entry> command will let you launch an entry from a macro, and you can include delays between each item that is launched. Press the hotkey and type 'macros [create]' to get started.", "Keybreeze Desktop");
        }

        private void txtLabel_TextChanged(object sender, EventArgs e)
        {
            
        }

        private void txtLabel_Enter(object sender, EventArgs e)
        {
            LaunchLabelForm();
        }

        private void txtLabel_Click(object sender, EventArgs e)
        {
            LaunchLabelForm();
        }

        private void LaunchLabelForm()
        {
            if (tabControl1.SelectedIndex == 2)
            {
                MessageBox.Show("A folder can only be added for keywords, text functions or macros. Letter commands cannot be placed into a folder.", "Keybreeze Desktop");
                return;
            }
            frmFolders frmLabels_instance = new frmFolders(this);
            /*
            string labels = txtLabel.Text.Replace(", ", ",");
            string[] label_array = labels.Split(',');
            foreach (string this_label in label_array)
            {
                if (this_label.Length > 0)
                    frmLabels_instance.lstEntryLabels.Items.Add(this_label);
            }
             * */
            frmLabels_instance.txtFolderForThisEntry.Text = txtFolder.Text;
            frmLabels_instance.ShowDialog();
        }

        private void btnChangeFolder_Click(object sender, EventArgs e)
        {
            LaunchLabelForm();
        }

        private void label11_Click(object sender, EventArgs e)
        {
            if (label11.Text == "Support Keybreeze Desktop")
            {
                frmMain_instance.RunURL("https://www.keybreeze.com/donate?from=keybreeze_desktop");
            }
            else
            {
                // what is a Keybreeze folder?
                frmMain_instance.RunURL("http://www.keybreeze.com/article/6");
            }
        }

        private void label15_Click_1(object sender, EventArgs e)
        {
            frmMain_instance.RunURL("http://www.keybreeze.com/article/49");
        }
    }
}
