﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Keybreeze
{
    public partial class frmAdd_Extra : Form
    {
        // this form can be used for changing an existing letter command or setting options for a new one. the "adding_new" variable represents this.
        public Boolean adding_new = false;
        private List<string> letter_name = new List<string>();
        private List<string> letter_url1 = new List<string>();
        private List<string> letter_url2 = new List<string>();
        private List<string> letter_url3 = new List<string>();

        frmAdd frmAdd_instance;
        public frmAdd_Extra(frmAdd parent)
        {
            InitializeComponent();
            frmAdd_instance = parent;
        }

        private void cmdCancel_Click(object sender, EventArgs e)
        {
            frmAdd_instance.Enabled = true;
            this.Close();
        }

        private void frmAdd_Extra_Load(object sender, EventArgs e)
        {
            frmAdd_instance.Enabled = false;
        }

        private void frmAdd_Extra_FormClosing(object sender, FormClosingEventArgs e)
        {
            // is the user adding a new letter command or changing an existing one?
            if (adding_new == false)
            {
                // the user is changing an existing letter command.
                // lets see if the user changed something to the letter command name or URL first.
                int return_val = Update_Letter_Name_and_URL();
                if (return_val == 1)
                {
                    // the user changed the name of the letter command or the URL. Refresh the listbox.
                    frmAdd_instance.LoadExistingLetterCommands();
                }
                else if (return_val == 0)
                {
                    // an error occurred... most likely the user tried to rename the letter command to something that already exists.
                    e.Cancel = true;
                }

                // now we need to add the character replacement options to the class.
                if (listBox1.Items.Count > 0)
                {
                    string letter_lowercase = txtLetter.Text;
                    foreach (Letter_Command this_letter_command in Globals.Letter_Commands)
                    {
                        if (this_letter_command.name == letter_lowercase)
                        {
                            // update the character replacement options for this letter command.
                            this_letter_command.replace_this.Clear();
                            this_letter_command.replace_with.Clear();
                            foreach (string s in listBox1.Items)
                            {
                                int find_arrows = s.IndexOf(">>");
                                string replace_this = s.Substring(0, find_arrows).Trim();
                                string replace_with = s.Substring(find_arrows + 2).Trim();
                                this_letter_command.replace_this.Add(replace_this);
                                this_letter_command.replace_with.Add(replace_with);
                            }
                            break;
                        }
                    }
                    Globals.LetterCommands_Save_To_File();
                }
            }
            else
            {
                // the user is adding a new letter command.
                if (listBox1.Items.Count > 0)
                {
                    foreach (string s in listBox1.Items)
                    {
                        int find_arrows = s.IndexOf(">>");
                        string find_what = s.Substring(0, find_arrows).Trim();
                        string replace_with = s.Substring(find_arrows + 2).Trim();
                        frmAdd_instance.replace_this.Add(find_what);
                        frmAdd_instance.replace_with.Add(replace_with);
                    }
                }
            }
            frmAdd_instance.Enabled = true;
        }

        private void label4_Click(object sender, EventArgs e)
        {
            // letter command help
            System.Diagnostics.Process.Start("http://www.keybreeze.com/article/4");
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (textBox1.Text.Length > 0 && textBox2.Text.Length > 0)
            {
                // add it to the listbox
                string line_to_add;
                line_to_add = textBox1.Text + " >> " + textBox2.Text;
                listBox1.Items.Add(line_to_add);
                textBox1.Text = "";
                textBox2.Text = "";
                textBox1.Focus();
                // we are going to add everything to the JSON class when the form closes.
            }
        }

        private void cmdChange_Click(object sender, EventArgs e)
        {
            
        }

        private void button2_Click(object sender, EventArgs e)
        {
            if (listBox1.SelectedIndex > -1)
            {
                listBox1.Items.Remove(listBox1.SelectedItem);
            }
            else
            {
                MessageBox.Show("Please select the item you want to remove first.", "Keybreeze Desktop");
            }
        }

        private void txtLetter_TextChanged(object sender, EventArgs e)
        {
            
        }

        private int Update_Letter_Name_and_URL()
        {
            // see if the user changed the letter command name or URL first.
            string this_letter_name = txtLetter.Text.ToLower();
            Boolean changed_it = false;
            if (txtLetter.Text != txtLetter.Tag.ToString() && txtLetter.Text.Length > 0)
            {
                // updating the name. we'll need to delete the current letter command and add a new one.
                // make sure the user isn't changing it to something that already exists.
                string current_letter;
                string selected_letter_command = txtLetter.Tag.ToString();
                if (this_letter_name != selected_letter_command)
                {
                    foreach (Letter_Command this_letter_command in Globals.Letter_Commands)
                    {
                        if (this_letter_command.name == this_letter_name)
                        {
                            MessageBox.Show("The letter command you typed is already in the list. Please select a different name or remove the old one first.", "Keybreeze Desktop");
                            return 0;
                        }
                    }
                }
                // delete the current one.
                string old_name = txtLetter.Tag.ToString();
                int letter_command_list_count = Globals.Letter_Commands.Count();
                for (int i = 0; i < letter_command_list_count; i++)
                {
                    if (Globals.Letter_Commands[i].name == old_name)
                    {
                        Globals.Letter_Commands.RemoveAt(i);
                        break;
                    }
                }
                // add new one.
                int alpha_order;
                bool added_new = false;
                int the_count = 0;

                foreach (Letter_Command this_letter_command in Globals.Letter_Commands)
                {
                    alpha_order = this_letter_command.name.CompareTo(this_letter_name);
                    if (alpha_order > 0)
                    {
                        // the new letter command is larger than the current one. Let's insert it into our class.
                        Letter_Command new_letter_command = new Letter_Command();
                        new_letter_command.name = this_letter_name;
                        // TO DO
                        //new_letter_command.path = ThePath;
                        new_letter_command.url = txtURL.Tag.ToString();
                        Globals.Letter_Commands.Insert(the_count, new_letter_command);
                        added_new = true;
                        break;
                    }
                    the_count += 1;
                }
                if (added_new == false)
                {
                    // we never added the new keyword to the list, so it must be the last one alphabetically. Add it now.
                    Letter_Command new_letter_command = new Letter_Command();
                    new_letter_command.name = this_letter_name;
                    // TO DO
                    //new_letter_command.path = ThePath;
                    new_letter_command.url = txtURL.Tag.ToString();
                    Globals.Letter_Commands.Add(new_letter_command);
                    added_new = true;
                }

                // update our JSON file.
                Globals.LetterCommands_Save_To_File();
                changed_it = true;
            }
            else if (txtURL.Text != txtURL.Tag.ToString())
            {
                // updating the target/url.
                foreach (Letter_Command this_letter_command in Globals.Letter_Commands)
                {
                    if (this_letter_command.name == this_letter_name)
                    {
                        // we found our letter command. Let's update the target/url.
                        this_letter_command.url = txtURL.Text;
                    }
                }
                Globals.LetterCommands_Save_To_File();
                changed_it = true;
            }
            else
            {
                changed_it = false;
            }

            if (changed_it == true)
            {
                
                int temp1 = listBox1.SelectedIndex + 1;
                int letter_command_count = 0;
                string current_letter_url1;
                string current_letter_url2;
                string current_letter_url3;

                letter_name.Clear();
                letter_url1.Clear();
                letter_url2.Clear();
                letter_url3.Clear();
                return 1;
            }
            else
            {
                return -1;
            }
        }

        private void txtLetter2_TextChanged(object sender, EventArgs e)
        {
        
        }

        private void txtLetter3_TextChanged(object sender, EventArgs e)
        {
        
        }
    }
}
