﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace Keybreeze
{
    public partial class frmAutoCompleteFolder : Form
    {
        frmMain frmMain_instance;
        string folder_path;
        public frmAutoCompleteFolder(frmMain parent, string parent_path)
        {
            InitializeComponent();
            frmMain_instance = parent;
            folder_path = parent_path;
        }

        private void frmAutoCompleteFolder_Load(object sender, EventArgs e)
        {
            dataGridView1.ColumnCount = 2;
            dataGridView1.Columns[0].Name = "Keyword";
            dataGridView1.Columns[1].Name = "Target";
            dataGridView1.Columns[0].Width = 250;
            dataGridView1.Columns[1].Width = 800;
            dataGridView1.RowHeadersWidth = 30;
            UpdateGrid(folder_path, "");
        }

        private void UpdateGrid(string the_path, string the_text)
        {
            // display keywords that match the_text as the user types.
            string current_keyword;
            string current_path;
            string current_target;
            string[] row;
            string the_text_lower = the_text.ToLower();
            int the_text_length = the_text.Length;

            dataGridView1.Rows.Clear();

            int loop_count = 0;
            for (int i = 0; i < Globals.keyword_count; i++)
            {
                current_path = Globals.Keyword_List[i].path;
                if (current_path == the_path)
                {
                    current_keyword = Globals.Keyword_List[i].name;
                    current_target = Globals.Keyword_List[i].target;
                    string current_target_reformatted = current_target;
                    if (current_target.IndexOf("function:") == 0)
                    {
                        // it's a text function. Let's preview some of the text.
                        current_target_reformatted = ReturnTextFromTextFunction(current_target, current_path);
                    }
                    if (the_text.Length == 0 || (current_keyword.Length >= the_text_length && current_keyword.ToLower().IndexOf(the_text_lower) != -1))
                    {
                        row = new string[] { current_keyword, current_target_reformatted };
                        dataGridView1.Rows.Add(row);
                        loop_count++;
                    }
                }
            }
        }

        private string ReturnTextFromTextFunction(string text_function_target, string kb_path)
        {
            int description_character_limit = 100;
            int is_text_function = text_function_target.IndexOf("function:");
            if (is_text_function == 0)
            {
                // it's a text function. let's get some of the text.
                string text_function_file = text_function_target.Substring(9);
                text_function_file = Globals.replace_invalid_filename_characters(text_function_file, kb_path);
                string text_file = Globals.WorkPath + "\\text functions\\" + text_function_file + ".txt";
                //string the_text = "";
                string the_text = "";
                try
                {
                    the_text += "text:" + File.ReadAllText(text_file);
                }
                catch
                {
                    the_text += "text:" + text_function_file;
                }
                int the_length = the_text.Length;
                if (the_length > description_character_limit)
                {
                    the_text = the_text.Substring(0, description_character_limit) + "...";
                }
                return the_text;
            }
            return text_function_target;
        }

        private void txtEntry_TextChanged(object sender, EventArgs e)
        {
            UpdateGrid(folder_path, txtEntry.Text);
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void btnLaunch_Click(object sender, EventArgs e)
        {
            Launch_Entry();
        }

        public void Launch_Entry()
        {
            this.Hide();
            int current_position = dataGridView1.CurrentRow.Index;
            string selected_keyword = dataGridView1.Rows[current_position].Cells[0].Value.ToString();
            frmMain_instance.LaunchEntry(selected_keyword);
            this.Close();
        }

        private void launchToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Launch_Entry();
        }

        private void copyTargetToClipboardToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Copy_Target();
        }

        public void Copy_Target()
        {
            int current_position = dataGridView1.CurrentRow.Index;
            string selected_keyword = dataGridView1.Rows[current_position].Cells[0].Value.ToString();
            string selected_target = dataGridView1.Rows[current_position].Cells[1].Value.ToString();
            if (selected_target.IndexOf("text:") == 0)
            {
                // it's a text function. return the actual text of the file.
                foreach (Keyword this_keyword in Globals.Keyword_List)
                {
                    if (this_keyword.name == selected_keyword && this_keyword.path == folder_path)
                    {
                        // we found our text function.
                        string text_function_file = this_keyword.target.Substring(9);
                        text_function_file = Globals.replace_invalid_filename_characters(text_function_file, folder_path);
                        string text_file = Globals.WorkPath + "\\text functions\\" + text_function_file + ".txt";
                        string the_text = "";
                        try
                        {
                            the_text = File.ReadAllText(text_file);
                        }
                        catch
                        {
                            the_text = text_function_file;
                        }
                        try
                        {
                            Clipboard.SetText(the_text);
                        }
                        catch
                        { }
                        break;
                    }
                }
            }
            else
            {
                // it's a regular target. copy it to the clipboard.
                Clipboard.Clear();
                Clipboard.SetText(selected_target);
            }
            this.Close();
        }

        private void openContainingFolderToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Open_Containing_Folder();
        }

        public void Open_Containing_Folder()
        {
            int current_position = dataGridView1.CurrentRow.Index;
            string selected_target = dataGridView1.Rows[current_position].Cells[1].Value.ToString();
            if (Directory.Exists(selected_target))
            {
                string containing_folder = Path.GetDirectoryName(selected_target);
                frmMain_instance.RunFolder(containing_folder);
            }
            else
            {
                MessageBox.Show(selected_target + " is not a directory.", "Keybreeze Desktop");
            }
            this.Close();
        }

        protected override bool ProcessCmdKey(ref Message msg, Keys keyData)
        {
            // this function runs when the user presses Ctrl, Tab, etc.
            if (keyData == (Keys.Control | Keys.M))
            {
                // user pressed Ctrl + M to show the menu.
                contextMenuStrip1.Show(dataGridView1, 200, 75);
                return true;
            }
            return base.ProcessCmdKey(ref msg, keyData);
        }

        private void btnOptions_Click(object sender, EventArgs e)
        {
            contextMenuStrip1.Show(btnOptions, 1, 1);
        }

        private void editToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Edit_Entry();
        }

        private void Edit_Entry()
        {
            int current_position = dataGridView1.CurrentRow.Index;
            string selected_keyword = dataGridView1.Rows[current_position].Cells[0].Value.ToString();
            frmMain_instance.ChangeKeyword(selected_keyword, folder_path);
        }

        private void removeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Remove_Entry();
        }

        private void Remove_Entry()
        {
            int current_position = dataGridView1.CurrentRow.Index;
            string entry_to_remove = dataGridView1.Rows[current_position].Cells[0].Value.ToString();
            string the_target = dataGridView1.Rows[current_position].Cells[1].Value.ToString();
            // the entry is a keyword. see if it's a macro.
            if (the_target == "macro")
            {
                DialogResult result;
                result = MessageBox.Show("Are you sure you want to remove the macro, " + entry_to_remove + ", from the command list?", "Keybreeze Desktop", MessageBoxButtons.YesNo);
                if (result == DialogResult.Yes)
                {
                    frmMain_instance.DeleteKeywordFromCommandList(entry_to_remove, folder_path, true);
                    string macro_file_path = Globals.WorkPath + "\\macros\\" + entry_to_remove + ".kmcr";
                    if (Globals.DoesFileExist(macro_file_path) == true)
                    {
                        DialogResult result2;
                        result2 = MessageBox.Show("Do you want to delete the macro file, " + macro_file_path + ", from your computer as well?", "Keybreeze Desktop", MessageBoxButtons.YesNo);
                        if (result2 == DialogResult.Yes)
                        {
                            // delete the macro file.
                            try
                            {
                                File.Delete(macro_file_path);
                                this.Close();
                            }
                            catch
                            {
                                MessageBox.Show("Keybreeze Desktop could not delete the macro file, possibly due to Windows permissions. You can manually delete this file by browsing to the path, " + macro_file_path, "Keybreeze Desktop");
                            }
                        }
                    }
                    frmMain_instance.HideMainForm();
                }
            }
            else
            {
                DialogResult result;
                result = MessageBox.Show("Are you sure you want to delete the keyword, " + entry_to_remove + "?", "Keybreeze Desktop", MessageBoxButtons.YesNo);
                if (result == DialogResult.Yes)
                {
                    frmMain_instance.DeleteKeywordFromCommandList(entry_to_remove, folder_path, true);
                    this.Close();
                }
            }
        }

        private void txtEntry_KeyDown(object sender, KeyEventArgs e)
        {
            try
            {
                int current_position = dataGridView1.CurrentRow.Index;
                if (e.KeyCode == Keys.Down)
                {
                    if (current_position < dataGridView1.Rows.Count - 1)
                    {
                        dataGridView1.Rows[current_position].Selected = false;
                        dataGridView1.Rows[current_position + 1].Selected = true;
                        dataGridView1.Rows[current_position + 1].Cells[0].Selected = true;
                    }
                    //dataGridView1.Rows[i].Cells[1].Selected = true;
                    e.Handled = true;
                }
                else if (e.KeyCode == Keys.Up)
                {
                    if (current_position > 0)
                    {
                        dataGridView1.Rows[current_position].Selected = false;
                        dataGridView1.Rows[current_position - 1].Selected = true;
                        dataGridView1.Rows[current_position - 1].Cells[0].Selected = true;
                    }
                    e.Handled = true;
                }
            }
            catch { }
        }

        private void dataGridView1_CellMouseDown(object sender, DataGridViewCellMouseEventArgs e)
        {
            try
            {
                if (e.Button == MouseButtons.Right)
                {
                    contextMenuStrip1.Show(Cursor.Position);
                }
            }
            catch { }
        }
    }
}
