﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Keybreeze
{
    public partial class frmCalculator : Form
    {
        double previous_result = 0;

        public frmCalculator()
        {
            InitializeComponent();
            int opacity = Globals.ReadIntFromINI("calculator", "opacity", Globals.SkinINI, 0);
            if (opacity > 0 && opacity < 100)
                Globals.MakeWindowTranslucent(this.Handle, opacity, false);
        }

        private void cmdCalculate_Click(object sender, EventArgs e)
        {
            string calculation_text = txtCalculate.Text;

            // if the first character is an operator, put the previous value before it.
            if (txtCalculate.Text.Length == 0)
                return;
            string first_char = txtCalculate.Text.Substring(0, 1);
            if (first_char == "+" || first_char == "-" || first_char == "*" || first_char == "/" || first_char == "\\" || first_char == "^")
            {
                calculation_text = previous_result + calculation_text;
            }
            Calculate_Expression(calculation_text);
            lblCalculate.Focus();
            lblCalculate.SelectionStart = lblCalculate.TextLength;
            lblCalculate.ScrollToCaret();
            txtCalculate.Focus();
        }

        private void frmCalculator_Load(object sender, EventArgs e)
        {
            string the_backcolor = Globals.ReadINI("calculator", "backcolor", Globals.SkinINI);
            string panel_backcolor = Globals.ReadINI("calculator", "panel_backcolor", Globals.SkinINI);
            string the_forecolor = Globals.ReadINI("calculator", "forecolor", Globals.SkinINI);
            string the_font = Globals.ReadINI("calculator", "font", Globals.SkinINI);
            int the_font_size = Globals.ReadIntFromINI("calculator", "font_size", Globals.SkinINI, 10);
            this.Width = Globals.ReadIntFromINI("calculator", "width", Globals.SkinINI);
            this.Height = Globals.ReadIntFromINI("calculator", "height", Globals.SkinINI);

            this.BackColor = System.Drawing.ColorTranslator.FromHtml(the_backcolor);
            txtCalculate.BackColor = System.Drawing.ColorTranslator.FromHtml(panel_backcolor);
            lblCalculate.BackColor = System.Drawing.ColorTranslator.FromHtml(the_backcolor);
            lblCalculate.ForeColor = System.Drawing.ColorTranslator.FromHtml(the_forecolor);
            lblInfo.ForeColor = System.Drawing.ColorTranslator.FromHtml(the_forecolor);
            txtCalculate.ForeColor = System.Drawing.ColorTranslator.FromHtml(the_forecolor);
            txtCalculate.Font = new System.Drawing.Font(the_font, the_font_size, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            lblCalculate.Font = new System.Drawing.Font(the_font, the_font_size, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
        }

        private void cmdCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void lblClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        public void Calculate_Expression(string TheText)
        {
            try
            {
                // does the expression have a left parentheses but no right parentheses? This is a common mistake, so let's fix it if that's the case.
                if (TheText.Length > 1 && TheText.IndexOf("(") != -1 && TheText.IndexOf(")") == -1)
                {
                    TheText = TheText + ")";
                }
                var result = new DataTable().Compute(TheText, null);
                string calc_result = result.ToString();
                previous_result = Convert.ToDouble(calc_result);
                lblCalculate.Text = lblCalculate.Text + TheText + System.Environment.NewLine + calc_result + System.Environment.NewLine;
                txtCalculate.Text = "";
            }
            catch
            {
                MessageBox.Show("Unable to calculate " + TheText + ".", "Keybreeze Desktop");
            }
        }

        private void frmCalculator_FormClosing(object sender, FormClosingEventArgs e)
        {
            Globals.WriteINI("calculator", "width", this.Width.ToString(), Globals.SkinINI);
            Globals.WriteINI("calculator", "height", this.Height.ToString(), Globals.SkinINI);
        }

        private void txtCalculate_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.F11)
            {
                StayOnTop();
            }
            if (e.Control && e.KeyCode == Keys.C && txtCalculate.SelectionLength == 0)
            {
                // user pressed ctrl + c, and nothing is highlighted. Let's copy the previous result.
                Clipboard.SetText(previous_result.ToString());
                e.SuppressKeyPress = true;
                return;
            }
        }

        private void lblCalculate_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.F11)
            {
                StayOnTop();
            }
        }

        private void StayOnTop()
        {
            if (this.TopMost == false)
                this.TopMost = true;
            else
                this.TopMost = false;
        }

        private void txtCalculate_TextChanged(object sender, EventArgs e)
        {

        }

        private void lblCalculate_TextChanged(object sender, EventArgs e)
        {

        }
    }
}
