﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Keybreeze
{
    public partial class frmCommandList : Form
    {
        int row_selection = 0;
        int cell_selection = 0;
        frmMain frmMain_instance;

        public frmCommandList(frmMain parent)
        {
            InitializeComponent();
            frmMain_instance = parent;
        }

        private void frmCommandList_Load(object sender, EventArgs e)
        {
            dataGridView1.ColumnCount = 3;
            dataGridView1.Columns[0].Name = "Keyword";
            dataGridView1.Columns[1].Name = "Path";
            dataGridView1.Columns[2].Name = "Target";
            dataGridView1.Columns[0].Width = 175;
            dataGridView1.Columns[1].Width = 175;
            dataGridView1.Columns[2].Width = 800;
            dataGridView1.RowHeadersWidth = 30;

            UpdateGrid("");
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void textBox1_TextChanged(object sender, EventArgs e)
        {

        }

        private void btnSearch_Click(object sender, EventArgs e)
        {
            bool found_it = false;
            int current_position = dataGridView1.CurrentRow.Index;

            dataGridView1.Rows[row_selection].Cells[cell_selection].Selected = false;

            found_it = SearchForText(txtSearch.Text, current_position + 1);

            if (found_it == false)
            {
                // does the user want to search from the beginning?
                if (current_position > 0)
                {
                    DialogResult result;
                    result = MessageBox.Show("The text you entered was not found. Do you want to continue searching from the beginning of the list?", "Keybreeze Desktop", MessageBoxButtons.YesNo);
                    if (result == DialogResult.Yes)
                    {
                        found_it = SearchForText(txtSearch.Text, 0);
                        if (found_it == false)
                        {
                            MessageBox.Show("The text you entered was not found.", "Keybreeze Desktop");
                            return;
                        }
                    }
                }
                else
                {
                    MessageBox.Show("The text you entered was not found.", "Keybreeze Desktop");
                    return;
                }
            }
        }

        private bool SearchForText(string the_text, int start_position)
        {
            string current_keyword;
            string current_path;
            string current_target;

            the_text = the_text.ToLower();
            for (int i = start_position; i < Globals.keyword_count; i++)
            {
                current_keyword = dataGridView1.Rows[i].Cells[0].Value.ToString().ToLower();
                current_path = dataGridView1.Rows[i].Cells[1].Value.ToString().ToLower();
                current_target = dataGridView1.Rows[i].Cells[2].Value.ToString().ToLower();
                if (current_keyword.IndexOf(the_text) > -1)
                {
                    // we found the string in the keyword for this row. let's select it.
                    dataGridView1.Rows[i].Selected = true;
                    dataGridView1.Rows[i].Cells[0].Selected = true;
                    cell_selection = 0;
                    row_selection = i;
                    return true;
                }
                else if (current_path.IndexOf(the_text) > -1)
                {
                    // we found the string in the path for this row. let's select it.
                    dataGridView1.Rows[i].Selected = true;
                    dataGridView1.Rows[i].Cells[1].Selected = true;
                    cell_selection = 1;
                    row_selection = i;
                    return true;
                }
                else if (current_target.IndexOf(the_text) > -1)
                {
                    // we found the string in the target for this row. let's select it.
                    dataGridView1.Rows[i].Selected = true;
                    dataGridView1.Rows[i].Cells[2].Selected = true;
                    cell_selection = 2;
                    row_selection = i;
                    return true;
                }
            }
            return false;
        }

        private void btnDelete_Click(object sender, EventArgs e)
        {
            int current_position = dataGridView1.CurrentRow.Index;
            string selected_keyword = dataGridView1.Rows[current_position].Cells[0].Value.ToString();
            string selected_path = dataGridView1.Rows[current_position].Cells[1].Value.ToString();

            DialogResult result;
            string message_text = "Are you sure you want to delete the keyword, " + selected_keyword + "?";
            if (selected_path.Length > 0)
            {
                message_text += Environment.NewLine + "Path: " + selected_path;
            }
            result = MessageBox.Show(message_text, "Keybreeze Desktop", MessageBoxButtons.YesNo);
            if (result == DialogResult.Yes)
            {
                Cursor.Current = Cursors.WaitCursor;
                this.Enabled = false;
                frmMain_instance.Enabled = false;
                frmMain_instance.DeleteKeywordFromCommandList(selected_keyword, selected_path, true);

                // remove the item in the list.
                dataGridView1.Rows.RemoveAt(current_position);
                this.Enabled = true;
                frmMain_instance.Enabled = true;
                Cursor.Current = Cursors.Default;
            }
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            frmMain_instance.ShowAddKeywordForm(true);
        }

        public void UpdateGrid(string new_keyword)
        {
            string current_keyword;
            string current_path;
            string current_target;
            int select_this_row = 0;
            string[] row;

            dataGridView1.Rows.Clear();

            int loop_count = 0;
            for (int i = 0; i < Globals.keyword_count; i++)
            {
                if (Globals.keyword_name_lower[i] == new_keyword.ToLower())
                {
                    select_this_row = loop_count;
                }
                current_keyword = Globals.Keyword_List[i].name;
                current_path = Globals.Keyword_List[i].path;
                current_target = Globals.Keyword_List[i].target;
                row = new string[] { current_keyword, current_path, current_target };
                dataGridView1.Rows.Add(row);
                loop_count++;
            }
            if (select_this_row > 0)
                dataGridView1.Rows[select_this_row].Cells[0].Selected = true;
        }

        private void btnEdit_Click(object sender, EventArgs e)
        {
            int current_position = dataGridView1.CurrentRow.Index;
            string selected_keyword = dataGridView1.Rows[current_position].Cells[0].Value.ToString();
            string selected_path = dataGridView1.Rows[current_position].Cells[1].Value.ToString();

            frmMain_instance.ChangeKeyword(selected_keyword, selected_path, true);
        }

        private void btnMoveDown_Click(object sender, EventArgs e)
        {
            int current_position = dataGridView1.CurrentRow.Index;
            if (current_position < dataGridView1.RowCount - 1)
            {
                // move the item down.
                btnMoveDown.Enabled = false;
                Globals.BackupCommandList();
                Keyword keyword_to_move = new Keyword();
                keyword_to_move = Globals.Keyword_List[current_position];
                Globals.Keyword_List.RemoveAt(current_position);
                Globals.Keyword_List.Insert(current_position + 1, keyword_to_move);
                int save_response = Globals.Keywords_Save_To_File();
                if (save_response == -1)
                {
                    MessageBox.Show("Keybreeze Desktop was unable to open the file, " + Globals.KeywordsJSONFile + ", for writing. Please make sure that you have read and write permission in Windows for this directory.", "Keybreeze Desktop");
                }
                else
                {
                    UpdateGrid(keyword_to_move.name);
                }
                btnMoveDown.Enabled = true;
            }
        }

        private void editAutoindex_Click(object sender, EventArgs e)
        {
            frmMain_instance.ShowOptionsWindow(true);
        }

        private void button2_Click(object sender, EventArgs e)
        {
            frmMain_instance.RunURL("http://www.keybreeze.com/tutorial/4");
        }

        private void btnMoveUp_Click(object sender, EventArgs e)
        {
            int current_position = dataGridView1.CurrentRow.Index;
            if (current_position > 0)
            {
                // move the item up.
                btnMoveUp.Enabled = false;
                Globals.BackupCommandList();
                Keyword keyword_to_move = new Keyword();
                keyword_to_move = Globals.Keyword_List[current_position];
                Globals.Keyword_List.RemoveAt(current_position);
                Globals.Keyword_List.Insert(current_position - 1, keyword_to_move);
                int save_response = Globals.Keywords_Save_To_File();
                if (save_response == -1)
                {
                    MessageBox.Show("Keybreeze Desktop was unable to open the file, " + Globals.KeywordsJSONFile + ", for writing. Please make sure that you have read and write permission in Windows for this directory.", "Keybreeze Desktop");
                }
                else
                {
                    UpdateGrid(keyword_to_move.name);
                }
                btnMoveUp.Enabled = true;
            }
        }

        private void label1_Click(object sender, EventArgs e)
        {
            
        }
    }
}
