﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace Keybreeze
{
    public partial class frmKeybreezeClipboard : Form
    {
        public string warning_text = "";
        frmMain frmMain_instance;
        public List<HiddenPotentialPasswords> hidden_potential_passwords = new List<HiddenPotentialPasswords>();

        public frmKeybreezeClipboard(frmMain parent)
        {
            frmMain_instance = parent;
            InitializeComponent();
        }

        private void frmKeybreezeClipboard_Load(object sender, EventArgs e)
        {
            warning_text = "[Warning: text is trimmed at " + frmMain_instance.keybreeze_clipboard_item_max_length + " characters.] ";
            // before we load everything, let's run the frmMain function one more time to refresh the clipboard.
            frmMain_instance.RefreshKeybreezeClipboard();
            int tab_to_toggle = Globals.ReadIntFromINI("clipboard", "tab_to_toggle", Globals.OptionsINI, 0);
            if (tab_to_toggle == 1)
            {
                useTabKeyToToggleRowsToolStripMenuItem.Checked = true;
            }
            dataGridView1.ColumnCount = 2;
            dataGridView1.Columns[0].Name = "Date/time";
            dataGridView1.Columns[1].Name = "Clipboard text";
            dataGridView1.Columns[0].Width = 200;
            dataGridView1.Columns[1].Width = 1000;
            dataGridView1.RowHeadersWidth = 30;
            UpdateGrid("");
        }

        private void clearAllToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // Delete all entries.
            frmMain_instance.keybreeze_clipboard.Clear();
            this.Close();
        }

        private void btnOptions_Click(object sender, EventArgs e)
        {
            ChangeMenu();
            contextMenuStrip1.Show(btnOptions, 1, 1);
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void UpdateGrid(string the_text = "")
        {
            // clear any existing items.
            dataGridView1.Rows.Clear();
            hidden_potential_passwords.Clear();

            // display the clipboard items matching the text that the user typed.
            string the_text_lower = the_text.ToLower();
            string[] row;
            for (int i = frmMain_instance.keybreeze_clipboard.Count - 1; i >= 0; i--)
            {
                Keybreeze_Clipboard this_item = frmMain_instance.keybreeze_clipboard[i];
                int index_position = i;
                if (the_text.Length == 0 || (this_item.clipboard_text.ToLower().IndexOf(the_text_lower) != -1))
                {
                    string text_for_datagridview = this_item.clipboard_text;
                    string local_warning_text = "";
                    if (text_for_datagridview.Length >= frmMain_instance.keybreeze_clipboard_item_max_length)
                    {
                        local_warning_text = warning_text;
                    }
                    if (text_for_datagridview.Length > 2000)
                    {
                        text_for_datagridview = text_for_datagridview.Substring(0, 2000);
                    }
                    if (frmMain_instance.clipboard_password_detection > 0 && text_for_datagridview.Length >= frmMain_instance.clipboard_password_detection_start && text_for_datagridview.Length <= frmMain_instance.clipboard_password_detection_end)
                    {
                        // the user configured Keybreeze to hide potential passwords, and text is within our range. Do we require certain characters, such as letters and numbers to be present?
                        if (frmMain_instance.clipboard_password_detection == 1)
                        {
                            // no other requirements. Hide this entry.
                            text_for_datagridview = HidePotentialPassword(text_for_datagridview, index_position);
                        }
                        else if (frmMain_instance.clipboard_password_detection == 2)
                        {
                            // does this contain letters and numbers?
                            bool found_letter = false;
                            bool found_number = false;
                            foreach (byte b in System.Text.Encoding.UTF8.GetBytes(text_for_datagridview.ToCharArray()))
                            {
                                if (found_letter == false && ((b >= 65 && b <= 90) || (b >= 97 && b <= 122)))
                                {
                                    found_letter = true;
                                }
                                if (found_number == false && b >= 48 && b <= 57)
                                {
                                    found_number = true;
                                }
                                if (found_letter == true && found_number == true)
                                {
                                    text_for_datagridview = HidePotentialPassword(text_for_datagridview, index_position);
                                    break;
                                }
                            }
                        }
                        else if (frmMain_instance.clipboard_password_detection == 3)
                        {
                            // does this contain letters, numbers, and no spaces?
                            bool found_letter = false;
                            bool found_number = false;
                            bool found_spaces = false;
                            foreach (byte b in System.Text.Encoding.UTF8.GetBytes(text_for_datagridview.ToCharArray()))
                            {
                                if (found_letter == false && ((b >= 65 && b <= 90) || (b >= 97 && b <= 122)))
                                {
                                    found_letter = true;
                                }
                                if (found_number == false && b >= 48 && b <= 57)
                                {
                                    found_number = true;
                                }
                                if (found_spaces == false && b == 32)
                                {
                                    found_spaces = true;
                                    break;
                                }
                            }
                            if (found_letter == true && found_number == true && found_spaces == false)
                            {
                                text_for_datagridview = HidePotentialPassword(text_for_datagridview, index_position);
                            }
                        }
                    }
                    text_for_datagridview = local_warning_text + text_for_datagridview;
                    row = new string[] { this_item.timestamp.ToString(), text_for_datagridview };
                    dataGridView1.Rows.Add(row);
                    dataGridView1.Rows[dataGridView1.RowCount - 1].Tag = index_position;
                }
            }
        }

        private string HidePotentialPassword(string the_text, int row_index)
        {
            string new_text = "";
            for (int i = 0; i < the_text.Length; i++)
            {
                if (i == 0)
                {
                    new_text += the_text.Substring(i, 1);
                }
                else
                {
                    new_text += "*";
                }
            }
            new_text += " [Potential password hidden from this window only]";
            HiddenPotentialPasswords new_potential_password = new HiddenPotentialPasswords();
            new_potential_password.row_index = row_index;
            new_potential_password.original_string = the_text;
            new_potential_password.converted_string = new_text;
            hidden_potential_passwords.Add(new_potential_password);
            showPotentialPasswordToolStripMenuItem.Visible = true;
            return new_text;
        }

        private void txtEntry_TextChanged(object sender, EventArgs e)
        {
            UpdateGrid(txtEntry.Text);
        }

        protected override bool ProcessCmdKey(ref Message msg, Keys keyData)
        {
            // this function runs when the user presses Ctrl, Tab, etc.
            if (keyData == (Keys.Control | Keys.M))
            {
                // user pressed Ctrl + M to show the menu.
                ChangeMenu();
                contextMenuStrip1.Show(dataGridView1, 200, 75);
                return true;
            }
            else if (keyData == (Keys.Control | Keys.C))
            {
                // user pressed Ctrl + C to copy.
                CopyEntryToClipboard();
                return true;
            }
            else if (keyData == (Keys.Control | Keys.A))
            {
                // user pressed Ctrl + A
                AddTextFunction();
                return true;
            }
            else if (keyData == (Keys.Control | Keys.S))
            {
                // user pressed Ctrl + S
                ShowOrHidePotentialPasswordInGrid();
                return true;
            }
            else if (keyData == (Keys.Control | Keys.C))
            {
                // user pressed Ctrl + C to copy.
                CopyEntryToClipboard();
                return true;
            }
            else if (keyData == Keys.Tab && useTabKeyToToggleRowsToolStripMenuItem.Checked == true)
            {
                // user pressed Tab, and the option is checked to toggle through our entries.
                int current_position = dataGridView1.CurrentRow.Index;
                int total_rows = dataGridView1.RowCount;
                if (current_position >= 0 && current_position < total_rows - 1)
                {
                    // move up
                    dataGridView1.Rows[current_position + 1].Cells[0].Selected = true;
                }
                else if (current_position == total_rows - 1)
                {
                    // we are at our last row, and the user pressed tab again. Select the first one.
                    dataGridView1.Rows[0].Cells[0].Selected = true;
                }
                return true;
            }
            return base.ProcessCmdKey(ref msg, keyData);
        }

        private void txtEntry_KeyDown(object sender, KeyEventArgs e)
        {
            try
            {
                int current_position = dataGridView1.CurrentRow.Index;
                if (e.KeyCode == Keys.Down)
                {
                    if (current_position < dataGridView1.Rows.Count - 1)
                    {
                        dataGridView1.Rows[current_position].Selected = false;
                        dataGridView1.Rows[current_position + 1].Selected = true;
                        dataGridView1.Rows[current_position + 1].Cells[0].Selected = true;
                    }
                    //dataGridView1.Rows[i].Cells[1].Selected = true;
                    e.Handled = true;
                }
                else if (e.KeyCode == Keys.Up)
                {
                    if (current_position > 0)
                    {
                        dataGridView1.Rows[current_position].Selected = false;
                        dataGridView1.Rows[current_position - 1].Selected = true;
                        dataGridView1.Rows[current_position - 1].Cells[0].Selected = true;
                    }
                    e.Handled = true;
                }
            }
            catch { }
        }

        private void dataGridView1_CellMouseDown(object sender, DataGridViewCellMouseEventArgs e)
        {
            try
            {
                if (e.Button == MouseButtons.Right)
                {
                    ChangeMenu();
                    contextMenuStrip1.Show(Cursor.Position);
                }
            }
            catch { }
        }

        private void CopyEntryToClipboard()
        {
            try
            {
                int current_position1 = dataGridView1.CurrentRow.Index;
            }
            catch (Exception e)
            {
                MessageBox.Show("Could not get text from DataGridView. " + e.Message, "Keybreeze Desktop");
                return;
            }
            this.Hide();
            int current_position = dataGridView1.CurrentRow.Index;
            int position_in_clipboard_history = Int32.Parse(dataGridView1.Rows[current_position].Tag.ToString());
            string text_to_copy = frmMain_instance.keybreeze_clipboard[position_in_clipboard_history].clipboard_text;
            // copy to clipboard.
            Clipboard.Clear();
            Clipboard.SetText(text_to_copy);
            this.Close();
        }

        private void copyToClipboardToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CopyEntryToClipboard();
        }

        private void btnInsert_Click(object sender, EventArgs e)
        {
            InsertEntryIntoPreviousWindow();
        }

        private void insertIntoPreviousWindowToolStripMenuItem_Click(object sender, EventArgs e)
        {
            InsertEntryIntoPreviousWindow();
        }

        private void InsertEntryIntoPreviousWindow()
        {
            // store the current clipboard text into a variable first.
            // then load the text function from the text file into a variable.
            string the_text = "";
            try
            {
                int current_position = dataGridView1.CurrentRow.Index;
                int position_in_clipboard_history = Int32.Parse(dataGridView1.Rows[current_position].Tag.ToString());
                the_text = frmMain_instance.keybreeze_clipboard[position_in_clipboard_history].clipboard_text;
            }
            catch (Exception e)
            {
                MessageBox.Show("Could not get text from DataGridView. " + e.Message, "Keybreeze Desktop");
                return;
            }
            frmMain_instance.running_text_function = true;
            string old_clipboard_data = Clipboard.GetText();
            string filename_valid_characters;

            //HideMainForm();
            this.Hide();

            // sometimes setting the clipboard text will not work because it's a shared resource. see http://stackoverflow.com/questions/68666/clipbrd-e-cant-open-error-when-setting-the-clipboard-from-net.
            // we are going to try multiple times to work around this issue.
            bool copy_success = false;
            string copy_error = "";
            for (int i = 0; i < 10; i++)
            {
                try
                {
                    Clipboard.SetText(the_text);
                    copy_success = true;
                    break;
                }
                catch (Exception e2)
                {
                    copy_error = e2.Message;
                }
                Application.DoEvents();
                Thread.Sleep(100);
            }
            if (copy_success == false)
            {
                MessageBox.Show("Keybreeze Desktop was unable to copy the text function to the clipboard and paste it in your application. " + copy_error, "Keybreeze Desktop");
                frmMain_instance.running_text_function = false;
                return;
            }

            /*
            try
            {
                Clipboard.SetText(the_text);
                if (Clipboard.GetText() == old_clipboard_data && old_clipboard_data != the_text)
                {
                    // setting the clipboard text didn't work. sleep for a bit and then try again.
                    // loops were causing the app to crash, so we are trying multiple try...catch blocks.
                    Clipboard.Clear();
                    Application.DoEvents();
                    System.Threading.Thread.Sleep(250);
                    try
                    {
                        Clipboard.SetText(the_text);
                        if (Clipboard.GetText() == old_clipboard_data && old_clipboard_data != the_text)
                        {
                            // setting the clipboard text didn't work. sleep for a bit and then try again.
                            // loops were causing the app to crash, so we are trying multiple try...catch blocks.
                            Clipboard.Clear();
                            Application.DoEvents();
                            System.Threading.Thread.Sleep(250);
                            try
                            {
                                Clipboard.SetText(the_text);
                            }
                            catch
                            { }
                        }
                    }
                    catch
                    { }
                }
            }
            catch
            { }

            try
            {
                if (Clipboard.GetText() == old_clipboard_data && old_clipboard_data != the_text)
                {
                    MessageBox.Show("Keybreeze was unable to copy the text function to the clipboard and paste it in your application. This has been a known issue with the VMware Remote MKS Plugin. If you have this plugin enabled in your browser, please try closing the tab where this plugin is running, or disable the plugin, and try running this text function again.", "Keybreeze Desktop");
                    return 0;
                }
            }
            catch
            {
                MessageBox.Show("Keybreeze was unable to copy the text function to the clipboard because another application has it locked. Please try again in a few seconds.");
                return 0;
            }
             */

            int sleep_time;
            sleep_time = Globals.ReadIntFromINI("options", "wait_time_before_inserting_text", Globals.OptionsINI, 450);
            int modifier_time = Globals.ReadIntFromINI("options", "wait_time_for_modifiers", Globals.OptionsINI, 50);
            System.Threading.Thread.Sleep(sleep_time);

            // simulate CTRL + V to paste the text
            frmMain.keybd_event(17, 0, 0, UIntPtr.Zero);
            System.Threading.Thread.Sleep(modifier_time);
            frmMain.keybd_event(86, 0, 0, UIntPtr.Zero);
            frmMain.keybd_event(86, 0, 0x2, UIntPtr.Zero);
            frmMain.keybd_event(17, 0, 0x2, UIntPtr.Zero);

            // set the clipboard data back to what it was previously
            try
            {
                System.Threading.Thread.Sleep(sleep_time);
                Clipboard.SetText(old_clipboard_data);
            }
            catch
            {
                // do nothing
            }
            frmMain_instance.running_text_function = false;
        }

        private void removeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // Remove the selected item.
            try
            {
                int current_position = dataGridView1.CurrentRow.Index;
                int position_in_clipboard_history = Int32.Parse(dataGridView1.Rows[current_position].Tag.ToString());
                // remove this item.
                frmMain_instance.keybreeze_clipboard.RemoveAt(position_in_clipboard_history);
                UpdateGrid();
            }
            catch (Exception e2)
            {
                MessageBox.Show("Could not get text from DataGridView. " + e2.Message, "Keybreeze Desktop");
                return;
            }
        }

        private void ShowOrHidePotentialPasswordInGrid()
        {
            bool found_entry = false;
            try
            {
                int current_position = dataGridView1.CurrentRow.Index;
                int position_in_clipboard_history = Int32.Parse(dataGridView1.Rows[current_position].Tag.ToString());
                // get this value from our class
                foreach (HiddenPotentialPasswords this_password in hidden_potential_passwords)
                {
                    if (this_password.row_index == position_in_clipboard_history)
                    {
                        // we found our password. Show it or hide it.
                        if (this_password.showing_password == false)
                        {
                            // show it.
                            dataGridView1.Rows[current_position].Cells[1].Value = this_password.original_string;
                            showPotentialPasswordToolStripMenuItem.Text = "Hide potential password [Ctrl + S]";
                            this_password.showing_password = true;
                        }
                        else
                        {
                            // hide it.
                            dataGridView1.Rows[current_position].Cells[1].Value = this_password.converted_string;
                            showPotentialPasswordToolStripMenuItem.Text = "Show potential password [Ctrl + S]";
                            this_password.showing_password = false;
                        }
                        found_entry = true;
                        break;
                    }
                }
            }
            catch (Exception e2)
            {

            }
            if (found_entry == false)
            {
                MessageBox.Show("Please right-click the value that has the potential password and then select this menu option again.", "Keybreeze Desktop");
            }
        }

        private void btnAddTextFunction_Click(object sender, EventArgs e)
        {
            AddTextFunction();
        }

        private void addTextFunctionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AddTextFunction();
        }

        private void AddTextFunction()
        {
            try
            {
                int current_position = dataGridView1.CurrentRow.Index;
                int position_in_clipboard_history = Int32.Parse(dataGridView1.Rows[current_position].Tag.ToString());
                string the_text = frmMain_instance.keybreeze_clipboard[position_in_clipboard_history].clipboard_text;
                frmAdd frmAdd_instance;
                frmAdd_instance = new frmAdd(frmMain_instance);
                frmAdd_instance.changing_entry = 3;
                frmAdd_instance.txtTextFunction.Text = the_text;
                frmAdd_instance.Show();
            }
            catch (Exception e2)
            {
                MessageBox.Show("Could not get text from DataGridView. " + e2.Message, "Keybreeze Desktop");
                return;
            }
            this.Close();
        }

        private void btnCopy_Click(object sender, EventArgs e)
        {
            CopyEntryToClipboard();
        }

        private void useTabKeyToToggleRowsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (useTabKeyToToggleRowsToolStripMenuItem.Checked == false)
            {
                useTabKeyToToggleRowsToolStripMenuItem.Checked = true;
                Globals.WriteINI("clipboard", "tab_to_toggle", "1", Globals.OptionsINI);
            }
            else
            {
                useTabKeyToToggleRowsToolStripMenuItem.Checked = false;
                Globals.WriteINI("clipboard", "tab_to_toggle", "0", Globals.OptionsINI);
            }
        }

        private void dataGridView1_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                InsertEntryIntoPreviousWindow();
                e.Handled = true;
            }
        }

        private void moreOptionsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmOptions frmOptions_instance = new frmOptions(frmMain_instance);
            frmOptions_instance.start_with_clipboard_tab = true;
            frmOptions_instance.Show();
        }

        private void dataGridView1_CellMouseDoubleClick(object sender, DataGridViewCellMouseEventArgs e)
        {
            ChangeMenu();
            contextMenuStrip1.Show(Cursor.Position);
        }

        private void showPotentialPasswordToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ShowOrHidePotentialPasswordInGrid();
        }

        private void ChangeMenu()
        {
            // this function will see if the selected record in the datagridview is a hidden password or one that is showing, and it will adjust the menu in either case.
            int current_position = dataGridView1.CurrentRow.Index;
            int position_in_clipboard_history = Int32.Parse(dataGridView1.Rows[current_position].Tag.ToString());
            bool found_it = false;
            foreach (HiddenPotentialPasswords this_password in hidden_potential_passwords)
            {
                if (this_password.row_index == position_in_clipboard_history && this_password.showing_password == false)
                {
                    showPotentialPasswordToolStripMenuItem.Visible = true;
                    showPotentialPasswordToolStripMenuItem.Text = "Show potential password [Ctrl + S]";
                    found_it = true;
                    break;
                }
                else if (this_password.row_index == position_in_clipboard_history && this_password.showing_password == true)
                {
                    showPotentialPasswordToolStripMenuItem.Visible = true;
                    showPotentialPasswordToolStripMenuItem.Text = "Hide potential password [Ctrl + S]";
                    found_it = true;
                    break;
                }
            }
            if (found_it == false)
            {
                showPotentialPasswordToolStripMenuItem.Visible = false;
            }
        }
    }

    public class HiddenPotentialPasswords
    {
        public int row_index = 0;
        public string original_string = "";
        public string converted_string = "";
        public bool showing_password = false;
    }
}
