﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Runtime.InteropServices;
using System.Diagnostics;

namespace Keybreeze
{
    public partial class frmMacroPlayback : Form
    {
        // keyboard API calls for macros:
        [DllImport("user32.dll")]
        static extern void keybd_event(byte bVk, byte bScan, uint dwFlags, UIntPtr dwExtraInfo);
        const int keybd_keyup = 0x2;
        const int keybd_keydown = 0x0;

        // mouse API calls for macros:
        [DllImport("user32.dll")]
        static extern bool SetCursorPos(int X, int Y);
        [DllImport("user32.dll")]
        static extern void mouse_event(uint dwFlags, int dx, int dy, int dwData, UIntPtr dwExtraInfo);
        const uint MOUSEEVENT_LEFTDOWN = 0x00000002;
        const uint MOUSEEVENT_LEFTUP = 0x00000004;
        const uint MOUSEEVENT_MIDDLEDOWN = 0x00000020;
        const uint MOUSEEVENT_MIDDLEUP = 0x00000040;
        const uint MOUSEEVENT_MOVE = 0x00000001;
        const uint MOUSEEVENT_ABSOLUTE = 0x00008000;
        const uint MOUSEEVENT_RIGHTDOWN = 0x00000008;
        const uint MOUSEEVENT_RIGHTUP = 0x00000010;
        const uint MOUSEEVENT_WHEEL = 0x00000800;
        const uint MOUSEEVENT_XDOWN = 0x00000080;
        const uint MOUSEEVENT_XUP = 0x00000100;

        const int GWL_EXSTYLE = -20;
        const int WS_EX_LAYERED = 0x80000;
        const int LWA_ALPHA = 0x2;
        const int LWA_COLORKEY = 0x1;
        const int HWND_TOPMOST = -1;
        const int HWND_NOTOPMOST = -2;
        const int HWND_BOTTOM = 1;
        const int SWP_NOACTIVATE = 0x0010;
        const int SWP_NOMOVE = 0x0002;
        const int SWP_NOSIZE = 0x0001;

        int wait_time_for_modifiers = Globals.ReadIntFromINI("macros", "wait_time_for_modifiers", Globals.OptionsINI, 50);
        int wait_time_between_keys = Globals.ReadIntFromINI("macros", "wait_time_between_keys", Globals.OptionsINI, 20);
        frmMain frmMain_instance;

        public string macro_file_name;
        public string macro_text;

        public frmMacroPlayback(frmMain parent)
        {
            InitializeComponent();
            frmMain_instance = parent;
        }

        private void frmMacroPlayback_Load(object sender, EventArgs e)
        {
            
        }

        private void frmMacroPlayback_Shown(object sender, EventArgs e)
        {
            // place form at bottom right corner.
            string hotkey = Globals.ReadINI("hotkey", "hotkey", Globals.OptionsINI);
            label1.Text = "Keybreeze Desktop macro is running. Click box or press " + hotkey + " to stop.";
            label1.Visible = true;
            this.Width = label1.Width + 20;
            Rectangle workingArea = Screen.GetWorkingArea(this);
            this.Location = new Point(workingArea.Right - Size.Width, workingArea.Bottom - Size.Height);
            this.Refresh();
            if (Globals.DoesFileExist(Globals.WorkPath + "\\macros\\" + macro_file_name) == true)
            {
                macro_text = File.ReadAllText(Globals.WorkPath + "\\macros\\" + macro_file_name);
                RunMacro();
            }
            else
            {
                // file doesn't exist. See if it's a legacy .mcr file.
                string new_macro_file_name = macro_file_name.Replace(".kmcr", ".mcr");
                if (Globals.DoesFileExist(Globals.WorkPath + "\\macros\\" + new_macro_file_name) == true)
                    MessageBox.Show("The macro, " + new_macro_file_name + ", was written in an old version of Keybreeze Desktop and is no longer supported. You can quickly rewrite it though with the Macro Editor. Press the hotkey and type 'macros [create]' to launch it.", "Keybreeze Desktop");
                else
                    MessageBox.Show("The macro file, " + Globals.WorkPath + "\\macros\\" + macro_file_name + ", does not exist.", "Keybreeze Desktop");
            }
            this.Close();
        }

        private void RunMacro()
        {
            int cursor_position = 0;
            int find_end_point = 0;
            string current_command_text = "";
            string single_character;

            frmMain_instance.macro_running = true;

            // process escape characters first.
            macro_text = macro_text.Replace("\\<", "breezeleftarrow");
            macro_text = macro_text.Replace("\\>", "breezerightarrow");
            macro_text = macro_text.Replace("\\" + '"', "breezequotes");

            do
            {
                if (frmMain_instance.macro_running == false)
                    break;
                try
                { single_character = macro_text.Substring(cursor_position, 1); }
                catch
                { return; }
                if (single_character == "<")
                {
                    // we are starting a new command.
                    find_end_point = macro_text.IndexOf(">", cursor_position);
                    if (find_end_point == -1)
                        return;
                    else
                    {
                        current_command_text = macro_text.Substring(cursor_position + 1, find_end_point - cursor_position - 1);
                        RunMacroCommand(current_command_text);
                        cursor_position = find_end_point + 1;
                    }
                }
                else
                {
                    cursor_position++;
                }
            } while (cursor_position < macro_text.Length);
        }

        private void RunMacroCommand(string current_command_text)
        {
            // the command will be before the space. the first parameter will be after the space (and before the equal sign).
            int find_space = current_command_text.IndexOf(" ");
            int find_equal_sign = current_command_text.IndexOf("=");
            if (find_space == -1)
                return;
            string the_command = current_command_text.Substring(0, find_space).ToLower();
            string the_parameters = current_command_text.Substring(find_space + 1);
            string parameter_name;
            string parameter_value;
            List<string> parameters = new List<string>();

            Application.DoEvents();
            switch (the_command)
            {
                case "comment":
                    // if it's a comment, do nothing.
                    break;
                case "global":
                    parameters = GetNextParameter(the_parameters);
                    foreach (string parameter in parameters)
                    {
                        parameter_name = parameter.Substring(0, parameter.IndexOf("="));
                        parameter_value = parameter.Substring(parameter.IndexOf("=") + 1);
                        if (parameter_name.ToLower() == "keydelay")
                        {
                            wait_time_between_keys = Convert.ToInt32(parameter_value);
                        }
                        else if (parameter_name.ToLower() == "modifierdelay")
                        {
                            wait_time_for_modifiers = Convert.ToInt32(parameter_value);
                        }
                    }
                    break;
                case "keydown":
                    // this should only have one parameter (the key that was pressed).
                    parameters = GetNextParameter(the_parameters);
                    foreach (string parameter in parameters)
                    {
                        parameter_name = parameter.Substring(0, parameter.IndexOf("="));
                        parameter_value = parameter.Substring(parameter.IndexOf("=") + 1);
                        if (parameter_name.ToLower() == "key")
                        {
                            Press_Key(parameter_value.ToLower(), "keydown");
                        }
                    }
                    break;
                case "keyup":
                    // this should only have one parameter (the key that was pressed).
                    parameters = GetNextParameter(the_parameters);
                    foreach (string parameter in parameters)
                    {
                        parameter_name = parameter.Substring(0, parameter.IndexOf("="));
                        parameter_value = parameter.Substring(parameter.IndexOf("=") + 1);
                        if (parameter_name.ToLower() == "key")
                        {
                            Press_Key(parameter_value.ToLower(), "keyup");
                        }
                    }
                    break;
                case "keypress":
                    // this should only have one parameter (the key that was pressed).
                    parameters = GetNextParameter(the_parameters);
                    foreach (string parameter in parameters)
                    {
                        parameter_name = parameter.Substring(0, parameter.IndexOf("="));
                        parameter_value = parameter.Substring(parameter.IndexOf("=") + 1);
                        if (parameter_name.ToLower() == "key")
                        {
                            Press_Key(parameter_value.ToLower());
                        }
                    }
                    break;
                case "mouseclick":
                case "mousedown":
                case "mouseup":
                    // this should have two parameters (the button and the location).
                    string button = "";
                    string location = "";
                    parameters = GetNextParameter(the_parameters);
                    foreach (string parameter in parameters)
                    {
                        parameter_name = parameter.Substring(0, parameter.IndexOf("="));
                        parameter_value = parameter.Substring(parameter.IndexOf("=") + 1);
                        if (parameter_name.ToLower() == "button")
                        {
                            button = parameter_value.ToLower();
                        }
                        else if (parameter_name.ToLower() == "location")
                        {
                            location = parameter_value;
                        }
                    }
                    if (button.Length == 0 || location.Length == 0)
                        return;
                    if (location.IndexOf(",") == -1)
                        return;
                    string[] location_part = location.Split(',');
                    int left_position = Convert.ToInt32(location_part[0]);
                    int top_position = Convert.ToInt32(location_part[1]);
                    if (button == "left")
                    {
                        // alternate method for positioning the mouse. should not be necessary.
                        //Cursor.Position = new System.Drawing.Point(left_position, top_position);
                        SetCursorPos(left_position, top_position);
                        if (the_command == "mouseclick")
                        {
                            mouse_event(MOUSEEVENT_LEFTDOWN, left_position, top_position, 0, UIntPtr.Zero);
                            mouse_event(MOUSEEVENT_LEFTUP, left_position, top_position, 0, UIntPtr.Zero);
                        }
                        else if (the_command == "mousedown")
                        {
                            mouse_event(MOUSEEVENT_LEFTDOWN, left_position, top_position, 0, UIntPtr.Zero);
                        }
                        else if (the_command == "mouseup")
                        {
                            mouse_event(MOUSEEVENT_LEFTUP, left_position, top_position, 0, UIntPtr.Zero);
                        }
                    }
                    else if (button == "right")
                    {
                        if (the_command == "mouseclick")
                        {
                            mouse_event(MOUSEEVENT_RIGHTDOWN, left_position, top_position, 0, UIntPtr.Zero);
                            mouse_event(MOUSEEVENT_RIGHTUP, left_position, top_position, 0, UIntPtr.Zero);
                        }
                        else if (the_command == "mousedown")
                        {
                            mouse_event(MOUSEEVENT_RIGHTDOWN, left_position, top_position, 0, UIntPtr.Zero);
                        }
                        else if (the_command == "mouseup")
                        {
                            mouse_event(MOUSEEVENT_RIGHTUP, left_position, top_position, 0, UIntPtr.Zero);
                        }
                    }
                    break;
                case "delay":
                    // this should only have one parameter (the time).
                    parameters = GetNextParameter(the_parameters);
                    foreach (string parameter in parameters)
                    {
                        parameter_name = parameter.Substring(0, parameter.IndexOf("="));
                        parameter_value = parameter.Substring(parameter.IndexOf("=") + 1);
                        if (parameter_name.ToLower() == "time")
                        {
                            int delay_time = Convert.ToInt32(parameter_value);
                            Macro_Delay(delay_time);
                        }
                    }
                    break;
                case "type":
                    parameters = GetNextParameter(the_parameters);
                    foreach (string parameter in parameters)
                    {
                        parameter_name = parameter.Substring(0, parameter.IndexOf("="));
                        parameter_value = parameter.Substring(parameter.IndexOf("=") + 1);
                        if (parameter_name.ToLower() == "text")
                        {
                            Type_Characters(parameter_value);
                        }
                    }
                    break;
                case "inputbox":
                    string question = "";
                    string variable_name = "";
                    parameters = GetNextParameter(the_parameters);
                    foreach (string parameter in parameters)
                    {
                        parameter_name = parameter.Substring(0, parameter.IndexOf("="));
                        parameter_value = parameter.Substring(parameter.IndexOf("=") + 1);
                        if (parameter_name.ToLower() == "question")
                        {
                            question = parameter_value;
                        }
                        else if (parameter_name.ToLower() == "variable")
                        {
                            variable_name = parameter_value;
                        }
                    }
                    if (question.Length > 0 && variable_name.Length > 0)
                    {
                        frmInputBoxDialog frmInputBoxDialog_instance;
                        frmInputBoxDialog_instance = new frmInputBoxDialog();
                        frmInputBoxDialog_instance.ShowDialog();
                        string user_response = frmInputBoxDialog_instance.the_text;
                        if (user_response == "")
                        {
                            // the user didn't type anything or they hit Cancel. Stop the macro.
                            frmMain_instance.macro_running = false;
                            break;
                        }
                        else
                        {
                            // replace all instances of the variable name with whatever the user typed.
                            macro_text = macro_text.Replace(variable_name, user_response);
                        }
                    }
                    break;
                case "keybreeze":
                    string keybreeze_entry_text = "";
                    parameters = GetNextParameter(the_parameters);
                    foreach (string parameter in parameters)
                    {
                        parameter_name = parameter.Substring(0, parameter.IndexOf("="));
                        parameter_value = parameter.Substring(parameter.IndexOf("=") + 1);
                        if (parameter_name.ToLower() == "entry")
                        {
                            keybreeze_entry_text = parameter_value;
                        }
                    }
                    // we need to launch a Keybreeze Desktop entry.
                    frmMain_instance.LaunchEntry(keybreeze_entry_text);
                    break;
                case "messagebox":
                    string messagebox_text = "";
                    string messagebox_title = "";
                    parameters = GetNextParameter(the_parameters);
                    foreach (string parameter in parameters)
                    {
                        parameter_name = parameter.Substring(0, parameter.IndexOf("="));
                        parameter_value = parameter.Substring(parameter.IndexOf("=") + 1);
                        if (parameter_name.ToLower() == "text")
                        {
                            messagebox_text = parameter_value;
                        }
                        else if (parameter_name.ToLower() == "title")
                        {
                            messagebox_title = parameter_value;
                        }
                    }
                    MessageBox.Show(this, messagebox_text, messagebox_title);
                    break;
                case "positionwindow":
                    string position_title = "";
                    string position_class = "";
                    string position_left = "";
                    string position_top = "";
                    string position_width = "";
                    string position_height = "";
                    parameters = GetNextParameter(the_parameters);
                    foreach (string parameter in parameters)
                    {
                        parameter_name = parameter.Substring(0, parameter.IndexOf("="));
                        parameter_value = parameter.Substring(parameter.IndexOf("=") + 1);
                        if (parameter_name.ToLower() == "title")
                        {
                            position_title = parameter_value;
                        }
                        else if (parameter_name.ToLower() == "class")
                        {
                            position_class = parameter_value;
                        }
                        else if (parameter_name.ToLower() == "left")
                        {
                            if (parameter_value.Length > 0)
                                position_left = parameter_value;
                        }
                        else if (parameter_name.ToLower() == "top")
                        {
                            if (parameter_value.Length > 0)
                                position_top = parameter_value;
                        }
                        else if (parameter_name.ToLower() == "width")
                        {
                            if (parameter_value.Length > 0)
                                position_width = parameter_value;
                        }
                        else if (parameter_name.ToLower() == "height")
                        {
                            if (parameter_value.Length > 0)
                                position_height = parameter_value;
                        }
                    }
                    PositionWindow(position_title, position_class, position_left, position_top, position_width, position_height);
                    break;
                case "waitforwindow":
                    string the_title = "";
                    string the_class = "";
                    string the_action = "";
                    int the_timeout = 0;
                    int time_elapsed = 0;
                    parameters = GetNextParameter(the_parameters);
                    foreach (string parameter in parameters)
                    {
                        parameter_name = parameter.Substring(0, parameter.IndexOf("="));
                        parameter_value = parameter.Substring(parameter.IndexOf("=") + 1);
                        if (parameter_name.ToLower() == "title")
                        {
                            the_title = parameter_value;
                        }
                        else if (parameter_name.ToLower() == "class")
                        {
                            the_class = parameter_value;
                        }
                        else if (parameter_name.ToLower() == "action")
                        {
                            the_action = parameter_value;
                        }
                        else if (parameter_name.ToLower() == "timeout")
                        {
                            the_timeout = Convert.ToInt32(parameter_value);
                        }
                    }
                    if (the_timeout == 0)
                        the_timeout = 90000;
                    int return_val = 0;
                    if (the_action == "to_close")
                    {
                        do
                        {
                            return_val = WaitForWindow("to_close", the_class, the_title);
                            if (return_val == 0)
                                break;
                            System.Threading.Thread.Sleep(100);
                            time_elapsed = time_elapsed + 100;
                            Application.DoEvents();
                            if (frmMain_instance.macro_running == false)
                                break;
                        } while (time_elapsed < the_timeout);
                    }
                    else if (the_action == "to_open")
                    {
                        do
                        {
                            return_val = WaitForWindow("to_open", the_class, the_title);
                            if (return_val == 1)
                                break;
                            System.Threading.Thread.Sleep(100);
                            time_elapsed = time_elapsed + 100;
                            Application.DoEvents();
                            if (frmMain_instance.macro_running == false)
                                break;
                        } while (time_elapsed < the_timeout);
                    }
                    break;
            }
        }

        private void PositionWindow(string position_title, string position_class, string position_left, string position_top, string position_width, string position_height)
        {
            // this function will resize a window, or position a window at certain coordinates on the screen.
            IntPtr exact_matching_window_handle = IntPtr.Zero;
            IntPtr partial_matching_window_handle = IntPtr.Zero;
            // validate the fields first.
            // trim out any wildcard characters on the edges.
            if (position_title.Substring(0, 1) == "*")
                position_title = position_title.Substring(1);
            if (position_title.Substring(position_title.Length - 1) == "*")
                position_title = position_title.Substring(0, position_title.Length - 1);
            if (position_title.Length > 0)
            {
                if (position_title.ToLower() == "getforegroundwindow()")
                {
                    // hide our recording window so that it doesn't get grabbed as our foreground window.
                    this.Visible = false;
                    Application.DoEvents();
                    System.Threading.Thread.Sleep(100);
                    exact_matching_window_handle = frmMain.GetForegroundWindow();
                }
                else
                {
                    Process[] processlist = Process.GetProcesses();
                    string window_title;
                    int find_match;
                    bool partial_match = false;
                    foreach (Process process in processlist)
                    {
                        Application.DoEvents();
                        if (frmMain_instance.macro_running == false)
                            break;
                        if (!String.IsNullOrEmpty(process.MainWindowTitle))
                        {
                            window_title = process.MainWindowTitle;
                            find_match = window_title.IndexOf(position_title);
                            if (find_match == 0)
                            {
                                // we have an exact match. Find the window handle and exit the loop.
                                try
                                {
                                    if (position_class.Length == 0)
                                        exact_matching_window_handle = frmMain.FindWindow(null, process.MainWindowTitle);
                                    else
                                        exact_matching_window_handle = frmMain.FindWindow(position_class, process.MainWindowTitle);
                                }
                                catch { }
                                break;
                            }
                            else if (find_match > 0)
                            {
                                // we have a partial match. keep a record of this. if we don't find anything, set focus to it.
                                partial_match = true;
                                try
                                {
                                    if (position_class.Length == 0)
                                        partial_matching_window_handle = frmMain.FindWindow(null, process.MainWindowTitle);
                                    else
                                        partial_matching_window_handle = frmMain.FindWindow(position_class, process.MainWindowTitle);
                                }
                                catch { }
                            }
                        }
                    }
                    if (exact_matching_window_handle == IntPtr.Zero && partial_match == true)
                        exact_matching_window_handle = partial_matching_window_handle;
                }
            }
            else
            {
                // we only have a class name.
                exact_matching_window_handle = frmMain.FindWindow(position_class, null);
            }
            if (exact_matching_window_handle == IntPtr.Zero)
            {
                // we never found the window to resize.
                return;
            }
            // before moving/resizing, restore the window if it's minimized or maximized.
            frmMain.ShowWindow(exact_matching_window_handle, 9);
            // we may have to get the current window's coordinates.
            Globals.RECT varRect = new Globals.RECT();
            bool found_window_rect = false;
            if ((position_top != "" && position_left != "") && (position_width == "" && position_height == ""))
            {
                // move window only. Don't resize.
                int position_left_int = 0;
                int position_top_int = 0;
                if (position_left == "left")
                    position_left_int = 0;
                else if (position_left == "right")
                {
                    // move window to the right.
                    //int screen_width = Screen.FromHandle(exact_matching_window_handle).Bounds.Width;
                    int screen_width = Screen.FromHandle(exact_matching_window_handle).WorkingArea.Width;
                    if (found_window_rect == false)
                    {
                        Globals.GetWindowRect(exact_matching_window_handle, out varRect);
                        found_window_rect = true;
                    }
                    int lngHeight = varRect.Bottom - varRect.Top;
                    int lngWidth = varRect.Right - varRect.Left;
                    position_left_int = screen_width - lngWidth;
                }
                else
                {
                    // the position is a regular number.
                    position_left_int = Convert.ToInt32(position_left);
                }
                if (position_top == "top")
                    position_top_int = 0;
                else if (position_top == "bottom")
                {
                    // move window to the bottom.
                    //int screen_height = Screen.FromHandle(exact_matching_window_handle).Bounds.Height;
                    int screen_height = Screen.FromHandle(exact_matching_window_handle).WorkingArea.Height;
                    if (found_window_rect == false)
                    {
                        Globals.GetWindowRect(exact_matching_window_handle, out varRect);
                        found_window_rect = true;
                    }
                    int lngHeight = varRect.Bottom - varRect.Top;
                    int lngWidth = varRect.Right - varRect.Left;
                    position_top_int = screen_height - lngHeight;
                }
                else
                {
                    // the position is a regular number.
                    position_top_int = Convert.ToInt32(position_top);
                }
                Globals.SetWindowPos(exact_matching_window_handle, 0, position_left_int, position_top_int, 0, 0, SWP_NOSIZE);
            }
            else if ((position_top == "" && position_left == "") && (position_width != "" && position_height != ""))
            {
                // resize window only. Don't move.
                int position_width_int = Convert.ToInt32(position_width);
                int position_height_int = Convert.ToInt32(position_height);
                Globals.SetWindowPos(exact_matching_window_handle, 0, 0, 0, position_width_int, position_height_int, SWP_NOMOVE);
            }
            else if ((position_top != "" && position_left != "") && (position_width != "" && position_height != ""))
            {
                // move window AND resize it.
                int position_left_int = 0;
                int position_top_int = 0;

                // Resize window first.
                // for the width and height, did the user specify a percentage or a number?
                int position_width_int = 0;
                int position_height_int = 0;
                int find_percent = position_width.IndexOf("%");
                if (find_percent != -1)
                {
                    // the user specified a percentage for the width. Convert it to a decimal and multiply it by the screen size.
                    int width1 = Convert.ToInt32(position_width.Substring(0, find_percent));
                    double position_width_percent = width1 * .01;
                    //int screen_width = Screen.FromHandle(exact_matching_window_handle).Bounds.Width;
                    int screen_width = Screen.FromHandle(exact_matching_window_handle).WorkingArea.Width;
                    position_width_int = Convert.ToInt32(Math.Ceiling(screen_width * position_width_percent));
                }
                else
                    position_width_int = Convert.ToInt32(position_width);
                find_percent = position_height.IndexOf("%");
                if (find_percent != -1)
                {
                    // the user specified a percentage for the height. Convert it to a decimal and multiply it by the screen size.
                    int height1 = Convert.ToInt32(position_height.Substring(0, find_percent));
                    double position_height_percent = height1 * .01;
                    //int screen_height = Screen.FromHandle(exact_matching_window_handle).Bounds.Height;
                    int screen_height = Screen.FromHandle(exact_matching_window_handle).WorkingArea.Height;
                    position_height_int = Convert.ToInt32(Math.Ceiling(screen_height * position_height_percent));
                }
                else
                    position_height_int = Convert.ToInt32(position_height);

                // Move window.
                if (position_left == "left")
                    position_left_int = 0;
                else if (position_left == "right")
                {
                    // move window to the right.
                    //int screen_width = Screen.FromHandle(exact_matching_window_handle).Bounds.Width;
                    int screen_width = Screen.FromHandle(exact_matching_window_handle).WorkingArea.Width;
                    int lngHeight = position_height_int;
                    int lngWidth = position_width_int;
                    position_left_int = screen_width - lngWidth;
                }
                else
                {
                    // the position is a regular number.
                    position_left_int = Convert.ToInt32(position_left);
                }
                if (position_top == "top")
                    position_top_int = 0;
                else if (position_top == "bottom")
                {
                    // move window to the bottom.
                    //int screen_height = Screen.FromHandle(exact_matching_window_handle).Bounds.Height;
                    int screen_height = Screen.FromHandle(exact_matching_window_handle).WorkingArea.Height;
                    int lngHeight = position_height_int;
                    int lngWidth = position_width_int;
                    position_top_int = screen_height - lngHeight;
                }
                else
                {
                    // the position is a regular number.
                    position_top_int = Convert.ToInt32(position_top);
                }
                Globals.SetWindowPos(exact_matching_window_handle, 0, position_left_int, position_top_int, position_width_int, position_height_int, 0);
            }
            if (position_title.ToLower() == "getforegroundwindow()")
            {
                // show our recording window again.
                this.Visible = true;
            }
        }

        private int WaitForWindow(string action, string class_name, string title_name)
        {
            // this function will wait until the window opens or closes.
            // returns 1 if the window is open, 0 if it's closed.
            IntPtr exact_matching_window_handle = IntPtr.Zero;
            IntPtr partial_matching_window_handle = IntPtr.Zero;
            // validate the fields first.
            if (class_name.Length == 0 && title_name.Length == 0)
                return -1;
            // trim out any wildcard characters on the edges.
            if (title_name.Substring(0, 1) == "*")
                title_name = title_name.Substring(1);
            if (title_name.Substring(title_name.Length - 1) == "*")
                title_name = title_name.Substring(0, title_name.Length - 1);
            if (title_name.Length > 0)
            {
                Process[] processlist = Process.GetProcesses();
                string window_title;
                int find_match;
                bool partial_match = false;

                foreach (Process process in processlist)
                {
                    Application.DoEvents();
                    if (frmMain_instance.macro_running == false)
                        break;
                    if (!String.IsNullOrEmpty(process.MainWindowTitle))
                    {
                        window_title = process.MainWindowTitle;
                        find_match = window_title.IndexOf(title_name);
                        if (find_match == 0)
                        {
                            // we have an exact match. Find the window handle and exit the loop.
                            try
                            {
                                if (class_name.Length == 0)
                                    exact_matching_window_handle = Globals.FindWindow(null, process.MainWindowTitle);
                                else
                                    exact_matching_window_handle = Globals.FindWindow(class_name, process.MainWindowTitle);
                            }
                            catch { }
                            break;
                        }
                        else if (find_match > 0)
                        {
                            // we have a partial match. keep a record of this. if we don't find anything, set focus to it.
                            partial_match = true;
                            try
                            {
                                if (class_name.Length == 0)
                                    partial_matching_window_handle = Globals.FindWindow(null, process.MainWindowTitle);
                                else
                                    partial_matching_window_handle = Globals.FindWindow(class_name, process.MainWindowTitle);
                            }
                            catch { }
                        }
                    }
                }
                if (exact_matching_window_handle == IntPtr.Zero && partial_match == true)
                    exact_matching_window_handle = partial_matching_window_handle;
            }
            else
            {
                // we only have a class name.
                exact_matching_window_handle = Globals.FindWindow(class_name, null);
            }
            if (exact_matching_window_handle == IntPtr.Zero)
                return 0;
            else
                return 1;
        }

        private void Macro_Delay(int delay_time)
        {
            // if we sleep for several seconds, the application won't respond to any requests to stop the macro, so we need to cut the delay down to small increments to allow other actions.
            if (delay_time < 250)
            {
                // this is too small to matter.
                System.Threading.Thread.Sleep(delay_time);
                return;
            }
            else
            {
                int elapsed_time = 0;
                do
                {
                    if (frmMain_instance.macro_running == false)
                        break;
                    System.Threading.Thread.Sleep(250);
                    Application.DoEvents();
                    elapsed_time = elapsed_time + 250;
                } while (elapsed_time < delay_time);
            }
        }

        private void Type_Characters(string text, string action = "")
        {
            // for upper case characters, we need to hold Shift and type them. Any lower case letters need to be converted to upper case to get their appropiate ASCII code.
            /*  old method of inserting one character at a time... this didn't work for special characters.
            string this_letter;
             * for (int i = 0; i < text.Length; i++)
            {
                this_letter = text[i].ToString();
                Press_Key(this_letter);
                if (frmMain_instance.macro_running == false)
                    break;
                Application.DoEvents();
            }
             */
            Simulate.TextInput(text);
        }

        private void Press_Key(string key_stroke, string action = "")
        {
            // launch the specified keystroke using the keybd_event API calls.
            byte key_code = 0;
            bool found_match = false;
            string upper_case_letter;
            byte this_letter_byte;

            // if the key_stroke is only one character, see if we can get the ASCII code.
            if (key_stroke.Length == 1)
            {
                upper_case_letter = key_stroke.ToUpper();
                UInt16 unicode = upper_case_letter[0];
                this_letter_byte = (byte)unicode;
                if (this_letter_byte > 64 && this_letter_byte < 91)
                {
                    // the character is a letter.
                    if (key_stroke == upper_case_letter)
                    {
                        // it's an upper case letter, so we need to hold Shift before we type it.
                        if (action == "keydown")
                        {
                            keybd_event(16, 0, keybd_keydown, UIntPtr.Zero);
                            System.Threading.Thread.Sleep(wait_time_for_modifiers);
                            keybd_event(this_letter_byte, 0, keybd_keydown, UIntPtr.Zero);
                        }
                        else if (action == "keyup")
                        {
                            keybd_event(this_letter_byte, 0, keybd_keyup, UIntPtr.Zero);
                            keybd_event(16, 0, keybd_keyup, UIntPtr.Zero);
                        }
                        else
                        {
                            keybd_event(16, 0, keybd_keydown, UIntPtr.Zero);
                            System.Threading.Thread.Sleep(wait_time_for_modifiers);
                            keybd_event(this_letter_byte, 0, keybd_keydown, UIntPtr.Zero);
                            keybd_event(this_letter_byte, 0, keybd_keyup, UIntPtr.Zero);
                            keybd_event(16, 0, keybd_keyup, UIntPtr.Zero);
                        }
                        found_match = true;
                    }
                    else
                    {
                        // lower case letter
                        if (action == "keydown")
                        {
                            keybd_event(this_letter_byte, 0, keybd_keydown, UIntPtr.Zero);
                        }
                        else if (action == "keyup")
                        {
                            keybd_event(this_letter_byte, 0, keybd_keyup, UIntPtr.Zero);
                        }
                        else
                        {
                            keybd_event(this_letter_byte, 0, keybd_keydown, UIntPtr.Zero);
                            keybd_event(this_letter_byte, 0, keybd_keyup, UIntPtr.Zero);
                        }
                        found_match = true;
                    }
                }
                else if (this_letter_byte > 47 && this_letter_byte < 58)
                {
                    // it's a number.
                    if (action == "keydown")
                    {
                        keybd_event(this_letter_byte, 0, keybd_keydown, UIntPtr.Zero);
                    }
                    else if (action == "keyup")
                    {
                        keybd_event(this_letter_byte, 0, keybd_keyup, UIntPtr.Zero);
                    }
                    else
                    {
                        keybd_event(this_letter_byte, 0, keybd_keydown, UIntPtr.Zero);
                        keybd_event(this_letter_byte, 0, keybd_keyup, UIntPtr.Zero);
                    }
                    found_match = true;
                }
                else if (this_letter_byte == 32)
                {
                    // it's a space.
                    if (action == "keydown")
                    {
                        keybd_event(this_letter_byte, 0, keybd_keydown, UIntPtr.Zero);
                    }
                    else if (action == "keyup")
                    {
                        keybd_event(this_letter_byte, 0, keybd_keyup, UIntPtr.Zero);
                    }
                    else
                    {
                        keybd_event(this_letter_byte, 0, keybd_keydown, UIntPtr.Zero);
                        keybd_event(this_letter_byte, 0, keybd_keyup, UIntPtr.Zero);
                    }
                    found_match = true;
                }
                else if (this_letter_byte == 13 || this_letter_byte == 10)
                {
                    // it's a new line. do nothing.
                    found_match = true;
                }
                else
                {
                    // it's a special character. The ASCII code isn't enough. We need to find how to use the keybd_event call to insert it.
                    int return_code = TypeSpecialCharacter(key_stroke, action);
                    if (return_code == 0)
                        found_match = false;
                    else
                        found_match = true;
                }
            }
            else
            {
                // the keystroke is more than one character. Check for possible values.
                switch (key_stroke)
                {
                    case "lwin":
                        key_code = 91;
                        found_match = true;
                        break;
                    case "rwin":
                        key_code = 92;
                        found_match = true;
                        break;
                    case "lshiftkey":
                        key_code = 16;
                        found_match = true;
                        break;
                    case "rshiftkey":
                        key_code = 16;
                        found_match = true;
                        break;
                    case "lcontrolkey":
                        key_code = 17;
                        found_match = true;
                        break;
                    case "rcontrolkey":
                        key_code = 17;
                        found_match = true;
                        break;
                    case "lmenu":
                        key_code = 18;
                        found_match = true;
                        break;
                    case "rmenu":
                        key_code = 18;
                        found_match = true;
                        break;
                    case "return":
                    case "enter":
                        key_code = 13;
                        found_match = true;
                        break;
                    case "d1":
                        key_code = 49;
                        found_match = true;
                        break;
                    case "d2":
                        key_code = 50;
                        found_match = true;
                        break;
                    case "d3":
                        key_code = 51;
                        found_match = true;
                        break;
                    case "d4":
                        key_code = 52;
                        found_match = true;
                        break;
                    case "d5":
                        key_code = 53;
                        found_match = true;
                        break;
                    case "d6":
                        key_code = 54;
                        found_match = true;
                        break;
                    case "d7":
                        key_code = 55;
                        found_match = true;
                        break;
                    case "d8":
                        key_code = 56;
                        found_match = true;
                        break;
                    case "d9":
                        key_code = 57;
                        found_match = true;
                        break;
                    case "d0":
                        key_code = 48;
                        found_match = true;
                        break;
                    case "oemminus":
                        key_code = 0xBD;
                        found_match = true;
                        break;
                    case "oemplus":
                        key_code = 0xBB;
                        found_match = true;
                        break;
                    case "oemtilde":
                        key_code = 192;
                        found_match = true;
                        break;
                    case "oemopenbrackets":
                        key_code = 0xDB;
                        found_match = true;
                        break;
                    case "oem6":
                        key_code = 0xDD;
                        found_match = true;
                        break;
                    case "oem5":
                        key_code = 0xDC;
                        found_match = true;
                        break;
                    case "oem1":
                        key_code = 0xBA;
                        found_match = true;
                        break;
                    case "oem7":
                        key_code = 0xDE;
                        found_match = true;
                        break;
                    case "oemcomma":
                        key_code = 0xBC;
                        found_match = true;
                        break;
                    case "oemperiod":
                        key_code = 0xBE;
                        found_match = true;
                        break;
                    case "oemquestion":
                        key_code = 191;
                        found_match = true;
                        break;
                    case "escape":
                        key_code = 0x1B;
                        found_match = true;
                        break;
                    case "f1":
                        key_code = 0x70;
                        found_match = true;
                        break;
                    case "f2":
                        key_code = 0x71;
                        found_match = true;
                        break;
                    case "f3":
                        key_code = 0x72;
                        found_match = true;
                        break;
                    case "f4":
                        key_code = 0x73;
                        found_match = true;
                        break;
                    case "f5":
                        key_code = 0x74;
                        found_match = true;
                        break;
                    case "f6":
                        key_code = 0x75;
                        found_match = true;
                        break;
                    case "f7":
                        key_code = 0x76;
                        found_match = true;
                        break;
                    case "f8":
                        key_code = 0x77;
                        found_match = true;
                        break;
                    case "f9":
                        key_code = 0x78;
                        found_match = true;
                        break;
                    case "f10":
                        key_code = 0x79;
                        found_match = true;
                        break;
                    case "f11":
                        key_code = 0x7A;
                        found_match = true;
                        break;
                    case "f12":
                        key_code = 0x7B;
                        found_match = true;
                        break;
                    case "f13":
                        key_code = 0x7C;
                        found_match = true;
                        break;
                    case "f14":
                        key_code = 0x7D;
                        found_match = true;
                        break;
                    case "f15":
                        key_code = 0x7E;
                        found_match = true;
                        break;
                    case "f16":
                        key_code = 0x7F;
                        found_match = true;
                        break;
                    case "f17":
                        key_code = 0x80;
                        found_match = true;
                        break;
                    case "f18":
                        key_code = 0x81;
                        found_match = true;
                        break;
                    case "f19":
                        key_code = 0x82;
                        found_match = true;
                        break;
                    case "f20":
                        key_code = 0x83;
                        found_match = true;
                        break;
                    case "f21":
                        key_code = 0x84;
                        found_match = true;
                        break;
                    case "f22":
                        key_code = 0x85;
                        found_match = true;
                        break;
                    case "f23":
                        key_code = 0x86;
                        found_match = true;
                        break;
                    case "f24":
                        key_code = 0x87;
                        found_match = true;
                        break;
                    case "tab":
                        key_code = 0x09;
                        found_match = true;
                        break;
                    case "capital":
                        key_code = 0x14;
                        found_match = true;
                        break;
                    case "space":
                        key_code = 32;
                        found_match = true;
                        break;
                    case "up":
                        key_code = 0x26;
                        found_match = true;
                        break;
                    case "down":
                        key_code = 0x28;
                        found_match = true;
                        break;
                    case "left":
                        key_code = 0x25;
                        found_match = true;
                        break;
                    case "right":
                        key_code = 0x27;
                        found_match = true;
                        break;
                    case "apps":
                        key_code = 0x5D;
                        found_match = true;
                        break;
                    case "back":
                        key_code = 0x08;
                        found_match = true;
                        break;
                    case "insert":
                        key_code = 0x2D;
                        found_match = true;
                        break;
                    case "home":
                        key_code = 0x24;
                        found_match = true;
                        break;
                    case "pageup":
                        key_code = 0x21;
                        found_match = true;
                        break;
                    case "pagedown":
                        key_code = 0x22;
                        found_match = true;
                        break;
                    case "delete":
                        key_code = 0x2E;
                        found_match = true;
                        break;
                    case "end":
                        key_code = 0x23;
                        found_match = true;
                        break;
                    case "next":
                        key_code = 0x22;
                        found_match = true;
                        break;
                    case "pause":
                        key_code = 0x13;
                        found_match = true;
                        break;
                    case "scroll":
                        key_code = 0x91;
                        found_match = true;
                        break;
                    case "printscreen":
                        key_code = 0x2C;
                        found_match = true;
                        break;
                    case "numlock":
                        key_code = 0x90;
                        found_match = true;
                        break;
                    case "divide":
                        key_code = 0x6F;
                        found_match = true;
                        break;
                    case "multiply":
                        key_code = 0x6A;
                        found_match = true;
                        break;
                    case "subtract":
                        key_code = 0x6D;
                        found_match = true;
                        break;
                    case "add":
                        key_code = 0x6B;
                        found_match = true;
                        break;
                    case "decimal":
                        key_code = 0x6E;
                        found_match = true;
                        break;
                    case "NumPad0":
                        key_code = 0x60;
                        found_match = true;
                        break;
                    case "NumPad1":
                        key_code = 0x61;
                        found_match = true;
                        break;
                    case "NumPad2":
                        key_code = 0x62;
                        found_match = true;
                        break;
                    case "NumPad3":
                        key_code = 0x63;
                        found_match = true;
                        break;
                    case "NumPad4":
                        key_code = 0x64;
                        found_match = true;
                        break;
                    case "NumPad5":
                        key_code = 0x65;
                        found_match = true;
                        break;
                    case "NumPad6":
                        key_code = 0x66;
                        found_match = true;
                        break;
                    case "NumPad7":
                        key_code = 0x67;
                        found_match = true;
                        break;
                    case "NumPad8":
                        key_code = 0x68;
                        found_match = true;
                        break;
                    case "NumPad9":
                        key_code = 0x69;
                        found_match = true;
                        break;
                    default:
                        found_match = false;
                        break;
                }
                if (found_match == true)
                {
                    if (action == "keydown")
                        keybd_event(key_code, 0, keybd_keydown, UIntPtr.Zero);
                    else if (action == "keyup")
                        keybd_event(key_code, 0, keybd_keyup, UIntPtr.Zero);
                    else
                    {
                        keybd_event(key_code, 0, keybd_keydown, UIntPtr.Zero);
                        keybd_event(key_code, 0, keybd_keyup, UIntPtr.Zero);
                    }
                }
            }
            if (found_match == false)
            {
                MessageBox.Show("Unknown key was specified in macro: " + key_stroke + ". Please edit this part of the macro and correct this.", "Keybreeze Desktop");
            }
            System.Threading.Thread.Sleep(wait_time_between_keys);
        }

        private int TypeSpecialCharacter(string this_letter, string action = "")
        {
            bool ShiftIsDown = false;
            byte NewCode = 0;

            switch (this_letter)
            {
                case "!":
                    ShiftIsDown = true;
                    NewCode = 49;
                    break;
                case "@":
                    ShiftIsDown = true;
                    NewCode = 50;
                    break;
                case "#":
                    ShiftIsDown = true;
                    NewCode = 51;
                    break;
                case "$":
                    ShiftIsDown = true;
                    NewCode = 52;
                    break;
                case "%":
                    ShiftIsDown = true;
                    NewCode = 53;
                    break;
                case "^":
                    ShiftIsDown = true;
                    NewCode = 54;
                    break;
                case "&":
                    ShiftIsDown = true;
                    NewCode = 55;
                    break;
                case "*":
                    ShiftIsDown = true;
                    NewCode = 56;
                    break;
                case "(":
                    ShiftIsDown = true;
                    NewCode = 57;
                    break;
                case ")":
                    ShiftIsDown = true;
                    NewCode = 48;
                    break;
                case "_":
                    ShiftIsDown = true;
                    NewCode = 189;
                    break;
                case "+":
                    ShiftIsDown = true;
                    NewCode = 187;
                    break;
                case "~":
                    ShiftIsDown = true;
                    NewCode = 192;
                    break;
                case "{":
                    ShiftIsDown = true;
                    NewCode = 219;
                    break;
                case "}":
                    ShiftIsDown = true;
                    NewCode = 221;
                    break;
                case "|":
                    ShiftIsDown = true;
                    NewCode = 220;
                    break;
                case ":":
                    ShiftIsDown = true;
                    NewCode = 186;
                    break;
                case "?":
                    ShiftIsDown = true;
                    NewCode = 191;
                    break;
                case "-":
                    NewCode = 189;
                    break;
                case "=":
                    NewCode = 187;
                    break;
                case "[":
                    NewCode = 219;
                    break;
                case "]":
                    NewCode = 221;
                    break;
                case @"\":
                    NewCode = 220;
                    break;
                case ";":
                    NewCode = 186;
                    break;
                case "'":
                    NewCode = 222;
                    break;
                case ",":
                    NewCode = 188;
                    break;
                case ".":
                    NewCode = 190;
                    break;
                case "/":
                    NewCode = 191;
                    break;
                case "<":
                    ShiftIsDown = true;
                    NewCode = 188;
                    break;
                case ">":
                    ShiftIsDown = true;
                    NewCode = 190;
                    break;
                default:
                    NewCode = 0;
                    break;
            }

            if (NewCode != 0)
            {
                if (ShiftIsDown == true)
                {
                    if (action == "keydown")
                    {
                        keybd_event(16, 0, keybd_keydown, UIntPtr.Zero);
                        System.Threading.Thread.Sleep(wait_time_for_modifiers);
                        keybd_event(NewCode, 0, keybd_keydown, UIntPtr.Zero);
                    }
                    else if (action == "keyup")
                    {
                        keybd_event(NewCode, 0, keybd_keyup, UIntPtr.Zero);
                        System.Threading.Thread.Sleep(wait_time_between_keys);
                        keybd_event(16, 0, keybd_keyup, UIntPtr.Zero);
                    }
                    else
                    {
                        keybd_event(16, 0, keybd_keydown, UIntPtr.Zero);
                        System.Threading.Thread.Sleep(wait_time_for_modifiers);
                        keybd_event(NewCode, 0, keybd_keydown, UIntPtr.Zero);
                        keybd_event(NewCode, 0, keybd_keyup, UIntPtr.Zero);
                        System.Threading.Thread.Sleep(wait_time_between_keys);
                        keybd_event(16, 0, keybd_keyup, UIntPtr.Zero);
                    }
                }
                else
                {
                    if (action == "keydown")
                    {
                        keybd_event(NewCode, 0, keybd_keydown, UIntPtr.Zero);
                    }
                    else if (action == "keyup")
                    {
                        keybd_event(NewCode, 0, keybd_keyup, UIntPtr.Zero);
                    }
                    else
                    {
                        keybd_event(NewCode, 0, keybd_keydown, UIntPtr.Zero);
                        keybd_event(NewCode, 0, keybd_keyup, UIntPtr.Zero);
                        System.Threading.Thread.Sleep(wait_time_between_keys);
                    }
                }
            }
            else
            {
                // unknown character. let's try to use the SendInput() call.
                if (action == "keydown")
                {
                    Simulate.TextInput(this_letter, "keydown");
                }
                else if (action == "keyup")
                {
                    Simulate.TextInput(this_letter, "keyup");
                }
                else
                {
                    Simulate.TextInput(this_letter, "keypress");
                    System.Threading.Thread.Sleep(wait_time_between_keys);
                }
            }
            return 1;
        }

        private List<string> GetNextParameter(string the_parameters)
        {
            // return array of parameters and their values, with just an equal sign as the separator. Remove surrounding quotes as well.
            List<string> parameters = new List<string>();
            int cursor_position = 0;
            int find_equal_sign;
            int find_first_quote;
            int find_second_quote;
            string the_original_parameters = the_parameters;
            string this_parameter_name;
            string this_parameter_value;

            try
            {
                do
                {
                    find_equal_sign = the_parameters.IndexOf("=");
                    find_first_quote = the_parameters.IndexOf('"', find_equal_sign + 1);
                    find_second_quote = the_parameters.IndexOf('"', find_first_quote + 1);
                    if (find_second_quote == -1)
                        break;
                    this_parameter_name = the_parameters.Substring(0, find_equal_sign).Trim();
                    this_parameter_value = the_parameters.Substring(find_first_quote + 1, find_second_quote - find_first_quote - 1);
                    // process escape characters:
                    this_parameter_value = this_parameter_value.Replace("breezeleftarrow", "\\<");
                    this_parameter_value = this_parameter_value.Replace("breezerightarrow", "\\>");
                    this_parameter_value = this_parameter_value.Replace("breezequotes", "\\" + '"');
                    parameters.Add(this_parameter_name + "=" + this_parameter_value);
                    // trim this parameter out of the_parameters string so we can add the next parameter to the list.
                    cursor_position = find_second_quote + 1;
                    the_parameters = the_parameters.Substring(cursor_position);
                } while (cursor_position < the_original_parameters.Length - 1);
            }
            catch { }
            return parameters;
        }

        private void frmMacroPlayback_FormClosing(object sender, FormClosingEventArgs e)
        {
            frmMain_instance.macro_running = false;
        }

        private void label1_MouseDown(object sender, MouseEventArgs e)
        {
            frmMain_instance.macro_running = false;
        }

        private void frmMacroPlayback_MouseDown(object sender, MouseEventArgs e)
        {
            frmMain_instance.macro_running = false;
        }
    }

    public class Simulate
    {
        [DllImport("USER32.DLL", CharSet = CharSet.Unicode)]
        static extern UInt32 SendInput(UInt32 numberOfInputs, INPUT[] input, Int32 sizeOfInputStructure);

        [StructLayout(LayoutKind.Sequential, Size = 24)]
        struct KEYBDINPUT
        {
            public UInt16 Vk;
            public UInt16 Scan;
            public UInt32 Flags;
            public UInt32 Time;
            public UInt32 ExtraInfo;
        }

        [StructLayout(LayoutKind.Explicit)]
        private struct INPUT
        {
            [FieldOffset(0)]
            public int Type;
            [FieldOffset(4)]
            public KEYBDINPUT ki;
        }

        public static void TextInput(string text, string action = "keypress")
        {
            char[] chars = text.ToCharArray();

            for (int i = 0; i < chars.Length; i++)
            {
                UInt16 unicode = chars[i];

                INPUT down = new INPUT();
                down.Type = 1; //INPUT_KEYBOARD
                down.ki.Vk = 0;
                down.ki.Scan = unicode;
                down.ki.Time = 0;
                down.ki.Flags = 0x0004; //KEYEVENTF_UNICODE
                down.ki.ExtraInfo = 0;

                INPUT up = new INPUT();
                up.Type = 1; //INPUT_KEYBOARD
                up.ki.Vk = 0;
                up.ki.Scan = unicode;
                up.ki.Time = 0;
                up.ki.Flags = 0x0004; //KEYEVENTF_UNICODE
                up.ki.ExtraInfo = 0;

                INPUT[] input = new INPUT[2];
                if (action == "keydown")
                {
                    input[0] = down;
                    SendInput(1, input, Marshal.SizeOf(typeof(INPUT)));
                }
                else if (action == "keyup")
                {
                    input[0] = up;
                    SendInput(1, input, Marshal.SizeOf(typeof(INPUT)));
                }
                else
                {
                    input[0] = down;
                    input[1] = up;
                    SendInput(1, input, Marshal.SizeOf(typeof(INPUT)));
                }
            }
        }
    }
}
