﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace Keybreeze
{
    public partial class frmNote : Form
    {
        frmMain frmMain_instance;

        private const int cGrip = 32;      // Grip size
        private const int cCaption = 32;   // Caption bar height;

        public bool long_note = false;

        public string NotesDir = Globals.WorkPath + "\\notes";
        public int UniqueID;
        public bool existing_note;
        public string note_html = "";
        public string note_image = "";
        int counter = 0;
        int m_iWindowPosX;
        int m_iWindowPosY;
        int note_height;
        int note_width;
        bool is_reminder = false;
        string clipboard_text;

        public int save_attempts = 0;

        public frmNote(int tempUniqueID, bool tempExistingNote, bool reminder, frmMain parent)
        {
            // the note will have a unique ID to keep it separate from other notes.
            InitializeComponent();
            UniqueID = tempUniqueID;
            existing_note = tempExistingNote;
            is_reminder = reminder;
            frmMain_instance = parent;
        }

        private void frmNote_Load(object sender, EventArgs e)
        {
            string TXT_path = NotesDir + "\\" + UniqueID.ToString() + ".txt";
            string INI_path = NotesDir + "\\" + UniqueID.ToString() + ".ini";
            string note_text = "";

            if (existing_note == true && File.Exists(INI_path) == true)
            {
                // we have an existing note and the properties file exists. Let's load it.
                try
                {
                    note_text = File.ReadAllText(TXT_path);
                }
                catch
                {
                }
                txtNote.Text = note_text;
                lblNote.Text = note_text;
                FormatLabelLinks();
                if (is_reminder == true)
                {
                    LoadDefaultAppearanceSettings();
                    Globals.WriteINI("properties", "hiddenreminder", "0", INI_path);
                    CenterScreen();
                    txtNote.Visible = false;
                    lblTop.Visible = false;
                    lblNote.Visible = true;
                    pictureBox1.Visible = true;
                    KeepNoteOnTop(1);
                    PlaySound(Globals.WorkPath + "\\notes\\" + UniqueID + ".ini");
                }
                else
                {
                    LoadAppearanceFromPropertiesFile(INI_path);
                }
            }
            else if (existing_note == true && File.Exists(INI_path) == false)
            {
                // we have an existing note but there is no properties file. Load the defaults.
                try
                {
                    note_text = File.ReadAllText(TXT_path);
                }
                catch
                {
                }
                txtNote.Text = note_text;
                lblNote.Text = note_text;
                FormatLabelLinks();
                LoadDefaultAppearanceSettings();
            }
            else
            {
                // we are creating a new note.
                txtNote.Text = Globals.new_note_text;
                lblNote.Text = Globals.new_note_text;
                FormatLabelLinks();
                LoadDefaultAppearanceSettings();
            }

            // these two lines are required to give the label a transparent background.
            this.lblNote.BackColor = System.Drawing.Color.Transparent;
            this.lblNote.Parent = pictureBox1;

            if (IsOnScreen(this) == false)
            {
                this.Top = 0;
                this.Left = 0;
            }

            // should we keep the note on top of other windows?
            int on_top = Globals.ReadIntFromINI("properties", "on_top", INI_path);
            if (on_top == 1)
            {
                KeepNoteOnTop(1);
            }
        }

        private void txtNote_DoubleClick(object sender, EventArgs e)
        {
            if (long_note == false)
            {
                // only lock the note if we are using a sticky note and not a long note.
                lblNote.Text = txtNote.Text;
                FormatLabelLinks();
                txtNote.Visible = false;
                lblTop.Visible = false;
                pictureBox1.Visible = true;
                lblNote.Visible = true;
            }
        }

        private void FormatLabelLinks()
        {
            // this function will search our label text for any links, and it will format those portions as links.
            lblNote.Links.Clear();
            string note_text = lblNote.Text.ToLower();
            int cursor = 0;
            try
            {
                for (int i = 0; i < 50; i++)
                {
                    int find_http = note_text.IndexOf("http", cursor);
                    if (find_http != -1)
                    {
                        // find end of link.
                        int find_end_of_link = note_text.IndexOf("\n", find_http + 1);
                        if (find_end_of_link == -1)
                            find_end_of_link = note_text.IndexOf(" ", find_http + 1);
                        if (find_end_of_link == -1)
                            find_end_of_link = note_text.IndexOf("http", find_http + 1);
                        if (find_end_of_link == -1)
                        {
                            // didn't find a space or new line after the link. The link may go to the end, or it may stop with punctuation.
                            find_end_of_link = note_text.Length;
                        }
                        // remove any punctuation at the end.
                        string link_text = note_text.Substring(find_http, find_end_of_link - find_http);
                        string last_char = link_text.Substring(link_text.Length - 1);
                        if (last_char == "." || last_char == ",")
                        {
                            find_end_of_link--;
                            link_text = link_text.Substring(0, link_text.Length - 1);
                        }

                        lblNote.Links.Add(find_http, find_end_of_link - find_http, link_text);
                        cursor = find_http + 1;
                    }
                    else
                    {
                        // no more links. Exit the loop.
                        break;
                    }
                }
            }
            catch(Exception e2)
            {
                // something went wrong. Let's clear all the links.
                lblNote.Links.Clear();
            }
        }

        private void lblNote_DoubleClick(object sender, EventArgs e)
        {
            // we need to see if the clipboard text changed. This fixes a windows bug that is described here: http://stackoverflow.com/questions/2519587/is-there-any-way-to-disable-the-double-click-to-copy-functionality-of-a-net-l
            string new_clipboard_text = Clipboard.GetText();
            if (new_clipboard_text != clipboard_text)
            {
                // windows changed the clipboard text. undo this.
                try
                {
                    Clipboard.SetText(clipboard_text);
                }
                catch
                {
                }
            }
            pictureBox1.Visible = false;
            lblNote.Visible = false;
            txtNote.Visible = true;
            lblTop.Visible = true;
            txtNote.Focus();
            txtNote.SelectionStart = 0;
            txtNote.SelectionLength = txtNote.Text.Length;
        }

        protected override void WndProc(ref Message m)
        {
            /*
            const int wmNcHitTest = 0x84;
            const int htBottomLeft = 16;
            const int htBottomRight = 17;
            if (m.Msg == wmNcHitTest)
            {
                int x = (int)(m.LParam.ToInt64() & 0xFFFF);
                int y = (int)((m.LParam.ToInt64() & 0xFFFF0000) >> 16);
                Point pt = PointToClient(new Point(x, y));
                Size clientSize = ClientSize;
                if (pt.X >= clientSize.Width - 16 && pt.Y >= clientSize.Height - 16 && clientSize.Height >= 16)
                {
                    m.Result = (IntPtr)(IsMirrored ? htBottomLeft : htBottomRight);
                    AdjustTextandLabelToForm();
                    return;
                }
                AdjustTextandLabelToForm();
            }
            */
            const int RESIZE_HANDLE_SIZE = 10;

            switch (m.Msg)
            {
                case 0x0084/*NCHITTEST*/ :
                    base.WndProc(ref m);

                    if ((int)m.Result == 0x01/*HTCLIENT*/)
                    {
                        Point screenPoint = new Point(m.LParam.ToInt32());
                        Point clientPoint = this.PointToClient(screenPoint);
                        if (clientPoint.Y <= RESIZE_HANDLE_SIZE)
                        {
                            if (clientPoint.X <= RESIZE_HANDLE_SIZE)
                                m.Result = (IntPtr)13/*HTTOPLEFT*/ ;
                            else if (clientPoint.X < (Size.Width - RESIZE_HANDLE_SIZE))
                                m.Result = (IntPtr)12/*HTTOP*/ ;
                            else
                                m.Result = (IntPtr)14/*HTTOPRIGHT*/ ;
                        }
                        else if (clientPoint.Y <= (Size.Height - RESIZE_HANDLE_SIZE))
                        {
                            if (clientPoint.X <= RESIZE_HANDLE_SIZE)
                                m.Result = (IntPtr)10/*HTLEFT*/ ;
                            else if (clientPoint.X < (Size.Width - RESIZE_HANDLE_SIZE))
                                m.Result = (IntPtr)2/*HTCAPTION*/ ;
                            else
                                m.Result = (IntPtr)11/*HTRIGHT*/ ;
                        }
                        else
                        {
                            if (clientPoint.X <= RESIZE_HANDLE_SIZE)
                                m.Result = (IntPtr)16/*HTBOTTOMLEFT*/ ;
                            else if (clientPoint.X < (Size.Width - RESIZE_HANDLE_SIZE))
                                m.Result = (IntPtr)15/*HTBOTTOM*/ ;
                            else
                                m.Result = (IntPtr)17/*HTBOTTOMRIGHT*/ ;
                        }
                    }
                    AdjustTextandLabelToForm();
                    return;
            }
            base.WndProc(ref m);
        }

        private void AdjustTextandLabelToForm()
        {
            pictureBox1.Width = this.Width;
            pictureBox1.Height = this.Height;
            lblNote.Width = this.Width - 11;
            lblNote.Height = this.Height - 8;
            lblTop.Width = this.Width;
            if (long_note == false)
            {
                txtNote.Width = this.Width - 11;
                txtNote.Height = this.Height - 12;
            }
            else
            {
                txtNote.Width = this.Width - 11;
                txtNote.Height = this.Height - 25;
            }
        }

        private void pictureBox1_MouseMove(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
                Location = new Point(Location.X - m_iWindowPosX + e.X, Location.Y - m_iWindowPosY + e.Y);

            base.OnMouseMove(e);
        }

        private void pictureBox1_MouseDown(object sender, MouseEventArgs e)
        {
            if (Globals.sound_done_looping == false)
                Globals.sound_done_looping = true;

            m_iWindowPosX = e.X;
            m_iWindowPosY = e.Y;

            base.OnMouseDown(e);
        }

        private void lblNote_MouseMove(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
                Location = new Point(Location.X - m_iWindowPosX + e.X, Location.Y - m_iWindowPosY + e.Y);

            base.OnMouseMove(e);
        }

        private void lblNote_MouseDown(object sender, MouseEventArgs e)
        {
            if (Globals.sound_done_looping == false)
                Globals.sound_done_looping = true;

            // update by Andrew on May 21.
            if (frmMain_instance.Visible == true)
            {
                // hide our main form.
                frmMain_instance.HideMainForm(false, false);
            }

            m_iWindowPosX = e.X;
            m_iWindowPosY = e.Y;

            base.OnMouseDown(e);
            //tmrSaveNote.Enabled = true;
            if (tmrSaveNote.Enabled == false)
            {
                tmrSaveNote.Interval = 250;
                tmrSaveNote.Enabled = true;
            }
        }

        private void frmNote_Resize(object sender, EventArgs e)
        {
            AdjustTextandLabelToForm();
            //tmrSaveNote.Enabled = true;
            if (tmrSaveNote.Enabled == false)
            {
                tmrSaveNote.Interval = 250;
                tmrSaveNote.Enabled = true;
            }
        }

        private void txtNote_TextChanged(object sender, EventArgs e)
        {
            // commented out by Andrew on 5/27/2020. We are getting an intermittent exception saying that it cannot access this file because it's being used by another process. Let's try making the timer do this instead.
            //SaveNote(true);
            if (tmrSaveNote.Enabled == false)
            {
                tmrSaveNote.Interval = 250;
                tmrSaveNote.Enabled = true;
            }
        }

        private void tmrSaveNote_Tick(object sender, EventArgs e)
        {
            SaveNote();
            if (save_attempts == 0)
            {
                tmrSaveNote.Enabled = false;
                return;
            }
        }

        private void SaveNote(bool dont_save_properties = false)
        {
            try
            {
                // update by Andrew on 2/27/17.
                // We only want to save the note .txt if the text actually changed.
                string note_text_already_saved = "";
                string note_txt_file = NotesDir + "\\" + UniqueID.ToString() + ".txt";
                if (File.Exists(note_txt_file))
                    note_text_already_saved = File.ReadAllText(note_txt_file);
                if (note_text_already_saved != txtNote.Text)
                    File.WriteAllText(note_txt_file, txtNote.Text, Encoding.Unicode);
                if (dont_save_properties == false)
                    GenerateNoteProperties(NotesDir + "\\" + UniqueID.ToString() + ".ini");
                // if we are here, set our save_attempts back to 0.
                save_attempts = 0;
            }
            catch (Exception e)
            {
                if (save_attempts <= 5)
                {
                    // before throwing an error, give it a few more chances to save. It may have failed the first time due to antivirus currently locking the file.
                    tmrSaveNote.Interval = 500;
                    save_attempts++;
                }
                else
                {
                    save_attempts = 0;
                    MessageBox.Show(this, "Keybreeze Desktop could not write to the " + NotesDir + " directory to save the text of this note. " + e.Message, "Keybreeze Desktop");
                }
            }
        }

        private void CreateNoteTextFile()
        {
            try
            {
                System.IO.File.WriteAllText(NotesDir + "\\" + UniqueID.ToString() + ".txt", txtNote.Text, Encoding.Unicode);
                GenerateNoteProperties(NotesDir + "\\" + UniqueID.ToString() + ".ini");
            }
            catch
            {
                MessageBox.Show("Keybreeze Desktop could not write to the " + NotesDir + " directory to create a note. Please check your Windows permissions and make sure this directory exists.", "Keybreeze Desktop");
            }
        }

        private void GenerateNoteProperties(string file_path)
        {
            // this will generate properties for a note so they can be stored in a unique ID INI file.

            var cvt = new FontConverter();
            string text_font = cvt.ConvertToString(txtNote.Font);
            string text_forecolor = ColorTranslator.ToHtml(txtNote.ForeColor);
            int find_comma = text_font.LastIndexOf(",");
            if (find_comma > 0)
            {
                text_font = text_font.Substring(0, find_comma);
            }
            Globals.WriteINI("properties", "top", this.Top.ToString(), file_path);
            Globals.WriteINI("properties", "left", this.Left.ToString(), file_path);
            Globals.WriteINI("properties", "width", this.Width.ToString(), file_path);
            Globals.WriteINI("properties", "height", this.Height.ToString(), file_path);
            Globals.WriteINI("properties", "backcolor", note_html, file_path);
            Globals.WriteINI("properties", "image", note_image, file_path);
            if (long_note == false)
            {
                Globals.WriteINI("properties", "textcolor", text_forecolor, file_path);
                Globals.WriteINI("properties", "font", text_font, file_path);
                Globals.WriteINI("properties", "size", txtNote.Font.SizeInPoints.ToString(), file_path);
                Globals.WriteINI("properties", "bold", txtNote.Font.Bold.ToString(), file_path);
                Globals.WriteINI("properties", "italic", txtNote.Font.Italic.ToString(), file_path);
                Globals.WriteINI("properties", "underline", txtNote.Font.Underline.ToString(), file_path);
            }
        }

        public void CenterScreen()
        {
            int boundWidth = Screen.PrimaryScreen.Bounds.Width;
            int boundHeight = Screen.PrimaryScreen.Bounds.Height;
            int x = boundWidth - this.Width;
            int y = boundHeight - this.Height;
            this.Location = new Point(x / 2, y / 2);
        }

        private void LoadAppearanceFromPropertiesFile(string INI_File)
        {
            // this will load the note properties from the [uniqueID].txt file.
            int note_width = Globals.ReadIntFromINI("properties", "width", INI_File, 202);
            int note_height = Globals.ReadIntFromINI("properties", "height", INI_File, 160);
            int note_top = Globals.ReadIntFromINI("properties", "top", INI_File, 0);
            int note_left = Globals.ReadIntFromINI("properties", "left", INI_File, 0);
            note_html = Globals.ReadINI("properties", "backcolor", INI_File);
            note_image = Globals.ReadINI("properties", "image", INI_File);
            string note_textcolor = Globals.ReadINI("properties", "textcolor", INI_File);
            string note_font = Globals.ReadINI("properties", "font", INI_File);
            double note_font_size;
            
            try
            {
                txtNote.ForeColor = System.Drawing.ColorTranslator.FromHtml(note_textcolor);
                lblNote.ForeColor = txtNote.ForeColor;
                note_font_size = Convert.ToDouble(Globals.ReadINI("properties", "size", INI_File));
            }
            catch
            {
                note_font_size = 10;
            }
            string note_bold = Globals.ReadINI("properties", "bold", INI_File);
            string note_italic = Globals.ReadINI("properties", "italic", INI_File);
            string note_underline = Globals.ReadINI("properties", "underline", INI_File);

            this.Width = note_width;
            this.Height = note_height;
            this.Top = note_top;
            this.Left = note_left;

            // before we set the backcolor and image, we need to see if it's set. If not, pick a color.
            if (note_html == "0" || note_image == "0")
            {
                LoadColorAndImageForNote();
            }
            this.BackColor = System.Drawing.ColorTranslator.FromHtml(note_html);
            txtNote.BackColor = this.BackColor;
            lblTop.BackColor = this.BackColor;
            try
            {
                pictureBox1.Image = Image.FromFile(Globals.WorkPath + "\\notes\\themes\\" + Globals.note_theme + "\\" + note_image);
            }
            catch
            {
            }
            try
            {
                // set the fontstyle, bold, italic, underline, etc.
                FontStyle myfontstyle = FontStyle.Regular;
                if (note_bold == "True")
                    myfontstyle = myfontstyle | FontStyle.Bold;
                if (note_italic == "True")
                    myfontstyle = myfontstyle | FontStyle.Italic;
                if (note_underline == "True")
                    myfontstyle = myfontstyle | FontStyle.Underline;

                float note_font_size_float = (float)note_font_size;
                txtNote.Font = new System.Drawing.Font(note_font, note_font_size_float, myfontstyle, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
                lblNote.Font = new System.Drawing.Font(note_font, note_font_size_float, myfontstyle, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            }
            catch
            {
            }

            // set the 3d appearance.
            txtNote.Visible = false;
            lblTop.Visible = false;
            lblNote.Visible = true;
            pictureBox1.Visible = true;
        }

        public void PlaySound(string INI_File)
        {
            int play_sound = Globals.ReadIntFromINI("properties", "sound", INI_File);
            if (play_sound == 1)
            {
                string sound_file_path = Globals.ReadINI("properties", "sound_file", INI_File);
                int sound_loop = Globals.ReadIntFromINI("properties", "sound_loop", INI_File);
                if (sound_loop == 1)
                {
                    Globals.sound_play_sound(sound_file_path, true);
                }
                else
                {
                    Globals.sound_play_sound(sound_file_path);
                }
            }
        }

        private void LoadDefaultAppearanceSettings()
        {
            // load settings for this notes, e.g. size, color etc.
            LoadColorAndImageForNote();
            note_width = Globals.ReadIntFromINI("notes", "width", Globals.NotesINI, 202);
            note_height = Globals.ReadIntFromINI("notes", "height", Globals.NotesINI, 160);
            if (note_width < 15 || note_height < 15)
            {
                note_width = 60;
                note_height = 50;
            }
            this.Width = note_width;
            this.Height = note_height;
            AdjustTextandLabelToForm();

            this.BackColor = System.Drawing.ColorTranslator.FromHtml(note_html);
            txtNote.BackColor = this.BackColor;
            lblTop.BackColor = this.BackColor;
            try
            {
                pictureBox1.Image = Image.FromFile(Globals.WorkPath + "\\notes\\themes\\" + Globals.note_theme + "\\" + note_image);
            }
            catch
            {
            }
            try
            {
                string note_font = Globals.ReadINI("notes", "font", Globals.NotesINI);
                double note_font_size = Convert.ToDouble(Globals.ReadINI("notes", "size", Globals.NotesINI));
                float note_font_size_float = (float)note_font_size;
                txtNote.Font = new System.Drawing.Font(note_font, note_font_size_float, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
                lblNote.Font = new System.Drawing.Font(note_font, note_font_size_float, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            }
            catch
            {
            }
        }

        private void LoadColorAndImageForNote()
        {
            if (Globals.note_theme == "")
            {
                Globals.note_theme = Globals.ReadINI("notes", "color_theme", Globals.NotesINI);
                if (Globals.note_theme == "")
                {
                    Globals.note_theme = "light colors";
                    Globals.WriteINI("notes", "color_theme", "light colors", Globals.NotesINI);
                }
            }

            for (int i = 1; i <= 20; i++)
            {
                note_html = Globals.ReadINI(Globals.note_theme, i + "html", Globals.NotesINI);
                note_image = Globals.ReadINI(Globals.note_theme, i + "image", Globals.NotesINI);
                if (note_html.Length == 0 || note_image.Length == 0)
                {
                    break;
                }
                counter++;
            }
            Random random_num = new Random();
            int randomNumber = random_num.Next(1, counter);
            note_html = Globals.ReadINI(Globals.note_theme, randomNumber + "html", Globals.NotesINI);
            note_image = Globals.ReadINI(Globals.note_theme, randomNumber + "image", Globals.NotesINI);
        }

        private void lblTop_MouseMove(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
                Location = new Point(Location.X - m_iWindowPosX + e.X, Location.Y - m_iWindowPosY + e.Y);

            base.OnMouseMove(e);
        }

        private void lblTop_MouseDown(object sender, MouseEventArgs e)
        {
            m_iWindowPosX = e.X;
            m_iWindowPosY = e.Y;

            base.OnMouseDown(e);
            tmrSaveNote.Enabled = true;
        }

        private static bool IsOnScreen(Form form)
        {
            Screen[] screens = Screen.AllScreens;
            foreach (Screen screen in screens)
            {
                Point formTopLeft = new Point(form.Left, form.Top);

                if (screen.WorkingArea.Contains(formTopLeft))
                {
                    return true;
                }
            }

            return false;
        }

        private void keepToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (keepToolStripMenuItem.Checked == false)
            {
                KeepNoteOnTop(1);
            }
            else
            {
                KeepNoteOnTop(0);
            }
        }

        private void setReminderToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string note_backcolor = ColorTranslator.ToHtml(txtNote.BackColor);
            Globals.notes_open_reminder_window(txtNote.Text, note_backcolor, UniqueID);
        }

        private void hideNoteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            HideNote();
        }

        private void HideNote()
        {
            this.Visible = false;
            Globals.WriteINI("properties", "hidden", "1", NotesDir + "\\" + UniqueID + ".ini");
        }

        private void hideAllNotesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Globals.notes_hide_all_notes();
        }

        private void showHiddenNotesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Globals.notes_show_hidden_notes();
        }

        public void KeepNoteOnTop(int on_top)
        {
            // if on_top = 1, keep note on top.
            // if on_top = 0, undo this.
            if (on_top == 1)
            {
                this.TopMost = true;
                keepToolStripMenuItem.Checked = true;
                Globals.WriteINI("properties", "on_top", "1", NotesDir + "\\" + UniqueID + ".ini");
            }
            else
            {
                this.TopMost = false;
                keepToolStripMenuItem.Checked = false;
                Globals.WriteINI("properties", "on_top", "0", NotesDir + "\\" + UniqueID + ".ini");
            }
        }

        private void optionsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmNoteOptions frmNoteOptions_instance;
            frmNoteOptions_instance = new frmNoteOptions();
            frmNoteOptions_instance.note_id = UniqueID;
            frmNoteOptions_instance.Show();
        }

        private void keepAllNotesOnTopToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (keepAllNotesOnTopToolStripMenuItem.Checked == false)
            {
                Globals.notes_keep_all_notes_on_top(1);
                keepAllNotesOnTopToolStripMenuItem.Checked = true;
            }
            else
            {
                Globals.notes_keep_all_notes_on_top(0);
                keepAllNotesOnTopToolStripMenuItem.Checked = false;
            }
        }

        private void deleteNoteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Globals.notes_delete_note(UniqueID);
            this.Close();
        }

        private void minutesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SnoozeNote(2);
        }

        private void SnoozeNote(int minutes)
        {
            DateTime new_time = DateTime.Now.AddMinutes(minutes);
            int i = Globals.notes_set_reminder(1, new_time.Year, new_time.Month, new_time.Day, new_time.Hour, new_time.Minute, 0, UniqueID.ToString());
            if (i == 0)
            {
                MessageBox.Show("The Keybreeze Desktop configuration file could not be modified. Please make sure you have read and write access to the " + Globals.WorkPath + "\\notes directory and all subfolders.", "Keybreeze Desktop");
                return;
            }
            this.Visible = false;
        }

        private void minutesToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            SnoozeNote(5);
        }

        private void minutesToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            SnoozeNote(10);
        }

        private void minutesToolStripMenuItem3_Click(object sender, EventArgs e)
        {
            SnoozeNote(15);
        }

        private void minutesToolStripMenuItem4_Click(object sender, EventArgs e)
        {
            SnoozeNote(20);
        }

        private void minutesToolStripMenuItem5_Click(object sender, EventArgs e)
        {
            SnoozeNote(30);
        }

        private void minutesToolStripMenuItem6_Click(object sender, EventArgs e)
        {
            SnoozeNote(45);
        }

        private void hourToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SnoozeNote(60);
        }

        private void viewRemindersToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmNoteReminders frmNoteReminders_instance = new frmNoteReminders();
            frmNoteReminders_instance.Show();
        }

        private void lblNote_Click(object sender, EventArgs e)
        {
            // we need to save the clipboard text to fix a windows bug. Bug is described here: http://stackoverflow.com/questions/2519587/is-there-any-way-to-disable-the-double-click-to-copy-functionality-of-a-net-l
            clipboard_text = Clipboard.GetText();
        }

        private void contextMenuStrip1_Opening(object sender, CancelEventArgs e)
        {
            if (note_return_reminder_type() > 1)
                deleteNoteToolStripMenuItem.Text = "Dismiss reminder";
            else
                deleteNoteToolStripMenuItem.Text = "Delete note";
        }

        private int note_return_reminder_type()
        {
            int check_reminder;
            string reminders_file = Globals.WorkPath + "\\notes\\settings\\reminders.ini";
            int reminder_type = 0;

            for (int i = 1; i <= 100; i++)
            {
                check_reminder = Globals.ReadIntFromINI("reminders", i + "-noteid", reminders_file, 0);
                if (check_reminder == 0)
                    return 0;
                if (check_reminder == UniqueID)
                {
                    // we found the note reminder in the reminders.ini file.
                    // we don't want to delete a daily, weekly, or monthly reminder. These reminders should only be deleted from the 'View Reminders' window.
                    reminder_type = Globals.ReadIntFromINI("reminders", i + "-type", reminders_file, 0);
                    return reminder_type;
                }
            }
            return 0;
        }

        private void customToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmInputBoxDialog frmInputBoxDialog_instance;
            frmInputBoxDialog_instance = new frmInputBoxDialog();
            frmInputBoxDialog_instance.lblLabel.Text = "How long do you want to snooze the note?" + Environment.NewLine + "Examples: 2h, 3h, 25m, 1h30m";
            frmInputBoxDialog_instance.ShowDialog();
            string original_response = frmInputBoxDialog_instance.the_text;
            string user_response = original_response.ToLower();
            int the_hours = 0, the_minutes = 0;
            try
            {
                int find_h = user_response.IndexOf("h");
                int find_m = user_response.IndexOf("m");
                if (find_h == -1 && find_m == -1)
                {
                    MessageBox.Show("Could not understand " + original_response + ". Please specify 'h' for hours or 'm' for minutes." + Environment.NewLine + "Examples: 2h, 3h, 25m, 1h30m");
                    return;
                }
                if (find_h > 0)
                {
                    // get numbers before h.
                    string before_h = user_response.Substring(0, find_h);
                    the_hours = Convert.ToInt32(before_h);
                    user_response = user_response.Substring(find_h + 1);
                    // reset find_m
                    find_m = user_response.IndexOf("m");
                }
                if (find_m > 0)
                {
                    // get numbers before m
                    string before_m = user_response.Substring(0, find_m);
                    the_minutes = Convert.ToInt32(before_m);
                }
                int snooze_time = (the_hours * 60) + the_minutes;
                SnoozeNote(snooze_time);
            }
            catch
            {
                MessageBox.Show("Could not understand " + original_response + ". Please specify 'h' for hours or 'm' for minutes." + Environment.NewLine + "Examples: 2h, 3h, 25m, 1h30m");
                return;
            }

        }

        private void lblNote_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            string url = e.Link.LinkData.ToString();
            System.Diagnostics.Process.Start(url);
        }

        private void txtNote_MouseDown(object sender, MouseEventArgs e)
        {
            // update by Andrew on May 21.
            if (frmMain_instance.Visible == true)
            {
                // hide our main form.
                frmMain_instance.HideMainForm(false, false);
            }
        }

        private void switchToLongNoteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (switchToLongNoteToolStripMenuItem.Text.IndexOf("sticky") == -1)
            {
                Switch_To_Long_Note();
                long_note = true;
                switchToLongNoteToolStripMenuItem.Text = "Switch to sticky note";
            }
            else
            {
                Switch_To_Sticky_Note();
                long_note = false;
                switchToLongNoteToolStripMenuItem.Text = "Switch to long note";
            }
        }

        public void Switch_To_Long_Note()
        {
            string the_backcolor = Globals.ReadINI("calculator", "backcolor", Globals.SkinINI);
            string panel_backcolor = Globals.ReadINI("calculator", "panel_backcolor", Globals.SkinINI);
            string the_forecolor = Globals.ReadINI("calculator", "forecolor", Globals.SkinINI);
            string the_font = Globals.ReadINI("calculator", "font", Globals.SkinINI);
            int the_font_size = Globals.ReadIntFromINI("calculator", "font_size", Globals.SkinINI, 10);
            this.Width = Globals.ReadIntFromINI("calculator", "width", Globals.SkinINI);
            this.Height = Globals.ReadIntFromINI("calculator", "height", Globals.SkinINI);

            this.BackColor = System.Drawing.ColorTranslator.FromHtml(the_backcolor);
            txtNote.BackColor = System.Drawing.ColorTranslator.FromHtml(panel_backcolor);
            txtNote.BackColor = System.Drawing.ColorTranslator.FromHtml(the_backcolor);
            txtNote.ForeColor = System.Drawing.ColorTranslator.FromHtml(the_forecolor);
            txtNote.ForeColor = System.Drawing.ColorTranslator.FromHtml(the_forecolor);
            txtNote.Font = new System.Drawing.Font(the_font, the_font_size, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            lblNote.ForeColor = System.Drawing.ColorTranslator.FromHtml(the_forecolor);

            pictureBox1.Visible = false;
            lblNote.Visible = false;
            txtNote.Visible = true;
            lblTop.ForeColor = Color.Gray;
            lblTop.TextAlign = ContentAlignment.TopLeft;
            lblTop.Text = "drag, right-click, or press F11";
            //lblTop.Height = 11;
            lblTop.Height = 22;
            txtNote.Top = txtNote.Top + 11;
            txtNote.Height = this.Height - 25;
            lblTop.Visible = true;
            lblTop.BackColor = txtNote.BackColor;
            //txtNote.Multiline = true;
            txtNote.ScrollBars = ScrollBars.Vertical;
            txtNote.Focus();
            txtNote.SelectionStart = txtNote.Text.Length;
            txtNote.SelectionLength = 0;
            if (txtNote.Text.Length > 25)
            {
                this.Text = txtNote.Text.Substring(0, 25) + "...";
            }
            else
            {
                this.Text = txtNote.Text;
            }
            keepToolStripMenuItem.Text = "Keep note on top (F11)";
            this.ShowInTaskbar = true;
        }

        public void Switch_To_Sticky_Note()
        {
            string INI_path = NotesDir + "\\" + UniqueID.ToString() + ".ini";
            LoadAppearanceFromPropertiesFile(INI_path);

            pictureBox1.Visible = false;
            lblNote.Visible = false;
            txtNote.Visible = true;
            lblTop.ForeColor = Color.Gray;
            lblTop.Text = "";
            // increasing these values by 50%.
            lblTop.Height = 17;
            txtNote.Top = txtNote.Top - 17;
            txtNote.Height = this.Height + 25;
            lblTop.Visible = true;
            lblTop.BackColor = txtNote.BackColor;
            //txtNote.Multiline = false;
            txtNote.ScrollBars = ScrollBars.None;
            txtNote.Focus();
            txtNote.SelectionStart = txtNote.Text.Length;
            txtNote.SelectionLength = 0;
            if (txtNote.Text.Length > 25)
            {
                this.Text = txtNote.Text.Substring(0, 25) + "...";
            }
            else
            {
                this.Text = txtNote.Text;
            }
            keepToolStripMenuItem.Text = "Keep note on top (F11)";
            this.ShowInTaskbar = false;
        }

        private void txtNote_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.F11)
            {
                if (keepToolStripMenuItem.Checked == false)
                {
                    KeepNoteOnTop(1);
                }
                else
                {
                    KeepNoteOnTop(0);
                }
            }
            if ((e.KeyCode == Keys.L && e.Control))
            {
                if (long_note == true)
                {
                    Switch_To_Sticky_Note();
                    long_note = false;
                }
                else
                {
                    Switch_To_Long_Note();
                    long_note = true;
                }
            }
            if (e.Control && (e.KeyCode == Keys.A))
            {
                if (sender != null)
                    ((TextBox)sender).SelectAll();
                e.SuppressKeyPress = true;
                e.Handled = true;
            }
        }

        private void lblTop_DoubleClick(object sender, EventArgs e)
        {
            if (long_note == false)
            {
                if (txtNote.Visible == true)
                {
                    lblNote.Text = txtNote.Text;
                    FormatLabelLinks();
                    txtNote.Visible = false;
                    lblTop.Visible = false;
                    pictureBox1.Visible = true;
                    lblNote.Visible = true;
                }
                else
                {
                    pictureBox1.Visible = false;
                    lblNote.Visible = false;
                    txtNote.Visible = true;
                    lblTop.Visible = true;
                    txtNote.Focus();
                    txtNote.SelectionStart = 0;
                    txtNote.SelectionLength = txtNote.Text.Length;
                }
            }
        }

        private void pictureBox1_DoubleClick(object sender, EventArgs e)
        {
            // we need to see if the clipboard text changed. This fixes a windows bug that is described here: http://stackoverflow.com/questions/2519587/is-there-any-way-to-disable-the-double-click-to-copy-functionality-of-a-net-l
            string new_clipboard_text = Clipboard.GetText();
            if (new_clipboard_text != clipboard_text)
            {
                // windows changed the clipboard text. undo this.
                try
                {
                    Clipboard.SetText(clipboard_text);
                }
                catch
                {
                }
            }
            pictureBox1.Visible = false;
            lblNote.Visible = false;
            txtNote.Visible = true;
            lblTop.Visible = true;
            txtNote.Focus();
            txtNote.SelectionStart = 0;
            txtNote.SelectionLength = txtNote.Text.Length;
        }
    }
}
