﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace Keybreeze
{
    public partial class frmNoteOptions : Form
    {
        public int note_id;
        private bool override_combobox_change = false;
        public frmNoteOptions()
        {
            InitializeComponent();
        }

        private void frmNoteOptions_Load(object sender, EventArgs e)
        {
            int form_width = Globals.ReadIntFromINI("notes", "width", Globals.NotesINI, 200);
            int form_height = Globals.ReadIntFromINI("notes", "height", Globals.NotesINI, 200);
            int display_on_hotkey_press = Globals.ReadIntFromINI("notes", "display_on_hotkey_press", Globals.NotesINI, 1);
            int display_at_startup = Globals.ReadIntFromINI("notes", "display_at_startup", Globals.NotesINI, 1);
            string font = Globals.ReadINI("notes", "font", Globals.NotesINI);
            int font_size = Globals.ReadIntFromINI("notes", "size", Globals.NotesINI, 10);
            int font_bold = Globals.ReadIntFromINI("notes", "bold", Globals.NotesINI, 0);
            int font_italic = Globals.ReadIntFromINI("notes", "italic", Globals.NotesINI, 0);
            int font_underline = Globals.ReadIntFromINI("notes", "underline", Globals.NotesINI, 0);
            string textcolor = Globals.ReadINI("notes", "textcolor", Globals.NotesINI);
            string note_background_image = "";
            string theme_for_this_note = "";
            string note_html;
            string note_image;
            int image_location_within_dropdown;
            int counter = 0;

            override_combobox_change = true;
            foreach (Note n in Globals.notelist)
            {
                if (n.frmNote_instance.UniqueID == note_id)
                {
                    txtWidth.Text = n.frmNote_instance.Width.ToString();
                    txtHeight.Text = n.frmNote_instance.Height.ToString();
                    cmbFont.Items.Add(n.frmNote_instance.txtNote.Font.FontFamily.GetName(0));
                    font_size = Convert.ToInt32(n.frmNote_instance.txtNote.Font.Size);
                    for (int i = 8; i <= 36; i++)
                    {
                        if (i == font_size)
                            cmbSize.Items.Add(i);
                        else if (i != 13 && i != 15 && i != 17 && i != 19 && i != 20 && i != 21 && i != 22 && i != 23 && i != 25 && i != 26 && i != 27 && i != 28 && i != 29 && i != 30 && i != 31 && i != 32 && i != 33 && i != 34 && i != 35 && i < 37)
                            cmbSize.Items.Add(i);
                    }
                    cmbSize.SelectedItem = font_size;
                    if (n.frmNote_instance.txtNote.Font.Bold == true)
                    {
                        chkBold.Checked = true;
                    }
                    if (n.frmNote_instance.txtNote.Font.Italic == true)
                    {
                        chkItalic.Checked = true;
                    }
                    if (n.frmNote_instance.txtNote.Font.Underline == true)
                    {
                        chkUnderline.Checked = true;
                    }
                    lblTextColor.BackColor = n.frmNote_instance.txtNote.ForeColor;
                    lblBackColor.BackColor = n.frmNote_instance.txtNote.BackColor;
                    note_background_image = n.frmNote_instance.note_image;

                    break;
                }
                counter++;
            }

            // add fonts to combobox
            foreach (System.Drawing.FontFamily f in System.Drawing.FontFamily.Families)
            {
                Item item = new Item(f.Name, f.Name);
                cmbFont.Items.Add(item);
            }
            cmbFont.SelectedIndex = 0;

            // add color themes and templates to dropdown boxes.
            theme_for_this_note = LoadColorThemes(note_background_image);
            cmbColorTheme.SelectedItem = theme_for_this_note;
            for (int i = 1; i <= 20; i++)
            {
                note_html = Globals.ReadINI(theme_for_this_note, i + "html", Globals.NotesINI);
                note_image = Globals.ReadINI(theme_for_this_note, i + "image", Globals.NotesINI);
                if (note_html.Length == 0 || note_image.Length == 0)
                {
                    break;
                }
                cmbBackColor.Items.Add(note_image);
                if (note_image == note_background_image)
                    image_location_within_dropdown = i;
            }
            cmbBackColor.Items.Add("[custom]");
            cmbBackColor.SelectedItem = note_background_image;
            override_combobox_change = false;

            if (display_on_hotkey_press == 1)
                chkDisplayOnHotkey.Checked = true;
            if (display_at_startup == 1)
                chkDisplayAtStartup.Checked = true;
        }

        private void button2_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void chkDisplayNotesAtStartup_CheckedChanged(object sender, EventArgs e)
        {

        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {

        }

        private void checkBox2_CheckedChanged(object sender, EventArgs e)
        {

        }

        private string LoadColorThemes(string note_background_image)
        {
            // this will load all the color themes from the INI file to the application window.
            // it will also return the theme that includes the specific image for the note.

            string INI_Text;
            int find_start_bracket;
            int find_end_bracket;
            int last_pos = 10;
            string current_theme = "";
            int current_image_position;
            string theme_for_current_note = "";

            cmbColorTheme.Items.Clear();
            try
            {
                INI_Text = File.ReadAllText(Globals.NotesINI);
            }
            catch
            {
                return "";
            }
            // get the character position of the current image. We need to find out what theme it's in.
            current_image_position = INI_Text.IndexOf(note_background_image);
            for (int i = 1; i <= 50; i++)
            {
                if (last_pos > INI_Text.Length)
                {
                    break;
                }
                find_start_bracket = INI_Text.IndexOf("[", last_pos);
                if (find_start_bracket > current_image_position || find_start_bracket == -1 && theme_for_current_note == "")
                {
                    // we passed the theme where the image is located. use the previous theme.
                    theme_for_current_note = current_theme;
                }
                if (find_start_bracket > -1)
                {
                    // found a theme
                    find_end_bracket = INI_Text.IndexOf("]", find_start_bracket + 1);
                    current_theme = INI_Text.Substring(find_start_bracket + 1, find_end_bracket - find_start_bracket - 1);
                    cmbColorTheme.Items.Add(current_theme);
                    last_pos = find_end_bracket + 2;
                }
                else
                    break;
            }
            return theme_for_current_note;
        }

        // variables for loading fonts
        private class Item
        {
            public string Name;
            public string Value;
            public Item(string name, string value)
            {
                Name = name; Value = value;
            }
            public override string ToString()
            {
                return Name;
            }
        }

        private void label6_Click(object sender, EventArgs e)
        {

        }

        private void lblBackColor_Click(object sender, EventArgs e)
        {
            string old_color = ColorTranslator.ToHtml(lblBackColor.BackColor);
            colorDialog1.Color = lblBackColor.BackColor;
            colorDialog1.ShowDialog();
            lblBackColor.BackColor = colorDialog1.Color;
            override_combobox_change = true;
            string new_color = ColorTranslator.ToHtml(lblBackColor.BackColor);
            if (new_color != old_color)
                cmbBackColor.SelectedItem = "[custom]";
            override_combobox_change = false;
        }

        private void lblTextColor_Click(object sender, EventArgs e)
        {
            colorDialog1.Color = lblTextColor.BackColor;
            colorDialog1.ShowDialog();
            lblTextColor.BackColor = colorDialog1.Color;
        }

        private void cmbBackColor_SelectedIndexChanged(object sender, EventArgs e)
        {
            string note_html;
            string note_image;
            string current_selection = cmbBackColor.SelectedItem.ToString();
            string theme_for_this_note = cmbColorTheme.SelectedItem.ToString();

            if (override_combobox_change == false)
            {
                if (current_selection == "[custom]")
                {
                    colorDialog1.ShowDialog();
                    lblBackColor.BackColor = colorDialog1.Color;
                }
                else
                {
                    for (int i = 1; i <= 20; i++)
                    {
                        note_image = Globals.ReadINI(theme_for_this_note, i + "image", Globals.NotesINI);
                        if (note_image.Length == 0)
                            break;
                        else if (note_image == current_selection)
                        {
                            note_html = Globals.ReadINI(theme_for_this_note, i + "html", Globals.NotesINI);
                            lblBackColor.BackColor = System.Drawing.ColorTranslator.FromHtml(note_html);
                            break;
                        }
                    }
                }
            }
        }

        private void cmbColorTheme_SelectedIndexChanged(object sender, EventArgs e)
        {
            string note_image;
            string note_html;
            string current_theme = cmbColorTheme.SelectedItem.ToString();

            if (override_combobox_change == false)
            {
                cmbBackColor.Items.Clear();
                for (int i = 1; i <= 20; i++)
                {
                    note_image = Globals.ReadINI(current_theme, i + "image", Globals.NotesINI);
                    if (note_image.Length == 0)
                        break;
                    note_html = Globals.ReadINI(current_theme, i + "html", Globals.NotesINI);
                    cmbBackColor.Items.Add(note_image);
                    if (i == 1)
                    {
                        lblBackColor.BackColor = System.Drawing.ColorTranslator.FromHtml(note_html);
                        cmbBackColor.SelectedIndex = 0;
                    }
                }
                cmbBackColor.Items.Add("[custom]");
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            DialogResult result;
            result = MessageBox.Show("Are you sure you want to reset the default settings for all notes? This will only affect the appearance of notes. It will not change the text of your existing notes.", "Keybreeze Desktop", MessageBoxButtons.YesNo);
            if (result == DialogResult.Yes)
            {
                int counter = 0;
                foreach (string i in cmbFont.Items)
                {
                    if (i == "Arial")
                    {
                        cmbFont.SelectedIndex = counter;
                        break;
                    }
                    counter++;
                }
                cmbFont.SelectedItem = "Arial";
                cmbSize.SelectedItem = "11";
                txtWidth.Text = "174";
                txtHeight.Text = "130";
                chkBold.Checked = false;
                chkItalic.Checked = false;
                chkUnderline.Checked = false;
                lblTextColor.BackColor = Color.Black;
                lblBackColor.BackColor = System.Drawing.ColorTranslator.FromHtml("#8DCBFA");
                cmbColorTheme.SelectedItem = "light colors";
                Application.DoEvents();
                cmbBackColor.SelectedItem = "bluegradient.jpg";
                chkDisplayOnHotkey.Checked = true;
                chkDisplayAtStartup.Checked = true;
                chkApplyToAllNotes.Checked = true;
                // set color settings in the INI. Everything else should be set when the user closes the window.
                Globals.WriteINI("notes", "textcolor", "#000000", Globals.NotesINI);
                Globals.WriteINI("notes", "backcolor", "", Globals.NotesINI);
            }
        }

        private void frmNoteOptions_FormClosing(object sender, FormClosingEventArgs e)
        {
            int note_width;
            int note_height;
            if (!int.TryParse(txtWidth.Text, out note_width))
            {
                MessageBox.Show("Please type a number for the width.", "Keybreeze Desktop");
                e.Cancel = true;
                return;
            }
            if (!int.TryParse(txtHeight.Text, out note_height))
            {
                MessageBox.Show("Please type a number for the height.", "Keybreeze Desktop");
                e.Cancel = true;
                return;
            }

            string note_backcolor = ColorTranslator.ToHtml(lblBackColor.BackColor);
            string note_font = cmbFont.SelectedItem.ToString();
            double note_font_size = Convert.ToDouble(cmbSize.SelectedItem);
            Globals.note_theme = cmbColorTheme.SelectedItem.ToString();

            // update the current note first.
            foreach (Note n in Globals.notelist)
            {
                if (n.frmNote_instance.UniqueID == note_id)
                {
                    n.frmNote_instance.Width = note_width;
                    n.frmNote_instance.Height = note_height;
                    try
                    {
                        // set the fontstyle, bold, italic, underline, etc.
                        FontStyle myfontstyle = FontStyle.Regular;
                        if (chkBold.Checked == true)
                            myfontstyle = myfontstyle | FontStyle.Bold;
                        if (chkItalic.Checked == true)
                            myfontstyle = myfontstyle | FontStyle.Italic;
                        if (chkUnderline.Checked == true)
                            myfontstyle = myfontstyle | FontStyle.Underline;

                        float note_font_size_float = (float)note_font_size;
                        n.frmNote_instance.txtNote.Font = new System.Drawing.Font(note_font, note_font_size_float, myfontstyle, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
                        n.frmNote_instance.lblNote.Font = new System.Drawing.Font(note_font, note_font_size_float, myfontstyle, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
                        n.frmNote_instance.note_image = cmbBackColor.SelectedItem.ToString();
                        n.frmNote_instance.note_html = note_backcolor;
                    }
                    catch
                    {
                    }
                    n.frmNote_instance.txtNote.ForeColor = lblTextColor.BackColor;
                    n.frmNote_instance.lblNote.ForeColor = lblTextColor.BackColor;
                    n.frmNote_instance.BackColor = lblBackColor.BackColor;
                    n.frmNote_instance.txtNote.BackColor = lblBackColor.BackColor;
                    n.frmNote_instance.lblTop.BackColor = lblBackColor.BackColor;
                    if (cmbBackColor.SelectedItem.ToString() == "[custom]")
                    {
                        n.frmNote_instance.pictureBox1.Image = null;
                        n.frmNote_instance.pictureBox1.BackColor = lblBackColor.BackColor;
                    }
                    else
                    {
                        try
                        {
                            n.frmNote_instance.pictureBox1.Image = Image.FromFile(Globals.WorkPath + "\\notes\\themes\\" + Globals.note_theme + "\\" + cmbBackColor.SelectedItem.ToString());
                        }
                        catch
                        {
                        }
                    }
                    break;
                }
            }
            // save the note settings back to the properties file.
            string note_properties_path = Globals.WorkPath + "\\notes\\" + note_id.ToString() + ".ini";
            Globals.WriteINI("properties", "width", note_width.ToString(), note_properties_path);
            Globals.WriteINI("properties", "height", note_height.ToString(), note_properties_path);
            Globals.WriteINI("properties", "backcolor", note_backcolor, note_properties_path);
            Globals.WriteINI("properties", "image", cmbBackColor.SelectedItem.ToString(), note_properties_path);
            string note_forecolor = ColorTranslator.ToHtml(lblTextColor.BackColor);
            Globals.WriteINI("properties", "textcolor", note_forecolor, note_properties_path);
            Globals.WriteINI("properties", "font", note_font, note_properties_path);
            Globals.WriteINI("properties", "size", note_font_size.ToString(), note_properties_path);
            if (chkBold.Checked == true)
                Globals.WriteINI("properties", "bold", "1", note_properties_path);
            else
                Globals.WriteINI("properties", "bold", "0", note_properties_path);
            if (chkItalic.Checked == true)
                Globals.WriteINI("properties", "italic", "1", note_properties_path);
            else
                Globals.WriteINI("properties", "italic", "0", note_properties_path);
            if (chkUnderline.Checked == true)
                Globals.WriteINI("properties", "underline", "1", note_properties_path);
            else
                Globals.WriteINI("properties", "underline", "0", note_properties_path);


            if (chkApplyToAllNotes.Checked == true)
            {
                // update all future notes.
                Globals.WriteINI("notes", "width", txtWidth.Text, Globals.NotesINI);
                Globals.WriteINI("notes", "height", txtHeight.Text, Globals.NotesINI);
                Globals.WriteINI("notes", "font", cmbFont.SelectedItem.ToString(), Globals.NotesINI);
                Globals.WriteINI("notes", "size", cmbSize.SelectedItem.ToString(), Globals.NotesINI);
                if (chkBold.Checked == true)
                    Globals.WriteINI("notes", "bold", "1", Globals.NotesINI);
                else
                    Globals.WriteINI("notes", "bold", "0", Globals.NotesINI);
                if (chkItalic.Checked == true)
                    Globals.WriteINI("notes", "italic", "1", Globals.NotesINI);
                else
                    Globals.WriteINI("notes", "italic", "0", Globals.NotesINI);
                if (chkUnderline.Checked == true)
                    Globals.WriteINI("notes", "underline", "1", Globals.NotesINI);
                else
                    Globals.WriteINI("notes", "underline", "0", Globals.NotesINI);
            }

            if (chkDisplayAtStartup.Checked == true)
                Globals.WriteINI("notes", "display_at_startup", "1", Globals.NotesINI);
            else
                Globals.WriteINI("notes", "display_at_startup", "0", Globals.NotesINI);
            if (chkDisplayOnHotkey.Checked == true)
                Globals.WriteINI("notes", "display_on_hotkey_press", "1", Globals.NotesINI);
            else
                Globals.WriteINI("notes", "display_on_hotkey_press", "0", Globals.NotesINI);
        }
    }
}
