﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace Keybreeze
{
    public partial class frmNoteReminders : Form
    {
        public frmNoteReminders()
        {
            InitializeComponent();
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void frmNoteReminders_Load(object sender, EventArgs e)
        {
            int the_type;
            string note_id;
            string note_text;
            string RemindersINI = Globals.WorkPath + "\\notes\\settings\\reminders.ini";
            for (int i = 1; i <= 200; i++)
            {
                the_type = Globals.ReadIntFromINI("reminders", i + "-type", RemindersINI);
                if (the_type == 0)
                {
                    break;
                }
                else if (the_type == 1)
                {
                    // one time reminder
                    note_id = Globals.ReadINI("reminders", i + "-noteid", RemindersINI);
                    note_text = read_note_text(note_id);
                    listBox1.Items.Add("One time, " + ReturnDoubleDigitTime(i + "-year") + "-" + ReturnDoubleDigitTime(i + "-month") + "-" + ReturnDoubleDigitTime(i + "-dayofmonth") + ", " + ReturnDoubleDigitTime(i + "-timehour") + ":" + ReturnDoubleDigitTime(i + "-timeminute") + ": " + note_text);
                }
                else if (the_type == 2)
                {
                    // daily reminder
                    note_id = Globals.ReadINI("reminders", i + "-noteid", RemindersINI);
                    note_text = read_note_text(note_id);
                    listBox1.Items.Add("Daily, " + ReturnDoubleDigitTime(i + "-timehour") + ":" + ReturnDoubleDigitTime(i + "-timeminute") + ": " + note_text);
                }
                else if (the_type == 3)
                {
                    // weekly reminder
                    note_id = Globals.ReadINI("reminders", i + "-noteid", RemindersINI);
                    note_text = read_note_text(note_id);
                    listBox1.Items.Add("Weekly, on " + GetWeekDayFromNum(Globals.ReadIntFromINI("reminders", i + "-day", RemindersINI)) + ", " + ReturnDoubleDigitTime(i + "-timehour") + ":" + ReturnDoubleDigitTime(i + "-timeminute") + ": " + note_text);
                }
                else if (the_type == 4)
                {
                    // monthly reminder
                    note_id = Globals.ReadINI("reminders", i + "-noteid", RemindersINI);
                    note_text = read_note_text(note_id);
                    listBox1.Items.Add("Monthly, on day " + ReturnDoubleDigitTime(i + "-dayofmonth") + ", " + ReturnDoubleDigitTime(i + "-timehour") + ":" + ReturnDoubleDigitTime(i + "-timeminute") + ": " + note_text);
                }
            }
        }

        private string ReturnDoubleDigitTime(string input)
        {
            // reads the INI file and returns two digits for the hours/minutes/etc if only one digit is present.
            string RemindersINI = Globals.WorkPath + "\\notes\\settings\\reminders.ini";
            string this_value = Globals.ReadINI("reminders", input, RemindersINI);
            if (this_value.Length == 1)
                return "0" + this_value;
            else
                return this_value;
        }

        private string GetWeekDayFromNum(int day_num)
        {
            switch (day_num)
            {
                case 1:
                    return "Sunday";
                case 2:
                    return "Monday";
                case 3:
                    return "Tuesday";
                case 4:
                    return "Wednesday";
                case 5:
                    return "Thursday";
                case 6:
                    return "Friday";
                case 7:
                    return "Saturday";
                default:
                    return "";
            }
        }

        private string read_note_text(string note_id)
        {
            string filePath = Globals.WorkPath + "\\notes\\" + note_id + ".txt";
            string note_full_text = "";
            try
            {
                note_full_text = File.ReadAllText(filePath);
            }
            catch
            {
                note_full_text = "[cannot read file: " + note_id + ".txt" + "]";
            }
            if (note_full_text.Length > 30)
                note_full_text = note_full_text.Substring(0, 30);
            return note_full_text;
        }

        private void btnDelete_Click(object sender, EventArgs e)
        {
            string RemindersINI = Globals.WorkPath + "\\notes\\settings\\reminders.ini";

            if (listBox1.SelectedIndex > -1)
            {
                try
                {
                    int item_to_remove = listBox1.SelectedIndex + 1;
                    string note_id = Globals.ReadINI("reminders", item_to_remove + "-noteid", RemindersINI);
                    int hidden_reminder = Globals.ReadIntFromINI("properties", "hiddenreminder", Globals.WorkPath + "\\notes\\" + note_id + ".ini");
                    if (hidden_reminder == 1)
                    {
                        // this is a reminder without a note (the note gets created when the reminder pops up), so let's delete the hidden note.
                        System.IO.File.Delete(Globals.WorkPath + "\\notes\\" + note_id + ".txt");
                        System.IO.File.Delete(Globals.WorkPath + "\\notes\\" + note_id + ".ini");
                    }
                    Globals.WriteINI("reminders", item_to_remove + "-type", null, RemindersINI);
                    Globals.WriteINI("reminders", item_to_remove + "-year", null, RemindersINI);
                    Globals.WriteINI("reminders", item_to_remove + "-month", null, RemindersINI);
                    Globals.WriteINI("reminders", item_to_remove + "-dayofmonth", null, RemindersINI);
                    Globals.WriteINI("reminders", item_to_remove + "-timehour", null, RemindersINI);
                    Globals.WriteINI("reminders", item_to_remove + "-timeminute", null, RemindersINI);
                    Globals.WriteINI("reminders", item_to_remove + "-day", null, RemindersINI);
                    Globals.WriteINI("reminders", item_to_remove + "-lastran_month", null, RemindersINI);
                    Globals.WriteINI("reminders", item_to_remove + "-lastran_day", null, RemindersINI);
                    Globals.WriteINI("reminders", item_to_remove + "-lastran_dayofmonth", null, RemindersINI);
                    Globals.WriteINI("reminders", item_to_remove + "-lastran_hour", null, RemindersINI);
                    Globals.WriteINI("reminders", item_to_remove + "-lastran_minute", null, RemindersINI);
                    Globals.WriteINI("reminders", item_to_remove + "-noteid", null, RemindersINI);
                    Globals.Reorganize_reminders();
                    listBox1.Items.Remove(listBox1.SelectedItem);
                }
                catch
                {
                    MessageBox.Show("The reminder could not be deleted. The files in the " + Globals.WorkPath + " directory may be locked by Windows. Please try again.", "Keybreeze Desktop");
                }
            }
            else
            {
                MessageBox.Show("Please select the item you want to delete.", "Keybreeze Desktop");
            }
        }
    }
}
