﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Microsoft.Win32;
using System.IO;
using System.Net.NetworkInformation;
using System.Diagnostics;

namespace Keybreeze
{
    public partial class frmOptions : Form
    {
        frmMain frmMain_instance;
        bool changed_startup_setting = false;
        bool hotkey_modified = false;
        bool finished_loading = false;
        public bool start_with_autoindex_tab = false;
        public bool start_with_search_tab = false;
        public bool start_with_clipboard_tab = false;
        public string mac_address = "";

        public List<autoindex_entry> autoindex_entries = new List<autoindex_entry>();
        private List<search_entry> search_entries = new List<search_entry>();
        
        public frmOptions(frmMain parent)
        {
            InitializeComponent();
            frmMain_instance = parent;
        }

        private void frmOptions_Load(object sender, EventArgs e)
        {
            // GENERAL TAB
            string extracted = Globals.ReadINI("Extract", "Extract_Not_Install", Globals.InstallDir + "\\default settings\\options.ini");
            if (extracted != "1")
            {
                // we are using the installed version of Keybreeze Desktop, so check the registry.
                try
                {
                    RegistryKey rkApp = Registry.CurrentUser.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Run", true);
                    string RegKey = (string)rkApp.GetValue("Keybreeze5", "");
                    if (RegKey.Length > 0)
                        chkLoadAtStartup.Checked = true;
                    else
                        chkLoadAtStartup.Checked = false;
                    if (Globals.ReadIntFromINI("options", "default_browser", Globals.OptionsINI, 1) == 1)
                        chkDefaultWebBrowser.Checked = true;
                    else
                    {
                        chkDefaultWebBrowser.Checked = false;
                        edit_settings_1.Visible = true;
                    }
                    if (Globals.ReadIntFromINI("options", "windows_explorer", Globals.OptionsINI, 1) == 1)
                    {
                        chkWindowsExplorer.Checked = true;
                    }
                    else
                    {
                        chkWindowsExplorer.Checked = false;
                        edit_settings_2.Visible = true;
                    }
                }
                catch
                {
                    // we probably don't have registry access if we are here.
                    chkLoadAtStartup.Checked = false;
                }
            }
            else
            {
                // we are using the extracted version of keybreeze. don't check the registry.
                chkLoadAtStartup.Checked = false;
                chkLoadAtStartup.Enabled = false;
            }

            txtSearchURL.Text = Globals.ReadINI("options", "search_url", Globals.OptionsINI);
            int show_keyboard_help = Globals.ReadIntFromINI("options", "show_keyboard_help", Globals.OptionsINI, 1);
            if (show_keyboard_help == 1)
                chkShowKeyboardHelp.Checked = true;
            else
                chkShowKeyboardHelp.Checked = false;
            int when_pressing_shift = Globals.ReadIntFromINI("options", "when_pressing_shift", Globals.OptionsINI, 2);
            int when_pressing_tab = Globals.ReadIntFromINI("options", "when_pressing_tab", Globals.OptionsINI, 1);
            int when_pressing_ctrl = Globals.ReadIntFromINI("options", "when_pressing_ctrl", Globals.OptionsINI, 3);
            int when_pressing_enter_unrecognized_entry = Globals.ReadIntFromINI("options", "when_pressing_enter_unrecognized_entry", Globals.OptionsINI, 2);

            cmbShift.Items.Add("Do nothing");
            cmbShift.Items.Add("Move selection down");
            cmbShift.Items.Add("Search computer for entry");
            cmbShift.Items.Add("Search the web for entry");

            cmbTab.Items.Add("Do nothing");
            cmbTab.Items.Add("Move selection down");
            cmbTab.Items.Add("Search computer for entry");
            cmbTab.Items.Add("Search the web for entry");

            cmbCTRL.Items.Add("Do nothing");
            cmbCTRL.Items.Add("Move selection down");
            cmbCTRL.Items.Add("Search computer for entry");
            cmbCTRL.Items.Add("Search the web for entry");

            cmbEnterUnrecognizedCommand.Items.Add("Show error");
            cmbEnterUnrecognizedCommand.Items.Add("Search computer for entry");
            cmbEnterUnrecognizedCommand.Items.Add("Search the web for entry");

            if (when_pressing_shift == 0)
                cmbShift.SelectedIndex = 0;
            else if (when_pressing_shift == 1)
                cmbShift.SelectedIndex = 1;
            else if (when_pressing_shift == 2)
                cmbShift.SelectedIndex = 2;
            else
                cmbShift.SelectedIndex = 3;

            if (when_pressing_tab == 0)
                cmbTab.SelectedIndex = 0;
            else if (when_pressing_tab == 1)
                cmbTab.SelectedIndex = 1;
            else if (when_pressing_tab == 2)
                cmbTab.SelectedIndex = 2;
            else
                cmbTab.SelectedIndex = 3;

            if (when_pressing_ctrl == 0)
                cmbCTRL.SelectedIndex = 0;
            else if (when_pressing_ctrl == 1)
                cmbCTRL.SelectedIndex = 1;
            else if (when_pressing_ctrl == 2)
                cmbCTRL.SelectedIndex = 2;
            else
                cmbCTRL.SelectedIndex = 3;

            if (when_pressing_enter_unrecognized_entry == 0)
                cmbEnterUnrecognizedCommand.SelectedIndex = 0;
            else if (when_pressing_enter_unrecognized_entry == 1)
                cmbEnterUnrecognizedCommand.SelectedIndex = 1;
            else
                cmbEnterUnrecognizedCommand.SelectedIndex = 2;
            


            txtAutocompleteLimit.Text = Globals.ReadINI("options", "autocomplete_char_limit", Globals.OptionsINI);
            
            // HOTKEY TAB
            chkModifiers.Items.Add("None");
            chkModifiers.Items.Add("Shift +");
            chkModifiers.Items.Add("Ctrl +");
            chkModifiers.Items.Add("Alt +");
            chkModifiers.Items.Add("Windows key +");
            chkModifiers.Items.Add("Ctrl + Alt +");
            chkModifiers.Items.Add("Shift + Ctrl +");
            chkModifiers.Items.Add("Shift + Alt +");
            int hotkey_ctrl = Globals.ReadIntFromINI("hotkey", "hotkey_ctrl", Globals.OptionsINI, 0);
            int hotkey_alt = Globals.ReadIntFromINI("hotkey", "hotkey_alt", Globals.OptionsINI, 0);
            int hotkey_shift = Globals.ReadIntFromINI("hotkey", "hotkey_shift", Globals.OptionsINI, 0);
            int hotkey_windows = Globals.ReadIntFromINI("hotkey", "hotkey_windows", Globals.OptionsINI, 0);
            if (hotkey_shift == 0 && hotkey_ctrl == 0 && hotkey_alt == 0 && hotkey_windows == 0)
            {
                chkModifiers.SelectedIndex = 0;
            }
            else if (hotkey_shift == 1 && hotkey_ctrl == 0 && hotkey_alt == 0 && hotkey_windows == 0)
            {
                chkModifiers.SelectedIndex = 1;
            }
            else if (hotkey_shift == 0 && hotkey_ctrl == 1 && hotkey_alt == 0 && hotkey_windows == 0)
            {
                chkModifiers.SelectedIndex = 2;
            }
            else if (hotkey_shift == 0 && hotkey_ctrl == 0 && hotkey_alt == 1 && hotkey_windows == 0)
            {
                chkModifiers.SelectedIndex = 3;
            }
            else if (hotkey_shift == 0 && hotkey_ctrl == 0 && hotkey_alt == 0 && hotkey_windows == 1)
            {
                chkModifiers.SelectedIndex = 4;
            }
            else if (hotkey_shift == 0 && hotkey_ctrl == 1 && hotkey_alt == 1 && hotkey_windows == 0)
            {
                chkModifiers.SelectedIndex = 5;
            }
            else if (hotkey_shift == 1 && hotkey_ctrl == 1 && hotkey_alt == 0 && hotkey_windows == 0)
            {
                chkModifiers.SelectedIndex = 6;
            }
            else if (hotkey_shift == 1 && hotkey_ctrl == 0 && hotkey_alt == 1 && hotkey_windows == 0)
            {
                chkModifiers.SelectedIndex = 7;
            }
            else
                chkModifiers.SelectedIndex = 0;

            txtHotkey.Text = Globals.ReadINI("hotkey", "hotkey", Globals.OptionsINI);
            txtHotkey.Tag = Globals.ReadINI("hotkey", "hotkey_ascii_code", Globals.OptionsINI);

            // SKINS
            txtCurrentSkin.Text = Globals.ReadINI("options", "skin", Globals.OptionsINI);

            string skin_directory = Globals.WorkPath + "\\skins";
            string[] subdirectoryEntries = Directory.GetDirectories(skin_directory);
            string lowest_level_directory;
            int counter1 = 0;
            foreach (string subdirectory in subdirectoryEntries)
            {
                lowest_level_directory = Path.GetFileName(subdirectory);
                lstSkins.Items.Add(lowest_level_directory);
                if (txtCurrentSkin.Text == lowest_level_directory)
                    lstSkins.SelectedIndex = counter1;
                counter1++;
            }
            if (Globals.ReadIntFromINI("options", "always_center_keybreeze", Globals.OptionsINI, 1) == 1)
                rdoSkinAlwaysCenter.Checked = true;
            else
                rdoSkinRememberPosition.Checked = true;

            Load_Autoindex_Tab();
            Load_Search_Tab();

            // QuickTab tab
            // INI values:
            // 0 = nothing
            // 1 = insert hotkey
            // 2 = insert hotkey + space
            // 3 = insert hotkey + enter
            // 4 = insert shift
            // 5 = insert ctrl
            int quicktab_enabled = Globals.ReadIntFromINI("quicktab", "double_letter_enabled", Globals.OptionsINI, 1);
            if (quicktab_enabled == 1)
            {
                chkDoubleLetter.Checked = true;
                btnConfigureQuickTabWindowMappings.Enabled = true;
            }
            else
            {
                chkDoubleLetter.Checked = false;
                btnConfigureQuickTabWindowMappings.Enabled = false;
            }
            lblQuickTabSummary.Text = "QuickTab includes a variety of ways to control windows on your screen. You can press the hotkey on your keyboard, type a few letters that appear in a window's title, and press the hotkey again to set focus to that window. You can also press the hotkey and type first letter of the window caption twice. This will generally be faster than using Alt + Tab if you need to navigate between several windows.";
            lstQuickTab.Items.Add("hotkey + hotkey");
            lstQuickTab.Items.Add("hotkey + space");
            lstQuickTab.Items.Add("hotkey + enter");
            lstQuickTab.Items.Add("hotkey + shift");
            lstQuickTab.Items.Add("hotkey + ctrl");
            lstQuickTab.Items.Add("hotkey + tab");
            lstQuickTab.Items.Add("hotkey + F1");
            lstQuickTab.Items.Add("hotkey + F2");
            lstQuickTab.Items.Add("hotkey + F3");
            lstQuickTab.Items.Add("hotkey + F4");
            lstQuickTab.Items.Add("hotkey + F5");
            lstQuickTab.Items.Add("hotkey + F6");
            lstQuickTab.Items.Add("hotkey + F7");
            lstQuickTab.Items.Add("hotkey + F8");
            lstQuickTab.Items.Add("hotkey + F9");
            lstQuickTab.Items.Add("hotkey + F10");
            lstQuickTab.Items.Add("hotkey + F11");
            lstQuickTab.Items.Add("hotkey + F12");
            lstQuickTab.SelectedIndex = 0;
            cmbQuickTab.Items.Add("Nothing");
            cmbQuickTab.Items.Add("Insert hotkey");
            cmbQuickTab.Items.Add("Insert hotkey plus space");
            cmbQuickTab.Items.Add("Insert hotkey plus enter");
            cmbQuickTab.Items.Add("Minimize window");
            cmbQuickTab.Items.Add("Maximize window");
            cmbQuickTab.Items.Add("Close window");
            cmbQuickTab.Items.Add("Equivalent to Alt + Tab");
            cmbQuickTab.Items.Add("Position window");
            cmbQuickTab.Items.Add("Show help page");
            int hotkey_plus_hotkey = Globals.ReadIntFromINI("quicktab", "hotkey_plus_hotkey", Globals.OptionsINI, 1);
            cmbQuickTab.SelectedIndex = hotkey_plus_hotkey;
            
            // CLIPBOARD TAB
            int clipboard_enabled = Globals.ReadIntFromINI("clipboard", "enabled", Globals.OptionsINI, 1);
            int clipboard_item_max_length = Globals.ReadIntFromINI("clipboard", "clipboard_item_max_length", Globals.OptionsINI, 100000);
            int clipboard_max_count = Globals.ReadIntFromINI("clipboard", "clipboard_max_count", Globals.OptionsINI, 50);
            int clipboard_interval = Globals.ReadIntFromINI("clipboard", "interval", Globals.OptionsINI, 1000);
            int clipboard_tab_toggle = Globals.ReadIntFromINI("clipboard", "tab_to_toggle", Globals.OptionsINI, 0);
            int clipboard_clear_history = Globals.ReadIntFromINI("clipboard", "clear_history_when_clearing_clipboard", Globals.OptionsINI, 1);
            int clipboard_password_detection = Globals.ReadIntFromINI("clipboard", "clipboard_password_detection", Globals.OptionsINI, 2);
            int clipboard_password_detection_start = Globals.ReadIntFromINI("clipboard", "clipboard_password_detection_start", Globals.OptionsINI, 4);
            int clipboard_password_detection_end = Globals.ReadIntFromINI("clipboard", "clipboard_password_detection_end", Globals.OptionsINI, 30);
            if (clipboard_enabled == 1)
                chkEnableClipboardMonitoring.Checked = true;
            else
                chkEnableClipboardMonitoring.Checked = false;
            if (clipboard_item_max_length < 100 || clipboard_item_max_length > 1000000)
            {
                clipboard_item_max_length = 100000;
            }
            if (clipboard_max_count < 1 || clipboard_max_count > 250)
            {
                clipboard_max_count = 50;
            }
            if (clipboard_interval < 500 || clipboard_interval > 30000)
            {
                clipboard_interval = 1000;
            }
            if (clipboard_tab_toggle == 0)
                chkUseTabKeyToToggleRows.Checked = false;
            else
                chkUseTabKeyToToggleRows.Checked = true;
            if (clipboard_clear_history == 1)
                chkClearHistoryClearClipboard.Checked = true;
            else
                chkClearHistoryClearClipboard.Checked = false;
            txtClipboardMonitorInterval.Text = clipboard_interval.ToString();
            txtMaximumItemsForClipboard.Text = clipboard_max_count.ToString();
            txtMaximumTextLengthPerItem.Text = clipboard_item_max_length.ToString();
            cmbClipboardMonitorPasswordSetting.Items.Add("Do not hide anything");
            cmbClipboardMonitorPasswordSetting.Items.Add("Hide entries between " + clipboard_password_detection_start + " and " + clipboard_password_detection_end + " characters");
            cmbClipboardMonitorPasswordSetting.Items.Add("Hide entries between " + clipboard_password_detection_start + " and " + clipboard_password_detection_end + " characters and contain letters and numbers");
            cmbClipboardMonitorPasswordSetting.Items.Add("Hide entries between " + clipboard_password_detection_start + " and " + clipboard_password_detection_end + " characters and contain letters, numbers, and no spaces");
            cmbClipboardMonitorPasswordSetting.SelectedIndex = clipboard_password_detection;
            finished_loading = true;
        }

        private void Load_Autoindex_Tab()
        {
            // autoindex tab
            string autoindex_file = Globals.WorkPath + "\\autoindex\\indexoptions.ini";
            int refresh_interval = Globals.ReadIntFromINI("settings", "refresh_interval_minutes", autoindex_file, 10);
            if (refresh_interval > 60 || refresh_interval < 0)
                refresh_interval = 10;
            txtRefreshInterval.Text = refresh_interval.ToString();
            string item_path = "";
            int indexondemand;
            int subfolders;
            int item_checked;
            string filetypes;
            int toplevelfolders;
            for (int i = 1; i <= 250; i++)
            {
                item_path = Globals.ReadINI("locations", "item" + i + "-path", autoindex_file);
                if (item_path.Length > 0)
                {
                    lstAutoIndex.Items.Add(item_path);
                    indexondemand = Globals.ReadIntFromINI("locations", "item" + i + "-indexondemand", autoindex_file, 0);
                    subfolders = Globals.ReadIntFromINI("locations", "item" + i + "-subfolders", autoindex_file, 0);
                    filetypes = Globals.ReadINI("locations", "item" + i + "-filetypes", autoindex_file);
                    toplevelfolders = Globals.ReadIntFromINI("locations", "item" + i + "-toplevelfolders", autoindex_file, 0);
                    autoindex_entry this_autoindex_entry = new autoindex_entry();
                    this_autoindex_entry.autoindex_folder = item_path;
                    this_autoindex_entry.autoindex_indexondemand = indexondemand;
                    this_autoindex_entry.autoindex_subfolders = subfolders;
                    this_autoindex_entry.autoindex_toplevelfolders = toplevelfolders;
                    this_autoindex_entry.autoindex_filetypes = filetypes;
                    autoindex_entries.Add(this_autoindex_entry);
                    item_checked = Globals.ReadIntFromINI("locations", "item" + i + "-checked", autoindex_file, 1);
                    if (item_checked == 1)
                        lstAutoIndex.SetItemChecked(i - 1, true);
                }
                else
                    break;
            }
            if (lstAutoIndex.Items.Count > 0)
            {
                lstAutoIndex.SelectedIndex = 0;
                item_path = Globals.ReadINI("locations", "item1-path", autoindex_file);

                if (autoindex_entries[0].autoindex_indexondemand == 1)
                    chkIndexOnDemand.Checked = true;
                else
                    chkIndexOnDemand.Checked = false;
                if (autoindex_entries[0].autoindex_subfolders == 1)
                    chkSubfolders.Checked = true;
                else
                    chkSubfolders.Checked = false;
                if (autoindex_entries[0].autoindex_filetypes == "")
                    chkAllFileTypes.Checked = true;
                else
                {
                    txtFileTypes.Visible = true;
                    txtFileTypes.Text = autoindex_entries[0].autoindex_filetypes;
                    lblFileTypeHelp.Visible = true;
                }
                if (autoindex_entries[0].autoindex_toplevelfolders == 1)
                    chkTopLevelFolders.Checked = true;
            }

            try
            {
                if (start_with_autoindex_tab == true)
                    tabControl1.SelectTab(3);
                else if (start_with_search_tab == true)
                    tabControl1.SelectTab(4);
                else if (start_with_clipboard_tab == true)
                {
                    // the clipboard tab is our last one. Subtract 1 from that since we start the count at 0.
                    int last_tab = tabControl1.TabCount;
                    tabControl1.SelectTab(last_tab - 1);
                }
            }
            catch { }
        }

        private void Load_Search_Tab()
        {
            string search_dir = Globals.WorkPath + "\\search";
            string search_ini = Globals.WorkPath + "\\search\\search.ini";
            try
            {
                if (Directory.Exists(search_dir) == false)
                {
                    Directory.CreateDirectory(search_dir);
                }
                if (File.Exists(search_ini) == false)
                {
                    File.WriteAllText(search_ini, "");
                }
            }
            catch (Exception f)
            {
                MessageBox.Show("The search file (" + search_ini + ") could not be created. " + f.Message, "Keybreeze Desktop");
            }
            string item_path = "";
            int subfolders;
            int item_checked;
            string filetypes;
            int toplevelfolders;
            for (int i = 1; i <= 250; i++)
            {
                item_path = Globals.ReadINI("search", "item" + i + "-path", search_ini);
                if (item_path.Length > 0)
                {
                    search_entry this_search_entry = new search_entry();
                    lstSearchFolders.Items.Add(item_path);
                    subfolders = Globals.ReadIntFromINI("search", "item" + i + "-subfolders", search_ini, 0);
                    filetypes = Globals.ReadINI("search", "item" + i + "-filetypes", search_ini);
                    toplevelfolders = Globals.ReadIntFromINI("search", "item" + i + "-toplevelfolders", search_ini, 0);
                    this_search_entry.search_folder = item_path;
                    this_search_entry.search_subfolders = subfolders;
                    this_search_entry.search_toplevelfolders = toplevelfolders;
                    this_search_entry.search_filetypes = filetypes;
                    item_checked = Globals.ReadIntFromINI("search", "item" + i + "-checked", search_ini, 1);
                    search_entries.Add(this_search_entry);
                    if (item_checked == 1)
                        lstSearchFolders.SetItemChecked(i - 1, true);
                }
                else
                    break;
            }
            if (lstSearchFolders.Items.Count > 0)
            {
                lstSearchFolders.SelectedIndex = 0;
                item_path = Globals.ReadINI("search", "item1-path", search_ini);
                if (search_entries[0].search_subfolders == 1)
                    chkSearchSubfolders.Checked = true;
                if (search_entries[0].search_filetypes == "")
                    chkSearchIndexAllFileTypes.Checked = true;
                else
                {
                    txtSearchFileTypes.Visible = true;
                    txtSearchFileTypes.Text = search_entries[0].search_filetypes;
                    lblSearchFileTypeHelp.Visible = true;
                }
                if (search_entries[0].search_toplevelfolders == 1)
                    chkSearchTopLevelFolders.Checked = true;
            }
        }

        private void chkDefaultWebBrowser_CheckedChanged(object sender, EventArgs e)
        {
            if (finished_loading == true)
            {
                if (chkDefaultWebBrowser.Checked == false)
                    LoadExtraOptions(1);
                else
                    edit_settings_1.Visible = false;
            }
        }

        private void LoadExtraOptions(int type)
        {
            if (type == 1)
            {
                // using custom web browser.
                frmOptions_Extra frmOptions_Extra_instance = new frmOptions_Extra(this, frmMain_instance);
                this.Enabled = false;
                frmOptions_Extra_instance.option_referencing = 1;
                frmOptions_Extra_instance.label1.Text = "Please specify the path to your browser. Normally this will be an .exe file:";
                frmOptions_Extra_instance.txtTarget.Text = Globals.ReadINI("options", "custom_browser_path", Globals.OptionsINI);
                string custom_browser_prefix = Globals.ReadINI("options", "custom_browser_prefix", Globals.OptionsINI);
                string custom_browser_suffix = Globals.ReadINI("options", "custom_browser_suffix", Globals.OptionsINI);
                if (custom_browser_prefix.Length > 0 || custom_browser_suffix.Length > 0)
                {
                    frmOptions_Extra_instance.txtPrefix.Text = custom_browser_prefix;
                    frmOptions_Extra_instance.txtSuffix.Text = custom_browser_suffix;
                }
                frmOptions_Extra_instance.ShowDialog();
            }
            else if (type == 2)
            {
                // using custom file explorer (not Windows Explorer).
                frmOptions_Extra frmOptions_Extra_instance = new frmOptions_Extra(this, frmMain_instance);
                this.Enabled = false;
                frmOptions_Extra_instance.option_referencing = 2;
                frmOptions_Extra_instance.label1.Text = "Please specify the path to your file explorer. Normally this will be an .exe file:";
                frmOptions_Extra_instance.txtTarget.Text = Globals.ReadINI("options", "custom_explorer_path", Globals.OptionsINI);
                string custom_explorer_prefix = Globals.ReadINI("options", "custom_explorer_prefix", Globals.OptionsINI);
                string custom_explorer_suffix = Globals.ReadINI("options", "custom_explorer_suffix", Globals.OptionsINI);
                if (custom_explorer_prefix.Length > 0 || custom_explorer_suffix.Length > 0)
                {
                    frmOptions_Extra_instance.txtPrefix.Text = custom_explorer_prefix;
                    frmOptions_Extra_instance.txtSuffix.Text = custom_explorer_suffix;
                }
                frmOptions_Extra_instance.ShowDialog();
            }
            else if (type == 3)
            {
                // launch program when an unrecognized entry is typed.
                frmOptions_Extra frmOptions_Extra_instance = new frmOptions_Extra(this, frmMain_instance);
                this.Enabled = false;
                frmOptions_Extra_instance.option_referencing = 3;
                frmOptions_Extra_instance.label1.Text = "Please specify the location of the program or script you want to run. Normally this will be an .exe file or .bat file:";
                frmOptions_Extra_instance.ShowDialog();
            }
        }

        private void edit_settings_1_Click(object sender, EventArgs e)
        {
            LoadExtraOptions(1);
        }

        private void chkWindowsExplorer_CheckedChanged(object sender, EventArgs e)
        {
            if (finished_loading == true)
            {
                if (chkWindowsExplorer.Checked == false)
                    LoadExtraOptions(2);
                else
                    edit_settings_2.Visible = false;
            }
        }

        private void edit_settings_2_Click(object sender, EventArgs e)
        {
            LoadExtraOptions(2);
        }

        private void edit_settings_3_Click(object sender, EventArgs e)
        {
            LoadExtraOptions(3);
        }

        private void button1_Click(object sender, EventArgs e)
        {
            frmOptions_Extra2 frmOptions_Extra2_instance = new frmOptions_Extra2(this);
            this.Enabled = false;
            frmOptions_Extra2_instance.Text = "Autocomplete list";
            frmOptions_Extra2_instance.Show();
        }

        private void button4_Click(object sender, EventArgs e)
        {
            frmOptions_Extra2 frmOptions_Extra2_instance = new frmOptions_Extra2(this);
            this.Enabled = false;
            frmOptions_Extra2_instance.Text = "Items in history";
            frmOptions_Extra2_instance.Show();
        }

        private void button2_Click(object sender, EventArgs e)
        {
            DialogResult result;
            result = MessageBox.Show("Are you sure you want to clear the autocomplete list?", "Keybreeze Desktop", MessageBoxButtons.YesNo);
            if (result == DialogResult.Yes)
            {
                Globals.autocomplete_list.Clear();
                string AutoCompleteFile = Globals.WorkPath + "\\autocomplete.txt";
                try
                {
                    System.IO.File.WriteAllText(AutoCompleteFile, "", Encoding.Unicode);
                }
                catch
                {
                    MessageBox.Show("The autocomplete file, " + AutoCompleteFile + ", could not be cleared. Please check your windows permissions for this directory.", "Keybreeze Desktop");
                }
            }
        }

        private void btnClearHistory_Click(object sender, EventArgs e)
        {
            DialogResult result;
            result = MessageBox.Show("Are you sure you want to clear the history list?", "Keybreeze Desktop", MessageBoxButtons.YesNo);
            if (result == DialogResult.Yes)
            {
                try
                {
                    System.IO.File.WriteAllText(Globals.WorkPath + "\\history.json", "", Encoding.Unicode);
                    Globals.History_List.Clear();
                }
                catch
                {
                    MessageBox.Show("The history file, " + Globals.WorkPath + "\\history.json, could not be cleared. Please check your windows permissions for this directory.", "Keybreeze Desktop");
                }
            }
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            btnClose.Enabled = false;
            this.Close();
        }

        private void frmOptions_FormClosing(object sender, FormClosingEventArgs e)
        {
            // save the settings back to the INI file.
            int autocomplete_limit;
            if (!int.TryParse(txtAutocompleteLimit.Text, out autocomplete_limit))
            {
                MessageBox.Show("Please type a number for the autocomplete limit.", "Keybreeze Desktop");
                btnClose.Enabled = true;
                e.Cancel = true;
                return;
            }
            // GENERAL OPTIONS TAB
            string extracted = Globals.ReadINI("Extract", "Extract_Not_Install", Globals.InstallDir + "\\default settings\\options.ini");
            if (extracted != "1")
            {
                // we are using the installed version of keybreeze, so check the registry.
                if (changed_startup_setting == true)
                {
                    if (chkLoadAtStartup.Checked == true)
                    {
                        if (Globals.ReadIntFromINI("options", "skip_registry", Globals.OptionsINI) != 1)
                        {
                            try
                            {
                                // check registry key to see if it's already been set.
                                RegistryKey rkApp = Registry.CurrentUser.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Run", true);
                                string RegKey = (string)rkApp.GetValue("Keybreeze5", "");
                                if (RegKey.Length == 0)
                                {
                                    try
                                    {
                                        rkApp.SetValue("Keybreeze5", Application.ExecutablePath.ToString());
                                    }
                                    catch
                                    {
                                        MessageBox.Show("Keybreeze Desktop was unable to modify the registry to add itself as a startup program. You may not have permission in Windows to access the registry, or UAC (User Account Control) may be blocking Keybreeze Desktop from accessing it. You can check your UAC settings by going to Control Panel > Action Center > Change User Account Control settings.", "Keybreeze Desktop");
                                    }
                                }
                            }
                            catch
                            {
                                MessageBox.Show("Keybreeze Desktop was unable to modify the registry to add itself as a startup program. You may not have permission in Windows to access the registry, or UAC (User Account Control) may be blocking Keybreeze Desktop from accessing it. You can check your UAC settings by going to Control Panel > Action Center > Change User Account Control settings.", "Keybreeze Desktop");
                            }
                        }
                    }
                    else
                    {
                        // check the registry to see if it's already been set.
                        if (Globals.ReadIntFromINI("options", "skip_registry", Globals.OptionsINI) != 1)
                        {
                            try
                            {
                                RegistryKey rkApp = Registry.CurrentUser.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Run", true);
                                string RegKey = (string)rkApp.GetValue("Keybreeze5", "");
                                if (RegKey.Length > 0)
                                {
                                    try
                                    {
                                        rkApp.DeleteValue("Keybreeze5", false);
                                    }
                                    catch
                                    {
                                        MessageBox.Show("Keybreeze Desktop was unable to modify the registry to remove itself from the startup programs. Please check your Windows permissions.", "Keybreeze Desktop");
                                    }
                                }
                            }
                            catch
                            {
                                MessageBox.Show("Keybreeze Desktop was unable to modify the registry to remove itself from the startup programs. Please check your Windows permissions.", "Keybreeze Desktop");
                            }
                        }
                    }
                }
            }
            if (chkDefaultWebBrowser.Checked == true)
            {
                Globals.WriteINI("options", "default_browser", "1", Globals.OptionsINI);
            }
            else
            {
                Globals.WriteINI("options", "default_browser", "0", Globals.OptionsINI);
            }
            if (chkWindowsExplorer.Checked == true)
            {
                Globals.WriteINI("options", "windows_explorer", "1", Globals.OptionsINI);
                frmMain_instance.custom_explorer_path = "";
            }
            else
            {
                Globals.WriteINI("options", "windows_explorer", "0", Globals.OptionsINI);
                frmMain_instance.custom_explorer_path = Globals.ReadINI("options", "custom_explorer_path", Globals.OptionsINI);
            }

            // we only want to set show_keyboard_help in the INI. We don't want to set the global variable to 1 if it's currently 0 because we may not have loaded an instance of the form.
            // The user will need to close and reopen Keybreeze Desktop to see a change in this setting. If the user reloads Keybreeze Desktop, and the INI setting is equal to 1, then we will load an instance of the form.
            if (chkShowKeyboardHelp.Checked == true)
            {
                Globals.WriteINI("options", "show_keyboard_help", "1", Globals.OptionsINI);
            }
            else
            {
                Globals.WriteINI("options", "show_keyboard_help", "0", Globals.OptionsINI);
                Globals.show_keyboard_help = 0;
            }

            Globals.when_pressing_shift = cmbShift.SelectedIndex;
            Globals.when_pressing_tab = cmbTab.SelectedIndex;
            Globals.when_pressing_ctrl = cmbCTRL.SelectedIndex;
            Globals.WriteINI("options", "when_pressing_shift", cmbShift.SelectedIndex.ToString(), Globals.OptionsINI);
            Globals.WriteINI("options", "when_pressing_tab", cmbTab.SelectedIndex.ToString(), Globals.OptionsINI);
            Globals.WriteINI("options", "when_pressing_ctrl", cmbCTRL.SelectedIndex.ToString(), Globals.OptionsINI);
            Globals.WriteINI("options", "search_url", txtSearchURL.Text, Globals.OptionsINI);
            Globals.WriteINI("options", "when_pressing_enter_unrecognized_entry", cmbEnterUnrecognizedCommand.SelectedIndex.ToString(), Globals.OptionsINI);

            Globals.WriteINI("options", "autocomplete_char_limit", txtAutocompleteLimit.Text, Globals.OptionsINI);
            
            // HOTKEY TAB
            if (hotkey_modified == true)
            {
                if (chkModifiers.SelectedIndex == 0)
                {
                    Globals.WriteINI("hotkey", "hotkey_ctrl", "0", Globals.OptionsINI);
                    Globals.WriteINI("hotkey", "hotkey_alt", "0", Globals.OptionsINI);
                    Globals.WriteINI("hotkey", "hotkey_shift", "0", Globals.OptionsINI);
                    Globals.WriteINI("hotkey", "hotkey_windows", "0", Globals.OptionsINI);
                }
                else if (chkModifiers.SelectedIndex == 1)
                {
                    Globals.WriteINI("hotkey", "hotkey_ctrl", "0", Globals.OptionsINI);
                    Globals.WriteINI("hotkey", "hotkey_alt", "0", Globals.OptionsINI);
                    Globals.WriteINI("hotkey", "hotkey_shift", "1", Globals.OptionsINI);
                    Globals.WriteINI("hotkey", "hotkey_windows", "0", Globals.OptionsINI);
                }
                else if (chkModifiers.SelectedIndex == 1)
                {
                    Globals.WriteINI("hotkey", "hotkey_ctrl", "0", Globals.OptionsINI);
                    Globals.WriteINI("hotkey", "hotkey_alt", "0", Globals.OptionsINI);
                    Globals.WriteINI("hotkey", "hotkey_shift", "1", Globals.OptionsINI);
                    Globals.WriteINI("hotkey", "hotkey_windows", "0", Globals.OptionsINI);
                }
                else if (chkModifiers.SelectedIndex == 2)
                {
                    Globals.WriteINI("hotkey", "hotkey_ctrl", "1", Globals.OptionsINI);
                    Globals.WriteINI("hotkey", "hotkey_alt", "0", Globals.OptionsINI);
                    Globals.WriteINI("hotkey", "hotkey_shift", "0", Globals.OptionsINI);
                    Globals.WriteINI("hotkey", "hotkey_windows", "0", Globals.OptionsINI);
                }
                else if (chkModifiers.SelectedIndex == 3)
                {
                    Globals.WriteINI("hotkey", "hotkey_ctrl", "0", Globals.OptionsINI);
                    Globals.WriteINI("hotkey", "hotkey_alt", "1", Globals.OptionsINI);
                    Globals.WriteINI("hotkey", "hotkey_shift", "0", Globals.OptionsINI);
                    Globals.WriteINI("hotkey", "hotkey_windows", "0", Globals.OptionsINI);
                }
                else if (chkModifiers.SelectedIndex == 4)
                {
                    Globals.WriteINI("hotkey", "hotkey_ctrl", "0", Globals.OptionsINI);
                    Globals.WriteINI("hotkey", "hotkey_alt", "0", Globals.OptionsINI);
                    Globals.WriteINI("hotkey", "hotkey_shift", "0", Globals.OptionsINI);
                    Globals.WriteINI("hotkey", "hotkey_windows", "1", Globals.OptionsINI);
                }
                else if (chkModifiers.SelectedIndex == 5)
                {
                    Globals.WriteINI("hotkey", "hotkey_ctrl", "1", Globals.OptionsINI);
                    Globals.WriteINI("hotkey", "hotkey_alt", "1", Globals.OptionsINI);
                    Globals.WriteINI("hotkey", "hotkey_shift", "0", Globals.OptionsINI);
                    Globals.WriteINI("hotkey", "hotkey_windows", "0", Globals.OptionsINI);
                }
                else if (chkModifiers.SelectedIndex == 6)
                {
                    Globals.WriteINI("hotkey", "hotkey_ctrl", "1", Globals.OptionsINI);
                    Globals.WriteINI("hotkey", "hotkey_alt", "0", Globals.OptionsINI);
                    Globals.WriteINI("hotkey", "hotkey_shift", "1", Globals.OptionsINI);
                    Globals.WriteINI("hotkey", "hotkey_windows", "0", Globals.OptionsINI);
                }
                else if (chkModifiers.SelectedIndex == 7)
                {
                    Globals.WriteINI("hotkey", "hotkey_ctrl", "0", Globals.OptionsINI);
                    Globals.WriteINI("hotkey", "hotkey_alt", "1", Globals.OptionsINI);
                    Globals.WriteINI("hotkey", "hotkey_shift", "1", Globals.OptionsINI);
                    Globals.WriteINI("hotkey", "hotkey_windows", "0", Globals.OptionsINI);
                }
                string hotkey_char = txtHotkey.Text;
                string hotkey_ascii_code = txtHotkey.Tag.ToString();
                Globals.WriteINI("hotkey", "hotkey", hotkey_char, Globals.OptionsINI);
                Globals.WriteINI("hotkey", "hotkey_ascii_code", hotkey_ascii_code, Globals.OptionsINI);
                // change the hotkey in memory.
                frmMain_instance.UnregisterHotkeyHelper();
                frmMain_instance.SetHotKey();
            }

            // SKINS TAB
            if (rdoSkinAlwaysCenter.Checked == true)
            {
                Globals.WriteINI("options", "always_center_keybreeze", "1", Globals.OptionsINI);
                frmMain_instance.always_center_keybreeze = 1;
            }
            else
            {
                Globals.WriteINI("options", "always_center_keybreeze", "0", Globals.OptionsINI);
                frmMain_instance.always_center_keybreeze = 0;
            }
            // AUTOINDEX TAB
            SaveAutoIndexSettings();
            // SEARCH TAB
            SaveSearchSettings();

            // CLIPBOARD TAB
            if (chkEnableClipboardMonitoring.Checked == false)
            {
                Globals.WriteINI("clipboard", "enabled", "0", Globals.OptionsINI);
                frmMain_instance.tmrKeybreezeClipboard.Enabled = false;
            }
            else
            {
                Globals.WriteINI("clipboard", "enabled", "1", Globals.OptionsINI);
                frmMain_instance.tmrKeybreezeClipboard.Enabled = true;
            }

            int clipboard_interval = 0;
            if (!int.TryParse(txtClipboardMonitorInterval.Text, out clipboard_interval))
            {
                MessageBox.Show("Please type a number for the Clipboard interval.", "Keybreeze Desktop");
                btnClose.Enabled = true;
                e.Cancel = true;
                return;
            }
            if (clipboard_interval < 500 || clipboard_interval > 30000)
            {
                clipboard_interval = 1000;
            }
            frmMain_instance.tmrKeybreezeClipboard.Interval = clipboard_interval;
            Globals.WriteINI("clipboard", "interval", clipboard_interval.ToString(), Globals.OptionsINI);

            int max_items_for_clipboard = 0;
            if (!int.TryParse(txtMaximumItemsForClipboard.Text, out max_items_for_clipboard))
            {
                MessageBox.Show("Please type a number for the Clipboard maximum number of items to store.", "Keybreeze Desktop");
                btnClose.Enabled = true;
                e.Cancel = true;
                return;
            }
            if (max_items_for_clipboard <= 0 || max_items_for_clipboard > 250)
            {
                max_items_for_clipboard = 50;
            }
            frmMain_instance.keybreeze_clipboard_max_count = max_items_for_clipboard;
            Globals.WriteINI("clipboard", "clipboard_max_count", max_items_for_clipboard.ToString(), Globals.OptionsINI);

            int clipboard_item_max_length = 0;
            if (!int.TryParse(txtMaximumTextLengthPerItem.Text, out clipboard_item_max_length))
            {
                MessageBox.Show("Please type a number for the Clipboard maximum length per item.", "Keybreeze Desktop");
                btnClose.Enabled = true;
                e.Cancel = true;
                return;
            }
            if (clipboard_item_max_length <= 0 || clipboard_item_max_length > 1000000)
            {
                clipboard_item_max_length = 100000;
            }
            frmMain_instance.keybreeze_clipboard_item_max_length = clipboard_item_max_length;
            Globals.WriteINI("clipboard", "clipboard_item_max_length", clipboard_item_max_length.ToString(), Globals.OptionsINI);

            if (chkUseTabKeyToToggleRows.Checked == false)
            {
                Globals.WriteINI("clipboard", "tab_to_toggle", "0", Globals.OptionsINI);
            }
            else
            {
                Globals.WriteINI("clipboard", "tab_to_toggle", "1", Globals.OptionsINI);
            }
            if (chkClearHistoryClearClipboard.Checked == false)
            {
                Globals.WriteINI("clipboard", "clear_history_when_clearing_clipboard", "0", Globals.OptionsINI);
                frmMain_instance.keybreeze_clipboard_clear_history = false;
            }
            else
            {
                Globals.WriteINI("clipboard", "clear_history_when_clearing_clipboard", "1", Globals.OptionsINI);
                frmMain_instance.keybreeze_clipboard_clear_history = true;
            }
            Globals.WriteINI("clipboard", "clipboard_password_detection", cmbClipboardMonitorPasswordSetting.SelectedIndex.ToString(), Globals.OptionsINI);
            frmMain_instance.clipboard_password_detection = cmbClipboardMonitorPasswordSetting.SelectedIndex;
        }

        private void chkLoadAtStartup_CheckedChanged(object sender, EventArgs e)
        {
            changed_startup_setting = true;
        }

        private void SaveAutoIndexSettings()
        {
            string autoindex_file = Globals.WorkPath + "\\autoindex\\indexoptions.ini";
            // clear the file first.
            System.IO.File.WriteAllText(autoindex_file, "", Encoding.Unicode);
            // write some settings back to the file.
            Globals.WriteINI("settings", "popup_result_delay_with_no_matches", frmMain_instance.popup_result_delay_with_no_matches.ToString(), autoindex_file);
            Globals.WriteINI("settings", "popup_result_delay_with_matches", frmMain_instance.popup_result_delay_with_matches.ToString(), autoindex_file);
            Globals.WriteINI("settings", "popup_result_delay_with_folders_on_demand", frmMain_instance.popup_result_delay_with_folders_on_demand.ToString(), autoindex_file);
            int autoindex_refresh;
            if (!int.TryParse(txtRefreshInterval.Text, out autoindex_refresh))
                Globals.WriteINI("settings", "refresh_interval_minutes", "10", autoindex_file);
            else
                Globals.WriteINI("settings", "refresh_interval_minutes", txtRefreshInterval.Text, autoindex_file);

            int autoindex_counter = 1;

            foreach (autoindex_entry this_entry in autoindex_entries)
            {
                if (lstAutoIndex.GetItemChecked(autoindex_counter - 1) == true)
                    Globals.WriteINI("locations", "item" + autoindex_counter + "-checked", "1", autoindex_file);
                else
                    Globals.WriteINI("locations", "item" + autoindex_counter + "-checked", "0", autoindex_file);
                Globals.WriteINI("locations", "item" + autoindex_counter + "-path", this_entry.autoindex_folder, autoindex_file);
                Globals.WriteINI("locations", "item" + autoindex_counter + "-indexondemand", this_entry.autoindex_indexondemand.ToString(), autoindex_file);
                Globals.WriteINI("locations", "item" + autoindex_counter + "-subfolders", this_entry.autoindex_subfolders.ToString(), autoindex_file);
                Globals.WriteINI("locations", "item" + autoindex_counter + "-filetypes", this_entry.autoindex_filetypes, autoindex_file);
                Globals.WriteINI("locations", "item" + autoindex_counter + "-toplevelfolders", this_entry.autoindex_toplevelfolders.ToString(), autoindex_file);
                autoindex_counter++;
            }
        }

        private void SaveSearchSettings()
        {
            string search_file = Globals.WorkPath + "\\search\\search.ini";
            try
            {
                if (Directory.Exists(Globals.WorkPath + "\\search") == false)
                {
                    Directory.CreateDirectory(Globals.WorkPath + "\\search");
                }
                // clear the entries first.
                System.IO.File.WriteAllText(search_file, "", Encoding.Unicode);
            }
            catch (Exception f)
            {
                MessageBox.Show("Warning: Could not save search settings in " + search_file + ". " + f.Message, "Keybreeze Desktop");
            }

            int search_counter = 1;
            foreach (search_entry this_entry in search_entries)
            {
                if (lstSearchFolders.GetItemChecked(search_counter - 1) == true)
                    Globals.WriteINI("search", "item" + search_counter + "-checked", "1", search_file);
                else
                    Globals.WriteINI("search", "item" + search_counter + "-checked", "0", search_file);
                Globals.WriteINI("search", "item" + search_counter + "-path", this_entry.search_folder, search_file);
                Globals.WriteINI("search", "item" + search_counter + "-subfolders", this_entry.search_subfolders.ToString(), search_file);
                Globals.WriteINI("search", "item" + search_counter + "-filetypes", this_entry.search_filetypes, search_file);
                Globals.WriteINI("search", "item" + search_counter + "-toplevelfolders", this_entry.search_toplevelfolders.ToString(), search_file);
                search_counter++;
            }
        }

        private void txtHotkey_KeyDown(object sender, KeyEventArgs e)
        {
            // e.KeyCode is the letter/number/key.
            // e.KeyValue is the ascii code number.
            hotkey_modified = true;
            int NewCode = e.KeyValue;
            if (NewCode > 95 && NewCode < 112)
            {
                txtHotkey.Text = "";
                return;
            }
            switch (NewCode)
            {
                case 8:
                    txtHotkey.Text = "Backspace";
                    break;
                case 9:
                    txtHotkey.Text = "";
                    break;
                case 13:
                    txtHotkey.Text = "Enter";
                    break;
                case 32:
                    txtHotkey.Text = "Space";
                    break;
                case 112:
                    txtHotkey.Text = "F1";
                    break;
                case 113:
                    txtHotkey.Text = "F2";
                    break;
                case 114:
                    txtHotkey.Text = "F3";
                    break;
                case 115:
                    txtHotkey.Text = "F4";
                    break;
                case 116:
                    txtHotkey.Text = "F5";
                    break;
                case 117:
                    txtHotkey.Text = "F6";
                    break;
                case 118:
                    txtHotkey.Text = "F7";
                    break;
                case 119:
                    txtHotkey.Text = "F8";
                    break;
                case 120:
                    txtHotkey.Text = "F9";
                    break;
                case 121:
                    txtHotkey.Text = "F10";
                    break;
                case 122:
                    txtHotkey.Text = "F11";
                    break;
                case 123:
                    txtHotkey.Text = "F12";
                    break;
                case 124:
                    txtHotkey.Text = "F13";
                    break;
                case 125:
                    txtHotkey.Text = "F14";
                    break;
                case 126:
                    txtHotkey.Text = "F15";
                    break;
                case 127:
                    txtHotkey.Text = "F16";
                    break;
                case 128:
                    txtHotkey.Text = "F17";
                    break;
                case 129:
                    txtHotkey.Text = "F18";
                    break;
                case 130:
                    txtHotkey.Text = "F19";
                    break;
                case 131:
                    txtHotkey.Text = "F20";
                    break;
                case 144:
                    txtHotkey.Text = "Num Lock";
                    break;
                case 145:
                    txtHotkey.Text = "Scroll Lock";
                    break;
                case 45:
                    txtHotkey.Text = "Insert";
                    break;
                case 36:
                    txtHotkey.Text = "Home";
                    break;
                case 33:
                    txtHotkey.Text = "Page Up";
                    break;
                case 46:
                    txtHotkey.Text = "Delete";
                    break;
                case 35:
                    txtHotkey.Text = "End";
                    break;
                case 34:
                    txtHotkey.Text = "Page Down";
                    break;
                case 19:
                    txtHotkey.Text = "Pause";
                    break;
                case 186:
                    // semicolon
                    txtHotkey.Text = ";";
                    break;
                default:
                    txtHotkey.Text = e.KeyCode.ToString();
                    break;
            }
            txtHotkey.Tag = e.KeyValue.ToString();
        }

        private void chkSemicolonSpace_CheckedChanged(object sender, EventArgs e)
        {
            hotkey_modified = true;
        }

        private void chkSemicolonEnter_CheckedChanged(object sender, EventArgs e)
        {
            hotkey_modified = true;
        }

        private void chkModifiers_SelectedIndexChanged(object sender, EventArgs e)
        {
            hotkey_modified = true;
        }

        private void button2_Click_1(object sender, EventArgs e)
        {
            frmMain_instance.RunURL("http://www.keybreeze.com/plugins/skins");
        }

        private void btnMakeDefaultSkin_Click(object sender, EventArgs e)
        {
            if (lstSkins.SelectedItem.ToString() != txtCurrentSkin.Text)
            {
                txtCurrentSkin.Text = lstSkins.SelectedItem.ToString();
                Globals.WriteINI("options", "skin", txtCurrentSkin.Text, Globals.OptionsINI);
                Globals.WriteINI("options", "skin_position_top", null, Globals.OptionsINI);
                Globals.WriteINI("options", "skin_position_left", null, Globals.OptionsINI);
                MessageBox.Show("The default skin has been changed. You must close Keybreeze Desktop and re-open it for the change to take effect. You can close it by typing 'exit' as a keyword.", "Keybreeze Desktop");
            }
        }

        private void btnRemoveSkin_Click(object sender, EventArgs e)
        {
            if (txtCurrentSkin.Text == lstSkins.SelectedItem.ToString())
            {
                MessageBox.Show("You cannot remove the default skin. Please make another skin the default first.", "Keybreeze Desktop");
            }
            else
            {
                DialogResult result;
                result = MessageBox.Show("Are you sure you want to remove the skin, " + lstSkins.SelectedItem.ToString() + "?", "Keybreeze Desktop", MessageBoxButtons.YesNo);
                if (result == DialogResult.Yes)
                {
                    Directory.Delete(Globals.WorkPath + "\\skins\\" + lstSkins.SelectedItem.ToString(), true);
                    lstSkins.Items.RemoveAt(lstSkins.SelectedIndex);
                }
            }
        }

        private void cmdAddAutoIndex_Click(object sender, EventArgs e)
        {
            autoindexBrowseFolder.ShowDialog();
            if (autoindexBrowseFolder.SelectedPath.Length > 0)
            {
                autoindex_entry new_entry = new autoindex_entry();
                new_entry.autoindex_indexondemand = 0;
                new_entry.autoindex_subfolders = 0;
                new_entry.autoindex_filetypes = "";
                new_entry.autoindex_toplevelfolders = 0;
                new_entry.autoindex_folder = autoindexBrowseFolder.SelectedPath;
                autoindex_entries.Add(new_entry);
                lstAutoIndex.Items.Add(autoindexBrowseFolder.SelectedPath);
                lstAutoIndex.SetItemChecked(lstAutoIndex.Items.Count - 1, true);
                lstAutoIndex.SelectedIndex = lstAutoIndex.Items.Count - 1;
            }
        }

        private void cmdRemoveAutoIndex_Click(object sender, EventArgs e)
        {
            if (lstAutoIndex.SelectedItem.ToString() == "Desktop" || lstAutoIndex.SelectedItem.ToString() == "Start Menu" || lstAutoIndex.SelectedItem.ToString() == "Start Menu (All users)" || lstAutoIndex.SelectedItem.ToString() == "My Documents")
            {
                MessageBox.Show("You cannot remove a preset folder, such as " + lstAutoIndex.SelectedItem + ", from the list. Please uncheck the item if you do not want it to be indexed.", "Keybreeze Desktop");
                return;
            }
            autoindex_entries.RemoveAt(lstAutoIndex.SelectedIndex);
            lstAutoIndex.Items.RemoveAt(lstAutoIndex.SelectedIndex);
            lstAutoIndex.SelectedIndex = 0;
            SaveAutoIndexSettings();
        }

        private void label12_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Please type the file types you want Keybreeze Desktop to index, separated by commas, not including periods." + Environment.NewLine + Environment.NewLine + "Example: doc,jpg,bmp,gif,exe", "Keybreeze Desktop");
        }

        private void chkSubfolders_CheckedChanged(object sender, EventArgs e)
        {
            if (chkSubfolders.Checked == true)
            {
                autoindex_entries[lstAutoIndex.SelectedIndex].autoindex_subfolders = 1;
            }
            else
            {
                autoindex_entries[lstAutoIndex.SelectedIndex].autoindex_subfolders = 0;
            }
        }

        private void chkTopLevelFolders_CheckedChanged(object sender, EventArgs e)
        {
            if (chkTopLevelFolders.Checked == true)
            {
                autoindex_entries[lstAutoIndex.SelectedIndex].autoindex_toplevelfolders = 1;
            }
            else
            {
                autoindex_entries[lstAutoIndex.SelectedIndex].autoindex_toplevelfolders = 0;
            }
        }

        private void chkAllFileTypes_CheckedChanged(object sender, EventArgs e)
        {
            if (chkAllFileTypes.Checked == false)
            {
                txtFileTypes.Visible = true;
                lblFileTypeHelp.Visible = true;
            }
            else
            {
                txtFileTypes.Visible = false;
                txtFileTypes.Text = "";
                lblFileTypeHelp.Visible = false;
                autoindex_entries[lstAutoIndex.SelectedIndex].autoindex_filetypes = "";
            }
        }

        private void lstAutoIndex_SelectedIndexChanged(object sender, EventArgs e)
        {
            string autoindex_file = Globals.WorkPath + "\\autoindex\\indexoptions.ini";
            string item_path;
            int number_plus_one = lstAutoIndex.SelectedIndex + 1;
            if (lstAutoIndex.SelectedIndex > -1)
            {
                item_path = Globals.ReadINI("locations", "item" + number_plus_one + "-path", autoindex_file);
                if (autoindex_entries[lstAutoIndex.SelectedIndex].autoindex_indexondemand == 1)
                    chkIndexOnDemand.Checked = true;
                else
                    chkIndexOnDemand.Checked = false;
                if (autoindex_entries[lstAutoIndex.SelectedIndex].autoindex_subfolders == 1)
                    chkSubfolders.Checked = true;
                else
                    chkSubfolders.Checked = false;
                if (autoindex_entries[lstAutoIndex.SelectedIndex].autoindex_filetypes == "")
                    chkAllFileTypes.Checked = true;
                else
                {
                    chkAllFileTypes.Checked = false;
                    txtFileTypes.Visible = true;
                    txtFileTypes.Text = autoindex_entries[lstAutoIndex.SelectedIndex].autoindex_filetypes;
                    lblFileTypeHelp.Visible = true;
                }
                if (autoindex_entries[lstAutoIndex.SelectedIndex].autoindex_toplevelfolders == 1)
                    chkTopLevelFolders.Checked = true;
                else
                    chkTopLevelFolders.Checked = false;
            }
        }

        private void txtFileTypes_TextChanged(object sender, EventArgs e)
        {
            autoindex_entries[lstAutoIndex.SelectedIndex].autoindex_filetypes = txtFileTypes.Text;
        }

        private void lstQuickTab_SelectedIndexChanged(object sender, EventArgs e)
        {
            int this_item_setting;
            string key_combo = "";
            if (finished_loading == true)
            {
                if (lstQuickTab.SelectedIndex == 0)
                {
                    // hotkey + hotkey
                    key_combo = "hotkey_plus_hotkey";
                    this_item_setting = Globals.ReadIntFromINI("quicktab", "hotkey_plus_hotkey", Globals.OptionsINI, 1);
                    cmbQuickTab.SelectedIndex = this_item_setting;
                }
                else if (lstQuickTab.SelectedIndex == 1)
                {
                    // hotkey + space
                    key_combo = "hotkey_plus_space";
                    this_item_setting = Globals.ReadIntFromINI("quicktab", "hotkey_plus_space", Globals.OptionsINI, 2);
                    cmbQuickTab.SelectedIndex = this_item_setting;
                }
                else if (lstQuickTab.SelectedIndex == 2)
                {
                    // hotkey + enter
                    key_combo = "hotkey_plus_enter";
                    this_item_setting = Globals.ReadIntFromINI("quicktab", "hotkey_plus_enter", Globals.OptionsINI, 3);
                    cmbQuickTab.SelectedIndex = this_item_setting;
                }
                else if (lstQuickTab.SelectedIndex == 3)
                {
                    // hotkey + shift
                    key_combo = "hotkey_plus_shift";
                    this_item_setting = Globals.ReadIntFromINI("quicktab", "hotkey_plus_shift", Globals.OptionsINI, 0);
                    cmbQuickTab.SelectedIndex = this_item_setting;
                }
                else if (lstQuickTab.SelectedIndex == 4)
                {
                    // hotkey + ctrl
                    key_combo = "hotkey_plus_ctrl";
                    this_item_setting = Globals.ReadIntFromINI("quicktab", "hotkey_plus_ctrl", Globals.OptionsINI, 0);
                    cmbQuickTab.SelectedIndex = this_item_setting;
                }
                else if (lstQuickTab.SelectedIndex == 5)
                {
                    // hotkey + tab
                    key_combo = "hotkey_plus_tab";
                    this_item_setting = Globals.ReadIntFromINI("quicktab", "hotkey_plus_tab", Globals.OptionsINI, 0);
                    cmbQuickTab.SelectedIndex = this_item_setting;
                }
                else if (lstQuickTab.SelectedIndex == 6)
                {
                    // F1
                    key_combo = "hotkey_plus_f1";
                    this_item_setting = Globals.ReadIntFromINI("quicktab", "hotkey_plus_f1", Globals.OptionsINI, 0);
                    cmbQuickTab.SelectedIndex = this_item_setting;
                }
                else if (lstQuickTab.SelectedIndex == 7)
                {
                    key_combo = "hotkey_plus_f2";
                    this_item_setting = Globals.ReadIntFromINI("quicktab", "hotkey_plus_f2", Globals.OptionsINI, 0);
                    cmbQuickTab.SelectedIndex = this_item_setting;
                }
                else if (lstQuickTab.SelectedIndex == 8)
                {
                    key_combo = "hotkey_plus_f3";
                    this_item_setting = Globals.ReadIntFromINI("quicktab", "hotkey_plus_f3", Globals.OptionsINI, 0);
                    cmbQuickTab.SelectedIndex = this_item_setting;
                }
                else if (lstQuickTab.SelectedIndex == 9)
                {
                    key_combo = "hotkey_plus_f4";
                    this_item_setting = Globals.ReadIntFromINI("quicktab", "hotkey_plus_f4", Globals.OptionsINI, 0);
                    cmbQuickTab.SelectedIndex = this_item_setting;
                }
                else if (lstQuickTab.SelectedIndex == 10)
                {
                    key_combo = "hotkey_plus_f5";
                    this_item_setting = Globals.ReadIntFromINI("quicktab", "hotkey_plus_f5", Globals.OptionsINI, 0);
                    cmbQuickTab.SelectedIndex = this_item_setting;
                }
                else if (lstQuickTab.SelectedIndex == 11)
                {
                    key_combo = "hotkey_plus_f5";
                    this_item_setting = Globals.ReadIntFromINI("quicktab", "hotkey_plus_f6", Globals.OptionsINI, 0);
                    cmbQuickTab.SelectedIndex = this_item_setting;
                }
                else if (lstQuickTab.SelectedIndex == 12)
                {
                    key_combo = "hotkey_plus_f7";
                    this_item_setting = Globals.ReadIntFromINI("quicktab", "hotkey_plus_f7", Globals.OptionsINI, 0);
                    cmbQuickTab.SelectedIndex = this_item_setting;
                }
                else if (lstQuickTab.SelectedIndex == 13)
                {
                    key_combo = "hotkey_plus_f8";
                    this_item_setting = Globals.ReadIntFromINI("quicktab", "hotkey_plus_f8", Globals.OptionsINI, 0);
                    cmbQuickTab.SelectedIndex = this_item_setting;
                }
                else if (lstQuickTab.SelectedIndex == 14)
                {
                    key_combo = "hotkey_plus_f9";
                    this_item_setting = Globals.ReadIntFromINI("quicktab", "hotkey_plus_f9", Globals.OptionsINI, 0);
                    cmbQuickTab.SelectedIndex = this_item_setting;
                }
                else if (lstQuickTab.SelectedIndex == 15)
                {
                    key_combo = "hotkey_plus_f10";
                    this_item_setting = Globals.ReadIntFromINI("quicktab", "hotkey_plus_f10", Globals.OptionsINI, 0);
                    cmbQuickTab.SelectedIndex = this_item_setting;
                }
                else if (lstQuickTab.SelectedIndex == 16)
                {
                    key_combo = "hotkey_plus_f11";
                    this_item_setting = Globals.ReadIntFromINI("quicktab", "hotkey_plus_f11", Globals.OptionsINI, 0);
                    cmbQuickTab.SelectedIndex = this_item_setting;
                }
                else if (lstQuickTab.SelectedIndex == 17)
                {
                    key_combo = "hotkey_plus_f12";
                    this_item_setting = Globals.ReadIntFromINI("quicktab", "hotkey_plus_f12", Globals.OptionsINI, 0);
                    cmbQuickTab.SelectedIndex = this_item_setting;
                }
                if (cmbQuickTab.SelectedIndex == 8)
                {
                    // the user selected PositionWindow. Show our extra fields.
                    string pos = Globals.ReadINI("quicktab", key_combo + "_pos", Globals.OptionsINI);
                    txtPositionWindow.Text = pos;
                    txtPositionWindow.Visible = true;
                    lblPositionWindow.Visible = true;
                    lblPositionWindowExamples.Visible = true;
                }
                else
                {
                    txtPositionWindow.Visible = false;
                    lblPositionWindow.Visible = false;
                    lblPositionWindowExamples.Visible = false;
                }
            }
        }

        private void cmbQuickTab_SelectedIndexChanged(object sender, EventArgs e)
        {
            string key_combo = "";
            if (lstQuickTab.SelectedIndex == 0)
            {
                key_combo = "hotkey_plus_hotkey";
                Globals.WriteINI("quicktab", key_combo, cmbQuickTab.SelectedIndex.ToString(), Globals.OptionsINI);
            }
            else if (lstQuickTab.SelectedIndex == 1)
            {
                key_combo = "hotkey_plus_space";
                Globals.WriteINI("quicktab", key_combo, cmbQuickTab.SelectedIndex.ToString(), Globals.OptionsINI);
            }
            else if (lstQuickTab.SelectedIndex == 2)
            {
                key_combo = "hotkey_plus_enter";
                Globals.WriteINI("quicktab", key_combo, cmbQuickTab.SelectedIndex.ToString(), Globals.OptionsINI);
            }
            else if (lstQuickTab.SelectedIndex == 3)
            {
                key_combo = "hotkey_plus_shift";
                Globals.WriteINI("quicktab", key_combo, cmbQuickTab.SelectedIndex.ToString(), Globals.OptionsINI);
            }
            else if (lstQuickTab.SelectedIndex == 4)
            {
                key_combo = "hotkey_plus_ctrl";
                Globals.WriteINI("quicktab", key_combo, cmbQuickTab.SelectedIndex.ToString(), Globals.OptionsINI);
            }
            else if (lstQuickTab.SelectedIndex == 5)
            {
                key_combo = "hotkey_plus_tab";
                Globals.WriteINI("quicktab", key_combo, cmbQuickTab.SelectedIndex.ToString(), Globals.OptionsINI);
            }
            else if (lstQuickTab.SelectedIndex == 6)
            {
                key_combo = "hotkey_plus_f1";
                Globals.WriteINI("quicktab", key_combo, cmbQuickTab.SelectedIndex.ToString(), Globals.OptionsINI);
            }
            else if (lstQuickTab.SelectedIndex == 7)
            {
                key_combo = "hotkey_plus_f2";
                Globals.WriteINI("quicktab", key_combo, cmbQuickTab.SelectedIndex.ToString(), Globals.OptionsINI);
            }
            else if (lstQuickTab.SelectedIndex == 8)
            {
                key_combo = "hotkey_plus_f3";
                Globals.WriteINI("quicktab", key_combo, cmbQuickTab.SelectedIndex.ToString(), Globals.OptionsINI);
            }
            else if (lstQuickTab.SelectedIndex == 9)
            {
                key_combo = "hotkey_plus_f4";
                Globals.WriteINI("quicktab", key_combo, cmbQuickTab.SelectedIndex.ToString(), Globals.OptionsINI);
            }
            else if (lstQuickTab.SelectedIndex == 10)
            {
                key_combo = "hotkey_plus_f5";
                Globals.WriteINI("quicktab", key_combo, cmbQuickTab.SelectedIndex.ToString(), Globals.OptionsINI);
            }
            else if (lstQuickTab.SelectedIndex == 11)
            {
                key_combo = "hotkey_plus_f6";
                Globals.WriteINI("quicktab", key_combo, cmbQuickTab.SelectedIndex.ToString(), Globals.OptionsINI);
            }
            else if (lstQuickTab.SelectedIndex == 12)
            {
                key_combo = "hotkey_plus_f7";
                Globals.WriteINI("quicktab", key_combo, cmbQuickTab.SelectedIndex.ToString(), Globals.OptionsINI);
            }
            else if (lstQuickTab.SelectedIndex == 13)
            {
                key_combo = "hotkey_plus_f8";
                Globals.WriteINI("quicktab", key_combo, cmbQuickTab.SelectedIndex.ToString(), Globals.OptionsINI);
            }
            else if (lstQuickTab.SelectedIndex == 14)
            {
                key_combo = "hotkey_plus_f9";
                Globals.WriteINI("quicktab", key_combo, cmbQuickTab.SelectedIndex.ToString(), Globals.OptionsINI);
            }
            else if (lstQuickTab.SelectedIndex == 15)
            {
                key_combo = "hotkey_plus_f10";
                Globals.WriteINI("quicktab", key_combo, cmbQuickTab.SelectedIndex.ToString(), Globals.OptionsINI);
            }
            else if (lstQuickTab.SelectedIndex == 16)
            {
                key_combo = "hotkey_plus_f11";
                Globals.WriteINI("quicktab", key_combo, cmbQuickTab.SelectedIndex.ToString(), Globals.OptionsINI);
            }
            else if (lstQuickTab.SelectedIndex == 17)
            {
                key_combo = "hotkey_plus_f12";
                Globals.WriteINI("quicktab", key_combo, cmbQuickTab.SelectedIndex.ToString(), Globals.OptionsINI);
            }
            if (cmbQuickTab.SelectedIndex == 8)
            {
                // the user selected PositionWindow. Show our extra fields.
                string pos = Globals.ReadINI("quicktab", key_combo + "_pos", Globals.OptionsINI);
                txtPositionWindow.Text = pos;
                txtPositionWindow.Visible = true;
                lblPositionWindow.Visible = true;
                lblPositionWindowExamples.Visible = true;
            }
            else
            {
                txtPositionWindow.Visible = false;
                lblPositionWindow.Visible = false;
                lblPositionWindowExamples.Visible = false;
            }
        }

        private void button3_Click(object sender, EventArgs e)
        {
            frmMain_instance.RunURL("http://www.keybreeze.com/promote");
        }

        private void rdoSkinAlwaysCenter_CheckedChanged(object sender, EventArgs e)
        {

        }

        private void chkDoubleLetter_CheckedChanged(object sender, EventArgs e)
        {
            if (finished_loading == true)
            {
                int quicktab_enabled = Globals.ReadIntFromINI("quicktab", "double_letter_enabled", Globals.OptionsINI, 1);
                if (chkDoubleLetter.Checked == false)
                {
                    Globals.WriteINI("quicktab", "double_letter_enabled", "0", Globals.OptionsINI);
                    btnConfigureQuickTabWindowMappings.Enabled = false;
                }
                else
                {
                    Globals.WriteINI("quicktab", "double_letter_enabled", "1", Globals.OptionsINI);
                    btnConfigureQuickTabWindowMappings.Enabled = true;
                }
            }
        }

        private void chkIndexOnDemand_CheckedChanged(object sender, EventArgs e)
        {
            if (chkIndexOnDemand.Checked == true)
                autoindex_entries[lstAutoIndex.SelectedIndex].autoindex_indexondemand = 1;
            else
                autoindex_entries[lstAutoIndex.SelectedIndex].autoindex_indexondemand = 0;
        }

        private void label15_Click(object sender, EventArgs e)
        {
            MessageBox.Show("'Index on demand' will reindex the folder while you are typing a keyword. This works well for small folders like the Desktop or the Downloads folder.", "Keybreeze Desktop");
        }

        private void label19_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Keybreeze Desktop will search the web with this URL if you press one of the keys above. %keybreeze_entry% will be replaced with the entry that you type." + Environment.NewLine + Environment.NewLine + "Letter commands let you search websites for entries as well. Type 'add' as a keyword to see existing letter commands.", "Keybreeze Desktop");
        }

        private void label21_Click(object sender, EventArgs e)
        {
            
        }

        private void chkSearchOpenFolderWindows_CheckedChanged(object sender, EventArgs e)
        {

        }

        private void btnSearchAddFolder_Click(object sender, EventArgs e)
        {
            autoindexBrowseFolder.ShowDialog();
            if (autoindexBrowseFolder.SelectedPath.Length > 0)
            {
                search_entry new_entry = new search_entry();
                new_entry.search_subfolders = 1;
                new_entry.search_filetypes = "";
                new_entry.search_toplevelfolders = 1;
                new_entry.search_folder = autoindexBrowseFolder.SelectedPath;
                new_entry.search_checked = 1;
                search_entries.Add(new_entry);
                lstSearchFolders.Items.Add(autoindexBrowseFolder.SelectedPath);
                lstSearchFolders.SetItemChecked(lstSearchFolders.Items.Count - 1, true);
                lstSearchFolders.SelectedIndex = lstSearchFolders.Items.Count - 1;
                chkSearchSubfolders.Checked = true;
                chkSearchTopLevelFolders.Checked = true;
                chkSearchIndexAllFileTypes.Checked = true;
            }
        }

        private void btnSearchRemoveFolder_Click(object sender, EventArgs e)
        {
            if (lstSearchFolders.SelectedItem.ToString() == "Desktop" || lstSearchFolders.SelectedItem.ToString() == "Start Menu" || lstSearchFolders.SelectedItem.ToString() == "Start Menu (All users)" || lstSearchFolders.SelectedItem.ToString() == "My Documents")
            {
                MessageBox.Show("You cannot remove a preset folder, such as " + lstSearchFolders.SelectedItem + ", from the list. Please uncheck the item if you do not want it to be indexed.", "Keybreeze Desktop");
                return;
            }
            search_entries.RemoveAt(lstSearchFolders.SelectedIndex);
            lstSearchFolders.Items.RemoveAt(lstSearchFolders.SelectedIndex);
            try
            {
                lstSearchFolders.SelectedIndex = 0;
            }
            catch { }
            SaveSearchSettings();
        }

        private void chkSearchIndexAllFileTypes_CheckedChanged(object sender, EventArgs e)
        {
            if (chkSearchIndexAllFileTypes.Checked == false)
            {
                txtSearchFileTypes.Visible = true;
                lblSearchFileTypeHelp.Visible = true;
            }
            else
            {
                txtSearchFileTypes.Visible = false;
                txtSearchFileTypes.Text = "";
                lblSearchFileTypeHelp.Visible = false;
                search_entries[lstSearchFolders.SelectedIndex].search_filetypes = "";
            }
        }

        private void txtSearchFileTypes_TextChanged(object sender, EventArgs e)
        {
            search_entries[lstSearchFolders.SelectedIndex].search_filetypes = txtSearchFileTypes.Text;
        }

        private void lstSearchFolders_SelectedIndexChanged(object sender, EventArgs e)
        {
            string search_file = Globals.WorkPath + "\\search\\search.ini";
            string item_path;
            int number_plus_one = lstSearchFolders.SelectedIndex + 1;
            if (lstSearchFolders.SelectedIndex > -1)
            {
                item_path = Globals.ReadINI("search", "item" + number_plus_one + "-path", search_file);
                if (search_entries[lstSearchFolders.SelectedIndex].search_subfolders == 1)
                    chkSearchSubfolders.Checked = true;
                else
                    chkSearchSubfolders.Checked = false;
                if (search_entries[lstSearchFolders.SelectedIndex].search_filetypes == "")
                    chkSearchIndexAllFileTypes.Checked = true;
                else
                {
                    chkSearchIndexAllFileTypes.Checked = false;
                    txtSearchFileTypes.Visible = true;
                    txtSearchFileTypes.Text = search_entries[lstSearchFolders.SelectedIndex].search_filetypes;
                    lblSearchFileTypeHelp.Visible = true;
                }
                if (search_entries[lstSearchFolders.SelectedIndex].search_toplevelfolders == 1)
                    chkSearchTopLevelFolders.Checked = true;
                else
                    chkSearchTopLevelFolders.Checked = false;
            }
        }

        private void chkSearchSubfolders_CheckedChanged(object sender, EventArgs e)
        {
            if (chkSearchSubfolders.Checked == true)
            {
                search_entries[lstSearchFolders.SelectedIndex].search_subfolders = 1;
            }
            else
            {
                search_entries[lstSearchFolders.SelectedIndex].search_subfolders = 0;
            }
        }

        private void chkSearchTopLevelFolders_CheckedChanged(object sender, EventArgs e)
        {
            if (chkSearchTopLevelFolders.Checked == true)
            {
                search_entries[lstSearchFolders.SelectedIndex].search_toplevelfolders = 1;
            }
            else
            {
                search_entries[lstSearchFolders.SelectedIndex].search_toplevelfolders = 0;
            }
        }

        private void lblSearchFileTypeHelp_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Please type the file types you want Keybreeze Desktop to index, separated by commas, not including periods." + Environment.NewLine + Environment.NewLine + "Example: doc,jpg,bmp,gif,exe", "Keybreeze Desktop");
        }

        public string ReplaceAt(string input, int index, char newChar)
        {
            if (input == null)
            {
                throw new ArgumentNullException("input");
            }
            char[] chars = input.ToCharArray();
            chars[index] = newChar;
            return new string(chars);
        }

        private void txtPositionWindow_TextChanged(object sender, EventArgs e)
        {
            string key_combo;
            key_combo = lstQuickTab.SelectedItem.ToString().ToLower();
            key_combo = key_combo.Replace("+", "plus");
            key_combo = key_combo.Replace(" ", "_");
            Globals.WriteINI("quicktab", key_combo + "_pos", txtPositionWindow.Text, Globals.OptionsINI);
        }

        private void btnConfigureQuickTabWindowMappings_Click(object sender, EventArgs e)
        {
            frmOptions_QuickTab frmOptions_QuickTab_instance = new frmOptions_QuickTab();
            frmOptions_QuickTab_instance.ShowDialog();
        }

        private void label27_Click(object sender, EventArgs e)
        {

        }

        private void chkEnableClipboardMonitoring_CheckedChanged(object sender, EventArgs e)
        {
            if (chkEnableClipboardMonitoring.Checked == true)
            {
                txtClipboardMonitorInterval.Enabled = true;
                txtMaximumItemsForClipboard.Enabled = true;
                txtMaximumTextLengthPerItem.Enabled = true;
                chkUseTabKeyToToggleRows.Enabled = true;
                chkClearHistoryClearClipboard.Enabled = true;
            }
            else
            {
                txtClipboardMonitorInterval.Enabled = false;
                txtMaximumItemsForClipboard.Enabled = false;
                txtMaximumTextLengthPerItem.Enabled = false;
                chkUseTabKeyToToggleRows.Enabled = false;
                chkClearHistoryClearClipboard.Enabled = false;
            }
        }

        private void lblPositionWindowExamples_Click(object sender, EventArgs e)
        {
            frmMain_instance.RunURL("https://keybreeze.com/forum/post/57/control-windows-on-your-screen-with-quicktab");
        }
    }

    public class autoindex_entry
    {
        public int autoindex_checked;
        public string autoindex_folder;
        public int autoindex_indexondemand;
        public int autoindex_subfolders;
        public int autoindex_toplevelfolders;
        public string autoindex_filetypes;
    }

    public class search_entry
    {
        public int search_checked;
        public string search_folder;
        public int search_subfolders;
        public int search_toplevelfolders;
        public string search_filetypes;
    }
}
