﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Keybreeze
{
    public partial class frmOptions_QuickTab : Form
    {

        public List<QuickTab_Window_Mapping> quicktab_window_mappings = new List<QuickTab_Window_Mapping>();

        public frmOptions_QuickTab()
        {
            InitializeComponent();
        }

        private void frmOptions_QuickTab_Load(object sender, EventArgs e)
        {
            // load everything from the INI file.
            for (int i = 1; i < 27; i++)
            {
                if (Globals.ReadINI("quicktab", "window_mapping_letter_" + i, Globals.OptionsINI).Length > 0)
                {
                    QuickTab_Window_Mapping new_mapping = new QuickTab_Window_Mapping();
                    new_mapping.letter = Globals.ReadINI("quicktab", "window_mapping_letter_" + i, Globals.OptionsINI);
                    new_mapping.partial_window_title = Globals.ReadINI("quicktab", "window_mapping_title_" + i, Globals.OptionsINI);
                    new_mapping.class_name = Globals.ReadINI("quicktab", "window_mapping_class_" + i, Globals.OptionsINI);
                    quicktab_window_mappings.Add(new_mapping);
                    lstLetters.Items.Add("Hotkey + " + new_mapping.letter + new_mapping.letter);
                }
                else
                    break;
            }
            if (lstLetters.Items.Count > 0)
            {
                lstLetters.SelectedIndex = 0;
            }
        }

        private void label1_Click(object sender, EventArgs e)
        {

        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            frmInputBoxDialog frmInputBoxDialog_instance = new frmInputBoxDialog();
            frmInputBoxDialog_instance.lblLabel.Text = "Please enter the first letter of the window title.";
            frmInputBoxDialog_instance.ShowDialog();
            if (frmInputBoxDialog_instance.the_text.Length > 0)
            {
                string letter_only = frmInputBoxDialog_instance.the_text.Substring(0, 1).ToLower();
                string chosen_letter = "Hotkey + " + letter_only + letter_only;
                for (int i = 0; i < lstLetters.Items.Count; i++)
                {
                    if (lstLetters.Items[i].ToString() == chosen_letter)
                    {
                        MessageBox.Show("That letter has already been added.");
                        return;
                    }
                }
                QuickTab_Window_Mapping new_mapping = new QuickTab_Window_Mapping();
                new_mapping.letter = letter_only;
                quicktab_window_mappings.Add(new_mapping);
                lstLetters.Items.Add(chosen_letter);
                lstLetters.SelectedIndex = lstLetters.Items.Count - 1;
                txtPartialWindowTitle.Focus();
            }
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void txtPartialWindowTitle_TextChanged(object sender, EventArgs e)
        {
            int selected_index = lstLetters.SelectedIndex;
            if (selected_index > -1)
            {
                quicktab_window_mappings[selected_index].partial_window_title = txtPartialWindowTitle.Text;
            }
        }

        private void btnRemove_Click(object sender, EventArgs e)
        {
            int selected_index = lstLetters.SelectedIndex;
            if (selected_index > -1)
            {
                quicktab_window_mappings.RemoveAt(selected_index);
                lstLetters.Items.RemoveAt(selected_index);
            }
            if (lstLetters.Items.Count > 0)
                lstLetters.SelectedIndex = 0;
            else
            {
                txtPartialWindowTitle.Text = "";
                txtClassName.Text = "";
            }
        }

        private void lstLetters_SelectedIndexChanged(object sender, EventArgs e)
        {
            int selected_index = lstLetters.SelectedIndex;
            if (selected_index > -1)
            {
                txtClassName.Text = quicktab_window_mappings[selected_index].class_name;
                txtPartialWindowTitle.Text = quicktab_window_mappings[selected_index].partial_window_title;
            }
        }

        private void txtClassName_TextChanged(object sender, EventArgs e)
        {
            int selected_index = lstLetters.SelectedIndex;
            if (selected_index > -1)
            {
                quicktab_window_mappings[selected_index].class_name = txtClassName.Text;
            }
        }

        private void frmOptions_QuickTab_FormClosing(object sender, FormClosingEventArgs e)
        {
            // save everything to the INI file.
            for (int i = 1; i < 27; i++)
            {
                if (Globals.ReadINI("quicktab", "window_mapping_letter_" + i, Globals.OptionsINI).Length > 0)
                {
                    Globals.WriteINI("quicktab", "window_mapping_letter_" + i, "", Globals.OptionsINI);
                    Globals.WriteINI("quicktab", "window_mapping_title_" + i, "", Globals.OptionsINI);
                    Globals.WriteINI("quicktab", "window_mapping_class_" + i, "", Globals.OptionsINI);
                }
                else
                    break;
            }
            int count = 0;
            foreach (QuickTab_Window_Mapping this_mapping in quicktab_window_mappings)
            {
                count++;
                Globals.WriteINI("quicktab", "window_mapping_letter_" + count, this_mapping.letter, Globals.OptionsINI);
                Globals.WriteINI("quicktab", "window_mapping_title_" + count, this_mapping.partial_window_title, Globals.OptionsINI);
                Globals.WriteINI("quicktab", "window_mapping_class_" + count, this_mapping.class_name, Globals.OptionsINI);
            }
        }
    }

    public class QuickTab_Window_Mapping
    {
        public string letter = "";
        public string partial_window_title = "";
        public string class_name = "";
    }
}
