/*
Keybreeze Desktop
Copyright  2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Etier.IconHelper;
using System.Diagnostics;

namespace Keybreeze
{
    public partial class frmPopup : Form
    {
        frmMain frmMain_instance;
        
        public int total_rows_visible = Globals.ReadIntFromINI("autocomplete", "items", Globals.SkinINI, 5);

        // global variables for the highlight style.
        public Color autocomplete_highlight_color;
        public Color autocomplete_keyword_color;
        public Color autocomplete_keyword_selected_color;
        public bool autocomplete_highlight = true;
        public string autocomplete_keyword_font = Globals.ReadINI("autocomplete", "keyword_font", Globals.SkinINI);
        public int autocomplete_keyword_font_size = Globals.ReadIntFromINI("autocomplete", "keyword_font_size", Globals.SkinINI, 10);
        public int autocomplete_keyword_selected_font_size = Globals.ReadIntFromINI("autocomplete", "keyword_selected_font_size", Globals.SkinINI, 10);
        public int autocomplete_keyword_font_bold = Globals.ReadIntFromINI("autocomplete", "keyword_font_bold", Globals.SkinINI, 0);
        public int autocomplete_keyword_selected_font_bold = Globals.ReadIntFromINI("autocomplete", "keyword_selected_font_bold", Globals.SkinINI, 0);
        public int autocomplete_keyword_font_italic = Globals.ReadIntFromINI("autocomplete", "keyword_font_italic", Globals.SkinINI, 0);
        public int autocomplete_keyword_selected_font_italic = Globals.ReadIntFromINI("autocomplete", "keyword_selected_font_italic", Globals.SkinINI, 0);

        public Color autocomplete_description_color;
        public Color autocomplete_description_selected_color;
        public string autocomplete_description_font = Globals.ReadINI("autocomplete", "description_font", Globals.SkinINI);
        public int autocomplete_description_font_size = Globals.ReadIntFromINI("autocomplete", "description_font_size", Globals.SkinINI, 10);
        public int autocomplete_description_selected_font_size = Globals.ReadIntFromINI("autocomplete", "description_selected_font_size", Globals.SkinINI, 10);
        public int autocomplete_description_font_bold = Globals.ReadIntFromINI("autocomplete", "description_font_bold", Globals.SkinINI, 0);
        public int autocomplete_description_selected_font_bold = Globals.ReadIntFromINI("autocomplete", "description_selected_font_bold", Globals.SkinINI, 0);
        public int autocomplete_description_font_italic = Globals.ReadIntFromINI("autocomplete", "description_font_italic", Globals.SkinINI, 0);
        public int autocomplete_description_selected_font_italic = Globals.ReadIntFromINI("autocomplete", "description_selected_font_italic", Globals.SkinINI, 0);

        public string autocomplete_file_extension_font = Globals.ReadINI("autocomplete", "file_extension_font", Globals.SkinINI);
        public int autocomplete_file_extension_font_bold = Globals.ReadIntFromINI("autocomplete", "file_extension_font_bold", Globals.SkinINI, 10);
        public int autocomplete_file_extension_font_italic = Globals.ReadIntFromINI("autocomplete", "file_extension_font_italic", Globals.SkinINI, 0);
        public int autocomplete_file_extension_font_size = Globals.ReadIntFromINI("autocomplete", "file_extension_font_size", Globals.SkinINI, 0);

        public int autocomplete_max_matches = Globals.ReadIntFromINI("options", "autocomplete_max_matches", Globals.OptionsINI, 15);

        public struct autocomplete_file_extension_label
        {
            public string extension;
            public Color forecolor;
        }
        public List<autocomplete_file_extension_label> autocomplete_file_extension_labels = new List<autocomplete_file_extension_label>();
        public Color row_file_extension_label_files_forecolor;
        public Color row_file_extension_label_other_forecolor;

        // 3/14/2015 Update. Going to use a struct for autocomplete matches.
        public struct autocomplete_matches
        {
            // row_type can be "keyword", "previous entry", or "autoindex".
            public string row_type;
            public string row_name;
            public string row_target;
            public string row_path;
            public string file_ext;
        }
        public List<autocomplete_matches> autocomplete_match_list = new List<autocomplete_matches>();
        // selected_row_index is the visible row that's selected. selected_match_index is the selected match index
        // we may delete these.
        public int selected_row_index = 0;
        public int selected_match_index = 0;


        // make some variables for the autocomplete popup list
        public List<Panel> row_panels = new List<Panel>();
        public List<PictureBox> row_icons = new List<PictureBox>();
        public List<Label> row_keywords = new List<Label>();
        public List<Label> row_descriptions = new List<Label>();
        public List<Label> row_file_extension_labels = new List<Label>();
        //public List<Label> row_paths = new List<Label>();

        // popup_list_elements will contain a list of keywords that match the user's entry.
        //public List<int> popup_list_elements = new List<int>();
        // popup_list_element_type will contain 0 for a keyword, 1 for autocomplete, and 2 for autoindex.
        // public List<int> popup_list_element_type = new List<int>();
        // the autoindex_icon_tracker is used for tracking icon positions in the autoindex_imageLists.
        public int autoindex_icon_tracker = 0;
        // resolve_icons_to_full_paths is for LNK files. It will provide the real icon instead of the shortcut icon where the annoying arrow is overlayed onto the original icon.
        public int resolve_icons_to_full_paths = 0;

        public frmPopup(frmMain parent)
        {
            if (Globals.logging == 1)
                Globals.DebugWrite("frmPopup -- starting function.");
            if (Globals.logging == 1)
                Globals.DebugWrite("InitializeComponent()");
            InitializeComponent();
            frmMain_instance = parent;

            // set up the size, position of elements.
            string form_width = Globals.ReadINI("autocomplete", "width", Globals.SkinINI);
            string form_height = Globals.ReadINI("autocomplete", "height", Globals.SkinINI);
            string backcolor = Globals.ReadINI("autocomplete", "backcolor", Globals.SkinINI);
            string background_image = Globals.ReadINI("autocomplete", "background_image", Globals.SkinINI);
            this.Width = Globals.ReadIntFromINI("autocomplete", "width", Globals.SkinINI, 406);
            this.Height = Globals.ReadIntFromINI("autocomplete", "height", Globals.SkinINI, 206);
            int opacity = Globals.ReadIntFromINI("autocomplete", "opacity", Globals.SkinINI, 0);
            if (opacity > 0 && opacity < 100)
                Globals.MakeWindowTranslucent(this.Handle, opacity, false);

            this.BackColor = System.Drawing.ColorTranslator.FromHtml(backcolor);
            imageListSmall.ColorDepth = ColorDepth.Depth32Bit;
            imageListLarge.ColorDepth = ColorDepth.Depth32Bit;

            // should we resolve icons of .lnk files to their real paths? This will hide the arrow that it placed over the icon.
            resolve_icons_to_full_paths = Globals.ReadIntFromINI("options", "resolve_icons_to_full_paths", Globals.OptionsINI, 1);

            CreateAutoComplete();
            if (background_image.Length > 0)
            {
                pictureBox1.Height = this.Height;
                pictureBox1.Width = this.Width;
                pictureBox1.Image = Image.FromFile(Globals.SkinFolder + "\\" + background_image);
                pictureBox1.Visible = true;
                for (int i = 0; i < total_rows_visible; i++)
                {
                    row_panels[i].BackColor = System.Drawing.Color.Transparent;
                    row_panels[i].Parent = pictureBox1;
                    row_keywords[i].BackColor = System.Drawing.Color.Transparent;
                    //row_keywords[i].Parent = pictureBox1;
                    row_descriptions[i].BackColor = System.Drawing.Color.Transparent;
                    //row_descriptions[i].Parent = pictureBox1;
                    if (Globals.skin_icon_type == 1)
                    {
                        row_icons[i].BackColor = System.Drawing.Color.Transparent;
                        //row_icons[i].Parent = pictureBox1;
                    }
                }
            }
            btnEnter.Top = 0;
            btnEnter.Left = this.Width;
            if (Globals.logging == 1)
                Globals.DebugWrite("frmPopup() -- ending function.");
        }

        private void Form1_Load(object sender, EventArgs e)
        {

        }

        public int Row_Add_Row(string row_name, string row_target, string row_type, string row_path)
        {
            // add an entry to the autocomplete popup list. entry will either be a previously typed
            // entry or something from the autoindex.
            // returns -1 if the threshold has been reached or an error occurred. Otherwise it returns 1.

            // have we hit our threshold of items?
            if (autocomplete_match_list.Count >= autocomplete_max_matches)
            {
                return -1;
            }
            // add the entry to our list of matches first.
            autocomplete_matches new_row = new frmPopup.autocomplete_matches();
            new_row.row_name = row_name;
            new_row.row_target = row_target;
            new_row.row_path = row_path;
            new_row.row_type = row_type;
            if (Globals.skin_icon_type != 0)
                new_row.file_ext = Globals.GetTargetType_For_AutoComplete(row_target);
            autocomplete_match_list.Add(new_row);
            int listCount = autocomplete_match_list.Count;
            int row_position = listCount - 1;

            // now let's add it to our rows as long as the row is visible, i.e. we can't display the seventh match if we only have six rows.
            if (listCount > 0 && listCount <= total_rows_visible)
            {
                try
                {
                    /*
                     * // 3/10/2016 update by Andrew: commenting this out because it was causing problems with icons not refreshing after the first autocomplete entry.
                    if (row_keywords[row_position].Text == row_name)
                    {
                        // the row hasn't changed since before, so there is no need to refresh the icons or keyword labels.
                        return;
                    }
                     */
                    if (Globals.skin_icon_type == 1)
                    {
                        // show the icon.
                        // if this is our first one, let's clear the image list.
                        if (row_position == 0)
                            Clear_AutoComplete_Icons();
                        Add_Icon_To_AutoComplete(row_position, row_target, new_row.file_ext);
                    }
                    else if (Globals.skin_icon_type == 2)
                    {
                        // need to set forecolor and text of this one.
                        string target_file_ext = Globals.GetTargetType(row_target);
                        row_file_extension_labels[row_position].Text = target_file_ext;
                        bool found_label = false;
                        foreach (autocomplete_file_extension_label this_label in autocomplete_file_extension_labels)
                        {
                            if (this_label.extension == target_file_ext)
                            {
                                row_file_extension_labels[row_position].ForeColor = this_label.forecolor;
                                found_label = true;
                                break;
                            }
                        }
                        if (found_label == false)
                        {
                            // we haven't found a label yet. Use the files label.
                            row_file_extension_labels[row_position].ForeColor = row_file_extension_label_files_forecolor;
                        }
                    }
                    else if (Globals.skin_icon_type == 3)
                    {
                        // need to set forecolor and text of this one.
                        string target_file_ext = Globals.GetTargetType(row_target);
                        bool found_label = false;
                        foreach (autocomplete_file_extension_label this_label in autocomplete_file_extension_labels)
                        {
                            if (this_label.extension == target_file_ext)
                            {
                                row_keywords[row_position].ForeColor = this_label.forecolor;
                                found_label = true;
                                break;
                            }
                        }
                        if (found_label == false)
                        {
                            // we haven't found a label yet. Use the files label.
                            row_keywords[row_position].ForeColor = row_file_extension_label_files_forecolor;
                        }
                    }
                }
                catch { }
                row_keywords[row_position].Text = row_name;
                row_descriptions[row_position].Text = Globals.ReturnShortenedPath(row_target, row_path);
                row_descriptions[row_position].Tag = row_position;
                /*
                if (row_path != null && row_path.Length > 0 && frmMain_instance.show_paths == true)
                {
                    row_keywords[row_position].AutoSize = true;
                    row_paths[row_position].AutoSize = true;
                    row_paths[row_position].Left = row_keywords[row_position].Left + row_keywords[row_position].Width + 12;
                    row_paths[row_position].Text = row_path;
                    row_paths[row_position].Visible = true;
                }
                 * */
                if (listCount == 1)
                {
                    // we are adding our first item. Let's highlight it.
                    HighlightRow(0, true);
                }
            }
            return 1;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            
        }

        public void button2_Click(object sender, EventArgs e)
        {

        }

        private void CreateAutoComplete()
        {
            // this will create elements for the autocomplete list -- labels, panels, icons, etc.
            // set variables and read all the INI settings first.

            if (Globals.logging == 1)
                Globals.DebugWrite("CreateAutoComplete() -- starting function.");

            int panel_height = Globals.ReadIntFromINI("autocomplete", "panel_height", Globals.SkinINI, 40);
            int panel_height_so_far = Globals.ReadIntFromINI("autocomplete", "panel_top", Globals.SkinINI, 1);
            int panel_left = Globals.ReadIntFromINI("autocomplete", "panel_top", Globals.SkinINI, 1);
            int panel_width_offset = Globals.ReadIntFromINI("autocomplete", "panel_width_offset", Globals.SkinINI, 0);
            int keyword_left = Globals.ReadIntFromINI("autocomplete", "keyword_left", Globals.SkinINI, 34);
            int keyword_top = Globals.ReadIntFromINI("autocomplete", "keyword_top", Globals.SkinINI, 3);
            int description_left = Globals.ReadIntFromINI("autocomplete", "description_left", Globals.SkinINI, 35);
            int description_top = Globals.ReadIntFromINI("autocomplete", "description_top", Globals.SkinINI, 21);
            int show_description = Globals.ReadIntFromINI("autocomplete", "show_description", Globals.SkinINI, 1);
            int keyword_width_offset = Globals.ReadIntFromINI("autocomplete", "keyword_width_offset", Globals.SkinINI, -40);
            int description_width_offset = Globals.ReadIntFromINI("autocomplete", "description_width_offset", Globals.SkinINI, -40);
            int keyword_bold = Globals.ReadIntFromINI("autocomplete", "keyword_font_bold", Globals.SkinINI, 0);
            int keyword_italic = Globals.ReadIntFromINI("autocomplete", "keyword_font_italic", Globals.SkinINI, 0);
            int description_bold = Globals.ReadIntFromINI("autocomplete", "description_font_bold", Globals.SkinINI, 0);
            int description_italic = Globals.ReadIntFromINI("autocomplete", "description_font_italic", Globals.SkinINI, 0);

            int icon_top = Globals.ReadIntFromINI("autocomplete", "icon_top", Globals.SkinINI, 4);
            int icon_left = Globals.ReadIntFromINI("autocomplete", "icon_left", Globals.SkinINI, 3);
            int file_extension_font_bold = Globals.ReadIntFromINI("autocomplete", "file_extension_font_bold", Globals.SkinINI, 0);
            int file_extension_font_italic = Globals.ReadIntFromINI("autocomplete", "file_extension_font_italic", Globals.SkinINI, 0);

            string temp_autocomplete_highlight_color = Globals.ReadINI("autocomplete", "highlightcolor", Globals.SkinINI);
            string temp_autocomplete_keyword_color = Globals.ReadINI("autocomplete", "keyword_color", Globals.SkinINI);
            string temp_autocomplete_keyword_selected_color = Globals.ReadINI("autocomplete", "keyword_selected_color", Globals.SkinINI);
            string temp_autocomplete_description_color = Globals.ReadINI("autocomplete", "description_color", Globals.SkinINI);
            string temp_autocomplete_description_selected_color = Globals.ReadINI("autocomplete", "description_selected_color", Globals.SkinINI);

            //int path_showing = Globals.ReadIntFromINI("autocomplete", "path_showing", Globals.SkinINI, 0);

            if (temp_autocomplete_highlight_color.Length == 0)
            {
                autocomplete_highlight = false;
            }
            else
            {
                autocomplete_highlight = true;
                autocomplete_highlight_color = System.Drawing.ColorTranslator.FromHtml(temp_autocomplete_highlight_color);
            }
            autocomplete_keyword_color = System.Drawing.ColorTranslator.FromHtml(temp_autocomplete_keyword_color);
            if (temp_autocomplete_keyword_selected_color.Length == 0)
                autocomplete_keyword_selected_color = autocomplete_keyword_color;
            else
                autocomplete_keyword_selected_color = System.Drawing.ColorTranslator.FromHtml(temp_autocomplete_keyword_selected_color);
            autocomplete_description_color = System.Drawing.ColorTranslator.FromHtml(temp_autocomplete_description_color);
            if (temp_autocomplete_description_selected_color.Length == 0)
                autocomplete_description_selected_color = autocomplete_description_color;
            else
                autocomplete_description_selected_color = System.Drawing.ColorTranslator.FromHtml(temp_autocomplete_description_selected_color);

            if (total_rows_visible > 8)
                total_rows_visible = 8;
            else if (total_rows_visible < 1)
                total_rows_visible = 1;

            for (int i = 0; i < total_rows_visible; i++)
            {
                Panel row_panel = new Panel();
                row_panels.Add(row_panel);
                // only create a picture box if we are showing icons.
                if (Globals.skin_icon_type == 1)
                {
                    // we are showing icons for each entry.
                    PictureBox row_icon = new PictureBox();
                    row_icons.Add(row_icon);
                }
                else if (Globals.skin_icon_type == 2)
                {
                    // we are showing file extension labels for each entry.
                    Label row_file_extension_label = new Label();
                    row_file_extension_labels.Add(row_file_extension_label);
                }
                Label row_keyword = new Label();
                Label row_description = new Label();
                row_keywords.Add(row_keyword);
                row_descriptions.Add(row_description);

                row_panels[i].Size = new System.Drawing.Size(this.Width - 2 + panel_width_offset, panel_height);
                row_panels[i].Location = new System.Drawing.Point(panel_left, panel_height_so_far);
                row_panels[i].BackColor = Color.Transparent;
                panel_height_so_far = panel_height_so_far + panel_height + 1;

                this.Controls.Add(row_panels[i]);
                //label_width = row_panels[i].Width - 40;

                if (Globals.skin_icon_type == 1)
                {
                    // we are showing the respective icons for each entry.
                    row_icons[i].Size = new System.Drawing.Size(32, 32);
                    row_icons[i].Location = new System.Drawing.Point(icon_left, icon_top);
                }
                else if (Globals.skin_icon_type == 2)
                {
                    // we are showing file extension labels for each entry.
                    row_file_extension_labels[i].AutoSize = true;
                    row_file_extension_labels[i].Location = new System.Drawing.Point(icon_left, icon_top);
                    row_file_extension_labels[i].Visible = true;
                    FontStyle file_extension_style = FontStyle.Regular;
                    if (file_extension_font_bold == 1)
                    {
                        file_extension_style = file_extension_style | FontStyle.Bold;
                    }
                    if (file_extension_font_italic == 1)
                    {
                        file_extension_style = file_extension_style | FontStyle.Italic;
                    }
                    row_file_extension_labels[i].Font = new System.Drawing.Font(autocomplete_file_extension_font, autocomplete_file_extension_font_size, file_extension_style, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
                }
                // this must be set to false to avoid flickering as the user types keywords.
                row_keywords[i].AutoSize = false;
                row_keywords[i].Size = new System.Drawing.Size(row_panels[i].Width + keyword_width_offset, 18);
                row_keywords[i].Location = new System.Drawing.Point(keyword_left, keyword_top);
                row_keywords[i].ForeColor = autocomplete_keyword_color;
                FontStyle myfontstyle1 = FontStyle.Regular;
                if (keyword_bold == 1)
                {
                    myfontstyle1 = myfontstyle1 | FontStyle.Bold;
                }
                if (keyword_italic == 1)
                {
                    myfontstyle1 = myfontstyle1 | FontStyle.Italic;
                }
                FontStyle myfontstyle2 = FontStyle.Regular;
                if (description_bold == 1)
                {
                    myfontstyle2 = myfontstyle2 | FontStyle.Bold;
                }
                if (description_italic == 1)
                {
                    myfontstyle2 = myfontstyle2 | FontStyle.Italic;
                }
                row_keywords[i].Font = new System.Drawing.Font(autocomplete_keyword_font, autocomplete_keyword_font_size, myfontstyle1, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
                row_descriptions[i].AutoSize = false;
                row_descriptions[i].Size = new System.Drawing.Size(row_panels[i].Width + description_width_offset, 18);
                row_descriptions[i].Location = new System.Drawing.Point(description_left, description_top);
                row_descriptions[i].ForeColor = autocomplete_description_color;
                row_descriptions[i].Font = new System.Drawing.Font(autocomplete_description_font, autocomplete_description_font_size, myfontstyle2, System.Drawing.GraphicsUnit.Point, ((byte)(0)));

                /*
                if (path_showing == 1)
                {
                    frmMain_instance.show_paths = true;
                    int path_left_offset = Globals.ReadIntFromINI("autocomplete", "path_left_offset", Globals.SkinINI, 0);
                    string path_font = Globals.ReadINI("autocomplete", "path_font", Globals.SkinINI);
                    int path_font_size = Globals.ReadIntFromINI("autocomplete", "path_font_size", Globals.SkinINI, 0);
                    string path_color_html = Globals.ReadINI("autocomplete", "path_color", Globals.SkinINI);

                    Label row_path = new Label();
                    row_paths.Add(row_path);
                    row_paths[i].AutoSize = true;
                    row_paths[i].Size = new System.Drawing.Size(row_panels[i].Width + keyword_width_offset, 18);
                    row_paths[i].Location = new System.Drawing.Point(keyword_left, keyword_top);
                    Color path_color = System.Drawing.ColorTranslator.FromHtml(path_color_html);
                    row_paths[i].ForeColor = path_color;
                    row_paths[i].Font = new System.Drawing.Font(path_font, path_font_size, myfontstyle1, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
                }
                else
                {
                    frmMain_instance.show_paths = false;
                }
                 * */

                if (show_description == 0)
                    row_descriptions[i].Visible = false;
                if (Globals.skin_icon_type == 1)
                    row_panels[i].Controls.Add(row_icons[i]);
                else if (Globals.skin_icon_type == 2)
                {
                    row_panels[i].Controls.Add(row_file_extension_labels[i]);
                }
                row_panels[i].Controls.Add(row_keywords[i]);
                row_panels[i].Controls.Add(row_descriptions[i]);
                /*
                if (path_showing == 1)
                {
                    row_panels[i].Controls.Add(row_paths[i]);
                }
                 */
            }
            
            // add a click event to the controls we just created.
            if (total_rows_visible > 0)
            {
                row_keywords[0].MouseDown += new System.Windows.Forms.MouseEventHandler(row_item_click_0);
                row_descriptions[0].MouseDown += new System.Windows.Forms.MouseEventHandler(row_item_click_0);
                if (Globals.skin_icon_type == 1)
                    row_icons[0].MouseDown += new System.Windows.Forms.MouseEventHandler(row_item_click_0);
            }
            if (total_rows_visible > 1)
            {
                row_keywords[1].MouseDown += new System.Windows.Forms.MouseEventHandler(row_item_click_1);
                row_descriptions[1].MouseDown += new System.Windows.Forms.MouseEventHandler(row_item_click_1);
                if (Globals.skin_icon_type == 1)
                    row_icons[1].MouseDown += new System.Windows.Forms.MouseEventHandler(row_item_click_1);
            }
            if (total_rows_visible > 2)
            {
                row_keywords[2].MouseDown += new System.Windows.Forms.MouseEventHandler(row_item_click_2);
                row_descriptions[2].MouseDown += new System.Windows.Forms.MouseEventHandler(row_item_click_2);
                if (Globals.skin_icon_type == 1)
                    row_icons[2].MouseDown += new System.Windows.Forms.MouseEventHandler(row_item_click_2);
            }
            if (total_rows_visible > 3)
            {
                row_keywords[3].MouseDown += new System.Windows.Forms.MouseEventHandler(row_item_click_3);
                row_descriptions[3].MouseDown += new System.Windows.Forms.MouseEventHandler(row_item_click_3);
                if (Globals.skin_icon_type == 1)
                    row_icons[3].MouseDown += new System.Windows.Forms.MouseEventHandler(row_item_click_3);
            }
            if (total_rows_visible > 4)
            {
                row_keywords[4].MouseDown += new System.Windows.Forms.MouseEventHandler(row_item_click_4);
                row_descriptions[4].MouseDown += new System.Windows.Forms.MouseEventHandler(row_item_click_4);
                if (Globals.skin_icon_type == 1)
                    row_icons[4].MouseDown += new System.Windows.Forms.MouseEventHandler(row_item_click_4);
            }
            if (total_rows_visible > 5)
            {
                row_keywords[5].MouseDown += new System.Windows.Forms.MouseEventHandler(row_item_click_5);
                row_descriptions[5].MouseDown += new System.Windows.Forms.MouseEventHandler(row_item_click_5);
                if (Globals.skin_icon_type == 1)
                    row_icons[5].MouseDown += new System.Windows.Forms.MouseEventHandler(row_item_click_5);
            }
            if (total_rows_visible > 6)
            {
                row_keywords[6].MouseDown += new System.Windows.Forms.MouseEventHandler(row_item_click_6);
                row_descriptions[6].MouseDown += new System.Windows.Forms.MouseEventHandler(row_item_click_6);
                if (Globals.skin_icon_type == 1)
                    row_icons[6].MouseDown += new System.Windows.Forms.MouseEventHandler(row_item_click_6);
            }
            if (total_rows_visible > 7)
            {
                row_keywords[7].MouseDown += new System.Windows.Forms.MouseEventHandler(row_item_click_7);
                row_descriptions[7].MouseDown += new System.Windows.Forms.MouseEventHandler(row_item_click_7);
                if (Globals.skin_icon_type == 1)
                    row_icons[7].MouseDown += new System.Windows.Forms.MouseEventHandler(row_item_click_7);
            }

            // if we are showing file extension labels, we need to set variables for the colors.
            if (Globals.skin_icon_type == 2 || Globals.skin_icon_type == 3)
            {
                try
                {
                    row_file_extension_label_files_forecolor = System.Drawing.ColorTranslator.FromHtml(Globals.ReadINI("autocomplete", "row_file_extension_label_files_forecolor", Globals.SkinINI));
                }
                catch
                {
                    row_file_extension_label_files_forecolor = Color.White;
                }
                for (int i = 1; i < 50; i++)
                {
                    string current_label = Globals.ReadINI("autocomplete", "row_file_extension_label" + i + "a", Globals.SkinINI);
                    if (current_label != "")
                    {
                        string current_label_forecolor = Globals.ReadINI("autocomplete", "row_file_extension_label" + i + "b", Globals.SkinINI);
                        autocomplete_file_extension_label new_label = new autocomplete_file_extension_label();
                        new_label.extension = current_label;
                        try
                        {
                            new_label.forecolor = System.Drawing.ColorTranslator.FromHtml(current_label_forecolor);
                        }
                        catch
                        {
                            new_label.forecolor = Color.White;
                        }
                        autocomplete_file_extension_labels.Add(new_label);
                    }
                    else
                        break;
                }
            }
        }

        public void HighlightRow(int row_num, bool highlight = true)
        {
            if (highlight == true)
            {
                if (autocomplete_keyword_selected_color != autocomplete_keyword_color)
                    row_keywords[row_num].ForeColor = autocomplete_keyword_selected_color;
                if (autocomplete_description_selected_color != autocomplete_description_color)
                    row_descriptions[row_num].ForeColor = autocomplete_description_selected_color;
                // check for bold, italic, or size changes for keyword.
                if (autocomplete_keyword_selected_font_size != autocomplete_keyword_font_size || autocomplete_keyword_selected_font_bold != autocomplete_keyword_font_bold || autocomplete_keyword_selected_font_italic != autocomplete_keyword_font_italic)
                {
                    FontStyle myfontstyle = FontStyle.Regular;
                    if (autocomplete_keyword_selected_font_bold == 1)
                    {
                        myfontstyle = myfontstyle | FontStyle.Bold;
                    }
                    if (autocomplete_keyword_selected_font_italic == 1)
                    {
                        myfontstyle = myfontstyle | FontStyle.Italic;
                    }
                    row_keywords[row_num].Font = new System.Drawing.Font(autocomplete_keyword_font, autocomplete_keyword_selected_font_size, myfontstyle, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
                }
                // check for bold, italic, or size changes for description.
                if (autocomplete_description_selected_font_size != autocomplete_description_font_size || autocomplete_description_selected_font_bold != autocomplete_description_font_bold || autocomplete_description_selected_font_italic != autocomplete_description_font_italic)
                {
                    FontStyle myfontstyle = FontStyle.Regular;
                    if (autocomplete_description_selected_font_bold == 1)
                    {
                        myfontstyle = myfontstyle | FontStyle.Bold;
                    }
                    if (autocomplete_description_selected_font_italic == 1)
                    {
                        myfontstyle = myfontstyle | FontStyle.Italic;
                    }
                    row_descriptions[row_num].Font = new System.Drawing.Font(autocomplete_description_font, autocomplete_description_selected_font_size, myfontstyle, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
                }
                if (autocomplete_highlight == true)
                    row_panels[row_num].BackColor = autocomplete_highlight_color;
            }
            else
            {
                // remove the highlight
                if (autocomplete_keyword_selected_color != autocomplete_keyword_color)
                    row_keywords[row_num].ForeColor = autocomplete_keyword_color;
                if (autocomplete_description_selected_color != autocomplete_description_color)
                    row_descriptions[row_num].ForeColor = autocomplete_description_color;
                // check for bold, italic, or size changes for keyword.
                if (autocomplete_keyword_selected_font_size != autocomplete_keyword_font_size || autocomplete_keyword_selected_font_bold != autocomplete_keyword_font_bold || autocomplete_keyword_selected_font_italic != autocomplete_keyword_font_italic)
                {
                    FontStyle myfontstyle = FontStyle.Regular;
                    if (autocomplete_keyword_font_bold == 1)
                    {
                        myfontstyle = myfontstyle | FontStyle.Bold;
                    }
                    if (autocomplete_keyword_font_italic == 1)
                    {
                        myfontstyle = myfontstyle | FontStyle.Italic;
                    }
                    row_keywords[row_num].Font = new System.Drawing.Font(autocomplete_keyword_font, autocomplete_keyword_font_size, myfontstyle, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
                }
                // check for bold, italic, or changes for description.
                if (autocomplete_description_selected_font_size != autocomplete_description_font_size || autocomplete_description_selected_font_bold != autocomplete_description_font_bold || autocomplete_description_selected_font_italic != autocomplete_description_font_italic)
                {
                    FontStyle myfontstyle = FontStyle.Regular;
                    if (autocomplete_description_font_bold == 1)
                    {
                        myfontstyle = myfontstyle | FontStyle.Bold;
                    }
                    if (autocomplete_description_font_italic == 1)
                    {
                        myfontstyle = myfontstyle | FontStyle.Italic;
                    }
                    row_descriptions[row_num].Font = new System.Drawing.Font(autocomplete_description_font, autocomplete_description_font_size, myfontstyle, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
                }
                if (autocomplete_highlight == true)
                    row_panels[row_num].BackColor = Color.Transparent;
            }
        }

        public void SetupContextMenu()
        {
            // this will configure options for the popup menu such as Edit, Delete, etc.
            // we want to show Launch, Edit, and Delete for keywords.
            // we want to show Launch and Delete for autocomplete entries.
            // we want to show Launch, Add custom keyword, and Remove for autoindex entries.
            //if (popup_list_element_type[row_selection] == 0)
            if (autocomplete_match_list[selected_match_index].row_type == "keyword")
            {
                launchToolStripMenuItem.Visible = true;
                toolStripSeparator1.Visible = true;
                addCustomKeywordToolStripMenuItem.Visible = false;
                changeToolStripMenuItem.Visible = true;
                removeToolStripMenuItem.Visible = true;
            }
            //else if (popup_list_element_type[row_selection] == 1)
            else if (autocomplete_match_list[selected_match_index].row_type == "previous entry")
            {
                launchToolStripMenuItem.Visible = true;
                toolStripSeparator1.Visible = true;
                addCustomKeywordToolStripMenuItem.Visible = false;
                changeToolStripMenuItem.Visible = false;
                removeToolStripMenuItem.Visible = true;
            }
            else
            {
                launchToolStripMenuItem.Visible = true;
                toolStripSeparator1.Visible = true;
                addCustomKeywordToolStripMenuItem.Visible = true;
                changeToolStripMenuItem.Visible = false;
                removeToolStripMenuItem.Visible = true;
            }
        }

        private void row_item_click_0(object sender, MouseEventArgs e)
        {
            // select the item first
            HighlightRow(selected_row_index, false);
            Change_Selection(0);
            HighlightRow(selected_row_index, true);
            // show the context menu
            SetupContextMenu();
            contextMenu1.Show(row_descriptions[0], e.Location);
        }
        private void row_item_click_1(object sender, MouseEventArgs e)
        {
            // select the item first
            HighlightRow(selected_row_index, false);
            Change_Selection(1);
            HighlightRow(selected_row_index, true);
            // show the context menu
            SetupContextMenu();
            contextMenu1.Show(row_descriptions[1], e.Location);
        }
        private void row_item_click_2(object sender, MouseEventArgs e)
        {
            // select the item first
            HighlightRow(selected_row_index, false);
            Change_Selection(2);
            HighlightRow(selected_row_index, true);
            // show the context menu
            SetupContextMenu();
            contextMenu1.Show(row_descriptions[2], e.Location);
        }
        private void row_item_click_3(object sender, MouseEventArgs e)
        {
            // select the item first
            HighlightRow(selected_row_index, false);
            Change_Selection(3);
            HighlightRow(selected_row_index, true);
            // show the context menu
            SetupContextMenu();
            contextMenu1.Show(row_descriptions[3], e.Location);
        }
        private void row_item_click_4(object sender, MouseEventArgs e)
        {
            // select the item first
            HighlightRow(selected_row_index, false);
            Change_Selection(4);
            HighlightRow(selected_row_index, true);
            // show the context menu
            SetupContextMenu();
            contextMenu1.Show(row_descriptions[4], e.Location);
        }
        private void row_item_click_5(object sender, MouseEventArgs e)
        {
            // select the item first
            HighlightRow(selected_row_index, false);
            Change_Selection(5);
            HighlightRow(selected_row_index, true);
            // show the context menu
            SetupContextMenu();
            contextMenu1.Show(row_descriptions[5], e.Location);
        }
        private void row_item_click_6(object sender, MouseEventArgs e)
        {
            // select the item first
            HighlightRow(selected_row_index, false);
            Change_Selection(6);
            HighlightRow(selected_row_index, true);
            // show the context menu
            SetupContextMenu();
            contextMenu1.Show(row_descriptions[6], e.Location);
        }
        private void row_item_click_7(object sender, MouseEventArgs e)
        {
            // select the item first
            HighlightRow(selected_row_index, false);
            Change_Selection(7);
            HighlightRow(selected_row_index, true);
            // show the context menu
            SetupContextMenu();
            contextMenu1.Show(row_descriptions[7], e.Location);
        }

        private void Change_Selection(int new_selection)
        {
            int current_label_selection = selected_row_index;
            int current_selection = selected_match_index;
            int difference;
            
            // change the label selection first. Then find out where we moved to and move the overall row selection.
            selected_row_index = new_selection;
            difference = current_label_selection - selected_row_index;
            selected_match_index = selected_match_index - difference;
        }

        public string GetPathOfHTMLFile()
        {
            string path_to_file;

            path_to_file = Globals.WorkPath + "\\sample.html";

            // we need to see if the file exists first.
            if (File.Exists(path_to_file))
            {
                return path_to_file;
            }
            else
            {
                string text = "<html><body></body></html>";
                System.IO.File.WriteAllText(path_to_file, text);
                System.Threading.Thread.Sleep(100);
                if (File.Exists(path_to_file))
                {
                    return path_to_file;
                }
                else
                {
                    return "0";
                }
            }
        }

        public void contextMenu1_Opening(object sender, CancelEventArgs e)
        {

        }

        private void launchToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // user clicked "Launch" in the menu after clicking a keyword.
            string selected_keyword = autocomplete_match_list[selected_match_index].row_name;
            string selected_target = autocomplete_match_list[selected_match_index].row_target;
            frmMain_instance.LaunchEntry(selected_keyword, selected_match_index);

            /*
            // user clicked "Launch" in the menu after clicking a keyword.
            string selected_keyword;
            string selected_target = "";

            //if (popup_list_element_type[row_selection] == 0)
            if (autocomplete_match_list[row_selection].row_type == "keyword")
            {
                selected_target = Globals.keyword_targets[popup_list_elements[row_selection]];
            }
            else if (popup_list_element_type[row_selection] == 1)
            {
                selected_target = "previous entry";
            }
            else
            {
                selected_target = Globals.autoindex_filepaths[popup_list_elements[row_selection]];
            }
            selected_keyword = row_keywords[row_label_selection].Text;
            frmMain_instance.LaunchEntry(selected_keyword, selected_target);
             */
        }

        private void changeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // user clicked "Edit" in the menu after clicking a keyword.
            // pull up the Edit Keyword window.
            //frmMain_instance.ChangeKeyword(row_keywords[selected_row_index].Text);
            frmMain_instance.ChangeKeyword(autocomplete_match_list[selected_match_index].row_name, autocomplete_match_list[selected_match_index].row_path);
        }

        private void removeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // the user wants to delete a keyword or remove an entry from autocomplete. Let's see what it is first.
            string entry_to_remove = autocomplete_match_list[selected_match_index].row_name;
            string the_path = autocomplete_match_list[selected_match_index].row_path;
            string the_target = autocomplete_match_list[selected_match_index].row_target;

            bool is_a_keyword = false;

            if (autocomplete_match_list[selected_match_index].row_type == "previous entry")
            {
                string full_text;
                string AutoCompleteFile = Globals.WorkPath + "\\autocomplete.txt";
                try
                {
                    full_text = File.ReadAllText(AutoCompleteFile);
                }
                catch
                {
                    // could not open autocomplete.txt file.
                    return;
                }
                int counter = 0;
                foreach (string current_entry in Globals.autocomplete_list)
                {
                    if (current_entry == entry_to_remove)
                    {
                        Globals.autocomplete_list.RemoveAt(counter);
                        break;
                    }
                    counter++;
                }
                full_text = full_text.Replace(entry_to_remove + "|", "");
                // now save the autocomplete file...
                try
                {
                    System.IO.File.WriteAllText(AutoCompleteFile, full_text, Encoding.Unicode);
                }
                catch
                {
                    // we could not save anything to the autocomplete file.
                    return;
                }
                frmMain_instance.HideMainForm();
                return;
            }
            else if (autocomplete_match_list[selected_match_index].row_type == "autoindex")
            {
                // the entry is from the autoindex. Let's remove it.
                int the_counter = 0;
                foreach (string p in Globals.autoindex_filepaths)
                {
                    if (p == the_target)
                    {
                        // we found the autoindex target to remove
                        Globals.autoindex_filepaths.RemoveAt(the_counter);
                        Globals.autoindex_files.RemoveAt(the_counter);
                        // we want to add the target to a "black list" in the keybreeze directory.
                        try
                        {
                            using (StreamWriter sw = new StreamWriter(Globals.WorkPath + "\\autoindex\\blacklist.txt", true, Encoding.Unicode))
                            {
                                sw.WriteLine(the_target);
                            }
                        }
                        catch
                        {
                        }
                        break;
                    }
                    the_counter++;
                }
                frmMain_instance.HideMainForm();
                return;
            }

            string entry_to_remove_lower = entry_to_remove.ToLower();

            // loop through all the keywords and see if there is a match.
            for (int i = 0; i < Globals.keyword_count; i++)
            {
                //string current_keyword = Globals.keyword_names_lowercase[i];
                string current_keyword = Globals.keyword_name_lower[i];
                if (current_keyword == entry_to_remove_lower)
                {
                    // we have a match.
                    is_a_keyword = true;
                    break;
                }
            }

            if (is_a_keyword == true)
            {
                // NOTE: If changing the below, you may also need to change frmAutoCompleteFolder.
                // the entry is a keyword. see if it's a macro.
                if (the_target == "macro")
                {
                    DialogResult result;
                    result = MessageBox.Show("Are you sure you want to remove the macro, " + entry_to_remove + ", from the command list?", "Keybreeze Desktop", MessageBoxButtons.YesNo);
                    if (result == DialogResult.Yes)
                    {
                        frmMain_instance.DeleteKeywordFromCommandList(entry_to_remove, the_path, true);
                        string macro_file_path = Globals.WorkPath + "\\macros\\" + entry_to_remove + ".kmcr";
                        if (Globals.DoesFileExist(macro_file_path) == true)
                        {
                            DialogResult result2;
                            result2 = MessageBox.Show("Do you want to delete the macro file, " + macro_file_path + ", from your computer as well?", "Keybreeze Desktop", MessageBoxButtons.YesNo);
                            if (result2 == DialogResult.Yes)
                            {
                                // delete the macro file.
                                try
                                {
                                    File.Delete(macro_file_path);
                                }
                                catch
                                {
                                    MessageBox.Show("Keybreeze Desktop could not delete the macro file, possibly due to Windows permissions. You can manually delete this file by browsing to the path, " + macro_file_path, "Keybreeze Desktop");
                                }
                            }
                        }
                        frmMain_instance.HideMainForm();
                    }
                }
                else
                {
                    DialogResult result;
                    result = MessageBox.Show("Are you sure you want to delete the keyword, " + entry_to_remove + "?", "Keybreeze Desktop", MessageBoxButtons.YesNo);
                    if (result == DialogResult.Yes)
                    {
                        frmMain_instance.DeleteKeywordFromCommandList(entry_to_remove, the_path, true);
                        frmMain_instance.HideMainForm();
                    }
                }
            }

            /*
            string entry_to_remove = row_keywords[row_label_selection].Text;
            string the_target = row_descriptions[row_label_selection].Tag.ToString();
            string current_keyword;
            bool is_a_keyword = false;

            // see if it's an autocomplete entry first.
            if (popup_list_element_type[row_selection] == 1)
            {
                string full_text;
                string AutoCompleteFile = Globals.WorkPath + "\\autocomplete.txt";
                try
                {
                    full_text = File.ReadAllText(AutoCompleteFile);
                }
                catch
                {
                    // could not open autocomplete.txt file.
                    return;
                }
                int counter = 0;
                foreach (string current_entry in Globals.autocomplete_list)
                {
                    if (current_entry == entry_to_remove)
                    {
                        Globals.autocomplete_list.RemoveAt(counter);
                        break;
                    }
                    counter++;
                }
                full_text = full_text.Replace(entry_to_remove + "|", "");
                // now save the autocomplete file...
                try
                {
                    System.IO.File.WriteAllText(AutoCompleteFile, full_text, Encoding.Unicode);
                }
                catch
                {
                    // we could not save anything to the autocomplete file.
                    return;
                }
                frmMain_instance.HideMainForm();
                return;
            }
            else if (popup_list_element_type[row_selection] == 2)
            {
                // the entry is from the autoindex. Let's remove it.
                int the_counter = 0;
                foreach (string p in Globals.autoindex_filepaths)
                {
                    if (p == the_target)
                    {
                        // we found the autoindex target to remove
                        Globals.autoindex_filepaths.RemoveAt(the_counter);
                        Globals.autoindex_files.RemoveAt(the_counter);
                        // we want to add the target to a "black list" in the keybreeze directory.
                        try
                        {
                            using (StreamWriter sw = new StreamWriter(Globals.WorkPath + "\\autoindex\\blacklist.txt", true, Encoding.Unicode))
                            {
                                sw.WriteLine(the_target);
                            }
                        }
                        catch
                        {
                        }
                        break;
                    }
                    the_counter++;
                }
                frmMain_instance.HideMainForm();
                return;
            }

            // loop through all the keywords and see if there is a match.
            for (int i = 0; i < Globals.keyword_count; i++)
            {
                current_keyword = Globals.keyword_names_lowercase[i];
                if (current_keyword == entry_to_remove.ToLower())
                {
                    // we have a match.
                    is_a_keyword = true;
                    break;
                }
            }

            if (is_a_keyword == true)
            {
                // the entry is a keyword. see if it's a macro.
                if (the_target == "macro")
                {
                    DialogResult result;
                    result = MessageBox.Show("Are you sure you want to remove the macro, " + entry_to_remove + ", from the command list?", "Keybreeze Desktop", MessageBoxButtons.YesNo);
                    if (result == DialogResult.Yes)
                    {
                        frmMain_instance.DeleteKeywordFromCommandList(entry_to_remove, true);
                        string macro_file_path = Globals.WorkPath + "\\macros\\" + entry_to_remove + ".kmcr";
                        if (Globals.DoesFileExist(macro_file_path) == true)
                        {
                            DialogResult result2;
                            result2 = MessageBox.Show("Do you want to delete the macro file, " + macro_file_path + ", from your computer as well?", "Keybreeze Desktop", MessageBoxButtons.YesNo);
                            if (result2 == DialogResult.Yes)
                            {
                                // delete the macro file.
                                try
                                {
                                    File.Delete(macro_file_path);
                                }
                                catch
                                {
                                    MessageBox.Show("Keybreeze Desktop could not delete the macro file, possibly due to Windows permissions. You can manually delete this file by browsing to the path, " + macro_file_path, "Keybreeze Desktop");
                                }
                            }
                        }
                        frmMain_instance.HideMainForm();
                    }
                }
                else
                {
                    DialogResult result;
                    result = MessageBox.Show("Are you sure you want to delete the keyword, " + entry_to_remove + "?", "Keybreeze Desktop", MessageBoxButtons.YesNo);
                    if (result == DialogResult.Yes)
                    {
                        frmMain_instance.DeleteKeywordFromCommandList(entry_to_remove, true);
                        frmMain_instance.HideMainForm();
                    }
                }
            }
             */

        }

        public void Add_Icon_To_AutoComplete(int row_position, string row_target, string file_ext)
        {
            string test_string1 = "";
            try
            {
                // update by Andrew at October 22, 2017 11:56 AM.
                // This doesn't work at all. More testing is needed.
                // if we have already used an icon, we need to try to reuse it.
                /*
                if (row_position > 0)
                {
                    for (int i = 0; i < autocomplete_match_list.Count; i++)
                    {
                        if (row_target == autocomplete_match_list[i].row_target)
                        {
                            // we have already added an icon for this same target. Use the same icon.
                            imageListLarge.Images.Add(imageListLarge.Images[i]);
                            row_icons[row_position].Image = imageListLarge.Images[row_position];
                            return;
                        }
                        if (file_ext == autocomplete_match_list[i].file_ext && (file_ext == "dir" || file_ext == "unc" || file_ext == "unc_folder" || file_ext == "text" || file_ext == "web" || file_ext == "kb"))
                        {
                            // we have already added an icon for this same target. Use the same icon.
                            imageListLarge.Images.Add(imageListLarge.Images[i]);
                            row_icons[row_position].Image = imageListLarge.Images[row_position];
                            return;
                        }
                    }
                }
                */

                // first we need to see what the target is.
                if (file_ext == "dir" || file_ext == "unc" || file_ext == "unc_folder")
                {
                    // we need to use a folder icon.
                    string folder_icon = Globals.InstallDir + "\\images\\folder.png";
                    imageListLarge.Images.Add(Image.FromFile(folder_icon));
                    row_icons[row_position].Image = imageListLarge.Images[row_position];
                }
                else if (file_ext == "unc_file")
                {
                    // commented out by Andrew on 2/11. Need to use the workpath location so that we don't run into permission issues when creating the file.
                    //string html_file = Globals.InstallDir + "\\images\\sample.html";
                    string html_file = Globals.WorkPath + "\\sample.html";
                    imageListLarge.Images.Add(IconReader.GetFileIcon(html_file, 1, false));
                    row_icons[row_position].Image = imageListLarge.Images[row_position];
                }
                else if (file_ext == "text")
                {
                    // we need to use the text function icon.
                    string text_function_icon = Globals.InstallDir + "\\images\\text_function.png";
                    imageListLarge.Images.Add(Image.FromFile(text_function_icon));
                    row_icons[row_position].Image = imageListLarge.Images[row_position];
                }
                else if (file_ext == "kb")
                {
                    // we need to use the preset icon.
                    string preset_icon = Globals.InstallDir + "\\images\\preset.png";
                    imageListLarge.Images.Add(Image.FromFile(preset_icon));
                    row_icons[row_position].Image = imageListLarge.Images[row_position];
                }
                else if (file_ext == "web")
                {
                    string html_file = Globals.WorkPath + "\\sample.html";
                    imageListLarge.Images.Add(IconReader.GetFileIcon(html_file, 1, false));
                    row_icons[row_position].Image = imageListLarge.Images[row_position];
                }
                else if (file_ext == "err" || file_ext == "unknown")
                {
                    string file_blank_icon = Globals.InstallDir + "\\images\\file_blank.png";
                    imageListLarge.Images.Add(Image.FromFile(file_blank_icon));
                    row_icons[row_position].Image = imageListLarge.Images[row_position];
                }
                else
                {
                    // it's likely a file.
                    if (resolve_icons_to_full_paths == 1)
                    {
                        if (row_target.ToLower().IndexOf(".lnk") > -1)
                        {
                            if (File.Exists(row_target))
                            {
                                row_target = Globals.ResolveShortcut(row_target);
                            }
                        }
                    }
                    // are there any command line parameters? If so, we need to remove them before getting the icon.
                    string actual_exe_file = row_target;
                    int find_parameters = row_target.ToLower().IndexOf(".exe ");
                    if (find_parameters == -1)
                        find_parameters = row_target.ToLower().IndexOf(".exe\"");
                    if (find_parameters != -1)
                    {
                        actual_exe_file = row_target.Substring(0, find_parameters + 5).Trim();
                        // remove quotes.
                        actual_exe_file = actual_exe_file.Replace("\"", "");
                    }
                    if (File.Exists(actual_exe_file))
                    {
                        imageListLarge.Images.Add(IconReader.GetFileIcon(actual_exe_file, 1, false));
                        row_icons[row_position].Image = imageListLarge.Images[row_position];
                    }
                    else
                    {
                        // file doesn't exist. Show an error icon.
                        string file_blank_icon = Globals.InstallDir + "\\images\\file_blank.png";
                        imageListLarge.Images.Add(Image.FromFile(file_blank_icon));
                        row_icons[row_position].Image = imageListLarge.Images[row_position];
                    }
                }
            }
            catch { }

            /*
            try
            {
                // first we need to see what the target is.
                if (Directory.Exists(row_target))
                {
                    // we need to use a folder icon.
                    string folder_icon = Globals.InstallDir + "\\images\\folder.png";
                    imageListLarge.Images.Add(Image.FromFile(folder_icon));
                    row_icons[row_position].Image = imageListLarge.Images[row_position];
                }
                else if (File.Exists(row_target))
                {
                    if (resolve_icons_to_full_paths == 1)
                    {
                        if (row_target.ToLower().IndexOf(".lnk") > -1)
                            row_target = Globals.ResolveShortcut(row_target);
                    }
                    imageListLarge.Images.Add(IconReader.GetFileIcon(row_target, 1, false));
                    row_icons[row_position].Image = imageListLarge.Images[row_position];
                }
                else
                {
                    // it's something else, probably a web page
                    string html_file = Globals.InstallDir + "\\images\\sample.html";
                    if (File.Exists(html_file) == false)
                    {
                        File.WriteAllText(html_file, "");
                    }
                    imageListLarge.Images.Add(IconReader.GetFileIcon(html_file, 1, false));
                    row_icons[row_position].Image = imageListLarge.Images[row_position];
                }
            }
            catch { }
             */
        }

        public void Clear_AutoComplete_Icons()
        {
            autocompleteImageListLarge.Images.Clear();
            autocompleteImageListSmall.Images.Clear();
            imageListLarge.Images.Clear();
        }

        private void frmPopup_Paint(object sender, PaintEventArgs e)
        {
            // this draws a dark gray border around the edge of the form.
            string bordercolor = Globals.ReadINI("autocomplete", "bordercolor", Globals.SkinINI);
            if (bordercolor.Length > 0)
                ControlPaint.DrawBorder(e.Graphics, this.ClientRectangle, System.Drawing.ColorTranslator.FromHtml(bordercolor), ButtonBorderStyle.Solid);
        }

        private void addCustomKeywordToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // this will let the user to add a custom keyword for the highlighted autoindex entry.
            string entry = autocomplete_match_list[selected_match_index].row_name;
            string the_target = autocomplete_match_list[selected_match_index].row_target;

            if (the_target.IndexOf(".lnk") > -1)
                the_target = Globals.ResolveShortcut(the_target);
            
            frmMain_instance.ShowAddKeywordForm(false, entry, the_target);
            frmMain_instance.HideMainForm();
        }

        private void copyTargetToClipboardToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string selected_match_type = autocomplete_match_list[selected_match_index].row_type;
            string selected_match_target = autocomplete_match_list[selected_match_index].row_target;
            string selected_match_path = autocomplete_match_list[selected_match_index].row_path;
            if (selected_match_type == "previous entry")
            {
                // the user is copying a previous entry, so just copy the entry name or row name.
                try
                {
                    Clipboard.SetText(autocomplete_match_list[selected_match_index].row_name);
                }
                catch
                { }
            }
            else if (selected_match_target.IndexOf("function:") == 0)
            {
                // it's a text function. return the actual text of the file.
                string text_function_file = selected_match_target.Substring(9);
                text_function_file = Globals.replace_invalid_filename_characters(text_function_file, selected_match_path);
                string text_file = Globals.WorkPath + "\\text functions\\" + text_function_file + ".txt";
                string the_text = "";
                try
                {
                    the_text = File.ReadAllText(text_file);
                }
                catch
                {
                    the_text = text_function_file;
                }
                try
                {
                    Clipboard.SetText(the_text);
                }
                catch
                { }
            }
            else
            {
                // it's a regular target. copy it to the clipboard.
                try
                {
                    Clipboard.SetText(selected_match_target);
                }
                catch
                { }
            }
            frmMain_instance.HideMainForm();
            /*
            string selected_keyword;
            string selected_target = "";

            if (popup_list_element_type[row_selection] == 0)
            {
                selected_target = Globals.keyword_targets[popup_list_elements[row_selection]];
            }
            else if (popup_list_element_type[row_selection] == 1)
            {
                selected_target = "previous entry";
            }
            else
            {
                selected_target = Globals.autoindex_filepaths[popup_list_elements[row_selection]];
            }
            selected_keyword = row_keywords[row_label_selection].Text;
            if (selected_target == "previous entry")
            {
                try
                {
                    Clipboard.SetText(selected_keyword);
                }
                catch
                { }
            }
            else if (selected_target.IndexOf("function:") == 0)
            {
                // it's a text function. return the actual text of the file.
                string text_function_file = selected_target.Substring(9);
                text_function_file = Globals.replace_invalid_filename_characters(text_function_file);
                string text_file = Globals.WorkPath + "\\text functions\\" + text_function_file + ".txt";
                string the_text = "";
                try
                {
                    the_text = File.ReadAllText(text_file);
                }
                catch
                {
                    the_text = text_function_file;
                }
                try
                {
                    Clipboard.SetText(the_text);
                }
                catch
                { }
            }
            else
            {
                // it's a regular target. copy it to the clipboard.
                try
                {
                    Clipboard.SetText(selected_target);
                }
                catch
                { }
            }
            frmMain_instance.HideMainForm();
             */
        }

        private void openContainingFolderToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string selected_keyword;
            string selected_target = "";

            //if (popup_list_element_type[row_selection] == 0)
            if (autocomplete_match_list[selected_match_index].row_type == "keyword")
            {
                selected_target = autocomplete_match_list[selected_match_index].row_target;
            }
            //else if (popup_list_element_type[row_selection] == 1)
            if (autocomplete_match_list[selected_match_index].row_type == "previous entry")
            {
                // it's a previous entry. there is no containing folder to open.
                frmMain_instance.HideMainForm();
                return;
            }
            else
            {
                selected_target = autocomplete_match_list[selected_match_index].row_target;
            }
            selected_keyword = autocomplete_match_list[selected_match_index].row_name;
            if (selected_target == "previous entry")
            {
                return;
            }
            else if (selected_target.IndexOf("function:") == 0)
            {
                // it's a text function. just open the folder that contains all the text function files.
                frmMain_instance.RunFolder(Globals.WorkPath + "\\text functions");
            }
            else
            {
                // we need to find out if the target is already a folder. If so, just open it. If it points to a file, parse it out.
                string filename = Path.GetFileName(selected_target);
                if (filename.IndexOf(".") == -1)
                {
                    // the target is a folder. Just open it.
                    frmMain_instance.RunFolder(selected_target);
                }
                else
                {
                    // it's a file. open its containing folder.
                    frmMain_instance.RunFolder(selected_target.Substring(0, selected_target.Length - filename.Length));
                }
            }
            frmMain_instance.HideMainForm();
        }
    }
}