﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Keybreeze
{
    public partial class frmReminder : Form
    {
        int format24hourtime;
        DateTime label_time = new DateTime();
        public List<string> random_time_examples = new List<string>();

        public frmReminder()
        {
            InitializeComponent();
        }

        private void frmReminder_Load(object sender, EventArgs e)
        {
            cmbType.Items.Add("One time only");
            cmbType.Items.Add("Every day");
            cmbType.Items.Add("Every week");
            cmbType.Items.Add("Every month");
            cmbType.SelectedIndex = 0;

            format24hourtime = Globals.ReadIntFromINI("notes", "24hourtime", Globals.NotesINI, 0);
            if (format24hourtime == 0)
            {
                // show the time in 12 hour format and offer a link to switch to 24 hour.
                dateTimePicker1.CustomFormat = "dddd MMMM dd',' yyyy ' 'hh:mm tt";
                label3.Text = "24 hour time";
            }
            else
            {
                // show the time in 24 hour format and offer a link to switch to 12 hour.
                dateTimePicker1.CustomFormat = "dddd MMMM dd',' yyyy ' 'HH:mm";
                label3.Text = "12 hour time";
            }

            int play_sound = Globals.ReadIntFromINI("notes", "sound", Globals.NotesINI, 1);
            if (play_sound == 1)
            {
                string sound_file = Globals.ReadINI("notes", "sound_file", Globals.NotesINI);
                if (sound_file.Length == 0 || Globals.DoesFileExist(sound_file) == false)
                {
                    sound_file = Globals.InstallDir + "\\sounds\\chimes.wav";
                    Globals.WriteINI("notes", "sound_file", sound_file, Globals.NotesINI);
                }
            }

            int loop_sound = Globals.ReadIntFromINI("notes", "sound_loop", Globals.NotesINI);

            cmbDays.Items.Add("Sunday");
            cmbDays.Items.Add("Monday");
            cmbDays.Items.Add("Tuesday");
            cmbDays.Items.Add("Wednesday");
            cmbDays.Items.Add("Thursday");
            cmbDays.Items.Add("Friday");
            cmbDays.Items.Add("Saturday");
            cmbDays.SelectedIndex = 0;

            if (txtNote.Text == Globals.new_note_text)
            {
                txtNote.Focus();
                txtNote.SelectionStart = 0;
                txtNote.SelectionLength = txtNote.Text.Length;
            }
            Set_Up_Form();
            random_time_examples.Add("15m");
            random_time_examples.Add("30m");
            random_time_examples.Add("1h");
            random_time_examples.Add("1h30m");
            random_time_examples.Add("tomorrow at 8 am");
            random_time_examples.Add("this friday at 5 pm");
            Random rnd = new Random();
            int random_time = rnd.Next(0, random_time_examples.Count - 1);
            txtTime.Text = random_time_examples[random_time];
        }

        private void cmbType_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cmbType.SelectedIndex == 0)
            {
                // one time only
                lblOn.Visible = false;
                cmbDays.Visible = false;
                txtTime.Enabled = true;
                if (format24hourtime == 0)
                {
                    // show the time in 12 hour format and offer a link to switch to 24 hour.
                    dateTimePicker1.CustomFormat = "dddd MMMM dd',' yyyy ' 'hh:mm tt";
                    label3.Text = "24 hour time";
                }
                else
                {
                    // show the time in 24 hour format and offer a link to switch to 12 hour.
                    dateTimePicker1.CustomFormat = "dddd MMMM dd',' yyyy ' 'HH:mm";
                    label3.Text = "12 hour time";
                }
            }
            else if (cmbType.SelectedIndex == 1)
            {
                // every day
                lblOn.Visible = false;
                cmbDays.Visible = false;
                txtTime.Enabled = false;
                if (format24hourtime == 0)
                {
                    // show the time in 12 hour format and offer a link to switch to 24 hour.
                    dateTimePicker1.CustomFormat = "hh:mm tt";
                    label3.Text = "24 hour time";
                }
                else
                {
                    // show the time in 24 hour format and offer a link to switch to 12 hour.
                    dateTimePicker1.CustomFormat = "HH:mm";
                    label3.Text = "12 hour time";
                }
            }
            else if (cmbType.SelectedIndex == 2)
            {
                // every week
                lblOn.Visible = true;
                cmbDays.Visible = true;
                txtTime.Enabled = false;
                if (format24hourtime == 0)
                {
                    // show the time in 12 hour format and offer a link to switch to 24 hour.
                    dateTimePicker1.CustomFormat = "hh:mm tt";
                    label3.Text = "24 hour time";
                }
                else
                {
                    // show the time in 24 hour format and offer a link to switch to 12 hour.
                    dateTimePicker1.CustomFormat = "HH:mm";
                    label3.Text = "12 hour time";
                }
            }
            else if (cmbType.SelectedIndex == 3)
            {
                // every month
                lblOn.Visible = false;
                cmbDays.Visible = false;
                txtTime.Enabled = false;
                if (format24hourtime == 0)
                {
                    // show the time in 12 hour format and offer a link to switch to 24 hour.
                    dateTimePicker1.CustomFormat = "dd 'at' hh:mm tt";
                    label3.Text = "24 hour time";
                }
                else
                {
                    // show the time in 24 hour format and offer a link to switch to 12 hour.
                    dateTimePicker1.CustomFormat = "dd 'at' HH:mm";
                    label3.Text = "12 hour time";
                }
            }
        }

        private void label3_Click(object sender, EventArgs e)
        {
            string custom_format = dateTimePicker1.CustomFormat;
            int find_chars;

            if (label3.Text == "24 hour time")
            {
                // switch to 24 hour time
                find_chars = custom_format.LastIndexOf("hh:mm tt");
                custom_format = custom_format.Substring(0, find_chars) + "HH:mm";
                dateTimePicker1.CustomFormat = custom_format;
                Globals.WriteINI("notes", "24hourtime", "1", Globals.NotesINI);
                format24hourtime = 1;
                label3.Text = "12 hour time";
            }
            else
            {
                // switch to 12 hour time
                find_chars = custom_format.LastIndexOf("HH:mm");
                custom_format = custom_format.Substring(0, find_chars) + "hh:mm tt";
                dateTimePicker1.CustomFormat = custom_format;
                Globals.WriteINI("notes", "24hourtime", "0", Globals.NotesINI);
                format24hourtime = 0;
                label3.Text = "24 hour time";
            }
        }

        public void Set_Up_Form()
        {
            // Try to get the title of the active window.
            try
            {
                int wait_time3 = Globals.ReadIntFromINI("options", "wait_time_before_inserting_text", Globals.OptionsINI, 250);
                System.Threading.Thread.Sleep(wait_time3);
                IntPtr name_of_handle;
                name_of_handle = frmMain.GetForegroundWindow();
                string classname = frmMain.GetWindowClassName(name_of_handle);
                string titlename = frmMain.GetWindowTitle(name_of_handle);
                if (titlename.Length > 0)
                {
                    txtNote.Text = titlename + ": ";
                    txtNote.SelectionStart = 0;
                    txtNote.SelectionLength = txtNote.Text.Length;
                }
            }
            catch { }
        }

        private void cmdCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void cmdSet_Click(object sender, EventArgs e)
        {
            cmdSet_function();
        }

        private void cmdSet_function()
        {
            int return_val;
            int the_year;
            int the_month;
            int the_dayofmonth;
            string the_weekday;
            int the_weekday_int;
            int the_hour;
            int the_minute;
            int current_year = DateTime.Today.Year;
            int current_month = DateTime.Today.Month;
            int current_dayofmonth = DateTime.Today.Day;
            int current_weekday = GetNumFromWeekday(DateTime.Today.DayOfWeek.ToString());
            int current_hour = DateTime.Now.Hour;
            int current_minute = DateTime.Now.Minute;
            DialogResult result;

            if (txtNote.Tag == null)
                txtNote.Tag = "";

            if (cmbType.SelectedIndex == 0)
            {
                // one time reminder
                the_year = dateTimePicker1.Value.Year;
                the_month = dateTimePicker1.Value.Month;
                the_dayofmonth = dateTimePicker1.Value.Day;
                the_weekday = dateTimePicker1.Value.DayOfWeek.ToString();
                the_hour = dateTimePicker1.Value.Hour;
                the_minute = dateTimePicker1.Value.Minute;
                bool set_in_future = false;

                if (the_year > current_year)
                    set_in_future = true;
                else if (the_month > current_month)
                    set_in_future = true;
                else if (the_dayofmonth > current_dayofmonth)
                    set_in_future = true;
                else if (the_hour > current_hour)
                    set_in_future = true;
                else if (the_minute > current_minute && the_hour >= current_hour)
                    set_in_future = true;

                if (set_in_future == true)
                {
                    // if we are creating a new note, e.g. there is no unique ID in the tag field, we need to create the note files.
                    if (txtNote.Tag.ToString().Length == 0)
                    {
                        return_val = create_note_files();
                        if (return_val == 0)
                            return;
                    }
                    else
                    {
                        // the note already exists, but we need to modify the INI file and add the sound options.
                        int temp_uniqueID = Convert.ToInt32(txtNote.Tag.ToString());
                        Globals.WriteINI("properties", "sound", Globals.ReadINI("notes", "sound", Globals.NotesINI), Globals.WorkPath + "\\notes\\" + temp_uniqueID.ToString() + ".ini");
                        Globals.WriteINI("properties", "sound_file", Globals.ReadINI("notes", "sound_file", Globals.NotesINI), Globals.WorkPath + "\\notes\\" + temp_uniqueID.ToString() + ".ini");
                        Globals.WriteINI("properties", "sound_loop", Globals.ReadINI("notes", "sound_loop", Globals.NotesINI), Globals.WorkPath + "\\notes\\" + temp_uniqueID.ToString() + ".ini");
                    }
                    the_weekday_int = GetNumFromWeekday(the_weekday);
                    return_val = Globals.notes_set_reminder(1, the_year, the_month, the_dayofmonth, the_hour, the_minute, 0, txtNote.Tag.ToString());
                    if (return_val == 0)
                    {
                        MessageBox.Show("An error occurred while setting the reminder. Please make sure you have read/write permission to the " + Globals.WorkPath + " directory and all subfolders.", "Keybreeze Desktop");
                    }
                    else
                    {
                        this.Hide();
                        //MessageBox.Show("The reminder has been set. Keybreeze Desktop must be running in your system tray for the reminder to appear on time. If it is not running, the reminder will appear the next time you launch Keybreeze Desktop.", "Keybreeze Desktop");
                        this.Close();
                        return;
                    }
                }
                else
                {
                    MessageBox.Show("The reminder must be set in the future.", "Keybreeze Desktop");
                    return;
                }
            }
            else if (cmbType.SelectedIndex == 1)
            {
                // daily reminder
                the_hour = dateTimePicker1.Value.Hour;
                the_minute = dateTimePicker1.Value.Minute;
                // if we are creating a new note, e.g. there is no unique ID in the tag field, we need to create the note files.
                if (txtNote.Tag.ToString().Length == 0)
                {
                    return_val = create_note_files();
                    if (return_val == 0)
                        return;
                }
                else
                {
                    // the note already exists, but we need to modify the INI file and add the sound options.
                    int temp_uniqueID = Convert.ToInt32(txtNote.Tag.ToString());
                    Globals.WriteINI("properties", "sound", Globals.ReadINI("notes", "sound", Globals.NotesINI), Globals.WorkPath + "\\notes\\" + temp_uniqueID.ToString() + ".ini");
                    Globals.WriteINI("properties", "sound_file", Globals.ReadINI("notes", "sound_file", Globals.NotesINI), Globals.WorkPath + "\\notes\\" + temp_uniqueID.ToString() + ".ini");
                    Globals.WriteINI("properties", "sound_loop", Globals.ReadINI("notes", "sound_loop", Globals.NotesINI), Globals.WorkPath + "\\notes\\" + temp_uniqueID.ToString() + ".ini");
                }
                bool daily_reminder_already_past = false;
                if (current_hour > the_hour)
                    daily_reminder_already_past = true;
                else if (current_hour >= the_hour && current_minute > the_minute)
                    daily_reminder_already_past = true;
                if (daily_reminder_already_past == true)
                {
                    // the daily reminder has been set for a time that already went by today, so we don't want it to pop up until tomorrow. make a note of this in the function call.
                    return_val = Globals.notes_set_reminder(2, 0, 0, 0, the_hour, the_minute, 0, txtNote.Tag.ToString(), current_dayofmonth);
                }
                else
                {
                    return_val = Globals.notes_set_reminder(2, 0, 0, 0, the_hour, the_minute, 0, txtNote.Tag.ToString());
                }
                if (return_val == 0)
                {
                    MessageBox.Show("An error occurred while setting the reminder. Please make sure you have read/write permission to the " + Globals.WorkPath + " directory and all subfolders.", "Keybreeze Desktop");
                }
                else
                {
                    this.Hide();
                    //MessageBox.Show("The reminder has been set. Keybreeze Desktop must be running in your system tray for the reminder to appear on time. If it is not running, the reminder will appear the next time you launch Keybreeze Desktop.", "Keybreeze Desktop");
                    this.Close();
                    return;
                }
            }
            else if (cmbType.SelectedIndex == 2)
            {
                // weekly reminder
                the_weekday = cmbDays.Text;
                the_hour = dateTimePicker1.Value.Hour;
                the_minute = dateTimePicker1.Value.Minute;
                // if we are creating a new note, e.g. there is no unique ID in the tag field, we need to create the note files.
                if (txtNote.Tag.ToString().Length == 0)
                {
                    return_val = create_note_files();
                    if (return_val == 0)
                        return;
                }
                else
                {
                    // the note already exists, but we need to modify the INI file and add the sound options.
                    int temp_uniqueID = Convert.ToInt32(txtNote.Tag.ToString());
                    Globals.WriteINI("properties", "sound", Globals.ReadINI("notes", "sound", Globals.NotesINI), Globals.WorkPath + "\\notes\\" + temp_uniqueID.ToString() + ".ini");
                    Globals.WriteINI("properties", "sound_file", Globals.ReadINI("notes", "sound_file", Globals.NotesINI), Globals.WorkPath + "\\notes\\" + temp_uniqueID.ToString() + ".ini");
                    Globals.WriteINI("properties", "sound_loop", Globals.ReadINI("notes", "sound_loop", Globals.NotesINI), Globals.WorkPath + "\\notes\\" + temp_uniqueID.ToString() + ".ini");
                }
                bool weekly_reminder_already_past = false;
                if (current_weekday > GetNumFromWeekday(the_weekday))
                    weekly_reminder_already_past = true;
                else if (current_weekday >= GetNumFromWeekday(the_weekday) && current_hour > the_hour)
                    weekly_reminder_already_past = true;
                else if (current_weekday >= GetNumFromWeekday(the_weekday) && current_hour >= the_hour && current_minute > the_minute)
                    weekly_reminder_already_past = true;
                if (weekly_reminder_already_past == true)
                {
                    // the weekly reminder has been set for a time/day that already went by, so we don't want it to pop up until next week. make a note of this in the function call.
                    return_val = Globals.notes_set_reminder(3, 0, 0, 0, the_hour, the_minute, GetNumFromWeekday(the_weekday), txtNote.Tag.ToString(), current_dayofmonth);
                }
                else
                {
                    return_val = Globals.notes_set_reminder(3, 0, 0, 0, the_hour, the_minute, GetNumFromWeekday(the_weekday), txtNote.Tag.ToString());
                }
                if (return_val == 0)
                {
                    MessageBox.Show("An error occurred while setting the reminder. Please make sure you have read/write permission to the " + Globals.WorkPath + " directory and all subfolders.", "Keybreeze Desktop");
                }
                else
                {
                    this.Hide();
                    //MessageBox.Show("The reminder has been set. Keybreeze Desktop must be running in your system tray for the reminder to appear on time. If it is not running, the reminder will appear the next time you launch Keybreeze Desktop.", "Keybreeze Desktop");
                    this.Close();
                    return;
                }
            }
            else if (cmbType.SelectedIndex == 3)
            {
                // monthly reminder
                the_dayofmonth = dateTimePicker1.Value.Day;
                // see if the user picked the 29th, 30th, or 31st. If so, the reminder won't appear on all months.
                if (the_dayofmonth == 29 || the_dayofmonth == 30 || the_dayofmonth == 31)
                {
                    result = MessageBox.Show("You are setting a monthly reminder to appear on day " + the_dayofmonth + " of every month. Some months do not have this day, so the reminder will not appear every single month. Are you sure you want to continue?", "Warning", MessageBoxButtons.YesNo);
                    if (result != DialogResult.Yes)
                    {
                        return;
                    }
                }
                the_hour = dateTimePicker1.Value.Hour;
                the_minute = dateTimePicker1.Value.Minute;
                // if we are creating a new note, e.g. there is no unique ID in the tag field, we need to create the note files.
                if (txtNote.Tag.ToString().Length == 0)
                {
                    return_val = create_note_files();
                    if (return_val == 0)
                        return;
                }
                else
                {
                    // the note already exists, but we need to modify the INI file and add the sound options.
                    int temp_uniqueID = Convert.ToInt32(txtNote.Tag.ToString());
                    Globals.WriteINI("properties", "sound", Globals.ReadINI("notes", "sound", Globals.NotesINI), Globals.WorkPath + "\\notes\\" + temp_uniqueID.ToString() + ".ini");
                    Globals.WriteINI("properties", "sound_file", Globals.ReadINI("notes", "sound_file", Globals.NotesINI), Globals.WorkPath + "\\notes\\" + temp_uniqueID.ToString() + ".ini");
                    Globals.WriteINI("properties", "sound_loop", Globals.ReadINI("notes", "sound_loop", Globals.NotesINI), Globals.WorkPath + "\\notes\\" + temp_uniqueID.ToString() + ".ini");
                }
                bool monthly_reminder_already_past = false;
                if (current_dayofmonth > the_dayofmonth)
                    monthly_reminder_already_past = true;
                else if (current_dayofmonth > the_dayofmonth && current_hour > the_hour)
                    monthly_reminder_already_past = true;
                else if (current_dayofmonth > the_dayofmonth && current_hour >= the_hour && current_minute > the_minute)
                    monthly_reminder_already_past = true;
                if (monthly_reminder_already_past == true)
                {
                    // the monthly reminder has been set for a day that already went by, so we don't want it to pop up until next month. make a note of this in the function call.
                    return_val = Globals.notes_set_reminder(4, 0, 0, the_dayofmonth, the_hour, the_minute, 0, txtNote.Tag.ToString(), 0, current_month);
                }
                else
                {
                    return_val = Globals.notes_set_reminder(4, 0, 0, the_dayofmonth, the_hour, the_minute, 0, txtNote.Tag.ToString());
                }
                if (return_val == 0)
                {
                    MessageBox.Show("An error occurred while setting the reminder. Please make sure you have read/write permission to the " + Globals.WorkPath + " directory and all subfolders.", "Keybreeze Desktop");
                }
                else
                {
                    this.Hide();
                    //MessageBox.Show("The reminder has been set. Keybreeze Desktop must be running in your system tray for the reminder to appear on time. If it is not running, the reminder will appear the next time you launch Keybreeze Desktop.", "Keybreeze Desktop");
                    this.Close();
                    return;
                }
            }
        }

        private int Validate_Time(string original_response)
        {
            string user_response = original_response.ToLower();
            int the_months = 0, the_weeks = 0, the_days = 0, the_hours = 0, the_minutes = 0;
            try
            {
                DateTime day_to_show = DateTime.Now;
                string day_to_show_string = "";
                int find_tomorrow;
                if (user_response.Length > 10)
                {
                    // look at days.
                    find_tomorrow = user_response.IndexOf("tomorrow");
                    if (find_tomorrow != -1)
                    {
                        day_to_show = DateTime.Now.AddDays(1);
                        day_to_show_string = "tomorrow";
                    }
                    int find_day_of_week = user_response.IndexOf("sunday");
                    if (find_day_of_week != -1)
                    {
                        // find the next sunday from today.
                        for (int i = 1; i <= 7; i++)
                        {
                            DateTime next_day = DateTime.Now.AddDays(i);
                            if (next_day.DayOfWeek == DayOfWeek.Sunday)
                            {
                                day_to_show = next_day;
                                day_to_show_string = "sunday";
                                break;
                            }
                        }
                    }
                    find_day_of_week = user_response.IndexOf("monday");
                    if (find_day_of_week != -1)
                    {
                        for (int i = 1; i <= 7; i++)
                        {
                            DateTime next_day = DateTime.Now.AddDays(i);
                            if (next_day.DayOfWeek == DayOfWeek.Monday)
                            {
                                day_to_show = next_day;
                                break;
                            }
                        }
                    }
                    find_day_of_week = user_response.IndexOf("tuesday");
                    if (find_day_of_week != -1)
                    {
                        for (int i = 1; i <= 7; i++)
                        {
                            DateTime next_day = DateTime.Now.AddDays(i);
                            if (next_day.DayOfWeek == DayOfWeek.Tuesday)
                            {
                                day_to_show = next_day;
                                break;
                            }
                        }
                    }
                    find_day_of_week = user_response.IndexOf("wednesday");
                    if (find_day_of_week != -1)
                    {
                        for (int i = 1; i <= 7; i++)
                        {
                            DateTime next_day = DateTime.Now.AddDays(i);
                            if (next_day.DayOfWeek == DayOfWeek.Wednesday)
                            {
                                day_to_show = next_day;
                                break;
                            }
                        }
                    }
                    find_day_of_week = user_response.IndexOf("thursday");
                    if (find_day_of_week != -1)
                    {
                        for (int i = 1; i <= 7; i++)
                        {
                            DateTime next_day = DateTime.Now.AddDays(i);
                            if (next_day.DayOfWeek == DayOfWeek.Thursday)
                            {
                                day_to_show = next_day;
                                break;
                            }
                        }
                    }
                    find_day_of_week = user_response.IndexOf("friday");
                    if (find_day_of_week != -1)
                    {
                        for (int i = 1; i <= 7; i++)
                        {
                            DateTime next_day = DateTime.Now.AddDays(i);
                            if (next_day.DayOfWeek == DayOfWeek.Friday)
                            {
                                day_to_show = next_day;
                                break;
                            }
                        }
                    }
                    find_day_of_week = user_response.IndexOf("saturday");
                    if (find_day_of_week != -1)
                    {
                        for (int i = 1; i <= 7; i++)
                        {
                            DateTime next_day = DateTime.Now.AddDays(i);
                            if (next_day.DayOfWeek == DayOfWeek.Saturday)
                            {
                                day_to_show = next_day;
                                break;
                            }
                        }
                    }
                    // the user specified a day to show the reminder. Find the time, such as Sunday at 3 pm, or Friday at 7:00 am, etc.
                    string text_after_day = "";
                    if (find_tomorrow != -1)
                    {
                        text_after_day = user_response.Substring(find_tomorrow + 8).Trim();
                    }
                    else
                    {
                        int find_day = user_response.IndexOf("day");
                        text_after_day = user_response.Substring(find_day + 3).Trim();
                    }
                    // find a number, since that should represent the hour.
                    string hour_so_far = "";
                    string minute_so_far = "";
                    bool minutes_next = false;
                    foreach (char this_char in text_after_day)
                    {
                        if (this_char >= '0' && this_char <= '9')
                        {
                            // we found a number. Is this the hour or the minutes?
                            if (minutes_next == false)
                                hour_so_far += this_char;
                            else
                                minute_so_far += this_char;
                        }
                        else if (hour_so_far.Length > 0 && (this_char == ' ' || this_char == ':'))
                        {
                            // minutes should be next.
                            minutes_next = true;
                        }
                    }
                    string ampm = "";
                    if (text_after_day.IndexOf("am") != -1)
                        ampm = "am";
                    else if (text_after_day.IndexOf("pm") != -1)
                        ampm = "pm";
                    if (minute_so_far.Length == 0)
                        minute_so_far = "00";
                    string complete_time_string = day_to_show.ToShortDateString() + " " + hour_so_far + ":" + minute_so_far + " " + ampm;
                    DateTime time_validation;
                    if (DateTime.TryParse(complete_time_string, out time_validation))
                    {
                        label_time = time_validation;
                        return 1;
                    }
                    else
                    {
                        return -1;
                    }
                }
                else
                {
                    // the user may have specified hours and minutes instead.
                    int find_months = user_response.IndexOf("n");
                    int find_w = user_response.IndexOf("w");
                    int find_d = user_response.IndexOf("d");
                    int find_h = user_response.IndexOf("h");
                    int find_m = user_response.IndexOf("m");
                    if (find_months == -1 && find_w == -1 && find_d == -1 && find_h == -1 && find_m == -1)
                    {
                        return -1;
                    }
                    if (find_months > 0)
                    {
                        // get numbers before month.
                        string before_months = user_response.Substring(0, find_months);
                        the_months = Convert.ToInt32(before_months);
                        user_response = user_response.Substring(find_months + 1);
                        // reset
                        find_w = user_response.IndexOf("w");
                        find_d = user_response.IndexOf("d");
                        find_h = user_response.IndexOf("h");
                        find_m = user_response.IndexOf("m");
                    }
                    if (find_w > 0)
                    {
                        // get numbers before w.
                        string before_w = user_response.Substring(0, find_w);
                        the_weeks = Convert.ToInt32(before_w);
                        user_response = user_response.Substring(find_w + 1);
                        // reset
                        find_d = user_response.IndexOf("d");
                        find_h = user_response.IndexOf("h");
                        find_m = user_response.IndexOf("m");
                    }
                    if (find_d > 0)
                    {
                        // get numbers before d.
                        string before_d = user_response.Substring(0, find_d);
                        the_days = Convert.ToInt32(before_d);
                        user_response = user_response.Substring(find_d + 1);
                        // reset 
                        find_h = user_response.IndexOf("h");
                        find_m = user_response.IndexOf("m");
                    }
                    if (find_h > 0)
                    {
                        // get numbers before h.
                        string before_h = user_response.Substring(0, find_h);
                        the_hours = Convert.ToInt32(before_h);
                        user_response = user_response.Substring(find_h + 1);
                        // reset 
                        find_m = user_response.IndexOf("m");
                    }
                    if (find_m > 0)
                    {
                        // get numbers before m
                        string before_m = user_response.Substring(0, find_m);
                        the_minutes = Convert.ToInt32(before_m);
                    }
                    int snooze_time = (the_months * 43800) + (the_weeks * 10080) + (the_days * 1440) + (the_hours * 60) + the_minutes;
                    DateTime time_validation = new DateTime();
                    time_validation = DateTime.Now.AddMinutes(snooze_time);
                    label_time = time_validation;
                    return 1;
                }
            }
            catch
            {
                // we don't want to display a message box while the user is typing a keyword.
                //MessageBox.Show("Could not understand " + original_response + ". Please specify 'h' for hours or 'm' for minutes." + Environment.NewLine + "Examples: 2h, 3h, 25m, 1h30m");
                return -1;
            }
            return 0;
        }

        private int GetNumFromWeekday(string the_weekday)
        {
            switch (the_weekday)
            {
                case "Sunday":
                    return 1;
                case "Monday":
                    return 2;
                case "Tuesday":
                    return 3;
                case "Wednesday":
                    return 4;
                case "Thursday":
                    return 5;
                case "Friday":
                    return 6;
                case "Saturday":
                    return 7;
                default:
                    return 0;
            }
        }

        private int create_note_files()
        {
            // this will create the necessary files for the note -- the .txt file and .ini file.
            // if this is updated, a similar function in frmSpeedReminder may also need to be updated.
            string NotesDir = Globals.WorkPath + "\\notes";
            int UniqueID;
            string INI_Text;

            Random random = new Random();
            UniqueID = random.Next(1, 999999999);
            txtNote.Tag = UniqueID.ToString();

            INI_Text = "[properties]" + System.Environment.NewLine;
            INI_Text = INI_Text + "hiddenreminder=1" + System.Environment.NewLine;
            INI_Text = INI_Text + "top=0" + System.Environment.NewLine;
            INI_Text = INI_Text + "left=0" + System.Environment.NewLine;
            INI_Text = INI_Text + "width=" + Globals.ReadINI("notes", "width", Globals.NotesINI) + System.Environment.NewLine;
            INI_Text = INI_Text + "height=" + Globals.ReadINI("notes", "height", Globals.NotesINI) + System.Environment.NewLine;
            INI_Text = INI_Text + "backcolor=0" + System.Environment.NewLine;
            INI_Text = INI_Text + "image=0" + System.Environment.NewLine;
            INI_Text = INI_Text + "textcolor=" + Globals.ReadINI("notes", "textcolor", Globals.NotesINI) + System.Environment.NewLine;
            INI_Text = INI_Text + "font=" + Globals.ReadINI("notes", "font", Globals.NotesINI) + System.Environment.NewLine;
            INI_Text = INI_Text + "size=" + Globals.ReadINI("notes", "size", Globals.NotesINI) + System.Environment.NewLine;
            INI_Text = INI_Text + "bold=" + Globals.ReadINI("notes", "bold", Globals.NotesINI) + System.Environment.NewLine;
            INI_Text = INI_Text + "italic=" + Globals.ReadINI("notes", "italic", Globals.NotesINI) + System.Environment.NewLine;
            INI_Text = INI_Text + "underline=" + Globals.ReadINI("notes", "underline", Globals.NotesINI) + System.Environment.NewLine;
            INI_Text = INI_Text + "sound=" + Globals.ReadINI("notes", "sound", Globals.NotesINI) + System.Environment.NewLine;
            INI_Text = INI_Text + "sound_file=" + Globals.ReadINI("notes", "sound_file", Globals.NotesINI) + System.Environment.NewLine;
            INI_Text = INI_Text + "sound_loop=" + Globals.ReadINI("notes", "sound_loop", Globals.NotesINI) + System.Environment.NewLine;
            try
            {
                System.IO.File.WriteAllText(NotesDir + "\\" + UniqueID.ToString() + ".txt", txtNote.Text, Encoding.Unicode);
                System.IO.File.WriteAllText(NotesDir + "\\" + UniqueID.ToString() + ".ini", INI_Text, Encoding.Unicode);
                return 1;
            }
            catch (Exception e)
            {
                MessageBox.Show(this, "Keybreeze Desktop could not write to the " + NotesDir + " directory to save the text of this note. " + e.Message, "Warning");
                return 0;
            }
        }

        private void chkSound_CheckedChanged(object sender, EventArgs e)
        {
            
        }

        private void txtSound_TextChanged(object sender, EventArgs e)
        {
            
        }

        private void chkLoop_CheckedChanged(object sender, EventArgs e)
        {
            
        }

        private void btnViewReminders_Click(object sender, EventArgs e)
        {
            frmNoteReminders frmNoteReminders_instance = new frmNoteReminders();
            frmNoteReminders_instance.Show();
        }

        private void label2_Click(object sender, EventArgs e)
        {
            // to do
        }

        private void txtTime_TextChanged(object sender, EventArgs e)
        {
            int success = Validate_Time(txtTime.Text);
            if (success == 1)
            {
                dateTimePicker1.Value = label_time;
            }
            else
            {
                // ignore
            }
        }

        private void txtNote_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Control && (e.KeyCode == Keys.A))
            {
                if (sender != null)
                    ((TextBox)sender).SelectAll();
                e.SuppressKeyPress = true;
                e.Handled = true;
            }
            if (e.Shift)
            {
                if (e.KeyCode == Keys.Enter)
                {
                    e.Handled = true;
                    e.SuppressKeyPress = true;
                    string current_text = txtNote.Text;
                    txtNote.Text = txtNote.Text + Environment.NewLine;
                    txtNote.Select(txtNote.Text.Length, 0);
                }
            }
            else if (e.KeyCode == Keys.Enter)
            {
                // the user pressed Enter without pressing Shift.
                cmdSet_function();
            }
        }

        private void txtNote_TextChanged(object sender, EventArgs e)
        {

        }

        private void txtNote_Enter(object sender, EventArgs e)
        {
            this.AcceptButton = null;
        }

        private void txtNote_Leave(object sender, EventArgs e)
        {
            this.AcceptButton = cmdSet;
        }
    }
}
