﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Threading;
using System.Collections;

namespace Keybreeze
{
    public partial class frmSearch : Form
    {
        public frmMain frmMain_instance;
        private List<search_entry2> search_entries = new List<search_entry2>();
        public bool StopSearching = false;
        public string global_search_string;
        public int global_search_string_length;
        public string global_search_string_lowercase;
        public int search_result_count = 0;
        public bool skip_directory = false;
        public bool skip_file = false;
        public List<string> warnings = new List<string>();
        public List<string> flagged_folders = new List<string>();
        public bool search_within_files = false;
        public bool match_case = false;
        public string optional_folder_to_search = "";

        List<string> columns = new List<string>();

        // drag and drop
        private bool itemDragStart = false;
        string dragItemTempFileName = string.Empty;
        //A prefix used in filename to identify a drag from the application
        //Make it more unique
        internal const string DRAG_SOURCE_PREFIX = "__DragNDrop__Temp__";
        //The item which we drag is being stored here
        //could be a path to a location or a filestream
        internal static object objDragItem;
        //A FileSystemWatcher used to monitor the System's Temp Directory for a drag from the application
        private static FileSystemWatcher tempDirectoryWatcher;
        //A Hashtable to keep multiple FileSystemWatchers 
        public static Hashtable watchers = null;
        #region FileSystemWatcher Methods
        /// <summary>
        /// A temp file is created in the temp folder with the prefix wen a drag is initiated
        /// At that point we will add a filesystem watch to all the Logical Drives present.
        /// And will clear the watchers when the drag is completed or canceled
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private static void TempDirectoryWatcherCreated(object sender, FileSystemEventArgs e)
        {
            try
            {
                CreateFileWatchers();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "DragNDrop Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        /// <summary>
        /// This Event is triggered when the temp file created is Dropped into the target folder.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private static void FileWatcherCreated(object sender, FileSystemEventArgs e)
        {
            try
            {
                OnFileDropPathFound(e.FullPath);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "DragNDrop Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        /// <summary>
        /// Methodd Called when the Dropped Path is found
        /// </summary>
        /// <param name="dropedFilePath"></param>
        private static void OnFileDropPathFound(string dropedFilePath)
        {
            try
            {
                if (dropedFilePath.Trim() != string.Empty && objDragItem != null)
                {
                    string dropPath = dropedFilePath.Substring(0, dropedFilePath.LastIndexOf('\\'));

                    if (File.Exists(dropedFilePath))
                        File.Delete(dropedFilePath);

                    //Use your Download Code here
                    MessageBox.Show(String.Format("{0} dropped to {1}", objDragItem.ToString(), dropPath));
                }
                objDragItem = null;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "DragNDrop Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Creating File Watchers for all Logical Drives in the System
        /// </summary>
        public static void CreateFileWatchers()
        {
            try
            {
                if (watchers == null)
                {
                    int i = 0;
                    Hashtable tempWatchers = new Hashtable();
                    FileSystemWatcher watcher;
                    //Adding FileSystemWatchers and adding Created event to it 
                    foreach (string driveName in Directory.GetLogicalDrives())
                    {
                        //this checking may sound absurd to you.
                        //but in OS like Windows 2000,
                        //even if there is no floppy drive present
                        //An "A:/" will be shown in My Computer.
                        //To avoid exceptions like this I've added this check.
                        if (Directory.Exists(driveName))
                        {
                            watcher = new FileSystemWatcher();
                            watcher.Filter = string.Format("{0}*.tmp", DRAG_SOURCE_PREFIX);
                            watcher.NotifyFilter = NotifyFilters.FileName;
                            watcher.Created += new FileSystemEventHandler(FileWatcherCreated);
                            watcher.IncludeSubdirectories = true;
                            watcher.Path = driveName;
                            watcher.EnableRaisingEvents = true;
                            tempWatchers.Add("file_watcher" + i.ToString(), watcher);
                            i = i + 1;
                        }
                    }
                    watchers = tempWatchers;
                    tempWatchers = null;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "DragNDrop Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        /// <summary>
        /// Removes the FileSystemWatchers watching the Logical Drives
        /// </summary>
        public static void ClearFileWatchers()
        {
            try
            {
                if (watchers != null && watchers.Count > 0)
                {
                    for (int i = 0; i < watchers.Count; i++)
                    {
                        ((FileSystemWatcher)watchers["file_watcher" + i.ToString()]).Dispose();
                    }
                    watchers.Clear();
                    watchers = null;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "DragNDrop Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        #endregion

        public frmSearch(frmMain parent)
        {
            InitializeComponent();
            frmMain_instance = parent;
        }

        private void frmSearch_Load(object sender, EventArgs e)
        {
            columns.Add("file");
            columns.Add("date modified");
            columns.Add("size (mb)");
            columns.Add("path");
            DataGridView_Setup_Columns();

            // set up our width and height from the INI file.
            int this_width = Globals.ReadIntFromINI("search_window", "this_width", Globals.OptionsINI, 1000);
            int this_height = Globals.ReadIntFromINI("search_window", "this_height", Globals.OptionsINI, 685);
            if (this_width > 200)
            {
                this.Width = this_width;
            }
            if (this_height > 200)
            {
                this.Height = this_height;
            }
            this.CenterToScreen();
            if (optional_folder_to_search.Length == 0)
                Get_Search_Directories();
        }

        public void Get_Search_Directories()
        {
            // read our search.ini file to get a list of directories to search.
            List<string> search_directories = new List<string>();
            string search_dir = Globals.WorkPath + "\\search";
            string search_ini = Globals.WorkPath + "\\search\\search.ini";
            try
            {
                if (Directory.Exists(search_dir) == false)
                {
                    Directory.CreateDirectory(search_dir);
                }
                if (File.Exists(search_ini) == false)
                {
                    File.WriteAllText(search_ini, "");
                }
            }
            catch (Exception f)
            {
                MessageBox.Show("The search file (" + search_ini + ") could not be read. " + f.Message, "Keybreeze Desktop");
            }
            string item_path = "";
            int subfolders;
            int item_checked;
            string filetypes;
            int search_folder_names;
            for (int i = 1; i <= 250; i++)
            {
                item_path = Globals.ReadINI("search", "item" + i + "-path", search_ini);
                item_checked = Globals.ReadIntFromINI("search", "item" + i + "-checked", search_ini, 1);
                if (item_path.Length > 0 && item_checked == 1)
                {
                    search_entry2 this_search_entry = new search_entry2();
                    subfolders = Globals.ReadIntFromINI("search", "item" + i + "-subfolders", search_ini, 0);
                    filetypes = Globals.ReadINI("search", "item" + i + "-filetypes", search_ini);
                    search_folder_names = Globals.ReadIntFromINI("search", "item" + i + "-toplevelfolders", search_ini, 0);
                    this_search_entry.search_subfolders = subfolders;
                    this_search_entry.search_folder_names = search_folder_names;
                    this_search_entry.search_filetypes = filetypes;
                    this_search_entry.search_folder = Globals.Replace_Special_Folder_Name(item_path);
                    search_entries.Add(this_search_entry);
                }
                else
                    break;
            }
        }

        public void Search_Start_New_Thread()
        {
            // loop through our directories to search.
            foreach (search_entry2 this_entry in search_entries)
            {
                if (Directory.Exists(this_entry.search_folder) == true)
                {
                    // directory exists.
                    if (this_entry.search_subfolders == 1)
                    {
                        // search all subfolders in this directory. Then search for files.
                        // search top level directory as well.
                        Search_Start_Scan_Files_In_Directory(this_entry.search_folder, this_entry.search_filetypes);
                        Search_Start_Scan_Subfolders(this_entry.search_folder, this_entry.search_filetypes, this_entry.search_folder_names);
                    }
                    else
                    {
                        // we are only searching the top level directory.
                        Search_Start_Scan_Files_In_Directory(this_entry.search_folder, this_entry.search_filetypes);
                    }
                }
                else
                {
                    // the directory does not exist.
                    warnings.Add("Directory does not exist -- " + this_entry.search_folder);
                    return;
                }
                if (StopSearching == true)
                {
                    break;
                }
            }
            string result_label;
            string warning_label;
            if (search_result_count == 1)
                result_label = "result.";
            else
                result_label = "results.";
            if (warnings.Count == 1)
                warning_label = "warning.";
            else
                warning_label = "warnings.";
            if (warnings.Count == 0)
                Search_Update_Status(search_result_count + " " + result_label);
            else
                Search_Update_Status(search_result_count + " " + result_label + " " + warnings.Count + " " + warning_label);
            StopSearching = true;
            try
            {
                this.Invoke(new EventHandler(delegate
                {
                    btnStopSearch.Text = "Close [Esc]";
                }));
            }
            catch { }
        }

        public void Search_Start_Scan_Subfolders(string top_level_folder, string filetypes, int search_folder_names)
        {
            if (StopSearching == true)
            {
                Search_Update_Status("User canceled search.");
                return;
            }
            if (search_folder_names == 1)
            {
                // see if our search string is in the folder name.
                string top_level_folder_name = Path.GetFileName(top_level_folder);
                top_level_folder_name = top_level_folder_name.Replace("_", " ");
                top_level_folder_name = top_level_folder_name.Replace("-", "");
                if (top_level_folder_name.Length >= global_search_string_length)
                {
                    int find_instance;
                    if (match_case == false)
                        find_instance = top_level_folder_name.ToLower().IndexOf(global_search_string_lowercase);
                    else
                        find_instance = top_level_folder_name.IndexOf(global_search_string);
                    if (find_instance != -1)
                    {
                        // we found a match.
                        DateTime date_modified = Directory.GetLastWriteTime(top_level_folder);
                        DirectoryInfo DirInfo = new DirectoryInfo(top_level_folder);
                        long ttl = 0;
                        foreach (FileInfo fi in DirInfo.EnumerateFiles("*.*", SearchOption.AllDirectories))
                        {
                            ttl += fi.Length;
                        }
                        var length = ttl;
                        Search_Scan_Flag(false, top_level_folder, date_modified, 0, 0, "", length);
                    }
                }
            }
            Search_Update_Status("Scanning " + top_level_folder);
            foreach (string folder in Directory.EnumerateDirectories(top_level_folder))
            {
                try
                {
                    if (StopSearching == true)
                    {
                        Search_Update_Status("User canceled search.");
                        return;
                    }

                    // search files within this folder.
                    Search_Start_Scan_Files_In_Directory(folder, filetypes);

                    // recursive function search subfolders within our current subfolder.
                    Search_Start_Scan_Subfolders(folder, filetypes, search_folder_names);
                }
                catch (Exception e)
                {
                    warnings.Add("Error while scanning " + top_level_folder + ". " + e.Message);
                }
            }
        }

        public void Search_Start_Scan_Files_In_Directory(string folder, string filetypes)
        {
            // this searches for files in the top level folder (no subfolders).
            SearchOption My_SearchOption = SearchOption.TopDirectoryOnly;
            if (filetypes == "")
            {
                // we are searching all file types.
                foreach (string file in Directory.EnumerateFiles(folder, "*.*", My_SearchOption))
                {
                    Search_Start_Scan_Filename(file, global_search_string_length, global_search_string, global_search_string_lowercase);
                    if (search_within_files == true)
                        Search_Start_Scan_Within_File(file, global_search_string_length, global_search_string, global_search_string_lowercase);
                    if (StopSearching == true)
                    {
                        Search_Update_Status("User canceled search.");
                        return;
                    }
                    if (skip_directory == true)
                    {
                        skip_directory = false;
                        warnings.Add("User skipped directory: " + folder);
                        break;
                    }
                }
            }
            else
            {
                // we are only processing certain file types.
                try
                {
                    string[] FileTypeArray = filetypes.Split(',');
                    foreach (string filetype in FileTypeArray)
                    {
                        foreach (string file in Directory.EnumerateFiles(folder, "*" + filetype, My_SearchOption))
                        {
                            Search_Start_Scan_Filename(file, global_search_string_length, global_search_string, global_search_string_lowercase);
                            if (search_within_files == true)
                                Search_Start_Scan_Within_File(file, global_search_string_length, global_search_string, global_search_string_lowercase);
                            if (StopSearching == true)
                            {
                                Search_Update_Status("User canceled search.");
                                return;
                            }
                            if (skip_directory == true)
                            {
                                skip_directory = false;
                                warnings.Add("User skipped directory: " + folder);
                                break;
                            }
                        }
                    }
                }
                catch
                {
                    warnings.Add("An error occurred while processing the file types. Please make sure they are formatted correctly. e.g. *.* for all file types or *.txt,*.log,*.ini for multiple file types.");
                    return;
                }
            }
        }

        public void Search_Start_Scan_Within_File(string file, int search_string_length, string search_string, string search_string_lowercase)
        {
            string in_directory = Path.GetDirectoryName(file);
            string filename_only = Path.GetFileName(file).ToLower();
            foreach (string line in File.ReadLines(file))
            {
                if (StopSearching == true)
                {
                    Search_Update_Status("User canceled search.");
                    return;
                }
                if (skip_file == true)
                {
                    skip_file = false;
                    warnings.Add("User skipped file: " + file);
                    break;
                }
                string line_lower = line.ToLower();
                // update by Andrew on March 18th, 2018.
                line_lower = line_lower.Replace("_", " ");
                line_lower = line_lower.Replace("-", "");
                int find_instance;
                if (match_case == false)
                    find_instance = line_lower.IndexOf(search_string_lowercase);
                else
                    find_instance = line.IndexOf(search_string);
                if (find_instance != -1)
                {
                    // we found a match.
                    DateTime date_modified = File.GetLastWriteTime(file);
                    var length = new System.IO.FileInfo(file).Length;
                    Search_Scan_Flag(false, file, date_modified, 0, 0, "", length);
                    // break after finding one match within the file.
                    break;
                }
            }
            return;
        }

        public void Search_Start_Scan_Filename(string file, int search_string_length, string search_string, string search_string_lowercase)
        {
            if (StopSearching == true)
            {
                Search_Update_Status("User canceled search.");
                return;
            }
            string in_directory = Path.GetDirectoryName(file);
            string filename_only = Path.GetFileName(file);
            // update by Andrew on March 18th, 2018.
            filename_only = filename_only.Replace("_", " ");
            filename_only = filename_only.Replace("-", "");
            filename_only = filename_only.Replace(" ", "");
            string filename_only_lower = filename_only.ToLower();
            if (filename_only.Length >= search_string_length)
            {
                int find_instance;
                if (match_case == false)
                    find_instance = filename_only_lower.IndexOf(search_string_lowercase);
                else
                    find_instance = filename_only.IndexOf(search_string);
                if (find_instance != -1)
                {
                    // we found a match.
                    DateTime date_modified = File.GetLastWriteTime(file);
                    var length = new System.IO.FileInfo(file).Length;
                    Search_Scan_Flag(false, file, date_modified, 0, 0, "", length);
                }
            }
            return;
        }

        public void Search_Start(string search_string, string optional_folder_to_search_l = "")
        {
            global_search_string = search_string;
            
            // Added by Andrew on March 18th, 2018.
            global_search_string = global_search_string.Replace("_", " ");
            global_search_string = global_search_string.Replace("-", "");
            global_search_string = global_search_string.Replace(" ", "");
            global_search_string_lowercase = global_search_string.ToLower();
            global_search_string_length = global_search_string.Length;

            this.Text = "Search results for '" + search_string + "'";
            btnStopSearch.Text = "Stop search [Esc]";
            global_search_string_length = global_search_string.Length;
            global_search_string_lowercase = global_search_string.ToLower();
            StopSearching = false;
            warnings.Clear();
            flagged_folders.Clear();
            dataGridView1.Rows.Clear();
            search_result_count = 0;
            if (optional_folder_to_search_l.Length > 0)
            {
                if (search_entries.Count > 0)
                    search_entries.Clear();
                search_entry2 this_search_entry = new search_entry2();
                this_search_entry.search_subfolders = 1;
                this_search_entry.search_folder_names = 1;
                this_search_entry.search_filetypes = "";
                this_search_entry.search_folder = Globals.Replace_Special_Folder_Name(optional_folder_to_search_l);
                search_entries.Add(this_search_entry);
                optional_folder_to_search = optional_folder_to_search_l;
                this.Text = this.Text + " in " + optional_folder_to_search_l;
            }
            // Start new thread to scan files.
            Thread workerThread = new Thread(Search_Start_New_Thread);
            workerThread.IsBackground = true;
            workerThread.Start();
        }

        public void Search_Update_Status(string update_status)
        {
            try
            {
                this.Invoke(new EventHandler(delegate
                {
                    lblStatus.Text = update_status;
                }));
            }
            catch { }
        }

        private string DataGridView_Setup_Columns()
        {
            dataGridView1.Font = new System.Drawing.Font("Arial", 9, FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            foreach (string this_item in columns)
            {
                DataGridViewTextBoxColumn new_column = new DataGridViewTextBoxColumn();
                if (this_item == "file")
                {
                    new_column.Name = "File";
                    new_column.Width = 250;
                }
                else if (this_item == "line")
                {
                    new_column.Name = "Line";
                    new_column.Width = 200;
                    new_column.ValueType = typeof(Int64);
                }
                else if (this_item == "line number")
                {
                    new_column.Name = "Line #";
                    new_column.Width = 50;
                }
                else if (this_item == "date modified")
                {
                    new_column.Name = "Date Modified";
                    new_column.Width = 140;
                }
                else if (this_item == "date created")
                {
                    new_column.Name = "Date created";
                    new_column.Width = 140;
                }
                else if (this_item == "size (mb)")
                {
                    new_column.Name = "Size (MB)";
                    new_column.Width = 60;
                    new_column.ValueType = typeof(double);
                }
                else if (this_item == "directory")
                {
                    new_column.Name = "Directory";
                    new_column.Width = 600;
                }
                else if (this_item == "path")
                {
                    new_column.Name = "Path";
                    new_column.Width = 1200;
                }
                dataGridView1.Columns.Add(new_column);
            }
            return "success";
        }

        private void Search_Scan_Flag(bool content_match, string file_path, DateTime date_modified, int match_position, int line_number, string line_text, double file_length)
        {
            // This function will add a record to our dataGridView.
            this.Invoke(new EventHandler(delegate
            {
                int index = dataGridView1.Rows.Add();
                DataGridViewRow new_row = dataGridView1.Rows[index];
                int column_cursor = 0;
                foreach (string this_column in columns)
                {
                    string this_column_lower = this_column.ToLower();
                    if (this_column_lower == "file")
                    {
                        new_row.Cells[column_cursor].Value = Path.GetFileName(file_path);
                    }
                    else if (this_column_lower == "line")
                    {
                        new_row.Cells[column_cursor].Value = line_text;
                    }
                    else if (this_column_lower == "line number")
                    {
                        new_row.Cells[column_cursor].Value = line_number;
                    }
                    else if (this_column_lower == "size (mb)")
                    {
                        double size_double = ConvertSize(file_length, "MB");
                        double rounded_size;
                        if (size_double < 1)
                            rounded_size = Math.Round(size_double, 2);
                        else
                            rounded_size = Math.Round(size_double, 0);
                        new_row.Cells[column_cursor].Value = rounded_size;
                    }
                    else if (this_column_lower == "directory")
                    {
                        new_row.Cells[column_cursor].Value = Path.GetDirectoryName(file_path);
                    }
                    else if (this_column_lower == "path")
                    {
                        new_row.Cells[column_cursor].Value = file_path;
                    }
                    else if (this_column_lower == "date modified")
                    {
                        new_row.Cells[column_cursor].Value = date_modified;
                    }
                    column_cursor++;
                }
            }));
            search_result_count++;
        }
        private double ConvertSize(double bytes, string type)
        {
            try
            {
                const int CONVERSION_VALUE = 1024;
                //determine what conversion they want
                type = type.ToUpper();
                switch (type)
                {
                    case "BY":
                        //convert to bytes (default)
                        return bytes;
                    case "KB":
                        //convert to kilobytes
                        return (bytes / CONVERSION_VALUE);
                    case "MB":
                        //convert to megabytes
                        return (bytes / CalculateSquare(CONVERSION_VALUE));
                    case "GB":
                        //convert to gigabytes
                        return (bytes / CalculateCube(CONVERSION_VALUE));
                    case "TB":
                        //convert to terabytes
                        return (bytes / CalculateFour(CONVERSION_VALUE));
                    default:
                        //default
                        return bytes;
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                return 0;
            }
        }
        private double CalculateSquare(Int32 number)
        {
            return Math.Pow(number, 2);
        }
        private double CalculateCube(Int32 number)
        {
            return Math.Pow(number, 3);
        }
        private double CalculateFour(Int64 number)
        {
            return Math.Pow(number, 4);
        }

        private void btnStopSearch_Click(object sender, EventArgs e)
        {
            if (btnStopSearch.Text.IndexOf("Stop") != -1)
            {
                StopSearching = true;
                skipDirectorydToolStripMenuItem.Visible = false;
                Search_Update_Status("User canceled search.");
                btnStopSearch.Text = "Close [Esc]";
            }
            else if (btnStopSearch.Text.IndexOf("Close") != -1)
            {
                this.Close();
            }
        }

        private void btnLaunch_Click(object sender, EventArgs e)
        {
            
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Launch_Open_File();
        }

        private void Launch_Open_File()
        {
            try
            {
                if (dataGridView1.RowCount == 0)
                {
                    MessageBox.Show("No rows selected.", "Keybreeze Desktop");
                    return;
                }
                int current_row = dataGridView1.CurrentCell.RowIndex;
                string log_path = dataGridView1.Rows[current_row].Cells.GetCellValueFromColumnHeader("Path").ToString();
                frmMain_instance.RunTarget(log_path);
            }
            catch (Exception e)
            {
                MessageBox.Show("An error occurred while running the target. " + e.Message, "Keybreeze Desktop");
            }
        }

        private void Launch_Open_Containing_Folder()
        {
            if (dataGridView1.RowCount == 0)
            {
                MessageBox.Show("No rows selected.", "Keybreeze Desktop");
                return;
            }
            int current_row = dataGridView1.CurrentCell.RowIndex;
            string the_path = dataGridView1.Rows[current_row].Cells.GetCellValueFromColumnHeader("Path").ToString();
            string directory_name = Path.GetDirectoryName(the_path);
            frmMain_instance.RunTarget(directory_name);
        }

        private void openContainingFolderToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Launch_Open_Containing_Folder();
        }

        private void Launch_Add_Keyword()
        {
            if (dataGridView1.RowCount == 0)
            {
                MessageBox.Show("No rows selected.", "Keybreeze Desktop");
                return;
            }
            int current_row = dataGridView1.CurrentCell.RowIndex;
            string the_path = dataGridView1.Rows[current_row].Cells.GetCellValueFromColumnHeader("Path").ToString();
            string filename_without_ext = Path.GetFileNameWithoutExtension(the_path);
            frmMain_instance.ShowAddKeywordForm(false, filename_without_ext, the_path);
        }

        private void addKeywordToKeybreezeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Launch_Add_Keyword();
        }

        private void frmSearch_FormClosing(object sender, FormClosingEventArgs e)
        {
            StopSearching = true;
        }

        private void dataGridView1_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                // user pressed the Enter key. Let's open the file.
                Launch_Open_File();
                e.Handled = true;
            }
            else if (e.KeyCode == Keys.A)
            {
                Launch_Add_Keyword();
            }
            else if (e.KeyCode == Keys.O)
            {
                Launch_Open_Containing_Folder();
            }
            else if (e.KeyCode == Keys.D)
            {
                skip_directory = true;
            }
            else if (e.KeyCode == Keys.F)
            {
                skip_file = true;
            }
            else if (e.KeyCode == Keys.W)
            {
                Options_Search_Within_Files();
            }
            else if (e.KeyCode == Keys.M)
            {
                Options_Search_Match_Case();
            }
        }

        private void btnOptions_Click(object sender, EventArgs e)
        {
            contextMenuStrip1.Show(Cursor.Position);
        }

        private void dataGridView1_MouseDown(object sender, MouseEventArgs e)
        {
            try
            {
                if (e.Button == MouseButtons.Right)
                {
                    contextMenuStrip1.Show(Cursor.Position);
                }
                else
                {
                    ClearDragData();
                    if (e.Button == MouseButtons.Left)
                    {
                        DataGridView.HitTestInfo hit = dataGridView1.HitTest(e.X, e.Y);
                        if (hit.RowIndex == -1)
                        {
                            // the user is clicking a column heading. We do not want to start the drag and drop.
                            return;
                        }
                        else
                        {
                            List<string> selected_files = DataGridView_Get_Selected_Paths();
                            if (selected_files.Count > 0)
                            {
                                objDragItem = selected_files[0];
                                itemDragStart = true;
                            }
                        }
                    }
                }
            }
            catch { }
        }

        private List<string> DataGridView_Get_Selected_Paths()
        {
            List<string> return_list = new List<string>();
            foreach (DataGridViewCell c in dataGridView1.SelectedCells)
            {
                bool already_added = false;
                int selected_row = c.RowIndex;
                string filepath = dataGridView1.Rows[selected_row].Cells.GetCellValueFromColumnHeader("Path").ToString();
                // check for duplicates.
                foreach (string file_added in return_list)
                {
                    if (filepath == file_added)
                    {
                        // already added this one.
                        already_added = true;
                        break;
                    }
                }
                if (already_added == false)
                    return_list.Add(filepath);
            }
            return return_list;
        }

        private void ClearDragData()
        {
            try
            {
                objDragItem = null;
                dragItemTempFileName = string.Empty;
                itemDragStart = false;
                ClearFileWatchers();
            }
            catch (Exception ex)
            {
                MessageBox.Show("The drag and drop did not complete successfully. Please try again, or press CTRL + O to open the folder in Windows Explorer and then complete the drag and drop." + Environment.NewLine + Environment.NewLine + ex.Message, "Keybreeze Download Organizer");
            }
        }

        private void btnLaunch_Click_1(object sender, EventArgs e)
        {
            Launch_Open_File();
        }

        private void dataGridView1_DoubleClick(object sender, EventArgs e)
        {
            Launch_Open_File();
        }

        private void btnUpdate_Click(object sender, EventArgs e)
        {

        }

        private void copyFilenameToClipboardToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (dataGridView1.RowCount == 0)
            {
                MessageBox.Show("No rows selected.", "Keybreeze Desktop");
                return;
            }
            int current_row = dataGridView1.CurrentCell.RowIndex;
            string filename = dataGridView1.Rows[current_row].Cells.GetCellValueFromColumnHeader("File").ToString();
            Clipboard.Clear();
            Clipboard.SetText(filename);
        }

        private void copyPathToClipboardToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (dataGridView1.RowCount == 0)
            {
                MessageBox.Show("No rows selected.", "Keybreeze Desktop");
                return;
            }
            int current_row = dataGridView1.CurrentCell.RowIndex;
            string full_path = dataGridView1.Rows[current_row].Cells.GetCellValueFromColumnHeader("Path").ToString();
            string directory_name = Path.GetDirectoryName(full_path);
            Clipboard.Clear();
            Clipboard.SetText(directory_name);
        }

        private void copyFullPathToClipboardToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (dataGridView1.RowCount == 0)
            {
                MessageBox.Show("No rows selected.", "Keybreeze Desktop");
                return;
            }
            int current_row = dataGridView1.CurrentCell.RowIndex;
            string full_path = dataGridView1.Rows[current_row].Cells.GetCellValueFromColumnHeader("Path").ToString();
            Clipboard.Clear();
            Clipboard.SetText(full_path);
        }

        private void frmSearch_ResizeEnd(object sender, EventArgs e)
        {
            Globals.WriteINI("search_window", "this_height", this.Height.ToString(), Globals.OptionsINI);
            Globals.WriteINI("search_window", "this_width", this.Width.ToString(), Globals.OptionsINI);
        }

        private void chkSearchWithinFiles_CheckedChanged(object sender, EventArgs e)
        {
            
        }

        private void dataGridView1_CellContentClick(object sender, DataGridViewCellEventArgs e)
        {

        }

        private void lblStatus_Click(object sender, EventArgs e)
        {
            if (warnings.Count > 0)
            {
                string warnings_string = "";
                foreach (string warning in warnings)
                {
                    warnings_string = warnings_string + warning + Environment.NewLine;
                }
                frmSearch_Warnings frmSearch_Warnings_instance = new frmSearch_Warnings();
                frmSearch_Warnings_instance.textBox1.Text = warnings_string;
                frmSearch_Warnings_instance.textBox1.SelectionStart = 0;
                frmSearch_Warnings_instance.ShowDialog();
            }
        }

        private void skipDirectorydToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show("While the scan is in progress, you can press D on your keyboard to skip the current directory.", "Keybreeze Desktop");
        }

        private void directoriesToSearchToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmOptions frmOptions_instance = new frmOptions(frmMain_instance);
            frmOptions_instance.start_with_search_tab = true;
            frmOptions_instance.Show();
        }

        private void skipFilefToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show("While the scan is in progress, you can press F on your keyboard to skip the current file if it's large and taking too long to scan.", "Keybreeze Desktop");
        }

        private void repeatSearchSearchWithinFileswToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Options_Search_Within_Files();
        }

        private void Options_Search_Within_Files()
        {
            StopSearching = true;
            Application.DoEvents();
            System.Threading.Thread.Sleep(500);
            Application.DoEvents();
            if (repeatSearchSearchWithinFileswToolStripMenuItem.Checked == false)
            {
                search_within_files = true;
                repeatSearchSearchWithinFileswToolStripMenuItem.Checked = true;
                lblOptions.Text = "Options: Search within files";
                if (match_case == true)
                    lblOptions.Text = lblOptions.Text + ", match case";
                Search_Start(global_search_string, optional_folder_to_search);
            }
            else
            {
                search_within_files = false;
                repeatSearchSearchWithinFileswToolStripMenuItem.Checked = false;
                lblOptions.Text = "";
                if (match_case == true)
                    lblOptions.Text = "Options: Match case";
                Search_Start(global_search_string, optional_folder_to_search);
            }
        }

        private void repeatSearchMatchCasemToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Options_Search_Match_Case();
        }

        private void Options_Search_Match_Case()
        {
            StopSearching = true;
            Application.DoEvents();
            System.Threading.Thread.Sleep(500);
            Application.DoEvents();
            if (repeatSearchMatchCasemToolStripMenuItem.Checked == false)
            {
                match_case = true;
                repeatSearchMatchCasemToolStripMenuItem.Checked = true;
                lblOptions.Text = "Options: Match case";
                if (search_within_files == true)
                    lblOptions.Text = lblOptions.Text + ", search within files";
                Search_Start(global_search_string, optional_folder_to_search);
            }
            else
            {
                match_case = false;
                repeatSearchMatchCasemToolStripMenuItem.Checked = false;
                lblOptions.Text = "";
                if (search_within_files == true)
                    lblOptions.Text = "Options: Search within files";
                Search_Start(global_search_string, optional_folder_to_search);
            }
        }

        private void dataGridView1_MouseMove(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.None)
                return;
            if (itemDragStart && objDragItem != null)
            {
                List<string> selected_files = DataGridView_Get_Selected_Paths();
                if (selected_files.Count == 0)
                    return;
                string selected_file = selected_files[0];
                dragItemTempFileName = selected_file;
                try
                {
                    Util.CreateDragItemTempFile(dragItemTempFileName);

                    string[] fileList = new string[] { dragItemTempFileName };
                    DataObject fileDragData = new DataObject(DataFormats.FileDrop, fileList);
                    DoDragDrop(fileDragData, DragDropEffects.Copy);

                    ClearDragData();
                }
                catch (Exception ex)
                {
                    MessageBox.Show("The drag and drop did not complete successfully. Please try again, or press CTRL + O to open the folder in Windows Explorer and then complete the drag and drop." + Environment.NewLine + Environment.NewLine + ex.Message, "Keybreeze Download Organizer");
                }
            }
        }
    }

    public class search_entry2
    {
        public int search_checked;
        public string search_folder;
        public int search_subfolders;
        public int search_folder_names;
        public string search_filetypes;
    }

    public static class DataGridHelper
    {
        public static object GetCellValueFromColumnHeader(this DataGridViewCellCollection CellCollection, string HeaderText)
        {
            return CellCollection.Cast<DataGridViewCell>().First(c => c.OwningColumn.HeaderText == HeaderText).Value;
        }
    }

    internal static class Util
    {
        internal static void CreateDragItemTempFile(string dragItemTempFileName)
        {
            FileStream fsDropFile = null;

            try
            {
                //fsDropFile = new FileStream(dragItemTempFileName, FileMode.Create);
                fsDropFile = new FileStream(dragItemTempFileName, FileMode.Open);
            }
            catch (IOException ex)
            {
                MessageBox.Show("The drag and drop did not complete successfully. Please try again, or press CTRL + O to open the folder in Windows Explorer and then complete the drag and drop." + Environment.NewLine + Environment.NewLine + ex.Message, "Keybreeze Download Organizer");
            }
            finally
            {
                if (fsDropFile != null)
                {
                    fsDropFile.Flush();
                    fsDropFile.Close();
                    fsDropFile.Dispose();
                }
            }
        }
    }
}