﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace Keybreeze
{
    public partial class frmSkin : FormBase
    {
        frmMain frmMain_instance;

        //private Bitmap bitmap;
        public Bitmap bitmap;

        public frmSkin(frmMain parent)
        {
            this.Visible = false;
            InitializeComponent();
            frmMain_instance = parent;
            Bitmap newBitmap;
            try
            {
                string skin_template_name = Globals.ReadINI("options", "skin", Globals.OptionsINI);
                string skin_file_name = Globals.ReadINI("skin", "file", Globals.SkinINI);
                newBitmap = Image.FromFile(Globals.WorkPath + "\\skins\\" + skin_template_name + "\\" + skin_file_name) as Bitmap;
                this.SetBitmap(newBitmap, 255);
                if (bitmap != null)
                    bitmap.Dispose();
                bitmap = newBitmap;
            }
            catch
            {
                MessageBox.Show("The Keybreeze Desktop skin could not be loaded. The file may be corrupted or missing. Keybreeze Desktop will load without a skin. You can try resetting the skin from the Options screen to resolve the issue.", "Skin error");
                Globals.valid_skin = false;
                frmMain_instance.TransparencyKey = System.Drawing.Color.Orange;
                frmMain_instance.BackColor = System.Drawing.Color.Black;
                frmMain_instance.txtMain.BackColor = Color.FromArgb(25, 25, 25);
                //frmMain_instance.txtMain.Top = frmMain_instance.txtMain.Top + 9;
                //frmMain_instance.txtMain.Left = frmMain_instance.txtMain.Left + 4;
                Globals.description_character_limit = 40;
                this.Close();
            }
        }

        private void frmSkin_Load(object sender, EventArgs e)
        {
            
        }

        public void SetBitmap(Bitmap bitmap)
        {
            SetBitmap(bitmap, 255);
        }

        // allow the user to drag the form.
        /*
        protected override void WndProc(ref Message m)
        {
            if (m.Msg == 163 && this.ClientRectangle.Contains(this.PointToClient(new Point(m.LParam.ToInt32()))) && m.WParam.ToInt32() == 2)
                m.WParam = (IntPtr)1;

            base.WndProc(ref m);

            if (m.Msg == 132 && m.Result.ToInt32() == 1)
                m.Result = (IntPtr)2;
        }
         */

        /// <para>Changes the current bitmap with a custom opacity level.  Here is where all happens!</para>
        public void SetBitmap(Bitmap bitmap, byte opacity)
        {
            if (bitmap.PixelFormat != PixelFormat.Format32bppArgb)
                throw new ApplicationException("The bitmap must be 32ppp with alpha-channel.");

            // The ideia of this is very simple,
            // 1. Create a compatible DC with screen;
            // 2. Select the bitmap with 32bpp with alpha-channel in the compatible DC;
            // 3. Call the UpdateLayeredWindow.

            IntPtr screenDc = Win32.GetDC(IntPtr.Zero);
            IntPtr memDc = Win32.CreateCompatibleDC(screenDc);
            IntPtr hBitmap = IntPtr.Zero;
            IntPtr oldBitmap = IntPtr.Zero;

            try
            {
                hBitmap = bitmap.GetHbitmap(Color.FromArgb(0));  // grab a GDI handle from this GDI+ bitmap
                oldBitmap = Win32.SelectObject(memDc, hBitmap);

                Win32.Size size = new Win32.Size(bitmap.Width, bitmap.Height);
                Win32.Point pointSource = new Win32.Point(0, 0);
                Win32.Point topPos = new Win32.Point(Left, Top);
                Win32.BLENDFUNCTION blend = new Win32.BLENDFUNCTION();
                blend.BlendOp = Win32.AC_SRC_OVER;
                blend.BlendFlags = 0;
                blend.SourceConstantAlpha = opacity;
                blend.AlphaFormat = Win32.AC_SRC_ALPHA;

                Win32.UpdateLayeredWindow(Handle, screenDc, ref topPos, ref size, memDc, ref pointSource, 0, ref blend, Win32.ULW_ALPHA);
            }
            finally
            {
                Win32.ReleaseDC(IntPtr.Zero, screenDc);
                if (hBitmap != IntPtr.Zero)
                {
                    Win32.SelectObject(memDc, oldBitmap);
                    //Windows.DeleteObject(hBitmap); // The documentation says that we have to use the Windows.DeleteObject... but since there is no such method I use the normal DeleteObject from Win32 GDI and it's working fine without any resource leak.
                    Win32.DeleteObject(hBitmap);
                }
                Win32.DeleteDC(memDc);
            }
        }

        
        protected override CreateParams CreateParams
        {
            get
            {
                CreateParams cp = base.CreateParams;
                cp.ExStyle |= 0x00080000; // This form has to have the WS_EX_LAYERED extended style
                return cp;
            }
        }

        private void frmSkin_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button != MouseButtons.Right)
            {
                frmMain_instance.MovingForm();
            }
        }

        private void frmSkin_MouseUp(object sender, MouseEventArgs e)
        {
            if (e.Button != MouseButtons.Right)
            {
                frmMain_instance.MoveChildForms();
                frmMain_instance.Visible = true;
                //frmMain_instance.TopMost = true;
            }
        }

        private void cutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmMain_instance.menuCut();
            frmMain_instance.txtMain.Focus();
        }

        private void copyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmMain_instance.menuCopy();
            frmMain_instance.txtMain.Focus();
        }

        private void pasteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmMain_instance.menuPaste();
            frmMain_instance.txtMain.Focus();
        }

        private void selectAllToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmMain_instance.menuSelectAll();
            frmMain_instance.txtMain.Focus();
        }

        private void toolStripMenuItem1_Click(object sender, EventArgs e)
        {
            frmMain_instance.menuAddCommand();
        }

        private void commandListToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmMain_instance.menuCommandList();
        }

        private void resetPositionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmMain_instance.menuResetPosition();
        }

        private void createANoteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmMain_instance.menuCreateNote();
        }

        private void hideAllNotesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmMain_instance.menuHideAllNotes();
        }

        private void resetNotePositionsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmMain_instance.menuResetNotePositions();
        }

        private void showHiddenNotesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmMain_instance.menuShowHiddenNotes();
        }

        private void viewRemindersToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmMain_instance.menuNoteReminders();
        }

        private void optionsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmMain_instance.menuShowOptionsWindow();
        }

        private void updateIndexToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmMain_instance.menuUpdateIndex();
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmMain_instance.menuAbout();
        }

        private void tutorialToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmMain_instance.menuTutorial();
        }

        private void knowledgebaseToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmMain_instance.menuKnowledgebase();
        }

        private void reportABugToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmMain_instance.menuReportABug();
        }

        private void checkForUpdateToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmMain_instance.menuCheckForUpdate();
        }

        private void turnOffHotkeyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmMain_instance.menuDisableHotkey();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmMain_instance.ExitApplication(true);
        }

        private void forumToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmMain_instance.menuForum();
        }

    }

    class APIHelp
    {
        // Required constants
        public const Int32 WS_EX_LAYERED = 0x80000;
        public const Int32 HTCAPTION = 0x02;
        public const Int32 WM_NCHITTEST = 0x84;
        public const Int32 WM_NCLBUTTONDOWN = 0xA1;
        public const Int32 ULW_ALPHA = 0x02;
        public const byte AC_SRC_OVER = 0x00;
        public const byte AC_SRC_ALPHA = 0x01;

        public enum Bool
        {
            False = 0,
            True = 1
        }

        [StructLayout(LayoutKind.Sequential)]
        public struct Point
        {
            public Int32 x;
            public Int32 y;

            public Point(Int32 x, Int32 y) { this.x = x; this.y = y; }
        }

        [StructLayout(LayoutKind.Sequential)]
        public struct Size
        {
            public Int32 cx;
            public Int32 cy;

            public Size(Int32 cx, Int32 cy) { this.cx = cx; this.cy = cy; }
        }

        [StructLayout(LayoutKind.Sequential, Pack = 1)]
        struct ARGB
        {
            public byte Blue;
            public byte Green;
            public byte Red;
            public byte Alpha;
        }

        [StructLayout(LayoutKind.Sequential, Pack = 1)]
        public struct BLENDFUNCTION
        {
            public byte BlendOp;
            public byte BlendFlags;
            public byte SourceConstantAlpha;
            public byte AlphaFormat;
        }

        [DllImport("user32.dll", ExactSpelling = true, SetLastError = true)]
        public static extern Bool UpdateLayeredWindow(IntPtr hwnd, IntPtr hdcDst, ref Point pptDst, ref Size psize, IntPtr hdcSrc, ref Point pprSrc, Int32 crKey, ref BLENDFUNCTION pblend, Int32 dwFlags);

        [DllImport("gdi32.dll", ExactSpelling = true, SetLastError = true)]
        public static extern IntPtr CreateCompatibleDC(IntPtr hDC);

        [DllImport("user32.dll", ExactSpelling = true, SetLastError = true)]
        public static extern IntPtr GetDC(IntPtr hWnd);

        [DllImport("user32.dll", ExactSpelling = true)]
        public static extern int ReleaseDC(IntPtr hWnd, IntPtr hDC);

        [DllImport("gdi32.dll", ExactSpelling = true, SetLastError = true)]
        public static extern Bool DeleteDC(IntPtr hdc);

        [DllImport("gdi32.dll", ExactSpelling = true)]
        public static extern IntPtr SelectObject(IntPtr hDC, IntPtr hObject);

        [DllImport("gdi32.dll", ExactSpelling = true, SetLastError = true)]
        public static extern Bool DeleteObject(IntPtr hObject);
    }

    class Win32
    {
        public enum Bool
        {
            False = 0,
            True
        };


        [StructLayout(LayoutKind.Sequential)]
        public struct Point
        {
            public Int32 x;
            public Int32 y;

            public Point(Int32 x, Int32 y) { this.x = x; this.y = y; }
        }


        [StructLayout(LayoutKind.Sequential)]
        public struct Size
        {
            public Int32 cx;
            public Int32 cy;

            public Size(Int32 cx, Int32 cy) { this.cx = cx; this.cy = cy; }
        }


        [StructLayout(LayoutKind.Sequential, Pack = 1)]
        struct ARGB
        {
            public byte Blue;
            public byte Green;
            public byte Red;
            public byte Alpha;
        }


        [StructLayout(LayoutKind.Sequential, Pack = 1)]
        public struct BLENDFUNCTION
        {
            public byte BlendOp;
            public byte BlendFlags;
            public byte SourceConstantAlpha;
            public byte AlphaFormat;
        }


        public const Int32 ULW_COLORKEY = 0x00000001;
        public const Int32 ULW_ALPHA = 0x00000002;
        public const Int32 ULW_OPAQUE = 0x00000004;

        public const byte AC_SRC_OVER = 0x00;
        public const byte AC_SRC_ALPHA = 0x01;


        [DllImport("user32.dll", ExactSpelling = true, SetLastError = true)]
        public static extern Bool UpdateLayeredWindow(IntPtr hwnd, IntPtr hdcDst, ref Point pptDst, ref Size psize, IntPtr hdcSrc, ref Point pprSrc, Int32 crKey, ref BLENDFUNCTION pblend, Int32 dwFlags);

        [DllImport("user32.dll", ExactSpelling = true, SetLastError = true)]
        public static extern IntPtr GetDC(IntPtr hWnd);

        [DllImport("user32.dll", ExactSpelling = true)]
        public static extern int ReleaseDC(IntPtr hWnd, IntPtr hDC);

        [DllImport("gdi32.dll", ExactSpelling = true, SetLastError = true)]
        public static extern IntPtr CreateCompatibleDC(IntPtr hDC);

        [DllImport("gdi32.dll", ExactSpelling = true, SetLastError = true)]
        public static extern Bool DeleteDC(IntPtr hdc);

        [DllImport("gdi32.dll", ExactSpelling = true)]
        public static extern IntPtr SelectObject(IntPtr hDC, IntPtr hObject);

        [DllImport("gdi32.dll", ExactSpelling = true, SetLastError = true)]
        public static extern Bool DeleteObject(IntPtr hObject);
    }
 }
