﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Keybreeze
{
    public partial class frmSpeedReminder : Form
    {
        DateTime label_time = new DateTime();
        frmMain frmMain_instance;
        public frmSpeedReminder(frmMain parent)
        {
            InitializeComponent();
            frmMain_instance = parent;
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        public void Set_Up_Form(string original_response)
        {
            int snooze_time = Validate_Time(original_response);
            if (snooze_time == -1)
                return;
            label1.Text = "Keybreeze Desktop will pop up a reminder at the time below with the following text." + Environment.NewLine + 
                          "Press Enter now, or modify the text or time below, and press Enter.";
            txtTime.Text = original_response;
            // Try to get the title of the active window.
            try
            {
                int wait_time3 = Globals.ReadIntFromINI("options", "wait_time_before_inserting_text", Globals.OptionsINI, 250);
                System.Threading.Thread.Sleep(wait_time3);
                IntPtr name_of_handle;
                name_of_handle = frmMain.GetForegroundWindow();
                string classname = frmMain.GetWindowClassName(name_of_handle);
                string titlename = frmMain.GetWindowTitle(name_of_handle);
                if (titlename.Length > 0)
                {
                    txtNoteText.Text = titlename + ": ";
                    txtNoteText.SelectionStart = txtNoteText.Text.Length;
                }
            }
            catch { }
        }

        private void frmSpeedReminder_Load(object sender, EventArgs e)
        {

        }

        private void btnOk_Click(object sender, EventArgs e)
        {
            string time_string = txtTime.Text;
            int snooze_time = Validate_Time(time_string);
            if (snooze_time == -1)
            {
                MessageBox.Show("Could not understand " + time_string + ". Please specify 'h' for hours or 'm' for minutes." + Environment.NewLine + "Examples: 2h, 3h, 25m, 1h30m");
                return;
            }
            int UniqueID = create_note_files(txtNoteText.Text);
            SnoozeNote(UniqueID);
        }

        private int Validate_Time(string original_response)
        {
            string user_response = original_response.ToLower();
            int the_hours = 0, the_minutes = 0;
            try
            {
                DateTime day_to_show = DateTime.Now;
                string day_to_show_string = "";
                int find_tomorrow;
                if (user_response.Length > 10)
                {
                    // look at days.
                    find_tomorrow = user_response.IndexOf("tomorrow");
                    if (find_tomorrow != -1)
                    {
                        day_to_show = DateTime.Now.AddDays(1);
                        day_to_show_string = "tomorrow";
                    }
                    int find_day_of_week = user_response.IndexOf("sunday");
                    if (find_day_of_week != -1)
                    {
                        // find the next sunday from today.
                        for (int i = 1; i <= 7; i++)
                        {
                            DateTime next_day = DateTime.Now.AddDays(i);
                            if (next_day.DayOfWeek == DayOfWeek.Sunday)
                            {
                                day_to_show = next_day;
                                day_to_show_string = "sunday";
                                break;
                            }
                        }
                    }
                    find_day_of_week = user_response.IndexOf("monday");
                    if (find_day_of_week != -1)
                    {
                        for (int i = 1; i <= 7; i++)
                        {
                            DateTime next_day = DateTime.Now.AddDays(i);
                            if (next_day.DayOfWeek == DayOfWeek.Monday)
                            {
                                day_to_show = next_day;
                                break;
                            }
                        }
                    }
                    find_day_of_week = user_response.IndexOf("tuesday");
                    if (find_day_of_week != -1)
                    {
                        for (int i = 1; i <= 7; i++)
                        {
                            DateTime next_day = DateTime.Now.AddDays(i);
                            if (next_day.DayOfWeek == DayOfWeek.Tuesday)
                            {
                                day_to_show = next_day;
                                break;
                            }
                        }
                    }
                    find_day_of_week = user_response.IndexOf("wednesday");
                    if (find_day_of_week != -1)
                    {
                        for (int i = 1; i <= 7; i++)
                        {
                            DateTime next_day = DateTime.Now.AddDays(i);
                            if (next_day.DayOfWeek == DayOfWeek.Wednesday)
                            {
                                day_to_show = next_day;
                                break;
                            }
                        }
                    }
                    find_day_of_week = user_response.IndexOf("thursday");
                    if (find_day_of_week != -1)
                    {
                        for (int i = 1; i <= 7; i++)
                        {
                            DateTime next_day = DateTime.Now.AddDays(i);
                            if (next_day.DayOfWeek == DayOfWeek.Thursday)
                            {
                                day_to_show = next_day;
                                break;
                            }
                        }
                    }
                    find_day_of_week = user_response.IndexOf("friday");
                    if (find_day_of_week != -1)
                    {
                        for (int i = 1; i <= 7; i++)
                        {
                            DateTime next_day = DateTime.Now.AddDays(i);
                            if (next_day.DayOfWeek == DayOfWeek.Friday)
                            {
                                day_to_show = next_day;
                                break;
                            }
                        }
                    }
                    find_day_of_week = user_response.IndexOf("saturday");
                    if (find_day_of_week != -1)
                    {
                        for (int i = 1; i <= 7; i++)
                        {
                            DateTime next_day = DateTime.Now.AddDays(i);
                            if (next_day.DayOfWeek == DayOfWeek.Saturday)
                            {
                                day_to_show = next_day;
                                break;
                            }
                        }
                    }
                    // the user specified a day to show the reminder. Find the time, such as Sunday at 3 pm, or Friday at 7:00 am, etc.
                    string text_after_day = "";
                    if (find_tomorrow != -1)
                    {
                        text_after_day = user_response.Substring(find_tomorrow + 8).Trim();
                    }
                    else
                    {
                        int find_day = user_response.IndexOf("day");
                        text_after_day = user_response.Substring(find_day + 3).Trim();
                    }
                    // find a number, since that should represent the hour.
                    string hour_so_far = "";
                    string minute_so_far = "";
                    bool minutes_next = false;
                    foreach (char this_char in text_after_day)
                    {
                        if (this_char >= '0' && this_char <= '9')
                        {
                            // we found a number. Is this the hour or the minutes?
                            if (minutes_next == false)
                                hour_so_far += this_char;
                            else
                                minute_so_far += this_char;
                        }
                        else if (hour_so_far.Length > 0 && (this_char == ' ' || this_char == ':'))
                        {
                            // minutes should be next.
                            minutes_next = true;
                        }
                    }
                    string ampm = "";
                    if (text_after_day.IndexOf("am") != -1)
                        ampm = "am";
                    else if (text_after_day.IndexOf("pm") != -1)
                        ampm = "pm";
                    if (minute_so_far.Length == 0)
                        minute_so_far = "00";
                    string complete_time_string = day_to_show.ToShortDateString() + " " + hour_so_far + ":" + minute_so_far + " " + ampm;
                    DateTime time_validation;
                    if (DateTime.TryParse(complete_time_string, out time_validation))
                    {
                        label_time = time_validation;
                        return 1;
                    }
                    else
                    {
                        return -1;
                    }
                }
                else
                {
                    // the user may have specified hours and minutes instead.
                    int find_h = user_response.IndexOf("h");
                    int find_m = user_response.IndexOf("m");
                    if (find_h == -1 && find_m == -1)
                    {
                        return -1;
                    }
                    if (find_h > 0)
                    {
                        // get numbers before h.
                        string before_h = user_response.Substring(0, find_h);
                        the_hours = Convert.ToInt32(before_h);
                        user_response = user_response.Substring(find_h + 1);
                        // reset find_m
                        find_m = user_response.IndexOf("m");
                    }
                    if (find_m > 0)
                    {
                        // get numbers before m
                        string before_m = user_response.Substring(0, find_m);
                        the_minutes = Convert.ToInt32(before_m);
                    }
                    int snooze_time = (the_hours * 60) + the_minutes;
                    DateTime time_validation = new DateTime();
                    time_validation = DateTime.Now.AddMinutes(snooze_time);
                    label_time = time_validation;
                    return 1;
                }
            }
            catch
            {
                // we don't want to display a message box while the user is typing a keyword.
                //MessageBox.Show("Could not understand " + original_response + ". Please specify 'h' for hours or 'm' for minutes." + Environment.NewLine + "Examples: 2h, 3h, 25m, 1h30m");
                return -1;
            }
            return 0;
        }

        private void SnoozeNote(int UniqueID)
        {
            int i = Globals.notes_set_reminder(1, label_time.Year, label_time.Month, label_time.Day, label_time.Hour, label_time.Minute, 0, UniqueID.ToString());
            if (i == 0)
            {
                MessageBox.Show("The Keybreeze Desktop configuration file could not be modified. Please make sure you have read and write access to the " + Globals.WorkPath + "\\notes directory and all subfolders.", "Keybreeze Desktop");
                return;
            }
            this.Close();
        }

        private int create_note_files(string note_text)
        {
            // this will create the necessary files for the note -- the .txt file and .ini file.
            // it will return a uniqueid if successful or -1 if it failed.
            string NotesDir = Globals.WorkPath + "\\notes";
            int UniqueID;
            string INI_Text;

            Random random = new Random();
            UniqueID = random.Next(1, 999999999);

            INI_Text = "[properties]" + System.Environment.NewLine;
            INI_Text = INI_Text + "hiddenreminder=1" + System.Environment.NewLine;
            INI_Text = INI_Text + "top=0" + System.Environment.NewLine;
            INI_Text = INI_Text + "left=0" + System.Environment.NewLine;
            INI_Text = INI_Text + "width=" + Globals.ReadINI("notes", "width", Globals.NotesINI) + System.Environment.NewLine;
            INI_Text = INI_Text + "height=" + Globals.ReadINI("notes", "height", Globals.NotesINI) + System.Environment.NewLine;
            INI_Text = INI_Text + "backcolor=0" + System.Environment.NewLine;
            INI_Text = INI_Text + "image=0" + System.Environment.NewLine;
            INI_Text = INI_Text + "textcolor=" + Globals.ReadINI("notes", "textcolor", Globals.NotesINI) + System.Environment.NewLine;
            INI_Text = INI_Text + "font=" + Globals.ReadINI("notes", "font", Globals.NotesINI) + System.Environment.NewLine;
            INI_Text = INI_Text + "size=" + Globals.ReadINI("notes", "size", Globals.NotesINI) + System.Environment.NewLine;
            INI_Text = INI_Text + "bold=" + Globals.ReadINI("notes", "bold", Globals.NotesINI) + System.Environment.NewLine;
            INI_Text = INI_Text + "italic=" + Globals.ReadINI("notes", "italic", Globals.NotesINI) + System.Environment.NewLine;
            INI_Text = INI_Text + "underline=" + Globals.ReadINI("notes", "underline", Globals.NotesINI) + System.Environment.NewLine;
            INI_Text = INI_Text + "sound=" + Globals.ReadINI("notes", "sound", Globals.NotesINI) + System.Environment.NewLine;
            INI_Text = INI_Text + "sound_file=" + Globals.ReadINI("notes", "sound_file", Globals.NotesINI) + System.Environment.NewLine;
            INI_Text = INI_Text + "sound_loop=" + Globals.ReadINI("notes", "sound_loop", Globals.NotesINI) + System.Environment.NewLine;
            try
            {
                System.IO.File.WriteAllText(NotesDir + "\\" + UniqueID.ToString() + ".txt", note_text, Encoding.Unicode);
                System.IO.File.WriteAllText(NotesDir + "\\" + UniqueID.ToString() + ".ini", INI_Text, Encoding.Unicode);
                return UniqueID;
            }
            catch (Exception e)
            {
                MessageBox.Show(this, "Keybreeze Desktop could not write to the " + NotesDir + " directory to save the text of this note. " + e.Message, "Warning");
                return -1;
            }
        }

        private void txtNoteText_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Control && (e.KeyCode == Keys.A))
            {
                if (sender != null)
                    ((TextBox)sender).SelectAll();
                e.SuppressKeyPress = true;
                e.Handled = true;
            }
        }

        private void txtTime_TextChanged(object sender, EventArgs e)
        {
            int success = Validate_Time(txtTime.Text);
            if (success == 1)
            {
                lblTimeValidation.Text = label_time.ToString("dddd, yyyy/MM/dd HH:mm");
            }
            else
            {
                lblTimeValidation.Text = "Cannot validate time";
            }
        }

        private void txtNoteText_TextChanged(object sender, EventArgs e)
        {

        }
    }
}
