﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace Keybreeze_install
{
    public static class SetupGlobals
    {
        [DllImport("kernel32")]
        private static extern int GetPrivateProfileString(string section, string key, string def, StringBuilder retVal, int size, string filePath);

        [DllImport("KERNEL32.DLL", EntryPoint = "WritePrivateProfileStringW",
          SetLastError = true,
          CharSet = CharSet.Unicode, ExactSpelling = true,
          CallingConvention = CallingConvention.StdCall)]
        private static extern int WritePrivateProfileString(
          string lpAppName,
          string lpKeyName,
          string lpString,
          string lpFilename);

        public static string Keybreeze3Path = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + "\\Keybreeze Data";
        public static string Keybreeze4Path = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + "\\Keybreeze4";
        public static string Keybreeze5Path = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + "\\Keybreeze5";
        public static string InstallDir = Path.GetDirectoryName(Application.ExecutablePath);
        public static bool closing_program = false;

        public static string ReadINI(string Section, string Key, string thePath)
        {
            StringBuilder temp = new StringBuilder(255);
            int i = GetPrivateProfileString(Section, Key, "", temp, 255, thePath);
            return temp.ToString();
        }

        public static int ReadIntFromINI(string Section, string Key, string thePath, int set_default = -1)
        {
            // this will read an integer from an INI file. If the integer is blank, normally it would
            // error out. However, this function allows the option to set a default value to insert in 
            // case the value is currently blank in the INI.
            StringBuilder temp = new StringBuilder(255);
            int i = GetPrivateProfileString(Section, Key, "", temp, 255, thePath);
            string the_text = temp.ToString();
            try
            {
                return Convert.ToInt32(the_text);
            }
            catch
            {
                if (set_default > -1)
                    WriteINI(Section, Key, set_default.ToString(), thePath);
                else
                    set_default = 0;

                return set_default;
            }

        }

        public static int WriteINI(string Section, string Key, string Value, string thePath)
        {
            int try_once;
            try
            {
                try_once = WritePrivateProfileString(Section, Key, Value, thePath);
                if (try_once == 0)
                    return 0;
                else
                    return 1;
            }
            catch
            {
                return 0;
            }
        }

        public static void ExitApplication(bool LaunchKeybreeze)
        {
            // launch keybreeze.exe and exit this utility.
            if (LaunchKeybreeze == true)
            {
                System.Diagnostics.Process.Start(InstallDir + "\\keybreeze.exe");
            }
            closing_program = true;
            System.Environment.Exit(-1);
        }

        public static void Copy(string sourceDirectory, string targetDirectory)
        {
            DirectoryInfo diSource = new DirectoryInfo(sourceDirectory);
            DirectoryInfo diTarget = new DirectoryInfo(targetDirectory);

            CopyAll(diSource, diTarget);
        }

        public static void CopyAll(DirectoryInfo source, DirectoryInfo target)
        {
            Directory.CreateDirectory(target.FullName);

            // Copy each file into the new directory.
            foreach (FileInfo fi in source.GetFiles())
            {
                fi.CopyTo(Path.Combine(target.FullName, fi.Name), true);
            }

            // Copy each subdirectory using recursion.
            foreach (DirectoryInfo diSourceSubDir in source.GetDirectories())
            {
                DirectoryInfo nextTargetSubDir =
                    target.CreateSubdirectory(diSourceSubDir.Name);
                CopyAll(diSourceSubDir, nextTargetSubDir);
            }
        }
    }
}
