﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace Keybreeze_install
{
    public partial class frmSetupHotkey : Form
    {
        public frmSetupHotkey()
        {
            InitializeComponent();
        }

        private void frmHotkey_Load(object sender, EventArgs e)
        {
            // see if the user did the extract option or install option.
            string extracted = SetupGlobals.ReadINI("Extract", "Extract_Not_Install", SetupGlobals.InstallDir + "\\default settings\\options.ini");
            if (extracted == "1")
            {
                SetupGlobals.Keybreeze5Path = SetupGlobals.InstallDir;
            }
            if (File.Exists(SetupGlobals.Keybreeze5Path + "\\options.ini") == true)
            {
                // this utility has already been ran for this user. we don't want to run it again.
                SetupGlobals.ExitApplication(true);
                this.Close();
                return;
            }

            // Check for Keybreeze version 4. We need to copy this data to the Keybreeze5 folder.
            if (Directory.Exists(SetupGlobals.Keybreeze5Path) == false && Directory.Exists(SetupGlobals.Keybreeze4Path) == true)
            {
                try
                {
                    SetupGlobals.Copy(SetupGlobals.Keybreeze4Path, SetupGlobals.Keybreeze5Path);
                    // delete the old Skins folder and put the new one in place.
                    Directory.Delete(SetupGlobals.Keybreeze5Path + "\\skins", true);
                    SetupGlobals.Copy(SetupGlobals.InstallDir + "\\default settings\\skins", SetupGlobals.Keybreeze5Path + "\\skins");
                    // update our skin to the Keybreeze 5 skin.
                    SetupGlobals.WriteINI("options", "skin", "Keybreeze Slate", SetupGlobals.Keybreeze5Path + "\\options.ini");
                }
                catch (Exception e2)
                {
                    MessageBox.Show("Could not copy data from the " + SetupGlobals.Keybreeze4Path + " path to the " + SetupGlobals.Keybreeze5Path + ". " + e2.Message);
                    SetupGlobals.ExitApplication(false);
                }
            }

            label1.Text = "Welcome to Keybreeze Desktop! It will stay hidden when you are not using it. When you want to perform any task on your computer, press a hotkey to display Keybreeze Desktop, type a few letters of a command, and press Enter to launch it." + Environment.NewLine + Environment.NewLine + "We recommend using the semicolon as your hotkey because this lets you navigate around your computer in the fastest possible way. Just tap the semicolon key with your pinky finger and start typing a command. Your fingers will stay on the home row of the keyboard, so you can launch anything in a couple of seconds. If you need to type a semicolon in a text field, you can tap the key twice or press Semicolon + Space or Semicolon + Enter to simulate the respective keystrokes." + Environment.NewLine + Environment.NewLine + "If you want to use a different hotkey, you can select it below. Other popular hotkeys include Shift + Space and Alt + Space.";
            chkModifiers.Items.Add("None");
            chkModifiers.Items.Add("Shift +");
            chkModifiers.Items.Add("Ctrl +");
            chkModifiers.Items.Add("Alt +");
            chkModifiers.Items.Add("Windows key +");
            chkModifiers.Items.Add("Ctrl + Alt +");
            chkModifiers.Items.Add("Shift + Ctrl +");
            chkModifiers.Items.Add("Shift + Alt +");
            chkModifiers.SelectedIndex = 0;
        }

        private void txtHotkey_KeyDown(object sender, KeyEventArgs e)
        {
            // e.KeyCode is the letter/number/key.
            // e.KeyValue is the ascii code number.
            int NewCode = e.KeyValue;
            if (NewCode > 95 && NewCode < 112)
            {
                txtHotkey.Text = "";
                return;
            }
            switch (NewCode)
            {
                case 8:
                    txtHotkey.Text = "Backspace";
                    break;
                case 9:
                    txtHotkey.Text = "";
                    break;
                case 13:
                    txtHotkey.Text = "Enter";
                    break;
                case 32:
                    txtHotkey.Text = "Space";
                    break;
                case 112:
                    txtHotkey.Text = "F1";
                    break;
                case 113:
                    txtHotkey.Text = "F2";
                    break;
                case 114:
                    txtHotkey.Text = "F3";
                    break;
                case 115:
                    txtHotkey.Text = "F4";
                    break;
                case 116:
                    txtHotkey.Text = "F5";
                    break;
                case 117:
                    txtHotkey.Text = "F6";
                    break;
                case 118:
                    txtHotkey.Text = "F7";
                    break;
                case 119:
                    txtHotkey.Text = "F8";
                    break;
                case 120:
                    txtHotkey.Text = "F9";
                    break;
                case 121:
                    txtHotkey.Text = "F10";
                    break;
                case 122:
                    txtHotkey.Text = "F11";
                    break;
                case 123:
                    txtHotkey.Text = "F12";
                    break;
                case 124:
                    txtHotkey.Text = "F13";
                    break;
                case 125:
                    txtHotkey.Text = "F14";
                    break;
                case 126:
                    txtHotkey.Text = "F15";
                    break;
                case 127:
                    txtHotkey.Text = "F16";
                    break;
                case 128:
                    txtHotkey.Text = "F17";
                    break;
                case 129:
                    txtHotkey.Text = "F18";
                    break;
                case 130:
                    txtHotkey.Text = "F19";
                    break;
                case 131:
                    txtHotkey.Text = "F20";
                    break;
                case 144:
                    txtHotkey.Text = "Num Lock";
                    break;
                case 145:
                    txtHotkey.Text = "Scroll Lock";
                    break;
                case 45:
                    txtHotkey.Text = "Insert";
                    break;
                case 36:
                    txtHotkey.Text = "Home";
                    break;
                case 33:
                    txtHotkey.Text = "Page Up";
                    break;
                case 46:
                    txtHotkey.Text = "Delete";
                    break;
                case 35:
                    txtHotkey.Text = "End";
                    break;
                case 34:
                    txtHotkey.Text = "Page Down";
                    break;
                case 19:
                    txtHotkey.Text = "Pause";
                    break;
                case 186:
                    // semicolon
                    txtHotkey.Text = ";";
                    break;
                default:
                    txtHotkey.Text = e.KeyCode.ToString();
                    break;
            }
            txtHotkey.Tag = e.KeyValue.ToString();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private bool CheckForOldVersion()
        {
            // was the user using a version of Keybreeze prior to version 4, and are they installing this for the first time for this Windows account?
            string keywords_file = SetupGlobals.Keybreeze5Path + "\\keywords.ini";
            if (Directory.Exists(SetupGlobals.Keybreeze3Path) == true && File.Exists(keywords_file) == false)
            {
                frmSetupPriorVersion frmSetupPriorVersion_instance;
                frmSetupPriorVersion_instance = new frmSetupPriorVersion();
                frmSetupPriorVersion_instance.Show();
                return true;
            }
            else
            {
                SetupGlobals.ExitApplication(true);
                return false;
            }
        }

        private void SaveHotkeyToINIFile()
        {
            if (chkModifiers.SelectedIndex == 0)
            {
                SetupGlobals.WriteINI("quicktab", "hotkey_plus_enter", "3", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("quicktab", "hotkey_plus_space", "2", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("quicktab", "hotkey_plus_hotkey", "1", SetupGlobals.Keybreeze5Path + "\\options.ini");
            }
            else
            {
                SetupGlobals.WriteINI("quicktab", "hotkey_plus_enter", "0", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("quicktab", "hotkey_plus_space", "0", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("quicktab", "hotkey_plus_hotkey", "0", SetupGlobals.Keybreeze5Path + "\\options.ini");
            }
            if (chkModifiers.SelectedIndex == 0)
            {
                SetupGlobals.WriteINI("hotkey", "hotkey_ctrl", "0", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("hotkey", "hotkey_alt", "0", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("hotkey", "hotkey_shift", "0", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("hotkey", "hotkey_windows", "0", SetupGlobals.Keybreeze5Path + "\\options.ini");
            }
            else if (chkModifiers.SelectedIndex == 1)
            {
                SetupGlobals.WriteINI("hotkey", "hotkey_ctrl", "0", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("hotkey", "hotkey_alt", "0", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("hotkey", "hotkey_shift", "1", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("hotkey", "hotkey_windows", "0", SetupGlobals.Keybreeze5Path + "\\options.ini");
            }
            else if (chkModifiers.SelectedIndex == 1)
            {
                SetupGlobals.WriteINI("hotkey", "hotkey_ctrl", "0", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("hotkey", "hotkey_alt", "0", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("hotkey", "hotkey_shift", "1", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("hotkey", "hotkey_windows", "0", SetupGlobals.Keybreeze5Path + "\\options.ini");
            }
            else if (chkModifiers.SelectedIndex == 2)
            {
                SetupGlobals.WriteINI("hotkey", "hotkey_ctrl", "1", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("hotkey", "hotkey_alt", "0", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("hotkey", "hotkey_shift", "0", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("hotkey", "hotkey_windows", "0", SetupGlobals.Keybreeze5Path + "\\options.ini");
            }
            else if (chkModifiers.SelectedIndex == 3)
            {
                SetupGlobals.WriteINI("hotkey", "hotkey_ctrl", "0", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("hotkey", "hotkey_alt", "1", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("hotkey", "hotkey_shift", "0", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("hotkey", "hotkey_windows", "0", SetupGlobals.Keybreeze5Path + "\\options.ini");
            }
            else if (chkModifiers.SelectedIndex == 4)
            {
                SetupGlobals.WriteINI("hotkey", "hotkey_ctrl", "0", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("hotkey", "hotkey_alt", "0", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("hotkey", "hotkey_shift", "0", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("hotkey", "hotkey_windows", "1", SetupGlobals.Keybreeze5Path + "\\options.ini");
            }
            else if (chkModifiers.SelectedIndex == 5)
            {
                SetupGlobals.WriteINI("hotkey", "hotkey_ctrl", "1", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("hotkey", "hotkey_alt", "1", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("hotkey", "hotkey_shift", "0", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("hotkey", "hotkey_windows", "0", SetupGlobals.Keybreeze5Path + "\\options.ini");
            }
            else if (chkModifiers.SelectedIndex == 6)
            {
                SetupGlobals.WriteINI("hotkey", "hotkey_ctrl", "1", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("hotkey", "hotkey_alt", "0", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("hotkey", "hotkey_shift", "1", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("hotkey", "hotkey_windows", "0", SetupGlobals.Keybreeze5Path + "\\options.ini");
            }
            else if (chkModifiers.SelectedIndex == 7)
            {
                SetupGlobals.WriteINI("hotkey", "hotkey_ctrl", "0", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("hotkey", "hotkey_alt", "1", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("hotkey", "hotkey_shift", "1", SetupGlobals.Keybreeze5Path + "\\options.ini");
                SetupGlobals.WriteINI("hotkey", "hotkey_windows", "0", SetupGlobals.Keybreeze5Path + "\\options.ini");
            }
            string hotkey_char = txtHotkey.Text;
            string hotkey_ascii_code = txtHotkey.Tag.ToString();
            SetupGlobals.WriteINI("hotkey", "hotkey", hotkey_char, SetupGlobals.Keybreeze5Path + "\\options.ini");
            SetupGlobals.WriteINI("hotkey", "hotkey_ascii_code", hotkey_ascii_code, SetupGlobals.Keybreeze5Path + "\\options.ini");
        }

        private void frmSetupHotkey_FormClosing(object sender, FormClosingEventArgs e)
        {
            // copy the options.ini file from the %installdir% to the workpath directory so we know this setup utility has already ran.
            // then write the hotkey setting.
            if (SetupGlobals.closing_program == false)
            {
                this.Visible = false;
                try
                {
                    if (Directory.Exists(SetupGlobals.Keybreeze5Path) == false)
                    {
                        Directory.CreateDirectory(SetupGlobals.Keybreeze5Path);
                        System.Threading.Thread.Sleep(250);
                    }
                    if (File.Exists(SetupGlobals.Keybreeze5Path + "\\options.ini") == false)
                        File.Copy(SetupGlobals.InstallDir + "\\default settings\\options.ini", SetupGlobals.Keybreeze5Path + "\\options.ini", false);
                }
                catch
                {
                    MessageBox.Show("The options.ini file could not be copied from " + SetupGlobals.InstallDir + "\\default settings" + " to " + SetupGlobals.Keybreeze5Path + ". Please make sure the file exists and your Windows permissions are correct. Then try reinstalling Keybreeze Desktop.", "Keybreeze Desktop Setup");
                    SetupGlobals.ExitApplication(false);
                }
                System.Threading.Thread.Sleep(500);
                SaveHotkeyToINIFile();

                if (SetupGlobals.closing_program == false)
                {
                    bool version_exists = CheckForOldVersion();
                    if (version_exists == true)
                    {
                        e.Cancel = true;
                        return;
                    }
                    else
                    {
                        SetupGlobals.ExitApplication(false);
                    }
                }
            }
        }
    }
}
