﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace Keybreeze_install
{
    public partial class frmSetupPriorVersion : Form
    {
        private bool import_settings = true;
        public frmSetupPriorVersion()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            label1.Text = "It looks like you have a prior version of Keybreeze is installed. Keybreeze 4 can import your keywords, text functions, and letter commands into this version, or you can start with fresh settings. Here are a few things you should know about the new version:" + Environment.NewLine + Environment.NewLine + "- Existing notes, reminders, skins, and macros cannot be imported into this new version. However, all of your keywords, text functions, and letter commands can be imported." + Environment.NewLine + "- Your old Keybreeze settings will be saved in the " + SetupGlobals.Keybreeze3Path + " directory. New settings will be stored in " + SetupGlobals.Keybreeze5Path + "." + Environment.NewLine + "- Keybreeze 4 was rewritten from scratch in C#. Some things may look a little different, but all the major features are still there." + Environment.NewLine + "- There are now two keyword lists -- your custom list of keywords that you're used to, and a new list that Keybreeze generates in the background called autoindex. Autoindex will include commonly used files on your Desktop, Start Menu, and My Documents folder, and Keybreeze will automatically refresh this list. You can adjust these settings by typing 'options' as a keyword.";
        }

        private void btnDoNotImport_Click(object sender, EventArgs e)
        {
            import_settings = false;
            this.Close();
        }

        private void frmSetupPriorVersion_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (import_settings == true)
            {
                // import settings from the old work path to the new one.
                // start with the letter commands file.
                if (File.Exists(SetupGlobals.Keybreeze3Path + "\\lettercommands.ini") == false || File.Exists(SetupGlobals.Keybreeze3Path + "\\keywords.ini") == false)
                {
                    MessageBox.Show("The settings for your old Keybreeze version could not be found. Expected to find " + SetupGlobals.Keybreeze3Path + "\\lettercommands.ini and " + SetupGlobals.Keybreeze3Path + "\\keywords.ini, but these files do not exist. You might be using a version of Keybreeze that is too old to work with this one. It's recommended that you not import any settings. You can manually import your keywords.ini file by copying it from the old location to " + SetupGlobals.Keybreeze5Path + ".", "Keybreeze Setup");
                    e.Cancel = true;
                    return;
                }
                try
                {
                    System.IO.File.Copy(SetupGlobals.Keybreeze3Path + "\\lettercommands.ini", SetupGlobals.Keybreeze5Path + "\\lettercommands.ini", false);
                    System.IO.File.Copy(SetupGlobals.Keybreeze3Path + "\\keywords.ini", SetupGlobals.Keybreeze5Path + "\\keywords.ini", false);
                    SetupGlobals.WriteINI("letters", "s-replacements", "# >> %23,.@ >> %40,.& >> %26,.? >> %3F,.+ >> %2B,.", SetupGlobals.Keybreeze5Path + "\\lettercommands.ini");
                }
                catch
                {
                    MessageBox.Show("Some files could not be copied from the " + SetupGlobals.Keybreeze3Path + " directory to the " + SetupGlobals.Keybreeze5Path + " directory. Please check your Windows permissions and try installing again.", "Keybreeze Setup");
                    SetupGlobals.ExitApplication(false);
                    return;
                }
                // import text functions
                string old_textfunction_directory = SetupGlobals.Keybreeze3Path + "\\scripts";
                string new_textfunction_directory = SetupGlobals.Keybreeze5Path + "\\text functions";
                if (Directory.Exists(new_textfunction_directory) == false)
                {
                    Directory.CreateDirectory(new_textfunction_directory);
                    System.Threading.Thread.Sleep(200);
                }
                try
                {
                    string filename_only;
                    string extension;
                    string note_check;
                    foreach (var file in Directory.GetFiles(old_textfunction_directory))
                    {
                        filename_only = Path.GetFileName(file);
                        extension = RightStr(filename_only, 4);
                        if (extension == ".txt")
                        {
                            note_check = LeftStr(filename_only, 4);
                            if (note_check != "note")
                            {
                                // the file has a .txt extension, and it doesn't start with note, so it's most likely a text function file. Copy it to the new directory.
                                File.Copy(file, Path.Combine(new_textfunction_directory, filename_only), false);
                            }
                        }
                    }
                }
                catch
                {
                    MessageBox.Show("The text function files could not be copied from " + old_textfunction_directory + " to " + new_textfunction_directory + ". You will need to manually copy these files over for your text functions to work properly.", "Keybreeze Setup");
                }
            }
            SetupGlobals.ExitApplication(true);
        }

        private void btnImportSettings_Click(object sender, EventArgs e)
        {
            import_settings = true;
            this.Close();
        }

        private string RightStr(string txt, int length)
        {
            // returns the right portion of a string.
            if (txt.Length < length)
                return txt;
            else
            {
                string modified_txt;
                modified_txt = txt.Substring(txt.Length - length);
                return modified_txt;
            }
        }

        private string LeftStr(string txt, int length)
        {
            // returns the left portion of a string.
            if (txt.Length < length)
                return txt;
            else
            {
                string modified_txt;
                modified_txt = txt.Substring(0, length);
                return modified_txt;
            }
        }
    }
}
