﻿/*
Keybreeze Desktop
Copyright © 2021 Keybreeze LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.If not, see<http://www.gnu.org/licenses/>.
*/

using System;
using System.IO;
using System.Collections.Generic;
using System.Diagnostics;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Threading;
using System.Net.NetworkInformation;
using Microsoft.Win32;

namespace Keybreeze
{
    public partial class frmMain : Form
    {
        [DllImport("user32.dll")]
        public static extern bool RegisterHotKey(IntPtr hWnd, int id, int fsModifiers, int vlc);
        [DllImport("user32.dll")]
        public static extern bool UnregisterHotKey(IntPtr hWnd, int id);
        [DllImport("user32.dll", CharSet = CharSet.Auto, ExactSpelling = true)]
        public static extern IntPtr GetForegroundWindow();
        [DllImport("user32.dll", CharSet = CharSet.Auto, ExactSpelling = true)]
        public static extern IntPtr GetActiveWindow();
        [DllImport("user32.dll", CharSet = CharSet.Auto)]
        public static extern int GetClassName(IntPtr hWnd, StringBuilder lpClassName, int nMaxCount);
        [DllImport("user32", CharSet = CharSet.Auto, SetLastError = true)]
        internal static extern int GetWindowText(IntPtr hWnd, [Out, MarshalAs(UnmanagedType.LPTStr)] StringBuilder lpString, int nMaxCount);
        [DllImport("kernel32")]
        private static extern long WritePrivateProfileString(string section, string key, string val, string filePath);
        [DllImport("kernel32")]
        private static extern int GetPrivateProfileString(string section, string key, string def, StringBuilder retVal, int size, string filePath);
        
        // keybd_event is used for simulating keystrokes for text functions.
        [DllImport("user32.dll")]
        // update by Andrew on 1/26/2020. Made this public so that it can be referenced in the frmKeyboardClipboard class.
        public static extern void keybd_event(byte bVk, byte bScan, uint dwFlags, UIntPtr dwExtraInfo);
        const int keybd_keyup = 0x2;
        const int keybd_keydown = 0x0;

        [DllImport("user32.dll", SetLastError = true, CharSet = CharSet.Auto)]
        static extern uint RegisterWindowMessage(string lpString);
        [DllImport("user32", CharSet = CharSet.Ansi, SetLastError = true, ExactSpelling = true)]
        public static extern bool SetForegroundWindow(IntPtr hwnd);
        [DllImport("user32.dll")]
        public static extern IntPtr FindWindow(String sClassName, String sAppName);
        [DllImportAttribute("User32.DLL")]
        public static extern bool ShowWindow(IntPtr hWnd, int nCmdShow);
        [DllImport("user32.dll")]
        static extern bool IsZoomed(IntPtr hWnd);
        [DllImport("user32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        static extern bool IsIconic(IntPtr hWnd);
        [DllImport("user32.dll", CharSet = CharSet.Auto, ExactSpelling = true)]
        public static extern IntPtr GetDesktopWindow();
        [DllImport("user32.dll")]
        public static extern int SendMessage(IntPtr hWnd, int wMsg, int wParam, int lParam);

        // moving windows on screen.
        const int SWP_NOACTIVATE = 0x0010;
        const int SWP_NOMOVE = 0x0002;
        const int SWP_NOSIZE = 0x0001;

        const int WM_SYSCOMMAND = 0x112;
        const int SC_SCREENSAVE = 0xF140;
        const int SC_CLOSE = 0xF060;

        const int EWX_LOGOFF = 0;
        const int EWX_REBOOT = 2;
        const int EWX_SHUTDOWN = 1;
        const int EWX_FORCE = 4;

        private Icon icon_file = new Icon(Globals.InstallDir + @"\images\systray.ico");

        private bool systray_menu_opening = false;
        private int MOD_ALT = 0x1;
        private int MOD_CONTROL = 0x2;
        private int MOD_SHIFT = 0x4;
        private int MOD_WIN = 0x8;
        private uint msg;
        private uint msg2;
        private uint msg3;
        frmSkin Skin;
        frmPopup Popup;
        frmCommandList frmCommandList_instance;

        public string PreviousKeywordMatch = "";
        public string PreviousEntry = "";
        public string history_selection = "";
        public int PreviousTempCount = 0;
        public bool Done_Searching_Autoindex = true;
        public bool macro_running = false;
        private int display_notes_on_hotkey_press;
        public int always_center_keybreeze;

        private byte skin_opacity = 255;

        public string custom_explorer_path = "";
        public string custom_browser_path = "";
        public string custom_browser_prefix = "";
        public string custom_browser_suffix = "";

        public int popup_result_delay_with_no_matches;
        public int popup_result_delay_with_matches;
        public int popup_result_delay_with_folders_on_demand;
        private bool popup_showing = false;
        private bool just_created_a_note = false;
        //public bool show_paths = false;
        // initial_key_pressed will tell us if the user pressed a key after pressing the hotkey. This fixes the problem with modifier hotkeys interfering with quicktab.
        public bool initial_key_pressed = false;

        // we need this hack to force the form to be invisible at startup.
        private bool form_just_loaded = true;

        // allowshowdisplay and SetVisibleCore are required to hide Keybreeze Desktop upon startup.
        private bool allowshowdisplay = false;

        // used to control dragging the form when there is no external skin.
        int m_iWindowPosX;
        int m_iWindowPosY;

        // This is used to cancel the autoindex searches.
        private bool user_pressed_new_key = false;

        // This is used to cross-check the matches between the autoindex and the autoindex on demand.
        private List<string> autoindex_matches = new List<string>();

        // this is used to track when the user presses Shift without pressing anything else.
        private bool user_pressed_shift = false;
        private bool user_pressed_ctrl = false;

        // Popup Helper is used to tell the user to press Shift or Tab to search the web.
        public frmPopupHelper frmPopupHelper_instance;

        // Keybreeze clipboard monitor
        public List<Keybreeze_Clipboard> keybreeze_clipboard = new List<Keybreeze_Clipboard>();
        public int keybreeze_clipboard_max_count = 50;
        public int keybreeze_clipboard_item_max_length = 100000;
        public bool keybreeze_clipboard_clear_history = false;
        public bool running_text_function = false;
        public int clipboard_password_detection = 2;
        public int clipboard_password_detection_start = 4;
        public int clipboard_password_detection_end = 30;

        protected override void SetVisibleCore(bool value)
        {
            base.SetVisibleCore(allowshowdisplay ? value : allowshowdisplay);
        }

        public frmMain()
        {
            // did the user extract or install Keybreeze Desktop?
            string extracted = Globals.ReadINI("Extract", "Extract_Not_Install", Globals.InstallDir + "\\default settings\\options.ini");
            if (extracted == "1")
                Globals.WorkPath = Globals.InstallDir;
            else
                Globals.WorkPath = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + "\\Keybreeze5";
            // turn on logging if enabled. if we fail before this point, it is most likely a problem with the .NET installation.
            Globals.logging = Globals.ReadIntFromINI("options", "logging", Globals.WorkPath + "\\options.ini", 0);
            if (Globals.logging == 1)
                Globals.DebugWrite("Logging set to " + Globals.logging.ToString());
            // make sure they ran the keybreeze_install.exe file first.
            if (File.Exists(Globals.WorkPath + "\\options.ini") == false)
            {
                if (Globals.logging == 1)
                    Globals.DebugWrite("Options.ini does not exist. Running keybreeze_install.exe");
                System.Diagnostics.Process.Start(Globals.InstallDir + "\\keybreeze_install.exe");
                ExitApplication(true);
                return;
            }
            if (File.Exists(Globals.InstallDir + "\\Newtonsoft.Json.dll") == false)
            {
                MessageBox.Show("A required file, " + Globals.InstallDir + "\\Newtonsoft.Json.dll, is missing. Please reinstall the program to correct the problem.", "Keybreeze Desktop");
                ExitApplication(true);
                return;
            }
            // check for an update.
            try
            {
                System.Diagnostics.Process.Start(Globals.InstallDir + "\\updatecheck.exe");
            }
            catch { }
            Globals.KeywordsJSONFile = Globals.WorkPath + "\\keywords.json";
            Globals.FoldersJSONFile = Globals.WorkPath + "\\folders.json";
            Globals.LetterCommandsJSONFile = Globals.WorkPath + "\\lettercommands.json";
            Globals.OptionsINI = Globals.WorkPath + "\\options.ini";
            Globals.NotesINI = Globals.WorkPath + "\\notes\\settings\\notes.ini";

            Globals.when_pressing_shift = Globals.ReadIntFromINI("options", "when_pressing_shift", Globals.OptionsINI, 3);
            Globals.when_pressing_tab = Globals.ReadIntFromINI("options", "when_pressing_tab", Globals.OptionsINI, 1);
            Globals.when_pressing_ctrl = Globals.ReadIntFromINI("options", "when_pressing_ctrl", Globals.OptionsINI, 2);
            
            if (Globals.ReadIntFromINI("options", "windows_explorer", Globals.OptionsINI, 0) == 0)
                custom_explorer_path = Globals.ReadINI("options", "custom_explorer_path", Globals.OptionsINI);

            if (Globals.ReadIntFromINI("options", "default_browser", Globals.OptionsINI, 0) == 0)
            {
                custom_browser_path = Globals.ReadINI("options", "custom_browser_path", Globals.OptionsINI);
                if (custom_browser_path.Length > 0)
                {
                    custom_browser_prefix = Globals.ReadINI("options", "custom_browser_prefix", Globals.OptionsINI);
                    custom_browser_suffix = Globals.ReadINI("options", "custom_browser_suffix", Globals.OptionsINI);
                }
            }

            CheckRequiredFiles();
            Globals.CreateKeywordsList();
            Globals.CreateLetterCommandsList();
            CheckForINIChanges();
            Globals.Count_History_Items();
            Globals.Update_AutoComplete_List();
            Globals.notes_load_existing(this);
            GetLocalDrives();
            always_center_keybreeze = Globals.ReadIntFromINI("options", "always_center_keybreeze", Globals.OptionsINI, 1);
            display_notes_on_hotkey_press = Globals.ReadIntFromINI("notes", "display_on_hotkey_press", Globals.NotesINI, 1);
            if (Globals.logging == 1)
                Globals.DebugWrite("InitializeComponent()");
            InitializeComponent();
            if (Globals.valid_skin == false)
            {
                // no skin is configured. The user should have already been warned. Just show the default form.
                if (Globals.logging == 1)
                    Globals.DebugWrite("No skin is configured. Showing blank form.");
                this.BackColor = System.Drawing.Color.Black;
                this.txtMain.BackColor = Color.FromArgb(25, 25, 25);
                this.txtMain.Top = this.txtMain.Top + 9;
                this.txtMain.Left = this.txtMain.Left + 4;
                Globals.description_character_limit = 40;
            }
            else
            {
                // load the skin.
                Globals.SkinFolder = Globals.WorkPath + "\\skins\\" + Globals.ReadINI("options", "skin", Globals.OptionsINI);
                Globals.SkinINI = Globals.WorkPath + "\\skins\\" + Globals.ReadINI("options", "skin", Globals.OptionsINI) + "\\settings.ini";
                Globals.skin_icon_type = Globals.ReadIntFromINI("autocomplete", "skin_icon_type", Globals.SkinINI, 1);
                Globals.skin_embedded = Globals.ReadIntFromINI("skin", "embedded", Globals.SkinINI, 0);
                Globals.skin_top_offset = Globals.ReadIntFromINI("skin", "top_offset", Globals.SkinINI, 0);
                Globals.skin_left_offset = Globals.ReadIntFromINI("skin", "left_offset", Globals.SkinINI, 0);
                Globals.top_offset = Globals.ReadIntFromINI("autocomplete", "top_offset", Globals.SkinINI, 0);
                Globals.left_offset = Globals.ReadIntFromINI("autocomplete", "left_offset", Globals.SkinINI, 0);
                Globals.center_top_offset = Globals.ReadIntFromINI("skin", "center_top_offset", Globals.SkinINI, 0);
                Globals.center_left_offset = Globals.ReadIntFromINI("skin", "center_left_offset", Globals.SkinINI, 0);

                this.Width = Globals.ReadIntFromINI("skin", "main_width", Globals.SkinINI, 424);
                txtMain.Width = this.Width - (Globals.ReadIntFromINI("skin", "main_textbox_width_offset", Globals.SkinINI, 4));

                string font_face = Globals.ReadINI("skin", "main_textbox_font", Globals.SkinINI);
                double font_size;
                try
                {
                    font_size = Convert.ToDouble(Globals.ReadINI("skin", "main_textbox_size", Globals.SkinINI));
                }
                catch
                {
                    font_size = 12;
                }
                if (font_size < 6) font_size = 12;
                FontStyle myfontstyle = FontStyle.Regular;
                if (Globals.ReadIntFromINI("skin", "main_textbox_bold", Globals.SkinINI, 0) == 1)
                    myfontstyle = myfontstyle | FontStyle.Bold;
                if (Globals.ReadIntFromINI("skin", "main_textbox_italic", Globals.SkinINI, 0) == 1)
                    myfontstyle = myfontstyle | FontStyle.Italic;
                float font_size_float = (float)font_size;
                if (font_face.Length > 0)
                {
                    txtMain.Font = new System.Drawing.Font(font_face, font_size_float, myfontstyle, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
                }
                else
                {
                    txtMain.Font = new System.Drawing.Font("Microsoft Sans Serif", font_size_float, myfontstyle, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
                }

                if (Globals.skin_embedded == 0)
                {
                    this.TransparencyKey = Color.FromArgb(64, 64, 64);
                    Skin = new frmSkin(this);
                }
                else
                {
                    // the skin is part of the main form.
                    if (Globals.logging == 1)
                        Globals.DebugWrite("Skin is part of the main form.");
                    txtMain.Top = txtMain.Top + 9;
                    txtMain.Left = txtMain.Left + 4;
                    try
                    {
                        this.BackColor = System.Drawing.ColorTranslator.FromHtml(Globals.ReadINI("skin", "main_textbox_backcolor", Globals.SkinINI));
                        txtMain.BackColor = System.Drawing.ColorTranslator.FromHtml(Globals.ReadINI("skin", "main_textbox_backcolor", Globals.SkinINI));
                        txtMain.Left = Globals.ReadIntFromINI("skin", "main_textbox_left", Globals.SkinINI, 6);
                        txtMain.Top = Globals.ReadIntFromINI("skin", "main_textbox_top", Globals.SkinINI, 4);
                        pictureBox1.Visible = true;
                        pictureBox1.Image = Image.FromFile(Globals.SkinFolder + "\\" + Globals.ReadINI("skin", "file", Globals.SkinINI));
                        pictureBox1.Width = Globals.ReadIntFromINI("skin", "main_width", Globals.SkinINI, this.Width);
                        pictureBox1.Height = Globals.ReadIntFromINI("skin", "main_height", Globals.SkinINI, this.Height);
                        this.Width = pictureBox1.Width;
                        this.Height = pictureBox1.Height;
                        txtMain.Width = this.Width - Globals.ReadIntFromINI("skin", "main_textbox_width_offset", Globals.SkinINI, 8);
                        Globals.description_character_limit = 40;
                    }
                    catch
                    {
                        if (Globals.logging == 1)
                            Globals.DebugWrite("Exception caught while showing embedded skin. Handling exception to avoid crash...");
                        Globals.valid_skin = false;
                        this.BackColor = System.Drawing.Color.Black;
                        this.txtMain.BackColor = Color.FromArgb(25, 25, 25);
                        this.txtMain.Top = this.txtMain.Top + 9;
                        this.txtMain.Left = this.txtMain.Left + 4;
                        Globals.description_character_limit = 40;
                    }
                    
                }
            }
            Popup = new frmPopup(this);
            if (Globals.logging == 1)
                Globals.DebugWrite("Registering Windows messages...");
            msg = RegisterWindowMessage("add_keyword_to_keybreeze");
            msg2 = RegisterWindowMessage("bring_keybreeze_to_front");
            msg3 = RegisterWindowMessage("launch_keybreeze_entry");

            // initialize the autoindex.
            int update_timer = Globals.ReadIntFromINI("settings", "refresh_interval_minutes", Globals.WorkPath + "\\autoindex\\indexoptions.ini", 10);
            if (update_timer > 0)
            {
                Globals.autoindex_update();
                if (update_timer > 60)
                    update_timer = 60;
                update_timer = update_timer * 60 * 1000;
                tmrAutoIndexRefresh.Enabled = true;
                tmrAutoIndexRefresh.Interval = update_timer;
            }
            if (Globals.logging == 1)
                Globals.DebugWrite("frmMain() -- adding EventHandler(OnDeactivate).");
            this.Deactivate += new EventHandler(OnDeactivate);
            try
            {
                Globals.description_character_limit = Globals.ReadIntFromINI("autocomplete", "description_character_limit", Globals.SkinINI);
            }
            catch
            {
            }
            if (Globals.description_character_limit < 20)
                Globals.description_character_limit = 20;

            systrayicon.Icon = icon_file;

            if (Globals.valid_skin == true && Globals.skin_embedded == 0) Skin.Hide();
            SetHotKey();
            if (Globals.command_line_parameters.Length > 0)
            {
                // the user probably right-clicked a file and selected 'Add keyword to Keybreeze Desktop', or it may be a launch instruction from a macro. Let's check.
                if (Globals.command_line_parameters.IndexOf("launch_keybreeze_entry:") == 0)
                {
                    // it's a launch instruction.
                    if (Globals.logging == 1)
                        Globals.DebugWrite("frmMain() -- command line parameters detected. Launching entry: Globals.command_line_parameters.Substring(23). Full entry: " + Globals.command_line_parameters);
                    LaunchEntry(Globals.command_line_parameters.Substring(23));
                }
                else
                {
                    string file_path = Globals.command_line_parameters.Trim();
                    string file_name = Path.GetFileNameWithoutExtension(file_path);
                    if (file_path.ToLower().IndexOf(".lnk") > -1)
                    {
                        file_path = Globals.ResolveShortcut(file_path);
                    }
                    if (Globals.logging == 1)
                        Globals.DebugWrite("frmMain() -- command line parameters detected. ShowAddKeywordForm().");
                    ShowAddKeywordForm(false, file_name, file_path);
                }
            }

            int opacity_percent = Globals.ReadIntFromINI("skin", "opacity", Globals.SkinINI, 0);
            if (opacity_percent > 0 && opacity_percent < 100)
            {
                double double_number = 255 * (opacity_percent * 0.01);
                int int_number = Convert.ToInt32(double_number);
                skin_opacity = (byte)int_number;
                if (Globals.logging == 1)
                    Globals.DebugWrite("frmMain() -- setting opacity to " + opacity_percent.ToString());
                if (Globals.skin_embedded == 1)
                    Globals.MakeWindowTranslucent(this.Handle, opacity_percent, false);
            }
            popup_result_delay_with_no_matches = Globals.ReadIntFromINI("settings", "popup_result_delay_with_no_matches", Globals.WorkPath + "\\autoindex\\indexoptions.ini", 125);
            if (popup_result_delay_with_no_matches < 100)
                popup_result_delay_with_no_matches = 100;
            popup_result_delay_with_matches = Globals.ReadIntFromINI("settings", "popup_result_delay_with_matches", Globals.WorkPath + "\\autoindex\\indexoptions.ini", 500);
            if (popup_result_delay_with_matches < 100)
                popup_result_delay_with_matches = 100;
            popup_result_delay_with_folders_on_demand = Globals.ReadIntFromINI("settings", "popup_result_delay_with_folders_on_demand", Globals.WorkPath + "\\autoindex\\indexoptions.ini", 250);
            if (popup_result_delay_with_folders_on_demand < 100)
                popup_result_delay_with_folders_on_demand = 100;
            tmrShowAutoIndexResults.Interval = popup_result_delay_with_no_matches;
            btnOK.Top = txtMain.Top;
            btnOK.Left = this.Width;
            btnCancel.Top = txtMain.Top;
            btnCancel.Width = this.Width + 20;

            // create sample.html file for autocomplete.
            string html_file = Globals.WorkPath + "\\sample.html";
            try
            {
                if (File.Exists(html_file) == false)
                {
                    File.WriteAllText(html_file, "");
                }
            }
            catch (Exception e3)
            {
                MessageBox.Show("Could not create " + html_file + ". " + e3.Message);
            }

            // load clipboard settings
            keybreeze_clipboard_item_max_length = Globals.ReadIntFromINI("clipboard", "clipboard_item_max_length", Globals.WorkPath + "\\options.ini", 100000);
            keybreeze_clipboard_max_count = Globals.ReadIntFromINI("clipboard", "clipboard_max_count", Globals.WorkPath + "\\options.ini", 50);
            clipboard_password_detection = Globals.ReadIntFromINI("clipboard", "clipboard_password_detection", Globals.OptionsINI, 2);
            clipboard_password_detection_start = Globals.ReadIntFromINI("clipboard", "clipboard_password_detection_start", Globals.OptionsINI, 4);
            clipboard_password_detection_end = Globals.ReadIntFromINI("clipboard", "clipboard_password_detection_end", Globals.OptionsINI, 30);
            if (keybreeze_clipboard_item_max_length <= 0)
                keybreeze_clipboard_item_max_length = 100000;
            if (keybreeze_clipboard_max_count <= 0 || keybreeze_clipboard_max_count > 250)
                keybreeze_clipboard_max_count = 50;
            if (Globals.ReadIntFromINI("clipboard", "enabled", Globals.WorkPath + "\\options.ini", 1) == 0)
            {
                // disable the keybreeze clipboard
                tmrKeybreezeClipboard.Enabled = false;
            }
            else
            {
                // enable the clipboard
                int clipboard_interval = Globals.ReadIntFromINI("clipboard", "interval", Globals.WorkPath + "\\options.ini", 750);
                if (clipboard_interval <= 500 || clipboard_interval > 30000)
                {
                    clipboard_interval = 750;
                }
                tmrKeybreezeClipboard.Interval = clipboard_interval;
                tmrKeybreezeClipboard.Enabled = true;
            }
            if (Globals.ReadIntFromINI("clipboard", "clear_history_when_clearing_clipboard", Globals.WorkPath + "\\options.ini", 1) == 0)
            {
                keybreeze_clipboard_clear_history = false;
            }
            else
            {
                keybreeze_clipboard_clear_history = true;
            }

            // check frmPopupHelper.
            Globals.show_keyboard_help = Globals.ReadIntFromINI("options", "show_keyboard_help", Globals.OptionsINI, 1);
            if (Globals.show_keyboard_help == 1)
            {
                // load frmPopupHelper.
                frmPopupHelper_instance = new frmPopupHelper(this);
                frmPopupHelper_instance.Top = Popup.Top;
                frmPopupHelper_instance.Left = Popup.Left;
                frmPopupHelper_instance.Visible = false;
                string lbl_so_far = "";
                if (Globals.when_pressing_ctrl == 2)
                    lbl_so_far = "Enter or Ctrl to search your computer, ";
                else if (Globals.when_pressing_shift == 2)
                    lbl_so_far = "Enter or Shift to search your computer, ";
                else if (Globals.when_pressing_tab == 2)
                    lbl_so_far = "Enter or Tab to search your computer, ";
                //if (lbl_so_far == "")
                //    lbl_so_far = "Press ";
                if (Globals.when_pressing_ctrl == 3)
                    lbl_so_far += "Ctrl to search the web, ";
                else if (Globals.when_pressing_shift == 3)
                    lbl_so_far += "Shift to search the web, ";
                else if (Globals.when_pressing_tab == 3)
                    lbl_so_far += "Tab to search the web, ";
                //if (lbl_so_far == "")
                //    lbl_so_far = "Press ";
                lbl_so_far += "F1 for help.";
                frmPopupHelper_instance.lblMain.Text = lbl_so_far;
                frmPopupHelper_instance.Width = this.Width;
                if (frmPopupHelper_instance.lblMain.Width > frmPopupHelper_instance.Width)
                {
                    // this fixes 125% font size in Windows.
                    frmPopupHelper_instance.Width = frmPopupHelper_instance.lblMain.Width;
                }
            }

            // set the text box color since this doesn't work above for some reason.
            // the older skins used to have the text color set to FFD57B. We don't want this. Change it to white if it's set to this.
            string main_text_forecolor = Globals.ReadINI("skin", "main_textbox_forecolor", Globals.SkinINI);
            if (main_text_forecolor == "#FFD57B")
                main_text_forecolor = "#ffffff";
            txtMain.ForeColor = System.Drawing.ColorTranslator.FromHtml(main_text_forecolor);
        }

        private void frmMain_Load(object sender, EventArgs e)
        {
            if (Globals.skin_embedded == 0)
            {
                this.StartPosition = 0;
                Globals.main_textbox_left = Globals.ReadIntFromINI("skin", "main_textbox_left", Globals.SkinINI, 24);
                Globals.main_textbox_top = Globals.ReadIntFromINI("skin", "main_textbox_top", Globals.SkinINI, 26);
                this.Location = new Point(Skin.Location.X + Globals.main_textbox_left, Skin.Location.Y + Globals.main_textbox_top);
            }
        }

        public void SetHotKey()
        {
            // this function will set the system-wide hotkey for Keybreeze Desktop.
            // set up any modifiers first, like CTRL, ALT, SHIFT, etc.
            string hotkey_ctrl = Globals.ReadINI("hotkey", "hotkey_ctrl", Globals.OptionsINI);
            string hotkey_alt = Globals.ReadINI("hotkey", "hotkey_alt", Globals.OptionsINI);
            string hotkey_shift = Globals.ReadINI("hotkey", "hotkey_shift", Globals.OptionsINI);
            string hotkey_windows = Globals.ReadINI("hotkey", "hotkey_windows", Globals.OptionsINI);
            int hotkey_ctrl_num = 0;
            int hotkey_alt_num = 0;
            int hotkey_shift_num = 0;
            int hotkey_windows_num = 0;

            if (hotkey_ctrl == "1")
                hotkey_ctrl_num = MOD_CONTROL;

            if (hotkey_alt == "1")
                hotkey_alt_num = MOD_ALT;

            if (hotkey_shift == "1")
                hotkey_shift_num = MOD_SHIFT;

            if (hotkey_windows == "1")
                hotkey_windows_num = MOD_WIN;
            
            // grab the actual hotkey from the options file
            string hotkey = Globals.ReadINI("hotkey", "hotkey", Globals.OptionsINI);
            int hotkey_code = Globals.ReadIntFromINI("hotkey", "hotkey_ascii_code", Globals.OptionsINI);
            if (hotkey == "[space]") hotkey = " ";
            if (hotkey.Length == 0 || hotkey_code == 0)
            {
                Globals.WriteINI("hotkey", "hotkey_ctrl", "0", Globals.OptionsINI);
                Globals.WriteINI("hotkey", "hotkey_alt", "1", Globals.OptionsINI);
                Globals.WriteINI("hotkey", "hotkey_shift", "0", Globals.OptionsINI);
                Globals.WriteINI("hotkey", "hotkey_windows", "0", Globals.OptionsINI);
                Globals.WriteINI("hotkey", "hotkey", "[space]", Globals.OptionsINI);
                Globals.WriteINI("hotkey", "hotkey_ascii_code", "32", Globals.OptionsINI);
                RegisterHotKey(this.Handle, 1, 1, 0x20);
                MessageBox.Show("The Keybreeze Desktop hotkey did not save properly. It has been reset to Alt + Space. You can change it by typing options as a keyword.", "Keybreeze Desktop");
                return;
            }
            else
            {
                // set the hotkey
                RegisterHotKey(this.Handle, 1, hotkey_ctrl_num | hotkey_alt_num | hotkey_shift_num | hotkey_windows_num, hotkey_code);
            }
        }

        private void OnDeactivate(Object sender, EventArgs e)
        {
            // the user clicked off the form.
            // hide it as long as the user didn't click the background form.

            if (this.Visible == false)
                return;
            IntPtr name_of_handle;
            string classname;
            string titlename;
            name_of_handle = GetForegroundWindow();
            classname = GetWindowClassName(name_of_handle);
            titlename = GetWindowTitle(name_of_handle);
            if (classname.IndexOf("WindowsForms") != -1 && titlename.IndexOf("Keybreeze Interface") != -1)
            {
                // the user most likely clicked the Keybreeze skin. We do not want to hide the form.
            }
            else
            {
                // the user clicked somewhere outside the main UI. Hide the form.
                if (this.Visible == true)
                    HideMainForm();
            }
        }

        protected override void WndProc(ref Message m)
        {
            // read all incoming messages.
            if (m.Msg == 0x0312)
            {
                // the user pressed the hotkey.
                if (form_just_loaded == true)
                {
                    this.Visible = false;
                    form_just_loaded = false;
                }
                if (this.Visible == false)
                {
                    if (macro_running == true)
                        macro_running = false;
                    if (display_notes_on_hotkey_press == 1)
                        Globals.notes_bring_notes_to_front(true);
                    Set_Keybreeze_Position();
                    // we need to refresh the PNG to fix a Windows API bug -- hibernating the computer and then resuming will cause the skin to appear invisible until you refresh the bitmap. 
                    // my testing showed that this did not cause a memory or resource leak, but further testing may be necessary. The only other way to prevent the PNG from disappearing was to drag it with the mouse before hibernating the computer.
                    if (Globals.valid_skin == true && Globals.skin_embedded == 0)
                    {
                        Skin.Visible = true;
                        //Skin.TopMost = true;
                        Skin.Activate();
                        if (Skin.bitmap != null)
                            Skin.SetBitmap(Skin.bitmap, skin_opacity);
                        this.StartPosition = 0;
                        this.Location = new Point(Skin.Location.X + Globals.main_textbox_left, Skin.Location.Y + Globals.main_textbox_top);
                    }
                    this.txtMain.Text = "";
                    this.allowshowdisplay = true;
                    this.Visible = true;
                    //this.TopMost = true;
                    this.Activate();
                }
                else
                {
                    // the user pressed the hotkey while the interface was showing. check quicktab.
                    if (txtMain.Text.Length > 0)
                        QuickTab_SetFocusToWindow(txtMain.Text);
                    else
                    {
                        QuickTab_ControlWindow("hotkey_plus_hotkey");
                    }
                }
            }
            else if (m.Msg == msg)
            {
                // the user is trying to add a keyword to Keybreeze through the right-click file method.
                try
                {
                    System.Threading.Thread.Sleep(100);
                    string file_path = File.ReadAllText(Globals.WorkPath + "\\command_line_argument.txt");
                    file_path = file_path.Trim();
                    string file_name = Path.GetFileNameWithoutExtension(file_path);
                    if (file_path.ToLower().IndexOf(".lnk") > -1)
                    {
                        file_path = Globals.ResolveShortcut(file_path);
                    }
                        
                    ShowAddKeywordForm(false, file_name, file_path);
                }
                catch
                {
                    MessageBox.Show("Keybreeze Desktop could not open the file, " + Globals.WorkPath + "\\command_line_argument.txt for reading. Please try again or make sure the file exists.", "Keybreeze Desktop");
                }
            }
            else if (m.Msg == msg2)
            {
                // the user launched Keybreeze a second time. Let's just activate this instance.
                if (Globals.valid_skin == true && Globals.skin_embedded == 0)
                {
                    Skin.Visible = true;
                    //Skin.TopMost = true;
                    Skin.Activate();
                }
                Set_Keybreeze_Position();
                this.Visible = true;
                //this.TopMost = true;
                this.Activate();
            }
            else if (m.Msg == msg3)
            {
                // the user passed a launch instruction to Keybreeze, most likely through a macro.
                try
                {
                    System.Threading.Thread.Sleep(100);
                    string launch_instruction = File.ReadAllText(Globals.WorkPath + "\\command_line_argument.txt");
                    launch_instruction = launch_instruction.Substring(23);
                    launch_instruction = launch_instruction.Trim();
                    LaunchEntry(launch_instruction);
                }
                catch
                {
                    MessageBox.Show("Keybreeze Desktop could not open the file, " + Globals.WorkPath + "\\command_line_argument.txt for reading. Please try again or make sure the file exists.", "Keybreeze Desktop");
                }
            }

            // the following line might be causing the mouse cursor to jump on some machines.
            base.WndProc(ref m);

            if (Globals.valid_skin == false)
            {
                if (m.Msg == 163 && this.ClientRectangle.Contains(this.PointToClient(new Point(m.LParam.ToInt32()))) && m.WParam.ToInt32() == 2)
                    m.WParam = (IntPtr)1;

                base.WndProc(ref m);

                if (m.Msg == 132 && m.Result.ToInt32() == 1)
                    m.Result = (IntPtr)2;
            }
        }

        public int QuickTab_ControlWindow(string KeyCombination)
        {
            // before we run a quicktab command, we need to see if there are any notes that are on top of all other windows. We need to turn this off so the QuickTab command doesn't affect the note.
            List<int> note_ids_on_top = new List<int>();
            foreach (Note n in Globals.notelist)
            {
                if (n.frmNote_instance.TopMost == true)
                {
                    note_ids_on_top.Add(n.frmNote_instance.UniqueID);
                    n.frmNote_instance.TopMost = false;
                    // make it invisible to see if that works.
                }
            }
            int INI_setting = Globals.ReadIntFromINI("quicktab", KeyCombination, Globals.OptionsINI, 0);
            int wait_time = Globals.ReadIntFromINI("quicktab", "wait_time", Globals.OptionsINI, 75);
            int key_wait_time = Globals.ReadIntFromINI("quicktab", "key_wait_time", Globals.OptionsINI, 50);
            if (INI_setting == 0)
            {
                return 0;
            }
            else if (INI_setting == 1)
            {
                // insert hotkey
                UnregisterHotKey(this.Handle, 1);
                HideMainForm();
                System.Threading.Thread.Sleep(wait_time);
                int hotkey_char = Globals.ReadIntFromINI("hotkey", "hotkey_ascii_code", Globals.OptionsINI);
                byte hotkey_char_byte = Convert.ToByte(hotkey_char);
                keybd_event(hotkey_char_byte, 0, 0, UIntPtr.Zero);
                keybd_event(hotkey_char_byte, 0, 0x2, UIntPtr.Zero);
                SetHotKey();
            }
            else if (INI_setting == 2)
            {
                // insert a hotkey plus space.
                UnregisterHotKey(this.Handle, 1);
                HideMainForm();
                System.Threading.Thread.Sleep(wait_time);
                int hotkey_char = Globals.ReadIntFromINI("hotkey", "hotkey_ascii_code", Globals.OptionsINI);
                byte hotkey_char_byte = Convert.ToByte(hotkey_char);
                keybd_event(hotkey_char_byte, 0, 0, UIntPtr.Zero);
                keybd_event(hotkey_char_byte, 0, 0x2, UIntPtr.Zero);
                keybd_event(32, 0, 0, UIntPtr.Zero);
                keybd_event(32, 0, 0x2, UIntPtr.Zero);
                SetHotKey();
            }
            else if (INI_setting == 3)
            {
                // insert a hotkey plus enter.
                UnregisterHotKey(this.Handle, 1);
                HideMainForm();
                System.Threading.Thread.Sleep(wait_time);
                int hotkey_char = Globals.ReadIntFromINI("hotkey", "hotkey_ascii_code", Globals.OptionsINI);
                byte hotkey_char_byte = Convert.ToByte(hotkey_char);
                keybd_event(hotkey_char_byte, 0, 0, UIntPtr.Zero);
                keybd_event(hotkey_char_byte, 0, 0x2, UIntPtr.Zero);
                keybd_event(13, 0, 0, UIntPtr.Zero);
                keybd_event(13, 0, 0x2, UIntPtr.Zero);
                SetHotKey();
            }
            else if (INI_setting == 4)
            {
                // minimize window
                HideMainForm();
                System.Threading.Thread.Sleep(wait_time);
                IntPtr ActiveWindow = GetForegroundWindow();
                ShowWindow(ActiveWindow, 2);
            }
            else if (INI_setting == 5)
            {
                // maximize window
                HideMainForm();
                System.Threading.Thread.Sleep(wait_time);
                IntPtr ActiveWindow = GetForegroundWindow();
                ShowWindow(ActiveWindow, 3);
            }
            else if (INI_setting == 6)
            {
                // close window
                HideMainForm();
                System.Threading.Thread.Sleep(wait_time);
                IntPtr ActiveWindow = GetForegroundWindow();
                SendMessage(ActiveWindow, WM_SYSCOMMAND, SC_CLOSE, 0);
            }
            else if (INI_setting == 7)
            {
                // equivalent to Alt + Tab
                HideMainForm();
                System.Threading.Thread.Sleep(wait_time);
                keybd_event(18, 0, 0, UIntPtr.Zero);
                System.Threading.Thread.Sleep(key_wait_time);
                keybd_event(9, 0, 0, UIntPtr.Zero);
                keybd_event(9, 0, 0x2, UIntPtr.Zero);
                System.Threading.Thread.Sleep(key_wait_time);
                keybd_event(18, 0, 0x2, UIntPtr.Zero);
            }
            else if (INI_setting == 8)
            {
                // position topmost window. Get the coordinates from the INI file, and strip out the parameters. It should be in a format like left=0,top=0,width=50%,height=50%.
                HideMainForm();
                System.Threading.Thread.Sleep(wait_time);
                string the_coordinates = Globals.ReadINI("quicktab", KeyCombination + "_pos", Globals.OptionsINI);
                if (the_coordinates.Length == 0 || the_coordinates.IndexOf(",") == -1 || the_coordinates.IndexOf("=") == -1)
                    return 1;
                string position_left = "";
                string position_top = "";
                string position_width = "";
                string position_height = "";
                string[] words = the_coordinates.Split(',');
                foreach (string word in words)
                {
                    int find_equal = word.IndexOf("=");
                    if (find_equal == -1)
                        return 1;
                    string before_equal = word.Substring(0, find_equal).Trim();
                    string after_equal = word.Substring(find_equal + 1).Trim();
                    if (before_equal.ToLower() == "left")
                        position_left = after_equal;
                    else if (before_equal.ToLower() == "top")
                        position_top = after_equal;
                    else if (before_equal.ToLower() == "width")
                        position_width = after_equal;
                    else if (before_equal.ToLower() == "height")
                        position_height = after_equal;
                }
                PositionWindow("getforegroundwindow()", "", position_left, position_top, position_width, position_height);
            }
            else if (INI_setting == 9)
            {
                // show help page.
                RunURL("http://www.keybreeze.com/article/1");
            }

            // if we previously changed any notes, we need to change them back.
            foreach (int this_note_id in note_ids_on_top)
            {
                foreach (Note n in Globals.notelist)
                {
                    if (n.frmNote_instance.UniqueID == this_note_id)
                    {
                        // this was previously set on top. We need to put it on top again.
                        //n.frmNote_instance.TopMost = true;
                    }
                }
            }
            return 1;
        }

        public void HideMainForm(bool pressed_escape = false, bool send_notes_to_back = true)
        {
            // this procedure runs when the user hits Escape to hide the form, or clicks away, or launches a target.
            // If the user pressed Escape, we want to send the notes to the back.
            // we actually need to send the notes to the back in all cases. For some reason, Firefox seems to be the only window that doesn't retain focus after doing this. Keeping the notes in front allowed Firefox to retain focus, but if you don't have any notes showing, Firefox still doesn't retain focus, so this appears to be a Firefox issue.
            // Possible workaround: Set focus to Firefox after launching web entries.
            //if (pressed_escape == true)
            txtMain.Text = "";
            Application.DoEvents();
            if (true)
            {
                if (send_notes_to_back == true)
                {
                    if (display_notes_on_hotkey_press == 1 && just_created_a_note == false && this.Visible == true)
                        Globals.notes_bring_notes_to_front(false);
                }
                // If the user pressed Escape, we need to briefly show the popup form before we hide frmMain. This fixes a bug where Keybreeze Desktop retains focus after pressing the hotkey and then the Escape if there is a note showing and if there is no frmSkin loaded.
                if (Globals.notelist.Count > 0 && Globals.skin_embedded == 1 && this.Visible == true && popup_showing == false)
                {
                    Popup.Visible = true;
                    popup_showing = true;
                }
            }
            if (Globals.show_keyboard_help == 1)
            {
                frmPopupHelper_instance.Visible = false;
                tmrShowKeyboardHelp.Enabled = false;
            }
            Popup.Visible = false;
            popup_showing = false;
            initial_key_pressed = false;
            Globals.history_position = -1;
            this.Visible = false;
            if (Globals.valid_skin == true && Globals.skin_embedded == 0)
            {
                Skin.Visible = false;
            }
            if (just_created_a_note == true)
                just_created_a_note = false;
        }

        public void QuickTab_SetFocusToWindow(string input = "", bool first_letter_twice = false)
        {
            // this will set focus to a window where the window caption contains the input string.
            input = input.ToLower();
            string window_title;
            int find_match;
            IntPtr exact_matching_window_handle = IntPtr.Zero;
            IntPtr partial_matching_window_handle = IntPtr.Zero;
            bool partial_match = false;
            bool is_window_minimized;
            bool is_window_maximized;

            if (Globals.logging == 1)
                Globals.DebugWrite("QuickTab_SetFocusToWindow -- starting function with parameters, " + input + ", " + first_letter_twice);
            if (first_letter_twice == true)
            {
                // the user pressed the hotkey and typed a letter twice. Search for windows that start with this letter.
                string first_letter_of_input = input.Substring(0, 1).ToLower();
                // see if this letter has a window priority in the INI file.
                if (Globals.ReadINI("quicktab", "window_mapping_letter_1", Globals.OptionsINI).Length > 0)
                {
                    for (int i = 1; i < 27; i++)
                    {
                        string this_letter = Globals.ReadINI("quicktab", "window_mapping_letter_" + i, Globals.OptionsINI);
                        if (this_letter == first_letter_of_input)
                        {
                            string priority_window_title = Globals.ReadINI("quicktab", "window_mapping_title_" + i, Globals.OptionsINI);
                            string priority_window_class = Globals.ReadINI("quicktab", "window_mapping_class_" + i, Globals.OptionsINI);
                            if (priority_window_class.Length > 0)
                            {
                                // let's see if the window class is open.
                                if (Globals.logging == 1)
                                    Globals.DebugWrite("Seeing if " + priority_window_class + " class is open...");
                                try
                                {
                                    HideMainForm();
                                    exact_matching_window_handle = FindWindow(priority_window_class, null);
                                    if (exact_matching_window_handle != IntPtr.Zero)
                                    {
                                        is_window_minimized = IsIconic(exact_matching_window_handle);
                                        is_window_maximized = IsZoomed(exact_matching_window_handle);
                                        if (is_window_minimized == true)
                                        {
                                            bool show_window_response = ShowWindow(exact_matching_window_handle, 9);
                                            if (Globals.logging == 1)
                                                Globals.DebugWrite("Restored minimized window. " + show_window_response);
                                        }
                                        bool bring_to_foreground_response = SetForegroundWindow(exact_matching_window_handle);
                                        if (Globals.logging == 1)
                                            Globals.DebugWrite("Brought window to foreground. " + bring_to_foreground_response);
                                        return;
                                    }
                                    else
                                    {
                                        //MessageBox.Show("could not find class");
                                        if (Globals.logging == 1)
                                            Globals.DebugWrite("Could not find class");
                                    }
                                }
                                catch { }
                            }
                            else if (priority_window_title.Length > 0)
                            {
                                // let's see if the partial window title is open.
                                if (Globals.logging == 1)
                                    Globals.DebugWrite("Checking partial window titles for " + input);
                                Process[] processlist2 = Process.GetProcesses();
                                foreach (Process process in processlist2)
                                {
                                    if (!String.IsNullOrEmpty(process.MainWindowTitle))
                                    {
                                        window_title = process.MainWindowTitle;
                                        find_match = window_title.IndexOf(priority_window_title);
                                        if (find_match > -1)
                                        {
                                            // we have an exact or partial match. set focus to the window and exit the loop.
                                            try
                                            {
                                                HideMainForm();
                                                exact_matching_window_handle = FindWindow(null, process.MainWindowTitle);
                                                is_window_minimized = IsIconic(exact_matching_window_handle);
                                                is_window_maximized = IsZoomed(exact_matching_window_handle);
                                                if (is_window_minimized == true)
                                                    ShowWindow(exact_matching_window_handle, 9);
                                                SetForegroundWindow(exact_matching_window_handle);
                                            }
                                            catch { }
                                            return;
                                        }
                                    }
                                }
                                // if we are here, we didn't find an exact match yet. Look at Explorer windows, and then look at partial matches.
                                SHDocVw.ShellWindows shellWindows2 = new SHDocVw.ShellWindows();
                                string filename2;
                                foreach (SHDocVw.InternetExplorer ie in shellWindows2)
                                {
                                    filename2 = Path.GetFileNameWithoutExtension(ie.FullName).ToLower();

                                    if (filename2.Equals("explorer"))
                                    {
                                        // the user has a Windows Explorer window open. Search for our /input.
                                        string location_url = ie.LocationURL.ToLower();
                                        if (location_url.IndexOf("/" + priority_window_title) != -1)
                                        {
                                            // we found our input.
                                            HideMainForm();
                                            exact_matching_window_handle = (IntPtr)ie.HWND;
                                            is_window_minimized = IsIconic(exact_matching_window_handle);
                                            is_window_maximized = IsZoomed(exact_matching_window_handle);
                                            if (is_window_minimized == true)
                                                ShowWindow(exact_matching_window_handle, 9);
                                            SetForegroundWindow(exact_matching_window_handle);
                                            return;
                                        }
                                    }
                                }
                            }
                        }
                        else if (this_letter.Length == 0)
                        {
                            // the user doesn't have any more letters configured. Exit our loop.
                            break;
                        }
                    }
                }
                Process[] processlist = Process.GetProcesses();
                foreach (Process process in processlist)
                {
                    if (!String.IsNullOrEmpty(process.MainWindowTitle))
                    {
                        window_title = process.MainWindowTitle.ToLower();
                        // Update by Andrew on 12/31/2020. Added exclusion for SystemSettings since this is not a taskbar window.
                        if (window_title.Substring(0, 1) == first_letter_of_input && process.ProcessName != "SystemSettings")
                        {
                            // we found a match. Set focus to the window.
                            try
                            {
                                HideMainForm();
                                exact_matching_window_handle = FindWindow(null, process.MainWindowTitle);
                                is_window_minimized = IsIconic(exact_matching_window_handle);
                                is_window_maximized = IsZoomed(exact_matching_window_handle);
                                if (is_window_minimized == true)
                                    ShowWindow(exact_matching_window_handle, 9);
                                SetForegroundWindow(exact_matching_window_handle);
                                return;
                            }
                            catch { }
                        }
                    }
                }
                // if we are still here, we didn't find a window. Let's look at Explorer windows.
                SHDocVw.ShellWindows shellWindows = new SHDocVw.ShellWindows();
                string filename;
                foreach (SHDocVw.InternetExplorer ie in shellWindows)
                {
                    filename = Path.GetFileNameWithoutExtension(ie.FullName).ToLower();

                    if (filename.Equals("explorer"))
                    {
                        // the user has a Windows Explorer window open. Search for our /input.
                        string location_url = ie.LocationURL.ToLower();
                        if (location_url.IndexOf("/" + first_letter_of_input) != -1)
                        {
                            // we found our input.
                            HideMainForm();
                            exact_matching_window_handle = (IntPtr)ie.HWND;
                            is_window_minimized = IsIconic(exact_matching_window_handle);
                            is_window_maximized = IsZoomed(exact_matching_window_handle);
                            if (is_window_minimized == true)
                                ShowWindow(exact_matching_window_handle, 9);
                            SetForegroundWindow(exact_matching_window_handle);
                            return;
                        }
                    }
                }
            }
            else
            {
                // the user pressed the hotkey, typed a few letters of a window caption, and pressed the hotkey again. Search for matching window captions.
                // This doesn't work for Windows Explorer windows. To look at Explorer windows, we have to look at the SHDocVw COM call.
                Process[] processlist = Process.GetProcesses();
                foreach (Process process in processlist)
                {
                    if (!String.IsNullOrEmpty(process.MainWindowTitle))
                    {
                        window_title = process.MainWindowTitle.ToLower();
                        find_match = window_title.IndexOf(input);
                        if (find_match == 0)
                        {
                            // we have an exact match. set focus to the window and exit the loop.
                            try
                            {
                                HideMainForm();
                                exact_matching_window_handle = FindWindow(null, process.MainWindowTitle);
                                is_window_minimized = IsIconic(exact_matching_window_handle);
                                is_window_maximized = IsZoomed(exact_matching_window_handle);
                                if (is_window_minimized == true)
                                    ShowWindow(exact_matching_window_handle, 9);
                                SetForegroundWindow(exact_matching_window_handle);
                            }
                            catch { }
                            return;
                        }
                        else if (find_match > 0)
                        {
                            // we have a partial match. keep a record of this. if we don't find anything, set focus to it.
                            partial_match = true;
                            partial_matching_window_handle = FindWindow(null, process.MainWindowTitle);
                        }
                    }
                }
                // if we are here, we didn't find an exact match yet. Look at Explorer windows, and then look at partial matches.
                SHDocVw.ShellWindows shellWindows = new SHDocVw.ShellWindows();
                string filename;
                foreach (SHDocVw.InternetExplorer ie in shellWindows)
                {
                    filename = Path.GetFileNameWithoutExtension(ie.FullName).ToLower();

                    if (filename.Equals("explorer"))
                    {
                        // the user has a Windows Explorer window open. Search for our /input.
                        string location_url = ie.LocationURL.ToLower();
                        if (location_url.IndexOf("/" + input) != -1)
                        {
                            // we found our input.
                            HideMainForm();
                            exact_matching_window_handle = (IntPtr)ie.HWND;
                            is_window_minimized = IsIconic(exact_matching_window_handle);
                            is_window_maximized = IsZoomed(exact_matching_window_handle);
                            if (is_window_minimized == true)
                                ShowWindow(exact_matching_window_handle, 9);
                            SetForegroundWindow(exact_matching_window_handle);
                            return;
                        }
                    }
                }
                // if we are still here, look at partial matches
                if (partial_match == true)
                {
                    try
                    {
                        HideMainForm();
                        is_window_minimized = IsIconic(partial_matching_window_handle);
                        if (is_window_minimized == true)
                            ShowWindow(partial_matching_window_handle, 9);
                        SetForegroundWindow(partial_matching_window_handle);
                    }
                    catch { }
                }
            }

        }

        public static string GetWindowClassName(IntPtr hWnd)
        {
            StringBuilder buffer = new StringBuilder(128);

            GetClassName(hWnd, buffer, buffer.Capacity);

            return buffer.ToString();
        }
        
        public static string GetWindowTitle(IntPtr hWnd)
        {
            StringBuilder buffer = new StringBuilder(128);

            GetWindowText(hWnd, buffer, buffer.Capacity);

            return buffer.ToString();
        }

        public void ExitApplication(bool force_close = false)
        {
            try
            {
                UnregisterHotKey(this.Handle, 1);
                systrayicon.Visible = false;
                systrayicon.Icon = null;
                systrayicon.Visible = false;
                systrayicon.Dispose();
                if (force_close == true)
                    System.Environment.Exit(-1);
                else
                    Application.Exit();
            }
            catch { }
        }

        public void LaunchEntry(string TextString, int autocomplete_selected_index = -1)
        {
            try
            {
                if (Popup.Visible == true && Popup.autocomplete_match_list[Popup.selected_match_index].row_target == "keybreeze folder")
                {
                    // the user is launching a keybreeze folder.
                    string folder_path = Popup.autocomplete_match_list[Popup.selected_match_index].row_name;
                    frmAutoCompleteFolder frmAutoCompleteFolder_instance = new frmAutoCompleteFolder(this, folder_path);
                    frmAutoCompleteFolder_instance.Text = folder_path + " - Keybreeze Folder Viewer";
                    frmAutoCompleteFolder_instance.Show();
                    HideMainForm();
                    Add_Item_To_History(folder_path);
                }
                else if (Popup.Visible == true && Popup.autocomplete_match_list[Popup.selected_match_index].row_target != "previous entry")
                {
                    // the user is launching an entry from the autocomplete popup list, and it's not a previous entry. Previous entries can be letter commands, file paths, calculations, etc. We want to evaluate these later.
                    string target = Popup.autocomplete_match_list[Popup.selected_match_index].row_target;
                    string keyword = Popup.autocomplete_match_list[Popup.selected_match_index].row_name;
                    string thepath = Popup.autocomplete_match_list[Popup.selected_match_index].row_path;
                    HideMainForm();
                    RunTarget(target, "", keyword, thepath);
                    Add_Item_To_History(keyword);
                }
                else
                {
                    // the user launched an unknown entry. Search all the lists to see what it might be.
                    // if the Popup form is visible, let's populate the TextString with whatever is highlighted in the list, probably a previous entry.
                    string the_path = "";
                    if (Popup.Visible == true)
                    {
                        string target = Popup.autocomplete_match_list[Popup.selected_match_index].row_target;
                        if (target == "previous entry")
                            target = Popup.autocomplete_match_list[Popup.selected_match_index].row_name;
                        TextString = target;
                        the_path = Popup.autocomplete_match_list[Popup.selected_match_index].row_path;
                    }
                    else
                    {
                        if (TextString == "")
                        {
                            // the Popup window isn't showing and the user typed nothing before pressing Enter. The user is probably running a QuickTab option.
                            int qt_return_value = QuickTab_ControlWindow("hotkey_plus_enter");
                            if (qt_return_value == 1)
                                return;
                        }
                    }
                    string TheText = TextString.ToLower();
                    string current_keyword;
                    string first_part;
                    string command_line_parameters = "";
                    int text_length = TextString.Length;
                    int result_count = 0;
                    int FoundIt = 0;
                    int return_value = 0;
                    if (TheText.IndexOf(",") > -1)
                    {
                        // the user typed a comma, so he/she might be trying to launch a keyword with a parameter specified after it.
                        command_line_parameters = TextString.Substring(TheText.IndexOf(",") + 1);
                        TextString = TextString.Substring(0, TheText.IndexOf(","));
                        TheText = TextString.ToLower();
                        text_length = TextString.Length;
                    }
                    // loop through all the keywords and see which one matches the entry.
                    for (int i = 0; i < Globals.keyword_count; i++)
                    {
                        current_keyword = Globals.keyword_name_lower[i];
                        if (text_length <= current_keyword.Length)
                        {
                            first_part = current_keyword.Substring(0, text_length);
                            if (first_part == TheText)
                            {
                                // we have an exact match. Run the target.
                                HideMainForm();
                                the_path = Globals.Keyword_List[i].path;
                                RunTarget(Globals.Keyword_List[i].target, command_line_parameters, "", the_path);
                                if (command_line_parameters.Length > 0)
                                    Add_Item_To_History(current_keyword + "," + command_line_parameters);
                                else
                                    Add_Item_To_History(current_keyword);
                                result_count = 1;
                                break;
                            }
                        }
                    }

                    // if we have not found an exact match yet, let's search on partial matches.
                    if (result_count == 0)
                    {
                        for (int i = 0; i < Globals.keyword_count; i++)
                        {
                            //current_keyword = Globals.Keyword_List[i].name_lowercase;
                            current_keyword = Globals.keyword_name_lower[i];
                            FoundIt = current_keyword.IndexOf(TheText);
                            if (FoundIt > 0)
                            {
                                // we found a partial match. Run the target.
                                HideMainForm();
                                RunTarget(Globals.Keyword_List[i].target, "", "", the_path);
                                Add_Item_To_History(current_keyword);
                                result_count = 1;
                                break;
                            }
                        }
                    }

                    // the unknown entry might be from the automatic index list.
                    int counter2 = 0;
                    if (result_count == 0)
                    {
                        foreach (string index_entry in Globals.autoindex_files)
                        {
                            if (index_entry.ToLower() == TheText)
                            {
                                // it is from the automatic index list.
                                HideMainForm();
                                RunTarget(Globals.autoindex_filepaths[counter2], command_line_parameters);
                                Add_Item_To_History(Globals.autoindex_files[counter2]);
                                result_count = 1;
                                break;
                            }
                            counter2++;
                        }
                    }

                    if (command_line_parameters.Length > 0)
                    {
                        // if we've reached this point, we no longer have a need for command line parameters. Include the entry and the parameters as one string.
                        TextString = TextString + "," + command_line_parameters;
                        TheText = TextString.ToLower();
                        text_length = TextString.Length;
                    }

                    // Trim the text, because sometimes users will type extra spaces to avoid
                    // selecting from the autocomplete.
                    if (result_count == 0)
                    {
                        TextString = TextString.Trim();

                        // check letter commands
                        HideMainForm();
                        return_value = RunLetterCommand(TextString);
                        if (return_value == 1)
                        {
                            // it was a letter command and it successfully ran.
                            result_count = 1;
                        }

                        if (result_count == 0)
                        {
                            // we have not found any matches yet, so let's see if it's a direct path
                            // to a file, folder, or web page.
                            HideMainForm();
                            return_value = RunUnknownEntry(TextString);
                        }
                        // we don't want to add a calculation to the autocomplete list or an unknown entry.
                        if (return_value != 9 && return_value != 10)
                            return_value = Globals.Update_AutoComplete_List(TextString);
                        Add_Item_To_History(TextString);
                    }
                }

                // Firefox doesn't retain focus if this is down here. We need to move it up in this function.
                /*
                Application.DoEvents();
                //System.Threading.Thread.Sleep(200);
                HideMainForm();
                Application.DoEvents();
                 */
                /*
                if (text_length == 0)
                    return;
                if (target.Length > 0 && target != "previous entry")
                {
                    // the user typed a keyword or autoindex entry from the main toolbar. Run it.
                    RunTarget(target, "", TextString);
                    Add_Item_To_History(TextString);
                    result_count = 1;
                }
                else if (target == "previous entry")
                {
                    RunUnknownEntry(TextString);
                    Add_Item_To_History(TextString);
                    result_count = 1;
                }
                else
                {
                
                }
                 */
            }
            catch { }
        }

        public void Add_Item_To_History(string TheText)
        {
            int history_count = Globals.History_List.Count;
            if (history_count > 0)
            {
                // make sure the previous item is not identical.
                string last_item = Globals.History_List[history_count - 1].e;
                if (last_item.ToLower() != TheText.ToLower())
                {
                    if (history_count >= 100)
                    {
                        // we have 100 items. Let's elinate the oldest one.
                        Globals.History_List.RemoveAt(0);
                    }
                    History new_history_item = new History();
                    new_history_item.e = TheText;
                    Globals.History_List.Add(new_history_item);
                    Globals.History_Save_To_File();
                }
            }
            else
            {
                // there is nothing in our history. Add the item.
                History new_history_item = new History();
                new_history_item.e = TheText;
                Globals.History_List.Add(new_history_item);
                Globals.History_Save_To_File();
            }
        }

        public void txtMain_TextChanged(object sender, EventArgs e)
        {
            if (this.Visible == false)
                return;
            txtMain_TextChanged_Delay();
        }

        public void txtMain_TextChanged_Delay()
        {
            // loop through all the keywords to see if we have a match
            string TextString = txtMain.Text.ToLower();
            string current_keyword;
            int text_length = TextString.Length;
            if (text_length > 0)
                initial_key_pressed = true;
            // reset some variables.
            if (Globals.show_keyboard_help == 1)
            {
                tmrShowKeyboardHelp.Enabled = false;
            }
            int result_count = 0;
            int exact_match_count = 0;
            int FoundIt = 0;
            string first_part;
            List<int> partial_match_list_index = new List<int>();

            // first let's cancel our autoindex searches if they are going on.
            //tmrShowAutoIndexResults.Enabled = false;
            //tmrIndexOnDemand.Enabled = false;
            user_pressed_new_key = true;

            // clear our current autocomlete popup list since the user just changed the entry.
            Popup.autocomplete_match_list.Clear();

            if (history_selection.Length > 0 && history_selection == txtMain.Text)
            {
                // the user is accessing an item from history. We don't want to show the autocomplete list.
                return;
            }

            // if the user is continuing to type a keyword that's alright selected, we don't want to select it again.
            if (text_length > 2 && text_length > PreviousEntry.Length && Popup.row_keywords[0].Text.ToLower().IndexOf(TextString) > -1)
            {
                ClearAutoCompleteList(1);
            }
            else
            {
                ClearAutoCompleteList(0);
            }

            // see if we have any exact matches or partial matches so far...
            if (text_length > 1)
            {
                partial_match_list_index.Clear();
                // search folders first
                foreach (Keyword_Folder this_folder in Globals.Keyword_Folders)
                {
                    if (text_length <= this_folder.path.Length)
                    {
                        FoundIt = this_folder.path.ToLower().IndexOf(TextString);
                        if (FoundIt == 0)
                        {
                            // exact match
                            int popup_return = Popup.Row_Add_Row(this_folder.path, "keybreeze folder", "", null);
                            PreviousKeywordMatch = this_folder.path;
                            PreviousEntry = TextString;
                            exact_match_count = 1;
                            result_count = 1;
                            if (popup_return == -1)
                            {
                                ShowPopupForm();
                                return;
                            }
                        }
                        else if (FoundIt > 0)
                        {
                            // partial match
                            // currently we are not going to add partial folder matches to the autocomplete.
                        }
                    }
                }
                for (int i = 0; i < Globals.keyword_count; i++)
                {
                    current_keyword = Globals.keyword_name_lower[i];
                    if (text_length <= current_keyword.Length)
                    {
                        FoundIt = current_keyword.IndexOf(TextString);
                        if (FoundIt == 0)
                        {
                            // we found an exact match.
                            int popup_return = Popup.Row_Add_Row(Globals.Keyword_List[i].name, Globals.Keyword_List[i].target, "keyword", Globals.Keyword_List[i].path);
                            PreviousKeywordMatch = current_keyword;
                            PreviousEntry = TextString;
                            exact_match_count = 1;
                            result_count = 1;
                            if (popup_return == -1)
                            {
                                ShowPopupForm();
                                return;
                            }
                        }
                        else if (FoundIt > 0)
                        {
                            // we found a partial match.
                            partial_match_list_index.Add(i);
                            result_count = 1;
                        }
                    }
                }

                if (text_length == 2 && exact_match_count == 0)
                {
                    if (TextString.Substring(0, 1) == TextString.Substring(1, 1))
                    {
                        // the user typed the same character twice. Check QuickTab. Do we need to exclude anything?
                        bool use_quicktab = true;
                        if (TextString == "ww")
                        {
                            // the user presed ww twice, do they want to type a website or use quicktab?
                            string include_ww = Globals.ReadINI("quicktab", "include_ww", Globals.OptionsINI);
                            if (include_ww != "1")
                            {
                                use_quicktab = false;
                            }
                        }
                        if (TextString == @"\\")
                        {
                            // we should always ignore \\ because the user is typing a network path.
                            use_quicktab = false;
                        }
                        if (use_quicktab == true)
                        {
                            int quicktab_enabled = Globals.ReadIntFromINI("quicktab", "double_letter_enabled", Globals.OptionsINI, 1);
                            if (quicktab_enabled == 1)
                            {
                                QuickTab_SetFocusToWindow(TextString, true);
                                return;
                            }
                        }
                    }
                }

                // add the partial matches into the autocomplete list
                foreach (int i in partial_match_list_index)
                {
                    int popup_return = Popup.Row_Add_Row(Globals.Keyword_List[i].name, Globals.Keyword_List[i].target, "keyword", Globals.Keyword_List[i].path);
                    PreviousKeywordMatch = Globals.keyword_name_lower[i];
                    PreviousEntry = TextString;
                    if (popup_return == -1)
                    {
                        ShowPopupForm();
                        return;
                    }
                }

                // let's search the autocomplete list.
                if (result_count == 0)
                {
                    int autocomplete_counter = 0;
                    foreach (string entry in Globals.autocomplete_list)
                    {
                        if (text_length <= entry.Length)
                        {
                            first_part = entry.Substring(0, text_length);
                            if (first_part == TextString)
                            {
                                // we found a match. find out what it is (website, file, folder, etc.)
                                if (entry.IndexOf(":\\") != -1 || entry.IndexOf(":/") != -1)
                                {
                                    if (entry.IndexOf(".") != -1)
                                    {
                                        // it is probably a file
                                        int popup_return = Popup.Row_Add_Row(entry, "previous entry", "previous entry", null);
                                        PreviousEntry = TextString;
                                        PreviousKeywordMatch = entry;
                                        result_count = 1;
                                        if (popup_return == -1)
                                        {
                                            ShowPopupForm();
                                            return;
                                        }
                                    }
                                    else
                                    {
                                        // it is probably a folder
                                        int popup_return = Popup.Row_Add_Row(entry, "previous entry", "previous entry", null);
                                        PreviousEntry = TextString;
                                        PreviousKeywordMatch = entry;
                                        result_count = 1;
                                        if (popup_return == -1)
                                        {
                                            ShowPopupForm();
                                            return;
                                        }
                                    }
                                }
                                else
                                {
                                    // it is a website or a letter command
                                    int popup_return = Popup.Row_Add_Row(entry, "previous entry", "previous entry", null);
                                    PreviousEntry = TextString;
                                    PreviousKeywordMatch = entry;
                                    result_count = 1;
                                    if (popup_return == -1)
                                    {
                                        ShowPopupForm();
                                        return;
                                    }
                                }
                            }
                        }
                        autocomplete_counter += 1;
                    }
                }
                // did we find any results? if so, show the popup form.
                if (result_count > 0)
                    ShowPopupForm();
                
                // finally, let's look at the autoindex list
                if (result_count == 0)
                {
                    // 3/6/2018 update: look at spacing first.
                    // look at each word between the spaces.
                    int find_space = TextString.IndexOf(" ");
                    // if user added space at the end, it's probably because they are typing a letter command and don't want to search the command list for each word.
                    bool is_space_at_end = false;
                    if (TextString.Substring(TextString.Length - 1) == " ")
                        is_space_at_end = true;
                    if (find_space != -1 && is_space_at_end == false)
                    {
                        // search for each word in the command list.
                        string[] text_entry_words = TextString.Split(' ');
                        for (int i = 0; i < Globals.keyword_count; i++)
                        {
                            current_keyword = Globals.keyword_name_lower[i];
                            if (text_length <= current_keyword.Length && current_keyword.IndexOf(" ") != -1)
                            {
                                // Both our text entry and current keyword in the list have spaces. Loop through each word to see if it's there.
                                string[] keyword_words = current_keyword.Split(' ');
                                bool found_all_words = false;
                                foreach (string text_entry_word in text_entry_words)
                                {
                                    bool found_this_word = false;
                                    // this wasn't working when typing "click j" and trying to match "jquery click"
                                    //if (text_entry_word.Length <= 1)
                                    //    continue;
                                    foreach (string keyword_word in keyword_words)
                                    {
                                        if (keyword_word.IndexOf(text_entry_word) == 0)
                                        {
                                            found_this_word = true;
                                            found_all_words = true;
                                            break;
                                        }
                                    }
                                    if (found_this_word == false)
                                    {
                                        found_all_words = false;
                                        break;
                                    }
                                }
                                if (found_all_words == true)
                                {
                                    // we found an exact match.
                                    int popup_return = Popup.Row_Add_Row(Globals.Keyword_List[i].name, Globals.Keyword_List[i].target, "keyword", Globals.Keyword_List[i].path);
                                    PreviousKeywordMatch = current_keyword;
                                    PreviousEntry = TextString;
                                    exact_match_count = 1;
                                    result_count = 1;
                                    if (popup_return == -1)
                                    {
                                        ShowPopupForm();
                                        return;
                                    }
                                }
                            }
                        }
                    }
                    if (result_count == 0)
                    {
                        // Update by Andrew on March 17th, 2018.
                        // still no results. Look at dashes and underscores. Also remove spaces unless the user added spaces at the end to get rid of the autocomplete.
                        //TextString = TextString.Replace("_", " ");
                        TextString = TextString.Replace("_", "");
                        TextString = TextString.Replace("-", "");
                        bool space_at_end = false;
                        bool second_space_at_end = false;
                        if (TextString.Length > 0 && TextString.Substring(TextString.Length - 1) == " ")
                            space_at_end = true;
                        if (TextString.Length >= 2 && TextString.Substring(TextString.Length - 2) == " ")
                            second_space_at_end = true;
                        TextString = TextString.Replace(" ", "");
                        if (space_at_end == true)
                            TextString = TextString + " ";
                        if (second_space_at_end == true)
                            TextString = TextString + " ";
                        for (int i = 0; i < Globals.keyword_count; i++)
                        {
                            current_keyword = Globals.keyword_name_lower[i];
                            current_keyword = current_keyword.Replace("_", "");
                            current_keyword = current_keyword.Replace("-", "");
                            current_keyword = current_keyword.Replace(" ", "");
                            if (text_length <= current_keyword.Length)
                            {
                                FoundIt = current_keyword.IndexOf(TextString);
                                if (FoundIt == 0)
                                {
                                    // we found an exact match.
                                    int popup_return = Popup.Row_Add_Row(Globals.Keyword_List[i].name, Globals.Keyword_List[i].target, "keyword", Globals.Keyword_List[i].path);
                                    PreviousKeywordMatch = current_keyword;
                                    PreviousEntry = TextString;
                                    exact_match_count = 1;
                                    result_count = 1;
                                    if (popup_return == -1)
                                    {
                                        ShowPopupForm();
                                        return;
                                    }
                                }
                                else if (FoundIt > 0)
                                {
                                    // we found a partial match.
                                    partial_match_list_index.Add(i);
                                    result_count = 1;
                                }
                            }
                        }
                        foreach (int i in partial_match_list_index)
                        {
                            int popup_return = Popup.Row_Add_Row(Globals.Keyword_List[i].name, Globals.Keyword_List[i].target, "keyword", Globals.Keyword_List[i].path);
                            PreviousKeywordMatch = Globals.keyword_name_lower[i];
                            PreviousEntry = TextString;
                            if (popup_return == -1)
                            {
                                ShowPopupForm();
                                return;
                            }
                        }
                    }
                    // Update by Andrew on March 17th.
                    // still no results. Look at targets only if user is not typing a file path.
                    bool typing_file_path = false;
                    if (TextString.Length > 2)
                    {
                        if (TextString.Substring(1, 1) == ":")
                            typing_file_path = true;
                    }
                    if (result_count == 0 && typing_file_path == false)
                    {
                        for (int i = 0; i < Globals.keyword_count; i++)
                        {
                            current_keyword = Globals.Keyword_List[i].target.ToLower();
                            if (text_length <= current_keyword.Length)
                            {
                                FoundIt = current_keyword.IndexOf(TextString);
                                if (FoundIt == 0)
                                {
                                    // we found an exact match.
                                    int popup_return = Popup.Row_Add_Row(Globals.Keyword_List[i].name, Globals.Keyword_List[i].target, "keyword", Globals.Keyword_List[i].path);
                                    PreviousKeywordMatch = current_keyword;
                                    PreviousEntry = TextString;
                                    exact_match_count = 1;
                                    result_count = 1;
                                    if (popup_return == -1)
                                    {
                                        ShowPopupForm();
                                        return;
                                    }
                                }
                                else if (FoundIt > 0)
                                {
                                    // we found a partial match.
                                    partial_match_list_index.Add(i);
                                    result_count = 1;
                                }
                            }
                        }
                        foreach (int i in partial_match_list_index)
                        {
                            int popup_return = Popup.Row_Add_Row(Globals.Keyword_List[i].name, Globals.Keyword_List[i].target, "keyword", Globals.Keyword_List[i].path);
                            PreviousKeywordMatch = Globals.keyword_name_lower[i];
                            PreviousEntry = TextString;
                            if (popup_return == -1)
                            {
                                ShowPopupForm();
                                return;
                            }
                        }
                    }
                    if (Popup.row_keywords[0].Text == "")
                    {
                        // no results, so let's hide the popup window.
                        Popup.Visible = false;
                        popup_showing = false;
                        // show the keyboard help.
                        if (Globals.show_keyboard_help == 1)
                        {
                            // first let's hide it if it's currently showing (since the user just typed a character). Wait a second or two before showing it again.
                            frmPopupHelper_instance.Visible = false;
                            tmrShowKeyboardHelp.Interval = 1500;
                            tmrShowKeyboardHelp.Enabled = true;
                        }
                    }
                    tmrShowAutoIndexResults.Interval = popup_result_delay_with_no_matches;
                    tmrShowAutoIndexResults.Enabled = true;

                    // did we find any results? if so, show the popup form.
                    if (result_count > 0)
                        ShowPopupForm();
                }
                else if (text_length > 2)
                {
                    // we have at least one match so far, and the user typed more than 2 characters.
                    tmrShowAutoIndexResults.Interval = popup_result_delay_with_matches;
                    tmrShowAutoIndexResults.Enabled = true;
                }
            }
            else if (text_length == 1)
            {
                // We only have one character, and the user may keep typing something after that 
                // character. To avoid delays, let's allow 500 milliseconds to pass. If the user types
                // another letter within 500 milliseconds, don't do anything. Otherwise, select all
                // keywords that start with that letter.

                tmrAutoCompleteDelay.Interval = 500;
                tmrAutoCompleteDelay.Enabled = true;
            }
            else if (text_length == 0)
            {
                Popup.Visible = false;
                popup_showing = false;
            }

            // the following is a test to see if the text ever updates before this procedure finishes.
            /*
            if (txtMain.Text.Length != text_length)
            {
                MessageBox.Show("they are off track: " + txtMain.Text.Length + ", " + text_length);
            }*/
        }

        private void tmrAutoCompleteDelay_Tick(object sender, EventArgs e)
        {
            // this timer is activated when the user types one character in the main text field.
            // keybreeze will pause for a half second to see if the user continues typing. If so, exit.
            // if the user just types one character, show the auto-complete list.
            // this half-second delay is useful because it saves processing time, i.e. if the user
            // just types one character, we will have a ton of search results. We don't want to display 
            // all this information if the user is just going to continue typing.

            if (this.Visible == false)
                return;

            string TextString = txtMain.Text.ToLower();
            string current_keyword;
            string first_part;
            int text_length = txtMain.Text.Length;

            // check to see if we have no characters or more than one character. If we do, exit.
            if (text_length == 0 || text_length > 1)
            {
                tmrAutoCompleteDelay.Enabled = false;
                return;
            }

            /*
            if (Popup.Visible == false)
            {
                popup_showing = false;
                tmrAutoCompleteDelay.Enabled = false;
                return;
            }
            else
            {
                popup_showing = true;
            }
             */

            if (TextString.Length == 1)
            {
                for (int i = 0; i < Globals.keyword_count; i++)
                {
                    current_keyword = Globals.keyword_name_lower[i];
                    if (text_length <= current_keyword.Length)
                    {
                        first_part = current_keyword.Substring(0, text_length);
                        if (first_part == TextString)
                        {
                            // we have an exact match.
                            int popup_return = Popup.Row_Add_Row(current_keyword, Globals.Keyword_List[i].target, "keyword", Globals.Keyword_List[i].path);
                            PreviousKeywordMatch = current_keyword;
                            PreviousEntry = TextString;
                            PreviousTempCount = i;
                            ShowPopupForm();
                            if (popup_return == -1)
                            {
                                tmrAutoCompleteDelay.Enabled = false;
                                return;
                            }
                        }
                    }
                }
            }
            tmrAutoCompleteDelay.Enabled = false;
            return;
        }

        public void MoveChildForms()
        {
            if (Globals.valid_skin == true && Globals.skin_embedded == 0)
            {
                this.Location = new Point(Skin.Location.X + Globals.main_textbox_left, Skin.Location.Y + Globals.main_textbox_top);
            }
            int screenHeight = Screen.PrimaryScreen.Bounds.Height;
            if (this.Top + this.Height + 50 + Popup.Height > screenHeight)
            {
                // position the Popup window above the main window instead of below it.
                if (Globals.valid_skin == true && Globals.skin_embedded == 0)
                    Popup.Top = Skin.Top - Popup.Height - Globals.top_offset;
                else
                    Popup.Top = this.Top - Popup.Height - Globals.top_offset;
            }
            else
            {
                if (Globals.valid_skin == true && Globals.skin_embedded == 0)
                    Popup.Top = Skin.Top + Skin.Height + Globals.top_offset;
                else
                    Popup.Top = this.Top + this.Height + Globals.top_offset;
            }
            Popup.Left = this.Left - 19 + Globals.left_offset;
            if (always_center_keybreeze == 0)
            {
                // set new coordinates in the INI file.
                Globals.WriteINI("options", "skin_position_top", this.Top.ToString(), Globals.OptionsINI);
                Globals.WriteINI("options", "skin_position_left", this.Left.ToString(), Globals.OptionsINI);
            }
            if (txtMain.Text.Length > 0)
            {
                Popup.Visible = true;
                popup_showing = true;
            }
            if (Globals.show_keyboard_help == 1)
            {
                frmPopupHelper_instance.Top = Popup.Top;
                frmPopupHelper_instance.Left = Popup.Left;
                frmPopupHelper_instance.Visible = false;
                tmrShowKeyboardHelp.Enabled = false;
            }
            this.Visible = true;
        }

        public void Set_Keybreeze_Position()
        {
            if (always_center_keybreeze == 1)
            {
                menuResetPosition(false);
            }
            else
            {
                // we are remembering the previous position.
                int screenHeight = Screen.PrimaryScreen.Bounds.Height;
                int skin_top = Globals.ReadIntFromINI("options", "skin_position_top", Globals.OptionsINI, -99);
                int skin_left = Globals.ReadIntFromINI("options", "skin_position_left", Globals.OptionsINI, -99);
                // -99 represents a value that is not set yet.
                if (skin_top == -99 || skin_left == -99)
                {
                    // the position is no longer in the INI file. Reset it.
                    menuResetPosition(false);
                }
                else
                {
                    this.Top = skin_top;
                    this.Left = skin_left;
                    if (Globals.valid_skin == true && Globals.skin_embedded == 0)
                    {
                        if (Skin.bitmap != null)
                            Skin.SetBitmap(Skin.bitmap, skin_opacity);
                        //Skin.Location = new Point(skin_left - 24, skin_top - 26);
                        Skin.Location = new Point(this.Left - 24 + Globals.skin_left_offset, this.Top - 26 + Globals.skin_top_offset);
                    }
                    if (this.Top + this.Height + 50 + Popup.Height > screenHeight)
                    {
                        // position the Popup window above the main window instead of below it.
                        if (Globals.valid_skin == true && Globals.skin_embedded == 0)
                            Popup.Top = Skin.Top - Popup.Height - Globals.top_offset;
                        else
                            Popup.Top = this.Top - Popup.Height - Globals.top_offset;
                    }
                    else
                    {
                        if (Globals.valid_skin == true && Globals.skin_embedded == 0)
                            Popup.Top = Skin.Top + Skin.Height + Globals.top_offset;
                        else
                            Popup.Top = this.Top + this.Height + Globals.top_offset;
                    }
                    Popup.Left = this.Left - 12 + Globals.left_offset;

                    if (Globals.show_keyboard_help == 1)
                    {
                        frmPopupHelper_instance.Top = Popup.Top;
                        frmPopupHelper_instance.Left = Popup.Left;
                    }
                }
            }
        }

        public void MovingForm()
        {
            this.Visible = false;
            Popup.Visible = false;
            if (Globals.show_keyboard_help == 1)
            {
                frmPopupHelper_instance.Visible = false;
                tmrShowKeyboardHelp.Enabled = false;
            }
            popup_showing = false;
        }

        private void btnOK_Click(object sender, EventArgs e)
        {
            initial_key_pressed = false;
            if (Popup.Visible == true)
            {
                // the user launched an entry from the autocomplete popup list.
                LaunchEntry(txtMain.Text, Popup.selected_match_index);
            }
            else
            {
                // the user launched an unknown entry.
                LaunchEntry(txtMain.Text);
            }
            /*
            if (Popup.row_selection == 0)
            {
                if (Popup.popup_list_element_type.Count() > 0)
                {
                    if (Popup.popup_list_element_type[0] == 0)
                        LaunchEntry(Globals.keyword_names[Popup.popup_list_elements[Popup.row_selection]], Globals.keyword_targets[Popup.popup_list_elements[Popup.row_selection]]);
                    else if (Popup.popup_list_element_type[0] == 1)
                        LaunchEntry(Popup.row_keywords[0].Text, "previous entry");
                    else if (Popup.popup_list_element_type[0] == 2)
                        LaunchEntry(Globals.autoindex_files[Popup.popup_list_elements[Popup.row_selection]], Globals.autoindex_filepaths[Popup.popup_list_elements[Popup.row_selection]]);
                }
                else
                {
                    // unknown entry
                    LaunchEntry(txtMain.Text);
                }
            }
            else
            {
                if (Popup.popup_list_element_type[Popup.row_selection] == 0)
                    LaunchEntry(Globals.keyword_names[Popup.popup_list_elements[Popup.row_selection]], Globals.keyword_targets[Popup.popup_list_elements[Popup.row_selection]]);
                else if (Popup.popup_list_element_type[Popup.row_selection] == 1)
                    LaunchEntry(Popup.row_keywords[Popup.row_label_selection].Text, "previous entry");
                else if (Popup.popup_list_element_type[Popup.row_selection] == 2)
                    LaunchEntry(Globals.autoindex_files[Popup.popup_list_elements[Popup.row_selection]], Globals.autoindex_filepaths[Popup.popup_list_elements[Popup.row_selection]]);
            }
             */
        }

        /*
        public void AddKeywordToAutoComplete(int keyword_index, int icon_index, string keyword_name, string target)
        {
            int listCount = 0;
            int rowPosition;

            Popup.popup_list_elements.Add(keyword_index);
            Popup.popup_list_element_type.Add(0);
            try
            {
                if (Globals.skin_icon_type == 1)
                    Popup.autocompleteImageListLarge.Images.Add(Popup.imageListLarge.Images[icon_index]);
            }
            catch { }
            listCount = Popup.popup_list_elements.Count();

            // if the entry is between 0 and 5, we should display it in the list.
            if (listCount > 0 && listCount < Popup.total_rows_visible + 1)
            {
                rowPosition = listCount - 1;
                //Popup.row_panels[0].BackColor = System.Drawing.ColorTranslator.FromHtml(Popup.autocomplete_highlight_color);
                Popup.HighlightRow(0, true);
                try
                {
                    if (Globals.skin_icon_type == 1)
                        Popup.row_icons[rowPosition].Image = Popup.imageListLarge.Images[icon_index];
                }
                catch { }
                Popup.row_keywords[rowPosition].Text = keyword_name;
                Popup.row_descriptions[rowPosition].Text = ReturnShortenedPath(target);
                Popup.row_descriptions[rowPosition].Tag = target;
            }
        }
         */

        /*
        public void AddEntryToAutoComplete(int icon_index, int entry_index, string autoindex = "", string index_on_demand_path = "")
        {
            int listCount = 0;
            int rowPosition;

            if (autoindex == "")
            {
                // we are adding a regular entry, e.g. web page, google search, wikipedia search, etc.
                Popup.popup_list_elements.Add(entry_index);
                Popup.popup_list_element_type.Add(1);
                try
                {
                    if (Globals.skin_icon_type == 1)
                        Popup.autocompleteImageListLarge.Images.Add(Popup.imageListLarge.Images[icon_index]);
                }
                catch { }
                listCount = Popup.popup_list_elements.Count();
                if (listCount > 0 && listCount < Popup.row_number + 1)
                {
                    rowPosition = listCount - 1;
                    //Popup.row_panels[0].BackColor = System.Drawing.ColorTranslator.FromHtml(Popup.autocomplete_highlight_color);
                    Popup.HighlightRow(0, true);
                    try
                    {
                        if (Globals.skin_icon_type == 1)
                            Popup.row_icons[rowPosition].Image = Popup.imageListLarge.Images[icon_index];
                    }
                    catch { }
                    Popup.row_keywords[rowPosition].Text = Globals.autocomplete_list[entry_index];
                    Popup.row_descriptions[rowPosition].Text = "previous entry";
                    Popup.row_descriptions[rowPosition].Tag = "previous entry";
                }
            }
            else if (index_on_demand_path != "")
            {
                // we are adding an entry that was found on demand.

                frmPopup.autocomplete_list_row new_row = new frmPopup.autocomplete_list_row();
                
                Popup.autocomplete_list.Add(new_row);
            }
            else
            {
                // we are adding an entry from the autoindex to the list.
                try
                {
                    Popup.popup_list_elements.Add(entry_index);
                    Popup.popup_list_element_type.Add(2);
                    if (Globals.skin_icon_type == 1)
                        Popup.Add_Icon_To_AutoComplete(Globals.autoindex_filepaths[entry_index]);
                    listCount = Popup.popup_list_elements.Count();
                }
                catch
                {
                }
                if (listCount > 0 && listCount < Popup.row_number + 1)
                {
                    rowPosition = listCount - 1;
                    //Popup.row_panels[0].BackColor = System.Drawing.ColorTranslator.FromHtml(Popup.autocomplete_highlight_color);
                    Popup.HighlightRow(0, true);
                    try
                    {
                        if (Globals.skin_icon_type == 1)
                            Popup.row_icons[rowPosition].Image = Popup.autocompleteImageListLarge.Images[rowPosition];
                    }
                    catch { }
                    Popup.row_keywords[rowPosition].Text = Globals.autoindex_files[entry_index];
                    Popup.row_descriptions[rowPosition].Text = ReturnShortenedPath(Globals.autoindex_filepaths[entry_index]);
                    Popup.row_descriptions[rowPosition].Tag = Globals.autoindex_filepaths[entry_index];
                }
            }
        }
         */

        private void ShowPopupForm()
        {
            if (popup_showing == false && Globals.history_position == -1)
            {
                Popup.Visible = true;
                txtMain.Focus();
                // added 11/18/2013.
                txtMain.SelectionStart = txtMain.Text.Length;
                popup_showing = true;
                if (Globals.show_keyboard_help == 1 && frmPopupHelper_instance.Visible == true)
                {
                    frmPopupHelper_instance.Visible = false;
                    tmrShowKeyboardHelp.Enabled = false;
                }
            }
        }

        public void ClearAutoCompleteList(int KeepFirstRow)
        {
            Popup.autocomplete_match_list.Clear();
            Popup.selected_match_index = 0;
            /*
            Popup.popup_list_elements.Clear();
            Popup.popup_list_element_type.Clear();
            Popup.row_selection = 0;
             */
            if (Globals.skin_icon_type == 1)
            {
                Popup.autocompleteImageListLarge.Images.Clear();
                Popup.autocompleteImageListSmall.Images.Clear();
                Popup.imageListLarge.Images.Clear();
            }
            else if (Globals.skin_icon_type == 2)
            {
                for (int i = KeepFirstRow; i < Popup.total_rows_visible; i++)
                {
                    Popup.row_file_extension_labels[i].Text = "";
                }
            }
            Popup.autoindex_icon_tracker = 0;
            Popup.selected_row_index = 0;
            for (int i = KeepFirstRow; i < Popup.total_rows_visible; i++)
            {
                Popup.row_keywords[i].Text = "";
                Popup.row_descriptions[i].Text = "";
                Popup.row_descriptions[i].Tag = "";
                Popup.HighlightRow(i, false);
                if (Globals.skin_icon_type == 1)
                    Popup.row_icons[i].Image = null;
            }
        }

        private void btnCancel_Click_1(object sender, EventArgs e)
        {
            // the user likely pressed Escape, which clicks the Cancel button. Let's hide the main form and send the notes to the back.
            HideMainForm(true);
        }

        private void txtMain_KeyDown(object sender, KeyEventArgs e)
        {
            if (user_pressed_shift == true)
                user_pressed_shift = false;
            if (user_pressed_ctrl == true)
                user_pressed_ctrl = false;
            // see if the user pressed the arrow keys to change the selection.
            if (e.KeyCode == Keys.Down)
            {
                // if the popup list is not showing, the user may be trying to access the history.
                if (Popup.Visible == false)
                {
                    if (Globals.History_List.Count > 0)
                    {
                        if (Globals.history_position == -1)
                            Globals.history_position = Globals.History_List.Count;
                        if (Globals.history_position > 0)
                            Globals.history_position = Globals.history_position - 1;
                        string last_history_item = Globals.History_List[Globals.history_position].e;
                        txtMain.Text = last_history_item;
                        txtMain.Select(txtMain.Text.Length, 0);
                    }
                }
                else
                {
                    MoveSelectionDown();
                }
            }
            else if (e.KeyCode == Keys.Up)
            {
                // if the popup list is not showing, the user may be trying to access the history.
                if (Popup.Visible == false)
                {
                    if (Globals.History_List.Count > 0)
                    {
                        if (Globals.history_position != -1 && Globals.history_position < Globals.History_List.Count - 1)
                        {
                            // we are not at our max yet. Let's move up one.
                            Globals.history_position++;
                            string last_history_item = Globals.History_List[Globals.history_position].e;
                            txtMain.Text = last_history_item;
                            txtMain.Select(txtMain.Text.Length, 0);
                        }
                    }
                }
                else
                {
                    MoveSelectionUp();
                }
            }
            else if (e.KeyCode == Keys.Oemcomma)
            {
                // the user pressed the comma, which means they might be wanting to launch a keyword with a parameter after it.
                if (Popup.row_keywords[Popup.selected_match_index].Text.Length > 0 && Popup.row_descriptions[Popup.selected_match_index].Text.IndexOf("previous entry") == -1)
                {
                    txtMain.Text = Popup.row_keywords[Popup.selected_match_index].Text;
                    txtMain.SelectionStart = txtMain.Text.Length;
                }
            }
            // added the following to enable CTRL + A on 11/18/2013.
            else if (e.Control && e.KeyCode == Keys.A)
            {
                txtMain.SelectAll();
                // the SupressKeyPress command prevents the "beep" sound from occurring.
                e.SuppressKeyPress = true;
            }
            else if (e.Control && e.KeyCode == Keys.Back)
            {
                // this is used for Ctrl + Backspace.
                e.SuppressKeyPress = true;
                // if a space is on the end, we want to remove it. Otherwise CTRL + backspace won't do anything here.
                txtMain.Text = txtMain.Text.TrimEnd();
                if (txtMain.Text.Split(' ').Count() > 1)
                {
                    //remoce last word form list and put it back together (gotta love lambda)
                    txtMain.Text = txtMain.Text.Split(' ').Take(txtMain.Text.Split(' ').Count() - 1).Aggregate((a, b) => a + " " + b) + " ";
                    //set selection at the end
                    txtMain.SelectionStart = txtMain.Text.Length;
                }
                else if (txtMain.Text.Split(' ').Count() == 1)
                {
                    txtMain.Text = "";
                }
            }
            else if (e.KeyCode == Keys.ShiftKey)
            {
                user_pressed_shift = true;
            }
            else if (e.KeyCode == Keys.ControlKey)
            {
                user_pressed_ctrl = true;
            }
        }

        protected override bool ProcessCmdKey(ref Message msg, Keys keyData)
        {
            // this function runs when the user presses Ctrl, Tab, etc.
            if (keyData == (Keys.Control | Keys.M) && Popup.Visible == true)
            {
                // user pressed Ctrl + M to show the menu.
                ShowPopupMenu();
                return true;
            }
            // this doesn't work because it seems like you can't detect CTRL being pressed by itself. Instead, we are checking for this in the KeyUp event.
            // else if (keyData == (Keys.ControlKey) && user_pressed_nonCMD_key == false)
            else if (keyData == (Keys.Tab))
            {
                if (Globals.when_pressing_tab == 1)
                    MoveSelectionDown();
                else if (Globals.when_pressing_tab == 2)
                {
                    SearchComputer(txtMain.Text);
                    HideMainForm();
                    return true;
                }
                else if (Globals.when_pressing_tab == 3)
                {
                    SearchTheWeb(txtMain.Text);
                    HideMainForm();
                    return true;
                }
            }
            return base.ProcessCmdKey(ref msg, keyData);
        }

        public void ShowPopupMenu()
        {
            // this displays the context menu for launching, editing, or deleting an entry.
            Popup.SetupContextMenu();
            int position = Popup.selected_row_index;
            Popup.contextMenu1.Show(Popup.row_descriptions[position], 1, 1);
        }

        public void MoveSelectionDown()
        {
            int total_matches = Popup.autocomplete_match_list.Count();
            if (Popup.selected_match_index >= (total_matches - 1))
            {
                // we have our last match selected already, so we can't move down.
                return;
            }
            else if (Popup.selected_row_index < (Popup.total_rows_visible - 1))
            {
                // we are not at the last visible row yet, so we can move the selection down without scrolling down.
                Popup.HighlightRow(Popup.selected_row_index, false);
                Popup.selected_row_index++;
                Popup.selected_match_index++;
                Popup.HighlightRow(Popup.selected_row_index, true);
            }
            else
            {
                // we are moving down past our last visible row, so we have to scroll down.
                Popup.selected_row_index = Popup.total_rows_visible - 1;
                Popup.selected_match_index++;
                int new_top_row = Popup.selected_match_index - Popup.total_rows_visible + 1;
                for (int i = 0; i < Popup.total_rows_visible; i++)
                {
                    try
                    {
                        if (Globals.skin_icon_type == 1)
                            Popup.row_icons[i].Image = Popup.imageListLarge.Images[new_top_row + i];
                        else if (Globals.skin_icon_type == 2)
                        {
                            Popup.row_file_extension_labels[i].Text = Popup.autocomplete_match_list[new_top_row + i].file_ext;
                        }
                        else if (Globals.skin_icon_type == 3)
                        {
                            // to do: this still doesn't work.
                        }
                    }
                    catch { }
                    Popup.row_keywords[i].Text = Popup.autocomplete_match_list[new_top_row + i].row_name;
                    Popup.row_descriptions[i].Text = Globals.ReturnShortenedPath(Popup.autocomplete_match_list[new_top_row + i].row_target, Popup.autocomplete_match_list[new_top_row + i].row_path);
                    /*
                    if (show_paths == true)
                        Popup.row_paths[i].Text = Popup.autocomplete_match_list[new_top_row + i].row_path;
                     * */
                }
            }


            /*
            int LastPosition = Popup.popup_list_elements.Count() - 1;
            int NewTopRow = 0;
            string tempkeyword;
            string temptarget;

            if (Popup.row_selection < LastPosition)
            {
                // we are not at the last item yet, so we can still move down.
                // reset the current item's backcolor so it no longer looks highlighted.
                // then highlight the next item.
                if (Popup.row_label_selection == Popup.total_rows_visible - 1)
                {
                    // we need to programmatically "scroll" down.
                    Popup.row_selection += 1;
                    NewTopRow = Popup.row_selection - Popup.total_rows_visible + 1;
                    for (int i = 0; i < Popup.popup_list_elements.Count() - 1; i++)
                    {
                        if (Popup.popup_list_element_type[NewTopRow + i] == 2)
                        {
                            // autoindex
                            tempkeyword = Globals.autoindex_files[Popup.popup_list_elements[NewTopRow + i]];
                            temptarget = Globals.autoindex_filepaths[Popup.popup_list_elements[NewTopRow + i]];
                            try
                            {
                                if (Globals.skin_icon_type == 1)
                                    Popup.row_icons[i].Image = Popup.autocompleteImageListLarge.Images[NewTopRow + i];
                            }
                            catch { }
                            Popup.row_keywords[i].Text = tempkeyword;
                            Popup.row_descriptions[i].Text = ReturnShortenedPath(temptarget);
                            Popup.row_descriptions[i].Tag = temptarget;
                            Popup.autoindex_icon_tracker++;
                        }
                        else
                        {
                            // regular keyword
                            tempkeyword = Globals.keyword_names[Popup.popup_list_elements[NewTopRow + i]];
                            temptarget = Globals.keyword_targets[Popup.popup_list_elements[NewTopRow + i]];
                            try
                            {
                                if (Globals.skin_icon_type == 1)
                                    Popup.row_icons[i].Image = Popup.autocompleteImageListLarge.Images[NewTopRow + i];
                            }
                            catch { }
                            Popup.row_keywords[i].Text = tempkeyword;
                            Popup.row_descriptions[i].Text = ReturnShortenedPath(temptarget);
                            Popup.row_descriptions[i].Tag = temptarget;
                        }
                        if (i == Popup.total_rows_visible - 1)
                            break;
                    }
                }
                else
                {
                    Popup.HighlightRow(Popup.row_label_selection, false);
                    Popup.row_selection += 1;
                    Popup.row_label_selection += 1;
                    Popup.HighlightRow(Popup.row_label_selection, true);
                }
            }
             */
        }

        public void MoveSelectionUp()
        {
            int total_matches = Popup.autocomplete_match_list.Count();
            if (Popup.selected_match_index == 0)
            {
                // we are at the top, so we can't move up.
                return;
            }
            else if (Popup.selected_row_index > 0)
            {
                // we are not at the first visible row yet, so we can move the selection up without scrolling up.
                Popup.HighlightRow(Popup.selected_row_index, false);
                Popup.selected_row_index--;
                Popup.selected_match_index--;
                Popup.HighlightRow(Popup.selected_row_index, true);
            }
            else
            {
                // we are moving up past our first visible row, so we have to scroll up.
                Popup.selected_row_index = 0;
                Popup.selected_match_index--;
                int new_top_row = Popup.selected_match_index;
                for (int i = 0; i < Popup.total_rows_visible; i++)
                {
                    try
                    {
                        if (Globals.skin_icon_type == 1)
                            Popup.row_icons[i].Image = Popup.imageListLarge.Images[new_top_row + i];
                    }
                    catch { }
                    Popup.row_keywords[i].Text = Popup.autocomplete_match_list[new_top_row + i].row_name;
                    Popup.row_descriptions[i].Text = Globals.ReturnShortenedPath(Popup.autocomplete_match_list[new_top_row + i].row_target, Popup.autocomplete_match_list[new_top_row + i].row_path);
                    /*
                    if (show_paths == true)
                        Popup.row_paths[i].Text = Popup.autocomplete_match_list[new_top_row + i].row_path;
                     */
                }
            }

            /*
            string tempkeyword;
            string temptarget;

            if (Popup.row_selection > 0)
            {
                // we are not at the first item yet, so we can still move up.
                // reset the current item's backcolor so it no longer looks highlighted.
                // then highlight the previous item.
                if (Popup.row_label_selection == 0)
                {
                    // we are near the bottom of the list and currently selecting the topmost item,
                    // so we need to "scroll" up.
                    Popup.row_selection = Popup.row_selection - 1;
                    for (int i = 0; i < Popup.popup_list_elements.Count() - 1; i++)
                    {
                        if (Popup.popup_list_element_type[Popup.row_selection + i] == 2)
                        {
                            // autoindex
                            tempkeyword = Globals.autoindex_files[Popup.popup_list_elements[Popup.row_selection + i]];
                            temptarget = Globals.autoindex_filepaths[Popup.popup_list_elements[Popup.row_selection + i]];
                            try
                            {
                                if (Globals.skin_icon_type == 1)
                                    Popup.row_icons[i].Image = Popup.autocompleteImageListLarge.Images[Popup.row_selection + i];
                            }
                            catch { }
                            Popup.row_keywords[i].Text = tempkeyword;
                            Popup.row_descriptions[i].Text = ReturnShortenedPath(temptarget);
                            Popup.row_descriptions[i].Tag = temptarget;
                            Popup.autoindex_icon_tracker++;
                            if (i == Popup.total_rows_visible - 1)
                                break;
                        }
                        else
                        {
                            tempkeyword = Globals.keyword_names[Popup.popup_list_elements[Popup.row_selection + i]];
                            temptarget = Globals.keyword_targets[Popup.popup_list_elements[Popup.row_selection + i]];
                            try
                            {
                                if (Globals.skin_icon_type == 1)
                                    Popup.row_icons[i].Image = Popup.autocompleteImageListLarge.Images[Popup.row_selection + i];
                            }
                            catch { }
                            Popup.row_keywords[i].Text = tempkeyword;
                            Popup.row_descriptions[i].Text = ReturnShortenedPath(temptarget);
                            Popup.row_descriptions[i].Tag = temptarget;
                            if (i == Popup.total_rows_visible - 1)
                                break;
                        }
                    }
                }
                else
                {
                    Popup.HighlightRow(Popup.row_label_selection, false);
                    Popup.row_selection = Popup.row_selection - 1;
                    Popup.row_label_selection = Popup.row_label_selection - 1;
                    Popup.HighlightRow(Popup.row_label_selection, true);
                }
            }
             */
        }

        private void txtMain_KeyUp(object sender, KeyEventArgs e)
        {
            if (this.Visible == false)
                return;
            // we don't really want the cursor to go back each time the user hits up.
            if (e.KeyCode == Keys.Up)
            {
                txtMain.SelectionStart = txtMain.Text.Length;
            }
            if (txtMain.Text.Length < 2)
            {
                // see if the user pressed Space for QuickTab.
                if (e.KeyCode == Keys.Space)
                {
                    // check quicktab.
                    QuickTab_ControlWindow("hotkey_plus_space");
                }
            }
            else if (txtMain.Text.Length > 0)
            {
                // see if the user pressed Shift without pressing any other keys.
                if (e.KeyCode == Keys.ShiftKey && user_pressed_shift == true)
                {
                    if (Globals.when_pressing_shift == 1)
                        MoveSelectionDown();
                    else if (Globals.when_pressing_shift == 2)
                    {
                        SearchComputer(txtMain.Text);
                        HideMainForm();
                        return;
                    }
                    else if (Globals.when_pressing_shift == 3)
                    {
                        SearchTheWeb(txtMain.Text);
                        HideMainForm();
                        return;
                    }
                }
            }
            if (initial_key_pressed == false)
            {
                // we only want to look at quicktab if the user has not typed any other keys yet.
                if (e.KeyCode == Keys.ShiftKey)
                {
                    // check quicktab.
                    QuickTab_ControlWindow("hotkey_plus_shift");
                }
                else if (e.KeyCode == Keys.Enter)
                {
                    // check quicktab.
                    if (txtMain.Text.Length == 0)
                    {
                        QuickTab_ControlWindow("hotkey_plus_enter");
                    }
                }
                else if (e.KeyCode == Keys.ControlKey)
                {
                    // user pressed CTRL without pressing any other keys.
                    // check quicktab.
                    if (txtMain.Text.Length == 0)
                    {
                        QuickTab_ControlWindow("hotkey_plus_ctrl");
                    }
                }
                else if (e.KeyCode == Keys.Tab)
                {
                    // check quicktab.
                    if (txtMain.Text.Length == 0)
                    {
                        QuickTab_ControlWindow("hotkey_plus_tab");
                    }
                }
            }
            else if (e.KeyCode == Keys.ControlKey && user_pressed_ctrl == true)
            {
                // the user pressed and released CTRL after typing a few entries. Let's see if we should do anything.
                if (Globals.when_pressing_ctrl == 1)
                    MoveSelectionDown();
                else if (Globals.when_pressing_ctrl == 2)
                {
                    SearchComputer(txtMain.Text);
                    HideMainForm();
                    return;
                }
                else if (Globals.when_pressing_ctrl == 3)
                {
                    SearchTheWeb(txtMain.Text);
                    HideMainForm();
                    return;
                }
            }
            if (e.KeyCode == Keys.F1)
                QuickTab_ControlWindow("hotkey_plus_f1");
            if (e.KeyCode == Keys.F2)
                QuickTab_ControlWindow("hotkey_plus_f2");
            if (e.KeyCode == Keys.F3)
                QuickTab_ControlWindow("hotkey_plus_f3");
            if (e.KeyCode == Keys.F4)
                QuickTab_ControlWindow("hotkey_plus_f4");
            if (e.KeyCode == Keys.F5)
                QuickTab_ControlWindow("hotkey_plus_f5");
            if (e.KeyCode == Keys.F6)
                QuickTab_ControlWindow("hotkey_plus_f6");
            if (e.KeyCode == Keys.F7)
                QuickTab_ControlWindow("hotkey_plus_f7");
            if (e.KeyCode == Keys.F8)
                QuickTab_ControlWindow("hotkey_plus_f8");
            if (e.KeyCode == Keys.F9)
                QuickTab_ControlWindow("hotkey_plus_f9");
            if (e.KeyCode == Keys.F10)
                QuickTab_ControlWindow("hotkey_plus_f10");
            if (e.KeyCode == Keys.F11)
                QuickTab_ControlWindow("hotkey_plus_f11");
            if (e.KeyCode == Keys.F12)
                QuickTab_ControlWindow("hotkey_plus_f12");
        }

        public void SearchComputer(string TheText, string folder = "")
        {
            frmSearch frmSearch_instance = new frmSearch(this);
            frmSearch_instance.Show();
            frmSearch_instance.Search_Start(TheText, folder);
        }

        public void SearchTheWeb(string TheText)
        {
            // search the web.
            string search_url = "";
            string search_url_parameters;
            int returnval;
            search_url = Globals.ReadINI("options", "search_url", Globals.OptionsINI);
            if (search_url.Length == 0)
            {
                search_url = "http://www.google.com/search?hl=en&q=%keybreeze_entry%";
                Globals.WriteINI("options", "search_url", search_url, Globals.OptionsINI);
            }
            if (search_url.IndexOf("%keybreeze_entry%") == -1)
            {
                // add %keybreeze_entry% to the end.
                search_url = search_url + "%keybreeze_entry%";
                Globals.WriteINI("options", "search_url", search_url, Globals.OptionsINI);
            }
            search_url = search_url.ToLower().Replace("%keybreeze_entry%", TheText);
            returnval = RunURL(search_url);
            if (returnval == -1)
            {
                MessageBox.Show("Keybreeze Desktop is unable to find a web browser on your system. It appears that either you do not have a browser installed, or you have not selected a default browser in Windows. Please install Internet Explorer, Firefox, Chrome, or another browser and set it as the default Windows browser. Then try again.", "Keybreeze Desktop");
            }
        }

        public int RunTarget(string TheTarget, string command_line_parameters = "", string keyword = "", string the_path = "")
        {
            // did the user include % in the path? If so, it's probably an environment variable.
            if (TheTarget.IndexOf("%") != -1)
            {
                TheTarget = Environment.ExpandEnvironmentVariables(TheTarget);
            }
            string target_type;
            int returnval;
            target_type = Globals.GetTargetType(TheTarget);
            switch (target_type)
            {
                case "kb":
                    // preset
                    string preset_keyword;
                    preset_keyword = TheTarget.Substring(7);
                    returnval = RunPresetKeyword(preset_keyword);
                    if (returnval == -1) return -1;
                    break;
                case "web":
                    // website
                    returnval = RunURL(TheTarget);
                    if (returnval == -1) return -1;
                    break;
                case "text":
                    // text function
                    string text_function;
                    text_function = TheTarget.Substring(9);
                    returnval = RunTextFunction(text_function, the_path);
                    if (returnval == -1) return -1;
                    break;
                case "exe":
                    // file with or without command line switches
                    returnval = RunFile(TheTarget, command_line_parameters);
                    if (returnval == -1) return -1;
                    break;
                case "dir":
                    // folder
                    if (command_line_parameters.Length > 0)
                    {
                        // the user typed a comma and a search string after the folder. Let's search for the string within that folder.
                        SearchComputer(command_line_parameters, TheTarget);
                    }
                    else
                    {
                        returnval = RunFolder(TheTarget);
                        if (returnval == -1) return -1;
                    }
                    break;
                case "unc":
                    // network location
                    returnval = RunFolder(TheTarget);
                    if (returnval == -1) return -1;
                    break;
                case "unc_folder":
                    // network location
                    returnval = RunFolder(TheTarget);
                    if (returnval == -1) return -1;
                    break;
                case "unc_file":
                    returnval = RunFile(TheTarget, command_line_parameters);
                    if (returnval == -1) return -1;
                    break;
                case "err":
                    // file or folder does not exist
                    MessageBox.Show("Path does not exist -- " + TheTarget + ". You can edit the path by right-clicking the keyword and clicking Edit.", "Keybreeze Desktop");
                    break;
                case "mcr":
                    // macro
                    frmMacroPlayback frmMacroPlayback_instance;
                    frmMacroPlayback_instance = new frmMacroPlayback(this);
                    frmMacroPlayback_instance.macro_file_name = keyword + ".kmcr";
                    frmMacroPlayback_instance.Show();
                    break;
                case "remind":
                    // speed reminder
                    string remind_time;
                    remind_time = TheTarget.Substring(7);
                    frmSpeedReminder frmSpeedReminder_instance;
                    frmSpeedReminder_instance = new frmSpeedReminder(this);
                    frmSpeedReminder_instance.Set_Up_Form(remind_time);
                    frmSpeedReminder_instance.Show();
                    break;
                default:
                    // file with any extension.
                    returnval = RunFile(TheTarget, command_line_parameters);
                    if (returnval == -1) return -1;
                    break;
            }
            return 1;
        }

        public int RunURL(string TheURL)
        {
            // add http:// if it's not there and https:// or ftp:// or anything else 
            // is not being used.
            if (TheURL.IndexOf("://") == -1)
            {
                TheURL = "http://" + TheURL;
            }
            try
            {
                if (custom_browser_path.Length > 0)
                {
                    ProcessStartInfo startInfo = new ProcessStartInfo(custom_browser_path);
                    startInfo.Arguments = custom_browser_prefix + "\"" + TheURL + "\"" + custom_browser_suffix;
                    Process.Start(startInfo);
                }
                else
                {
                    Uri myUri = new Uri(TheURL);
                    System.Diagnostics.Process.Start(myUri.AbsoluteUri);
                }
            }
            catch (System.ComponentModel.Win32Exception noBrowser)
            {
                if (noBrowser.ErrorCode == -2147467259)
                    return -1;
            }
            catch (System.Exception)
            {
                return -1;
            }
            return 1;
        }

        public int RunFile(string filepath, string optional_command_line_parameters = "")
        {
            int FindEXE;
            
            // look for command line switches
            FindEXE = filepath.ToLower().IndexOf(".exe");
            if (FindEXE == -1) FindEXE = filepath.ToLower().LastIndexOf(".lnk");
            if (FindEXE == -1) FindEXE = filepath.ToLower().LastIndexOf(".bat");
            if (FindEXE > -1)
            {
                // it's an .exe or .lnk file. let's check for command line parameters.
                if (FindEXE < (filepath.Length - 4))
                {
                    // there are characters present after .exe or .lnk, most likely switches/parameters.
                    // we need to separate the two.
                    string exe_file;
                    string command_line_switches;
                    exe_file = filepath.Substring(0, FindEXE + 5).Trim();
                    command_line_switches = filepath.Substring(FindEXE + 5).Trim();
                    // make the sure the exe file exists. Remove any surrounding quotes because for some reason File.Exists returns false if it has quotes. http://stackoverflow.com/questions/3315983/illegal-character-in-path.
                    exe_file = exe_file.Replace("\"", "");
                    if (File.Exists(exe_file))
                    {
                        // check for keybreeze-specific command line switches (minimized, maximized).
                        if (command_line_switches.IndexOf("iwstate1") > -1)
                        {
                            // run it minimized, but see if there are any other switches
                            command_line_switches = command_line_switches.Replace("iwstate1", "");
                            ProcessStartInfo startInfo = new ProcessStartInfo(exe_file);
                            startInfo.WindowStyle = ProcessWindowStyle.Minimized;
                            startInfo.Arguments = command_line_switches + optional_command_line_parameters;
                            Process.Start(startInfo);
                            return 1;
                        }
                        else if (command_line_switches.IndexOf("xwstate1") > -1)
                        {
                            // run it maximized, but see if there are any other switches
                            command_line_switches = command_line_switches.Replace("xwstate1", "");
                            ProcessStartInfo startInfo = new ProcessStartInfo(exe_file);
                            startInfo.WindowStyle = ProcessWindowStyle.Maximized;
                            startInfo.Arguments = command_line_switches + optional_command_line_parameters;
                            Process.Start(startInfo);
                            return 1;
                        }
                        if (optional_command_line_parameters.Length > 0)
                        {
                            // if we are running cmd.exe, and we already have a parameter after it, we don't want to pass them as "arguments". We have to include them directly in the full filepath.
                            // Example: "c:\\windows\\system32\\cmd.exe /k c:\\Err\\Err.exe 80004005"
                            if (filepath.ToLower().IndexOf("cmd.exe") != -1)
                            {
                                System.Diagnostics.Process.Start(exe_file, command_line_switches + " " + optional_command_line_parameters);
                                filepath = filepath + " " + optional_command_line_parameters;
                            }
                            else
                            {
                                ProcessStartInfo startInfo = new ProcessStartInfo(exe_file);
                                startInfo.Arguments = optional_command_line_parameters;
                                Process.Start(startInfo);
                            }
                        }
                        else
                        {
                            System.Diagnostics.Process.Start(exe_file, command_line_switches);
                        }
                        return 1;
                    }
                    else
                    {
                        return -1;
                    }
                }
            }
            if (File.Exists(filepath))
            {
                if (optional_command_line_parameters.Length > 0)
                {
                    try
                    {
                        ProcessStartInfo startInfo = new ProcessStartInfo(filepath);
                        startInfo.Arguments = optional_command_line_parameters;
                        Process.Start(startInfo);
                    }
                    catch(Exception e)
                    {
                        MessageBox.Show("Could not launch " + filepath + ". " + e.Message, "Keybreeze Desktop");
                    }
                }
                else
                {
                    try
                    {
                        // if it's an lnk file, we need to find the target to fix the issue with File Redirection in Windows.
                        if (filepath.IndexOf(".lnk") > 1)
                            filepath = Globals.ResolveShortcut(filepath);
                        System.Diagnostics.Process.Start(filepath);
                    }
                    catch(Exception e)
                    {
                        MessageBox.Show("Could not launch " + filepath + ". " + e.Message, "Keybreeze Desktop");
                    }
                }
            }
            else
            {
                return -1;
            }
            return 1;
        }

        public int RunFolder(string filepath)
        {
            if (custom_explorer_path.Length > 0)
            {
                try
                {
                    string prefix = Globals.ReadINI("options", "custom_explorer_prefix", Globals.OptionsINI);
                    string suffix = Globals.ReadINI("options", "custom_explorer_suffix", Globals.OptionsINI);
                    RunFile(custom_explorer_path + " " + prefix + "\" " + filepath + "\" " + suffix);
                }
                catch (Exception e)
                {
                    return -1;
                }
            }
            else
            {
                if (Directory.Exists(filepath) || File.Exists(filepath))
                {
                    Thread t = new Thread(new ThreadStart(delegate()
                    {
                        System.Diagnostics.Process.Start(filepath);
                    }));
                    try
                    {
                        t.Start();
                    }
                    catch { }
                }
                else
                {
                    return -1;
                }
            }
            return 1;
        }

        public int RunLetterCommand(string TheText)
        {
            int find_space;
            int find_colon;

            // we need to see if the text is a letter command first.
            // search for a colon or a space.

            find_space = TheText.IndexOf(" ");
            find_colon = TheText.IndexOf(":");
            if (find_space > 0 || find_colon > 0)
            {
                // a space or a colon exists, so it might be a letter command. check the letter command class.
                // if the user typed a colon instead of a space, let's just reset the find_space variable
                // so that it represents the location of the colon (since this is where the letter command ends) and the entry starts.
                if (find_colon > 0 && find_space > find_colon) find_space = find_colon;
                string first_part;
                string beginning_url;
                
                // if the user typed :/ or :\, they probably are typing a path. In that case, exit.
                if (TheText.IndexOf(":/") > 0 || TheText.IndexOf(":\\") > 0)
                    return -1;

                // get the letter or word before the space
                first_part = TheText.Substring(0, find_space);
                first_part = first_part.ToLower().Trim();
                string search_query = TheText.Substring(find_space + 1);
                // look up the letter or word to see if the letter command exists.
                foreach (Letter_Command this_letter_command in Globals.Letter_Commands)
                {
                    if (this_letter_command.name == first_part)
                    {
                        string url_target = this_letter_command.url;
                        // see if any characters should be replaced.
                        if (this_letter_command.replace_this.Count > 0)
                        {
                            int replacement_count = this_letter_command.replace_this.Count();
                            for (int i = 0; i < replacement_count; i++)
                            {
                                string replace_this = this_letter_command.replace_this[i];
                                string replace_with = this_letter_command.replace_with[i];
                                search_query = search_query.Replace(replace_this, replace_with);
                            }
                        }
                        url_target = url_target.Replace("%keybreeze_entry%", search_query); 
                        RunURL(url_target);
                        return 1;
                    }
                }
            }
            return 0;
        }

        public void ChangeKeyword(string keyword, string the_path, bool command_list_showing = false)
        {
            string the_target;
            int find_minimized = 0;
            int find_maximized = 0;
            string full_text;
            if (the_path == null)
                the_path = "";

            the_target = Globals.Keyword_List[GetListPositionFromKeyword(keyword, the_path)].target;
            if (the_target == "macro")
            {
                HideMainForm();
                if (File.Exists(Globals.InstallDir + "\\Keybreeze Macro Recorder.exe") == false)
                {
                    DialogResult result;
                    result = MessageBox.Show("Keybreeze Macro Recorder.exe could not be found on your computer. You may need to download it from the Keybreeze website. Do you want to go to the Downloads page now?", "Keybreeze Desktop", MessageBoxButtons.YesNo);
                    if (result == DialogResult.Yes)
                    {
                        RunURL("http://www.keybreeze.com/download");
                    }
                }
                else
                {
                    RunFile(Globals.InstallDir + "\\Keybreeze Macro Recorder.exe", Globals.WorkPath + "\\macros\\" + keyword + ".kmcr");
                }
            }
            else
            {

                frmAdd frmAdd_instance;
                frmAdd_instance = new frmAdd(this);
                // are we editing a text function?
                if (the_target.Length > 9 && the_target.Substring(0, 9) == "function:")
                {
                    string file_name = the_target.Substring(9);
                    file_name = Globals.replace_invalid_filename_characters(file_name, the_path);
                    string file_path = Globals.WorkPath + "\\text functions\\" + file_name + ".txt";
                    try
                    {
                        full_text = File.ReadAllText(file_path);
                    }
                    catch
                    {
                        MessageBox.Show("The file that contains the text for this text function, " + file_path + ", could not be found, or you may not have permission to view it in Windows. Keybreeze Desktop will provide you with a blank text field so you can re-type this data if you wish, or you can cancel out of the window and check your Windows permissions first.", "Keybreeze Desktop");
                        full_text = "";
                        try
                        {
                            System.IO.File.WriteAllText(file_path, full_text, Encoding.Unicode);
                        }
                        catch
                        {
                        }
                    }
                    frmAdd_instance.changing_entry = 2;
                    frmAdd_instance.Text = "Edit keyword";
                    frmAdd_instance.cmdAdd.Text = "Change";
                    frmAdd_instance.txtKeyword2.Text = keyword;
                    frmAdd_instance.txtKeyword2.Tag = keyword;
                    frmAdd_instance.txtFolder.Text = the_path;
                    foreach (Keyword_Folder this_folder in Globals.Keyword_Folders)
                    {
                        if (this_folder.path == the_path)
                        {
                            frmAdd_instance.selected_folder_guid = this_folder.GUID;
                            break;
                        }
                    }
                    frmAdd_instance.txtFolder.Tag = the_path;
                    frmAdd_instance.txtTextFunction.Text = full_text;
                    frmAdd_instance.txtTextFunction.Tag = full_text;
                    frmAdd_instance.checkBox1.Visible = false;
                }
                else
                {
                    frmAdd_instance.changing_entry = 1;
                    frmAdd_instance.Text = "Edit keyword";
                    frmAdd_instance.cmdAdd.Text = "Change";
                    frmAdd_instance.txtKeyword.Text = keyword;
                    frmAdd_instance.txtKeyword.Tag = keyword;
                    frmAdd_instance.txtFolder.Text = the_path;
                    foreach (Keyword_Folder this_folder in Globals.Keyword_Folders)
                    {
                        if (this_folder.path == the_path)
                        {
                            frmAdd_instance.selected_folder_guid = this_folder.GUID;
                            break;
                        }
                    }
                    frmAdd_instance.txtFolder.Tag = the_path;
                    // remove bullet point (•) if one exists. These were used in prior versions of Keybreeze Desktop to mark parameters.
                    // now we just use spaces.
                    if (the_target.IndexOf(".exe•") > -1)
                        the_target = the_target.Replace(".exe•", ".exe ");
                    if (the_target.IndexOf(".exe�") > -1)
                        the_target = the_target.Replace(".exe�", ".exe ");

                    frmAdd_instance.comboBox1.Items.Add("Normal window");
                    frmAdd_instance.comboBox1.Items.Add("Minimized");
                    frmAdd_instance.comboBox1.Items.Add("Maximized");
                    // see if we are using any parameters to launch it minimized or maximized.
                    find_minimized = the_target.IndexOf("iwstate1");
                    find_maximized = the_target.IndexOf("xwstate1");
                    if (find_minimized > -1)
                    {
                        the_target = the_target.Replace("iwstate1", "");
                        frmAdd_instance.comboBox1.SelectedIndex = 1;
                        frmAdd_instance.comboBox1.Tag = 1;
                    }
                    else if (find_maximized > -1)
                    {
                        the_target = the_target.Replace("xwstate1", "");
                        frmAdd_instance.comboBox1.SelectedIndex = 2;
                        frmAdd_instance.comboBox1.Tag = 2;
                    }
                    else
                    {
                        frmAdd_instance.comboBox1.SelectedIndex = 0;
                        frmAdd_instance.comboBox1.Tag = 0;
                    }

                    frmAdd_instance.txtTarget.Text = the_target;
                    frmAdd_instance.txtTarget.Tag = the_target;
                    frmAdd_instance.checkBox1.Visible = false;
                }

                if (command_list_showing == true)
                    frmAdd_instance.command_list_showing = true;

                frmAdd_instance.Show();
                HideMainForm();
            }
        }

        public void ShowAddKeywordForm(bool command_list_showing, string predefined_keyword = "", string predefined_target = "")
        {
            frmAdd frmAdd_instance;
            frmAdd_instance = new frmAdd(this);
            if (command_list_showing == true)
                frmAdd_instance.command_list_showing = true;
            frmAdd_instance.txtKeyword.Text = predefined_keyword;
            frmAdd_instance.txtTarget.Text = predefined_target;
            frmAdd_instance.Show();
        }

        public int RunPresetKeyword(string keyword)
        {
            string tempstring;
            bool retval;
            int returnval;
            switch (keyword)
            {
                case "add":
                    // add a keyword
                    ShowAddKeywordForm(false);
                    return 1;
                case "application data":
                    // open application data directory.
                    try
                    {
                        RunFolder(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData));
                    }
                    catch (Exception e)
                    {
                        MessageBox.Show("An error occurred while opening the application data directory. " + e.Message, "Keybreeze Desktop");
                    }
                    return 1;
                case "clear clipboard":
                    try
                    {
                        Clipboard.Clear();
                        if (keybreeze_clipboard_clear_history == true)
                        {
                            keybreeze_clipboard.Clear();
                        }
                    }
                    catch
                    {
                    }
                    return 1;
                case "clear formatting":
                    string clipboard_text;
                    clipboard_text = Clipboard.GetText();
                    try
                    {
                        Clipboard.SetText(clipboard_text);
                    }
                    catch
                    {
                    }
                    return 1;
                case "control panel":
                    tempstring = Environment.GetFolderPath(Environment.SpecialFolder.System) + "\\control.exe";
                    returnval = RunFile(tempstring);
                    return 1;
                case "exit":
                    ExitApplication(true);
                    return 1;
                case "help":
                    RunURL("http://www.keybreeze.com/support");
                    return 1;
                case "hibernate":
                    retval = Application.SetSuspendState(PowerState.Hibernate, false, false);
                    if (retval == false)
                        return -3;
                    else
                        return 1;
                case "hide notes":
                    Globals.notes_hide_all_notes();
                    return 1;
                case "letter commands":
                    // this is a legacy preset keyword (it should only be in the list if users updated from an early version).
                    ShowAddKeywordForm(false);
                    return 1;
                case "list":
                    // open the command list
                    frmCommandList_instance = new frmCommandList(this);
                    frmCommandList_instance.Show();
                    return 1;
                case "export entries":
                    frmExport frmExport_instance = new frmExport();
                    frmExport_instance.Show();
                    return 1;
                case "import entries":
                    frmImport frmImport_instance = new frmImport();
                    frmImport_instance.Show();
                    return 1;
                case "downloads folder":
                    Process.Start("shell:Downloads");
                    return 1;
                case "desktop folder":
                    RunFolder(Environment.GetFolderPath(Environment.SpecialFolder.Desktop));
                    return 1;
                case "log off":
                    int force_option = Globals.ReadIntFromINI("options", "force_shutdown_restart_logoff", Globals.OptionsINI, 0);
                    if (force_option == 0)
                        Process.Start("shutdown", "/l /t 0");
                    else
                        Process.Start("shutdown", "/l /f /t 0");
                    Application.Exit();
                    return 1;
                case "macros [create]":
                    if (File.Exists(Globals.InstallDir + "\\Keybreeze Macro Recorder.exe") == false)
                    {
                        DialogResult result;
                        result = MessageBox.Show("Keybreeze Macro Recorder.exe could not be found on your computer. You may need to download it from the Keybreeze website. Do you want to go to the Downloads page now?", "Keybreeze Desktop", MessageBoxButtons.YesNo);
                        if (result == DialogResult.Yes)
                        {
                            RunURL("http://www.keybreeze.com/download");
                        }
                        return 1;
                    }
                    System.Diagnostics.Process.Start(Globals.InstallDir + "\\Keybreeze Macro Recorder.exe");
                    return 1;
                case "make window translucent [function]":
                    HideMainForm();
                    int wait_time = Globals.ReadIntFromINI("options", "wait_time_before_inserting_text", Globals.OptionsINI, 250);
                    System.Threading.Thread.Sleep(wait_time);
                    IntPtr Handle = GetForegroundWindow();
                    frmInputBox frmInputBox_instance;
                    frmInputBox_instance = new frmInputBox();
                    frmInputBox_instance.action = "make window translucent";
                    frmInputBox_instance.Text = "How much translucency?";
                    frmInputBox_instance.lblText.Text = "How much translucency do you want this window to have? Please enter a percentage.";
                    frmInputBox_instance.sample_IntPtr = Handle;
                    frmInputBox_instance.Show();
                    return 1;
                case "menu":
                    // bring up the context menu
                    contextMenuStrip1.Show(this, new System.Drawing.Point(20, 20));
                    return 1;
                case "msconfig":
                    tempstring = Environment.GetFolderPath(Environment.SpecialFolder.System) + "\\msconfig.exe";
                    returnval = RunFile(tempstring);
                    return 1;
                case "my computer":
                    System.Diagnostics.Process.Start("explorer", "::{20D04FE0-3AEA-1069-A2D8-08002B30309D}");
                    return 1;
                case "my documents":
                    tempstring = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
                    returnval = RunFolder(tempstring);
                    return 1;
                case "my music":
                    tempstring = Environment.GetFolderPath(Environment.SpecialFolder.MyMusic);
                    returnval = RunFolder(tempstring);
                    return 1;
                case "my pictures":
                    tempstring = Environment.GetFolderPath(Environment.SpecialFolder.MyPictures);
                    returnval = RunFolder(tempstring);
                    return 1;
                case "note":
                    // create a sticky note
                    Globals.notes_create_note(this);
                    just_created_a_note = true;
                    return 1;
                case "options":
                    // open the Options window
                    ShowOptionsWindow();
                    return 1;
                case "reboot":
                    int force_option2 = Globals.ReadIntFromINI("options", "force_shutdown_restart_logoff", Globals.OptionsINI, 0);
                    if (force_option2 == 0)
                        Process.Start("shutdown", "/r /t 0");
                    else
                        Process.Start("shutdown", "/r /f /t 0");
                    Application.Exit();
                    return 1;
                case "regedit":
                    tempstring = Environment.GetFolderPath(Environment.SpecialFolder.System) + "\\regedt32.exe";
                    returnval = RunFile(tempstring);
                    return 1;
                case "reminder [note]":
                    // launch a reminder note
                    Globals.notes_open_reminder_window();
                    return 1;
                case "remove formatting":
                    // remove formatting from the clipboard text. And trim it.
                    clipboard_text = Clipboard.GetText().Trim();
                    try
                    {
                        Clipboard.SetText(clipboard_text);
                    }
                    catch
                    {
                    }
                    return 1;
                case "restart":
                    int force_option4 = Globals.ReadIntFromINI("options", "force_shutdown_restart_logoff", Globals.OptionsINI, 0);
                    if (force_option4 == 0)
                        Process.Start("shutdown", "/r /t 0");
                    else
                        Process.Start("shutdown", "/r /f /t 0");
                    Application.Exit();
                    return 1;
                case "screen saver":
                    HideMainForm();
                    IntPtr lDesktopWindow = GetDesktopWindow();
                    SendMessage(lDesktopWindow, WM_SYSCOMMAND, SC_SCREENSAVE, 0);
                    return 1;
                case "set window on top [function]":
                    HideMainForm();
                    int wait_time3 = Globals.ReadIntFromINI("options", "wait_time_before_inserting_text", Globals.OptionsINI, 250);
                    System.Threading.Thread.Sleep(wait_time3);
                    IntPtr Handle3 = GetForegroundWindow();
                    Globals.SetWindowOnTop(Handle3);
                    return 1;
                case "show hidden notes":
                    Globals.notes_show_hidden_notes();
                    return 1;
                case "shut down":
                    int force_option3 = Globals.ReadIntFromINI("options", "force_shutdown_restart_logoff", Globals.OptionsINI, 0);
                    if (force_option3 == 0)
                        Process.Start("shutdown", "/s /t 0");
                    else
                        Process.Start("shutdown", "/s /f /t 0");
                    Application.Exit();
                    return 1;
                case "standby":
                    retval = Application.SetSuspendState(PowerState.Suspend, false, false);
                    if (retval == false)
                        return -3;
                    else
                        return 1;
                case "undo make window translucent [function]":
                    // make the currently active window opaque.
                    HideMainForm();
                    int wait_time2 = Globals.ReadIntFromINI("options", "wait_time_before_inserting_text", Globals.OptionsINI, 250);
                    System.Threading.Thread.Sleep(wait_time2);
                    IntPtr Handle2 = GetForegroundWindow();
                    Globals.MakeWindowTranslucent(Handle2, 255, true);
                    return 1;
                case "undo set window on top [function]":
                    // we need to use the inputbox for this because hiding the form will set focus to a different window.
                    HideMainForm();
                    frmInputBox frmInputBox_instance2;
                    frmInputBox_instance2 = new frmInputBox();
                    frmInputBox_instance2.action = "undo set window on top";
                    frmInputBox_instance2.Text = "Identify window";
                    frmInputBox_instance2.lblText.Text = "Please enter a few letters that appear in the title bar of the window that you no longer want on top.";
                    frmInputBox_instance2.Show();
                    return 1;
                case "update index":
                    Globals.autoindex_update();
                    return 1;
                case "view reminders":
                    frmNoteReminders frmNoteReminders_instance = new frmNoteReminders();
                    frmNoteReminders_instance.Show();
                    return 1;
                case "clipboard monitor":
                    if (tmrKeybreezeClipboard.Enabled == false)
                    {
                        MessageBox.Show("The Clipboard Monitor is not currently running. Please press the hotkey and type 'options' as a keyword and check the Clipboard tab to enable it.", "Keybreeze Desktop");
                    }
                    else if (keybreeze_clipboard.Count == 0)
                    {
                        MessageBox.Show("The Clipboard Monitor is currently empty. Copy some text from other applications first. Then run this keyword again to view a history of items in your clipboard.", "Keybreeze Desktop");
                    }
                    else
                    {
                        frmKeybreezeClipboard frmKeybreezeClipboard_instance = new frmKeybreezeClipboard(this);
                        frmKeybreezeClipboard_instance.Show();
                    }
                    return 1;
                default:
                    return -2;
            }
        }

        public int ChangeTargetForKeyword(string TheKeyword, string TheTarget)
        {
            // find the position of the keyword first
            TheKeyword = TheKeyword.ToLower();
            int the_position = 0;
            bool found_it = false;
            for (int i = 0; i < Globals.keyword_count; i++)
            {
                if (Globals.keyword_name_lower[i] == TheKeyword)
                {
                    found_it = true;
                    break;
                }
                the_position++;
            }
            if (found_it == false)
            {
                return 0;
            }
            else
            {
                Globals.Keyword_List[the_position].target = TheTarget;
                Globals.Keywords_Save_To_File();
                return 1;
            }
        }

        public int RunTextFunction(string TheTarget, string ThePath)
        {
            // store the current clipboard text into a variable first.
            // then load the text function from the text file into a variable.
            string the_text = "";
            string old_clipboard_data = Clipboard.GetText();
            string filename_valid_characters;

            HideMainForm();

            filename_valid_characters = Globals.replace_invalid_filename_characters(TheTarget, ThePath);
            string text_file = Globals.WorkPath + "\\text functions\\" + filename_valid_characters + ".txt";
            try
            {
                the_text = File.ReadAllText(text_file);
            }
            catch
            {
                MessageBox.Show("The file that contains the text for this text function, " + text_file + ", could not be found, or you may not have permission to access it in Windows. Please check your Windows permissions for this directory, or edit the text function and re-type the text.", "Keybreeze Desktop");
                return -1;
            }

            running_text_function = true;
            // replace any timestamp values with the real thing.
            int timestamp_start = the_text.IndexOf("<{");
            string timestamp_input;
            string timestamp_output;
            string pretext;
            string posttext;
            int custom_sleep_time = 0;
            while (timestamp_start > -1)
            {
                int timestamp_end = the_text.IndexOf("}>");
                if (timestamp_end > timestamp_start)
                {
                    try
                    {
                        timestamp_input = the_text.Substring(timestamp_start + 2, timestamp_end - timestamp_start - 2);
                        if (timestamp_input.ToLower().IndexOf("sleep=") != -1)
                        {
                            int find_equals = timestamp_input.IndexOf("=");
                            string sleep_string = timestamp_input.Substring(find_equals + 1).Trim();
                            custom_sleep_time = Convert.ToInt32(sleep_string);
                            pretext = the_text.Substring(0, timestamp_start);
                            posttext = the_text.Substring(timestamp_end + 2);
                            the_text = pretext + posttext;
                        }
                        else
                        {
                            timestamp_output = DateTime.Now.ToString(timestamp_input);
                            pretext = the_text.Substring(0, timestamp_start);
                            posttext = the_text.Substring(timestamp_end + 2);
                            the_text = pretext + timestamp_output + posttext;
                        }
                    }
                    catch
                    {
                        break;
                    }
                }
                else
                {
                    // the timestamp has no end point. break out of the loop.
                    break;
                }
                timestamp_start = the_text.IndexOf("<{");
            }

            // sometimes setting the clipboard text will not work because it's a shared resource. see http://stackoverflow.com/questions/68666/clipbrd-e-cant-open-error-when-setting-the-clipboard-from-net.
            // we are going to try multiple times to work around this issue.
            bool copy_success = false;
            for (int i = 0; i < 10; i++)
            {
                try
                {
                    Clipboard.SetText(the_text);
                    copy_success = true;
                    break;
                }
                catch { }
                Application.DoEvents();
                Thread.Sleep(100);
            }
            if (copy_success == false)
            {
                MessageBox.Show("Keybreeze Desktop was unable to copy the text function to the clipboard and paste it in your application. This has been a known issue with the VMware Remote MKS Plugin. If you have this plugin enabled in your browser, please try closing the tab where this plugin is running, or disable the plugin, and try running this text function again.", "Keybreeze Desktop");
                return -1;
            }

            /*
            try
            {
                Clipboard.SetText(the_text);
                if (Clipboard.GetText() == old_clipboard_data && old_clipboard_data != the_text)
                {
                    // setting the clipboard text didn't work. sleep for a bit and then try again.
                    // loops were causing the app to crash, so we are trying multiple try...catch blocks.
                    Clipboard.Clear();
                    Application.DoEvents();
                    System.Threading.Thread.Sleep(250);
                    try
                    {
                        Clipboard.SetText(the_text);
                        if (Clipboard.GetText() == old_clipboard_data && old_clipboard_data != the_text)
                        {
                            // setting the clipboard text didn't work. sleep for a bit and then try again.
                            // loops were causing the app to crash, so we are trying multiple try...catch blocks.
                            Clipboard.Clear();
                            Application.DoEvents();
                            System.Threading.Thread.Sleep(250);
                            try
                            {
                                Clipboard.SetText(the_text);
                            }
                            catch
                            { }
                        }
                    }
                    catch
                    { }
                }
            }
            catch
            { }

            try
            {
                if (Clipboard.GetText() == old_clipboard_data && old_clipboard_data != the_text)
                {
                    MessageBox.Show("Keybreeze was unable to copy the text function to the clipboard and paste it in your application. This has been a known issue with the VMware Remote MKS Plugin. If you have this plugin enabled in your browser, please try closing the tab where this plugin is running, or disable the plugin, and try running this text function again.", "Keybreeze Desktop");
                    return 0;
                }
            }
            catch
            {
                MessageBox.Show("Keybreeze was unable to copy the text function to the clipboard because another application has it locked. Please try again in a few seconds.");
                return 0;
            }
             */

            int sleep_time;
            if (custom_sleep_time > 0)
                sleep_time = custom_sleep_time;
            else
                sleep_time = Globals.ReadIntFromINI("options", "wait_time_before_inserting_text", Globals.OptionsINI, 450);
            int modifier_time = Globals.ReadIntFromINI("options", "wait_time_for_modifiers", Globals.OptionsINI, 50);
            System.Threading.Thread.Sleep(sleep_time);

            // simulate CTRL + V to paste the text
            keybd_event(17, 0, 0, UIntPtr.Zero);
            System.Threading.Thread.Sleep(modifier_time);
            keybd_event(86, 0, 0, UIntPtr.Zero);
            keybd_event(86, 0, 0x2, UIntPtr.Zero);
            keybd_event(17, 0, 0x2, UIntPtr.Zero);

            // set the clipboard data back to what it was previously
            try
            {
                System.Threading.Thread.Sleep(sleep_time);
                Clipboard.SetText(old_clipboard_data);
            }
            catch
            {
                // do nothing
            }
            running_text_function = false;
            return 1;
        }

        public int ChangeKeywordInCommandList(string old_keyword, string new_keyword, string old_path, string new_path, string new_target)
        {
            int return_val;
            string old_keyword_lower = old_keyword.ToLower();
            // validate the new keyword and target first
            if (new_keyword.Length == 0 || new_target.Length == 0)
            {
                return -1;
            }

            // make sure the old keyword exists.
            bool already_exists = false;
            int the_count = 0;

            for (int i = 0; i < Globals.keyword_count; i++)
            {
                if (Globals.keyword_name_lower[i] == old_keyword_lower)
                {
                    already_exists = true;
                    break;
                }
                the_count += 1;
            }

            if (already_exists == true)
            {
                // the existing keyword exists. let's delete it first and then add the new one.
                return_val = DeleteKeywordFromCommandList(old_keyword, old_path, false);
                return_val = AddKeywordToCommandList(new_keyword, new_path, new_target, true);
                return 1;
            }
            else
                return -1;
        }

        public int DeleteKeywordFromCommandList(string the_keyword, string the_path, bool refresh_icons = true)
        {
            // remove the keyword from our list.
            string the_keyword_lower = the_keyword.ToLower();
            string the_path_lower = the_path.ToLower();
            string the_target = "";
            bool found_it = false;

            for (int i = 0; i < Globals.keyword_count; i++)
            {
                if (Globals.keyword_name_lower[i] == the_keyword_lower && Globals.Keyword_List[i].path.ToLower() == the_path_lower)
                {
                    the_target = Globals.Keyword_List[i].target;
                    Globals.Keyword_List.RemoveAt(i);
                    Globals.keyword_name_lower.RemoveAt(i);
                    found_it = true;
                    break;
                }
            }

            if (found_it == false)
                return 0;

            Globals.Keywords_Save_To_File();
            
            // was the keyword a text function? If so, delete the text function file.
            if (the_target.Length > 9 && the_target.Substring(0, 9) == "function:")
            {
                try
                {
                    string filename_based_on_keyword = Globals.replace_invalid_filename_characters(the_keyword, the_path);
                    System.IO.File.Delete(Globals.WorkPath + "\\text functions\\" + filename_based_on_keyword + ".txt");
                }
                catch
                {
                    try
                    {
                        System.IO.File.Delete(Globals.WorkPath + "\\text functions\\" + the_keyword + ".txt");
                    }
                    catch
                    {
                    }
                }
            }
            
            return 1;
        }

        public void ShowOptionsWindow(bool autoindex = false)
        {
            if (autoindex == false)
            {
                frmOptions frmOptions_instance = new frmOptions(this);
                frmOptions_instance.Show();
            }
            else
            {
                frmOptions frmOptions_instance = new frmOptions(this);
                frmOptions_instance.start_with_autoindex_tab = true;
                frmOptions_instance.Show();
            }
        }

        public int AddKeywordToCommandList(string TheKeyword, string ThePath, string TheTarget, bool replace_existing)
        {
            // validate the keyword and target first
            if (TheKeyword.Length == 0 || TheTarget.Length == 0)
            {
                return -1;
            }

            Globals.BackupCommandList();
            // see if the keyword already exists.
            string new_keyword = TheKeyword.ToLower();
            bool already_exists = false;
            int the_count = 0;

            foreach (Keyword this_keyword in Globals.Keyword_List)
            {
                if (this_keyword.name.ToLower() == new_keyword)
                {
                    already_exists = true;
                    break;
                }
                the_count++;
            }

            if (already_exists == true && replace_existing == true)
            {
                // the user wants to replace the existing keyword. Let's just update the target.
                Globals.Keyword_List[the_count].target = TheTarget;
                Globals.Keywords_Save_To_File();
                // since the target changed, we should update the icons...
                return 1;
            }
            else if (already_exists == true)
            {
                // the keyword already exists, but the user didn't specify that it should be 
                // replaced, so lets just return 2 to specify that it already exists.
                return 2;
            }

            // keyword does not exist, so lets add it.
            // we need to see where the keyword will be placed in the list. It needs to be alphabetical.
            // use the CompareTo() function to see where it might be.
            int alpha_order;
            bool added_new = false;
            the_count = 0;

            //foreach (string current_keyword in Globals.keyword_names)
            foreach (Keyword current_keyword in Globals.Keyword_List)
            {
                alpha_order = current_keyword.name.CompareTo(new_keyword);
                if (alpha_order > 0)
                {
                    // the new keyword is larger than the current keyword. Let's insert it into our class.
                    Keyword new_keyword_item = new Keyword();
                    new_keyword_item.name = TheKeyword;
                    new_keyword_item.path = ThePath;
                    new_keyword_item.target = TheTarget;
                    Globals.Keyword_List.Insert(the_count, new_keyword_item);
                    added_new = true;
                    break;
                }
                the_count += 1;
            }
            if (added_new == false)
            {
                // we never added the new keyword to the list, so it must be the last one alphabetically. Add it now.
                Keyword new_keyword_item = new Keyword();
                new_keyword_item.name = TheKeyword;
                new_keyword_item.path = ThePath;
                new_keyword_item.target = TheTarget;
                Globals.Keyword_List.Add(new_keyword_item);
                added_new = true;
            }

            // update our JSON file.
            int save_response = Globals.Keywords_Save_To_File();
            if (save_response == -1)
            {
                MessageBox.Show("Keybreeze Desktop was unable to open the file, " + Globals.KeywordsJSONFile + ", for writing. Please make sure that you have read and write permission in Windows for this directory.", "Keybreeze Desktop");
                return -1;
            }
            else
                return 1;
        }

        /*
        public int AddKeywordToCommandList(string TheKeyword, string TheTarget, bool replace_existing)
        {
            // validate the keyword and target first
            if (TheKeyword.Length == 0 || TheTarget.Length == 0)
            {
                return -1;
            }

            Globals.BackupCommandList();
            // see if the keyword already exists.
            string new_keyword = TheKeyword.ToLower();
            bool already_exists = false;
            int the_count = 0;

            foreach (Keyword this_keyword in Globals.Keyword_List)
            {
                if (this_keyword.name.ToLower() == new_keyword)
                {
                    already_exists = true;
                    break;
                }
                the_count++;
            }

            if (already_exists == true && replace_existing == true)
            {
                // the user wants to replace the existing keyword. Let's just update the target.
                Globals.Keyword_List[the_count].target = TheTarget;
                Globals.Keywords_Save_To_File();
                // since the target changed, we should update the icons...
                return 1;
            }
            else if (already_exists == true)
            {
                // the keyword already exists, but the user didn't specify that it should be 
                // replaced, so lets just return 2 to specify that it already exists.
                return 2;
            }

            // keyword does not exist, so lets add it.
            // we need to see where the keyword will be placed in the list. It needs to be alphabetical.
            // use the CompareTo() function to see where it might be.
            List<string> temp_keyword_names = new List<string>();
            List<string> temp_keyword_names_lower = new List<string>();
            List<string> temp_keyword_targets = new List<string>();
            int alpha_order;
            bool added_new = false;
            the_count = 0;

            //foreach (string current_keyword in Globals.keyword_names)
            foreach (Keyword current_keyword in Globals.Keyword_List)
            {
                alpha_order = current_keyword.name.CompareTo(new_keyword);
                if (alpha_order == 1 && added_new == false)
                {
                    // the new keyword is smaller than the current_keyword in the list.
                    // lets add it to the list. then add the current keyword to the list.
                    temp_keyword_names.Add(TheKeyword);
                    temp_keyword_names_lower.Add(TheKeyword.ToLower());
                    temp_keyword_targets.Add(TheTarget);
                    temp_keyword_names.Add(current_keyword.name);
                    temp_keyword_names_lower.Add(current_keyword.name.ToLower());
                    temp_keyword_targets.Add(current_keyword.target);
                    added_new = true;
                }
                else
                {
                    // the new keyword is larger than the current keyword. Let's make a new list and continue checking until we find where to place the new keyword.
                    temp_keyword_names.Add(current_keyword.name);
                    temp_keyword_names_lower.Add(current_keyword.name.ToLower());
                    temp_keyword_targets.Add(current_keyword.target);
                }
                the_count += 1;
            }
            if (added_new == false)
            {
                // we never added the new keyword to the list, so it must be the last one alphabetically. Add it now.
                temp_keyword_names.Add(TheKeyword);
                temp_keyword_names_lower.Add(TheKeyword.ToLower());
                temp_keyword_targets.Add(TheTarget);
                added_new = true;
            }


            // update our Keyword_List so that we have everything alphabetically.
            Globals.Keyword_List.Clear();
            Globals.keyword_name_lower.Clear();
            
            for (int i = 0; i < the_count; i++)
            {
                Keyword this_keyword = new Keyword();
                this_keyword.name = temp_keyword_names[i];
                Globals.keyword_name_lower.Add(temp_keyword_names_lower[i]);
                this_keyword.target = temp_keyword_targets[i];
                Globals.Keyword_List.Add(this_keyword);
            }

            // update our JSON file.
            int save_response = Globals.Keywords_Save_To_File();
            if (save_response == -1)
            {
                MessageBox.Show("Keybreeze Desktop was unable to open the file, " + Globals.KeywordsJSONFile + ", for writing. Please make sure that you have read and write permission in Windows for this directory.", "Keybreeze Desktop");
                return -1;
            }
            else
                return 1;
        }
         * */

        public int GetListPositionFromKeyword(string the_keyword, string the_path)
        {
            // this will take a keyword and return its position within the list of keywords.
            // it's useful for getting the target, icon, and any other information when you only have the keyword.
            the_keyword = the_keyword.ToLower();
            if (the_path == null)
                the_path = "";
            string the_path_lower = the_path.ToLower();
            //foreach (Keyword this_keyword in Globals.Keyword_List)
            for (int i = 0; i < Globals.keyword_count; i++)
            {
                if (Globals.keyword_name_lower[i] == the_keyword && Globals.Keyword_List[i].path.ToLower() == the_path_lower)
                    return i;
            }

            // if we are here, we did not find the keyword in the list. Return -1.
            return -1;
        }

        public void UpdateCommandList(string new_keyword)
        {
            frmCommandList_instance.UpdateGrid(new_keyword);
        }

        public int Does_Keyword_Exist(string the_keyword)
        {
            string the_keyword_lower = the_keyword.ToLower();
            //foreach (Keyword this_keyword in Globals.Keyword_List)
            for (int i = 0; i < Globals.keyword_count; i++)
            {
                if (Globals.keyword_name_lower[i] == the_keyword_lower)
                {
                    return 1;
                }
            }

            // if we are still here, we did not find the keyword.
            return 0;
        }

        public int RunUnknownEntry(string TheText)
        {
            int return_value;
            string target_type = Globals.GetTargetType(TheText);
            bool is_calculation = false;

            // did the user type a calculation?
            if (TheText.IndexOf("+") > -1 || TheText.IndexOf("-") > -1 || TheText.IndexOf("*") > -1 || TheText.IndexOf("/") > -1 || TheText.IndexOf("\\") > -1 || TheText.IndexOf("^") > -1)
            {
                string calculation_text = TheText.ToUpper();
                is_calculation = true;
                for (int c = 0; c < calculation_text.Length; c++)
                {
                    int ascii_code = (int)calculation_text[c];
                    if (ascii_code > 64 && ascii_code < 90)
                    {
                        is_calculation = false;
                        break;
                    }
                }
            }
            else
            {
                is_calculation = false;
            }
            if (is_calculation == true)
            {
                frmCalculator frmCalculator_instance = new frmCalculator();
                frmCalculator_instance.Calculate_Expression(TheText);
                frmCalculator_instance.Show();
                return 9;
            }

            if (target_type == "dir" || target_type == "unc" || target_type == "unc_folder")
            {
                // it's a folder or network location
                return_value = RunFolder(TheText);
                if (return_value == -1)
                {
                    MessageBox.Show("An error occurred. Are you sure the folder exists?", "Keybreeze Desktop");
                }
            }
            else if (target_type == "web" || target_type == "err")
            {
                // it might be a website, search query, or letter command.
                // check letter commands first
                return_value = RunLetterCommand(TheText);
                if (return_value != 1)
                {
                    // it is not a letter command. check other possibilities.
                    // see if there is a space
                    if (TheText.IndexOf(" ") == -1 && TheText.IndexOf(".") > -1)
                    {
                        // no spaces, and there is a dot, so it's probably a website
                        RunURL(TheText);
                    }
                    else if (TheText.IndexOf("http") == 0)
                    {
                        // it starts with http. It's a website.
                        RunURL(TheText);
                    }
                    else
                    {
                        // we have not found any matches yet, so we should search the computer, web, or show an error.
                        int when_pressing_enter_unrecognized_entry = Globals.ReadIntFromINI("options", "when_pressing_enter_unrecognized_entry", Globals.OptionsINI, 2);
                        if (when_pressing_enter_unrecognized_entry == 0)
                            MessageBox.Show("The entry you typed was not recognized.", "Keybreeze Desktop");
                        else if (when_pressing_enter_unrecognized_entry == 1)
                            SearchComputer(TheText);
                        else
                            SearchTheWeb(TheText);
                        return 10;
                    }
                }
            }
            else
            {
                // it's a file
                return_value = RunFile(TheText);
                if (return_value == -1)
                {
                    MessageBox.Show("An error occurred. Are you sure the file exists?", "Keybreeze Desktop");
                }
            }
            return 1;
        }

        private void tmrShowAutoIndexResults_Tick(object sender, EventArgs e)
        {
            // this will search the automatic index list to see if anything matches 
            // the entry the user is typing.


            if (this.Visible == false)
            {
                tmrShowAutoIndexResults.Enabled = false;
                return;
            }

            if (history_selection.Length > 0 && history_selection == txtMain.Text)
            {
                // the user is accessing an item from history. We don't want to show the autocomplete list.
                tmrShowAutoIndexResults.Enabled = false;
                return;
            }

            // make sure Done_Searching_Autoindex is actually reset to true. As of 3/30, this doesn't always reset to true, causing autoindex to never work again until you close Keybreeze Desktop and reopen it.
            if (Done_Searching_Autoindex == true)
                Search_Autoindex_List();

            tmrShowAutoIndexResults.Enabled = false;
            return;
        }

        private void Search_Autoindex_Folders_On_Demand()
        {
            // this will search files in certain folders for matches after the autoindex has finished running.
            // It's useful when searching for items that have just been added to the Desktop or Downloads folder.

            // if we already have too many items in the autocomplete list, we don't want to add more.
            if (Popup.autocomplete_match_list.Count >= Popup.autocomplete_max_matches)
            {
                return;
            }
            if (Popup.Visible == false)
            {
                popup_showing = false;
            }
            string TextString = txtMain.Text;
            if (TextString.Length < 3)
                return;
            string TextStringLower = TextString.ToLower();
            List<string> new_matches = new List<string>();
            new_matches = Globals.autoindex_on_demand(TextStringLower, autoindex_matches);
            if (new_matches.Count > 0)
                ShowPopupForm();
            foreach (string new_match in new_matches)
            {
                string filename_only = Path.GetFileNameWithoutExtension(new_match);
                int popup_return = Popup.Row_Add_Row(filename_only, new_match, "autoindex", null);
                PreviousEntry = TextString;
                if (popup_return == -1)
                    return;
            }
        }

        private void Search_Autoindex_List()
        {
            try
            {
                string TextString = txtMain.Text.ToLower();
                // added by Andrew on March 18th, 2018.
                TextString = TextString.Replace("_", " ");
                TextString = TextString.Replace("-", "");
                string entry_lowercase = "";
                int text_length = TextString.Length;
                int FindIt = 0;
                int autoindex_counter = 0;
                int result_count = 0;
                int popup_result = 0;

                List<int> partial_match_list_index = new List<int>();

                if (text_length < 2)
                    return;

                // if we already have too many items in the autocomplete list, we don't want to add more.
                if (Popup.autocomplete_match_list.Count >= Popup.autocomplete_max_matches)
                {
                    return;
                }

                partial_match_list_index.Clear();
                autoindex_matches.Clear();
                Done_Searching_Autoindex = false;
                foreach (string entry in Globals.autoindex_files)
                {
                    if (text_length <= entry.Length)
                    {
                        entry_lowercase = entry.ToLower();
                        // added by Andrew on March 18th, 2018.
                        entry_lowercase = entry_lowercase.Replace("_", " ");
                        entry_lowercase = entry_lowercase.Replace("-", "");
                        FindIt = entry_lowercase.IndexOf(TextString);
                        if (FindIt == 0)
                        {
                            // we found an exact match.
                            if (Globals.skin_icon_type == 1)
                            {
                                autoindex_matches.Add(Globals.autoindex_filepaths[autoindex_counter]);
                                popup_result = Popup.Row_Add_Row(entry, Globals.autoindex_filepaths[autoindex_counter], "autoindex", null);
                                //AddEntryToAutoComplete(4, autoindex_counter, "autoindex");
                            }
                            else
                            {
                                autoindex_matches.Add(Globals.autoindex_filepaths[autoindex_counter]);
                                Popup.Row_Add_Row(entry, Globals.autoindex_filepaths[autoindex_counter], "autoindex", null);
                                //AddEntryToAutoComplete(0, autoindex_counter, "autoindex");
                            }
                            result_count++;
                            PreviousKeywordMatch = entry.ToLower();
                            PreviousEntry = TextString;
                            if (popup_result == -1)
                            {
                                ShowPopupForm();
                                Done_Searching_Autoindex = true;
                                return;
                            }
                        }
                        else if (FindIt > 0)
                        {
                            // we found a partial match.
                            // only add a partial match to the list if it's 3 characters or greater.
                            if (text_length > 2)
                                partial_match_list_index.Add(autoindex_counter);
                        }
                    }
                    autoindex_counter++;
                }

                // add the partial matches into the autocomplete list
                foreach (int e in partial_match_list_index)
                {
                    if (Globals.skin_icon_type == 1)
                    {
                        autoindex_matches.Add(Globals.autoindex_filepaths[e]);
                        //AddEntryToAutoComplete(4, e, "autoindex");
                        popup_result = Popup.Row_Add_Row(Globals.autoindex_files[e], Globals.autoindex_filepaths[e], "autoindex", null);
                    }
                    else
                    {
                        autoindex_matches.Add(Globals.autoindex_filepaths[e]);
                        //AddEntryToAutoComplete(0, e, "autoindex");
                        popup_result = Popup.Row_Add_Row(Globals.autoindex_files[e], Globals.autoindex_filepaths[e], "autoindex", null);
                    }
                    result_count++;
                    PreviousKeywordMatch = Globals.autoindex_files[e].ToLower();
                    PreviousEntry = TextString;
                    if (popup_result == -1)
                    {
                        ShowPopupForm();
                        Done_Searching_Autoindex = true;
                        return;
                    }
                }
                if (result_count > 0)
                    ShowPopupForm();
                Done_Searching_Autoindex = true;

                // Now we want to search our folders that have indexing on demand.
                tmrIndexOnDemand.Interval = popup_result_delay_with_folders_on_demand;
                tmrIndexOnDemand.Enabled = true;
            }
            catch
            {
                // we ran into an error at some point. We don't need to display this to the user since this runs in the background with a separate thread, but let's make sure we set Done_Searching_Autoindex to true so that it can run again later.
                Done_Searching_Autoindex = true;
            }
        }

        private void tmrNoteAlarms_Tick(object sender, EventArgs e)
        {
            // this will check if any note reminders exist. If they do, evaluate the reminders.
            string reminders_file = Globals.WorkPath + "\\notes\\settings\\reminders.ini";
            string full_text;

            try
            {
                if (Globals.DoesFileExist(reminders_file) == false)
                {
                    System.IO.File.WriteAllText(reminders_file, "", Encoding.Unicode);
                    full_text = "";
                }
                else
                    full_text = File.ReadAllText(reminders_file);
            }
            catch
            {
                MessageBox.Show("The file that stores note reminders, " + reminders_file + ", could not be read. Please make sure this file exists and that you have read and write permission to the directory.", "Keybreeze Desktop");
                return;
            }
            if (full_text.Length < 30)
            {
                // if it's less than 30 characters in length, they have not set any reminders.
                return;
            }
            else
            {
                notes_check_reminders();
            }
        }

        public void notes_check_reminders()
        {
            string reminders_file = Globals.WorkPath + "\\notes\\settings\\reminders.ini";
            string full_text;

            try
            {
                if (Globals.DoesFileExist(reminders_file) == false)
                {
                    System.IO.File.WriteAllText(reminders_file, "", Encoding.Unicode);
                    full_text = "";
                }
                else
                    full_text = File.ReadAllText(reminders_file);
            }
            catch
            {
                MessageBox.Show("The file that stores note reminders, " + reminders_file + ", could not be read. Please make sure this file exists and that you have read and write permission to the directory.", "Keybreeze Desktop");
                return;
            }
            if (full_text.Length < 30)
            {
                // if it's less than 30 characters in length, they have not set any reminders.
                return;
            }
            else
            {
                // check all the reminders.
                int the_type;
                int the_year;
                int the_month;
                int the_dayofmonth;
                int the_hour;
                int the_minute;
                int the_weekday;
                int lastran_dayofmonth;
                int lastran_month;
                int noteid;
                int current_year = DateTime.Today.Year;
                int current_month = DateTime.Today.Month;
                int current_dayofmonth = DateTime.Today.Day;
                int current_hour = DateTime.Now.Hour;
                int current_minute = DateTime.Now.Minute;
                int current_weekday = GetNumFromWeekday(DateTime.Today.DayOfWeek.ToString());
                for (int i = 1; i <= 100; i++)
                {
                    the_type = Globals.ReadIntFromINI("reminders", i + "-type", reminders_file);
                    if (the_type == 1)
                    {
                        // one time reminder
                        the_year = Globals.ReadIntFromINI("reminders", i + "-year", reminders_file);
                        the_month = Globals.ReadIntFromINI("reminders", i + "-month", reminders_file);
                        the_dayofmonth = Globals.ReadIntFromINI("reminders", i + "-dayofmonth", reminders_file);
                        the_hour = Globals.ReadIntFromINI("reminders", i + "-timehour", reminders_file);
                        the_minute = Globals.ReadIntFromINI("reminders", i + "-timeminute", reminders_file);
                        bool time_to_show_reminder = false;
                        if (current_year > the_year)
                            time_to_show_reminder = true;
                        else if (current_year >= the_year && current_month > the_month)
                            time_to_show_reminder = true;
                        else if (current_year >= the_year && current_month >= the_month && current_dayofmonth > the_dayofmonth)
                            time_to_show_reminder = true;
                        else if (current_year >= the_year && current_month >= the_month && current_dayofmonth >= the_dayofmonth && current_hour > the_hour)
                            time_to_show_reminder = true;
                        else if (current_year >= the_year && current_month >= the_month && current_dayofmonth >= the_dayofmonth && current_hour >= the_hour && current_minute >= the_minute)
                            time_to_show_reminder = true;
                        if (time_to_show_reminder == true)
                        {
                            // display the reminder
                            noteid = Globals.ReadIntFromINI("reminders", i + "-noteid", reminders_file);
                            Globals.notes_show_note(noteid, this, true);
                            // since this is just a one time reminder, we want to clear everything in the INI file for this reminder.
                            Globals.WriteINI("reminders", i + "-type", null, reminders_file);
                            Globals.WriteINI("reminders", i + "-year", null, reminders_file);
                            Globals.WriteINI("reminders", i + "-month", null, reminders_file);
                            Globals.WriteINI("reminders", i + "-dayofmonth", null, reminders_file);
                            Globals.WriteINI("reminders", i + "-timehour", null, reminders_file);
                            Globals.WriteINI("reminders", i + "-timeminute", null, reminders_file);
                            Globals.WriteINI("reminders", i + "-day", null, reminders_file);
                            Globals.WriteINI("reminders", i + "-noteid", null, reminders_file);
                            Globals.Reorganize_reminders();
                        }
                    }
                    else if (the_type == 2)
                    {
                        // daily reminder
                        the_hour = Globals.ReadIntFromINI("reminders", i + "-timehour", reminders_file);
                        the_minute = Globals.ReadIntFromINI("reminders", i + "-timeminute", reminders_file);
                        lastran_dayofmonth = Globals.ReadIntFromINI("reminders", i + "-lastran_dayofmonth", reminders_file);
                        if (lastran_dayofmonth != current_dayofmonth)
                        {
                            // this reminder has not yet popped up today. check the time.
                            bool time_to_show_reminder = false;
                            if (current_hour > the_hour)
                                time_to_show_reminder = true;
                            else if (current_hour >= the_hour && current_minute >= the_minute)
                                time_to_show_reminder = true;
                            if (time_to_show_reminder == true)
                            {
                                // display the reminder
                                noteid = Globals.ReadIntFromINI("reminders", i + "-noteid", reminders_file);
                                Globals.notes_show_note(noteid, this, true);
                                Globals.WriteINI("reminders", i + "-lastran_dayofmonth", current_dayofmonth.ToString(), reminders_file);
                            }
                        }
                    }
                    else if (the_type == 3)
                    {
                        // weekly reminder
                        bool time_to_show_reminder = false;
                        int days_since_last_reminder = 0;
                        the_hour = Globals.ReadIntFromINI("reminders", i + "-timehour", reminders_file);
                        the_minute = Globals.ReadIntFromINI("reminders", i + "-timeminute", reminders_file);
                        the_weekday = Globals.ReadIntFromINI("reminders", i + "-day", reminders_file);
                        lastran_dayofmonth = Globals.ReadIntFromINI("reminders", i + "-lastran_dayofmonth", reminders_file);
                        if (lastran_dayofmonth != current_dayofmonth)
                        {
                            // it's been at least a day since the weekly reminder popped up. we need to see if it's been 7 days.
                            if (lastran_dayofmonth > current_dayofmonth)
                            {
                                // we are in a new month (or year). see if it's been 7 days.
                                int previous_month;
                                if (current_month == 1)
                                    previous_month = 12;
                                else
                                    previous_month = current_month - 1;
                                int days_in_previous_month = System.DateTime.DaysInMonth(current_year, previous_month);
                                days_since_last_reminder = current_dayofmonth + (days_in_previous_month - lastran_dayofmonth);
                            }
                            else
                            {
                                // we are in the same month. see if it's been 7 days.
                                days_since_last_reminder = current_dayofmonth - lastran_dayofmonth;
                            }
                            // if it's a new reminder, the last ran day will be 0. let's account for that.
                            if (lastran_dayofmonth == 0)
                                days_since_last_reminder = 7;

                            if (days_since_last_reminder > 7)
                                time_to_show_reminder = true;
                            else if (days_since_last_reminder == 7 && current_hour > the_hour)
                                time_to_show_reminder = true;
                            else if (days_since_last_reminder == 7 && current_hour >= the_hour && current_minute >= the_minute)
                                time_to_show_reminder = true;
                            if (time_to_show_reminder == true)
                            {
                                // display the reminder
                                noteid = Globals.ReadIntFromINI("reminders", i + "-noteid", reminders_file);
                                Globals.notes_show_note(noteid, this, true);

                                // if the reminder is showing up late, we don't want to write the current 'day of month'
                                // back to the INI, since that would cause the next reminder to show up 7 days later.
                                // instead, we need to find out which day of the month the weekday fell on.
                                DateTime my_date;
                                string my_date_day;
                                bool set_date = false;
                                for (int j = 0; j >= -7; j--)
                                {
                                    my_date = DateTime.Today.AddDays(j);
                                    my_date_day = my_date.DayOfWeek.ToString();
                                    if (GetNumFromWeekday(my_date_day) == the_weekday)
                                    {
                                        // we found the date of the last weekday where this reminder was supposed to appear. Update the INI file.
                                        int last_day_of_month = my_date.Day;
                                        Globals.WriteINI("reminders", i + "-lastran_dayofmonth", last_day_of_month.ToString(), reminders_file);
                                        set_date = true;
                                        break;
                                    }
                                }
                                // if we couldn't find the weekday for whatever reason, just set the next reminder to appear 7 days after the current day.
                                if (set_date == false)
                                    Globals.WriteINI("reminders", i + "-lastran_dayofmonth", current_dayofmonth.ToString(), reminders_file);
                            }
                        }
                    }
                    else if (the_type == 4)
                    {
                        // monthly reminder
                        bool time_to_show_reminder = false;
                        the_hour = Globals.ReadIntFromINI("reminders", i + "-timehour", reminders_file);
                        the_minute = Globals.ReadIntFromINI("reminders", i + "-timeminute", reminders_file);
                        the_dayofmonth = Globals.ReadIntFromINI("reminders", i + "-dayofmonth", reminders_file);
                        lastran_month = Globals.ReadIntFromINI("reminders", i + "-lastran_month", reminders_file);
                        lastran_dayofmonth = Globals.ReadIntFromINI("reminders", i + "-lastran_dayofmonth", reminders_file);
                        if (lastran_month != current_month && current_dayofmonth > the_dayofmonth)
                        {
                            // we are in a new month and the day to show the monthly reminder has passed. let's show it.
                            time_to_show_reminder = true;
                        }
                        else if (lastran_month != current_month && the_dayofmonth == current_dayofmonth)
                        {
                            // we are in a new month and the day is the same day that the reminder is supposed to appear.
                            if (current_hour > the_hour)
                                time_to_show_reminder = true;
                            else if (current_hour >= the_hour && current_minute >= the_minute)
                                time_to_show_reminder = true;
                        }
                        if (time_to_show_reminder == true)
                        {
                            // display the reminder
                            noteid = Globals.ReadIntFromINI("reminders", i + "-noteid", reminders_file);
                            Globals.notes_show_note(noteid, this, true);
                            Globals.WriteINI("reminders", i + "-lastran_month", current_month.ToString(), reminders_file);
                        }
                    }
                    else if (the_type == 0)
                        break;
                }
            }
        }

        private int GetNumFromWeekday(string the_weekday)
        {
            switch (the_weekday)
            {
                case "Sunday":
                    return 1;
                case "Monday":
                    return 2;
                case "Tuesday":
                    return 3;
                case "Wednesday":
                    return 4;
                case "Thursday":
                    return 5;
                case "Friday":
                    return 6;
                case "Saturday":
                    return 7;
                default:
                    return 0;
            }
        }

        public void CheckForINIChanges()
        {
            // this will see if a new version was just installed. if it was, it will add any INI changes.
            int current_version = Globals.ReadIntFromINI("options", "current_version", Globals.OptionsINI, 0);
            Globals.WriteINI("options", "current_version", Globals.version_numeric.ToString(), Globals.OptionsINI);
            if (current_version < 560)
            {
                CheckDPISettingsAndWarnUser();
                int return_val1 = AddKeywordToCommandList("c - clipboard monitor", "", "preset:clipboard monitor", false);
                // need to edit all modern skins and set main_textbox_bold=0 for all of them.
                string this_skin_ini = "";
                this_skin_ini = Globals.WorkPath + "\\skins\\2017 Cool\\settings.ini";
                if (File.Exists(this_skin_ini) == true)
                {
                    Globals.WriteINI("skin", "main_textbox_bold", "0", this_skin_ini);
                }
                this_skin_ini = Globals.WorkPath + "\\skins\\2017 Cool - No icons\\settings.ini";
                if (File.Exists(this_skin_ini) == true)
                {
                    Globals.WriteINI("skin", "main_textbox_bold", "0", this_skin_ini);
                }
                this_skin_ini = Globals.WorkPath + "\\skins\\2017 Warm\\settings.ini";
                if (File.Exists(this_skin_ini) == true)
                {
                    Globals.WriteINI("skin", "main_textbox_bold", "0", this_skin_ini);
                }
                this_skin_ini = Globals.WorkPath + "\\skins\\Flat Modern Wide\\settings.ini";
                if (File.Exists(this_skin_ini) == true)
                {
                    Globals.WriteINI("skin", "main_textbox_bold", "0", this_skin_ini);
                }
                this_skin_ini = Globals.WorkPath + "\\skins\\Keybreeze Blue\\settings.ini";
                if (File.Exists(this_skin_ini) == true)
                {
                    Globals.WriteINI("skin", "main_textbox_bold", "0", this_skin_ini);
                }
                this_skin_ini = Globals.WorkPath + "\\skins\\Keybreeze Green\\settings.ini";
                if (File.Exists(this_skin_ini) == true)
                {
                    Globals.WriteINI("skin", "main_textbox_bold", "0", this_skin_ini);
                }
                this_skin_ini = Globals.WorkPath + "\\skins\\Keybreeze Purple\\settings.ini";
                if (File.Exists(this_skin_ini) == true)
                {
                    Globals.WriteINI("skin", "main_textbox_bold", "0", this_skin_ini);
                }
                this_skin_ini = Globals.WorkPath + "\\skins\\Keybreeze Red\\settings.ini";
                if (File.Exists(this_skin_ini) == true)
                {
                    Globals.WriteINI("skin", "main_textbox_bold", "0", this_skin_ini);
                }
                this_skin_ini = Globals.WorkPath + "\\skins\\Keybreeze Slate\\settings.ini";
                if (File.Exists(this_skin_ini) == true)
                {
                    Globals.WriteINI("skin", "main_textbox_bold", "0", this_skin_ini);
                }
                this_skin_ini = Globals.WorkPath + "\\skins\\Simple - No Icons\\settings.ini";
                if (File.Exists(this_skin_ini) == true)
                {
                    Globals.WriteINI("skin", "main_textbox_bold", "0", this_skin_ini);
                }
                this_skin_ini = Globals.InstallDir + "\\default settings\\skins\\2017 Cool\\settings.ini";
                if (File.Exists(this_skin_ini) == true)
                {
                    Globals.WriteINI("skin", "main_textbox_bold", "0", this_skin_ini);
                }
                this_skin_ini = Globals.InstallDir + "\\default settings\\skins\\2017 Cool - No Icons\\settings.ini";
                if (File.Exists(this_skin_ini) == true)
                {
                    Globals.WriteINI("skin", "main_textbox_bold", "0", this_skin_ini);
                }
                this_skin_ini = Globals.InstallDir + "\\default settings\\skins\\2017 Warm\\settings.ini";
                if (File.Exists(this_skin_ini) == true)
                {
                    Globals.WriteINI("skin", "main_textbox_bold", "0", this_skin_ini);
                }
                this_skin_ini = Globals.InstallDir + "\\default settings\\skins\\Flat Modern Wide\\settings.ini";
                if (File.Exists(this_skin_ini) == true)
                {
                    Globals.WriteINI("skin", "main_textbox_bold", "0", this_skin_ini);
                }
                this_skin_ini = Globals.InstallDir + "\\default settings\\skins\\Keybreeze Blue\\settings.ini";
                if (File.Exists(this_skin_ini) == true)
                {
                    Globals.WriteINI("skin", "main_textbox_bold", "0", this_skin_ini);
                }
                this_skin_ini = Globals.InstallDir + "\\default settings\\skins\\Keybreeze Green\\settings.ini";
                if (File.Exists(this_skin_ini) == true)
                {
                    Globals.WriteINI("skin", "main_textbox_bold", "0", this_skin_ini);
                }
                this_skin_ini = Globals.InstallDir + "\\default settings\\skins\\Keybreeze Purple\\settings.ini";
                if (File.Exists(this_skin_ini) == true)
                {
                    Globals.WriteINI("skin", "main_textbox_bold", "0", this_skin_ini);
                }
                this_skin_ini = Globals.InstallDir + "\\default settings\\skins\\Keybreeze Red\\settings.ini";
                if (File.Exists(this_skin_ini) == true)
                {
                    Globals.WriteINI("skin", "main_textbox_bold", "0", this_skin_ini);
                }
                this_skin_ini = Globals.InstallDir + "\\default settings\\skins\\Keybreeze Slate\\settings.ini";
                if (File.Exists(this_skin_ini) == true)
                {
                    Globals.WriteINI("skin", "main_textbox_bold", "0", this_skin_ini);
                }
                this_skin_ini = Globals.InstallDir + "\\default settings\\skins\\Simple - No Icons\\settings.ini";
                if (File.Exists(this_skin_ini) == true)
                {
                    Globals.WriteINI("skin", "main_textbox_bold", "0", this_skin_ini);
                }
                // Copy our new notes over.
                if (Directory.Exists(Globals.WorkPath + "\\notes\\themes\\dark colors") == false)
                {
                    Directory.CreateDirectory(Globals.WorkPath + "\\notes\\themes\\dark colors");
                }
                if (Directory.Exists(Globals.WorkPath + "\\notes\\themes\\light colors") == false)
                {
                    Directory.CreateDirectory(Globals.WorkPath + "\\notes\\themes\\light colors");
                }
                string this_note = "";
                string dest_note = "";
                this_note = Globals.InstallDir + "\\default settings\\notes\\themes\\light colors\\bluegradient.jpg";
                dest_note = Globals.WorkPath + "\\notes\\themes\\light colors\\bluegradient.jpg";
                if (File.Exists(this_note) == true)
                {
                    File.Copy(this_note, dest_note, true);
                }
                this_note = Globals.InstallDir + "\\default settings\\notes\\themes\\light colors\\greengradient.jpg";
                dest_note = Globals.WorkPath + "\\notes\\themes\\light colors\\greengradient.jpg";
                if (File.Exists(this_note) == true)
                {
                    File.Copy(this_note, dest_note, true);
                }
                this_note = Globals.InstallDir + "\\default settings\\notes\\themes\\light colors\\pinkgradient.jpg";
                dest_note = Globals.WorkPath + "\\notes\\themes\\light colors\\pinkgradient.jpg";
                if (File.Exists(this_note) == true)
                {
                    File.Copy(this_note, dest_note, true);
                }
                this_note = Globals.InstallDir + "\\default settings\\notes\\themes\\light colors\\purplegradient.jpg";
                dest_note = Globals.WorkPath + "\\notes\\themes\\light colors\\purplegradient.jpg";
                if (File.Exists(this_note) == true)
                {
                    File.Copy(this_note, dest_note, true);
                }
                this_note = Globals.InstallDir + "\\default settings\\notes\\themes\\light colors\\redgradient.jpg";
                dest_note = Globals.WorkPath + "\\notes\\themes\\light colors\\redgradient.jpg";
                if (File.Exists(this_note) == true)
                {
                    File.Copy(this_note, dest_note, true);
                }
                this_note = Globals.InstallDir + "\\default settings\\notes\\themes\\light colors\\yellowgradient.jpg";
                dest_note = Globals.WorkPath + "\\notes\\themes\\light colors\\yellowgradient.jpg";
                if (File.Exists(this_note) == true)
                {
                    File.Copy(this_note, dest_note, true);
                }

                // dark colors
                this_note = Globals.InstallDir + "\\default settings\\notes\\themes\\dark colors\\darkbluegradient.jpg";
                dest_note = Globals.WorkPath + "\\notes\\themes\\dark colors\\darkbluegradient.jpg";
                if (File.Exists(this_note) == true)
                {
                    File.Copy(this_note, dest_note, true);
                }
                this_note = Globals.InstallDir + "\\default settings\\notes\\themes\\dark colors\\darkgreengradient.jpg";
                dest_note = Globals.WorkPath + "\\notes\\themes\\dark colors\\darkgreengradient.jpg";
                if (File.Exists(this_note) == true)
                {
                    File.Copy(this_note, dest_note, true);
                }
                this_note = Globals.InstallDir + "\\default settings\\notes\\themes\\dark colors\\darkpurplegradient.jpg";
                dest_note = Globals.WorkPath + "\\notes\\themes\\dark colors\\darkpurplegradient.jpg";
                if (File.Exists(this_note) == true)
                {
                    File.Copy(this_note, dest_note, true);
                }
                this_note = Globals.InstallDir + "\\default settings\\notes\\themes\\dark colors\\darkredgradient.jpg";
                dest_note = Globals.WorkPath + "\\notes\\themes\\dark colors\\darkredgradient.jpg";
                if (File.Exists(this_note) == true)
                {
                    File.Copy(this_note, dest_note, true);
                }
            }
            if (current_version < 550)
            {
                // also need to copy the 2017 skins if they don't already exist.
                if (Directory.Exists(Globals.WorkPath + "\\skins\\2017 Cool") == false && Directory.Exists(Globals.InstallDir + "\\default settings\\skins\\2017 Cool") == true)
                {
                    try
                    {
                        string SourcePath = Globals.InstallDir + "\\default settings\\skins\\2017 Cool";
                        string DestinationPath = Globals.WorkPath + "\\skins\\2017 Cool";
                        Directory.CreateDirectory(DestinationPath);
                        foreach (string dirPath in Directory.GetDirectories(SourcePath, "*", SearchOption.AllDirectories))
                            Directory.CreateDirectory(dirPath.Replace(SourcePath, DestinationPath));
                        //Copy all the files & Replaces any files with the same name
                        foreach (string newPath in Directory.GetFiles(SourcePath, "*.*", SearchOption.AllDirectories))
                            File.Copy(newPath, newPath.Replace(SourcePath, DestinationPath), true);
                    }
                    catch { }
                }
                if (Directory.Exists(Globals.WorkPath + "\\skins\\2017 Warm") == false && Directory.Exists(Globals.InstallDir + "\\default settings\\skins\\2017 Warm") == true)
                {
                    try
                    {
                        string SourcePath = Globals.InstallDir + "\\default settings\\skins\\2017 Warm";
                        string DestinationPath = Globals.WorkPath + "\\skins\\2017 Warm";
                        Directory.CreateDirectory(DestinationPath);
                        foreach (string dirPath in Directory.GetDirectories(SourcePath, "*", SearchOption.AllDirectories))
                            Directory.CreateDirectory(dirPath.Replace(SourcePath, DestinationPath));
                        //Copy all the files & Replaces any files with the same name
                        foreach (string newPath in Directory.GetFiles(SourcePath, "*.*", SearchOption.AllDirectories))
                            File.Copy(newPath, newPath.Replace(SourcePath, DestinationPath), true);
                    }
                    catch { }
                }
                if (Directory.Exists(Globals.WorkPath + "\\skins\\2017 Cool - No Icons") == false && Directory.Exists(Globals.InstallDir + "\\default settings\\skins\\2017 Cool - No Icons") == true)
                {
                    try
                    {
                        string SourcePath = Globals.InstallDir + "\\default settings\\skins\\2017 Cool - No Icons";
                        string DestinationPath = Globals.WorkPath + "\\skins\\2017 Cool - No Icons";
                        Directory.CreateDirectory(DestinationPath);
                        foreach (string dirPath in Directory.GetDirectories(SourcePath, "*", SearchOption.AllDirectories))
                            Directory.CreateDirectory(dirPath.Replace(SourcePath, DestinationPath));
                        //Copy all the files & Replaces any files with the same name
                        foreach (string newPath in Directory.GetFiles(SourcePath, "*.*", SearchOption.AllDirectories))
                            File.Copy(newPath, newPath.Replace(SourcePath, DestinationPath), true);
                    }
                    catch { }
                }
            }
            if (current_version < 510)
            {
                int return_val1 = AddKeywordToCommandList("10 minute reminder note", "", "remind:10m", false);
                int return_val2 = AddKeywordToCommandList("15 minute reminder note", "", "remind:15m", false);
                int return_val3 = AddKeywordToCommandList("30 minute reminder note", "", "remind:30m", false);
                // also need to copy the Flat Modern Wide skin if it doesn't already exist.
                if (Directory.Exists(Globals.WorkPath + "\\skins\\Flat Modern Wide") == false && Directory.Exists(Globals.InstallDir + "\\default settings\\skins\\Flat Modern Wide") == true)
                {
                    try
                    {
                        string SourcePath = Globals.InstallDir + "\\default settings\\skins\\Flat Modern Wide";
                        string DestinationPath = Globals.WorkPath + "\\skins\\Flat Modern Wide";
                        Directory.CreateDirectory(DestinationPath);
                        foreach (string dirPath in Directory.GetDirectories(SourcePath, "*", SearchOption.AllDirectories))
                            Directory.CreateDirectory(dirPath.Replace(SourcePath, DestinationPath));
                        //Copy all the files & Replaces any files with the same name
                        foreach (string newPath in Directory.GetFiles(SourcePath, "*.*", SearchOption.AllDirectories))
                            File.Copy(newPath, newPath.Replace(SourcePath, DestinationPath), true);
                    }
                    catch { }
                }
            }
            if (current_version < 504)
            {
                AddKeywordToCommandList("application data (appdata) folder", "", "preset:application data", false);
            }
            if (current_version < 501)
            {
                // make sure Export entries and Import entries are in the command list.
                int return_val =  AddKeywordToCommandList("export entries", "", "preset:export entries", false);
                int return_val2 = AddKeywordToCommandList("import entries", "", "preset:import entries", false);
            }
            if (current_version < 404)
            {
                try
                {
                    Globals.WriteINI("autocomplete", "top_offset", "4", Globals.WorkPath + "\\skins\\Default Blue\\settings.ini");
                    Globals.WriteINI("autocomplete", "top_offset", "4", Globals.WorkPath + "\\skins\\Default Brown\\settings.ini");
                    Globals.WriteINI("autocomplete", "top_offset", "4", Globals.WorkPath + "\\skins\\Default Gray\\settings.ini");
                    Globals.WriteINI("autocomplete", "top_offset", "4", Globals.WorkPath + "\\skins\\Default Green\\settings.ini");
                    Globals.WriteINI("autocomplete", "top_offset", "4", Globals.WorkPath + "\\skins\\Default Red\\settings.ini");
                }
                catch
                { }
            }
        }

        public void CheckRequiredFiles()
        {
            // this will make sure that all the required config files for Keybreeze Desktop exist. It will first 
            // check the %appdata% directory. If they are not there, then it will check the program files
            // directory and copy them to the user's %appdata% directory.

            // if the options.ini is not in the user's %appdata% directory, then we need to exit the 
            // program and run keybreeze_install.exe.
            if (Globals.logging == 1)
                Globals.DebugWrite("CheckRequiredFiles -- starting function.");
            string default_settings_path = Globals.InstallDir + "\\default settings";
            string autoindex_config_file = Globals.WorkPath + "\\autoindex\\indexoptions.ini";

            if (Directory.Exists(Globals.WorkPath + "\\macros") == false)
            {
                try
                {
                    System.IO.Directory.CreateDirectory(Globals.WorkPath + "\\macros");
                }
                catch
                { }
            }
            if (Globals.ReadIntFromINI("options", "launched_once", Globals.OptionsINI, 0) == 0)
            {
                // the user is running keybreeze for the first time. Show the tutorial.
                RunURL("http://www.keybreeze.com/tutorial/4");
                Globals.WriteINI("options", "launched_once", "1", Globals.OptionsINI);
            }
            if (Globals.DoesFileExist(Globals.WorkPath + "\\autocomplete.txt") == false)
            {
                string temp_text = "";
                try
                {
                    System.IO.File.WriteAllText(Globals.WorkPath + "\\autocomplete.txt", temp_text, Encoding.Unicode);
                }
                catch
                {
                    MessageBox.Show("Keybreeze Desktop cannot write to the " + Globals.WorkPath + " directory. Please check your Windows permissions. The program will now close.", "Keybreeze Desktop");
                    ExitApplication(true);
                    return;
                }
            }
            if (Globals.DoesFileExist(Globals.OptionsINI) == false)
            {
                if (Globals.DoesFileExist(default_settings_path + "\\options.ini") == false)
                {
                    MessageBox.Show("A required configuration file for Keybreeze Desktop is missing. This file is options.ini which should be stored in either " + Globals.WorkPath + " or " + default_settings_path + ". It's recommended that you reinstall Keybreeze Desktop to correct the problem.", "Keybreeze Desktop");
                    ExitApplication(true);
                    return;
                }
                else
                {
                    System.Diagnostics.Process.Start(Globals.InstallDir + "\\keybreeze_install.exe");
                    ExitApplication(true);
                    return;
                }
            }


            if (Globals.DoesFileExist(Globals.KeywordsJSONFile) == false && Globals.DoesFileExist(Globals.WorkPath + "\\keywords.ini") == false)
            {
                // keywords.json and keywords.ini do not exist in the work path. Let's copy the keywords.json from the default settings.
                if (Globals.DoesFileExist(default_settings_path + "\\keywords.json") == false)
                {
                    MessageBox.Show("A required configuration file for Keybreeze Desktop is missing. This file is keywords.json, which should be stored in either " + Globals.WorkPath + " or " + default_settings_path + ". It's recommended that you reinstall Keybreeze Desktop to correct the problem.", "Keybreeze Desktop");
                    ExitApplication(true);
                    return;
                }
                else
                {
                    System.IO.File.Copy(default_settings_path + "\\keywords.json", Globals.KeywordsJSONFile, false);
                }
            }
            if (Globals.DoesFileExist(Globals.LetterCommandsJSONFile) == false && Globals.DoesFileExist(Globals.WorkPath + "\\lettercommands.ini") == false)
            {
                // lettercommands.json and lettercommands.ini do not exist in the work path. Let's copy the lettercommands.json from the default settings.
                if (Globals.DoesFileExist(default_settings_path + "\\lettercommands.json") == false)
                {
                    MessageBox.Show("A required configuration file for Keybreeze Desktop is missing. This file is lettercommands.json, which should be stored in either " + Globals.WorkPath + " or " + default_settings_path + ". It's recommended that you reinstall Keybreeze Desktop to correct the problem.", "Keybreeze Desktop");
                    ExitApplication(true);
                    return;
                }
                else
                {
                    System.IO.File.Copy(default_settings_path + "\\lettercommands.json", Globals.LetterCommandsJSONFile, false);
                }
            }

            if (Globals.DoesFileExist(Globals.WorkPath + "\\search\\search.ini") == false)
            {
                if (Globals.DoesFileExist(default_settings_path + "\\search\\search.ini") == true)
                {
                    if (Directory.Exists(Globals.WorkPath + "\\search") == false)
                        Directory.CreateDirectory(Globals.WorkPath + "\\search");
                    System.IO.File.Copy(default_settings_path + "\\search\\search.ini", Globals.WorkPath + "\\search\\search.ini");
                }
            }
            

            if (Globals.DoesFileExist(Globals.NotesINI) == false)
            {
                if (Globals.DoesFileExist(default_settings_path + "\\notes\\settings\\notes.ini") == false)
                {
                    MessageBox.Show("A required configuration file for Keybreeze Desktop is missing. This file is notes.ini which should be stored in either " + Globals.WorkPath + "\\notes\\settings or " + default_settings_path + "\\notes\\settings. It's recommended that you reinstall Keybreeze Desktop to correct the problem.", "Keybreeze Desktop");
                    ExitApplication(true);
                    return;
                }
                else
                {
                    // copy file from installation folder to %appdata%
                    try
                    {
                        if (Directory.Exists(Globals.WorkPath + "\\notes") == false)
                        {
                            Directory.CreateDirectory(Globals.WorkPath + "\\notes");
                            System.Threading.Thread.Sleep(100);
                        }
                        if (Directory.Exists(Globals.WorkPath + "\\notes\\settings") == false)
                        {
                            Directory.CreateDirectory(Globals.WorkPath + "\\notes\\settings");
                            System.Threading.Thread.Sleep(100);
                        }
                        if (Directory.Exists(Globals.WorkPath + "\\notes\\themes") == false)
                        {
                            Directory.CreateDirectory(Globals.WorkPath + "\\notes\\themes");
                            // copy all the themes over.
                            string SourcePath = Globals.InstallDir + "\\default settings\\notes\\themes";
                            string DestinationPath = Globals.WorkPath + "\\notes\\themes";
                            foreach (string dirPath in Directory.GetDirectories(SourcePath, "*", SearchOption.AllDirectories))
                                Directory.CreateDirectory(dirPath.Replace(SourcePath, DestinationPath));
                            foreach (string newPath in Directory.GetFiles(SourcePath, "*.*", SearchOption.AllDirectories))
                                File.Copy(newPath, newPath.Replace(SourcePath, DestinationPath));

                            System.Threading.Thread.Sleep(100);
                        }
                        System.IO.File.Copy(default_settings_path + "\\notes\\settings\\notes.ini", Globals.NotesINI, false);
                    }
                    catch
                    {
                        MessageBox.Show("Could not copy the notes.ini file to " + Globals.NotesINI + ". Please make sure this directory exists. Then try re-running Keybreeze Desktop.", "Keybreeze Desktop");
                        ExitApplication(true);
                        return;
                    }
                }
            }
            if (Globals.DoesFileExist(autoindex_config_file) == false)
            {
                if (Globals.DoesFileExist(default_settings_path + "\\autoindex\\indexoptions.ini") == false)
                {
                    MessageBox.Show("A required configuration file for Keybreeze Desktop is missing. This file is indexoptions.ini which should be stored in either " + Globals.WorkPath + "\\autoindex or " + default_settings_path + "\\autoindex\\indexoptions.ini. It's recommended that you reinstall Keybreeze Desktop to correct the problem.", "Keybreeze Desktop");
                    ExitApplication(true);
                    return;
                }
                else
                {
                    // copy file from installation folder to %appdata%
                    try
                    {
                        if (Directory.Exists(Globals.WorkPath + "\\autoindex") == false)
                        {
                            Directory.CreateDirectory(Globals.WorkPath + "\\autoindex");
                            System.Threading.Thread.Sleep(100);
                        }
                        System.IO.File.Copy(default_settings_path + "\\autoindex\\indexoptions.ini", autoindex_config_file, false);
                    }
                    catch
                    {
                        MessageBox.Show("Could not copy the indexoptions.ini file to " + autoindex_config_file + ". Please make sure this directory exists. Then try re-running Keybreeze Desktop.", "Keybreeze Desktop");
                        ExitApplication(true);
                        return;
                    }
                }
            }
            string skin = Globals.ReadINI("options", "skin", Globals.OptionsINI);
            if (Directory.Exists(Globals.WorkPath + "\\skins\\" + skin) == false)
            {
                if (Directory.Exists(default_settings_path + "\\skins\\" + skin) == false)
                {
                    // no skin available.
                    MessageBox.Show("The Keybreeze Desktop skin could not be loaded, so the user interface may not look pretty. Press the hotkey and type 'options' as a keyword to reset the skin.", "Keybreeze Desktop");
                    Globals.valid_skin = false;
                }
                else
                {
                    DirectoryInfo di;
                    string skin_folder = "";
                    string workpath_location;
                    foreach (string newPath in Directory.GetFiles(default_settings_path + "\\skins", "*.*", SearchOption.AllDirectories))
                    {
                        string[] parts = newPath.Split('\\');
                        workpath_location = newPath.Replace(default_settings_path + "\\skins", Globals.WorkPath + "\\skins");
                        skin_folder = parts[parts.Length - 2];
                        if (skin_folder != "skins")
                        {
                            if (Directory.Exists(workpath_location) == false)
                            {
                                try
                                {
                                    di = Directory.CreateDirectory(Globals.WorkPath + "\\skins\\" + skin_folder);
                                }
                                catch
                                {
                                }
                            }
                            try
                            {
                                File.Copy(newPath, workpath_location);
                            }
                            catch
                            {
                            }
                        }
                        else
                        {
                            try
                            {
                                File.Copy(newPath, workpath_location);
                            }
                            catch
                            {
                            }
                        }
                    }
                }
            }
        }

        private void updateIndexToolStripMenuItem_Click(object sender, EventArgs e)
        {
            menuUpdateIndex();
        }

        private void reportABugToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            menuReportABug();
        }

        private void viewReminderseToolStripMenuItem_Click(object sender, EventArgs e)
        {
            menuNoteReminders();
        }

        private void systrayicon_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == System.Windows.Forms.MouseButtons.Right)
            {
                systray_menu_opening = true;
            }
            else
            {
                if (form_just_loaded == true)
                    form_just_loaded = false;
                Set_Keybreeze_Position();
                if (Globals.valid_skin == true && Globals.skin_embedded == 0)
                {
                    Skin.Visible = true;
                    //Skin.TopMost = true;
                    Skin.Activate();
                    if (Skin.bitmap != null)
                        Skin.SetBitmap(Skin.bitmap, skin_opacity);
                }
                this.txtMain.Text = "";
                this.allowshowdisplay = true;
                this.Visible = true;
                //this.TopMost = true;
                this.Activate();
            }
        }

        private void toolStripMenuItem3_Click(object sender, EventArgs e)
        {
            menuAddCommand();
        }

        public void menuAddCommand()
        {
            ShowAddKeywordForm(false);
            if (this.Visible == true)
                HideMainForm();
        }

        public void menuCommandList()
        {
            frmCommandList_instance = new frmCommandList(this);
            frmCommandList_instance.Show();
            if (this.Visible == true)
                HideMainForm();
        }

        public void menuUpdateIndex()
        {
            Globals.autoindex_update();
            if (this.Visible == true)
                HideMainForm();
        }

        public void menuResetPosition(bool show_keybreeze = true)
        {
            int screenWidth = Screen.PrimaryScreen.Bounds.Width;
            int screenHeight = Screen.PrimaryScreen.Bounds.Height;
            int x = screenWidth - this.Width;
            int y = screenHeight - this.Height;
            if (Globals.valid_skin == true && Globals.skin_embedded == 0)
            {
                this.Location = new Point(x / 2, y / 2);
                // move the skin 30 pixels up to account for the taskbar and popup menu. even if the taskbar is hidden, the main form doesn't look right when it's directly centered.
                this.Top = this.Top - 30;
                // move the position according to the offsets in the INI file as well.
                // I don't think we're using these any more since the position offsets are set in the skin file.
                //this.Top = this.Top + Globals.ReadIntFromINI("options", "center_top_offset", Globals.OptionsINI, 0);
                //this.Left = this.Left + Globals.ReadIntFromINI("options", "center_left_offset", Globals.OptionsINI, 0);
                if (Skin.bitmap != null)
                    Skin.SetBitmap(Skin.bitmap, skin_opacity);
                if (Globals.valid_skin == true && Globals.skin_embedded == 0)
                {
                    if (Skin.bitmap != null)
                        Skin.SetBitmap(Skin.bitmap, skin_opacity);
                    // To do: this needs to be an offset in the INI file.
                    Skin.Location = new Point(this.Left - 24 + Globals.skin_left_offset, this.Top - 26 + Globals.skin_top_offset);
                    //Skin.Location = new Point(this.Left, this.Top);
                }
                if (this.Top + this.Height + 50 + Popup.Height > screenHeight)
                {
                    // position the Popup window above the main window instead of below it.
                    if (Globals.valid_skin == true && Globals.skin_embedded == 0)
                        Popup.Top = Skin.Top - Popup.Height - Globals.top_offset;
                    else
                        Popup.Top = this.Top - Popup.Height - Globals.top_offset;
                }
                else
                {
                    if (Globals.valid_skin == true && Globals.skin_embedded == 0)
                        Popup.Top = Skin.Top + Skin.Height + Globals.top_offset;
                    else
                        Popup.Top = this.Top + this.Height + Globals.top_offset;
                }
                Popup.Left = this.Left - 12 + Globals.left_offset;

                this.Location = new Point(Skin.Location.X + Globals.main_textbox_left, Skin.Location.Y + Globals.main_textbox_top);

                if (show_keybreeze == true)
                {
                    Skin.Visible = true;
                    //Skin.TopMost = true;
                    Skin.Activate();
                }
            }
            else
            {
                this.Location = new Point(x / 2, y / 2);
                this.Top = this.Top - 30;
                this.Top = this.Top + Globals.center_top_offset;
                this.Left = this.Left + Globals.center_left_offset;
                if (this.Top + this.Height + 50 + Popup.Height > screenHeight)
                {
                    // position the Popup window above the main window instead of below it.
                    if (Globals.valid_skin == true && Globals.skin_embedded == 0)
                        Popup.Top = Skin.Top - Popup.Height - Globals.top_offset;
                    else
                        Popup.Top = this.Top - Popup.Height - Globals.top_offset;
                }
                else
                {
                    if (Globals.valid_skin == true && Globals.skin_embedded == 0)
                        Popup.Top = Skin.Top + Skin.Height + Globals.top_offset;
                    else
                        Popup.Top = this.Top + this.Height + Globals.top_offset;
                }
                Popup.Left = this.Left - 12 + Globals.left_offset;
            }
            // set new coordinates in the INI file.
            if (always_center_keybreeze == 0)
            {
                Globals.WriteINI("options", "skin_position_top", this.Top.ToString(), Globals.OptionsINI);
                Globals.WriteINI("options", "skin_position_left", this.Left.ToString(), Globals.OptionsINI);
            }
            if (show_keybreeze == true)
            {
                Popup.Visible = false;
                if (Globals.show_keyboard_help == 1)
                {
                    frmPopupHelper_instance.Visible = false;
                    tmrShowKeyboardHelp.Enabled = false;
                }
                popup_showing = false;
                this.Visible = true;
                //this.TopMost = true;
                this.Activate();
            }
            if (Globals.show_keyboard_help == 1)
            {
                frmPopupHelper_instance.Top = Popup.Top;
                frmPopupHelper_instance.Left = Popup.Left;
            }
        }

        public void menuNoteReminders()
        {
            frmNoteReminders frmNoteReminders_instance = new frmNoteReminders();
            frmNoteReminders_instance.Show();
            if (this.Visible == true)
                HideMainForm();
        }

        public void menuCreateNote()
        {
            Globals.notes_create_note(this);
        }

        public void menuHideAllNotes()
        {
            Globals.notes_hide_all_notes();
            if (this.Visible == true)
                HideMainForm();
        }

        public void menuShowHiddenNotes()
        {
            if (this.Visible == true)
                HideMainForm();
            Globals.notes_show_hidden_notes();
        }

        public void menuResetNotePositions()
        {
            Globals.notes_reset_positions();
            if (this.Visible == true)
                HideMainForm();
        }

        private void commandListToolStripMenuItem_Click(object sender, EventArgs e)
        {
            menuCommandList();
        }
        
        public void menuShowOptionsWindow()
        {
            frmOptions frmOptions_instance = new frmOptions(this);
            frmOptions_instance.Show();
            if (this.Visible == true)
                HideMainForm();
        }

        private void resetPositionToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            menuResetPosition();
        }

        private void createANoteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            menuCreateNote();
        }

        private void hideAllNotesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            menuHideAllNotes();
        }

        private void resetNotePositionsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            menuResetNotePositions();
        }

        private void showHiddenNotesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            menuShowHiddenNotes();
        }

        private void optionsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            menuShowOptionsWindow();
        }

        private void aboutToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            menuAbout();
        }

        public void menuAbout()
        {
            frmAbout frmAbout_instance;
            frmAbout_instance = new frmAbout(this);
            frmAbout_instance.Show();
            if (this.Visible == true)
                HideMainForm();
        }

        public void menuTutorial()
        {
            RunURL("http://www.keybreeze.com/tutorial/4");
            if (this.Visible == true)
                HideMainForm();
        }

        public void menuKnowledgebase()
        {
            RunURL("http://www.keybreeze.com/support");
            if (this.Visible == true)
                HideMainForm();
        }

        public void menuReportABug()
        {
            RunURL("http://www.keybreeze.com/support/bugreport");
            if (this.Visible == true)
                HideMainForm();
        }

        public void menuCheckForUpdate()
        {
            RunURL("http://keybreeze.com/download/releasenotes/keybreeze?version=" + Globals.version_numeric);
            if (this.Visible == true)
                HideMainForm();
        }

        public List<string> GetMacAddresses()
        {
            const int MIN_MAC_ADDR_LENGTH = 12;
            string macAddress = string.Empty;
            long maxSpeed = -1;
            List<string> mac_addresses = new List<string>();
            foreach (NetworkInterface nic in NetworkInterface.GetAllNetworkInterfaces())
            {
                if (Globals.logging == 1)
                {
                    Globals.DebugWrite(
                        "found mac address: " + nic.GetPhysicalAddress() +
                        " type: " + nic.NetworkInterfaceType);
                }
                string tempMac = nic.GetPhysicalAddress().ToString();
                if (tempMac.Length > 0)
                {
                    mac_addresses.Add(tempMac);
                }
            }
            return mac_addresses;
        }

        public string ReplaceAt(string input, int index, char newChar)
        {
            if (input == null)
            {
                throw new ArgumentNullException("input");
            }
            char[] chars = input.ToCharArray();
            chars[index] = newChar;
            return new string(chars);
        }

        public void menuDisableHotkey()
        {
            if (turnOffHotkeyToolStripMenuItem.Text == "Disable hotkey")
            {
                UnregisterHotKey(this.Handle, 1);
                turnOffHotkeyToolStripMenuItem.Text = "Enable hotkey";
                if (Globals.valid_skin == true && Globals.skin_embedded == 0)
                    Skin.turnOffHotkeyToolStripMenuItem.Text = "Enable hotkey";
            }
            else
            {
                SetHotKey();
                turnOffHotkeyToolStripMenuItem.Text = "Disable hotkey";
                if (Globals.valid_skin == true && Globals.skin_embedded == 0)
                    Skin.turnOffHotkeyToolStripMenuItem.Text = "Disable hotkey";
            }
            if (this.Visible == true)
                HideMainForm();
        }

        public void UnregisterHotkeyHelper()
        {
            // the API call cannot be accessed outside this form, so we publicly call this function instead.
            UnregisterHotKey(this.Handle, 1);
        }

        public void menuForum()
        {
            RunURL("http://www.keybreeze.com/forum");
        }

        public void menuCut()
        {
            if (txtMain.SelectedText.Length > 0)
            {
                try
                {
                    int selection_start = txtMain.SelectionStart;
                    Clipboard.SetText(txtMain.SelectedText);
                    txtMain.Text = txtMain.Text.Replace(txtMain.SelectedText, "");
                    txtMain.SelectionStart = selection_start;
                }
                catch
                { }
            }
        }

        public void menuCopy()
        {
            if (txtMain.SelectedText.Length > 0)
            {
                try
                {
                    Clipboard.SetText(txtMain.SelectedText);
                }
                catch
                { }
            }
        }

        public void menuPaste()
        {
            txtMain.Paste();
        }

        public void menuSelectAll()
        {
            txtMain.SelectAll();
        }

        private void generalHelpToolStripMenuItem_Click(object sender, EventArgs e)
        {
            menuTutorial();
        }

        private void knowledgebaseToolStripMenuItem_Click(object sender, EventArgs e)
        {
            menuKnowledgebase();
        }

        private void checkForUpdateToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            menuCheckForUpdate();
        }

        private void turnOffHotkeyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            menuDisableHotkey();
        }

        private void exitKeybreezeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // Exit Keybreeze
            ExitApplication(true);
        }

        private void cutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            menuCut();
        }

        private void contextMenuStrip1_Opening(object sender, CancelEventArgs e)
        {
            if (systray_menu_opening == true)
            {
                cutToolStripMenuItem.Visible = false;
                copyToolStripMenuItem.Visible = false;
                pasteToolStripMenuItem.Visible = false;
                toolStripMenuItem2.Visible = false;
                toolStripMenuItem1.Visible = false;
                systray_menu_opening = false;
            }
            else
            {
                cutToolStripMenuItem.Visible = true;
                copyToolStripMenuItem.Visible = true;
                pasteToolStripMenuItem.Visible = true;
                toolStripMenuItem2.Visible = true;
                toolStripMenuItem1.Visible = true;
            }
        }

        private void copyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            menuCopy();
        }

        private void pasteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            menuPaste();
        }

        private void toolStripMenuItem2_Click(object sender, EventArgs e)
        {
            menuSelectAll();
        }

        private void tmrAutoIndexRefresh_Tick(object sender, EventArgs e)
        {
            Globals.autoindex_update();
        }

        private void forumToolStripMenuItem_Click(object sender, EventArgs e)
        {
            menuForum();
        }

        private void pictureBox1_MouseDown(object sender, MouseEventArgs e)
        {
            // used to drag the form when there is no external skin.
            m_iWindowPosX = e.X;
            m_iWindowPosY = e.Y;
            base.OnMouseDown(e);
        }

        private void pictureBox1_MouseMove(object sender, MouseEventArgs e)
        {
            // used to drag the form when there is no external skin.
            if (e.Button == MouseButtons.Left)
                Location = new Point(Location.X - m_iWindowPosX + e.X, Location.Y - m_iWindowPosY + e.Y);
            base.OnMouseMove(e);
        }

        private void pictureBox1_MouseUp(object sender, MouseEventArgs e)
        {
            MoveChildForms();
        }

        private void tmrIndexOnDemand_Tick(object sender, EventArgs e)
        {
            if (this.Visible == false)
                return;
            Search_Autoindex_Folders_On_Demand();
            tmrIndexOnDemand.Enabled = false;
            return;
        }

        private void tmrShowKeyboardHelp_Tick(object sender, EventArgs e)
        {
            // this shows our keyboard help if the user is typing and pauses for a second or two.
            if (Globals.show_keyboard_help == 1 && tmrShowKeyboardHelp.Enabled == true && txtMain.Text.Length > 0 && Popup.Visible == false)
            {
                frmPopupHelper_instance.Visible = true;
                txtMain.Focus();
                tmrShowKeyboardHelp.Enabled = false;
            }
        }

        private void GetLocalDrives()
        {
            // this will get a list of local drives to look for icons for keywords later. We do not want to get icons for files on mapped network drives since they involve large delays.
            DriveInfo[] allDrives = DriveInfo.GetDrives();
            foreach (DriveInfo d in allDrives)
            {
                string drive_letter = d.Name.Substring(0, 1);
                if (d.DriveType.ToString() == "Fixed")
                {
                    Globals.local_drives.Add(drive_letter.ToLower());
                }
            }
        }

        private void PositionWindow(string position_title, string position_class, string position_left, string position_top, string position_width, string position_height)
        {
            // this function will resize a window, or position a window at certain coordinates on the screen.
            IntPtr exact_matching_window_handle = IntPtr.Zero;
            IntPtr partial_matching_window_handle = IntPtr.Zero;
            // validate the fields first.
            // trim out any wildcard characters on the edges.
            if (position_title.Substring(0, 1) == "*")
                position_title = position_title.Substring(1);
            if (position_title.Substring(position_title.Length - 1) == "*")
                position_title = position_title.Substring(0, position_title.Length - 1);
            if (position_title.Length > 0)
            {
                if (position_title.ToLower() == "getforegroundwindow()")
                {
                    // hide our recording window so that it doesn't get grabbed as our foreground window.
                    // shouldn't be necessary here.
                    // this.Visible = false;
                    Application.DoEvents();
                    System.Threading.Thread.Sleep(100);
                    exact_matching_window_handle = frmMain.GetForegroundWindow();
                }
                else
                {
                    Process[] processlist = Process.GetProcesses();
                    string window_title;
                    int find_match;
                    bool partial_match = false;
                    foreach (Process process in processlist)
                    {
                        Application.DoEvents();
                        if (!String.IsNullOrEmpty(process.MainWindowTitle))
                        {
                            window_title = process.MainWindowTitle;
                            find_match = window_title.IndexOf(position_title);
                            if (find_match == 0)
                            {
                                // we have an exact match. Find the window handle and exit the loop.
                                try
                                {
                                    if (position_class.Length == 0)
                                        exact_matching_window_handle = frmMain.FindWindow(null, process.MainWindowTitle);
                                    else
                                        exact_matching_window_handle = frmMain.FindWindow(position_class, process.MainWindowTitle);
                                }
                                catch { }
                                break;
                            }
                            else if (find_match > 0)
                            {
                                // we have a partial match. keep a record of this. if we don't find anything, set focus to it.
                                partial_match = true;
                                try
                                {
                                    if (position_class.Length == 0)
                                        partial_matching_window_handle = frmMain.FindWindow(null, process.MainWindowTitle);
                                    else
                                        partial_matching_window_handle = frmMain.FindWindow(position_class, process.MainWindowTitle);
                                }
                                catch { }
                            }
                        }
                    }
                    if (exact_matching_window_handle == IntPtr.Zero && partial_match == true)
                        exact_matching_window_handle = partial_matching_window_handle;
                }
            }
            else
            {
                // we only have a class name.
                exact_matching_window_handle = frmMain.FindWindow(position_class, null);
            }
            if (exact_matching_window_handle == IntPtr.Zero)
            {
                // we never found the window to resize.
                return;
            }
            // before moving/resizing, restore the window if it's minimized or maximized.
            ShowWindow(exact_matching_window_handle, 9);
            // we may have to get the current window's coordinates.
            Globals.RECT varRect = new Globals.RECT();
            bool found_window_rect = false;
            if ((position_top != "" && position_left != "") && (position_width == "" && position_height == ""))
            {
                // move window only. Don't resize.
                int position_left_int = 0;
                int position_top_int = 0;
                if (position_left == "left")
                    position_left_int = 0;
                else if (position_left == "right")
                {
                    // move window to the right.
                    //int screen_width = Screen.FromHandle(exact_matching_window_handle).Bounds.Width;
                    int screen_width = Screen.FromHandle(exact_matching_window_handle).WorkingArea.Width;
                    if (found_window_rect == false)
                    {
                        Globals.GetWindowRect(exact_matching_window_handle, out varRect);
                        found_window_rect = true;
                    }
                    int lngHeight = varRect.Bottom - varRect.Top;
                    int lngWidth = varRect.Right - varRect.Left;
                    position_left_int = screen_width - lngWidth;
                }
                else
                {
                    // the position is a regular number.
                    position_left_int = Convert.ToInt32(position_left);
                }
                if (position_top == "top")
                    position_top_int = 0;
                else if (position_top == "bottom")
                {
                    // move window to the bottom.
                    //int screen_height = Screen.FromHandle(exact_matching_window_handle).Bounds.Height;
                    int screen_height = Screen.FromHandle(exact_matching_window_handle).WorkingArea.Height;
                    if (found_window_rect == false)
                    {
                        Globals.GetWindowRect(exact_matching_window_handle, out varRect);
                        found_window_rect = true;
                    }
                    int lngHeight = varRect.Bottom - varRect.Top;
                    int lngWidth = varRect.Right - varRect.Left;
                    position_top_int = screen_height - lngHeight;
                }
                else
                {
                    // the position is a regular number.
                    position_top_int = Convert.ToInt32(position_top);
                }
                Globals.SetWindowPos(exact_matching_window_handle, 0, position_left_int, position_top_int, 0, 0, SWP_NOSIZE);
            }
            else if ((position_top == "" && position_left == "") && (position_width != "" && position_height != ""))
            {
                // resize window only. Don't move.
                int position_width_int = Convert.ToInt32(position_width);
                int position_height_int = Convert.ToInt32(position_height);
                Globals.SetWindowPos(exact_matching_window_handle, 0, 0, 0, position_width_int, position_height_int, SWP_NOMOVE);
            }
            else if ((position_top != "" && position_left != "") && (position_width != "" && position_height != ""))
            {
                // move window AND resize it.
                int position_left_int = 0;
                int position_top_int = 0;

                // Resize window first.
                // for the width and height, did the user specify a percentage or a number?
                int position_width_int = 0;
                int position_height_int = 0;
                int find_percent = position_width.IndexOf("%");
                if (find_percent != -1)
                {
                    // the user specified a percentage for the width. Convert it to a decimal and multiply it by the screen size.
                    int width1 = Convert.ToInt32(position_width.Substring(0, find_percent));
                    double position_width_percent = width1 * .01;
                    //int screen_width = Screen.FromHandle(exact_matching_window_handle).Bounds.Width;
                    int screen_width = Screen.FromHandle(exact_matching_window_handle).WorkingArea.Width;
                    position_width_int = Convert.ToInt32(Math.Ceiling(screen_width * position_width_percent));
                }
                else
                    position_width_int = Convert.ToInt32(position_width);
                find_percent = position_height.IndexOf("%");
                if (find_percent != -1)
                {
                    // the user specified a percentage for the height. Convert it to a decimal and multiply it by the screen size.
                    int height1 = Convert.ToInt32(position_height.Substring(0, find_percent));
                    double position_height_percent = height1 * .01;
                    //int screen_height = Screen.FromHandle(exact_matching_window_handle).Bounds.Height;
                    int screen_height = Screen.FromHandle(exact_matching_window_handle).WorkingArea.Height;
                    position_height_int = Convert.ToInt32(Math.Ceiling(screen_height * position_height_percent));
                }
                else
                    position_height_int = Convert.ToInt32(position_height);

                // Move window.
                if (position_left == "left")
                    position_left_int = 0;
                else if (position_left == "right")
                {
                    // move window to the right.
                    //int screen_width = Screen.FromHandle(exact_matching_window_handle).Bounds.Width;
                    int screen_width = Screen.FromHandle(exact_matching_window_handle).WorkingArea.Width;
                    int lngHeight = position_height_int;
                    int lngWidth = position_width_int;
                    position_left_int = screen_width - lngWidth;
                }
                else
                {
                    // the position is a regular number.
                    position_left_int = Convert.ToInt32(position_left);
                }
                if (position_top == "top")
                    position_top_int = 0;
                else if (position_top == "bottom")
                {
                    // move window to the bottom.
                    //int screen_height = Screen.FromHandle(exact_matching_window_handle).Bounds.Height;
                    int screen_height = Screen.FromHandle(exact_matching_window_handle).WorkingArea.Height;
                    int lngHeight = position_height_int;
                    int lngWidth = position_width_int;
                    position_top_int = screen_height - lngHeight;
                }
                else
                {
                    // the position is a regular number.
                    position_top_int = Convert.ToInt32(position_top);
                }
                Globals.SetWindowPos(exact_matching_window_handle, 0, position_left_int, position_top_int, position_width_int, position_height_int, 0);
            }
            /*
            if (position_title.ToLower() == "getforegroundwindow()")
            {
                // show our recording window again.
                this.Visible = true;
            }
            */
        }

        private void tmrKeybreezeClipboard_Tick(object sender, EventArgs e)
        {
            RefreshKeybreezeClipboard();
        }

        public void RefreshKeybreezeClipboard()
        {
            // this will monitor the clipboard and provide some history for the user.
            if (running_text_function == true)
            {
                // text functions use the clipboard. We do not want this function to run if we are actively using a text function.
                return;
            }
            try
            {
                string clipboard_text = Clipboard.GetText();
                int keybreeze_clipboard_count = keybreeze_clipboard.Count();
                if (clipboard_text.Length > 0)
                {
                    if (clipboard_text.Length > keybreeze_clipboard_item_max_length)
                    {
                        // this clipboard text exceeds our max length. Trim it.
                        clipboard_text = clipboard_text.Substring(0, keybreeze_clipboard_item_max_length);
                    }
                    if (keybreeze_clipboard_count == 0 || keybreeze_clipboard[keybreeze_clipboard.Count - 1].clipboard_text != clipboard_text)
                    {
                        // we either don't have anything saved in the Keybreeze clipboard, or the user just copied something new to the clipboard. Let's add it to the Keybreeze clipboard.
                        // loop this multiple times in case the user significantly dropped the maximum number of items to store.
                        for (int i = 0; i < 500; i++)
                        {
                            keybreeze_clipboard_count = keybreeze_clipboard.Count();
                            if (keybreeze_clipboard_count >= keybreeze_clipboard_max_count)
                            {
                                // we are maxed out. Let's remove the oldest item so we can add the new one.
                                keybreeze_clipboard.RemoveAt(0);
                            }
                            else
                            {
                                break;
                            }
                        }
                        Keybreeze_Clipboard new_item = new Keybreeze_Clipboard();
                        new_item.clipboard_text = clipboard_text;
                        keybreeze_clipboard.Add(new_item);
                    }
                }
            }
            catch (Exception e2)
            {
                // comment this out before creating production build.
                if (Globals.logging == 1)
                    Globals.DebugWrite("Error occurred while adding text to the Clipboard Monitor." + e2.Message);
                //MessageBox.Show("Error occurred while adding text to the Clipboard Monitor." + e2.Message, "Keybreeze Desktop");
            }
        }

        private void CheckDPISettingsAndWarnUser()
        {
            float dpiX, dpiY;
            Graphics graphics = this.CreateGraphics();
            dpiX = graphics.DpiX;
            dpiY = graphics.DpiY;
            // 100% DPI: 96, 96
            // 125% DPI: 120, 120
            // 150% DPI: 144, 144
            // 125% DPI doesn't really affect the existing skins. 150% does impact them though.
            if (dpiX >= 144 || dpiY >= 144)
            {
                MessageBox.Show("It appears that you have Windows configured at 150% DPI or higher, which will cause fonts and windows to appear larger. If Keybreeze Desktop appears to be distorted, please right-click the system tray icon and click Options > Skins, and change the skin to one that supports 150% DPI.", "Keybreeze Desktop");
            }
        }
    }
}
